local DFGraphics = require('DFCommon.Graphics')
local DFFile = require('DFCommon.File')
local DFUtil = require('DFCommon.Util')

local AssetSet = {}

-- CONSTRUCTOR --
function AssetSet.new( )
	
	-- PRE-CONSTRUCTOR --
    local self = DFUtil.deepCopy( AssetSet )	
    
	self._textures = {}
	self._spriteSheets = {}
    
    return self
end

-- PUBLIC FUNCTIONS --
function AssetSet:unload()
	
	-- ToDo: Make it possible to unload a asset-set that is being loaded
	assert(self:isLoaded())
		
	for sFilename, _ in pairs(self._textures) do
		DFGraphics.unloadTexture( sFilename )
	end
	self._textures = {}
	
	for sFilename, _ in pairs(self._spriteSheets) do
		DFGraphics.unloadSpriteSheet( sFilename )
	end
	self._spriteSheets = {}
end

function AssetSet:preloadTexture( sFilename )
	
	if self._textures[sFilename] == nil then
		self._textures[sFilename] = DFGraphics.loadTexture( sFilename, true )
	end
	return self._textures[sFilename]
end

function AssetSet:preloadSpriteSheet( sFilename )

	if self._spriteSheets[sFilename] == nil then
		self._spriteSheets[sFilename] = DFGraphics.loadSpriteSheet( sFilename, true )
	end
	return self._spriteSheets[sFilename]
end

function AssetSet:isLoaded()

	for _, rTexture in pairs(self._textures) do
		if not rTexture:isLoaded() then
			return false
		end
	end
	
	for _, rSpriteSheet in pairs(self._spriteSheets) do
		if not rSpriteSheet.texture:isLoaded() then
			return false
		end
	end
	
	return true
end

function AssetSet:getFailedFiles()

	local tFailed = {}
	
	for sFilename, rTexture in pairs(self._textures) do
		if rTexture:isLoaded() and not rTexture:loadSucceeded() then
			table.insert(tFailed, sFilename)
		end
	end
	
	for sFilename, rSpriteSheet in pairs(self._spriteSheets) do
		if rSpriteSheet.texture:isLoaded() and not rSpriteSheet.texture:loadSucceeded() then
			table.insert(tFailed, sFilename)
		end
	end
	
	return tFailed
	
end

function AssetSet:getMissingFiles()

	local sMissing = ""
	
	for sFilename, rTexture in pairs(self._textures) do
		if not rTexture:isLoaded() then
			sMissing = sMissing .. "TEXTURE: " .. sFilename .. "\n"
		end
	end
	
	for sFilename, rSpriteSheet in pairs(self._spriteSheets) do
		if not rSpriteSheet.texture:isLoaded() then
			sMissing = sMissing .. "SPRITESHEET: " .. sFilename .. "\n"
		end
	end
	
	return sMissing
end

return AssetSet