-- "m" for "module"
local m = {}

TT_Info = -5
TT_Gameplay = -4
TT_System = -3
TT_Error = -2
TT_Warning = -1 

TT_ENABLED = {
    [TT_Error]=true,
    [TT_Warning]=true,
    [TT_System]=true,
    [TT_Gameplay]=true,    
    [TT_Info]=true,
}

function Trace(type, sContents)
    if not sContents then
        sContents = type
        type = TT_Info
    end
    
    if TT_ENABLED[type] then
        if type == TT_Error then
            print(debug.traceback())
            sContents = 'TT_Error: ' .. sContents
        elseif type == TT_Warning then
            sContents = 'TT_Warning: ' .. sContents
        end
	    print(sContents)
    end
end

-- like Trace, but calls print() instead of print(string.format(...))
function Print(type, ...)
    local sType
    if TT_ENABLED[type] or type==nil then
        if type == TT_Error then
            print(debug.traceback())
            sType = 'TT_Error: '
        elseif type == TT_Warning then
            sType = 'TT_Warning: '
        end
        print(sType, ...)
    end
end

function m.printTable(t, ttype, prefix)
    prefix = prefix or ""
    if not t or not type(t) == "table" then
        Print(ttype, prefix.."NIL")
        return
    end
    for k,v in pairs(t) do
        Print(ttype, prefix,k,v)
        if type(v) == "table" then
            m.printTable(v, ttype, prefix .. " - ")
        end
    end
end

function m.deepCopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, getmetatable(object))
    end
    return _copy(object)
end

-- just here for us to put a breakpoint.
function m.assert(cond, ...)
    if not cond then
        assert(cond, ...)
    end
end

function m.timedCallback(delay, loop, func, ...)
    local timer = MOAITimer.new()
    timer:setSpan( delay )
    if loop then timer:setMode(MOAITimer.LOOP) end
    timer:setListener( MOAITimer.EVENT_TIMER_END_SPAN, function () func(unpack( arg )) end)
    timer:start()
end

function m.sleep(time)
	local rTimer = MOAITimer.new()
	rTimer:setSpan(0, time)
    MOAICoroutine.blockOnAction(rTimer:start())	
end

function m.mergeArrayInto(listA, listB)
    for i,val in ipairs(listB) do
        table.insert(listA, val)
    end
end

function m.split(input, sep)
    local sep, fields = sep or ":", {}
    local pattern = string.format("([^%s]+)", sep)
    input:gsub(pattern, function(c) fields[#fields+1] = c end)
    return fields
end

function m.removeFromArray(array, removeValue)
    for index, value in ipairs(array) do
        if value == removeValue then
            table.remove(array, index)
            return
        end
    end
end

function m.arrayRandom(array)
    return array[math.random(1,#array)]
end

function m.arrayShuffle(array)
    -- see: http://en.wikipedia.org/wiki/Fisher-Yates_shuffle
    local n = #array
    while n >= 2 do
        -- n is now the last pertinent index
        local k = math.random(n) -- 1 <= k <= n
        -- Quick swap
        array[n], array[k] = array[k], array[n]
        n = n - 1
    end

    return array
end

-- Return an empty table set up to inherit from |parent_class|
function m.createSubclass(parentClass)
	assert(parentClass)
	local klass = {}
	klass.Parent = parentClass
	setmetatable(klass, { __index = parentClass })

    while parentClass do
        klass._UserData = parentClass
        parentClass = parentClass.Parent
    end

	return klass
end

return m
