local DFUtil = require('DFCommon.Util')
local function _createCopy(tOrigTable, tOverrides)
    local tNewTable = DFUtil.deepCopy(tOrigTable)
    if tOverrides then
        for key, value in pairs(tOverrides) do
            tNewTable[key] = value
        end
    end
    return tNewTable
end

local Thug = 
{
    sName = "NAME009TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Thug/Thug01",
                "Characters/Villains/Thug/Thug01_2",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/ThugAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons3',
    },
    tPortraitIndex = {
        default = 'thug',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'ThugEncounter',
    tRewardOnKO = { 0, 5 },
}

local ThugIntroOverride =
{
    tInitialStats = { { nLevel = 1, nPower = 3, nAwareness = 1, nMaxHP = 20 } },
}

local ThugLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 3, nAwareness = 1, nMaxHP = 20 } },
}

local ThugB = 
{
    sName = "NAME009TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Thug/Thug01_B",
                "Characters/Villains/Thug/Thug01_B_2",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/ThugAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons3',
    },
    tPortraitIndex = {
        default = 'thug_b',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    bTriggerAIAbility = true,
    sSequentialResponseSet = 'ThugEncounter',
    tRewardOnKO = { 0, 10 },
}

local ThugBLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 6, nAwareness = 1, nMaxHP = 40 } },
}

local ThugBLvl2Override =
{
    tInitialStats = { { nLevel = 2, nPower = 14, nAwareness = 1, nMaxHP = 140 } },
}

local SeniorThug = 
{
    sName = "NAME031TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Thug/Thug01_B",
                "Characters/Villains/Thug/Thug01_B_2",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/ThugAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons3',
    },
    tPortraitIndex = {
        default = 'thug_b',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    bTriggerAIAbility = true,
    sSequentialResponseSet = 'ThugEncounter',
    tRewardOnKO = { 0, 10 },
}

local SeniorThugLvl1Override =
{
    tInitialStats = { { nLevel = 5, nPower = 18, nAwareness = 1, nMaxHP = 125 } },
}
local SeniorThugLv21Override =
{
    tInitialStats = { { nLevel = 5, nPower = 20, nAwareness = 1, nMaxHP = 300 } },
}

local SeniorHenchmanRanged =
{
    sName = "NAME032TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01",
                "Characters/Henchmen/Henchman_01/Henchman01_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanraygun'
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
}

local SeniorHenchmanRangedLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 16, nAwareness = 1, nMaxHP = 150 } },
}
local SeniorHenchmanRangedLvl2Override =
{
    tInitialStats = { { nLevel = 1, nPower = 25, nAwareness = 1, nMaxHP = 300 } },
}


local HenchmanMelee =
{                       
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01",
                "Characters/Henchmen/Henchman_01/Henchman01_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanBAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanpunch',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
}

local HenchmanMeleeLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 2, nAwareness = 1, nMaxHP = 75 } },
}

local HenchmanRanged =
{
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01",
                "Characters/Henchmen/Henchman_01/Henchman01_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanraygun'
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
}

local HenchmanRangedLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 2, nAwareness = 1, nMaxHP = 50 } },
}

local MSHenchmanMelee =
{                       
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01_B_MadScientist",
                "Characters/Henchmen/Henchman_01/Henchman01_B_MadScientist_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanBAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman_madscientist',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanpunch',
        'henchmanbigpunch',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
    tRewardOnKO = { 0, 10 },
}

local MSHenchmanMeleeLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 8, nAwareness = 1, nMaxHP = 45 } },
}
local MSHenchmanMeleeLvl2Override =
{
    tInitialStats = { { nLevel = 2, nPower = 18, nAwareness = 1, nMaxHP = 80 } },
}

local MSHenchmanRanged =
{
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01_MadScientist",
                "Characters/Henchmen/Henchman_01/Henchman01_MadScientist_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MSHenchmanAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman_madscientist',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanraygun',
        'henchmanspecialraygun',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
    tRewardOnKO = { 0, 10 },
}

local MSHenchmanRangedLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 14, nAwareness = 1, nMaxHP = 35 } },
}
local MSHenchmanRangedLvl2Override =
{
    tInitialStats = { { nLevel = 2, nPower = 18, nAwareness = 1, nMaxHP = 80 } },
}

local SkullHenchmanMelee =
{                       
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01_B_Skullface",
                "Characters/Henchmen/Henchman_01/Henchman01_B_Skullface_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanBAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman_skullface',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanpunch',
        'henchmanbigpunch',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
    tRewardOnKO = { 0, 15 },
}

local SkullHenchmanMeleeLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 16, nAwareness = 1, nMaxHP = 100 } },
}

local SkullHenchmanRanged =
{
    sName = "NAME010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Henchmen/Henchman_01/Henchman01_Skullface",
                "Characters/Henchmen/Henchman_01/Henchman01_Skullface_2"
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/HenchmanAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'henchman_skullface',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/Henchmen.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'henchmanraygun',
        'henchmanspecialraygun',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sRandomResponseSet = 'HenchmanHeroReact',
    tRewardOnKO = { 0, 15 },
}

local SkullHenchmanRangedLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 18, nAwareness = 1, nMaxHP = 80 } },
}

--little alien in the saucer
local AlienRanged =
{
    sName = "NAME022TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Alien_Sm/Alien_Sm",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/AlienAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'alien_ranged',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/alien_ranged.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'alienlaser',
    },
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'AlienEncounter',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 30 },
    bNoShadow = true,
}

local AlienRangedLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 35, nAwareness = 1, nMaxHP = 400 } },
}


local AlienMelee_Male = 
{
    sName = "NAME018TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Alien_Body/Body",
                "Characters/Villains/Alien_Body/Heads",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/AlienBody_BodyAnims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/Alien_Body/Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/AlienBody_MaleAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'alien_melee_male',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/alien_melee_male.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'alienpunch',
        'alienpunch2',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'AlienBodyEncounter',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
    sCustomDeathThread = 'spawnCitizen',
    sCitizenCharDefToSpawn = 'CitiMan',
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/Alien_Death',
}

local AlienMeleeMaleLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 30, nAwareness = 1, nMaxHP = 550 } },
}

local AlienMelee_Female = 
{
    sName = "NAME017TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Alien_Body/Body",
                "Characters/Villains/Alien_Body/Heads",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/AlienBody_BodyAnims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/Alien_Body/Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/AlienBody_FemaleAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'alien_melee_female',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/alien_melee_female.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'alienpunch',
        'alienpunch2',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'AlienBodyEncounter',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
    sCustomDeathThread = 'spawnCitizen',
    sCitizenCharDefToSpawn = 'CitiGirl',
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/Alien_Death',
}

local AlienMeleeFemaleLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 30, nAwareness = 1, nMaxHP = 550 } },
}

local AlienMelee_Alien = 
{
    sName = "NAME022TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Alien_Body/Body",
                "Characters/Villains/Alien_Body/Heads_Alien",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/AlienBody_BodyAnims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/Alien_Body/Heads_Alien", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/AlienBody_AlienAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'alien_melee_alien',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "CharacterIcons/alien_melee_alien.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'alienpunch',
        'alienpunch2',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'AlienBodyEncounter_Alien',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
    sCustomDeathThread = 'spawnEnemy',
    sEnemyCharDefToSpawn = 'AlienRangedLvl1',
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/Alien_Death',
}

local AlienMeleeAlienLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 30, nAwareness = 1, nMaxHP = 450 } },
}

return
{   
    ThugIntro = _createCopy(Thug, ThugIntroOverride),  
    ThugLvl1 = _createCopy(Thug, ThugLvl1Override),      

    ThugBLvl1 = _createCopy(ThugB, ThugBLvl1Override),    
    ThugBLvl2 = _createCopy(ThugB, ThugBLvl2Override),

    SeniorThugLvl1 = _createCopy(SeniorThug, SeniorThugLvl1Override),  

    SeniorHenchmanRangedLvl1 = _createCopy(SeniorHenchmanRanged, SeniorHenchmanRangedLvl1Override),    
    SeniorHenchmanRangedLvl2 = _createCopy(SeniorHenchmanRanged, SeniorHenchmanRangedLvl2Override),    
    
    HenchmanMeleeLvl1 = _createCopy(HenchmanMelee, HenchmanMeleeLvl1Override),

    HenchmanRangedLvl1 = _createCopy(HenchmanRanged, HenchmanRangedLvl1Override),

    MSHenchmanMeleeLvl1 = _createCopy(MSHenchmanMelee, MSHenchmanMeleeLvl1Override),
    MSHenchmanMeleeLvl2 = _createCopy(MSHenchmanMelee, MSHenchmanMeleeLvl2Override),

    MSHenchmanRangedLvl1 = _createCopy(MSHenchmanRanged, MSHenchmanRangedLvl1Override),
    MSHenchmanRangedLvl2 = _createCopy(MSHenchmanRanged, MSHenchmanRangedLvl2Override),

    SkullHenchmanMeleeLvl1 = _createCopy(SkullHenchmanMelee, SkullHenchmanMeleeLvl1Override),

    SkullHenchmanRangedLvl1 = _createCopy(SkullHenchmanRanged, SkullHenchmanRangedLvl1Override),
    
    AlienRangedLvl1 = _createCopy(AlienRanged, AlienRangedLvl1Override),
    
    AlienMeleeAlienLvl1 = _createCopy(AlienMelee_Alien, AlienMeleeAlienLvl1Override),
    
    AlienMeleeMaleLvl1 = _createCopy(AlienMelee_Male, AlienMeleeMaleLvl1Override),
    
    AlienMeleeFemaleLvl1 = _createCopy(AlienMelee_Female, AlienMeleeFemaleLvl1Override),
}