local DFUtil = require('DFCommon.Util')
local function _createCopy(tOrigTable, tOverrides)
    local tNewTable = DFUtil.deepCopy(tOrigTable)
    if tOverrides then
        for key, value in pairs(tOverrides) do
            tNewTable[key] = value
        end
    end
    return tNewTable
end

--BOSSES
local SkullFace =
{
    sName = "NAME011TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/Villain_01/Villain_01",
                "Characters/Villains/Villain_01/Villain_01_2",
                "Characters/Villains/Villain_01/Villain_01_3",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/Villain01Anims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons3',
    },
    tPortraitIndex = {
        default = 'skullface',
    },
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'skullfacepunch',
--        'skullfacestun',
        'skullfaceblast',
    },
    bTriggerAIAbility = true,
}

local SkullFaceLvl1Override =
{
    tInitialStats = { { nLevel = 1, nPower = 12, nAwareness = 1, nMaxHP = 80 } },
}
local SkullFaceLvl2Override =
{
    tInitialStats = { { nLevel = 2, nPower = 30, nAwareness = 1, nMaxHP = 250 } },
}
local SkullFaceLvl3Override =
{
    tInitialStats = { { nLevel = 3, nPower = 40, nAwareness = 1, nMaxHP = 300 } },
}
local SkullFaceLvl4Override =
{
    tInitialStats = { { nLevel = 4, nPower = 28, nAwareness = 1, nMaxHP = 300, nCharisma = 5, nResearch = 0 } },
}
local SkullFaceLvl5Override =
{
    tInitialStats = { { nLevel = 5, nPower = 34, nAwareness = 1, nMaxHP = 360, nCharisma = 5, nResearch = 0 } },
}

local MadScientist =
{
    sName = "NAME012TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/MadScientist/MadScientist01",
                "Characters/Villains/MadScientist/MadScientist02",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MadScientistAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'madscientist',
    },
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'madscientistraygun'
    },
}

local MadScientistLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 30, nAwareness = 1, nMaxHP = 250 } },
}
local MadScientistLvl2Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 31, nAwareness = 1, nMaxHP = 210, nCharisma = 5, nResearch = 0 } },
}
local MadScientistLvl3Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 37, nAwareness = 1, nMaxHP = 270, nCharisma = 5, nResearch = 0 } },
}
local MadScientistLvl4Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 43, nAwareness = 1, nMaxHP = 330, nCharisma = 5, nResearch = 0 } },
}
local MadScientistLvl5Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 49, nAwareness = 1, nMaxHP = 390, nCharisma = 5, nResearch = 0 } },
}

local RobotFoot =
{
    sName = "NAME014TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/BossFights/RobotFoot/RobotFoot",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/DefaultGiantFootAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons3',
    },
    tPortraitIndex = {
        default = 'robotfoot',
    },
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    tScaleInfo =
    {
        SMALL = 1.5,
        MED = 1.5,
        LARGE = 3,
    },
    tAbilityDefKeys =
    {
        'robotfootstomp',
    },
    bNoHitReact = true,
    bTriggerAIAbility = true,
    bNoShadow = true,
    rHitEffectDefOverride =
    {
        sEffectPath = 'Effects/ImpactBolts.lua',
        offSetRange = { x = { -280, -250 }, y = { 130, 130 } },
        scaleRange = { 1.75, 2 },
    },
    tXOffsetOverride = { -324, -320, -320, -324, -324, -320, -320 },
    tYOffsetOverride = { 100, 40, 80, 30, 90 },
}

local RobotFootLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 20, nAwareness = 1, nMaxHP = 500 } },
}
local RobotFootLvl2Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 31, nAwareness = 1, nMaxHP = 210, nCharisma = 5, nResearch = 0 } },
}
local RobotFootLvl3Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 37, nAwareness = 1, nMaxHP = 270, nCharisma = 5, nResearch = 0 } },
}
local RobotFootLvl4Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 43, nAwareness = 1, nMaxHP = 330, nCharisma = 5, nResearch = 0 } },
}
local RobotFootLvl5Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 49, nAwareness = 1, nMaxHP = 390, nCharisma = 5, nResearch = 0 } },
}

local MonsterFoot =
{
    sName = "NAME013TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/BossFights/MonsterFoot/MonsterFoot",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/DefaultGiantFootAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'monsterfoot',
    },
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    tScaleInfo =
    {
        SMALL = 1.5,
        MED = 1.5,
        LARGE = 3,
    },
    bNoHitReact = true,
    bNoShadow = true,    
    bTriggerAIAbility = true,
    tAbilityDefKeys =
    {
        'monsterfootstomp',
    },
    tXOffsetOverride = { -324, -320, -320, -324, -324, -320, -320 },
    tYOffsetOverride = { 100, 40, 80, 30, 90 },
}

local MonsterFootLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 40, nAwareness = 1, nMaxHP = 1000, nCharisma = 5, nResearch = 0 } },
}
local MonsterFootLvl2Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 31, nAwareness = 1, nMaxHP = 210, nCharisma = 5, nResearch = 0 } },
}
local MonsterFootLvl3Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 37, nAwareness = 1, nMaxHP = 270, nCharisma = 5, nResearch = 0 } },
}
local MonsterFootLvl4Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 43, nAwareness = 1, nMaxHP = 330, nCharisma = 5, nResearch = 0 } },
}
local MonsterFootLvl5Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 49, nAwareness = 1, nMaxHP = 390, nCharisma = 5, nResearch = 0 } },
}

local CrowbarVillain = 
{
    sName = "NAME026TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Heroes/Male_01d/Male01d_fight_Lvl1",
                "Characters/Heroes/Male_01d/Male01d_fight_2_Lvl1",
                "Characters/Heroes/Male_01d/Male01d_fight_3_Lvl1",
            },
        },
         sAnimDefFileName = "Characters/AnimDefs/Male01dFightAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'male01d',
    },
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = true,
    tAbilityDefKeys =
    {
        'male01dpunch',
        'male01dcrowbar',
    },
    bTriggerAIAbility = true,
}

local CrowbarVillainLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 99, nAwareness = 1, nMaxHP = 1300, nCharisma = 5, nResearch = 0 } },
}

local AlienBoss =
{
    sName = "NAME023TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/BossFights/Alien_Lg/Alien_Lg",
                "Characters/BossFights/Alien_Lg/Alien_Lg_2",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/AlienBossAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'alien_boss',
    },
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = true,
    tAbilityDefKeys =
    {
            'alienBossLaser',
    },
    tScaleInfo =
    {
        SMALL = 1.25,
        MED = 1.25,
        LARGE = 1.25,
    },
    bTriggerAIAbility = true,
    bNoShadow = true,
}

local AlienBossLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 50, nAwareness = 1, nMaxHP = 1500, nCharisma = 5, nResearch = 0 } },
}

-- ==== MOST WANTED ===
--Dr. Shiv
local MostWanted01 =
{
    sName = "NAME028TEXT",
    sDescription = "RAPS001TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body01",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body01Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_PatchAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted01',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted01Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 125, nAwareness = 1, nMaxHP = 2500, nCharisma = 5, nResearch = 0 } },
}
--Dirty Rat
local MostWanted02 =
{
    sName = "NAME037TEXT",
    sDescription = "RAPS002TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body02",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body02Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Hat01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_MustacheThinAnims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_SunglassesAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted02',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted02.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted02Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 50, nAwareness = 1, nMaxHP = 800, nCharisma = 5, nResearch = 0 } },
}
--Admin of Murder
local MostWanted03 =
{
    sName = "NAME033TEXT",
    sDescription = "RAPS003TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body03",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body03Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Hat02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_MustacheAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted03',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted03.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted03Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 40, nAwareness = 1, nMaxHP = 500, nCharisma = 5, nResearch = 0 } },
}
--Slapshot
local MostWanted04 =
{
    sName = "NAME035TEXT",
    sDescription = "RAPS004TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body04",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body04Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_HockeyMaskAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted04',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted04.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted04Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 100, nAwareness = 1, nMaxHP = 1500, nCharisma = 5, nResearch = 0 } },
}
--Mustafa Jr.
local MostWanted05 =
{
    sName = "NAME039TEXT",
    sDescription = "RAPS005TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body05",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body05Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_GoateeAnims.lua" },        
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted05',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted03.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted05Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 50, nAwareness = 1, nMaxHP = 800, nCharisma = 5, nResearch = 0 } },
}
--Mr. Meathooks
local MostWanted06 =
{
    sName = "NAME029TEXT",
    sDescription = "RAPS006TEXT",
    
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body01",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body01Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_MustacheAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted06',
    },
    
    iconImgPath = "UI/icons/mostwanted06.png",
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted06Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 150, nAwareness = 1, nMaxHP = 3500, nCharisma = 5, nResearch = 0 } },
}

--Kneecapper
local MostWanted07 =
{
    sName = "NAME036TEXT",
    sDescription = "RAPS007TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body02",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body02Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head03Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Hat02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_GoateeAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted07',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted07.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted07Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 150, nAwareness = 1, nMaxHP = 3500, nCharisma = 5, nResearch = 0 } },
}

--Patches
local MostWanted08 =
{
    sName = "NAME040TEXT",
    sDescription = "RAPS008TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body03",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body03Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head02Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Hat01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_PatchAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted08',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted08.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted08Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 40, nAwareness = 1, nMaxHP = 500, nCharisma = 5, nResearch = 0 } },
}

--Heistmeister
local MostWanted09 =
{
    sName = "NAME038TEXT",
    sDescription = "RAPS009TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body04",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body04Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head03Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_MustacheThinAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted09',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted09.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted09Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 100, nAwareness = 1, nMaxHP = 1500, nCharisma = 5, nResearch = 0 } },
}
--Mugging Director
local MostWanted10 =
{
    sName = "NAME030TEXT",    
    sDescription = "RAPS010TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Villains/EliteThug/EliteThug_Body05",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/MostWanted_Body05Anims.lua",
    },
    tAdditionalSpriteSheets =
    {
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Heads", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_Head01Anims.lua" },
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories02", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_MustacheAnims.lua" },        
        { sSpriteSheetDefPath = "Characters/Villains/EliteThug/EliteThug_Accessories01", nYOffset = 0, sAnimDefFileName = "Characters/AnimDefs/MostWanted_SunglassesAnims.lua" },
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'mostwanted10',
    },
    centerToBaseX = 0,
    centerToBaseY = -10,
    initalAnimation = "fight_breathe",
    iconImgPath = "UI/icons/mostwanted10.png",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = true,
	bStunnable = false,
    tAbilityDefKeys =
    {
        'thugpunch',
        'thugbigswing',
    },
    bTriggerAIAbility = true,
    tKOAchievements =
    {
        'BeatFiftyThugs',
        'BeatTwoHundredThugs',
        'BeatOneThousandThugs',
    },
    sSequentialResponseSet = 'MostWantedEncounter',
    sLoseOutro = 'MostWantedLoseOutro',
    bForceSpecialDialog = true,
    tRewardOnKO = { 0, 5 },
}

local MostWanted10Lvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 125, nAwareness = 1, nMaxHP = 2500, nCharisma = 5, nResearch = 0 } },
}

-- GODZILLAS
local Beastfly =
{
    sName = "NAME034TEXT",
    tFightEventSpriteSheetInfo = {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/BossFights/Beastfly/Beastfly_Lg",
                "Characters/BossFights/Beastfly/Beastfly_Lg_2",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/BeastflyAnims.lua",
    },
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'beastfly',
    },
    initalAnimation = "breathe",
    initPos = { x = 0, y =  0},
    textColor = {255,110,110}, 
    walkingSpeed = 2,
    bIsMelee = false,
	bStunnable = false,
    tAbilityDefKeys =
    {
            'barf',
    },
    tScaleInfo =
    {
        SMALL = 2,
        MED = 2,
        LARGE = 2,
    },
    bTriggerAIAbility = true,
    bNoShadow = true,
}
local BeastflyLvl1Override = 
{
    tInitialStats = { { nLevel = 5, nPower = 50, nAwareness = 1, nMaxHP = 10000, nCharisma = 5, nResearch = 0 } },
}

--////////////  BOSS PROPS  ///////////
local RocketLauncher = { 	
    sName = "NAME016TEXT",
--    bManuallyTriggerAttack = true,
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Props/MadScientistProps",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/RocketLauncherAnim.lua",
    }, 
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'boxingglovemissile',
    },
    initalAnimation = "breathe",
    textColor = {255,120,110}, 
    tInitialStats = {
        { nLevel = 1, nPower = 18, nAwareness = 1, nMaxHP = 150 },
    },
    walkingSpeed = 4,
    bIsMelee = false,
    bNoHitReact = true,
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/RocketPunchExplode',
    tAbilityDefKeys =
    {
        'rocketfire',
    },
}   

local CloningMachine = { 	
    bManuallyTriggerAttack = true,
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Props/MadScientistProps",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/CloningMachineAnim.lua",
    }, 
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'cloningmachine',
    },
    initalAnimation = "breathe",
    textColor = {255,120,110}, 
    tInitialStats = {
        { nLevel = 1, nPower = 10, nAwareness = 1, nMaxHP = 200, nCharisma = 5, nResearch = 0 },
        { nLevel = 5, nPower = 9, nAwareness = 1, nMaxHP = 140, nCharisma = 10, nResearch = 0 },
        { nLevel = 10, nPower = 14, nAwareness = 1, nMaxHP = 190, nCharisma = 15, nResearch = 0 },
    },
    walkingSpeed = 4,
    bIsMelee = false,
    bNoHitReact = true,
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/RocketPunchExplode',
    tAbilityDefKeys =
    {
        'rocketfire',
    },
    bNoShadow = true,
}   

local LoveGunSmall = { 	
    sName = "NAME024TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Props/MadScientistProps",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/LoveGunSmallAnim.lua",
    }, 
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'boxingglovemissile',
    },
    initalAnimation = "breathe",
    textColor = {255,120,110}, 
    tInitialStats = {
        { nLevel = 1, nPower = 1, nAwareness = 1, nMaxHP = 150 },
    },
    walkingSpeed = 4,
    bIsMelee = false,
    bNoHitReact = true,
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/RocketPunchExplode',
    tAbilityDefKeys =
    {
        'smallloveblast',
    },
    bNoHitReact = true,
    bTriggerAIAbility = true,
    bNoShadow = true,
    sCustomDeathThread = 'loveGunExplosion',
    rHitEffectDefOverride =
    {
        sEffectPath = 'Effects/ImpactBolts.lua',
--        offSetRange = { x = { -280, -250 }, y = { 130, 130 } },
        scaleRange = { 1, 1 },
    },
}   

local LoveGunLarge = { 	
    sName = "NAME025TEXT",
    tFightEventSpriteSheetInfo =
    {
        tSpriteSheetDefPaths = 
        {
            default = 
            {
                "Characters/Props/MadScientistProps",
            },
        },
        sAnimDefFileName = "Characters/AnimDefs/LoveGunLargeAnim.lua",
    }, 
    tPortraitPaths = {
        default = 'UI/icons2',
    },
    tPortraitIndex = {
        default = 'boxingglovemissile',
    },
    initalAnimation = "breathe",
    textColor = {255,120,110}, 
    tInitialStats = {
        { nLevel = 1, nPower = 1, nAwareness = 1, nMaxHP = 300 },
    },
    walkingSpeed = 4,
    bIsMelee = false,
    bNoHitReact = true,
    sKOSoundCue = 'MMOJ_Audio/SFX/EnemySFX/RocketPunchExplode',
    tAbilityDefKeys =
    {
        'largeloveblast',
    },
    bNoHitReact = true,
    bTriggerAIAbility = true,
    bNoShadow = true,
    rHitEffectDefOverride =
    {
        sEffectPath = 'Effects/ImpactBolts.lua',
        offSetRange = { x = { -280, -250 }, y = { 130, 130 } },
        scaleRange = { 1, 1 },
    },
}   



tDefs = 
{
    SkullFaceLvl1 = _createCopy(SkullFace, SkullFaceLvl1Override),
    SkullFaceLvl2 = _createCopy(SkullFace, SkullFaceLvl2Override),
    SkullFaceLvl3 = _createCopy(SkullFace, SkullFaceLvl3Override),
    SkullFaceLvl4 = _createCopy(SkullFace, SkullFaceLvl4Override),
    SkullFaceLvl5 = _createCopy(SkullFace, SkullFaceLvl5Override),

    MadScientistLvl1 = _createCopy(MadScientist, MadScientistLvl1Override),
    MadScientistLvl2 = _createCopy(MadScientist, MadScientistLvl2Override),
    MadScientistLvl3 = _createCopy(MadScientist, MadScientistLvl3Override),
    MadScientistLvl4 = _createCopy(MadScientist, MadScientistLvl4Override),
    MadScientistLvl5 = _createCopy(MadScientist, MadScientistLvl5Override),

    RobotFootLvl1 = _createCopy(RobotFoot, RobotFootLvl1Override),
    RobotFootLvl2 = _createCopy(RobotFoot, RobotFootLvl2Override),
    RobotFootLvl3 = _createCopy(RobotFoot, RobotFootLvl3Override),
    RobotFootLvl4 = _createCopy(RobotFoot, RobotFootLvl4Override),
    RobotFootLvl5 = _createCopy(RobotFoot, RobotFootLvl5Override),

    MonsterFootLvl1 = _createCopy(MonsterFoot, MonsterFootLvl1Override),
    MonsterFootLvl2 = _createCopy(MonsterFoot, MonsterFootLvl2Override),
    MonsterFootLvl3 = _createCopy(MonsterFoot, MonsterFootLvl3Override),
    MonsterFootLvl4 = _createCopy(MonsterFoot, MonsterFootLvl4Override),
    MonsterFootLvl5 = _createCopy(MonsterFoot, MonsterFootLvl5Override),
        
    AlienBossLvl1 = _createCopy(AlienBoss, AlienBossLvl1Override),
    
    BeastflyLvl1 = _createCopy(Beastfly, BeastflyLvl1Override),
    
    MostWanted01Lvl1 = _createCopy(MostWanted01, MostWanted01Lvl1Override),
    MostWanted02Lvl1 = _createCopy(MostWanted02, MostWanted02Lvl1Override),
    MostWanted03Lvl1 = _createCopy(MostWanted03, MostWanted03Lvl1Override),
    MostWanted04Lvl1 = _createCopy(MostWanted04, MostWanted04Lvl1Override),
    MostWanted05Lvl1 = _createCopy(MostWanted05, MostWanted05Lvl1Override),
    MostWanted06Lvl1 = _createCopy(MostWanted06, MostWanted06Lvl1Override),
    MostWanted07Lvl1 = _createCopy(MostWanted07, MostWanted07Lvl1Override),
    MostWanted08Lvl1 = _createCopy(MostWanted08, MostWanted08Lvl1Override),
    MostWanted09Lvl1 = _createCopy(MostWanted09, MostWanted09Lvl1Override),
    MostWanted10Lvl1 = _createCopy(MostWanted10, MostWanted10Lvl1Override),

    RocketLauncher = RocketLauncher,
    CloningMachine = CloningMachine,
    LoveGunSmall = LoveGunSmall,
    LoveGunLarge = LoveGunLarge,

    CrowbarVillainLvl1 = _createCopy(CrowbarVillain, CrowbarVillainLvl1Override),
}

local tMostWantedThugs =
{
    'MostWanted10Lvl1', -- mugging director
    'MostWanted01Lvl1', -- dr. shiv
    'MostWanted08Lvl1', -- patches
    'MostWanted09Lvl1', -- heistmeister
    'MostWanted06Lvl1', -- mr. meathooks
    'MostWanted07Lvl1', -- kneecapper
    'MostWanted05Lvl1', -- mustafa jr.
    'MostWanted03Lvl1', -- admin of murder
    'MostWanted04Lvl1', -- slapshot
    'MostWanted02Lvl1', -- dirty rat    
}

return
{
    tDefs = tDefs,    
    tMostWantedThugs = tMostWantedThugs,
}