local Ability = {}
local AbilityDamage = {}
--local AbilityBuffFriendly = {}
local AbilityTriggerAttack = {}
local AbilityCycleStance = {}
local Projectile = require('Ability/Projectile')
local util = require('DFUtil')
local Renderer = require('Renderer')
local SpriteSheet = require('SpriteSheet')
local EffectInstance = require('EffectInstance')
local DFAchievement = require('DFCommon.Achievement')
local CountdownUI = require('ui/CountdownUI')

Ability.bDebugUnlockAbilities = false

function Ability.getMaxLevel()
    return 3
end

Ability.tMiscTweaks =
{
   nManaCostForRoomLvl3 = 8,
   nManaCostForRoomLvl2 = 5,
}

function Ability.updateMiscTweakables(sDefKey, tValues)
    g_Game:updateDefWithTweak(Ability.tMiscTweaks, sDefKey, tValues)
end

function Ability:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)
    local Ob = {}
    Ob.__typename = "Ability"
    Ob.sDefKey = sDefKey
    if Ability.tAbilityReferencedData[Ob.sDefKey] then
        Ob.tRefData = Ability.tAbilityReferencedData[Ob.sDefKey] 
    end
    Ob.rEventActor = rEventActor
    Ob.rCharacter = rCharacter
    -- copy in the values from the def
    for key, value in pairs(rDef) do
        Ob[key] = value
    end

    Ob.rDef = rDef
    Ob.nCoolDown = Ob.rDef.nCoolDown or 0
    Ob.nElapsed = Ob.rDef.nCoolDown
    Ob.nChargeTime = 0
    Ob.tSaved = tSaved
    if not Ob.tSaved then
        Ob.tSaved = {}
    end
    Ob.bIsActive = false
    if not Ob.tSaved.nCurLevel then
        Ob.tSaved.nCurLevel = 1
    end
    if not Ob.tSaved.nNumTimesUsed then
        Ob.tSaved.nNumTimesUsed = 0
    end
    if not Ob.tSaved.bIsUnlocked then
        Ob.tSaved.bIsUnlocked = Ob.rDef.bIsUnlocked
    end
    if (rDef.nRealTimeCooldown or rDef.nRealTimeCooldowns) and not Ob.tSaved.nRemainingRTCooldown then
        Ob.tSaved.nRemainingRTCooldown = -1
    end

    Ob.nTurnElapsed = 0
    Ob.nTurnBegan = 0

    function Ob:getDef()
        return self.rDef
    end

    function Ob:getDescription()
        if self.rDef.sDescription then
            return g_DialogManager:getString(self.rDef.sDescription)
        else
            return ""
        end
    end

    function Ob:onTurnBegan()
        self.nTurnBegan = self.nTurnBegan + 1
    end

    function Ob:onTurnElapsed()
        self.nTurnElapsed = self.nTurnElapsed + 1
    end

    function Ob:resetNumTurns()
        if self.rDef.nCooldownTurns then
            self.nTurnElapsed = self.rDef.nCooldownTurns
        else
            self.nTurnElapsed = 0
        end
        self.nTurnBegan = 0
        local nRTCD = self:getRealTimeCooldown()
        if nRTCD then self.tSaved.nRemainingRTCooldown = nRTCD end
    end

    function Ob:getRealTimeCooldown()
        if self.rDef.nRealTimeCooldown then
            return self.rDef.nRealTimeCooldown 
        elseif self.rDef.nRealTimeCooldowns then
            return self.rDef.nRealTimeCooldowns[self:getCurLevel()]
        end
    end

    function Ob:getManaCost()
        local cost = 0
        if self.tRefData and self.tRefData.tManaCost then
            cost = self.tRefData.tManaCost[self.tSaved.nCurLevel]
        end
        if self.rCharacter then
            local nMult = self.rCharacter:getManaCostMultiplier()
            cost = cost - nMult * cost
            if cost < 0 then
                cost = 0
            end
        end
        return cost
    end

    function Ob:getTurnCooldownPct()
        --[[
        -- PROGRESS: testing mana. ignoring cooldown.
        if self.rDef.nCooldownTurns and self.nTurnElapsed then
            local nPct = self.nTurnElapsed / self.rDef.nCooldownTurns
            if nPct > 1 then
                nPct = 1
            end
            return nPct
        end
        ]]--
        if self.rCharacter and self.rCharacter:getCurMana() < self:getManaCost() then
            return 0
        end
        local nRTCD = self:getRealTimeCooldown()
        if nRTCD then
            return 1-math.max(0, self.tSaved.nRemainingRTCooldown / nRTCD)
        end
        return 1
    end

    function Ob:tickRealTimeCooldown(nElapsed)
        local nRTCD = self:getRealTimeCooldown()
        if nRTCD then
            self.tSaved.nRemainingRTCooldown = self.tSaved.nRemainingRTCooldown - nElapsed
        end
    end

    function Ob:isUnlocked()
        if Ability.bDebugUnlockAbilities then
            return true
        end
        if self.tSaved.bIsUnlocked then
            return true
        end
        return false
    end

    function Ob:setUnlocked(bUnlocked)
        self.tSaved.bIsUnlocked = bUnlocked
    end

    function Ob:getNumTimesUsed()
        return self.tSaved.nNumTimesUsed
    end

    function Ob:incrementNumTimesUsed(nAmount)
        self.tSaved.nNumTimesUsed = self.tSaved.nNumTimesUsed + 1
        if self.tSaved.nNumTimesUsed >= self:getNumForNextLevel() then
            self:incrementLevel()
            self.tSaved.nNumTimesUsed = 0
            return true
        end
        return false
    end

    function Ob:isAtMaxLevel()
        local nMaxLevel = Ability.getMaxLevel()
        if self.tSaved.nCurLevel >= nMaxLevel then
            return true
        end
        return false
    end

    function Ob:incrementLevel()
        local nMaxLevel = Ability.getMaxLevel()
        if self.tSaved.nCurLevel < nMaxLevel then
            self.tSaved.nCurLevel = self.tSaved.nCurLevel + 1

            -- handle case where cooldown shrinks
            local nRTCD = self:getRealTimeCooldown()
            if nRTCD then self.tSaved.nRemainingRTCooldown = math.min(self.tSaved.nRemainingRTCooldown, nRTCD) end

            if self:isAtMaxLevel() then
                DFAchievement.updateAchievement('MaxLevelOneAbility', 1)
                if self.rCharacter:areAllAbilitiesAtMaxLevel() then
                    DFAchievement.updateAchievement('MaxLevelAllAbilities', 1)
                    if self.rCharacter:isAtMaxLevel() then
                        DFAchievement.updateAchievement('MaxLevelAbilityAndHero', 1)
                        DFAchievement.updateAchievement('MaxLevelAbilityAndEightHero', self.rCharacter:getCharDefKey())
                    end
                end
            end
        end
    end

    function Ob:getNumForNextLevel()
        if Ability.tLevelImprovementInfo[self.tSaved.nCurLevel] then
            return Ability.tLevelImprovementInfo[self.tSaved.nCurLevel].nNumUseForNextLvl
        end
        return 1
    end

    function Ob:getCurLevel()
        return self.tSaved.nCurLevel
    end

    function Ob:getLevelMultiplier()
        local nLevel = self:getCurLevel()
        if nLevel > 1 then
            return (1 + nLevel / 2)
        else
            return nLevel
        end
    end


    function Ob:getUnlockLevel()
        return self.nMinLevel
    end

    function Ob:isDefault()
        return self.bIsDefault
    end

    function Ob:showInUI()
        return not self.bIsDefault and self.sIconName
    end

    function Ob:getName()
        return g_DialogManager:getString(self.sAbilityName)
    end

    function Ob:playAnim(rEventActor, sAnimName, bBlocking)
        local bFlipX = false
        if rEventActor:isHero() then
            bFlipX = true
        end
        return rEventActor:playAnimation(sAnimName, bFlipX, false, bBlocking)
    end

    function Ob:cooldownThread()
        local kTIME_STEP = 0.1
        self.nElapsed = 0
        while self.nElapsed < self.nCoolDown do
            util.sleep(kTIME_STEP)
            self.nElapsed = self.nElapsed + kTIME_STEP
        end
        self.nElapsed = self.nCoolDown
    end

    function Ob:canPerformAbility()
        if not self:isUnlocked() then
            return false
        end

        -- NOTE: goes straight to the character instead of the EventActor.
        if self.rCharacter and self.rCharacter:getCurMana() < self:getManaCost() then
            return false
        end
        if self.tSaved.nRemainingRTCooldown and self.tSaved.nRemainingRTCooldown > 0 then
            return false
        end
        --[[
        if self.rDef.nCooldownTurns then
            if self.nTurnElapsed < self.rDef.nCooldownTurns then
                return false
            end
        end
        ]]--
        if self:getCooldownPct() < 1 then
            return false
        end
        if self.bIsActive then
            return false
        end
        return true
    end

    function Ob:getCooldownPct()
        if self.nCoolDown ==  0 then
            return 1
        else
            return (self.nElapsed / self.nCoolDown)
        end
    end

    function Ob:getMaxChargeTime()
        return self.nMaxChargeTime or 1
    end

    function Ob:setChargeTime(nChargeTime)
        self.nChargeTime = nChargeTime
    end

    function Ob:getCharge() -- should be between 0 and 1
        if not self.nMaxChargeTime then
            return 1
        end
        return self.nChargeTime /  self.nMaxChargeTime
    end

    function Ob:isActive()
        return self.bIsActive
    end

    function Ob:getReqUpgradeLevel()
        if self.rDef.tReqLevel then
            if not self:isUnlocked() then
                if self.rDef.tReqLevel[1] then
                    return self.rDef.tReqLevel[1]
                end
            else
                if self.rDef.tReqLevel[self.tSaved.nCurLevel + 1] then
                    return self.rDef.tReqLevel[self.tSaved.nCurLevel + 1]
                end
            end
        end
        return 0
    end

    function Ob:getReqUpgradeAwareness()
        if self:isAtMaxLevel() then return 0 end

        if self.tRefData and self.tRefData.tManaCost then
             local nNextLevel = (self:isUnlocked() and self.tSaved.nCurLevel+1) or 1
             local cost = self.tRefData.tManaCost[nNextLevel]
             -- TODO/HACK: hardcoding numbers here that are defined in Character.
             -- Base mana pool and awareness<->mana conversion.
             -- Basically, we want to require enough awareness that they can actually use the ability.
             return math.max(0, cost - 3)
        end

        return 0
    end

    function Ob:getReqUpgradeRoomLevel()
        if self:isAtMaxLevel() then return 1 end

        if self.tRefData and self.tRefData.tManaCost then
            local nNextLevel = (self:isUnlocked() and self.tSaved.nCurLevel+1) or 1
            local cost = self.tRefData.tManaCost[nNextLevel]
            -- HACK: I just thought these were decent cutoffs.
            if cost >= Ability.tMiscTweaks.nManaCostForRoomLvl3 then
                return 3
            elseif cost >= Ability.tMiscTweaks.nManaCostForRoomLvl2 then
                return 2
            else
                return 1
            end
        end
        return 1
    end


    function Ob:getUpgradeTime()
        local nDuration = 0
        if self.rDef.tTrainTime then
            if not self:isUnlocked() then
                if self.rDef.tTrainTime[1] then
                    nDuration = self.rDef.tTrainTime[1]
                end
            else
                if self.rDef.tTrainTime[self.tSaved.nCurLevel + 1] then
                    nDuration = self.rDef.tTrainTime[self.tSaved.nCurLevel + 1]
                end
            end
        end
        if self.sDefKey then
            local sKey = 'TrainAbility'..self.sDefKey..tostring(self:getCurLevel() + 1)
            local nElapsed = self.rCharacter:getActivityProgress(sKey) * nDuration
            if nElapsed then
                nDuration = nDuration - nElapsed
                local rRoom = g_CurrentMap:getRoom("dojo")
                if rRoom then
                    local rHighestLevelProp = rRoom:getHighestLevelAvailableProp()
                    if rHighestLevelProp then
                        local nDurationMult = rHighestLevelProp:getPropDef().nDurationMultiplier or 1
                        local nSpeedMod = self.rCharacter:getTrainingSpeedMod()
                        if nSpeedMod > 0 then
                            nDuration = nDuration * (1 / nSpeedMod)
                        end
                        nDuration = nDuration * nDurationMult
                    end
                end
            end
        end
        return nDuration
    end

    -- we will preload all ability sounds
    function Ob:preloadSounds()
        if not self:isDefault() and self.rDef.sCueName then
            self.rAbilityCue = g_Audio:loadCue(self.rDef.sCueName)
        end
    end

    function Ob:unloadSounds()
        if self.rAbilityCue then
            self.rAbilityCue:unload()
            self.rAbilityCue = nil
        end
    end

    function Ob:screenShakeThread(rEvent)
        if self.rDef.nScreenShakeMag and self.rDef.nScreenShakeDelay then
            util.sleep(self.rDef.nScreenShakeDelay)
            rEvent:shakeThread(self.rDef.nScreenShakeMag)
        end
    end

    function Ob:performAbility(rEventActor, rTarget, rEvent) -- needs to be called from a thread  
        self.bIsActive = true
        --self.nTurnElapsed = 0
        self:resetNumTurns()

        -- NOTE: goes straight to the character instead of the EventActor.
        if self.rCharacter then 
            self.rCharacter:incrementMana(-self:getManaCost()) 
        end

        if self.rDef.sCueName then
            if self:isDefault() then
                rEvent:playDefaultAbilityCue(self.rDef.sCueName)
            else
                if self.rAbilityCue then
                    self.rAbilityCue:play()
                else
                    g_Audio:playCue(self.rDef.sCueName)
                end
            end
        end
        if self.rDef.nScreenShakeMag then
            if self.rDef.nScreenShakeDelay then
                self.rShakeThread = MOAIThread.new()
                self.rShakeThread:run(self.screenShakeThread, self, rEvent)
            else
                rEvent:shakeThread(self.rDef.nScreenShakeMag)
            end
        end
        if self.rDef.sEffectToTrigger then
            local rEffectInstance = EffectInstance:new(self.rDef.sEffectToTrigger, Renderer.getRenderLayer('EventFX'))
            local nScaleX = self.rDef.nEffectXScale or 0
            local nScaleY = self.rDef.nEffectYScale or 0
            rEffectInstance:setScl(nScaleX, nScaleY)
            if self.rDef.bAttachEffect then
                rEffectInstance:setParent(rEventActor)
                local nOffsetX = self.rDef.nEffectXOffset or 0
                local nOffsetY = self.rDef.nEffectYOffset or 0
                rEffectInstance:setLoc(nOffsetX, nOffsetY)
            else
                local x, y = rEventActor:getLoc()
                x = x + self.rDef.nEffectXOffset or 0
                y = y + self.rDef.nEffectYOffset or 0
                rEffectInstance:setLoc(x, y)
            end
            rEffectInstance:start()
        end           

        self:abilityThread(rEventActor, rTarget, rEvent) 
        if self.nCoolDown > 0 then
            self.rCooldownThread = MOAIThread.new()
            self.rCooldownThread:run(self.cooldownThread, self)
        end
        self.bIsActive = false
    end

    function Ob:abilityThread(rEventActor, rTarget, rEvent) -- needs to be called from a thread 
        if rDef.sCustomThreadName then
            local threadFncn = Ability.getCustomAbilityThread(rDef.sCustomThreadName)
            if threadFncn then
                self.rCustomThread = MOAIThread.new()
                self.rCustomThread:run(threadFncn, self, rEventActor, rTarget, rEvent)
            end
        end
        if self.sAnimName then
            self:playAnim(rEventActor, self.sAnimName, true)
        end
        local tBuffsToApply = nil
        if self.rDef.tBuffsByLevel then
            tBuffsToApply = self.rDef.tBuffsByLevel[self:getCurLevel()]
        else
            tBuffsToApply = self.rDef.tBuffs
        end

        if tBuffsToApply then
            -- apply some mods to the buff based on level, etc.
            if self.tRefData then
                if self.tRefData.tBuffDuration then
                    for _, rBuff in ipairs(tBuffsToApply) do
                        rBuff.nTurnDuration = self.tRefData.tBuffDuration[self:getCurLevel()]
                    end
                end
                if self.tRefData.tBaseAmount then
                    for _, rBuff in ipairs(tBuffsToApply) do
                        rBuff.nAmount = self.tRefData.tBaseAmount[self:getCurLevel()]
                    end
                end
            end

            -- apply the buff
            local tTargets = nil
            if self.bAffectAllFriendly then
                if rEventActor:isHero() then
                    tTargets = rEvent:getAllActiveHeroes()
                else
                    tTargets = rEvent:getAllEnemies()
                end
            elseif self.bAffectAllEnemy then
                if rEventActor:isHero() then
                    tTargets = rEvent:getAllEnemies()
                else
                    tTargets = rEvent:getAllActiveHeroes()
                end
            elseif self.bAffectAll then
                tTargets = {}
                local tEnemies = rEvent:getAllEnemies()
                for _,v in tEnemies do table.insert(tTargets,v) end
                local tFriendlies = rEvent:getAllActiveHeroes()
                for _,v in tFriendlies do table.insert(tTargets,v) end
            else
                tTargets = {}
                table.insert(tTargets, rEventActor)
            end

            for i, rNewTarget in ipairs(tTargets) do
                if rNewTarget ~= rEventActor and not rNewTarget:isDead() then
                    for c, rBuffDef in ipairs(tBuffsToApply) do
                        rNewTarget:addBuff(rBuffDef, rEventActor)
                    end
                end
            end
            -- SPECIAL HACK: apply buff to event actor last. Why? Because now that we have
            -- buffs dependent on stats, if you buff your own stat, it'll affect the buff!
            if not rEventActor:isDead() then
                for c, rBuffDef in ipairs(tBuffsToApply) do
                    if rEventActor.addBuff then -- wierd case..wtf. (NOTE: probably can replace this check with a check to see if we're the MM.)
                        rEventActor:addBuff(rBuffDef, rEventActor)
                    end
                end
            end
        end

        if self.nPostAnimDelay then
            util.sleep(self.nPostAnimDelay)
        end
    end

    return Ob
end

function AbilityDamage:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)
    local Ob = Ability:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)

    function Ob:abilityThread(rEventActor, rTarget, rEvent) -- needs to be called from a thread 
        if self.rDef.sCustomThreadName then
            local threadFncn = Ability.getCustomAbilityThread(rDef.sCustomThreadName)
            if threadFncn then
                self.rCustomThread = MOAIThread.new()
                self.rCustomThread:run(threadFncn, self, rEventActor, rTarget, rEvent)
            end
        end
        local rProjectileDef = self.rProjectileDef
        local rProjectile = nil
        if rProjectileDef then
            rProjectile = Projectile:new(rEventActor, rTarget, self.rProjectileDef, Renderer.getLayer('EventFX').RenderLayer, self:getCharge())
        end

		-- calculate base damage
        local nBaseDamage
        if self.tRefData and self.tRefData.tDamage then
            nBaseDamage = self.tRefData.tDamage[self:getCurLevel()]
        elseif self.rDef.tDamageByLevel then
            nBaseDamage = self.rDef.tDamageByLevel[self:getCurLevel()]
        else
            nBaseDamage = self.rDef.nDamageAmount or 0
        end

		-- modifier for critical strikes
		--local nCriticalDamage = nBaseDamage * nCritMultiplier

		-- modifier for stats
        local nDamageAmount = nBaseDamage  --* nCritMultiplier
        local nCritMultiplier = rEventActor:getAttackCriticalMultiplier()
        if self:isDefault() then
            nDamageAmount = nDamageAmount * rEventActor:getStatPower()
        else
            local nPowerFactor = self.rDef.nPowerFactor or .75
            local nIntFactor = self.rDef.nIntFactor or .75
            --nDamageAmount = nDamageAmount * (.33 * rEventActor:getStatPower() + .67 * rEventActor:getStatAwareness()) * rEventActor:getSpecialAbilityMult()
            nDamageAmount = nDamageAmount * (nPowerFactor * rEventActor:getStatPower() + nIntFactor * rEventActor:getStatAwareness()) * rEventActor:getSpecialAbilityMult()
        end
        
        local nAnimDuration = 0
        if self.rDef.tAnimNames then
            -- if we have a list of animations, pick one at random
            local numAnims = table.getn( self.rDef.tAnimNames )
            local randIndex = math.random(1, numAnims)
            nAnimDuration = self:playAnim(rEventActor, self.rDef.tAnimNames[randIndex], false)        
        elseif self.sAnimName then
            nAnimDuration = self:playAnim(rEventActor, self.sAnimName, false)
        end
        -- move to loc
        local nOrigX, nOrigY = rEventActor:getLoc()
        if self.bMoveToTarget and self.nMoveDuration then            
            rEventActor:moveToMeleeTarget(rEventActor, self.nMoveDuration)
        end
        if self.nDamageDelay then
            util.sleep(self.nDamageDelay)
            nAnimDuration = nAnimDuration - self.nDamageDelay 
        end
        if not rEventActor:isDead() then
            local tBuffsToApply = nil
            if self.rDef.tBuffsByLevel then
                tBuffsToApply = self.rDef.tBuffsByLevel[self:getCurLevel()]
            else
                tBuffsToApply = self.rDef.tBuffs
            end
            if self:isDefault() then
                local tDefaultAttackMods = rEventActor:getDefaultAttackMods()
                if tDefaultAttackMods then
                    for i, rInfo in ipairs(tDefaultAttackMods) do
                        if not rInfo.nChance then
                            rInfo.nChance = 100
                        end
                        local nChance = math.random(1, 100)
                        if nChance < (rInfo.nChance * 100) then
                            for c, rBuff in ipairs(rInfo.tBuffs) do
                                if not tBuffsToApply then
                                    tBuffsToApply = {}
                                end
                                table.insert(tBuffsToApply, rBuff)
                            end
                        end
                    end
                end
            end
			
			local bAttackBlocked = false

            while rProjectile and not rProjectile.bDone do
                coroutine.yield()
            end

            if tBuffsToApply then
                -- apply some mods to the buff based on level, etc.
                if self.tRefData then
                    if self.tRefData.tBuffDuration then
                        for _, rBuff in ipairs(tBuffsToApply) do
                            rBuff.nTurnDuration = self.tRefData.tBuffDuration[self:getCurLevel()]
                        end
                    end
                    if self.tRefData.tBaseAmount then
                        for _, rBuff in ipairs(tBuffsToApply) do
                            rBuff.nAmount = self.tRefData.tBaseAmount[self:getCurLevel()]
                        end
                    end
                end
            end

            -- do damage
            if self.bAffectAll then
                local tTargets = nil
                if rEventActor:isHero() then
                    tTargets = rEvent:getAllEnemies()
                else
                    tTargets = rEvent:getAllActiveHeroes()
                end
                for i, rNewTarget in ipairs(tTargets) do
		            local bAttackIsCritical = (nCritMultiplier ~= 1.0)
                    rEventActor:attack(rNewTarget, nDamageAmount*nCritMultiplier, self.rHitEffectDef, bAttackIsCritical)
                    if tBuffsToApply then
                        for c, rBuffDef in ipairs(tBuffsToApply) do
                            rNewTarget:addBuff(rBuffDef, rEventActor)
                        end
                    end
                end
            else
	            local nCritMultiplier = rEventActor:getAttackCriticalMultiplier()
	            local bAttackIsCritical = (nCritMultiplier ~= 1.0)
                rEventActor:attack(rTarget, nDamageAmount*nCritMultiplier, self.rHitEffectDef, bAttackIsCritical)
				bAttackBlocked = rTarget.bLastAttackWasBlocked

                if not rTarget:isDead() then
                    if tBuffsToApply then
                        for c, rBuffDef in ipairs(tBuffsToApply) do
                            rTarget:addBuff(rBuffDef, rEventActor)
                        end
                    end                
                end
            end

            if self:isDefault() and self.rCharacter then
                local nSplashDamagePct = self.rCharacter:getSplashDamagePct()
                if nSplashDamagePct > 0 then
                    local tTargets = rEvent:getAllEnemies()
                    local nSplashDamage = nDamageAmount * nCritMultiplier * nSplashDamagePct
                    for i, rNewTarget in ipairs(tTargets) do
                        if rTarget ~= rNewTarget then
                            rEventActor:attack(rNewTarget, nSplashDamage, self.rHitEffectDef, bAttackIsCritical)
                        end
                    end
                end
            end

			-- only play the impact cue if we didn't block the attack
            if self.rDef.nImpactCue and bAttackBlocked == false then
                g_Audio:playCue(self.rDef.nImpactCue)
            end
        end
        if nAnimDuration > 0 then
            util.sleep(nAnimDuration)
        end
        if not rEventActor:isDead() and not self.rDef.bDontGoToFightPose then
            self:playAnim(rEventActor, 'fight_breathe', false)
        end
        if self.nPostAnimDelay then
            util.sleep(self.nPostAnimDelay)
        end
        if self.bMoveToTarget then            
            rEventActor:setLoc(nOrigX, nOrigY)
        end
    end

    return Ob
end

--[[
function AbilityBuffFriendly:new(rDef, rEventActor, tSaved, rCharacter)
    local Ob = Ability:new(rDef, rEventActor, tSaved, rCharacter)

    function Ob:abilityThread(rEventActor, rTarget, rEvent) -- needs to be called from a thread       
        if rDef.sCustomThreadName then
            local threadFncn = Ability.getCustomAbilityThread(rDef.sCustomThreadName)
            if threadFncn then
                self.rCustomThread = MOAIThread.new()
                self.rCustomThread:run(threadFncn, self, rEventActor, rTarget, rEvent)
            end
        end
        if self.sAnimName then
            self:playAnim(rEventActor, self.sAnimName, true)
        end
        local tBuffsToApply = nil
        if self.rDef.tBuffsByLevel then
            tBuffsToApply = self.rDef.tBuffsByLevel[self:getCurLevel()]
        else
            tBuffsToApply = self.rDef.tBuffs
        end
        -- apply the buff
        if self.bAffectAll then
            if rEventActor:isHero() then
                tTargets = rEvent:getAllActiveHeroes()
            else
                tTargets = rEvent:getAllEnemies()
            end
            for i, rNewTarget in ipairs(tTargets) do
                if tBuffsToApply then
                    for c, rBuffDef in ipairs(tBuffsToApply) do
                        rNewTarget:addBuff(rBuffDef, rEventActor)
                    end
                end
            end
        else
            if not rEventActor:isDead() then
                if tBuffsToApply then
                    for c, rBuffDef in ipairs(tBuffsToApply) do
                        rEventActor:addBuff(rBuffDef, rEventActor)
                    end
                end                
            end
        end
        if self.nPostAnimDelay then
            util.sleep(self.nPostAnimDelay)
        end
    end

    return Ob
end
]]--

function AbilityTriggerAttack:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)
    local Ob = Ability:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)

    function Ob:abilityThread(rEventActor, rTarget, rEvent) -- needs to be called from a thread
        if self.rDef.sAbilityNameToTrigger and self.rDef.tTriggerActorNames and rEventActor and rTarget and rEvent then
            self.tTriggerActors = {}
            for i, sActorName in ipairs(self.rDef.tTriggerActorNames) do
                local rActor = rEvent:getActor(sActorName)
                if rActor then
                    table.insert(self.tTriggerActors, rActor)
                end
            end
			-- look for an animation list
			if self.rDef.tAnimNames then
				-- if we have a list of animations, pick one at random
				local numAnims = table.getn( self.rDef.tAnimNames )
				local randIndex = math.random(1, numAnims)
                self:playAnim(rEventActor, self.rDef.tAnimNames[randIndex], true)
            elseif self.rDef.sAnimName then
				-- no list of anims; just play the default
                self:playAnim(rEventActor, self.rDef.sAnimName, true)
            end
            -- fire the rockets
            for i, rActor in ipairs(self.tTriggerActors) do
                rActor:triggerAbility(self.rDef.sAbilityNameToTrigger, rTarget)
            end
            if self.rDef.sPostAttackAnim then
                self:playAnim(rEventActor, self.rDef.sPostAttackAnim)
            end
            if self.rDef.nPostAnimDelay then
                util.sleep(self.rDef.nPostAnimDelay)
            end            
        end
    end

    return Ob
end

function AbilityCycleStance:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)
    local Ob = Ability:new(rDef, rEventActor, tSaved, rCharacter, sDefKey)

    function Ob:abilityThread(rEventActor, rTarget, rEvent) -- needs to be called from a thread
        self.rEventActor:cycleStance()
    end

    return Ob
end

local function veteranFlySlam(rAbility, rEventActor, rTarget, rEvent)
    if rEventActor then
        local origX, origY = rEventActor:getLoc()
        rEventActor:setInvulnerable(true)
        g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/FlyOut')
        rEventActor:playAnimation('exit', true, false, true)
        rEventActor:setLoc(100, 370)
        rEventActor:playAnimation('fly', false, true, true)
        rEventActor:moveLoc(0, -300, nil, 0.5, MOAIEaseType.LINEAR) 
        
        util.sleep(0.5)
        rEventActor:setLoc(100, -120)
        g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/FlyAttackSlam')
        rEvent:performScreenShake(50)
        
        -- play effect
        local rEffectInstance = EffectInstance:new('Effects/LargeGroundImpact.lua', Renderer.getRenderLayer('EventFX'))
        rEffectInstance:setLoc(100, -120)
        rEffectInstance:setScl(5, 5)
        rEffectInstance:start()

        
        rEventActor:playAnimation('enter', true, false, true)
        
        util.sleep(0.2)
        rEventActor:playAnimation('exit', true, false, true)
        rEventActor:setLoc(origX, 370)
        rEventActor:playAnimation('fly', false, true, true)
        rEventActor:moveLoc(0, -300, nil, 0.5, MOAIEaseType.LINEAR) 
        util.sleep(0.5)
        rEventActor:setLoc(origX, origY)
        rEventActor:playAnimation('enter', true, false, true)
        
        rEventActor:setInvulnerable(false)
    end
end

local function mummyHand(rAbility, rEventActor, rTarget, rEvent)    
    if rEvent and rEvent.rLayer then
        util.sleep(1.25)
        local rHand = SpriteSheet:new('Characters/Heroes/Male_01b/Male01b_MummyHand', 'Characters/AnimDefs/MummyHand.lua', 0, 0, 'center', 'center')
        rEvent.rLayer:insertProp(rHand)
        rHand:setLoc(0, 0)
        rHand:setScl(3, 3)
        g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/MummyHandSmash')
        rHand:playAnimation('punch', false, false, true)
        rEvent.rLayer:removeProp(rHand)
        rHand:unloadSpriteSheet()
    end
end

local function rageRallyCry(rAbility, rEventActor, rTarget, rEvent)
    rEventActor:playAnimation('charge_up', true, false, false)
    util.sleep(1.5)
    rEventActor:playAnimation('fight_breathe', true, false, false)    
end

--GLOOMERANG
local function gloomerang(rAbility, rEventActor, rTarget, rEvent)
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer and rAbility and rEventActor and rEvent then    
        local x, y = rEventActor:getLoc()
        rEventActor:playAnimation('attack_gloomerang', true, false)
        util.sleep(1)
        local rGloomerang = SpriteSheet:new('Characters/Heroes/Female_02b/Female02b_fight_2_Lvl1', 'Characters/AnimDefs/Female02bFightAnims.lua')
        if rGloomerang then
            rLayer:insertProp(rGloomerang)
            rGloomerang:setLoc(x + 40, y)
            rGloomerang:playAnimation('gloomerang', true, false)
            rGloomerang:moveLoc(150, 50, 0.1, MOAIEaseType.LINEAR)
            util.sleep(0.1)
            rGloomerang:moveLoc(150, -50, 0.1, MOAIEaseType.LINEAR)
            util.sleep(0.1)
            rGloomerang:moveLoc(-150, -50, 0.1, MOAIEaseType.LINEAR)
            util.sleep(0.1)
            rGloomerang:moveLoc(-150, 50, 0.1, MOAIEaseType.LINEAR)
            util.sleep(0.1)
            rLayer:removeProp(rGloomerang)
            rGloomerang:unloadSpriteSheet()
            rEventActor:playAnimation('attack_gloomerang_catch', true, false, true)
        end
    end
end

-- COSMIC CROWBAR
local function crowbarboomerang(rAbility, rEventActor, rTarget, rEvent)
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer and rAbility and rEventActor and rEvent then    
        local x, y = rEventActor:getLoc()
        rEventActor:playAnimation('boomerangthrow', true, false)
        util.sleep(1)
        local rCrowbar = SpriteSheet:new('Characters/Heroes/Male_01d/Male01d_fight_2_Lvl1', 'Characters/AnimDefs/Male01dFightAnims.lua')
        if rCrowbar then
            rLayer:insertProp(rCrowbar)
            rCrowbar:setLoc(x + 40, y)
            rCrowbar:playAnimation('boomerang', true, false)
            rCrowbar:moveLoc(250, 0, 0.25, MOAIEaseType.LINEAR)
            util.sleep(0.75)
            rCrowbar:playAnimation('boomerang', false, false)
            rCrowbar:moveLoc(-250, 0, 0.25, MOAIEaseType.LINEAR)
            util.sleep(0.25)
            rLayer:removeProp(rCrowbar)
            rCrowbar:unloadSpriteSheet()
            rEventActor:playAnimation('boomerangcatch', true, false, true)
        end
    end
end

local function crowbarportal(rAbility, rEventActor, rTarget, rEvent)    
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer and rAbility and rEventActor and rEvent then    
        local x, y = rEventActor:getLoc()
        local rPortalFX = SpriteSheet:new('Characters/Heroes/Male_01d/Male01d_fight_2_Lvl1', 'Characters/AnimDefs/Male01dFightAnims.lua')
		local rEffectInstance = EffectInstance:new('Effects/CrowbarPortal.lua', rLayer)
		rEffectInstance:setScl(2.5, 2.5)
		rEffectInstance:setLoc(x + 65, y + 160)
		rEffectInstance:start()
		if rPortalFX then
            rLayer:insertProp(rPortalFX)
			rPortalFX:setLoc(x, y)
            rPortalFX:playAnimation('portalVFX', true, false, true)
			rLayer:removeProp(rPortalFX)
			rPortalFX:unloadSpriteSheet()	
        end
		rEffectInstance:stop()	
    end
end

local function techShield(rAbility, rEventActor, rTarget, rEvent)    
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        util.sleep(1)
        local x, y = rEventActor:getLoc()
        local rShield = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/TechShield.lua', 0, 0, 'center', 'center')
        rLayer:insertProp(rShield)
        rShield:setLoc(x, y + 80)
        rShield:playAnimation('start', false, false, true)
        rShield:playAnimation('loop', false, false, false)
        local nDuration = rAbility:getDef().nDuration or 5
        if nDuration then
            util.sleep(nDuration)
        end
        rLayer:removeProp(rShield)
        rShield:unloadSpriteSheet()
    end
end

local function techEyeBlast(rAbility, rEventActor, rTarget, rEvent)  
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        util.sleep(1.5)
        local rBeam = SpriteSheet:new('FX/FX_2', nil, nil, nil, 'center', 'center')
        rBeam:setSpriteIndex('Tech_ChestBeam.swf/0000')
        rBeam:setParent(rEventActor)
        rLayer:insertProp(rBeam)        
        rBeam:setLoc(460, 80)
        rBeam:setScl(-6, 1) 
		local rBeamFlair = SpriteSheet:new('FX/FX_5', 'Characters/AnimDefs/FX_5.lua', nil, nil, 'center', 'center')
        rLayer:insertProp(rBeamFlair)
		rBeamFlair:setParent(rEventActor)
		rBeamFlair:setLoc(390, 80)
        rBeamFlair:setScl(5, 2)
        rBeamFlair:playAnimation('atmaneyebeamflair')
		local x, y = rEventActor:getLoc()
		local rEffectInstance = EffectInstance:new('Effects/AtManEyeFlash.lua', rLayer)
		rEffectInstance:setScl(3, 3)
		rEffectInstance:setLoc(x, y + 80)
		rEffectInstance:start()
        util.sleep(0.35)
        rLayer:removeProp(rBeam)
        rBeam:unloadSpriteSheet()
		rLayer:removeProp(rBeamFlair)
        rBeamFlair:unloadSpriteSheet()
    end
end

local function robotEyeBeam(rAbility, rEventActor, rTarget, rEvent)  
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        util.sleep(0.95)
        local rBeam = SpriteSheet:new('FX/FX_2', nil, nil, nil, 'center', 'center')
        rBeam:setSpriteIndex('Robot_EyeBeam.swf/0000')
 		rLayer:insertProp(rBeam)       
        rBeam:setParent(rEventActor)
        rBeam:setLoc(592, 96)
        rBeam:setScl(8, 1)
		local rBeamFlair = SpriteSheet:new('FX/FX_5', 'Characters/AnimDefs/FX_5.lua', nil, nil, 'center', 'center')
        rLayer:insertProp(rBeamFlair)
		rBeamFlair:setParent(rEventActor)
        rBeamFlair:playAnimation('crimeboteyebeamflair')
		rBeamFlair:setLoc(360, 80)
        rBeamFlair:setScl(4, 2.5)
		local x, y = rEventActor:getLoc()
		local rEffectInstance = EffectInstance:new('Effects/CrimeBotEyeFlash.lua', rLayer)
		rEffectInstance:setScl(3, 3)
		rEffectInstance:setLoc(x + 30, y + 80)
		rEffectInstance:start()
        util.sleep(0.7)
        rLayer:removeProp(rBeam)
        rBeam:unloadSpriteSheet()
		rLayer:removeProp(rBeamFlair)
        rBeamFlair:unloadSpriteSheet()
    end
end

local function orbitalStrike(rAbility, rEventActor, rTarget, rEvent)  
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        util.sleep(1)
        local x, y = rEventActor:getLoc()
        local rBeam = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/FX_2.lua', nil, nil, 'center', 'center')
        rLayer:insertProp(rBeam)
        rBeam:playAnimation('orbitalstrike_beam')
        rBeam:setScl(2, 4)
		local rBeamImpact = SpriteSheet:new('FX/FX_5', 'Characters/AnimDefs/FX_5.lua', nil, nil, 'center', 'center')
        rLayer:insertProp(rBeamImpact)
        rBeamImpact:playAnimation('orbitalstrike_impact')
        rBeamImpact:setScl(4.5, 4.5)
        local targX, targY = rTarget:getLoc()
        rBeam:setLoc(targX, targY + 540)
		rBeamImpact:setLoc(targX, targY)
        rBeam:moveLoc(0, -320, 0.2)
        util.sleep(0.5)
        util.sleep(2)
        rLayer:removeProp(rBeam)
        rBeam:unloadSpriteSheet()
		rLayer:removeProp(rBeamImpact)
        rBeamImpact:unloadSpriteSheet()
    end
end

local function elecShield(rAbility, rEventActor, rTarget, rEvent)
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rEventActor and rLayer then
        local x, y = rEventActor:getLoc()
        local rShield = SpriteSheet:new('FX/FX_4', 'Characters/AnimDefs/LightningShield.lua', 0, 0, 'center', 'center')
        rLayer:insertProp(rShield)
        rShield:setLoc(x - 10, y + 80)
        rShield:playAnimation('loop', false, false, false)
        
        local nDuration = rAbility:getDef().nDuration or 5
        if nDuration then
            util.sleep(nDuration)
        end
        
        rLayer:removeProp(rShield)
        rShield:unloadSpriteSheet()        
    end
end

local function lightningStrike(rAbility, rEventActor, rTarget, rEvent)  
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        --local tLightnings = {}
        local tEnemies = rEvent:getAllEnemies()
        local nMult = 1
        util.sleep(0.5)
        for i, rEnemies in ipairs(tEnemies) do
            --util.sleep(math.random(0.2, 0.5))
            --local rLightning = SpriteSheet:new('FX/FX_2')
            --rLightning:setSpriteIndex('LightningStrike.swf/0000')
            --rLightning:setScl(0, 6)
			local x, y = rEnemies:getLoc()
			local t = math.random(0.1, 0.3)
			local rEffectInstance = EffectInstance:new('Effects/LightningStrike.lua', rLayer)
			--local rLightning = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/FX_2.lua', 0, 0, 'center', 'center')
			--rLayer:insertProp(rLightning)
			local randomScale = math.random(2, 4)
			--rLightning:setScl(1.5, 4.5)
			--rLightning:setLoc(x, y + 200)
			--rLightning:setRot(math.random(-10,10))
			--util.sleep(t)
			--rLightning:playAnimation('lightning_strike')
			rEffectInstance:setScl(1.5, 4.5)
			rEffectInstance:setLoc(x, y + 200)
			local rEffectImpact = EffectInstance:new('Effects/LightningImpact.lua', rLayer)
			
			rEffectImpact:setScl(randomScale, randomScale)
			rEffectImpact:setLoc(x, y)
			util.sleep(t)
			rEffectInstance:start()
			rEffectImpact:start()
            --if nMult == 1 then
            --    nMult = -1
            --else
            --    nMult = 1
            --end
            --rLightning:moveScl(math.random(50, 100) * 0.01 * nMult, -3, 0.25)
            --rLightning:setLoc(x, y + 20)
            --rLightning:moveLoc(0, -20, 0.25)
            --table.insert(tLightnings, rLightning)
            --rLayer:insertProp(rLightning)
			
            --util.sleep(0.15)
        end
        --util.sleep(0.5)
        --for i, rLightning in ipairs(tLightnings) do
        --    rLayer:removeProp(rLightning)
        --    rLightning:unloadSpriteSheet()
        --end
    end
end

local function robotGasGrenade(rAbility, rEventActor, rTarget, rEvent)
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        util.sleep(0.25)
        local x, y = rEventActor:getLoc()
        
        local rGrenade = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/FX_2.lua', 0, 0, 'center', 'center')
        rLayer:insertProp(rGrenade)
        local nXOffset = 30.0
        local nYOffset = 75.0
        rGrenade:setLoc(x+nXOffset, y+nYOffset)
        rGrenade:playAnimation('gas_grenade', true, false, false)  
        
        -- leave robot and arc to ground        
        local nXMovement = 120.0
        local nYMovement = 50.0
        local nMoveDuration = 0.6
        local nHalfMoveDuration = nMoveDuration * .5
        rGrenade:moveLoc(nXMovement, 0, nMoveDuration, MOAIEaseType.LINEAR)
        rGrenade:moveRot(-720, nMoveDuration, MOAIEaseType.LINEAR)
        rGrenade:moveLoc(0, nYMovement, nHalfMoveDuration, MOAIEaseType.SOFT_EASE_IN)
        util.sleep(nHalfMoveDuration)     
        rGrenade:moveLoc(0, -(nYMovement + nYOffset), nHalfMoveDuration, MOAIEaseType.SOFT_EASE_OUT)
        util.sleep(nHalfMoveDuration)    
        
        --bounce!
        local nBounceXMovement = 60.0
        local nBounceYMovement = 30.0        
        local nBounceDuration = 0.3
        local nHalfBounceDuration = nBounceDuration * .5        
        rGrenade:moveLoc(nBounceXMovement, 0, nBounceDuration, MOAIEaseType.LINEAR)
        rGrenade:moveRot(-360, nBounceDuration, MOAIEaseType.LINEAR)
        rGrenade:moveLoc(0, nBounceYMovement, nHalfBounceDuration, MOAIEaseType.SOFT_EASE_IN)
        util.sleep(nHalfBounceDuration)     
        rGrenade:moveLoc(0, -nBounceYMovement, nHalfBounceDuration, MOAIEaseType.SOFT_EASE_OUT)

        -- explode!! (start it early so it explodes when we get there)
        local rEffectInstance = EffectInstance:new('Effects/GasExplosion.lua', Renderer.getRenderLayer('EventFX'))
        rEffectInstance:setLoc(x+nXOffset+nXMovement+nBounceXMovement+20, y-50)
        rEffectInstance:setScl(4, 4)
        rEffectInstance:start()
        util.sleep(nHalfBounceDuration)          
        
        rLayer:removeProp(rGrenade)
    end
end

local function robotRocketStorm(rAbility, rEventActor, rTarget, rEvent)  
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    util.sleep(1.5)
    if rLayer then
        local kNUM_MISSILES = 7
        local tMissiles = {}
        local x, y = rEventActor:getLoc()
        local nNum = 0
        local bLeftHand = true
        for i = 1, kNUM_MISSILES do
            local rRocket = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/FX_2.lua', 0, 0, 'center', 'center')
            rLayer:insertProp(rRocket)
            local rocketX = x - 10
            if bLeftHand then
                rocketX = rocketX + 45
            end
            bLeftHand = not bLeftHand
                
            rRocket:setLoc(rocketX, y + 150)
            rRocket:moveLoc(0, 400, 0.5)
            rRocket:setRot(-90)
            rRocket:playAnimation('robot_rocketfists', false, false, false)
            table.insert(tMissiles, rRocket)
            util.sleep(0.1)
        end
        util.sleep(1)
        -- rain down
        for i, rRocket in ipairs(tMissiles) do
            local x = math.random(0, 300)
            local y = math.random(400, 600)
            rRocket:setLoc(x, y)
            util.sleep(0.1)
            rRocket:setRot(90)
            
            local moveSpeed = 0.3
            rRocket:moveLoc(0, -600, moveSpeed)
            util.sleep(moveSpeed)
            
            g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/RocketExplode')
            local rEffectInstance = EffectInstance:new('Effects/Explosion.lua', rLayer)
            rEffectInstance:setScl(3, 3)
            rEffectInstance:setLoc(x + 20, y - 700)
            rEffectInstance:start()
            
            rLayer:removeProp(rRocket)
        end       
    end
end

local function male01Throw(rAbility, rEventActor, rTarget, rEvent)
    local rLayer = Renderer.getLayer('EventFX').RenderLayer
    if rLayer then
        local rBoulder = SpriteSheet:new('FX/FX_2', 'Characters/AnimDefs/FX_2.lua', 0, 0, 'center', 'center')
		rBoulder:playAnimation('boulder', false, false, false)
        
		local x, y = rEventActor:getLoc()
        rBoulder:setParent(rEventActor)
        rLayer:insertProp(rBoulder)
        rBoulder:setLoc(0, 50)
        util.sleep(0.50)
        rBoulder:moveLoc(0, 120, 0.15, MOAIEaseType.LINEAR)
        util.sleep(0.15)
        rBoulder:moveLoc(-80, -24, 0.05, MOAIEaseType.LINEAR)
        util.sleep(0.85)
        rBoulder:moveLoc(80, 24, 0.25, MOAIEaseType.LINEAR)
        util.sleep(0.25)
        rBoulder:moveLoc(100, 0, 0.15, MOAIEaseType.LINEAR)
        util.sleep(0.15)
        rBoulder:moveLoc(80, -120, 0.25, MOAIEaseType.LINEAR)
		local rEffectInstance = EffectInstance:new('Effects/ImpactBoulder.lua', rLayer)
		rEffectInstance:setScl(3, 3)
		rEffectInstance:setLoc(x + 180, y + 30)
        g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/BoulderHit')
        util.sleep(0.35)

        -- do a shake
        rEvent:performScreenShake(20)
        rLayer:removeProp(rBoulder)
        rEffectInstance:start()
        rBoulder:unloadSpriteSheet()
    end
end

local function footStomp(rAbility, rEventActor, rTarget, rEvent)
    -- move to loc
    local nOrigX, nOrigY = rEventActor:getLoc()
    rEventActor:playAnimation('lift', false, false, false)
    g_Audio:playCue(rAbility.rDef.sLiftCueName)-- 'RobotFootLift')
    rEventActor:moveLoc(-200, 0, 0, 1, MOAIEaseType.LINEAR)
    util.sleep(1)
    
    g_Audio:playCue(rAbility.rDef.sStompCueName)-- 'RobotFootStomp')
    rEventActor:playAnimation('smash', false, false, false)
    util.sleep(0.5)
    rEvent:performScreenShake(50)
    util.sleep(1)

    rEventActor:playAnimation('lift', false, false, false)
    rEventActor:moveLoc(200, 0, 0, 1, MOAIEaseType.LINEAR)
    util.sleep(1)
    rEventActor:playAnimation('breathe', false, false, false)
    rEventActor:setLoc(nOrigX, nOrigY + 100)
    rEventActor:moveLoc(0, -100, 0, 0.25, MOAIEaseType.LINEAR)
    util.sleep(0.25)
    rEventActor:setLoc(nOrigX, nOrigY)
end

local function rocketFire(rAbility, rEventActor, rTarget, rEvent)
    local rMadScientist = rEvent:getActor('MadScientist')    
    if rMadScientist then
        g_Audio:playCue('MMOJ_Audio/SFX/EnemySFX/RemoteClick')
        rMadScientist:playAnimation("press_button", false, false, true)
        rMadScientist:playAnimation('press_button_laugh', false, false)
    end
end

local function crunchTime(rAbility, rEventActor, rTarget, rEvent)
    -- A buff is defined in the ability def, which gives us all the graphics and the morale decrease.
    -- This line just tells the event to let the heroes attack twice.
    rEvent:startCrunchTime(rAbility:getCurLevel())
end

local function deathCountdown(rAbility, rEventActor, rTarget, rEvent)
    -- Buff ends after 5 rounds.
    -- NOTE: For now we just count rounds by looking at the number of hero rounds elapsed.
    -- However, this lets the player screw themselves by activating right before the round ticks
    -- over. A fairer buff might only begin if it detects a certain number of hero actions
    -- in the round, i.e. >= 50% of the heroes acting.

    local nStartingRound = rEvent.nHeroRoundsComplete or 0
    local nCurrentRound = nStartingRound

    local rLayer = Renderer.getLayer('UIOverlay').RenderLayer
    local rCountdownUI = CountdownUI:new(rLayer)
    --rLayer:insertProp(rCountdownUI)
    rCountdownUI:setLoc(Renderer.getUIHalfWidth()-50, -100)
    rCountdownUI:setScl(2,2)
    rCountdownUI:setVisible(true)
    rCountdownUI:updateProgress(100)

    local tHeroTargets = rEvent:getAllActiveHeroes()
    local amt = rAbility:getCurLevel() * 10
    for i,v in ipairs(tHeroTargets) do
        v:addHealth(amt, true)
        --v:setOverlayText("+ HEALTH "..amt, 0.75, { 0, 1, 0 } )
    end

    while nCurrentRound < nStartingRound + 5 and not rEvent:isCompleting() and not rEvent:hasCompleted() do
        coroutine.yield()
        nCurrentRound = rEvent.nHeroRoundsComplete or 0
        rCountdownUI:updateProgress(100 * (1 + (nCurrentRound-nStartingRound)) / 6)
    end
    rCountdownUI:updateProgress(120)
    if not rEvent:isCompleting() and not rEvent:hasCompleted() then
        local tHeroTargets = rEvent:getAllActiveHeroes()
        for i,v in ipairs(tHeroTargets) do
            if not v:isDead() then v:takeDamage(9999) end
        end
    end
    if not rEvent:hasCompleted() then
        util.sleep(1)
    end
    rCountdownUI:setVisible(false)
    --rCountdownUI:updateProgress(100 * self.nElapsedTurns / self.nTurnDuration)
end

local function paradigmShift(rAbility, rEventActor, rTarget, rEvent)
    -- pause all combat
    -- animate a health bar swap
    -- swap the health (w/ hack for 100% health)
    -- resume combat.
    --
    rEvent:setCombatPaused(true)

    local tTargets={}

    --[[
    -- MTF: let's not do enemies for now.
    local tEnemyTargets = rEvent:getAllEnemies()
    for i,v in ipairs(tEnemyTargets) do
        if not v:isDead() then
            table.insert(tTargets, v)
        end
    end
    ]]--

    local tHeroTargets = rEvent:getAllActiveHeroes()
    for i,v in ipairs(tHeroTargets) do
        if not v:isDead() then
            table.insert(tTargets, v)
        end
    end

    util.sleep(.2)
    for i,v in ipairs(tTargets) do
        v:setHealthBarVisible(true)
        v.rHealthBar:update(v:getCurHP())
        local w,h = v.rHealthBar:getDims()
        v.rHealthBar:setPiv(0)
        v.rHealthBar:moveRot(0,0, 180, 1)
    end
    util.sleep(1)

    for i,v in ipairs(tTargets) do
        v.rHealthBar:setRot(0, 0, 0)
        local hp = v:getCurHP()
        local max = v:getMaxHP()
        v:setCurHP(math.max(max-hp,1), true)
    end
    util.sleep(.7)

    rEvent:setCombatPaused(false)
end

local function loadBalance(rAbility, rEventActor, rTarget, rEvent)
    --local tHeroes = rEvent:getAllHeroes()
    local tHeroes = rEvent:getAllHeroActors()
    local health = 10
    local i = 0
    for _,v in ipairs(tHeroes) do
        if not v:isDead() then
            health = health+v:getCurHP()
            i = i+1
        end
    end

    if i == 0 then return end

    health = math.ceil(health / i)

    for _,v in ipairs(tHeroes) do
        if not v:isDead() then
            v:setCurHP(health, true)
        end
    end
end

Ability.tConstructors =
{
    Ability = Ability.new,
    AbilityDamage = AbilityDamage.new,
    --AbilityBuffFriendly = AbilityBuffFriendly.new,
    AbilityTriggerAttack = AbilityTriggerAttack.new,
}

Ability.tCustomAbilityThreads =
{
    mummyHand = mummyHand,
    veteranFlySlam = veteranFlySlam,
    rageRallyCry = rageRallyCry,
    techShield = techShield,
    techEyeBlast = techEyeBlast,
    orbitalStrike = orbitalStrike,
    lightningStrike = lightningStrike,
    elecShield = elecShield,
    robotEyeBeam = robotEyeBeam,
    robotGasGrenade = robotGasGrenade,
    robotRocketStorm = robotRocketStorm,
    male01Throw = male01Throw,
    footStomp = footStomp,
    rocketFire = rocketFire,
    loadBalance = loadBalance,
    crunchTime = crunchTime,
    paradigmShift = paradigmShift,
    deathCountdown = deathCountdown,
    crowbarportal = crowbarportal,
    crowbarboomerang = crowbarboomerang,
    gloomerang = gloomerang,
}

function Ability.getConstructor(sName)
    if Ability.tConstructors[sName] then
        return Ability.tConstructors[sName]
    end
end

function Ability.getCustomAbilityThread(sName)
    return Ability.tCustomAbilityThreads[sName]
end

return Ability
