local Buff = {}
local StunBuff = {}
local ShieldBuff = {}
local HealBuff = {}
local ManaBuff = {}
local DamageBuff = {}
local PowerBuff = {}
local CritBuff = {}
local CurseBuff = {}
local CrunchTimeBuff = {}
local RevengeDamageBuff = {}
local DeathBuff = {}
local BlockEffectivenessBuff = {}
local BlockChanceBuff = {}
local VampireBuff = {}
local CoinBuff = {}
local DodgeBuff = {}
local util = require('DFUtil')
local Sprite = require('SpriteSheet')
local Renderer = require('Renderer')
local CountdownUI = require('ui/CountdownUI')

Buff.tAllOfThem = {}

function Buff:new(rDef, rSourceActor)
    local Ob = {}
    -- copy in the values from the def
    for key, value in pairs(rDef) do
        Ob[key] = value
    end

    Ob.rDef = rDef
    Ob.rSourceActor = rSourceActor
    Ob.nElapsedTurns = 0

    function Ob:getValue(amt)
        if type(amt) == "number" then
            return amt
        else
            if amt == "nPower" then
                return self.rSourceActor:getStatPower()
            elseif amt == "nLevel" then
                return self.rSourceActor:getStatLevel()
            elseif amt == "nAwareness" then
                return self.rSourceActor:getStatAwareness()
            elseif amt == "nSpecialAbilityMult" then
                return self.rSourceActor:getSpecialAbilityMult()
            else
                assert(false)
            end
        end
    end

    function Ob:getAmount()
        assert(not (self.nAmount and self.tAmounts))

        local amt = 0

        if self.nAmount then
            amt = self:getValue(self.nAmount)
            if self.multBy then
                amt = amt * self:getValue(self.multBy)
            end
        else
            for _,v in ipairs(self.tAmounts) do
                local val = self:getValue(v.nAmount)
                if v.multBy then
                    val = val * self:getValue(v.multBy)
                end
                amt = amt+val
            end
        end
        if self.bScaleByAbilityPower then
            amt = amt * self.rSourceActor:getSpecialAbilityMult()
        end
        return math.floor(amt+.5)
    end

    function Ob:applyBuff(rChar, rEvent)
		self.rChar = rChar
		self.rEvent = rEvent

		if self:canApplyBuff() then
            Buff.tAllOfThem[self] = 1
			if self.buffThread then
				self.rBuffThread = MOAIThread.new()
				self.rBuffThread:run(self.buffThread, self)
			end
			if self.rDef.sSpriteSheetName then           
                local sAlignX = 'center'
                if self.rDef.sAlignX then 
                    sAlignX = self.rDef.sAlignX
                end
                local sAlignY = 'center'
                if self.rDef.sAlignY then 
                    sAlignY = self.rDef.sAlignY
                end
				
                self.rSprite = Sprite:new(self.rDef.sSpriteSheetName, self.rDef.sAnimDefName, 0, 0, sAlignX, sAlignY)
				if self.rDef.sDefaultSpriteIndex then
					self.rSprite:setSpriteIndex(self.rDef.sDefaultSpriteIndex)
				end
				if self.rDef.sSpriteAnimName then
					self.rSprite:playAnimation(self.sSpriteAnimName)
				end
				if self.rDef.tColor then
					self.rSprite:setColor(self.rDef.tColor.r, self.rDef.tColor.g, self.rDef.tColor.b, self.rDef.tColor.a)
				end
                
                local nSpritePosX = 0
                if self.rDef.nSpriteOffsetX then 
                    nSpritePosX = self.rDef.nSpriteOffsetX
                end
                local nSpritePosY = 0
                if self.rDef.nSpriteOffsetY then 
                    nSpritePosY = self.rDef.nSpriteOffsetY
                end
                local nScale = 1
                if self.rDef.nScale then 
                    nScale = self.rDef.nScale
                    self.rSprite:setScl(nScale, nScale)
                end
                
				self.rSprite:setLoc(nSpritePosX, nSpritePosY)
				self.rSprite:setParent(rChar)
				self.rLayer = Renderer.getLayer('EventOverlay').RenderLayer
				self.rLayer:insertProp(self.rSprite)
			end
			if self.rDef.bHideChar then
				self.rChar:setVisible(false)
			end
            if self.onBuffStart then
                self:onBuffStart()
            end
		end
    end

    function Ob:endBuff()
        Buff.tAllOfThem[self] = nil
        if self.onBuffEnd then
            self:onBuffEnd()
        end
        if self.rChar then
            self.rChar:onBuffEnd(self)
        end
        if self.rDef.bHideChar then
            self.rChar:setVisible(true)
            if not self.rChar:isDead() then
                self.rChar:playAnimation('breathe')
            else
                self.rChar:playAnimation('knock_out')
            end            
        end
        if self.rSprite then
            self.rSprite:unloadSpriteSheet()
            self.rLayer:removeProp(self.rSprite)
        end
    end

	function Ob:canApplyBuff()
		return true
	end

    function Ob:playAnim(rChar, sAnimName, bBlocking)
        local bFlipX = false
        if rChar:isHero() then
            bFlipX = true
        end
        rChar:playAnimation(sAnimName, bFlipX, false, bBlocking)
    end

    function Ob:onTurnElapsed()        
        self.nElapsedTurns = self.nElapsedTurns + 1
        if not self.bIsInfinite and self.nTurnDuration and self.nTurnDuration >= 0 and self.nElapsedTurns >= self.nTurnDuration then
            self:endBuff()
        end
    end

    return Ob
end

function StunBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
			self.rChar:setCanAttack(false)
			self:playAnim(self.rChar, self.sAnimName, false)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:setCanAttack(true)
        end
    end

	function Ob:canApplyBuff()
		return self.rChar:stunnable()
	end

    return Ob
end

function ShieldBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self:getAmount()
            self.rChar:setOverlayText("SHIELD +"..self.nBuffedAmount, 2, { 0, 0, 1 } )
            self.rChar:setColor(0, 1, 0)
            self.rChar:addShield(self.nBuffedAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:addShield(0 - self.nBuffedAmount)
            self.rChar:setColor(1, 1, 1)
        end
    end

    return Ob
end

function BlockEffectivenessBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self.nAmount
            self.rChar:addBlockEffectiveness(self.nBuffedAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:addBlockEffectiveness(0 - self.nBuffedAmount)
        end
    end

    return Ob
end

function BlockChanceBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self.nAmount
            self.rChar:addBlockChance(self.nBuffedAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:addBlockChance(0 - self.nBuffedAmount)
        end
    end

    return Ob
end

function VampireBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self.nAmount
            self.rChar:addVampirePct(self.nBuffedAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar and self.nBuffedAmount then
            self.rChar:addVampirePct(-self.nBuffedAmount)
        end
    end

    return Ob
end

function HealBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)
    Ob.parentOnTurnElapsed = Ob.onTurnElapsed

    function Ob:buffThread()
    end

    function Ob:heal()
        local amt = self:getAmount()
        self.rChar:addHealth(amt, true)
        --self.rChar:setOverlayText("+ HEALTH "..amt, 0.75, { 0, 1, 0 } )
    end

    function Ob:onTurnElapsed()        
        if not self.rChar:isDead() then
            self:heal()
        end        
        self:parentOnTurnElapsed()
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:setColor(1, 1, 1)
        end
    end

    return Ob
end

function ManaBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)
    Ob.parentOnTurnElapsed = Ob.onTurnElapsed

    function Ob:buffThread()
    end

    function Ob:healMana()
        local amt = self:getAmount()
        self.rChar:addMana(amt, true)
        --self.rChar:setOverlayText("+"..amt.."FOCUS", 0.75, { 0, 1, 0 } )
    end

    function Ob:onTurnElapsed()        
        if not self.rChar:isDead() then
            self:healMana()
        end        
        self:parentOnTurnElapsed()
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:setColor(1, 1, 1)
        end
    end

    return Ob
end

function DamageBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)
    Ob.parentOnTurnElapsed = Ob.onTurnElapsed

    function Ob:buffThread()
    end

    function Ob:onBuffEnd()
    end

    function Ob:onTurnElapsed()        
        if not self.rChar:isDead() then
            self.rChar:takeDamage(self:getAmount())
        end        
        self:parentOnTurnElapsed()
    end

    return Ob
end

function CurseBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then            
            local nElapsedTime = 0
            local nIncrement = 0.2
            self.rChar:setOverlayText("CURSED!", 4, { 0, 0, 0 } )
            while nElapsedTime < self.rDef.nDuration do
                if not self.rChar:isDead() then
                    local nPct = nElapsedTime / self.rDef.nDuration
                    local nColor = 1 - nPct
                    self.rChar:setColor(nColor, nColor, nColor)
                    util.sleep(nIncrement)
                    nElapsedTime = nElapsedTime + nIncrement
                else
                    break
                end
            end
            self.rChar:takeDamage(self.rDef.nDamage)
        end
        self:endBuff()
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:setColor(1, 1, 1)
        end
    end

    return Ob
end

function CritBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self:getAmount()
            local nDuration = self.nDuration
            if self.nBuffedAmount then
                self.rChar:setOverlayText("CRIT +"..self.nBuffedAmount*100, 0, { 1, 1, 1 } )
                self.rChar:incrementCritStatMod(self.nBuffedAmount)
            end
            if nDuration then             
                util.sleep(nDuration)
            end
        end
        if not self.nTurnDuration and not self.bIsInfinite then
            self:endBuff()
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            if self.nBuffedAmount then
                self.rChar:setOverlayText("", 0.5, { 1, 1, 1 } )
                self.rChar:incrementPowerStatMod(-self.nBuffedAmount)                
            end
        end
    end

    function Ob:onTurnElapsed()        
        self.nElapsedTurns = self.nElapsedTurns + 1
        if self.nElapsedTurns >= self.nTurnDuration then
            self:endBuff()
        end
    end

    return Ob
end

function PowerBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:buffThread()
        if self.rChar then
            self.nBuffedAmount = self:getAmount()
            local nDuration = self.nDuration
            if self.nBuffedAmount then
                if self.nBuffedAmount > 0 then
                    self.rChar:setOverlayText("POW +"..self.nBuffedAmount, 0, { 1, 1, 1 } )
                else
                    self.rChar:setOverlayText("POW "..self.nBuffedAmount, 0, { 0.75, 0, 0 } )                    
                end
                self.rChar:incrementPowerStatMod(self.nBuffedAmount)
            end
            if nDuration then             
                util.sleep(nDuration)
            end
        end
        if not self.nTurnDuration and not self.bIsInfinite then
            self:endBuff()
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            if self.nBuffedAmount then
                self.rChar:setOverlayText("", 0.5, { 1, 1, 1 } )
                self.rChar:incrementPowerStatMod(-self.nBuffedAmount)                
            end
        end
    end

    return Ob
end

function CrunchTimeBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:onTurnElapsed()        
        if not self.rEvent or not self.rEvent.nCrunchTimes or self.rEvent.nCrunchTimes <= 0 then
            self:endBuff()
        end
    end

    function Ob:onBuffStart()
        if self.rChar then self.rChar:getHero():decreaseMorale() end
    end

    return Ob
end

function RevengeDamageBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:onBuffStart()
        if self.rChar then
            self.nBuffedAmount = self:getAmount()
            self.rChar:incrementRevengeDamageAmount(self.nBuffedAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:incrementRevengeDamageAmount(-self.nBuffedAmount)
        end
    end

    return Ob
end

function DeathBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)
    Ob.parentOnTurnElapsed = Ob.onTurnElapsed

    function Ob:onBuffStart()
        self.rLayer = Renderer.getLayer('EventOverlay').RenderLayer
        self.rCountdownUI = CountdownUI:new(self.rLayer)
        self.rCountdownUI:setLoc(0, 230)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(rChar)
        self.rCountdownUI:updateProgress(100 * self.nElapsedTurns / self.nTurnDuration)
    end

    function Ob:onBuffEnd()
        self.rCountdownUI:setVisible(false)
        if not self.rChar:isDead() and not self.rEvent:isCompleting() then self.rChar:takeDamage(9999) end
    end

    function Ob:onTurnBegan()
        self.rCountdownUI:updateProgress(100 * (self.nElapsedTurns+.5) / self.nTurnDuration)
        self.bBegan = true
    end

    function Ob:onTurnElapsed()        
        -- If the buff is started mid-turn, don't count it towards the progress countdown.
        if not self.bBegan then return end
        self.bBegan = false

        self.rCountdownUI:updateProgress(100 * (self.nElapsedTurns+1) / self.nTurnDuration)
        self:parentOnTurnElapsed()
    end
    return Ob
end

function CoinBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:onBuffStart()
        if self.rChar then
            self.rChar:addKORewardPct(self.rDef.nAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:addKORewardPct(-self.rDef.nAmount)
        end
    end

    return Ob
end

function DodgeBuff:new(rDef, rChar, rSourceActor)
    local Ob = Buff:new(rDef, rSourceActor)

    function Ob:onBuffStart()
        if self.rChar then
            self.rChar:addDodgeChance(self.rDef.nAmount)
        end
    end

    function Ob:onBuffEnd()
        if self.rChar then
            self.rChar:addDodgeChance(-self.rDef.nAmount)
        end
    end

    return Ob
end

Buff.tConstructors =
{
    Buff = Buff.new,
    StunBuff = StunBuff.new,
    ShieldBuff = ShieldBuff.new,
    HealBuff = HealBuff.new,
    ManaBuff = ManaBuff.new,
    CritBuff = CritBuff.new,
    DamageBuff = DamageBuff.new,
    PowerBuff = PowerBuff.new,
    CurseBuff = CurseBuff.new,
    BlockChanceBuff = BlockChanceBuff.new,
    BlockEffectivenessBuff = BlockEffectivenessBuff.new,
    RevengeDamageBuff = RevengeDamageBuff.new,
    CrunchTimeBuff = CrunchTimeBuff.new,
    DeathBuff = DeathBuff.new,
    VampireBuff = VampireBuff.new,
    CoinBuff = CoinBuff.new,
    DodgeBuff = DodgeBuff.new,    
}

function Buff.getConstructor(sName)
    if Buff.tConstructors[sName] then
        return Buff.tConstructors[sName]
    end
end

return Buff
