local Projectile = {}
local util = require('DFUtil')
local Renderer = require('Renderer')
local Sprite = require('SpriteSheet')

local kTIMESTEP = 0.03
local kDESIRED_DIST = 2

function Projectile:new(rSourceChar, rTargetChar, rProjectileDef, rRenderLayer, nCharge)
    if not rSourceChar or not rTargetChar then
        print("ERROR: Atempting to create a projectile without a target or source")
    end
    local Ob = MOAITransform.new()
    Ob.__typename = "Projectile"
    Ob.rRenderLayer = rRenderLayer or Renderer.getLayer('EventOverlay').RenderLayer    

    if rProjectileDef.sSpriteSheetName then
        local nScale = 1
        if rProjectileDef.nScaleRange then
            local nDiff = rProjectileDef.nScaleRange[2] - rProjectileDef.nScaleRange[1]
            nDiff = nDiff * nCharge -- charge is between 0 and 1
            nScale = rProjectileDef.nScaleRange[1] + nDiff
        end
        Ob.rSprite = Sprite:new(rProjectileDef.sSpriteSheetName, rProjectileDef.sAnimDefName, 0, 0, 'center', 'center')
        if rProjectileDef.sDefaultSpriteIndex then
            Ob.rSprite:setSpriteIndex(rProjectileDef.sDefaultSpriteIndex)
        end
        Ob.rSprite:setLoc(0, 0)
        Ob.rSprite:setScl(nScale, nScale)
        if rProjectileDef.nSpriteRot then
            Ob.rSprite:setAttr(MOAITransform2D.ATTR_Z_ROT, rProjectileDef.nSpriteRot)
        end
        Ob.rSprite:setParent(Ob)
    end

    Ob.rSourceChar = rSourceChar
    Ob.rProjectileDef = rProjectileDef
    Ob.rTargetChar = rTargetChar

    function Ob:getTargetLoc()
        if self.rProjectileDef.tAbsTargetLoc then
            return self.rProjectileDef.tAbsTargetLoc.x, self.rProjectileDef.tAbsTargetLoc.y
        end
        local nTargetLocX, nTargetLocY = self.rTargetChar:getLoc()
        local nFactor = Renderer.getTextureSizeFactor()
        nTargetLocX = nTargetLocX + self.rProjectileDef.nTargetOffsetX * nFactor
        nTargetLocY = nTargetLocY + self.rProjectileDef.nTargetOffsetY * nFactor
        return nTargetLocX, nTargetLocY
    end

    function Ob:getSourceLoc()
        if self.rProjectileDef.tAbsSourceLoc then
            return self.rProjectileDef.tAbsSourceLoc.x, self.rProjectileDef.tAbsSourceLoc.y
        end
        local nSourceX, nSourceY = self.rSourceChar:getLoc()
        local nFactor = Renderer.getTextureSizeFactor()
        nSourceX = nSourceX + self.rProjectileDef.nSourceOffsetX * nFactor
        nSourceY = nSourceY + self.rProjectileDef.nSourceOffsetY * nFactor
        return nSourceX, nSourceY
    end

    function Ob:moveThread()

        if self.rProjectileDef.nSpawnDelay then
            util.sleep(self.rProjectileDef.nSpawnDelay)
        end
        self.rRenderLayer:insertProp(self.rSprite)
        if self.rProjectileDef.sSpriteAnimName then
            self.rSprite:playAnimation(self.rProjectileDef.sSpriteAnimName)
        end
        local nSourceX, nSourceY = self:getSourceLoc()
        local nTargetLocX, nTargetLocY = self:getTargetLoc()

        local nDist = util.distance(nSourceX, nSourceY, nTargetLocX, nTargetLocY)
        local nLastDist = nDist
        local nDuration = nDist / self.rProjectileDef.nSpeed
        local nElapsedTime = 0
        self:setLoc(nSourceX, nSourceY)
        while nDist > kDESIRED_DIST and nLastDist >= nDist do

            coroutine.yield()

            nSourceX, nSourceY = self:getLoc()
            nTargetLocX, nTargetLocY = self:getTargetLoc()
            local alpha = nElapsedTime / nDuration
			local x = (1 - alpha) * nSourceX + alpha * nTargetLocX
			local y = (1 - alpha) * nSourceY + alpha * nTargetLocY		
			self:setLoc(x, y)
            nElapsedTime = nElapsedTime + kTIMESTEP

            --util.sleep(kTIMESTEP)
            nLastDist = nDist
            nDist = util.distance(nSourceX, nSourceY, nTargetLocX, nTargetLocY)

            -- temp testing hack woo
            if nDist < kDESIRED_DIST + alpha*1.1 then
                self.bDone = true
            end
        end
        -- reached target
        self.bDone = true
        self.rRenderLayer:removeProp(self.rSprite)    
    end

    local nSourceX, nSourceY = Ob:getSourceLoc()
    Ob:setLoc(nSourceX, nSourceY)

    Ob.rThread = MOAIThread.new()
    Ob.rThread:run(Ob.moveThread, Ob)    

    return Ob
end

return Projectile
