local Renderer = require('Renderer')
local Sprite = require('SpriteSheet')
local NothingActivity = require('activity/NothingActivity')
local RestActivity = require('activity/RestActivity')
local TrainActivity = require('activity/TrainActivity')
local TrainHPActivity = require('activity/TrainHPActivity')
local ResearchActivity = require('activity/ResearchActivity')
local PTJobActivity = require('activity/PTJobActivity')
local RecreationActivity = require('activity/RecreationActivity')
local TrainAbilityActivity = require('activity/TrainAbilityActivity')
local util = require('DFUtil')
local DFFile = require('DFCommon.File')
local Ability = require('Ability/Ability')
local Item = require('Item')
local SpriteSheet = require('SpriteSheet')
local DFGraphics = require('DFCommon.Graphics')
local DFAchievement = require('DFCommon.Achievement')
local EffectInstance = require('EffectInstance')
local DFUtil = require('DFCommon.Util')

local Character = {}
Character.tUIScreenInfo =
{
    SMALL = { sTextureSizeSuffix = '_Med', nTextureSizeFactor = 0.9 },
    MED = { sTextureSizeSuffix = '_Med', nTextureSizeFactor = 0.9 },
    LARGE = { key = 'LARGE', sTextureSizeSuffix = '_Lg', nTextureSizeFactor = 0.45 },
}

Character.kBLOCK_PCT = 0.4

-- TWEAKABLES
Character.tLevelToTitle =
{
    --"", -- never level up to lvl1
    "STXT055TEXT",
    "STXT056TEXT",
    "STXT057TEXT",
    "STXT059TEXT",
    "STXT060TEXT",
    "STXT061TEXT",
    "STXT062TEXT",
    "STXT063TEXT",
    "STXT064TEXT",
    "RANKS009TEXT",
    "RANKS010TEXT",
    "RANKS011TEXT",
    "RANKS012TEXT",
    "RANKS013TEXT",
    "RANKS014TEXT",
    "RANKS015TEXT",
    "RANKS016TEXT",
    "RANKS017TEXT",
    "RANKS018TEXT",
}
Character.tInvSlotUnlockInfo =
{
    { nLevel = 1 }, -- slot 1
    { nLevel = 4 }, -- slot 2
    { nLevel = 7 }, -- slot 3
    { nLevel = 10 }, -- slot 4
}
function Character.getMaxSlots()
    return 1
end 

Character.eMorale =
{
    LOW = 1,
    MEDLOW = 2,
    MED = 3,
    MEDHIGH = 4,
    HIGH = 5,
}

Character.tMoraleSprites=
{
'morale_superlow',
'morale_low',
'morale_med',
'morale_high',
'morale_superhigh',
}

Character.tMiscTweaks =
{
    tBlockPct =
    {
        0.0,    
        0.2,
        0.3,
        0.4,
        0.5,
    },
    tCriticalAttackChance =
    {
        0.0,
        0.075,
        0.15,
        0.30,
        0.50,
    },
    nCriticalAttackMultiplier = 1.25,
    kNUM_ACT_FOR_MORALE_DEC = 2,
    nBASE_MANA_POOL = 4, -- mana pool at awareness 1
    nAWARENESS_TO_MANA = 1,
    kMANA_PER_MINUTE = 4/2, -- 4 bars in 2 minutes
    kMANA_PER_MINUTE_BONUS = .075 -- every additional awa gives you 1 more mana, and .075 more regen / minute.
}

-- END TWEAKABLES
Character.tLevelImprovementInfo = dofile(DFFile.getDataPath('Characters/LevelImprovementInfo.lua'))

Character.tCharIconDecks = {}
Character.tCharDefinitions = {}
local tHeroDefs = dofile(DFFile.getDataPath('Characters/CharDefs/Heroes.lua'))
if tHeroDefs then
    for key, value in pairs(tHeroDefs) do
        Character.tCharDefinitions[key] = value
    end
end
local tHenchmanDefinitions = dofile(DFFile.getDataPath('Characters/CharDefs/Henchmen.lua'))
if tHenchmanDefinitions then
    for key, value in pairs(tHenchmanDefinitions) do
        Character.tCharDefinitions[key] = value
    end
end
local tVillainDefinitions = dofile(DFFile.getDataPath('Characters/CharDefs/Villains.lua'))
if tVillainDefinitions then
    if tVillainDefinitions.tDefs then
        for key, value in pairs(tVillainDefinitions.tDefs) do
            Character.tCharDefinitions[key] = value
        end
    end
    Character.tMostWantedThugs = tVillainDefinitions.tMostWantedThugs
end
local tOtherActorDefinitions = dofile(DFFile.getDataPath('Characters/CharDefs/OtherActors.lua'))
if tOtherActorDefinitions then
    for key, value in pairs(tOtherActorDefinitions) do
        Character.tCharDefinitions[key] = value
    end
end

Character.tInitialHero =
{
--  'TestMale01',
--'TestMale01b',
-- 'TestMale01c',
--'TestMale02',
--'TestMale02b',
--'TestFemale01',
--'TestFemale02',
--'Robot01',
}

local kPER_LEVEL_COST = 50

Character.eCharDirections =
{
    DIR_NW = 1,
    DIR_NE = 2,
    DIR_SW = 3,
    DIR_SE = 4,
}

Character.tIdles =
{
    "idle1",
    "blink",
}

Character.tIdlesBk =
{
    "idlebk1",
}

local tActivityClasses =
{
    Rest = RestActivity,
    Train = TrainActivity,
    HPTrain = TrainHPActivity,
    Research = ResearchActivity,
    Nothing = NothingActivity,
    PTJob = PTJobActivity,
    Recreation = RecreationActivity,
    AbilityTrain = TrainAbilityActivity,
}
Character.tActivityDefs = dofile(DFFile.getDataPath('Activities/ActivityDefs.lua'))
function Character.getActivityDef(sActivityType)
    if sActivityType then
        return Character.tActivityDefs[sActivityType]
    end
    return nil
end
function Character.updateActivityDef(sActivityType, tValues)
    g_Game:updateDefWithTweak(Character.tActivityDefs, sActivityType, tValues)
end

Character.tAbilityDefs = dofile(DFFile.getDataPath('Characters/Abilities/AbilityDefs.lua'))
Ability.tAbilityReferencedData = Character.tAbilityDefs._abilityReferencedData
--Character.tAbilityReferencedData = Character.tAbilityDefs._abilityReferencedData
function Character.getAbilityDef(sAbilityType)
    if sAbilityType then
        return Character.tAbilityDefs[sAbilityType]
    end
end
function Character.updateAbilityDef(sAbilityDef, tValues)
    g_Game:updateDefWithTweak(Character.tAbilityDefs, sAbilityDef, tValues)
end

local tCostumeInfo = dofile(DFFile.getDataPath('Characters/CostumeDefs.lua'))
Character.tCostumeList = tCostumeInfo.tList
Character.tCostumeDefs = tCostumeInfo.tDefs
function Character.getCostumeDef(sCostumeName)
    if sCostumeName then
        return Character.tCostumeDefs[sCostumeName]
    end
end
function Character.updateCostumeDef(sCostumeKey, tValues)
    g_Game:updateDefWithTweak(Character.tCostumeDefs, sCostumeKey, tValues)
end
function Character.getCostumeList()
    return Character.tCostumeList
end

local tPackInfo = dofile(DFFile.getDataPath('Characters/PackDefs.lua'))
Character.tPackList = tPackInfo.tList
Character.tPackDefinitions = tPackInfo.tDefs
function Character.getPackDef(sPackName)
    if sPackName then
        return Character.tPackDefinitions[sPackName]
    end
end
function Character.updatePackDef(sPackKey, tValues)
    g_Game:updateDefWithTweak(Character.tPackDefinitions, sPackKey, tValues)
end
function Character.getPackList()
    if Character.tPackListOverride then
        return Character.tPackListOverride
    else
        return Character.tPackList
    end
end
function Character.setPackListOverride(tPackList)
    Character.tPackListOverride = tPackList
end

function Character.getCharDef(sCharDefKey)
    if Character.tCharDefinitions and sCharDefKey then
        return Character.tCharDefinitions[sCharDefKey]
    end
    return nil
end
function Character.updateCharDef(sCharDefKey, tValues)
    g_Game:updateDefWithTweak(Character.tCharDefinitions, sCharDefKey, tValues)
end

function Character.updateLevelImprovementInfo(tInfo)
    if tInfo then
        Character.tLevelImprovementInfo = DFUtil.deepCopy(tInfo)
    end
end

function Character.updateTweakables(sDefKey, tValues)
    g_Game:updateDefWithTweak(Character.tMiscTweaks, sDefKey, tValues)
end

function Character.getMostWantedThugs()
    return Character.tMostWantedThugs
end

function Character.getShadowProp()
    -- we don't care about unloading this since the mm is always loaded
    local sSuffix = Character.tUIScreenInfo[Renderer.getCurScreenInfo().key].sTextureSizeSuffix
    if sSuffix then
        local rDeck = DFGraphics.loadSpriteSheet('Characters/MM/MM'..sSuffix, g_IsInStartupSequence)
        if rDeck then
            local rProp = MOAIProp2D.new()
            rProp:setDeck(rDeck)
            local nIndex = rDeck.names['Shadow']
            rProp:setIndex(nIndex)
            rProp:setScl(0.6)
            return rProp
        end
    end
    return nil
end

function Character:new(sCharDefKey, tSaved, packLevel)
	-- CONSTRUCTOR
	local Ob = MOAITransform.new()
    Ob.__typename = "Character"
    Ob.sCharDefKey = sCharDefKey
    Ob.tCharDef = Character.getCharDef(sCharDefKey)
    
    function Ob:getCharDefKey()
        return self.sCharDefKey
    end
    
    function Ob:loadSpriteSheets(tSpriteSheetDefPaths)
        if tSpriteSheetDefPaths then
            local tSprites = {}
            for i, sSpriteSheetDefPath in ipairs(tSpriteSheetDefPaths) do              
                local rSprite = nil
                local tScreenInfo = Character.tUIScreenInfo[Renderer.getCurScreenInfo().key]
                if not tScreenInfo then
                    return nil
                end
                rSprite = Sprite:new(sSpriteSheetDefPath, self.tCharDef.sAnimDefFileName, self.tCharDef.centerToBaseX, self.tCharDef.centerToBaseY, nil, nil, nil, nil, tScreenInfo, g_IsInStartupSequence)
                if rSprite then
                    rSprite.rCharEnt = self
                    rSprite:setParent(self)
                    table.insert(tSprites, rSprite)
                    self.tLayer.RenderLayer:insertProp(rSprite)
                    self.tLayer.Partition:insertProp(rSprite)
                end
            end
            return tSprites
        end
        return nil
    end

    function Ob:unloadSpriteSheets(tSprites)
        if not tSprites then
            tSprites = self.tSprites
        end
    
        for i, rSprite in ipairs(tSprites) do            
            self.tLayer.RenderLayer:removeProp(rSprite)
            self.tLayer.Partition:removeProp(rSprite)
            rSprite:unloadSpriteSheet()
        end
    end

    function Ob:setShadowVisible(bVisible)
        if self.rShadow then
            self.rShadow:setVisible(bVisible)
        end
    end

    function Ob:unlockCostume(sCostumeKey)
        self.tSaved.tUnlockedCostumes[sCostumeKey] = true
    end

    function Ob:getCostumeKeyList()
        if not self.tCharDef.tCostumeKeys then
            return nil
        end
        local tCostumeKeys = {}
        for i, sCostumeKey in ipairs(self.tCharDef.tCostumeKeys) do
            table.insert(tCostumeKeys, sCostumeKey)
        end
        if self.tCharDef.tConditionalCostumeKeys then
            for c, rInfo in ipairs(self.tCharDef.tConditionalCostumeKeys) do
                if rInfo.sEnvironmentFlag and rInfo.sReqValue then
                    if MOAIEnvironment[rInfo.sEnvironmentFlag] then
                        if type(MOAIEnvironment[rInfo.sEnvironmentFlag]) == 'string' then
                            if string.find(string.lower(MOAIEnvironment[rInfo.sEnvironmentFlag]), rInfo.sReqValue) then
                                table.insert(tCostumeKeys, rInfo.sCostumeKey)
                            end
                        end
                    end
                end
            end  
        end
        return tCostumeKeys
    end

    function Ob:isCostumeUnlocked(sCostumeKey)
        if sCostumeKey then
            local rDef = Character.getCostumeDef(sCostumeKey)
            if rDef and rDef.sDefKey then
                if self.tSaved.tUnlockedCostumes[rDef.sDefKey] then
                    return true
                else
                    return false
                end
            end
        end
        return false
    end

    function Ob:getCostumeKeyFromCostumeSet(sCostumeSet)
        local keys = self:getCostumeKeyList()
        if not keys then return end

        for _,key in ipairs(keys) do
            local def = Character.getCostumeDef(key)
            if def.sDefKey == sCostumeSet then
                return key
            end
        end
    end

    function Ob:setCostume(sCostumeSet)
        if sCostumeSet then

            if self.rCostumeItem then
                self.rCostumeItem:setOwner(nil)
                self.rCostumeItem = nil
            end

            local sPrevAnimation = nil
            self.tSaved.sCostumeSet = sCostumeSet
            if self.tSprites then
                for i, rSprite in ipairs(self.tSprites) do
                    local sAnim = rSprite:getCurrentAnimationName()
                    if sAnim and sAnim ~= "" then
                        sPrevAnimation = sAnim
                    end
                end
            end
            if self.tCharDef.tSpriteSheetDefPaths[sCostumeSet] then
                -- unload the previous sprites
                if self.tSprites then
                    self:unloadSpriteSheets(self.tSprites)
                end
                self:unloadPortraitSprites()
            end
            self.sCostumeSet = sCostumeSet

            local sCostumeKey = self:getCostumeKeyFromCostumeSet(sCostumeSet)
            if sCostumeKey then
                local def = Character.getCostumeDef(sCostumeKey)
                local rDef = def.tEquipDef
                if rDef then
                    local constructor = Item.getConstructor(rDef.sItemType)
                    local rItem = constructor(Item, rDef, sItemDefKey)
                    rItem:setOwner(self)
                    self.rCostumeItem = rItem
                end
            end

            self:loadPortraitSprites(sCostumeSet)
            self.tSprites = self:loadSpriteSheets(self.tCharDef.tSpriteSheetDefPaths[sCostumeSet]) 
            for i, rSprite in ipairs(self.tSprites) do
                rSprite.onFinger = self.onSpriteFinger
            end

            for key, tSpriteSheets in pairs(self.tExtraSpriteSheets) do
                self:clearExtraSprites(key)
                self:addExtraSprites(tSpriteSheets, key) -- adds the proper sprites for the costume
            end            
            if sPrevAnimation then
                self:playAnimation(sPrevAnimation)
            end

            if self.rShadow then
                self.tLayer.RenderLayer:removeProp(self.rShadow)
            end
            self.rShadow = Character.getShadowProp()
            if self.rShadow then
                self.tLayer.RenderLayer:insertProp(self.rShadow)
                self.rShadow:setParent(self) 
            end
            if self.rLevelUpSprite then
                self.tLayer.RenderLayer:insertProp(self.rLevelUpSprite)
            end
            self:updateLayer()
        end
    end

    function Ob:loadPortraitSprites(sCostumeSet)
        if self.tCharDef.tPortraitPaths and self.tCharDef.tPortraitPaths[sCostumeSet] then
            local sPath = self.tCharDef.tPortraitPaths[sCostumeSet]..Renderer.getTextureSizeSuffix()
            self.rPortraitSpriteSheet = DFGraphics.loadSpriteSheet(sPath, g_IsInStartupSequence)
            if self.rPortraitSpriteSheet then
                for sName, data in pairs(self.rPortraitSpriteSheet.names) do
                    DFGraphics.alignSprite(self.rPortraitSpriteSheet, sName, "left", "top", Renderer.getTextureSizeFactor(), Renderer.getTextureSizeFactor())
                end                    
                self.sCurPortraitPath = sPath
            end
        end
    end

    function Ob:unloadPortraitSprites()
        if self.sCurPortraitPath then
            DFGraphics.unloadSpriteSheet(self.sCurPortraitPath)
            self.sCurPortraitPath = nil
            self.rPortraitSpriteSheet = nil                    
        end
    end

    function Ob:getCostumeSetName()
        return self.sCostumeSet
    end

    function Ob:getPortraitProp()
        if self.rPortraitSpriteSheet and self.tSaved.sCostumeSet then
            local nIndex = self.tCharDef.tPortraitIndex[self.tSaved.sCostumeSet]
            if nIndex then
                local rProp = MOAIProp2D.new()
                rProp:setDeck(self.rPortraitSpriteSheet)
                rProp:setIndex(self.rPortraitSpriteSheet.names[nIndex])
                return rProp
            end
        end
        return nil
    end

    function Ob:addExtraSprites(tSpriteSheetPaths, sAnimKey)        
        if tSpriteSheetPaths and self.sCostumeSet and tSpriteSheetPaths[self.sCostumeSet] then
            self.tExtraSpriteSheets[sAnimKey] = tSpriteSheetPaths
            if not self.tExtraSprites[sAnimKey] then
                self.tExtraSprites[sAnimKey] = self:loadSpriteSheets(tSpriteSheetPaths[self.sCostumeSet])            
                -- set all the extra ones invisible by default
                for i, rSprite in ipairs(self.tExtraSprites[sAnimKey]) do            
                    rSprite.onFinger = self.onSpriteFinger
                    rSprite:setVisible(false)
                end
            end
        end
    end

    function Ob:clearExtraSprites(sAnimKey)
        if sAnimKey then
            self.tExtraSpriteSheets[sAnimKey] = nil
            if self.tExtraSprites[sAnimKey] then
                self:unloadSpriteSheets(self.tExtraSprites[sAnimKey])
                self.tExtraSprites[sAnimKey] = nil
            end        
        end
    end

    function Ob:getFightSpriteSheetPaths()
        return self.tCharDef.tFightSpriteSheetDefPaths[self.sCostumeSet]
    end

    function Ob:incrementStatMod(sStatName, nAmount)
        if sStatName and self.tStatMods[sStatName] then
            self.tStatMods[sStatName] = self.tStatMods[sStatName] + nAmount
        end
    end

    function Ob:getStatMod(sStatName)
        if sStatName and self.tStatMods[sStatName] then
            return self.tStatMods[sStatName]
        end
    end

    function Ob:addDefaultAttackMod(sKey, nChance, tBuffs)
        local rInfo = {}
        rInfo.sKey = sKey
        rInfo.nChance = nChance
        rInfo.tBuffs = tBuffs
        table.insert(self.tDefaultAttackMods, rInfo)
    end

    function Ob:removeDefaultAttackMod(sKey)
        for i, rInfo in ipairs(self.tDefaultAttackMods) do
            if rInfo.sKey == sKey then
                table.remove(self.tDefaultAttackMods, i)
                break
            end
        end
    end
    
    function Ob:getDefaultAttackMods()
        return self.tDefaultAttackMods
    end

    function Ob:addFightBuff(sKey, tBuffs)
        local rInfo = {}
        rInfo.sKey = sKey
        rInfo.tBuffs = tBuffs
        table.insert(self.tFightBuffs, rInfo)
    end     

    function Ob:removeFightBuff(sKey)
        for i, rInfo in ipairs(self.tFightBuffs) do
            if rInfo.sKey == sKey then
                table.remove(self.tFightBuffs, i)
                break
            end
        end
    end

    function Ob:getFightBuffs()
        return self.tFightBuffs
    end

    function Ob:getInventory()
        return self.tInventory
    end

    function Ob:getInventoryKeys()
        return self.tSaved.tItemDefKeys
    end

    -- if the slot is unlocked, then return the level at which it will be unlocked
    function Ob:getItemAtSlot(nSlot)
        if Character.tInvSlotUnlockInfo[nSlot] then
            local nLevelForSlot = Character.tInvSlotUnlockInfo[nSlot].nLevel
            if nLevelForSlot <= self:getStatLevel() then
                return self.tInventory[nSlot]
            else
                return nil, nLevelForSlot
            end
        end
        return nil
    end

    function Ob:getNextAvailableSlotIndex()
        for i, rInvSlot in ipairs(self.tInventory) do
            if rInvSlot == Item.kEMPTY_SLOT then
                if Character.tInvSlotUnlockInfo[i].nLevel <= self:getStatLevel() then
                    return i
                else
                    return nil
                end
            end
        end
        return nil
    end

    function Ob:hasAvailableSlots()
        if self:getNextAvailableSlotIndex() then
            return true
        else
            return false
        end
    end

    function Ob:unequipItem(rItem)
        for i, rExistingItem in ipairs(self.tInventory) do
            if rExistingItem == rItem then
                g_MiddleManager:addEquipmentToInventory(rItem:getItemDefKey())
                rItem:setOwner(nil)
                self.tInventory[i] = Item.kEMPTY_SLOT
                table.remove(self.tSaved.tItemDefKeys, i)
                return
            end
        end
    end

    function Ob:addMeteoriteToInventory(sMeteoriteDefKey, nLevel)
        if sMeteoriteDefKey and nLevel then
            self:removeEquippedMeteorite()
            local rDef = Item.getMeteoriteDef(sMeteoriteDefKey)
            if rDef then
                local constructor = Item.getConstructor('Meteorite')
                if constructor then
                    local rMeteorite = constructor(Item, rDef, sMeteoriteDefKey, nLevel)
                    if rMeteorite then
                        self.rEquippedMeteorite = rMeteorite
                        self.rEquippedMeteorite:setOwner(self)
                        self.tSaved.tEquippedMeteoriteInfo.sMeteoriteDefKey = sMeteoriteDefKey
                        self.tSaved.tEquippedMeteoriteInfo.nLevel = nLevel
                    end
                end
            end
        end
    end

    function Ob:removeEquippedMeteorite()
        if self.rEquippedMeteorite then
            g_MiddleManager:addMeteoriteToInventory(self.rEquippedMeteorite:getItemDefKey(), self.rEquippedMeteorite:getLevel())
            self.rEquippedMeteorite:setOwner(nil)            
            self.rEquippedMeteorite = nil
            if self.tSaved.tEquippedMeteoriteInfo then
                if self.tSaved.tEquippedMeteoriteInfo.sMeteoriteDefKey then
                    self.tSaved.tEquippedMeteoriteInfo.sMeteoriteDefKey = nil 
                end
                if self.tSaved.tEquippedMeteoriteInfo.nLevel then
                    self.tSaved.tEquippedMeteoriteInfo.nLevel = nil
                end
            end
        end
    end

    function Ob:getEquippedMeteorite()
        return self.rEquippedMeteorite
    end

    function Ob:isMM()
        return (self.addMMTrainIntActivityAtIndex and true)
    end

    function Ob:addItemToNextAvailableSlot(rItem)
        local nSlot = self:getNextAvailableSlotIndex()
        if nSlot and rItem then
            self.tInventory[nSlot] = rItem
            rItem:setOwner(self)
            self.tSaved.tItemDefKeys[nSlot] = rItem:getItemDefKey()
            return true
        end
        return false
    end

    function Ob:isAssignedToEvent()
        if self.rAssignedEvent then
            return true
        else
            return false
        end
    end

    function Ob:onAssignedToEvent(rEvent)
        self.rAssignedEvent = rEvent        
    end

    function Ob:getAssignedEvent()
        return self.rAssignedEvent
    end
	
	-- FUNCTIONS --
    function Ob:getCenterPosition(x, y)
        if self.tSprites and self.tSprites[1] then -- the first sprite is considered the base
            return self.tSprites[1]:getCenterPosition(x, y)
        end
        return nil
    end

    function Ob:getRootPosition(x, y)
        if self.tSprites and self.tSprites[1] then -- the first sprite is considered the base
            return self.tSprites[1]:getRootPosition(x, y)
        end
        return nil
    end

    function Ob:getIdleAnim()
        local tIdles = nil
        if self.eDirection == Character.eCharDirections.DIR_NW or self.eDirection == Character.eCharDirections.DIR_NE then
            tIdles = self.tIdlesBk
        else
            tIdles = self.tIdles
        end
        if tIdles then
            return tIdles[math.random(table.getn(tIdles))]
        end
        return nil
    end

    function Ob:getDefaultAbility()
        return self.rDefaultAbility
    end

    function Ob:getStanceAbility()
        return self.tAbilities["cyclestance"]
    end

    function Ob:getActiveAbility()
        if self.tSaved.sActiveAbilityKey and self.tAbilities[self.tSaved.sActiveAbilityKey] then
            return self.tAbilities[self.tSaved.sActiveAbilityKey]
        else
            local sDefKey = nil
            for sKey, rAbility in pairs(self.tAbilities) do
                if rAbility:isUnlocked() and rAbility:showInUI() then
                    sDefKey = sKey
                    break
                end
            end
            if sDefKey then
                self.tSaved.sActiveAbilityKey = sDefKey
                return self.tAbilities[self.tSaved.sActiveAbilityKey]
            end
        end
    end

    function Ob:setActiveAbility(sAbilityDefKey)
        if sAbilityDefKey and self.tAbilities[sAbilityDefKey] then
            self.tSaved.sActiveAbilityKey = sAbilityDefKey
        end
    end

    function Ob:getNumAvailableAbilities()
        local nNum = 0
        for sDefKey, rAbility in pairs(self.tAbilities) do
            if rAbility:showInUI() and rAbility:isUnlocked() then
                nNum = nNum + 1
            end
        end
        return nNum
    end

    function Ob:getNextAvailableAbilityKey()
        local sFirstDefKey = nil
        local sAbilityKey = nil
        local bFoundAbility = false
        local rActiveAbility = self:getActiveAbility()
        for sDefKey, rAbility in pairs(self.tAbilities) do
            if rAbility:isUnlocked() and rAbility:showInUI() then
                if not sFirstDefKey then
                    sFirstDefKey = sDefKey
                end
                if bFoundAbility then
                    sAbilityKey = sDefKey
                    break
                end
                if rAbility == rActiveAbility then
                    bFoundAbility = true
                end
            end
        end
        if not sAbilityKey then
            sAbilityKey = sFirstDefKey
        end
        return sAbilityKey
    end

    function Ob:getPrevAvailableAbilityKey()
        local sLastDefKey = nil
        local rActiveAbility = self:getActiveAbility()
        for sDefKey, rAbility in pairs(self.tAbilities) do
            if rAbility:isUnlocked() and rAbility:showInUI() then
                if rAbility == rActiveAbility then
                    if sLastDefKey then
                        break
                    end
                end
                sLastDefKey = sDefKey       
            end
        end
        return sLastDefKey
    end

    function Ob:getAbilities()
        return self.tAbilities
    end

    function Ob:getAbility(sDefKey)
        if sDefKey then
            return self.tAbilities[sDefKey]
        end
        return nil
    end

    function Ob:getAbilityAtLevel(nLevel)
        for i, rAbility in pairs(self.tAbilities) do
            if rAbility:getUnlockLevel() == nLevel then
                return rAbility
            end
        end
        return nil
    end

    function Ob:isFacingLeft()
        return self.eDirection == Character.eCharDirections.DIR_SW or self.eDirection == Character.eCharDirections.DIR_NW
    end

    function Ob:playAnimOnSet(animName, tSprites, bFlipXOverride)
        if not tSprites then
            return 0
        end
        for i, rSprite in ipairs(tSprites) do
            if not rSprite:getAnimation(animName) then
                rSprite:setVisible(false)    
                rSprite:stopCurrentAnimation()
            end
        end
        local nMaxTime = 0
        for i, rSprite in ipairs(tSprites) do
            bFlipX = false
            if not self:isFacingLeft() then
                bFlipX = true
            end
            if bFlipXOverride ~= nil then
                bFlipX = bFlipXOverride
            end
            if rSprite:getAnimation(animName) then
                rSprite:setVisible(true)
                local nDuration = rSprite:playAnimation(animName, bFlipX, false)
                if nDuration > nMaxTime then
                    nMaxTime = nDuration
                end
            else
                rSprite:setVisible(false)
            end
        end
        return nMaxTime
    end

    function Ob:playAnimation(animName, bBlocking, bFlipXOverride)
        if animName then
            self.sLastPlayedAnim = animName
        end
        local nMaxTime = 0
        local nDuration = 0
        local nMaxTime = self:playAnimOnSet(animName, self.tSprites, bFlipXOverride)
        for i, tSprites in pairs(self.tExtraSprites) do
            nDuration = self:playAnimOnSet(animName, tSprites, bFlipXOverride)
            if nDuration > nMaxTime then
                nMaxTime = nDuration
            end
        end
        if bBlocking and nMaxTime > 0 then
            util.sleep(nMaxTime)
        end
    end

    function Ob:setVisible(bVisible)
        for i, rSprite in ipairs(self.tSprites) do
            rSprite:setVisible(bVisible)
        end
        for i, tSprites in pairs(self.tExtraSprites) do
            for i, rSprite in ipairs(tSprites) do
                rSprite:setVisible(bVisible)
            end            
        end
        self:setShadowVisible(bVisible)
    end

    function Ob:insertIntoLayer(rRenderLayer)
        if self.rRenderLayer ~= nil then
            self:removeFromLayer(self.rRenderLayer)
        end
        self.rRenderLayer = rRenderLayer
        if self.rShadow then
            self.rRenderLayer:insertProp(self.rShadow)
        end
        for i, rSprite in ipairs(self.tSprites) do
            self.rRenderLayer:insertProp(rSprite)
        end
        for i, tSprites in pairs(self.tExtraSprites) do
            for i, rSprite in ipairs(tSprites) do
                self.rRenderLayer:insertProp(rSprite)
            end            
        end
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        if rOverlayLayer then
            rOverlayLayer:insertProp(self.rCharOverlayShadowText)
            rOverlayLayer:insertProp(self.rCharOverlayText)       
        end
        if self.rLevelUpSprite then
            self.rRenderLayer:insertProp(self.rLevelUpSprite)
        end
    end
    
    function Ob:removeFromLayer(rRenderLayer)
        if self.rRenderLayer ~= rRenderLayer then
            print("WARNING: attempting to remove character from layer it's not in")
        end
        for i, rSprite in ipairs(self.tSprites) do
            self.rRenderLayer:removeProp(rSprite)
        end
        for i, tSprites in pairs(self.tExtraSprites) do
            for i, rSprite in ipairs(tSprites) do
                self.rRenderLayer:removeProp(rSprite)
            end            
        end
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        if rOverlayLayer then
            rOverlayLayer:removeProp(self.rCharOverlayShadowText)
            rOverlayLayer:removeProp(self.rCharOverlayText)       
        end
        if self.rLevelUpSprite then
            self.rRenderLayer:remove(self.rLevelUpSprite)
        end
        self.rRenderLayer = nil
    end

	function Ob:updateLayer()
        if self.rInteractingProp and self.rInteractingPropUserIndex then
            local nPri = self.rInteractingProp:getInteractPri(self.rInteractingPropUserIndex)
            self:setPriority(nPri)
        else
            local rTile = self:getOccupyingTile()
            if rTile then
                local nPriority = g_CurrentMap:getRenderPriority(self, rTile)
                --            print('pri '..nPriority)
                self:setPriority(nPriority)
            end
        end
	end

    function Ob:setPriority(nPriority)
        for i, rSprite in ipairs(self.tSprites) do
            rSprite:setPriority(nPriority)
        end
        for i, tSprites in pairs(self.tExtraSprites) do
            for i, rSprite in ipairs(tSprites) do
                rSprite:setPriority(nPriority)
            end            
        end
        if self.rCharOverlayShadowText then
            self.rCharOverlayShadowText:setPriority(nPriority + 1)
        end
        if self.rCharOverlayText then
            self.rCharOverlayText:setPriority(nPriority + 2)
        end
        if self.rShadow then
            self.rShadow:setPriority(nPriority - 0.1)
        end
        if self.rLevelUpSprite then
            self.rLevelUpSprite:setPriority(nPriority + 3)
        end
    end

    function Ob:getPriority()
        local nMaxPri = 0
        for i, rSprite in ipairs(self.tSprites) do
            local nPri = rSprite:getPriority()
            if nPri and nPri > nMaxPri then
                nMaxPri = nPri
            end
        end
        return nMaxPri
    end

    function Ob:sayLine(lineCode, sUpperOrLower, bBlocking, bWaitForInput)
        if lineCode then
            local sLine = g_DialogManager:getString(lineCode)
            if sUpperOrLower == 'upper' then
                g_GUI:DisplayUpperDialogText(self, sLine, bBlocking, bWaitForInput)
            else
                g_GUI:DisplayLowerDialogText(self, sLine, bBlocking, bWaitForInput)
            end            
        end
    end

    function Ob:getOccupyingTile()
		local x, y = self:getLoc()
        x, y = self:getCenterPosition(x, y)
        local rTile, nRow, nCol = g_CurrentMap:getTileByCoords(x, y)
        return rTile, nRow, nCol
    end

    function Ob:getOccupyingRoom()
		local x, y = self:getLoc()
        return g_CurrentMap:getOccupyingRoom(x, y)
    end
	
	function Ob:setPosition(x, y, snapToTile)
	
		if snapToTile then
			x, y = g_CurrentMap:getTilePosAtWorldPos(x, y)
		end

		x, y = self:getRootPosition(x, y)		
		self:setLoc(x, y)

		local rTile = g_CurrentMap:getTileByCoords(x, y)        
        if self.rOccupyingTile then
            if rTile ~= self.rOccupyingTile then
                self.rOccupyingTile:removeOccupant(self)
            end
        end
        self.rOccupyingTile = rTile
        self.rOccupyingTile:addOccupant(self)

        -- debug
        local bDebugCharTiles = false
        if bDebugCharTiles then
            local rTile, nRow, nCol = g_CurrentMap:getTileByCoords(x, y)
            if self.rOldTile ~= rTile then
                if self.rOldTile then
                    self.rOldTile:setColor(1, 1, 1)
                end
                if rTile then
                    rTile:setColor(0.5, 0.5, 0.5)
                    local sTileType = g_CurrentMap:getTileTypeName(x, y)
                    print("Row: "..rTile.nRow.." Col: "..rTile.nCol.." Tile "..tostring(sTileType))
                end
                self.rOldTile = rTile           
            end
        end

		self:updateLayer()		
	end

    function Ob:setFacingDirection(sDir)
        local eDir = Character.eCharDirections["DIR_"..sDir]
        self.eDirection = eDir
    end
	
	function Ob:centerOnScreen(offsetX, offsetY, duration)
		local x, y = self:getLoc()
		x, y = self:getCenterPosition(x, y)
		Renderer.setCameraLoc(x + (offsetX or 0), y + (offsetY or 0), duration)
	end
	
	function Ob:getCurrentRoom()
		local x, y = self:getLoc()
		x, y = self:getCenterPosition(x, y)
		return g_CurrentMap:getRoomAtCoord(x, y)
	end
	
	function Ob:getCurrentTileCoords()
        local rTile, nRow, nCol = self:getOccupyingTile()
        if rTile then
            return g_CurrentMap:getWorldCoords(nRow, nCol)
        else
            print("ERROR: not on a valid tile")
        end
	end
	
	-- Stat Functions 
    function Ob:getDef()
        return self.tCharDef
    end
    
	function Ob:getName()
		return self.tCharDef.sName
	end

	function Ob:getStatLevel()
		return self.tSaved.tStats.nLevel
	end

    function Ob:getBaseStatPower()
        return self.tSaved.tStats.nPower 
    end

	function Ob:getSpecialAbilityMult()
        return self.tStatMods.nSpecialAbilityMult
    end

	function Ob:getStatPower() -- as second arg, base power stat
        local nModPower = self.tSaved.tStats.nPower + self.tStatMods.nPower
        if nModPower <= 0 then
            nModPower = 1
        end
		return nModPower, self.tSaved.tStats.nPower
	end

    function Ob:incrementNumTrainingSessions(nAmount)
        self.tSaved.nNumTrainingSessions = self.tSaved.nNumTrainingSessions + nAmount
    end

    function Ob:decrementNumTrainingSessions(nAmount)
        self.tSaved.nNumTrainingSessions = self.tSaved.nNumTrainingSessions - 1
        if self.tSaved.nNumTrainingSessions < 0 then
            self.tSaved.nNumTrainingSessions = 0
        end
    end

    function Ob:getNumTrainingSessions()
        return self.tSaved.nNumTrainingSessions
    end

	function Ob:incrementStatPower(nAmount)
		self.tSaved.tStats.nPower = self.tSaved.tStats.nPower + nAmount
	end
   
	function Ob:getStatAwareness()
        local nInt = self.tSaved.tStats.nAwareness + self.tStatMods.nAwareness
        if nInt <= 0 then
            nInt = 1
        end
		return nInt
	end

    function Ob:getBaseStatAwareness()
        if self.tSaved.tStats.nAwareness then
            return self.tSaved.tStats.nAwareness
        end
        return 0
    end

	function Ob:incrementStatAwareness(nAmount)
		self.tSaved.tStats.nAwareness = self.tSaved.tStats.nAwareness + nAmount
	end

	function Ob:getCurHP()
		return self.tSaved.tStats.nCurHP
	end

	function Ob:setCurHP(nPts)
		self.tSaved.tStats.nCurHP = nPts
	end

    function Ob:incrementHP(nPts, nMaxAmount)
        local nMax = nMaxAmount or self:getMaxHP()
        local nAmount = self:getCurHP() + nPts
        if nAmount >= nMax then
            nAmount = nMax
        end
        self:setCurHP(nAmount)
    end

	function Ob:getMaxHP()
		return (self.tSaved.tStats.nMaxHP + self.tStatMods.nMaxHP) * g_Game.GLOBAL_HEALTH_MOD
	end

	function Ob:incrementMaxHP(nAmount)
		self.tSaved.tStats.nMaxHP = self.tSaved.tStats.nMaxHP + nAmount
	end
	
    function Ob:getMaxMana()
        return Character.tMiscTweaks.nBASE_MANA_POOL + (self:getStatAwareness()-1) * Character.tMiscTweaks.nAWARENESS_TO_MANA + self.tStatMods.nMaxMana
    end

    function Ob:getCurMana()
        return math.floor(self.tSaved.tStats.nCurMana)
    end

	function Ob:setCurMana(nPts)
		self.tSaved.tStats.nCurHP = math.min(self:getMaxMana(), self.tSaved.tStats.nCurHP + nPts)
	end
    
    function Ob:incrementMana(nPts)
        local nMax = nMaxAmount or self:getMaxMana()
        local nAmount = self:getCurMana() + nPts
        self:setCurMana(nAmount)
    end
    
	function Ob:onTick(nElapsedTime)
        -- base mana regen rate + awa-based bonus
        local nAwa = self:getStatAwareness()
        local nRegenRate = Character.tMiscTweaks.kMANA_PER_MINUTE + Character.tMiscTweaks.kMANA_PER_MINUTE_BONUS * (nAwa - 1)
        self:incrementMana(nRegenRate * nElapsedTime / 60)

        -- check hp regen
        if self.nHealthRegenInBasePerSec and self.nHealthRegenInBasePerSec > 0 then
            self.nHPToAdd = self.nHPToAdd + self.nHealthRegenInBasePerSec * nElapsedTime
            local nHP = 0
            nHP, self.nHPToAdd = math.modf(self.nHPToAdd)
            if nHP > 0 then
                self:incrementHP(nHP)
            end
        end
    end

	function Ob:incrementMana(nPts)
		self.tSaved.tStats.nCurMana = math.min(self:getMaxMana(), self.tSaved.tStats.nCurMana + nPts)
	end

	function Ob:getPtsToAllocate()
		return self.tSaved.nPtsToAllocate
	end

	function Ob:setPtsToAllocate(nPts)
		self.tSaved.nPtsToAllocate = nPts
        if self.tSaved.nPtsToAllocate == 0 then
            if self.rLevelUpSprite then
                self.rLevelUpSprite:setVisible(true)
            end
        end
	end

    function Ob:improveMorale()
        if self.tSaved.tStats.eMorale == Character.eMorale.LOW then
            self.tSaved.tStats.eMorale = Character.eMorale.MEDLOW
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MEDLOW then
            self.tSaved.tStats.eMorale = Character.eMorale.MED
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MED then
            self.tSaved.tStats.eMorale = Character.eMorale.MEDHIGH
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MEDHIGH then
            self.tSaved.tStats.eMorale = Character.eMorale.HIGH
        end
    end

    function Ob:decreaseMorale()
        if self.bIsMoraleInvulnerable then
            return
        end
        if self.tSaved.tStats.eMorale == Character.eMorale.HIGH then
            self.tSaved.tStats.eMorale = Character.eMorale.MEDHIGH
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MEDHIGH then
            self.tSaved.tStats.eMorale = Character.eMorale.MED
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MED then
            self.tSaved.tStats.eMorale = Character.eMorale.MEDLOW            
        elseif self.tSaved.tStats.eMorale == Character.eMorale.MEDLOW then
            self.tSaved.tStats.eMorale = Character.eMorale.LOW
        end
    end

    function Ob:setMorale(eMorale)        
        self.tSaved.tStats.eMorale = eMorale
    end

    function Ob:getBlockChance()
        local eMorale = self:getMorale()
        local nPct = Character.tMiscTweaks.tBlockPct[eMorale]
        return nPct + self.tStatMods.nBlockChance
    end

    function Ob:getBlockEffectiveness()
        return Character.kBLOCK_PCT + self.tStatMods.nBlockEffectiveness
    end

    function Ob:getMorale()
        return self.tSaved.tStats.eMorale
    end

    function Ob:setMoraleInvulnerable(bSet)
        self.bIsMoraleInvulnerable = bSet
    end

    function Ob:setImproveTeammateMorale(bSet)
        self.bShouldImproveTeammateMorale = bSet
    end

    function Ob:shouldImproveTeammateMorale()
        return self.bShouldImproveTeammateMorale
    end

    function Ob:isAtMaxMorale()
        if self.tSaved.tStats.eMorale == Character.eMorale.HIGH then
            return true
        else
            return false
        end
    end

	function Ob:getAttackCriticalMultiplier(nChanceAdd)
        local eMorale = self:getMorale()
        local nChance = Character.tMiscTweaks.tCriticalAttackChance[eMorale]
		local nDiceRoll = math.random()
		if nDiceRoll < nChance + nChanceAdd then
			return Character.tMiscTweaks.nCriticalAttackMultiplier
		end
		return 1.0
	end

    function Ob:addHealthRegenInBase(nAmount)
        self.nHealthRegenInBasePerSec = self.nHealthRegenInBasePerSec + nAmount
        if self.nHealthRegenInBasePerSec < 0 then
            self.nHealthRegenInBasePerSec = 0
        end
    end

    function Ob:isAtMaxLevel()
        local nMaxLevel = table.getn(Character.tLevelImprovementInfo)
        if self.tSaved.tStats.nLevel >= nMaxLevel then
            return true
        end 
        return false
    end

	function Ob:incrementXP(nAmount)
        if self.tSaved.bPreppedForLevelUp then
            return -- must be promoted
        end

        if self:isAtMaxLevel() then
            return
        end

        self.tSaved.tStats.nXP = self.tSaved.tStats.nXP + nAmount
		local nXPForNextLevel = 1
		if Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel] and Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel].nXPForNextLevel then
			nXPForNextLevel = Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel].nXPForNextLevel
		end
		if self.tSaved.tStats.nXP >= nXPForNextLevel then
            self:prepForLevelUp()
			self.tSaved.tStats.nXP = self.tSaved.tStats.nXP - nXPForNextLevel
		end
	end

    function Ob:prepForLevelUp()
        if self.rLevelUpSprite then
            self.rLevelUpSprite:setVisible(true)
        end             
        self.tSaved.bPreppedForLevelUp = true
    end

    function Ob:isPreppedForLevelUp()
        return self.tSaved.bPreppedForLevelUp
    end

	function Ob:levelUp()
        self.tSaved.bPreppedForLevelUp = false
        if self.rLevelUpSprite then
            self.rLevelUpSprite:setVisible(false)
        end         
        local nMaxLevel = table.getn(Character.tLevelImprovementInfo)

		self.tSaved.bPromoted = true
		self.tSaved.tStats.nLevel = self.tSaved.tStats.nLevel + 1

        if self.tSaved.tStats.nLevel >= nMaxLevel then
            self.tSaved.tStats.nLevel = nMaxLevel
            DFAchievement.updateAchievement('MaxLevelOneHero', 1)
            if self:areAllAbilitiesAtMaxLevel() then
                DFAchievement.updateAchievement('MaxLevelAbilityAndHero', 1)
                DFAchievement.updateAchievement('MaxLevelAbilityAndEightHero', self:getCharDefKey())
            end
        end

--        self.rLevelUpOverlayText:revealAll()

        if Character.tLevelImprovementInfo then
            local rInfo = Character.tLevelImprovementInfo[self:getStatLevel()]
            if rInfo then
                self:incrementNumTrainingSessions(rInfo.nNumTrainingSessions)
            end
        end     
        g_TodoList:onHeroLevelUp(self)
        g_Game:saveGame()
	end

    function Ob:getCurXP()
        if self:isAtMaxLevel() then return 0 end
        return self.tSaved.tStats.nXP
    end

	function Ob:getXPPct()
		local nXPForNextLevel = 1
		return self:getCurXP() / self:getXPForNextLevel()
	end

    function Ob:getXPForNextLevel()
        local nXPForNextLevel = 1
		if Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel] and Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel].nXPForNextLevel then
			nXPForNextLevel = Character.tLevelImprovementInfo[self.tSaved.tStats.nLevel].nXPForNextLevel
		end
        return nXPForNextLevel
    end

    function Ob:areAllAbilitiesAtMaxLevel()
        local bAllAbilitiesMaxLevel = true
        for i, rAbility in pairs(self:getAbilities()) do
            if rAbility:showInUI() then
                if not rAbility:isAtMaxLevel() then
                    bAllAbilitiesMaxLevel = false
                    break
                end
            end
        end     
        return bAllAbilitiesMaxLevel
    end

    function Ob:addResearchActivityAtIndex(nIndex, sDefKey)
        self:setCurActivity('Research', sDefKey)
    end

    function Ob:addAbilityTrainingActivityAtIndex(nIndex, sDefKey)
        self:setCurActivity('AbilityTrain', sDefKey)
    end
	
	-- Stat improvement
	function Ob:improveStat(statName, maxValueName)
	
		local sDebug = Ob.sName .. ': ' .. statName .. ' improvement! '
	
		local tImprovementTable = tStatImprovementMatrix[statName]
		
		local s = 'n' .. statName		
		sDebug = sDebug .. tostring(Ob.tStats[s]) .. ' to '	
		local index = math.min(Ob.tStats.nLevel, #tImprovementTable)
		local nXPToAdd = tImprovementTable[index]
		if statName == 'XP' then
			self:incrementXP(nXPToAdd)
		else
			Ob.tStats[s] = Ob.tStats[s] + nXPToAdd
		end
		
		if maxValueName then
			local m = 'n' .. maxValueName
			if Ob.tStats[m] then
				Ob.tStats[s] = math.min(Ob.tStats[s], Ob.tStats[m])
			end
		end
		
		sDebug = sDebug .. tostring(Ob.tStats[s])
		
		print(sDebug)

		return nXPToAdd
	end
	
    function Ob:getActivityClass(activityType)
        return tActivityClasses[activityType]
    end

    function Ob:incrementTrainingSpeedMod(nAmount)
        self.nTrainingSpeedMod = self.nTrainingSpeedMod + nAmount
        if self.nTrainingSpeedMod < 0 then
            self.nTrainingSpeedMod = 0
        end
    end

    function Ob:getTrainingSpeedMod()
        return self.nTrainingSpeedMod
    end

    function Ob:getActivitySpeedMod(sActivityKey)
        if self.tActivitySpeedMods and sActivityKey and self.tActivitySpeedMods[sActivityKey] then
            return self.tActivitySpeedMods[sActivityKey]
        end
        return 0
    end

    function Ob:incrementActivitySpeedMod(sActivityKey, nAmount)
        if self.tActivitySpeedMods and sActivityKey and nAmount then
            if not self.tActivitySpeedMods[sActivityKey] then
                self.tActivitySpeedMods[sActivityKey] = 0
            end
            self.tActivitySpeedMods[sActivityKey] = self.tActivitySpeedMods[sActivityKey] + nAmount
        end
    end

    function Ob:incrementXPMuliplier(nAmount)
        self.nXPMultiplier = self.nXPMultiplier + nAmount
        if self.nXPMultiplier < 0 then
            self.nXPMultiplier = 0
        end
    end

    function Ob:getXPMuliplier()
        return self.nXPMultiplier
    end

    function Ob:incrementManaCostMultiplier(nAmount)
        self.nManaCostMult = self.nManaCostMult + nAmount
    end

    function Ob:getManaCostMultiplier()
        return self.nManaCostMult
    end

    function Ob:incrementSplashDamagePct(nAmount)
        self.nSplashDamagePct = self.nSplashDamagePct + nAmount
    end

    function Ob:getSplashDamagePct()
        return self.nSplashDamagePct
    end

    function Ob:setSuspendWander(bSuspend)
        self.bSuspendNothingWander = bSuspend
    end

	-- Activity Functions
	function Ob:setCurActivity(activityType, arg, tSaved)
        if not activityType then
            return nil
        end

		if self.rCurActivity then
            self.rCurActivity:End(true)
		end

		local rNewActivity = nil
        local activityClass = self:getActivityClass(activityType)
        if activityClass then
            local rDef = Character.getActivityDef(activityType)
            if rDef then
                if not tSaved then
                    tSaved = {}
                    tSaved.sActivityType = activityType
                    tSaved.tSaved = {}
                end
                self.tSaved.tCurActivitySaved = tSaved
                if activityType == "Nothing" and not arg then
                    arg = self.bSuspendNothingWander
                end
                rNewActivity = activityClass:new(Ob, rDef, nIndex, bRunUntilSuccess, tSaved.tSaved, arg)
            end
        end
		if rNewActivity == nil then
			print("ERROR: Attempting to create an activity of unknown type")
			return rNewActivity
		end

		self.rCurActivity = rNewActivity
		if self.bRunActivities then
            self.rCurActivity:Start()
		end      
		
		return rNewActivity
	end

    function Ob:runActivities(bRun)
        self.bRunActivities = bRun
        if bRun then
            if self.rCurActivity then
                self.rCurActivity:Start()
            else
                if self.tSaved.tCurActivitySaved and self.tSaved.tCurActivitySaved.sActivityType then
                    self:setCurActivity(self.tSaved.tCurActivitySaved.sActivityType, nil, self.tSaved.tCurActivitySaved)
                else
                    self:setCurActivity('Nothing')
                end
            end
        else
            if self.rCurActivity then
                self.rCurActivity:End(true)
            end
        end
    end

    function Ob:isIdle()
        local bIsIdle = true
        
        if self.rCurActivity and self.rCurActivity.rDef and self.rCurActivity.rDef.sActivityType ~= "Nothing" then
            bIsIdle = false
        end
        if self:isAssignedToEvent() then
            bIsIdle = false
        end
        
        return bIsIdle
    end
    
    function Ob:getCurActivity()
        return self.rCurActivity
    end

    function Ob:stopCurActivity()
        if self.rCurActivity then
            self.rCurActivity:End(true)
        end
        self.rCurActivity = nil
--        self.tSaved.tCurActivitySaved = nil
    end

    function Ob:getNumActForMoraleDec()
        local nNumActs = Character.tMiscTweaks.kNUM_ACT_FOR_MORALE_DEC
        local rRoom = g_CurrentMap:getRoom("recroom")
        if rRoom then
            nNumActs = rRoom:getRoomDef().nNumActsForMoraleDec or nNumActs
        end
        return nNumActs
    end

	function Ob:onActivityEnd(rActivity, bForceKilled)
		if rActivity:getActivityType() ~= 'Nothing' then
			print('Activity ended: ' .. rActivity:getActivityType())
		end

        if rActivity:getActivityType() ~= 'Nothing' and rActivity:getActivityType() ~= 'Rest' and rActivity:getActivityType() ~= 'Recreation' and rActivity:getActivityType() ~= 'MMMoraleActivity' then
            self.tSaved.nNumActivitiesWithNoRest = self.tSaved.nNumActivitiesWithNoRest + 1
            if self.tSaved.nNumActivitiesWithNoRest >= self:getNumActForMoraleDec() then
                self:decreaseMorale()
                self.tSaved.nNumActivitiesWithNoRest = 0
            end
        end

        self.rCurActivity = nil
		-- the topmost activity is always what ends
        if not bForceKilled then
            self.tSaved.tCurActivitySaved = nil
            self:setCurActivity('Nothing')
        end

		if rActivity:getActivityType() ~= 'Nothing' then
            g_Game:saveGame()
        end
	end

    function Ob:getActivityProgress(sActivityName)
        if self.tSaved.tActivityProgress[sActivityName] then
            return self.tSaved.tActivityProgress[sActivityName]
        end
        return 0
    end

    function Ob:setActivityProgress(sActivityName, nAmount)
        if not self.tSaved.tActivityProgress[sActivityName] then
            self.tSaved.tActivityProgress[sActivityName] = 0
        end
        self.tSaved.tActivityProgress[sActivityName]= nAmount
    end

    function Ob:setActivityProgressPaused(bPaused)
        self.bActivityProgressPaused = bPaused
    end

    function Ob:isActivityProgressPaused()
        return self.bActivityProgressPaused
    end

    -- returns true when progress is full
    function Ob:incrementActivityProgress(sActivityName, nAmount)
        if not self.tSaved.tActivityProgress[sActivityName] then
            self.tSaved.tActivityProgress[sActivityName] = 0
        end
        self.tSaved.tActivityProgress[sActivityName] = self.tSaved.tActivityProgress[sActivityName] + nAmount
        if self.tSaved.tActivityProgress[sActivityName] >= 1 then
            -- save out the remainder
            self.tSaved.tActivityProgress[sActivityName] = 1
            return true
        end
        return false
    end

	-- EVENT LISTENERS -- 
    function Ob:onFinger(eventType, sx, sy)
        if not g_Game:isActionEnabled('SelectHero') then
            return
        end
        if not self.bSelectable then
            return
        end
        if eventType == MOAITouchSensor.TOUCH_UP then            
            if not self.bSchedulerOpen then
                g_GUI:ShowHeroScheduler(self)
            end
            return true
        end
        return false
    end

    function Ob:onSchedulerOpened(bOpened)
        self.bSchedulerOpen = bOpened
    end

    function Ob:onSpriteFinger(eventType, sx, sy)
        if self.rCharEnt then
            return self.rCharEnt:onFinger(eventType, sx, sy)
        end
    end

	function Ob:playAttrChangeEffect(rEffectDef, sText)
	end

    function Ob:setSelectable(bSelectable)
        self.bSelectable = bSelectable
    end

    function Ob:isSelectable()
        return self.bSelectable
    end

    function Ob:setOverlayText(sText, nDuration, tColor)
        if sText then
            self.rCharOverlayShadowText:setString(sText)
            self.rCharOverlayText:setString(sText)
            self.rCharOverlayShadowText:setColor(0.1, 0.1, 0.1)
            self.rCharOverlayText:setColor(tColor[1], tColor[2], tColor[3])
            if nDuration > 0 then
                self.rCharOverlayShadowText:seekColor(0.1, 0.1, 0.1, 0, nDuration)
                self.rCharOverlayText:seekColor(tColor[1], tColor[2], tColor[3], 0, nDuration)
            end
        end
    end

    function Ob:setSpriteActive(bActive)
        if bActive then
            if self.tCharDef then
                self.tExtraSprites = {} -- contains the extra sprites
                self.tExtraSpriteSheets = {} -- contains the extra spritesheet to costume info
                self.rPortraitSpriteSheet = nil
                self.sCurPortraitPath = nil
                if not self.tSaved.sCostumeSet then
                    self.tSaved.sCostumeSet = self.tCharDef.sCostumeSet
                end
                self:setCostume(self.tSaved.sCostumeSet)
            end
            
            if self.sLastPlayedAnim then
                self:playAnimation(self.sLastPlayedAnim)
            else
                self:playAnimation(self.tCharDef.initalAnimation)
            end
        else
            self:unloadSpriteSheets(self.tSprites)
            if self.rShadow then
                self.tLayer.RenderLayer:removeProp(self.rShadow)
                self.rShadow = nil
            end            
            if self.rLevelUpSprite then
                self.tLayer.RenderLayer:removeProp(self.rLevelUpSprite)
            end
            -- might need to unload extra sprites	
        end
    end
    
    function Ob:setActive(bActive, bRestoreActivities)
        if bActive then
            self.bActive = true
            
            self:setSpriteActive(true)
            
            if bRestoreActivities then
                if not self.tSaved.tCurActivitySaved or not self.tSaved.tCurActivitySaved.sActivityType then
                    self:setCurActivity('Nothing')
                else
                    self:setCurActivity(self.tSaved.tCurActivitySaved.sActivityType, nil, self.tSaved.tCurActivitySaved)
                end
            else
                self:setCurActivity('Nothing')
            end

            if self.tCharDef.initTile then
                local x, y = g_CurrentMap:getWorldCoords(self.tCharDef.initTile.x, self.tCharDef.initTile.y)
                Ob:setPosition(x, y, true)        
            end

			-- show level up shadow if we're in the mood
			if Ob.tSaved.bPreppedForLevelUp then
                if Ob.rLevelUpSprite then
                    Ob.rLevelUpSprite:setVisible(true)
                end
			end
        else
            self.bActive = false
            self:stopCurActivity()

            self:setSpriteActive(false)
        end
    end
    
    function Ob:isActive()
        return self.bActive
    end

    function Ob:getPackLevel()
        return self.nPackLevel
    end

    function Ob:setWalkCancel(bWalkCancel)
        self.bWalkCancel = bWalkCancel
    end

	function Ob:WalkToTile(targRow, targCol, bTeleport, nXOffset, nYOffset)
		local rTile, sourceRow, sourceCol = self:getOccupyingTile()
        if not rTile then
            return
        end

		local dRow = targRow - sourceRow
		local dCol = targCol - sourceCol
		local dxx = dRow * dRow
		local dyy = dCol * dCol
		local distance = math.sqrt(dxx + dyy)
		
		if distance <= 0 then
			return
		end
		
		local aX, aY = g_CurrentMap:getWorldCoords(sourceRow, sourceCol)
		local bX, bY = g_CurrentMap:getWorldCoords(targRow, targCol)
        nXOffset = nXOffset or 0
        nYOffset = nYOffset or 0
        bX = bX + nXOffset
        bY = bY + nYOffset
        local dx = bX - aX
        local dy = bY - aY

		local breatheAnim = 'breathe'
		local walkAnim = 'walk'		
		
        if bTeleport then
            self:setPosition(bX, bY, true)
            self:playAnimation(breatheAnim)
            return 
        end

        local sFacing = "SW"
        if dx > 0 and dy > 0 then
            walkAnim = 'walk_reverse'
            breatheAnim = 'breathe_reverse'
            sFacing = "NE"
        elseif dx > 0 and dy <= 0 then
            walkAnim = 'walk'
            breatheAnim = 'breathe'
            sFacing = "SE"
        elseif dx < 0 and dy > 0 then
            walkAnim = 'walk_reverse'
            breatheAnim = 'breathe_reverse'
            sFacing = "NW"
        end

        self:setFacingDirection(sFacing)
		self:playAnimation(walkAnim)		
				
		local duration = distance / self.tCharDef.walkingSpeed
		
		local t = 0
		while t < duration do
		
			local alpha = t / duration
			local cx = (1 - alpha) * aX + alpha * bX
			local cy = (1 - alpha) * aY + alpha * bY
			
			self:setPosition(cx, cy, false)
		
			t = t + kTIMESTEP
			util.sleep(kTIMESTEP)

            if self.bWalkCancel then
                break
            end
		end
		
        if not self.bWalkCancel then
            self:setPosition(bX, bY, true)
            self:playAnimation(breatheAnim)
        end
        self.bWalkCancel = false
	end

    function Ob:playLevelUpEffect()
        local rEffectInstance = EffectInstance:new('Effects/LevelUp.lua', Renderer.getRenderLayer('GameplayOverlay'))
        g_Audio:playCue('MMOJ_Audio/UI/Promoted')
        if rEffectInstance then
            rEffectInstance:setParent(self)
            rEffectInstance:setLoc(0, 150)
            rEffectInstance:start()
        end
    end

    function Ob:setPlayEnterAnimForNothingActivity(bPlay)
        self.bPlayEnterAnimForNothingActivity = bPlay
    end

    function Ob:shouldPlayEnterAnimForNothingActivity()
        return self.bPlayEnterAnimForNothingActivity
    end

    function Ob:setInteractingProp(rProp, nUserIndex)
        self.rInteractingProp = rProp
        self.rInteractingPropUserIndex = nUserIndex
    end

	-- DEBUG
	function Ob:PrintActivityQueue()
		print("Activity Queue")
		for i, rActivity in ipairs(self.tActivities) do
			print(i..": "..rActivity:getActivityType())
		end
	end

    Ob.tSaved = tSaved

	if not Ob then
		return nil
	end

    Ob.tIdles = Character.tIdles
    Ob.tIdlesBk = Character.tIdlesBk

	-- init vars
	packLevel = packLevel or 1
	Ob.nPackLevel = packLevel

	-- stats
    if not Ob.tSaved.tStats then
        Ob.tSaved.tStats = {}
    end
    if not Ob.tSaved.tStats.nXP then
        Ob.tSaved.tStats.nXP = 0
    end
    if not Ob.tSaved.tStats.nLevel then
        Ob.tSaved.tStats.nLevel = Ob.tCharDef.tInitialStats[packLevel].nLevel
    end
    if not Ob.tSaved.tStats.nPower then
        Ob.tSaved.tStats.nPower = Ob.tCharDef.tInitialStats[packLevel].nPower
    end
    if not Ob.tSaved.nNumTrainingSessions then
        if Character.tLevelImprovementInfo then
            local rInfo = Character.tLevelImprovementInfo[Ob:getStatLevel()]
            if rInfo then
                Ob.tSaved.nNumTrainingSessions = rInfo.nNumTrainingSessions
            end
        else
            Ob.tSaved.nNumTrainingSessions = 0
        end
    end
    if not Ob.tSaved.tStats.nAwareness then
        Ob.tSaved.tStats.nAwareness = Ob.tCharDef.tInitialStats[packLevel].nAwareness
    end
    if not Ob.tSaved.tStats.nMaxHP then
        Ob.tSaved.tStats.nMaxHP = Ob.tCharDef.tInitialStats[packLevel].nMaxHP
    end
    if not Ob.tSaved.tStats.nCurHP then
        Ob.tSaved.tStats.nCurHP = Ob.tCharDef.tInitialStats[packLevel].nMaxHP
    end
    if not Ob.tSaved.tStats.nCharisma then
        Ob.tSaved.tStats.nCharisma = Ob.tCharDef.tInitialStats[packLevel].nCharisma
    end
    if not Ob.tSaved.tStats.eMorale then
        Ob.tSaved.tStats.eMorale = Character.eMorale.HIGH
    end
    if not Ob.tSaved.tStats.tFriendship then
        Ob.tSaved.tStats.tFriendship = {} -- I have no friends ... yet
    end
    if not Ob.tSaved.nPtsToAllocate then
        Ob.tSaved.nPtsToAllocate = 0
    end
    if not Ob.tSaved.tActivityProgress then
        Ob.tSaved.tActivityProgress = {}
    end
    if not Ob.tSaved.nNumActivitiesWithNoRest then
        Ob.tSaved.nNumActivitiesWithNoRest = 0
    end
    if not Ob.tSaved.tUnlockedCostumes then
        Ob.tSaved.tUnlockedCostumes = {}
    end
    if not Ob.tSaved.sCostumeSet then
        Ob.tSaved.sCostumeSet = Ob.tCharDef.sCostumeSet
    end

    Ob.sCostumeSet = Ob.tCharDef.sCostumeSet

    Ob.tStatMods = {}
    Ob.tStatMods.nPower = 0
    Ob.tStatMods.nAwareness = 0
    Ob.tStatMods.nBlockChance = 0
    Ob.tStatMods.nBlockEffectiveness = 0
    Ob.tStatMods.nMaxHP = 0
    Ob.tStatMods.nMaxMana = 0
    Ob.tStatMods.nCharisma = 0
    Ob.tStatMods.nShield = 0
    Ob.tStatMods.nSpecialAbilityMult = 1

    Ob.tActivitySpeedMods = {}

    if not Ob.tSaved.tStats.nCurMana then
        Ob.tSaved.tStats.nCurMana = Ob:getMaxMana()
    end
	-- activity queue
    Ob.bRunActivities = false

    -- whether the character can be selected on the UI
    Ob.bSelectable = true

    Ob.rRenderLayer = nil

    -- initialize the abilities
    Ob.tAbilities = {}
    Ob.rDefaultAbility = nil
    if Ob.tCharDef.tAbilityDefKeys then
        local abilityKeys = util.deepCopy(Ob.tCharDef.tAbilityDefKeys)
        table.insert(abilityKeys, "cyclestance")
        for i, sAbilityDefKey in ipairs(abilityKeys) do
            local rAbilityDef = Character.getAbilityDef(sAbilityDefKey)
            if rAbilityDef then
                if rAbilityDef.sAbilityType then
                    if not Ob.tSaved.tAbilities then
                        Ob.tSaved.tAbilities = {}
                    end
                    if not Ob.tSaved.tAbilities[rAbilityDef.sAbilityName] then
                        Ob.tSaved.tAbilities[rAbilityDef.sAbilityName] = {}
                    end
                    local constructor = Ability.getConstructor(rAbilityDef.sAbilityType)
                    if constructor then
                        local rAbility = constructor(nil, rAbilityDef, nil, Ob.tSaved.tAbilities[rAbilityDef.sAbilityName], Ob, sAbilityDefKey)
                        if rAbility:isDefault() then
                            Ob.rDefaultAbility = rAbility
                        end
                        Ob.tAbilities[sAbilityDefKey] = rAbility
                    end
                end
            end
        end
    end

    Ob.tAbilitiesUsedAtEvent = {}

    Ob.tInventory = {}
    Ob.tDefaultAttackMods = {}
    Ob.tFightBuffs = {}
    Ob.nTrainingSpeedMod = 0
    Ob.nXPMultiplier = 0
    Ob.bIsMoraleInvulnerable = false
    Ob.nHealthRegenInBasePerSec = 0
    Ob.nHPToAdd = 0
    Ob.nManaCostMult = 0
    Ob.nSplashDamagePct = 0

    local nMaxSlots = Character.getMaxSlots()
    for i = 1, nMaxSlots do 
        Ob.tInventory[i] = Item.kEMPTY_SLOT
    end

    if not Ob.tSaved.tItemDefKeys then
        Ob.tSaved.tItemDefKeys = {}
    else
        local nNumItemsAdded = 0
        for i, sItemDefKey in ipairs(Ob.tSaved.tItemDefKeys) do
            g_MiddleManager:addEquipmentToInventory(sItemDefKey)  
            if nNumItemsAdded < nMaxSlots then
                g_MiddleManager:giveEquipmentToHero(sItemDefKey, Ob)
                nNumItemsAdded = nNumItemsAdded + 1
            end
        end
    end

    if not Ob.tSaved.tEquippedMeteoriteInfo then
        Ob.tSaved.tEquippedMeteoriteInfo = {}
    else
        if Ob.tSaved.tEquippedMeteoriteInfo.sMeteoriteDefKey and Ob.tSaved.tEquippedMeteoriteInfo.nLevel then
            Ob:addMeteoriteToInventory(Ob.tSaved.tEquippedMeteoriteInfo.sMeteoriteDefKey, Ob.tSaved.tEquippedMeteoriteInfo.nLevel)
        end
    end

    Ob.rAssignedEvent = nil

    Ob:setFacingDirection("SW")	

    Ob.rCharOverlayShadowText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rCharOverlayShadowText:setRect(-300, 80, 300, 0)
	Ob.rCharOverlayShadowText:setLoc(2, 168)
    Ob.rCharOverlayShadowText:setColor(1, 0, 0)
	Ob.rCharOverlayShadowText:setParent(Ob)

    Ob.rCharOverlayText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rCharOverlayText:setRect(-300, 80, 300, 0)
	Ob.rCharOverlayText:setLoc(0, 170)
    Ob.rCharOverlayText:setColor(1, 0, 0)
	Ob.rCharOverlayText:setParent(Ob)

    local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
    if rOverlayLayer then
        rOverlayLayer:insertProp(Ob.rCharOverlayShadowText)
        rOverlayLayer:insertProp(Ob.rCharOverlayText)       
    end
      
    local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')

    local sSuffix = Character.tUIScreenInfo[Renderer.getCurScreenInfo().key].sTextureSizeSuffix
    if sSuffix then
        local rDeck = DFGraphics.loadSpriteSheet('UI/hud2'..sSuffix, g_IsInStartupSequence)
        if rDeck then
            Ob.rLevelUpSprite = MOAIProp2D.new()
            Ob.rLevelUpSprite:setDeck(rDeck)
            local nIndex = rDeck.names['promotion']
            Ob.rLevelUpSprite:setIndex(nIndex)
            Ob.rLevelUpSprite:setParent(Ob)
            Ob.rLevelUpSprite:setVisible(false)
            Ob.rLevelUpSprite:setLoc(-25, 200)
        end
    end   

	-- add to layers
    Ob.tLayer = Renderer.getLayer('Gameplay')	

    Ob.bActive = false

    Ob:loadPortraitSprites('lvl1')--default

    if Ob.tSaved.bPreppedForLevelUp then
        if Ob.rLevelUpSprite then
            Ob.rLevelUpSprite:setVisible(true)
        end             
    end

    Ob.bPlayEnterAnimForNothingActivity = false
    Ob.rInteractingProp = nil
    Ob.bActivityProgressPaused = false

	return Ob
end

return Character
