-- "m" for "module"
local m = {}

HACK_CHAR_SHRINK = 0.8

TT_Error = -2
TT_Warning = -1 
TT_Anna = 0
TT_Paul = 1

local ENABLED = {
    [TT_Error]=true,
    [TT_Warning]=true,
    [TT_Anna]=true,
    [TT_Paul]=true,
}

function Trace(type, ...)
    if ENABLED[type] or type==nil then
        if type == TT_Error then
            print(debug.traceback())
        end
	print(string.format(...))
    end
end

function m.deepCopy(t)
        if type(t) ~= 'table' then return t end
        local t2 = {}
        for k,v in pairs(t) do t2[k] = m.deepCopy(v) end
        return t2
end

function m.dofile(fileName)
	-- print("Deprecated: DFFile.dofile", debug.traceback())
    return dofile(fileName)
end

local file = require "DFCommon.File"
m.getAssetName = file.getAssetPath

function m.distance( x1, y1, x2, y2 )
    if x2 then
        return math.sqrt( ( ( x2 - x1 ) ^ 2 ) + ( ( y2 - y1 ) ^ 2 ) )
    else
        return math.sqrt( ( x1 ^ 2 ) + ( y1 ^ 2 ) )
    end
end

function m.distanceSquared( x1, y1, x2, y2 )
    if x2 then
        return ( ( x2 - x1 ) ^ 2 ) + ( ( y2 - y1 ) ^ 2 )
    else
        return ( x1 ^ 2 ) + ( y1 ^ 2 )
    end
end

function m.playSound( path, bLoop )
    if MOAIUntzSystem then        
        local sound = MOAIUntzSound.new ()
        sound:load( path )
        sound:setVolume( 1 )
        sound:setLooping( bLoop )
        sound:play()
    end
end

local floor = math.floor
local function divmod(x, divisor)
	local div = floor(x/divisor)
	local mod = x - div*divisor
	return div,mod
end

-- format seconds as something like "32m,16s"
function m.format_seconds(sec)
    if sec < 60 then
        return "<1M"
    else
		local min, sec = divmod(sec, 60)
		local hour, min = divmod(min, 60)
        --local sec = sec % 60        
        if hour < 1 then
            return string.format("%dM", min)
        elseif min < 1 and hour > 1 then
            return string.format("%dH", hour )
        else
            return string.format("%dH %dM", hour, min)
        end
    end
end

function m.countitems( func, state, var )    
    if not func then return 0 end
    
    -- init
    state = func
    func = next
    count = 0
    
    do
        while true do
            local key, value = func(state, var)
            var = key
            if not var then break end
            count = count + 1
        end
    end
    return count
end

-- loads the sprites, and also does a bunch of excitingly hacky stuff based on path. Flips, scales, dupes, etc.
function m.loadSpriteLibsMoai(lua)
    local data = dofile ( lua )
    local frames = data.frames

    local tex = MOAITexture.new ()
    tex:load ( data.texture )
    local xtex, ytex = tex:getSize ()

    -- Annotate the frame array with uv quads and geometry rects
    for i, frame in ipairs ( frames ) do
        -- convert frame.uvRect to frame.uvQuad to handle rotation
        local uv = frame.uvRect
        local q = {}
        if not frame.textureRotated then
            -- From Moai docs: "Vertex order is clockwise from upper left (xMin, yMax)"
            q.x0, q.y0 = uv.u0, uv.v0
            q.x1, q.y1 = uv.u1, uv.v0
            q.x2, q.y2 = uv.u1, uv.v1
            q.x3, q.y3 = uv.u0, uv.v1
        else
            -- Sprite data is rotated 90 degrees CW on the texture
            -- u0v0 is still the upper-left
            q.x3, q.y3 = uv.u0, uv.v0
            q.x0, q.y0 = uv.u1, uv.v0
            q.x1, q.y1 = uv.u1, uv.v1
            q.x2, q.y2 = uv.u0, uv.v1
        end
        frame.uvQuad = q

        -- convert frame.spriteColorRect and frame.spriteSourceSize
        -- to frame.geomRect.  Origin is at x0,y0 of original sprite
        local cr = frame.spriteColorRect
        local r = {}
        r.x0 = cr.x
        r.y0 = cr.y
        r.x1 = cr.x + cr.width
        r.y1 = cr.y + cr.height
        frame.geomRect = r

        -- fix up names.
        frame.name = string.gsub(string.lower(frame.name), "\\", "/")
        frame.name = string.gsub(frame.name, ".png$", "")
        frame.name = string.gsub(frame.name, ".gif$", "")

        -- TODO INPROGRESS HACK
        if not frame.scale then frame.scale = 2 end
    end


    -- Add in game-specific stuff: scale, move the root position, create new frames, etc
    local function offset(r, dx,dy)
        r.x0, r.x1 = r.x0 + dx, r.x1 + dx
        r.y0, r.y1 = r.y0 + dy, r.y1 + dy
    end
    local function scale(r, s) r.x0, r.x1, r.y0, r.y1 = r.x0*s, r.x1*s, r.y0*s, r.y1*s end
    local function flipU(r) r.x0, r.x1 = r.x1, r.x0 end
    local function flipV(r) r.y0, r.y1 = r.y1, r.y0 end
	-- TODO: wrong?
    local function flipUQ(r, rot) 
        if not rot then
			r.x0, r.x1, r.x2, r.x3 = r.x1, r.x0, r.x3, r.x2 
		else
			r.y0, r.y1, r.y2, r.y3 = r.y1, r.y0, r.y3, r.y2 
		end
	end
    local function flipVQ(r, rot) 
        if not rot then
			r.y0, r.y1, r.y2, r.y3 = r.y1, r.y0, r.y3, r.y2 
		else
			r.x0, r.x1, r.x2, r.x3 = r.x1, r.x0, r.x3, r.x2 
		end
	end

    local more = {}

    for i,frame in ipairs(frames) do
        local name = frame.name
        local srcWidth = frame.geomRect.x1-frame.geomRect.x0
        local srcHeight = frame.geomRect.y1-frame.geomRect.y0

        if string.match(name, "^chars/") then
            
            -- flip v axis, origin is bottom middle
            -- Why swap V? See http://getmoai.com/2011/04/moai-basics-part-1/#more-589, "UV Coordinates"
            --flipV(frame.uvRect)
            offset(frame.geomRect, -srcWidth/2, 0)
            scale(frame.geomRect, HACK_CHAR_SHRINK*(frame.scale or 1))

			--if string.match(name, "_shadow") then
				--print("offset",name)
            	--offset(frame.geomRect, 0, -srcHeight/2)
			--else
            	local new = m.deepCopy(frame)
            	new.name = new.name .. '_xflip'
            	flipUQ(new.uvQuad, frame.textureRotated)
            	table.insert(more, new)
			--end

        elseif string.match(name, "^ui/") then
			flipV(frame.geomRect)
            scale(frame.geomRect, frame.scale or 1)
			-- MTF: uh... why doesn't this work?
            --flipVQ(frame.uvRect, frame.textureRotated)
        end
    end
    for i,frame in ipairs(more) do
        table.insert(frames, frame)
    end
    more = nil

    -- Create deck
    local deck = MOAIGfxQuadDeck2D.new()
    deck.name_to_rect = {}
    deck.by_name = {}
    deck:setTexture(tex)
    deck:reserve(#frames)
    for i,frame in ipairs(frames) do
        if frame.name == nil then
            Trace(TT_Paul, "Don't know name for %s", i)
        end

        local q = frame.uvQuad
		local r = frame.geomRect
        deck:setUVQuad ( i, q.x0,q.y0, q.x1,q.y1, q.x2,q.y2, q.x3,q.y3 )
		deck:setRect(i, r.x0, r.y0, r.x1, r.y1)
        deck.name_to_rect[frame.name] = { r.x0, r.y0, r.x1, r.y1 }
        deck.by_name[frame.name] = i
    end

    function deck:getIndex(name)
        local i = self.by_name[name]
        if i == nil then
            Trace(nil, "WARNING: cannot find sprite for %s", tostring(name))
        end
        return i
    end
    return deck
end

function m.newSprite(spriteName, layerRef, deckRef, spriteRef, xoff, yoff)
    if not deckRef then deckRef = g_deck end
    if not layerRef then layerRef = g_char_layer end

    local idx = deckRef:getIndex(spriteName)
    local sprite = spriteRef or MOAIProp2D.new()    
    sprite:setDeck(deckRef)
    sprite:setIndex(idx)
	if xoff and yoff then sprite:setLoc(xoff,yoff) end
    layerRef:insertProp(sprite)

    return sprite
end

function m.sleep(time)
	local rTimer = MOAITimer.new()
	rTimer:setSpan(0, time)
    MOAIThread.blockOnAction(rTimer:start())	
end

-- returns 0-360
function m.angleBetweenDeg(x1, y1, x2, y2)
    -- TODO: correct the angle
    local angle = math.atan(y2,x2) - math.atan(y1,x1)
    return angle
end

function m.convertTimeString(nTime, bForceColon, bNoLeadingZero)
    local nRemainingTime = nTime
    local nRemainingMinutes = math.floor(nRemainingTime / 60)
    local nRemainingSeconds = math.floor(nRemainingTime % 60)
    local sRemainingMinText = tostring(nRemainingMinutes)
    local sRemainingSecText = tostring(nRemainingSeconds)
    local sTimeText = ""
    if nRemainingMinutes < 10 and not bNoLeadingZero then
        sRemainingMinText = "0"..sRemainingMinText
    end
    if nRemainingSeconds < 10 then
        sRemainingSecText = "0"..sRemainingSecText
    end
    if bForceColon or nRemainingSeconds % 2 == 0 then
        sTimeText = sRemainingMinText..":"..sRemainingSecText
    else
        sTimeText = sRemainingMinText.." "..sRemainingSecText
    end
    return sTimeText             
end

return m
