local EffectInstance = {}

--local Audio = require('Audio')
local Renderer = require('Renderer')
local Sprite = require('SpriteSheet')
local TextEffectInstance = require('TextEffectInstance')
local DFFile = require('DFCommon.File')
local util = require("DFUtil")

kEFFECTINSTANCE_SLEEP_TIME = 0.025

function EffectInstance:new(effectDefFile, renderLayer, priority, nScale, sTextEffectValue)
	-- CONSTRUCTOR
	local Ob = MOAITransform.new()
    Ob.__typename = "EffectInstance"
	if effectDefFile then
        Ob.rEffectDef = util.dofile(DFFile.getDataPath(effectDefFile))
	end
	
	if not Ob.rEffectDef then
		print("Unable to load effect definition: " .. effectDefFile)
		return nil
	end
	
	Ob.numEventsTriggered = 0
	Ob.activeEvents = { }
	Ob.renderLayer = renderLayer
	Ob.priority = priority
	Ob.isRunning = false
    Ob.nScale = nScale or 1
	
	function Ob:Execute()
		
		local t = 0
		local lastT = 0
		local isDone = false
		
		local numEvents = #self.rEffectDef.events
		
		while not isDone do
		
			-- Create new events
			for i=1,numEvents do
				local event = self.rEffectDef.events[i]
				if event.startTime >= lastT and (event.startTime < t or event.startTime == 0) then
					self.numEventsTriggered = self.numEventsTriggered + 1
					local eventInstance = { }
					eventInstance.event = event
					
					if event.eventType == "spriteAnimation" then
                        if self.renderLayer then
                            eventInstance.rSprite = Sprite:new(event.spriteSheet, event.sAnimDefFileName, 0, 0, 'center', 'center')
                            eventInstance.rSprite:setScl(self.nScale, self.nScale)
                            -- Add to renderer
                            self.renderLayer:insertProp(eventInstance.rSprite)
                            -- Setup the prop
                            eventInstance.rSprite:setLoc(event.offsetX, event.offsetY)
                            eventInstance.rSprite:setParent(self)
                            if self.priority then eventInstance.rSprite:setPriority(self.priority) end
                            eventInstance.rSprite:playAnimation(event.animation)
                        end
					elseif event.eventType == "textEffect" and sTextEffectValue then
                        if self.renderLayer then
                            eventInstance.rEffect = TextEffectInstance:new(event.textEffect, sTextEffectValue, self.renderLayer, event.offsetX, event.offsetY, self.priority)
                            eventInstance.rEffect:setParent(self)
                            eventInstance.rEffect:start()
                        end
					
					elseif event.eventType == "sound" then
--						Audio.playSound(event.sound, event.volume)
					end
					
					table.insert(self.activeEvents, eventInstance)
				end
			end
			
			-- Check for active events
			isDone = self.numEventsTriggered >= numEvents
			
			local numActiveEvents = #self.activeEvents
			for i=1,numActiveEvents do
				local index = numActiveEvents - i + 1
				local eventInstance = self.activeEvents[index]

				if eventInstance.event.eventType == "spriteAnimation" then
					if eventInstance.rSprite:isDone() then
						if self.renderLayer then
                            -- Remove from renderer
                            self.renderLayer:removeProp(eventInstance.rSprite)
                            eventInstance.rSprite:unloadSpriteSheet()				
                            -- Remove from list of active instances
                            table.remove(self.activeEvents, index)
                        end
					else
						isDone = false
					end
					
				elseif eventInstance.event.eventType == "textEffect" then
				
					if not eventInstance.rEffect or eventInstance.rEffect:isDone() then
					
						-- Remove from list of active instances
						table.remove(self.activeEvents, index)
					else
						isDone = false
					end
					
				elseif eventInstance.event.eventType == "sound" then
				
					-- ToDo: Check for completion
				
					-- Remove from list of active instances
					table.remove(self.activeEvents, index)
				end

			end
			
			-- Yield for other threads
			util.sleep(kEFFECTINSTANCE_SLEEP_TIME)
			
			-- Update timer
			lastT = t
			t = t + kEFFECTINSTANCE_SLEEP_TIME
		end
	
		self.rThread = nil
		self.isRunning = false
	end
	
	function Ob:start()
		self.rThread = MOAIThread.new()
		self.rThread:run(self.Execute, self)
		self.isRunning = true
	end
	
	function Ob:stop()
        local numActiveEvents = #self.activeEvents
        for i=1,numActiveEvents do
            local index = numActiveEvents - i + 1
            local eventInstance = self.activeEvents[index]
            
            if eventInstance.event.eventType == "spriteAnimation" then
                eventInstance.rSprite:stopCurrentAnimation()
            end
        end
--[[                    
		if self.rThread then
			self.rThread:stop()
		end
]]--
	end
	
	function Ob:isDone()
		return not self.isRunning
	end
	
	return Ob
end

return EffectInstance