local DFFile = require('DFCommon.File')
local DFUtil = require('DFCommon.Util')

local Item = {}
local EquippableItem = {}
local ConsumableItem = {}
local HealConsumableItem = {}
local ManaConsumableItem = {}
local BuffConsumableItem = {}
local RessurrectConsumableItem = {}
local MoraleEquippableItem = {}
local DefaultAttackModEquippableItem = {}
local TrainingModEquippableItem = {}
local FightBuffEquippableItem = {}
local XPMultEquippableItem = {}
local PermanentPropItem = {}
local HealthRegenInBaseItem = {}
local Meteorite = {}

Item.kEMPTY_SLOT = 1
Item.kMETEORITE_MAX_LEVEL = 6
Item.tMeteoriteNames =
{
    "MNAM001TEXT",
    "MNAM002TEXT",
    "MNAM003TEXT",
    "MNAM004TEXT",
    "MNAM005TEXT",
    "MNAM006TEXT",
}

local tConsumableDefInfo = dofile(DFFile.getDataPath("Items/ConsumableDefs.lua"))
Item.tConsumableDefs = tConsumableDefInfo.tDefs
Item.tConsumableList = tConsumableDefInfo.tList

local tEquipmentDefInfo = dofile(DFFile.getDataPath("Items/EquipmentDefs.lua"))
Item.tEquipmentDefs = tEquipmentDefInfo.tDefs
Item.tEquipmentList = tEquipmentDefInfo.tList

local tMeteoriteDefInfo = dofile(DFFile.getDataPath("Items/MeteoriteDefs.lua"))
Item.tMeteoriteDefs = tMeteoriteDefInfo.tDefs
Item.tMeteoriteList = tMeteoriteDefInfo.tList
Item.tMeteoriteSpawnInfo = tMeteoriteDefInfo.tSpawnInfo

function Item.getConsumableDef(sDefKey)
    if sDefKey then
        return Item.tConsumableDefs[sDefKey]
    end
end
function Item.updateConsumableDef(sDefKey, tValues)
    g_Game:updateDefWithTweak(Item.tConsumableDefs, sDefKey, tValues)
end

function Item.getEquipmentDef(sDefKey)
    if sDefKey then
        return Item.tEquipmentDefs[sDefKey]
    end
end

function Item.updateEquipmentDef(sDefKey, tValues)
    g_Game:updateDefWithTweak(Item.tEquipmentDefs, sDefKey, tValues)
end

function Item.getMeteoriteDef(sDefKey)
    if sDefKey then
        return Item.tMeteoriteDefs[sDefKey]
    end
end
function Item.updateMeteoriteDef(sDefKey, tValues)
    g_Game:updateDefWithTweak(Item.tMeteoriteDefs, sDefKey, tValues)
end

function Item.updateMeteoriteSpawnInfo(sDefKey, tValues)
    g_Game:updateDefWithTweak(Item.tMeteoriteSpawnInfo, sDefKey, tValues)    
end

function Item.getAllEquipmentDefKeys()
    return Item.tEquipmentList
end

function Item.getAllConsumableDefKeys()
    return Item.tConsumableList
end

function Item.getAllMeteoriteDefKeys()
    return Item.tMeteoriteList
end

function Item.getMeteoriteName(sItemDefKey, nLevel)
    local sName = ""
    if sItemDefKey and nLevel then
        local rDef = Item.getMeteoriteDef(sItemDefKey)
        if rDef then
            if rDef.sName then
                sName = sName..g_DialogManager:getString(rDef.sName)
            end
            if Item.tMeteoriteNames[nLevel] then
                sName = sName.." "..g_DialogManager:getString(Item.tMeteoriteNames[nLevel])
            end
        end
    end
    return sName
end

function Item.getMeteoriteCombo(sMeteoriteDefKeyA, nLevelA, sMeteoriteDefKeyB, nLevelB, sMeteoriteDefKeyC, nLevelC)
    if sMeteoriteDefKeyA and sMeteoriteDefKeyB and sMeteoriteDefKeyC and nLevelA and nLevelA <= Item.kMETEORITE_MAX_LEVEL and nLevelB and nLevelB <= Item.kMETEORITE_MAX_LEVEL and nLevelC and nLevelC <= Item.kMETEORITE_MAX_LEVEL then
        -- check same combo
        if nLevelA < Item.kMETEORITE_MAX_LEVEL then
            if sMeteoriteDefKeyA == sMeteoriteDefKeyB and sMeteoriteDefKeyB == sMeteoriteDefKeyC and nLevelA == nLevelB and nLevelB == nLevelC then
                return sMeteoriteDefKeyA, (nLevelA + 1)
            end
        end        
        -- check recipes
        if Item.tMeteoriteDefs then
            for sDefKey, rDef in pairs(Item.tMeteoriteDefs) do
                if rDef.tRecipe then
                    -- since this is always passed in in the same order...
                    if rDef.tRecipe[1] == sMeteoriteDefKeyA and rDef.tRecipe[2] == sMeteoriteDefKeyB and rDef.tRecipe[3] == sMeteoriteDefKeyC then
                        return sDefKey, nLevelA
                    end
                end
            end
        end
    end
    return nil
end

function Item.getRandomMeteoriteSpawn()
    if Item.tMeteoriteSpawnInfo then
        -- can only spawn meteorites starting from the second area
        local nRandNum = math.random(1, 100)
        for i, rSpawnInfo in ipairs(Item.tMeteoriteSpawnInfo) do
            if rSpawnInfo.nPossibility and rSpawnInfo.nMinMMLevel and rSpawnInfo.tMeteoriteInfos then
                if g_MiddleManager:getLevel() >= rSpawnInfo.nMinMMLevel then
                    if nRandNum <= rSpawnInfo.nPossibility then
                        -- get a random meteorite
                        local nNumItems = table.getn(rSpawnInfo.tMeteoriteInfos)
                        local rMeteoriteInfo = rSpawnInfo.tMeteoriteInfos[math.random(nNumItems)]
                        if rMeteoriteInfo and rMeteoriteInfo.sMeteoriteKey and rMeteoriteInfo.tPossibleLevels then
                            local nNumLevels = table.getn(rMeteoriteInfo.tPossibleLevels)
                            return rMeteoriteInfo.sMeteoriteKey, rMeteoriteInfo.tPossibleLevels[math.random(nNumLevels)]
                        end
                    end
                end
            end
        end
    end
    return nil
end

function Item.getItemThatGrantsConsumable(sConsumableDefKey)
    local sItemDefKey = nil
    
    for sDefKey, rDef in pairs(Item.tConsumableDefs) do
        if rDef.sGrantItemType and rDef.sGrantItemType == sConsumableDefKey then
            sItemDefKey = sDefKey
            break
        end
    end
    
    return sItemDefKey
end

function Item.getAllItemsThatGrantConsumables()
    local tItemDefs = {}
    
    for sDefKey, rDef in pairs(Item.tConsumableDefs) do
        if rDef.sGrantItemType then
            tItemDefs[sDefKey] = rDef
        end
    end
    
    return tItemDefs
end

function Item:new(rDef, sItemDefKey)
	local Ob = {}
    Ob.rDef = rDef
    Ob.rOwner = nil
    Ob.sItemDefKey = sItemDefKey

    function Ob:canUse(rEvent)
        return true
    end

    function Ob:getItemDef()
        return self.rDef
    end

    function Ob:getItemDefKey()
        return self.sItemDefKey
    end

    function Ob:getName()
        return g_DialogManager:getString(self.sName)
    end

    function Ob:getSellPrice()
        return math.floor(self.rDef.nCost / 2)
    end

    function Ob:getDescription()
        return self.sDescription
    end

    function Ob:setOwner(rOwner)
        self:onOwnerChanged(self.rOwner, rOwner)
        if self.rOwner and self.rOwner.onItemRemoved then
            self.rOwner:onItemRemoved(self)
        end        
        self.rOwner = rOwner
        if self.rOwner and self.rOwner.onItemAdded then
            self.rOwner:onItemAdded(self)
        end    
    end

    function Ob:getOwner()
        return self.rOwner
    end

	return Ob
end

function Meteorite:new(rDef, sItemDefKey, nLevel)
	local Ob = {}
    Ob.rDef = rDef
    Ob.nLevel = nLevel
    Ob.rOwner = nil
    Ob.sItemDefKey = sItemDefKey

    if rDef.tInfo and rDef.tInfo[nLevel] then
        Ob.rLevelInfo = rDef.tInfo[nLevel]
    end

    function Ob:getItemDef()
        return self.rDef
    end

    function Ob:getItemDefKey()
        return self.sItemDefKey
    end

    function Ob:getLevel()
        return self.nLevel
    end

    function Ob:getName()
        return g_DialogManager:getString(self.rDef.sName)
    end

    function Ob:getSellPrice()
        return self.rLevelInfo.nSellCost or 1
    end

    function Ob:getDescription()
        return self.rDef.sDescription
    end

    function Ob:setOwner(rOwner)
        if self.rOwner then
            if self.rLevelInfo.tFightBuffs then
                self.rOwner:removeFightBuff('Meteorite')
            end
            if self.rLevelInfo.tAttackMods then
                self.rOwner:removeDefaultAttackMod('Meteorite')
            end
            if self.rLevelInfo.nXPMultiplier then
                if self.rOwner.incrementXPMuliplier then
                    self.rOwner:incrementXPMuliplier(-self.rLevelInfo.nXPMultiplier)
                end
            end
            if self.rLevelInfo.nManaCostMultiplier then
                if self.rOwner.incrementManaCostMultiplier then
                    self.rOwner:incrementManaCostMultiplier(-self.rLevelInfo.nManaCostMultiplier)
                end
            end
            if self.rLevelInfo.nSplashDamageMultiplier then
                if self.rOwner.incrementSplashDamagePct then
                    self.rOwner:incrementSplashDamagePct(-self.rLevelInfo.nSplashDamageMultiplier)
                end
            end
        end        
        self.rOwner = rOwner
        if self.rOwner then
            if self.rLevelInfo.tFightBuffs then
                self.rOwner:addFightBuff('Meteorite', self.rLevelInfo.tFightBuffs.tBuffs)
            end
            if self.rLevelInfo.tAttackMods then
                self.rOwner:addDefaultAttackMod('Meteorite', self.rLevelInfo.tAttackMods.nChance, self.rLevelInfo.tAttackMods.tBuffs)
            end
            if self.rLevelInfo.nXPMultiplier then
                if self.rOwner.incrementXPMuliplier then
                    self.rOwner:incrementXPMuliplier(self.rLevelInfo.nXPMultiplier)
                end
            end
            if self.rLevelInfo.nManaCostMultiplier then
                if self.rOwner.incrementManaCostMultiplier then
                    self.rOwner:incrementManaCostMultiplier(self.rLevelInfo.nManaCostMultiplier)
                end
            end
            if self.rLevelInfo.nSplashDamageMultiplier then
                if self.rOwner.incrementSplashDamagePct then
                    self.rOwner:incrementSplashDamagePct(self.rLevelInfo.nSplashDamageMultiplier)
                end
            end
        end    
    end

    function Ob:getOwner()
        return self.rOwner
    end

	return Ob
end
   
function EquippableItem:new(rDef, sItemDefKey)
    local Ob = Item:new(rDef, sItemDefKey)
    Ob.rHero = nil

    function Ob:onOwnerChanged(rOldOwner, rNewOwner)
        if rOldOwner then
            if self.onEquipped then self:onEquipped(rOldOwner, false) end
            self:adjustStats(rOldOwner, false)
        end
        if rNewOwner then
            self:adjustStats(rNewOwner, true)
            if self.onEquipped then self:onEquipped(rNewOwner, true) end
        end
    end
    
    function Ob:adjustStats(rHero, bEquipped)
        if bEquipped then
            if self.rDef.tStatModifiers then
                if rHero.tSaved.tStats then
                    for i, tModInfo in ipairs(self.rDef.tStatModifiers) do
                        if tModInfo.statName and tModInfo.statAmount then
                            rHero:incrementStatMod(tModInfo.statName, tModInfo.statAmount)
                        end
                    end
                end
            end
            if self.rDef.tActivitySpeedModifiers then
                for key, nValue in pairs(self.rDef.tActivitySpeedModifiers) do
                    rHero:incrementActivitySpeedMod(key, nValue)
                end
            end
        else
            if self.rDef.tStatModifiers then
                if rHero.tSaved.tStats then
                    for i, tModInfo in ipairs(self.rDef.tStatModifiers) do
                        if tModInfo.statName and tModInfo.statAmount then
                            rHero:incrementStatMod(tModInfo.statName, -tModInfo.statAmount)
                        end
                    end
                end
            end
            if self.rDef.tActivitySpeedModifiers then
                for key, nValue in pairs(self.rDef.tActivitySpeedModifiers) do
                    rHero:incrementActivitySpeedMod(key, -nValue)
                end
            end
        end
    end

    return Ob
end

function MoraleEquippableItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if rHero.setMoraleInvulnerable then
                rHero:setMoraleInvulnerable(bEquipped)
            end
            if self.rDef.bImproveTeammates then
                if rHero.setImproveTeammateMorale then
                    rHero:setImproveTeammateMorale(bEquipped)
                end
            end
        end
    end

    return Ob
end

function DefaultAttackModEquippableItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if bEquipped then
                if rHero.addDefaultAttackMod then
                    rHero:addDefaultAttackMod(self.rDef.sModKey, self.rDef.nChance, self.rDef.tBuffs)
                end
            else
                if rHero.removeDefaultAttackMod then
                    rHero:removeDefaultAttackMod(self.rDef.sModKey)
                end
            end
        end
    end    

    return Ob
end

function TrainingModEquippableItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if bEquipped then
                if rHero.incrementTrainingSpeedMod then
                    rHero:incrementTrainingSpeedMod(self.rDef.nSpeedMultiplier)                
                end
            else
                if rHero.incrementTrainingSpeedMod then
                    rHero:incrementTrainingSpeedMod(-self.rDef.nSpeedMultiplier)                
                end
            end
        end
    end

    return Ob
end

function FightBuffEquippableItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if bEquipped then
                if rHero.addFightBuff then
                    rHero:addFightBuff(self.rDef.sModKey, self.rDef.tBuffs)
                end
            else
                if rHero.removeFightBuff then
                    rHero:removeFightBuff(self.rDef.sModKey)
                end
            end
        end
    end    

    return Ob
end

function XPMultEquippableItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if bEquipped then
                if rHero.incrementXPMuliplier then
                    rHero:incrementXPMuliplier(self.rDef.nMultiplier)                
                end
            else
                if rHero.incrementXPMuliplier then
                    rHero:incrementXPMuliplier(-self.rDef.nMultiplier)
                end
            end
        end
    end

    return Ob
end

function HealthRegenInBaseItem:new(rDef, sItemDefKey)
    local Ob = EquippableItem:new(rDef, sItemDefKey)

    function Ob:onEquipped(rHero, bEquipped)
        if rHero then
            if bEquipped then
                if rHero.addHealthRegenInBase and self.rDef.nHPPerSec then
                    rHero:addHealthRegenInBase(self.rDef.nHPPerSec)
                end
            else
                if rHero.addHealthRegenInBase and self.rDef.nHPPerSec then
                    rHero:addHealthRegenInBase(-self.rDef.nHPPerSec)
                end
            end
        end
    end

    return Ob
end

function ConsumableItem:new(rDef, sItemDefKey)
    local Ob = Item:new(rDef, sItemDefKey)
   
    function Ob:setEquipped(bEquipped)
        self.bEquipped = bEquipped
    end

    function Ob:isEquipped()
        return self.bEquipped
    end

    function Ob:onOwnerChanged(rOldOwner, rNewOwner)
    end

    Ob:setEquipped(false)

    return Ob
end

function PermanentPropItem:new(rDef, sItemDefKey)
    local Ob = ConsumableItem:new(rDef, sItemDefKey)
    
    function Ob:useItem(rEvent)
        -- do nothing
    end

    return Ob
end

function HealConsumableItem:new(rDef, sItemDefKey)
    local Ob = ConsumableItem:new(rDef, sItemDefKey)
    Ob.rDef = rDef

    function Ob:useItem(rEvent)
        if rEvent then
            local tHeroes = rEvent:getAllHeroActors()
            for i, rHeroActor in ipairs(tHeroes) do
                rHeroActor:addHealth(self.rDef.nHealthAmount)
                -- play an effect
                if not rHeroActor:isDead() then
                    rHeroActor:setOverlayText("+ "..rDef.nHealthAmount.."HEALTH", 2, {0, 1, 0})
                end
            end
            return true
        end
        return false
    end

    return Ob
end

function ManaConsumableItem:new(rDef, sItemDefKey)
    local Ob = ConsumableItem:new(rDef, sItemDefKey)
    Ob.rDef = rDef

    function Ob:useItem(rEvent)
        if rEvent then
            local tHeroes = rEvent:getAllHeroActors()
            for i, rHeroActor in ipairs(tHeroes) do
                rHeroActor:getHero():incrementMana(self.rDef.nManaAmount)
                -- play an effect
                if not rHeroActor:isDead() then
                    rHeroActor:setOverlayText("+ "..rDef.nManaAmount.."FOCUS", 2, {0, 1, 0})
                end
            end
            return true
        end
        return false
    end

    return Ob
end

function RessurrectConsumableItem:new(rDef, sItemDefKey)
    local Ob = RessurrectConsumableItem:new(rDef, sItemDefKey)
    Ob.rDef = rDef

    function Ob:useItem(rEvent)
        if rEvent then
            local bResurrected = false
            local tHeroes = rEvent:getAllHeroActors()
            for i, rHeroActor in ipairs(tHeroes) do
                if rHeroActor:isDead() then
                end
--                rHeroActor:setOverlayText("+ "..rDef.nHealthAmount.."HEALTH", 2, {0, 1, 0})
            end
            return true
        end
        return false
    end

    return Ob
end

function BuffConsumableItem:new(rDef, sItemDefKey)
    local Ob = ConsumableItem:new(rDef, sItemDefKey)
    Ob.rDef = rDef

    function Ob:useItem(rEvent)
        if rEvent then
            if self.rDef and self.rDef.tBuffs then
                local tActors = nil
                if self.rDef.bAffectFriendly then
                    tActors = rEvent:getAllHeroActors()
                else
                    tActors = rEvent:getAllEnemies()
                end
                for i, rActor in ipairs(tActors) do
                    for c, rBuffDef in ipairs(self.rDef.tBuffs) do
                        rActor:addBuff(rBuffDef, 1)
                    end
                    -- play an effect
                    if not rActor:isDead() then
                        if self.rDef.sOverlayText then
                            rActor:setOverlayText(self.rDef.sOverlayText, 2, {0, 0, 1})
                        end
                    end
                end            
                return true
            end
        end
        return false
    end

    return Ob
end

Item.tConstructors =
{
    Item = Item.new,   
    EquippableItem = EquippableItem.new,
    MoraleEquippableItem = MoraleEquippableItem.new,
    DefaultAttackModEquippableItem = DefaultAttackModEquippableItem.new,
    TrainingModEquippableItem = TrainingModEquippableItem.new,
    FightBuffEquippableItem = FightBuffEquippableItem.new,
    XPMultEquippableItem = XPMultEquippableItem.new,
    HealConsumableItem = HealConsumableItem.new,
    ManaConsumableItem = ManaConsumableItem.new,
    BuffConsumableItem = BuffConsumableItem.new,
    RessurrectConsumableItem = RessurrectConsumableItem.new,
    PermanentPropItem = PermanentPropItem.new,
    Meteorite = Meteorite.new,
    HealthRegenInBaseItem = HealthRegenInBaseItem.new,
}

function Item.getConstructor(sName)
    if Item.tConstructors[sName] then
        return Item.tConstructors[sName]
    end
end

return Item
