local MiddleManagerAvatar = require('MiddleManagerAvatar')
local DFAnalytics = require('DFCommon.Analytics')
local MiddleManager = {}
local Item = require('Item')
local TileMap = require('TileMap')
local DFFile = require('DFCommon.File')
local DFUtil = require('DFCommon.Util')

MiddleManager.kMAX_LEVEL = 10
MiddleManager.tLevelToTitle =
{
    "", -- never level up to lvl1
    "RANKS001TEXT",
    "RANKS002TEXT",
    "RANKS003TEXT",
    "RANKS004TEXT",
    "RANKS005TEXT",
    "RANKS006TEXT",
    "RANKS007TEXT",
    "RANKS008TEXT",
}
MiddleManager.tLevelImprovementInfo = dofile(DFFile.getDataPath('Characters/MMLevelImprovementInfo.lua'))

g_DebugResearch = false

function MiddleManager.updateLevelImprovementInfo(tInfo)
    if tInfo then
        MiddleManager.tLevelImprovementInfo = DFUtil.deepCopy(tInfo)
    end
end

function MiddleManager:new()
    local Ob = {}

    function Ob:addEquipmentToInventory(sItemDefKey)
        if self.tSaved.tEquipmentInfos[sItemDefKey] then
            self.tSaved.tEquipmentInfos[sItemDefKey].nNumItems = self.tSaved.tEquipmentInfos[sItemDefKey].nNumItems + 1
        else
            local rInfo = {}
            rInfo.sItemDefKey = sItemDefKey
            rInfo.nNumItems = 1                
            self.tSaved.tEquipmentInfos[sItemDefKey] = rInfo
        end
    end

    function Ob:addMeteoriteToInventory(sMeteoriteKey, nLevel)
        if self.tSaved.tMeteorites and sMeteoriteKey and nLevel and nLevel <= Item.kMETEORITE_MAX_LEVEL then
            if Item.getMeteoriteDef(sMeteoriteKey) then
                if not self.tSaved.tMeteorites[sMeteoriteKey] then
                    self.tSaved.tMeteorites[sMeteoriteKey] = {}
                    for i = 1, Item.kMETEORITE_MAX_LEVEL do
                        self.tSaved.tMeteorites[sMeteoriteKey][i] = 0
                    end
                end
                local nNumItems = self.tSaved.tMeteorites[sMeteoriteKey][nLevel]
                self.tSaved.tMeteorites[sMeteoriteKey][nLevel] = nNumItems + 1
            end
        end
    end

    function Ob:getNumMeteoritesInInventory(sMeteoriteKey, nLevel)
        if self.tSaved.tMeteorites and sMeteoriteKey and nLevel and nLevel <= Item.kMETEORITE_MAX_LEVEL then
            if self.tSaved.tMeteorites[sMeteoriteKey] and self.tSaved.tMeteorites[sMeteoriteKey][nLevel] then
                return self.tSaved.tMeteorites[sMeteoriteKey][nLevel]
            end
        end
        return 0
    end

    function Ob:getAllMeteoritesInInventory()
        local tMeteoriteInfos = {}
        for i, sDefKey in ipairs(Item.getAllMeteoriteDefKeys()) do
            for nLevel = 1, Item.kMETEORITE_MAX_LEVEL do
                local nNumItems = self:getNumMeteoritesInInventory(sDefKey, nLevel)
                if nNumItems > 0 then
                    local rInfo = {}
                    rInfo.sDefKey = sDefKey
                    rInfo.nLevel = nLevel
                    table.insert(tMeteoriteInfos, rInfo)
                end
            end
        end
        return tMeteoriteInfos
    end

    function Ob:giveHeroMeteorite(rHero, sMeteoriteDefKey, nLevel)
        if rHero and self.tSaved.tMeteorites and sMeteoriteDefKey and nLevel and nLevel <= Item.kMETEORITE_MAX_LEVEL then
            if self.tSaved.tMeteorites[sMeteoriteDefKey] and self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel] then
                local nNumItems = self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel]
                if nNumItems > 0 then
                    nNumItems = nNumItems - 1
                    self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel] = nNumItems       
                    rHero:addMeteoriteToInventory(sMeteoriteDefKey, nLevel)
                end
            end
        end
    end

    function Ob:removeMeteoriteFromInventory(sMeteoriteDefKey, nLevel, nNumToRemove)
        if self.tSaved.tMeteorites and sMeteoriteDefKey and nLevel then
            if self.tSaved.tMeteorites[sMeteoriteDefKey] and self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel] then
                nNumToRemove = nNumToRemove or 1
                local nNumItems = self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel]
                nNumItems = nNumItems - nNumToRemove
                if nNumItems < 0 then
                    nNumItems = 0
                end
                self.tSaved.tMeteorites[sMeteoriteDefKey][nLevel] = nNumItems       
            end
        end
    end

    function Ob:sellMeteorite(sMeteoriteDefKey, nLevel)
        if sMeteoriteDefKey and nLevel then
            local rDef = Item.getMeteoriteDef(sMeteoriteDefKey)
            if rDef then
                local nSellPrice = rDef.tSellPrice[nLevel] or 10
                self:removeMeteoriteFromInventory(sMeteoriteDefKey, nLevel)
                g_Game:addCash(nSellPrice, true)
            end
        end
    end

    function Ob:combineMeteorite(sMeteoriteDefKeyA, nLevelA, sMeteoriteDefKeyB, nLevelB, sMeteoriteDefKeyC, nLevelC)
        local nNumA = self:getNumMeteoritesInInventory(sMeteoriteDefKeyA, nLevelA)
        local nNumB = self:getNumMeteoritesInInventory(sMeteoriteDefKeyB, nLevelB)
        local nNumC = self:getNumMeteoritesInInventory(sMeteoriteDefKeyC, nLevelC)

        if nNumA > 0 and nNumB > 0 and nNumC > 0 then
            local sNewMeteoriteDefKey, nNewMeteoriteLevel = Item.getMeteoriteCombo(sMeteoriteDefKeyA, nLevelA, sMeteoriteDefKeyB, nLevelB, sMeteoriteDefKeyC, nLevelC)
            if sNewMeteoriteDefKey and nNewMeteoriteLevel then
                self:removeMeteoriteFromInventory(sMeteoriteDefKeyA, nLevelA)
                self:removeMeteoriteFromInventory(sMeteoriteDefKeyB, nLevelB)
                self:removeMeteoriteFromInventory(sMeteoriteDefKeyC, nLevelC)
                self:addMeteoriteToInventory(sNewMeteoriteDefKey, nNewMeteoriteLevel)

                return sNewMeteoriteDefKey, nNewMeteoriteLevel
            end
        end

        return nil
    end

    function Ob:onTick(elapsed)
        local tA = self:getActiveAbilities()
        for k,v in pairs(tA) do
            v:tickRealTimeCooldown(elapsed)
        end
    end

    function Ob:getNumEquipmentInInventory(sItemDefKey)
        if sItemDefKey then
            local rInfo = self.tSaved.tEquipmentInfos[sItemDefKey]
            if rInfo then
                return rInfo.nNumItems
            end
        end
        return 0
    end

    function Ob:giveEquipmentToHero(sItemDefKey, rHero)
        if sItemDefKey and rHero then
            if self.tSaved.tEquipmentInfos[sItemDefKey] then
                local rDef = Item.getEquipmentDef(sItemDefKey)
                if rDef then
                    local constructor = Item.getConstructor(rDef.sItemType)
                    if constructor then
                        local rItem = constructor(Item, rDef, sItemDefKey)
                        if rItem then
                            if rHero:addItemToNextAvailableSlot(rItem) then
                                self:removeEquipmentFromInventory(sItemDefKey)
                            end
                        end
                    end
                end
            end
        end
    end

    function Ob:removeEquipmentFromInventory(sItemDefKey)
        if self.tSaved.tEquipmentInfos[sItemDefKey] then
            self.tSaved.tEquipmentInfos[sItemDefKey].nNumItems = self.tSaved.tEquipmentInfos[sItemDefKey].nNumItems - 1
            if self.tSaved.tEquipmentInfos[sItemDefKey].nNumItems <= 0 then
                self.tSaved.tEquipmentInfos[sItemDefKey] = nil
            end
        end
    end

    function Ob:sellEquipment(sItemDefKey, nPrice)
        if sItemDefKey and nPrice then
            g_Game:addCash(nPrice)
            DFAnalytics.incrStat('csh_ernd_sell', nPrice)
            self:removeEquipmentFromInventory(sItemDefKey)
        end
    end

    function Ob:addConsumableItemToInventory(sItemDefKey, bIsTemporaryRenewableItem)       
        if self.tSaved.tConsumableInfos[sItemDefKey] then
            self.tSaved.tConsumableInfos[sItemDefKey].nNumItems = self.tSaved.tConsumableInfos[sItemDefKey].nNumItems + 1
        else
            local rInfo = {}
            rInfo.sItemDefKey = sItemDefKey
            rInfo.nNumItems = 1                
            self.tSaved.tConsumableInfos[sItemDefKey] = rInfo
        end

        if bIsTemporaryRenewableItem then
            if not self.tSaved.tConsumableInfos[sItemDefKey].nNumTemporaryRenewableItems or self.tSaved.tConsumableInfos[sItemDefKey].nNumTemporaryRenewableItems <= 0 then
                self.tSaved.tConsumableInfos[sItemDefKey].nNumTemporaryRenewableItems = 1
            else
                -- remove the one we just added
                self.tSaved.tConsumableInfos[sItemDefKey].nNumItems = self.tSaved.tConsumableInfos[sItemDefKey].nNumItems - 1
                if self.tSaved.tConsumableInfos[sItemDefKey].nNumItems <= 0 then
                    self.tSaved.tConsumableInfos[sItemDefKey] = nil
                end    
            end
        end
    end

    function Ob:getNumConsumablesInInventory(sItemDefKey)
        local count = 0
        
        if sItemDefKey then
            local rInfo = self.tSaved.tConsumableInfos[sItemDefKey]
            if rInfo then
                count = rInfo.nNumItems
            end
        end

        return count
    end

    function Ob:grantRenewableItems()
        local tRenewables = g_Game:getEnabledRenewableItems()
        
        for sItemDefKey, nCount in pairs(tRenewables) do
            self:addConsumableItemToInventory(sItemDefKey, true)
        end
    end
    
    function Ob:clearRenewableItems()
        for sItemDefKey, rInfo in pairs(self.tSaved.tConsumableInfos) do
            if rInfo.nNumTemporaryRenewableItems and rInfo.nNumItems then
                self.tSaved.tConsumableInfos[sItemDefKey].nNumItems = self.tSaved.tConsumableInfos[sItemDefKey].nNumItems - rInfo.nNumTemporaryRenewableItems
                self.tSaved.tConsumableInfos[sItemDefKey].nNumTemporaryRenewableItems = nil
                
                if self.tSaved.tConsumableInfos[sItemDefKey].nNumItems <= 0 then
                    self.tSaved.tConsumableInfos[sItemDefKey] = nil
                end     
            end
        end
    end
    
    function Ob:useConsumable(sItemDefKey, rEvent)
        if not rEvent then
            print('ERROR: Tried to use an item without an event')
            return false
        end
        if self.tSaved.tConsumableInfos[sItemDefKey] then
            local rDef = Item.getConsumableDef(sItemDefKey)
            if rDef then
                local constructor = Item.getConstructor(rDef.sItemType)
                if constructor then
                    local rItem = constructor(Item, rDef, sItemDefKey)
                    if rItem then                        
                        if rItem:useItem(rEvent) then
                            g_Game:getTutorialManager():onItemUsed()
                            self.tSaved.tConsumableInfos[sItemDefKey].nNumItems = self.tSaved.tConsumableInfos[sItemDefKey].nNumItems - 1
                            if self.tSaved.tConsumableInfos[sItemDefKey].nNumItems <= 0 then
                                self.tSaved.tConsumableInfos[sItemDefKey] = nil
                            end         
                            DFAnalytics.logEvent("csm_used", { type = sItemDefKey })
                            return true
                        end
                    end
                end
            end   
        else
            print('ERROR: '..sItemDefKey..' not found in the inventory')
        end
        return false
    end

    function Ob:getAllConsumableItemInfos()
        return self.tSaved.tConsumableInfos
    end

    
    function Ob:getAllEquipmentInfos()
        return self.tSaved.tEquipmentInfos
    end

    Ob.tEquipment = {}
    local tSaveTable = g_Game:getSaveTable()
    if not tSaveTable.tMiddleManager then
        tSaveTable.tMiddleManager = {}
    end
    Ob.tSaved = tSaveTable.tMiddleManager
    if not Ob.tSaved.nLevel then
        Ob.tSaved.nLevel = 1
    end
    if not Ob.tSaved.tEquipmentDefKeys then
        Ob.tSaved.tEquipmentDefKeys = {}
    else
        for i, sEquipmentDefKey in ipairs(Ob.tSaved.tEquipmentDefKeys) do
            Ob:addEquipmentToInventory(sEquipmentDefKey, true)
        end
    end
    if not Ob.tSaved.tMeteorites then
        Ob.tSaved.tMeteorites = {}
    end
    if not Ob.tSaved.tResearchProgress then
        Ob.tSaved.tResearchProgress = {}
    end
    if not Ob.tSaved.tConsumableInfos then
        Ob.tSaved.tConsumableInfos = {}
    end
    if not Ob.tSaved.tEquipmentInfos then
        Ob.tSaved.tEquipmentInfos = {}
    end
    if not Ob.tSaved.nStarRatingScore then -- accumulated score
        Ob.tSaved.nStarRatingScore = 0
    end
    if not Ob.tSaved.nStarRatingXP then
        Ob.tSaved.nStarRatingXP = 0
    end
    if not Ob.tSaved.nStarRatingLevel then
        Ob.tSaved.nStarRatingLevel = 1
    end
    Ob.tItemsBeingResearched = {}

    function Ob:createAvatar()
        self.rMiddleManager = MiddleManagerAvatar:new()
        self.rMiddleManager.getStatLevel = function() return self:getStarRatingLevel() end
    end

    function Ob:getAvatar()
        return self.rMiddleManager
    end

    function Ob:getLevel()
        return self.tSaved.nLevel
    end

    function Ob:isAtEndGame()
        return self.tSaved.bIsAtEndGame 
    end

    function Ob:setAtEndGame(bSet)
        self.tSaved.bIsAtEndGame = bSet
    end

    function Ob:incrementLevel(nIncrement)
        self.tSaved.nLevel = self.tSaved.nLevel + nIncrement
        if self.tSaved.nLevel > MiddleManager.kMAX_LEVEL then
            self.tSaved.bIsAtEndGame = true
            self.tSaved.nLevel = MiddleManager.kMAX_LEVEL
        else
            DFAnalytics.logEvent("prg_mmgr_lvl_" .. tostring(self.tSaved.nLevel))
            DFAnalytics.logStat("mmgr_lvl", self.tSaved.nLevel)
        end        
        g_Game:saveGame()
    end

    function Ob:onWindowClosed(rPane)
        if rPane == g_GUI.rEventEndScreenUI then -- should implement an isa
            g_GUI:AddSimpleMessagePane("The Middle Manager has LEVELED UP!  More territories unlocked!!")        
            g_GUI:removeWindowCloseListener(self)
        end
    end

    function Ob:onItemRemoved(rItemToRemove)
        if not rItemToRemove then
            return
        end
        for i, rItem in ipairs(self.tEquipment) do
            if rItem == rItemToRemove then
                table.remove(self.tEquipment, i)
                if self.tSaved.tEquipmentDefKeys[i] then
                    table.remove(self.tSaved.tEquipmentDefKeys, i)
                    return
                end
            end
        end
    end

    function Ob:sellConsumable(sItemDefKey, nPrice)
        if sItemDefKey and nPrice then
            g_Game:addCash(nPrice)
            DFAnalytics.incrStat('csh_ernd_sell', nPrice)
            local rInfo = self.tSaved.tConsumableInfos[sItemDefKey]
            if rInfo then
                rInfo.nNumItems = rInfo.nNumItems - 1
                if rInfo.nNumItems <= 0 then
                    self.tSaved.tConsumableInfos[sItemDefKey] = nil
                end
            end
        end
    end

    function Ob:getEquipmentDefKeysForResearch()
        local tKeys = {}
        local tAllDefKeys = Item.getAllEquipmentDefKeys()        
        if tAllDefKeys then
            for i, sKey in ipairs(tAllDefKeys) do
                if self:canResearch('equipment', sKey) then
                    table.insert(tKeys, sKey)
                end
            end
        end
        return tKeys
    end
    
    function Ob:getMeteoriteDefKeysForResearch()
        local tKeys = {}
        local tAllDefKeys = Item.getAllMeteoriteDefKeys()        
        if tAllDefKeys then
            for i, sKey in ipairs(tAllDefKeys) do
                if self:canResearch('meteorite', sKey) then
                    table.insert(tKeys, sKey)
                end
            end
        end
        return tKeys
    end

    function Ob:getConsumableDefKeysForResearch()
        local tKeys = {}
        local tAllDefKeys = Item.getAllConsumableDefKeys()
        if tAllDefKeys then
            for i, sKey in ipairs(tAllDefKeys) do
                if self:canResearch('consumable', sKey) then
                    table.insert(tKeys, sKey)
                end
            end
        end
        return tKeys
    end

    function Ob:getResearchProgress(sDefKey) -- range 0 - 1
        if sDefKey then
            if self.tSaved.tResearchProgress[sDefKey] then
                return self.tSaved.tResearchProgress[sDefKey]
            end
        end
        return 0
    end

    function Ob:incrementResearchProgress(sDefKey, nAmount)
        if sDefKey then
            if not self.tSaved.tResearchProgress[sDefKey] then
                self.tSaved.tResearchProgress[sDefKey] = 0
            end
            self.tSaved.tResearchProgress[sDefKey] = self.tSaved.tResearchProgress[sDefKey] + nAmount
            if self.tSaved.tResearchProgress[sDefKey] >= 1 then
                self:setItemNewFlag(sDefKey, true)
                self.tSaved.tResearchProgress[sDefKey] = 1
                g_TodoList:onResearched(sDefKey)
                return true
            end
        end
        return false
    end

    function Ob:setIsResearching(sDefKey, bIsResearching)
        if sDefKey then
            if bIsResearching then
                self.tItemsBeingResearched[sDefKey] = true
            else
                self.tItemsBeingResearched[sDefKey] = nil
            end
        end
    end
    
    function Ob:isResearching(sDefKey)
        if sDefKey then
            return self.tItemsBeingResearched[sDefKey]
        else
            return false
        end
    end

    function Ob:isResearched(sType, sDefKey)
        if g_DebugResearch then
            return true
        end
        if not sDefKey or not sType then
            return false
        end
        if self:getResearchProgress(sDefKey) >= 1 then
            return true
        end
        local rDef = nil
        if sType == 'roomprop' then
            rDef = TileMap.getPropDef(sDefKey)
        elseif sType == 'equipment' then
            rDef = Item.getEquipmentDef(sDefKey)
        elseif sType == 'consumable' then
            rDef = Item.getConsumableDef(sDefKey)
        elseif sType == 'meteorite' then
            rDef = Item.getMeteoriteDef(sDefKey)
        end
        if rDef then
            if rDef.bResearched then
                return true
            end
        end
        return false
    end

    function Ob:canResearch(sType, sDefKey)
        if not sDefKey and not sType then
            return false
        end
        local rDef = nil
        if sType == 'roomprop' then
            rDef = TileMap.getPropDef(sDefKey)
        elseif sType == 'equipment' then
            rDef = Item.getEquipmentDef(sDefKey)
        elseif sType == 'consumable' then
            rDef = Item.getConsumableDef(sDefKey)
        elseif sType == 'meteorite' then
            rDef = Item.getMeteoriteDef(sDefKey)
        end
        if not rDef then
            return false
        end
        if self:isResearched(sType, sDefKey) then
            return false
        end        
        if rDef.sDefKeyRequiredToResearch then
            return self:isResearched(sType, rDef.sDefKeyRequiredToResearch)
        end
        return true
    end

    function Ob:hasEquipmentToResearch()
        local tDefKeys = Item.getAllEquipmentDefKeys()
        if tDefKeys then
            for i, sDefKey in ipairs(tDefKeys) do
                local rDef = Item.getEquipmentDef(sDefKey)
                if rDef then
                    if not g_MiddleManager:isResearched('equipment', sDefKey) then
                        local rDef = Item.getEquipmentDef(sDefKey)
                        if rDef and not rDef.bHideInShop then
                            return true
                        end
                    end
                end
            end
        end
        return false
    end

    function Ob:hasConsumableToResearch()
        local tDefKeys = Item.getAllConsumableDefKeys()
        if tDefKeys then
            for i, sDefKey in ipairs(tDefKeys) do
                local rDef = Item.getEquipmentDef(sDefKey)
                if rDef then
                    if not g_MiddleManager:isResearched('consumable', sDefKey) then
                        local rDef = Item.getEquipmentDef(sDefKey)
                        if rDef and not rDef.bHideInShop then
                            return true
                        end
                    end
                end
            end
        end
        return false
    end

    -- Returns all abilities, as there's no notion of active vs. inactive MM abilities.
    function Ob:getActiveAbilities()
        return self.rMiddleManager:getAbilities()
    end

    function Ob:getStarRatingScore()
        return self.tSaved.nStarRatingScore
    end

    function Ob:incrementStarRatingScore(nAmount)
        self.tSaved.nStarRatingScore = self.tSaved.nStarRatingScore + nAmount
		if MOAIGameCenterIOS then
            MOAIGameCenterIOS.reportScore(self.tSaved.nStarRatingScore, 'com.doublefine.mmoj.branchrating') -- update the leaderboard name here
		end
        g_TodoList:onRatingScoreChanged(nAmount)
    end

    function Ob:areAnyItemsNew()
        if not self.tSaved then
            return false
        end
    
        local bAnyNew = false
    
        if self.tSaved.tNewShopItems then
            -- categories
            for sDefKey, bIsNew in pairs(self.tSaved.tNewShopItems) do
                if bIsNew then
                    bAnyNew = true
                    break
                end
            end
        end
            
        return bAnyNew
    end
    
    function Ob:areAnyItemsNewForCategory(sType)
        local bAnyNew = false
        
        local tDefKeys = {}
        
        if sType == 'consumable' or sType == 'Consumables' then
            tDefKeys = Item.getAllConsumableDefKeys()
        elseif sType == 'equipment' or sType == 'Equipment' then
            tDefKeys = Item.getAllEquipmentDefKeys()
        end
        
        for i, sDefKey in ipairs(tDefKeys) do
            if self:isItemNewInShop(sDefKey) then
                bAnyNew = true
                break
            end
        end
        
        return bAnyNew
    end
    
    function Ob:setItemNewFlagForCategory(sType, bIsNew)
        local tDefKeys = {}
        
        if sType == 'consumable' or sType == 'Consumables' then
            tDefKeys = Item.getAllConsumableDefKeys()
        elseif sType == 'equipment' or sType == 'Equipment' then
            tDefKeys = Item.getAllEquipmentDefKeys()
        end
        
        for i, sDefKey in ipairs(tDefKeys) do
            self:setItemNewFlag(sDefKey, bIsNew)
        end        
    end
    
    function Ob:setItemNewFlag(sDefKey, bIsNew)
        if not self.tSaved then
            return
        end
    
        if not self.tSaved.tNewShopItems then
            self.tSaved.tNewShopItems = {}
        end

        self.tSaved.tNewShopItems[sDefKey] = bIsNew
    end
    
    

    function Ob:isItemNewInShop(sDefKey)
        local bIsNew = false
        
        if self.tSaved and self.tSaved.tNewShopItems and self.tSaved.tNewShopItems[sDefKey] == true then
            bIsNew = true
        end
        
        return bIsNew
    end
    
    function Ob:getStarRatingXP()
        return self.tSaved.nStarRatingXP
    end

    function Ob:incrementStarRatingXP(nAmount)
        if self:isAtMaxStarLevel() then
            return false
        end

        self.tSaved.nStarRatingXP = self.tSaved.nStarRatingXP + nAmount
		local nXPForNextLevel = self:getStarRatingXPForNextLevel()
        if self.tSaved.nStarRatingXP >= nXPForNextLevel then
            self.tSaved.nStarRatingXP = self.tSaved.nStarRatingXP - nXPForNextLevel
            self.tSaved.nStarRatingLevel = self.tSaved.nStarRatingLevel + 1
            return true
        end
        return false
    end

    function Ob:getStarRatingLevel()
        return self.tSaved.nStarRatingLevel
    end

    function Ob:isAtMaxStarLevel()
        local nMaxLevel = table.getn(MiddleManager.tLevelImprovementInfo)
        if self.tSaved.nStarRatingLevel >= nMaxLevel then
            return true
        end 
        return false        
    end

	function Ob:getXPPct()
		return self:getStarRatingXP() / self:getStarRatingXPForNextLevel()
	end

    function Ob:getStarRatingXPForNextLevel()
		local nXPForNextLevel = 1       
		if MiddleManager.tLevelImprovementInfo[self.tSaved.nStarRatingLevel] and MiddleManager.tLevelImprovementInfo[self.tSaved.nStarRatingLevel].nXPForNextLevel then
			nXPForNextLevel = MiddleManager.tLevelImprovementInfo[self.tSaved.nStarRatingLevel].nXPForNextLevel
		end
        return nXPForNextLevel
    end

    return Ob
end

return MiddleManager
