local DFAudio = require('DFCommon.Audio')
local DFFile = require('DFCommon.File')
local DFSaveLoad = require('DFCommon.SaveLoad')
local MmojAudio = {}
local SoundCue = {}

local tSoundProjects =
{
    tFEV =
    {
        'MMOJ_Audio/MMOJ_Audio.fev',
    },
}

function MmojAudio:init()
    local Ob = {} 

    function Ob:playAmbience(sCuePath)
        if not MOAIFmodEventMgr then
            return
        end
        if sCuePath then
            if self.rCurAmbience then
                self.rCurAmbience:stop()
            end
            self.rCurAmbience = MOAIFmodEventMgr.playEvent2D(sCuePath, true)
            return self.rCurAmbience
        end
        return nil
    end

    function Ob:dumpMemoryInfo(sDumpTag)
        local currentAlloc, maxAlloc = MOAIFmodEventMgr.getMemoryStats()
        
        print("\n\n--------------------------------------------------------")
        print("Dump Audio, tag: " .. sDumpTag)
        print("currentAlloc (KB): " .. currentAlloc / 1024)
        print("maxAlloc (KB): " .. maxAlloc / 1024)
        print("--------------------------------------------------------\n")
    end

    function Ob:stopCurAmbience()
        if self.rCurAmbience then
            self.rCurAmbience:stop()
        end
    end

    function Ob:playMusic(sCuePath, bLooping)
        if not MOAIFmodEventMgr then
            return
        end
        if sCuePath then
            if self.rCurMusic then
                self.rCurMusic:stop()
                MOAIFmodEventMgr.unloadEvent(self.sCurMusicPath, true)
            end
            self.rCurMusic = MOAIFmodEventMgr.playEvent2D(sCuePath, bLooping)
            self.sCurMusicPath = sCuePath
        end
    end

    function Ob:stopMusic()
        if not MOAIFmodEventMgr then
            return
        end
        if self.rCurMusic then
            self.rCurMusic:stop()
            self.rCurMusic = nil
        end
    end

    function Ob:onEnterEvent()
        if not MOAIFmodEventMgr then
            return
        end

        --self:dumpMemoryInfo("enterEventBegin")
        
        -- force a blocking unload before we load more to prevent memory overlap
        MOAIFmodEventMgr.unloadGroup("MMOJ_Audio/SFX/Office", true)
        
        MOAIFmodEventMgr.loadGroup("MMOJ_Audio/SFX/Battle")        
        MOAIFmodEventMgr.loadGroup("MMOJ_Audio/SFX/HeroSFX")
        MOAIFmodEventMgr.loadGroup("MMOJ_Audio/SFX/EnemySFX")
        
        --self:dumpMemoryInfo("enterEventEnd")
    end
    
    function Ob:onEnterOffice()
        if not MOAIFmodEventMgr then
            return
        end
        
        --self:dumpMemoryInfo("enterOfficeBegin")
        
        -- force a blocking unload before we load more to prevent memory overlap
        MOAIFmodEventMgr.unloadGroup("MMOJ_Audio/SFX/Battle", true)
        MOAIFmodEventMgr.unloadGroup("MMOJ_Audio/SFX/HeroSFX", true)
        MOAIFmodEventMgr.unloadGroup("MMOJ_Audio/SFX/EnemySFX", true)
        
        MOAIFmodEventMgr.loadGroup("MMOJ_Audio/SFX/Office")
        
        --self:dumpMemoryInfo("enterOfficeEnd")
    end
    
    function Ob:playCue(sCueName, bLooping)
        if not MOAIFmodEventMgr then
            return nil
        end
        if sCueName then
            local rInstance = MOAIFmodEventMgr.playEvent2D(sCueName, bLooping)
            return rInstance
        end
        return nil
    end 

    if MOAIFmodEventMgr then
        local soundMemMB = 10
        
        -- on windows the sound allocations are slightly larger (about 40% actually), so allocate more memory there.
        if MOAIEnvironment.osBrand == "Windows" then
	        soundMemMB = 32
        end
    
		local bLowPerf = (MOAIEnvironment.devModel == 'Kindle Fire')
		if bLowPerf then print("Using low-performance audio") end
		local tInitParams =
		{
			voiceLRUMaxMB = 0,
			voiceLRUBufferMB = 0,      
            soundMemoryMB = soundMemMB,
            enableAuditioning = true,
            androidLowPerformance = bLowPerf,
		}
        MOAIFmodEventMgr.init( tInitParams )
		if tSoundProjects.tFEV then
			for _, sProject in pairs( tSoundProjects.tFEV ) do
				local bLoaded = MOAIFmodEventMgr.loadProject( DFFile.getAudioPath( sProject ) )    
                --print(sProject.."loaded: "..tostring(bLoaded))
			end
		end        
        local rMic = MOAIFmodEventMgr.getMicrophone() 
        rMic:setLoc(0, 0)
        
        -- preload the UI sounds
        MOAIFmodEventMgr.loadGroup("MMOJ_Audio/SFX/UI")    
    end

    Ob.tCueDefs = dofile(DFFile.getDataPath('Audio/SoundCues.lua'))

    function Ob:loadSettings()
        if MOAIFmodEventMgr then
            local kSETTING_FILE_PATH = "mmojsettings"
            
            local tSettings = DFSaveLoad.loadTable(kSETTING_FILE_PATH)
            
            if tSettings and false == tSettings.musicEnabled then
                MOAIFmodEventMgr.muteSoundCategory('music', true)
            end
            
            if tSettings and false == tSettings.sfxEnabled then
                MOAIFmodEventMgr.muteSoundCategory('UI', true)
                MOAIFmodEventMgr.muteSoundCategory('SFX', true)
                MOAIFmodEventMgr.muteSoundCategory('Ambience', true)
            end
        end
    end
    
    Ob:loadSettings()
    
    return Ob
end

function SoundCue:new(rCueInfo)
    local Ob = {}

    function Ob:load()
        if self.rCueInfo then
            for i, sSoundPath in ipairs(self.rCueInfo.tSounds) do
                local rSound = DFAudio.loadSound(sSoundPath)
                table.insert(self.tLoadedSounds, rSound)
            end
            self.bLoaded = true
        end
    end

    function Ob:unload()
        --????
    end

    function Ob:play(bLooping, nLoopStartTime, nLoopEndTime)
        if self.bLoaded then
            local nNumSounds = table.getn(self.tLoadedSounds)
            local rSound = self.tLoadedSounds[math.random(nNumSounds)]
            if rSound then
                rSound:setLooping(bLooping)
                if bLooping then
                    rSound:setLoopPoints(nLoopStartTime, nLoopEndTime)
                end
                rSound:play()
                self.rCurPlayingSound = rSound
            end
            self.bLoaded = true
        else
            if self.rCueInfo then
                local nNumSounds = table.getn(self.rCueInfo.tSounds)
                local nIndex = math.random(nNumSounds)
                if bLooping then
                    if not nLoopStartTime then
                        nLoopStartTime = self.rCueInfo.tLoopStartTime[nIndex]
                    end
                    if not nLoopEndTime then
                        nLoopEndTime = self.rCueInfo.tLoopEndTime[nIndex]
                    end
                end
                self.rCurPlayingSound = DFAudio.playSound(self.rCueInfo.tSounds[nIndex], false, bLooping, nLoopStartTime, nLoopEndTime)
            end
        end
    end

    function Ob:stop()
        if self.rCurPlayingSound then
            self.rCurPlayingSound:stop()
        end
    end

    Ob.bLoaded = false
    Ob.rCueInfo = rCueInfo
    Ob.rCurPlayingSound = nil
    Ob.tLoadedSounds = {}

    return Ob
end

return MmojAudio
