local PropTouchManager = {}
local DFFile = require('DFCommon.File')
local DFUtil = require('DFCommon.Util')
local TrainingSelectionUI = require('ui/TrainingSelectionUI')
local Renderer = require('Renderer')

function PropTouchManager:new()

    local Ob = {}
    
    function Ob:onTouchBed(rProp)
        local tSelectableHeroes = {}
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end

        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM015TEXT")
    end

    function Ob:onTouchWorkout(rProp)
        local tSelectableHeroes = {}
        
        -- TODO: filter out folks who can't train
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end
        
        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM016TEXT")
    end
    
    function Ob:onTouchWork(rProp)
        local tSelectableHeroes = {}

        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end
        
        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM019TEXT")
    end
    
    function Ob:onTouchTrainingDummy(rProp)
        local tSelectableHeroes = {}
        
        -- TODO: filter out folks who can't train
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end
                
        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM016TEXT")
    end
    
    function Ob:onTouchResearch(rProp)
        local tSelectableHeroes = {}

        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end
        
        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM018TEXT")
    end
    
    function Ob:onTouchRecreation(rProp)
        local tSelectableHeroes = {}

        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero and rHero:isIdle() then
                table.insert(tSelectableHeroes, rHero)
            end
        end
        
        g_GUI:ShowPropActionHeroSelector(rProp, tSelectableHeroes, self, "ROOM017TEXT")
    end
    
    function Ob:onTouchBulletinBoard(rProp)
        g_GUI:ShowBulletinBoard()
    end
    
    -- used by HeroChooserUI
    function Ob:onHeroChooserCancelled()
        self.rCurrentProp = nil
    end
    
    -- input handling (so we can filter stuff down to the UI screens we load)
    function Ob:onFinger(eventType, sx, sy)
        local handled = false
        
        if self.rTrainingSelectionUI then
            self.rTrainingSelectionUI:onFinger(eventType, sx, sy)
            handled = true
        end
        
        return handled
    end
    
    function Ob:onPropTouch(rProp)
        if rProp then
            print("hey somebody touched a prop: " .. rProp:getPropType())
            
            self.rCurrentProp = rProp
            
            local propType = rProp:getPropType()
            
            if propType == "bed" then
                self:onTouchBed(rProp)
            elseif propType == "workout" then
                self:onTouchWorkout(rProp)
            elseif propType == "work" then
                self:onTouchWork(rProp)
            elseif propType == "trainingdummy" then
                self:onTouchTrainingDummy(rProp)
            elseif propType == "research" then
                self:onTouchResearch(rProp)
            elseif propType == "recreation" then
                self:onTouchRecreation(rProp)
            elseif propType == "bboard" then
                self:onTouchBulletinBoard(rProp)
            elseif propType == "leaderboard" then
                if g_Game:isActionEnabled('ShowOptions') and MOAIGameCenterIOS then
                    MOAIGameCenterIOS.showDefaultLeaderboard()
                end
            elseif propType == "superiumtree" then
                g_Game:onTouchSuperiumTree(rProp)
            elseif propType == "mmmap" then
                if g_Game:isActionEnabled('OpenMap') then
                    g_GUI:ShowAlertTerritoryMap()
                end
            elseif propType == "roster" then
                if g_Game:isActionEnabled('ShowRoster') then
                    g_GUI:ShowRosterMenu()
                end
            elseif propType == "mostwantedposter" then
                g_GUI:ShowMostWantedUI()
            elseif propType == "MM_Crystal" then
                g_GUI:ShowMeteoriteCombiner()
            end

            if rProp.rPropDef and rProp.rPropDef.soundCue then
                g_Audio:playCue(rProp.rPropDef.soundCue)
            end
        end
    end

    Ob.tSelectableHeroes = {}
    
    return Ob
end

return PropTouchManager
