local DFGraphics = require('DFCommon.Graphics')
local Renderer = {}

local bDebugIphone3 = false
local bDebugIPad = false
local bDebugIPad3 = false
local bDebugIphone5 = false
local bSuperWideFakeDevice = false

local rViewport = nil
local rCamera = nil

local tLayers = {
	Tiles = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'Tiles',
	},
	Gameplay = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'Gameplay',
	},
	GameplayOverlay = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'GameplayOverlay',
	},
	EventBG = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'EventBG',
	},
	EventOverlay = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'EventOverlay',
	},
    EventFX = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'EventFX',
    },
	UI = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'UI',
	},
	UIOverlay = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'UIOverlay',
	},
	ProfileOverlay = {
		RenderLayer = nil,
		Partition = nil,
		Name = 'ProfileOverlay',
	},
	LayerRefs = { },
}
local rLayerGameplay = nil
local rLayerGameplayOverlay = nil
local rLayerUI = nil

local rPartitionGameplay = nil
local rPartitionGameplayOverlay = nil
local rPartitionUI = nil

local nCameraBaseScaleX = 1
local nCameraBaseScaleY = 1

local nCurXZoomScale = 1
local nCurYZoomScale = 1

local kMIN_ZOOM = 0.65
local kMAX_ZOOM = 2.5

Renderer.tScreenInfo =
{
    SMALL = { key = 'SMALL', sTextureSizeSuffix = '_Med', nTextureSizeFactor = 1 },
    MED = { key = 'MED', sTextureSizeSuffix = '_Med', nTextureSizeFactor = 1 },
    LARGE = { key = 'LARGE', sTextureSizeSuffix = '_Lg', nTextureSizeFactor = 0.5 },
}

local kDesiredScreenWidth = 960
local kDesiredScreenHeight = 640

local nScaledScreenWidth = kDesiredScreenWidth
local nScaledScreenHeight = kDesiredScreenHeight
nScaledHalfWidth = kDesiredScreenWidth * .5
nScaledHalfHeight = kDesiredScreenHeight * .5

function Renderer.computeScreenInfo(width, height)
    if MOAIEnvironment.devmem ~= nil and MOAIEnvironment.devmem < (300 * 1024 * 1024) then
        return Renderer.tScreenInfo.SMALL
    end

    if (width == 480 and height == 320) or
       (height == 480 and width == 320) then
        return Renderer.tScreenInfo.SMALL
    elseif (width == 2048 and height == 1536) or
           (height == 2048 and width == 1536) then
        return Renderer.tScreenInfo.LARGE
    else
        return Renderer.tScreenInfo.MED
    end    
end

function Renderer.resizeViewports(width, height)
    Renderer.tCurScreenInfo = Renderer.computeScreenInfo(width, height)    
    
    Renderer.scaleScreen(width, height)

    if g_GUI then
        g_GUI:onViewportResized()
    end
end

function Renderer.scaleScreen(width, height)
    print("Renderer.scaleScreen: " .. width .. "w " .. height .. "h")

    Renderer.nAspectRatio = width / height
    Renderer.nDisplayScaleX = 1.0
    Renderer.nDisplayScaleY = 1.0
    
    local nExpectedAspectRatio = kDesiredScreenWidth / kDesiredScreenHeight
    
    if Renderer.nAspectRatio < nExpectedAspectRatio then
        print("screen is taller than we would like!")
        -- if we're taller, we can scale based on the device's width and everything will still fit
        nScaledScreenWidth = kDesiredScreenWidth
        nScaledScreenHeight = kDesiredScreenWidth / Renderer.nAspectRatio
    elseif Renderer.nAspectRatio > nExpectedAspectRatio then
        print("screen is wider than we would like!")
        -- if we're wider, we need to scale based on height
        nScaledScreenWidth = kDesiredScreenHeight * Renderer.nAspectRatio
        nScaledScreenHeight = kDesiredScreenHeight
    end
    nScaledHalfWidth = nScaledScreenWidth * .5
    nScaledHalfHeight = nScaledScreenHeight * .5
    
    Renderer.nDisplayScaleX = width / nScaledScreenWidth
    Renderer.nDisplayScaleY = height / nScaledScreenHeight
    
    print("Renderer.scaleScreen: " .. nScaledScreenWidth .. "w(scaled) " .. nScaledScreenHeight .. "h(scaled)")
    
    Renderer.gameViewport:setScale(nScaledScreenWidth, nScaledScreenHeight)
    -- UI viewport is pixel scale by default
    Renderer.uiViewport:setOffset(-1, 1) -- 0, 0 is upper left
    Renderer.uiViewport:setScale(nScaledScreenWidth, nScaledScreenHeight)
    
    if Renderer.rProfileViewportWindow then
        Renderer.rProfileViewportWindow:setSize (nScaledScreenWidth, nScaledScreenHeight)
        Renderer.rProfileViewportWindow:setScale (nScaledScreenWidth, -nScaledScreenHeight)
    end
end

function Renderer.initializeRenderer()
    local rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinDeck2D.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.DECK2D_SHADER, rShaderOverride )
	
	rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinDeck2DTexOnly.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.DECK2D_TEX_ONLY_SHADER, rShaderOverride )
	
	rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinFont.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.FONT_SHADER, rShaderOverride )
	
	rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinLine.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.LINE_SHADER, rShaderOverride )
	
    rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinMeshDeck.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.MESH_DECK_SHADER, rShaderOverride )
	
	rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinMesh.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.MESH_SHADER, rShaderOverride )
	
	rShaderOverride = DFGraphics.loadShader ( "Shaders/builtinSkinnedMesh.shd" )
    MOAIShaderMgr.setShader ( MOAIShaderMgr.SKINNED_MESH_SHADER, rShaderOverride )

    local viewportSizeX = 960
    local viewportSizeY = 640
    
    if bDebugIphone3 then
        viewportSizeX = 480
        viewportSizeY = 320
    end
    if bDebugIPad then
        viewportSizeX = 1024
        viewportSizeY = 768
    end
    if bDebugIPad3 then
        viewportSizeX = 2048
        viewportSizeY = 1536
    end
    if bDebugIphone5 then
        viewportSizeX = 1136
        viewportSizeY = 640
    end
    if bSuperWideFakeDevice then
        viewportSizeX = 1440
        viewportSizeY = 640
    end
    
    -- Setup viewport
    local gameViewport, uiViewport = DFGraphics.createWindow("Middle Manager of JUSTICE", viewportSizeX, viewportSizeY, Renderer.resizeViewports)    
    Renderer.tCurScreenInfo = Renderer.computeScreenInfo(gameViewport.sizeX, gameViewport.sizeY)
    Renderer.gameViewport = gameViewport
    Renderer.uiViewport = uiViewport

    Renderer.scaleScreen(gameViewport.sizeX, gameViewport.sizeY)

	-- Setup camera
	rCamera = MOAICamera2D.new ()
	rCamera:setLoc( 0, 0, 0 )
    rCamera:setScl( nCameraBaseScaleX, nCameraBaseScaleY )

	-- Setup new layers
	tLayers.LayerRefs[1] = tLayers.Tiles
	tLayers.Tiles.RenderLayer = MOAILayer2D.new()
	tLayers.Tiles.RenderLayer:setViewport(gameViewport)
	tLayers.Tiles.RenderLayer:setCamera (rCamera)
    tLayers.Tiles.RenderLayer:setSortMode(MOAILayer2D.SORT_Y_DESCENDING)
	MOAISim.pushRenderPass(tLayers.Tiles.RenderLayer)
	
	tLayers.Tiles.Partition = MOAIPartition.new ()
	tLayers.Tiles.RenderLayer:setPartition(tLayers.Tiles.Partition)
	
	tLayers.LayerRefs[2] = tLayers.Gameplay
	tLayers.Gameplay.RenderLayer = MOAILayer2D.new()
	tLayers.Gameplay.RenderLayer:setViewport(gameViewport)
	tLayers.Gameplay.RenderLayer:setCamera (rCamera)
    --tLayers.Gameplay.RenderLayer:setSortMode(MOAILayer2D.SORT_VECTOR_ASCENDING)
	MOAISim.pushRenderPass(tLayers.Gameplay.RenderLayer)
	
	tLayers.Gameplay.Partition = MOAIPartition.new ()
	tLayers.Gameplay.RenderLayer:setPartition(tLayers.Gameplay.Partition)
		
	tLayers.LayerRefs[3] = tLayers.GameplayOverlay
	tLayers.GameplayOverlay.RenderLayer = MOAILayer2D.new()
	tLayers.GameplayOverlay.RenderLayer:setViewport(gameViewport)
	tLayers.GameplayOverlay.RenderLayer:setCamera (rCamera)
	MOAISim.pushRenderPass(tLayers.GameplayOverlay.RenderLayer)
	
	tLayers.GameplayOverlay.Partition = MOAIPartition.new ()
	tLayers.GameplayOverlay.RenderLayer:setPartition(tLayers.GameplayOverlay.Partition)

	local rEventCamera = MOAICamera2D.new ()
	rEventCamera:setLoc( 0, 0, 0 )
	rEventCamera:setScl(1, 1)

	tLayers.LayerRefs[4] = tLayers.EventBG
	tLayers.EventBG.RenderLayer = MOAILayer2D.new()
	tLayers.EventBG.RenderLayer:setViewport(gameViewport)
	tLayers.EventBG.RenderLayer:setCamera (rEventCamera)
	MOAISim.pushRenderPass(tLayers.EventBG.RenderLayer)
	
	tLayers.EventBG.Partition = MOAIPartition.new ()
	tLayers.EventBG.RenderLayer:setPartition(tLayers.EventBG.Partition)

	tLayers.LayerRefs[5] = tLayers.EventOverlay
	tLayers.EventOverlay.RenderLayer = MOAILayer2D.new()
	tLayers.EventOverlay.RenderLayer:setViewport(gameViewport)
	tLayers.EventOverlay.RenderLayer:setCamera (rEventCamera)
    tLayers.EventOverlay.RenderLayer:setSortMode(MOAILayer2D.SORT_Y_DESCENDING)
	MOAISim.pushRenderPass(tLayers.EventOverlay.RenderLayer)
	
	tLayers.EventOverlay.Partition = MOAIPartition.new ()
	tLayers.EventOverlay.RenderLayer:setPartition(tLayers.EventOverlay.Partition)	

	tLayers.LayerRefs[6] = tLayers.EventFX
	tLayers.EventFX.RenderLayer = MOAILayer2D.new()
	tLayers.EventFX.RenderLayer:setViewport(gameViewport)
	tLayers.EventFX.RenderLayer:setCamera (rEventCamera)
    tLayers.EventFX.RenderLayer:setSortMode(MOAILayer2D.SORT_Y_DESCENDING)
	MOAISim.pushRenderPass(tLayers.EventFX.RenderLayer)
	
	tLayers.EventFX.Partition = MOAIPartition.new ()
	tLayers.EventFX.RenderLayer:setPartition(tLayers.EventFX.Partition)	

	tLayers.LayerRefs[7] = tLayers.UI
	tLayers.UI.RenderLayer = MOAILayer2D.new()
	tLayers.UI.RenderLayer:setViewport(uiViewport)
	MOAISim.pushRenderPass(tLayers.UI.RenderLayer)

	tLayers.UI.Partition = MOAIPartition.new ()
	tLayers.UI.RenderLayer:setPartition(tLayers.UI.Partition)

    tLayers.LayerRefs[8] = tLayers.UIOverlay
	tLayers.UIOverlay.RenderLayer = MOAILayer2D.new()
	tLayers.UIOverlay.RenderLayer:setViewport(uiViewport)
	MOAISim.pushRenderPass(tLayers.UIOverlay.RenderLayer)

    tLayers.UIOverlay.Partition = MOAIPartition.new ()
	tLayers.UIOverlay.RenderLayer:setPartition(tLayers.UIOverlay.Partition)

    tLayers.LayerRefs[9] = tLayers.ProfileOverlay
	tLayers.ProfileOverlay.RenderLayer = MOAILayer2D.new()

    Renderer.rProfileViewportWindow = MOAIViewport.new()
    Renderer.rProfileViewportWindow:setSize ( gameViewport.sizeX, gameViewport.sizeY )
    Renderer.rProfileViewportWindow:setScale ( gameViewport.sizeX, -gameViewport.sizeY )
    Renderer.rProfileViewportWindow:setOffset ( -1, 1 )
	tLayers.ProfileOverlay.RenderLayer:setViewport(Renderer.rProfileViewportWindow)
	MOAISim.pushRenderPass(tLayers.ProfileOverlay.RenderLayer)

    tLayers.ProfileOverlay.Partition = MOAIPartition.new ()
	tLayers.ProfileOverlay.RenderLayer:setPartition(tLayers.ProfileOverlay.Partition)

end

function Renderer.ShowBase(bShow)
    local tLayers = nil
    if bShow then
        tLayers =
        {
           Renderer.getLayer('Tiles').RenderLayer,
           Renderer.getLayer('Gameplay').RenderLayer,
           Renderer.getLayer('GameplayOverlay').RenderLayer,
           Renderer.getLayer('EventBG').RenderLayer,
           Renderer.getLayer('EventOverlay').RenderLayer,
           Renderer.getLayer('EventFX').RenderLayer,
           Renderer.getLayer('UI').RenderLayer,
           Renderer.getLayer('UIOverlay').RenderLayer,
           Renderer.getLayer('ProfileOverlay').RenderLayer           
        }        
    else
        tLayers =
        {
           Renderer.getLayer('EventBG').RenderLayer,
           Renderer.getLayer('EventOverlay').RenderLayer,
           Renderer.getLayer('EventFX').RenderLayer,
           Renderer.getLayer('UI').RenderLayer,
           Renderer.getLayer('UIOverlay').RenderLayer,
           Renderer.getLayer('ProfileOverlay').RenderLayer            
        }
    end
    if tLayers then
        MOAIRenderMgr.setRenderTable(tLayers)
    end
end

function Renderer.getLayer(layerName)

	if not layerName then
		return nil
	end

    return tLayers[layerName]
end

function Renderer.getNumLayers()
	return #tLayers.LayerRefs
end

function Renderer.getLayerByIndex(index)
	return tLayers.LayerRefs[index]
end

function Renderer.getRenderLayer(layerName)

	local rLayer = Renderer.getLayer(layerName)
	if rLayer then
		return rLayer.RenderLayer
	end
	
end

function Renderer.clearLayers()
    for layerName, rLayer in pairs(tLayers.LayerRefs) do
        rLayer.RenderLayer:clear()
    end
end

function Renderer.shutdown()
    Renderer.clearLayers()
end

function Renderer.getPartition(layerName)

	local rLayer = getLayer(layerName)
	if rLayer then
		return rLayer.Partition
	end
	
end

function Renderer.setCameraParent(rParent, nXOffset, nYOffset)
    if rCamera then
        if rParent then
            rCamera:setParent(rParent)
            rCamera.rAttachedParent = rParent
            rCamera:setLoc(nXOffset, nYOffset)
        else            
            if rCamera.rAttachedParent then
                local x, y = rCamera:getLoc()
                local parentX, parentY = rCamera.rAttachedParent:getLoc()                
                rCamera:setParent(nil)
                rCamera:setLoc(parentX + x, parentY + y)
            end
        end
    end
end

function Renderer.setCameraLoc(x, y, duration)
	if duration and duration > 0 then
		if rCameraLocInterpolator then
			rCameraLocInterpolator:stop()
		end
		local ox, oy = rCamera:getLoc()
		rCameraLocInterpolator = rCamera:moveLoc( x - ox, y - oy, duration )
	else
        -- clamp camera values to screen bounds
        if g_CurrentMap then
            local topX, topY, downX, downY, leftX, leftY, rightX, rightY = g_CurrentMap:getMapExtents()

			local currentZoomScale = Renderer:getCurZoomScl()
            local tan27point5 = .52
            local tan62point5 = 1.92
			local adjustValueX = (Renderer.getUIHalfWidth() + (Renderer.getUIHalfHeight() * tan62point5)) * currentZoomScale
			local adjustValueY = (Renderer.getUIHalfHeight() + (Renderer.getUIHalfWidth() * tan27point5)) * currentZoomScale
            leftX = leftX + adjustValueX
			rightX = rightX - adjustValueX
            topY = topY - adjustValueY
			downY = downY + adjustValueY

            x = math.max( leftX, math.min(x, rightX) )
            y = math.max( downY, math.min(y, topY) )

			-- angle clamp on top-left
			if y > leftY and x < topX then
				local slope = (topY - leftY) / (topX - leftX)
				local allowedY = (slope * (x - leftX)) + leftY
				y = math.min( y, allowedY )
			end

			-- angle clamp on top-right
			if y > rightY and x > topX then
				local slope = (rightY - topY) / (rightX - topX)
				local allowedY = (slope * (x - topX)) + topY
				y = math.min( y, allowedY )
			end

			-- angle clamp on bottom-left
			if y < leftY and x < downX then
				local slope = (downY - leftY) / (downX - leftX)
				local allowedY = (slope * (x - leftX)) + leftY
				y = math.max( y, allowedY )
			end
            
			-- angle clamp on bottom-right
			if y < rightY and x > downX then
				local slope = (rightY - downY) / (rightX - downX)
				local allowedY = (slope * (x - downX)) + downY
				y = math.max( y, allowedY )
			end            
        end
        
		rCamera:setLoc( x, y )
	end
end

function Renderer.moveCamera(dx, dy, duration, inScreenSpace)
	
	if inScreenSpace then
		local sx, sy = rCamera:getScl()
		dx = dx * sx
		dy = dy * sy
	end
	
	local x, y = rCamera:getLoc()
	Renderer.setCameraLoc( x + dx, y + dy, duration )
end

function Renderer.setCameraScl(x, y, duration)
    if x < kMIN_ZOOM then
        x = kMIN_ZOOM
        y = kMIN_ZOOM
    elseif x > kMAX_ZOOM then
        x = kMAX_ZOOM
        y = kMAX_ZOOM
    end
    
    nCurXZoomScale = x
    nCurYZoomScale = y
    
	if duration and duration > 0 then
		if rCameraSclInterpolator then
			rCameraSclInterpolator:stop()
		end
		local ox, oy = rCamera:getScl()
		rCameraSclInterpolator = rCamera:moveScl( (x * nCameraBaseScaleX) - ox, (y * nCameraBaseScaleY) - oy, duration )
	else
		rCamera:setScl( x * nCameraBaseScaleX, y * nCameraBaseScaleY )
	end

	-- set the camera at the current position to make sure the bounds are clamped with the new scale
	local x, y = rCamera:getLoc()
	Renderer.setCameraLoc( x, y )
end

function Renderer.getCurZoomScl()
    return nCurXZoomScale, nCurYZoomScale
end

function Renderer.getDisplayScl()
    return Renderer.nDisplayScaleX, Renderer.nDisplayScaleY
end

function Renderer.getViewportSize()
    local viewportSizeX = 960
    local viewportSizeY = 640
	-- Don't fix this "bug"!
    if Renderer.gameViewport then
        return Renderer.gameViewport.sizeX, Renderer.gameViewport.sizeY
    else
        return viewportSizeX, viewportSizeY
    end
end

function Renderer.getUIWidth()
    return nScaledScreenWidth
end

function Renderer.getUIHalfWidth()
    return nScaledHalfWidth 
end

function Renderer.getUIHeight()
    return nScaledScreenHeight
end

function Renderer.getUIHalfHeight()
    return nScaledHalfHeight 
end

function Renderer.getTextureSizeSuffix()
    if Renderer.tCurScreenInfo then
        return Renderer.tCurScreenInfo.sTextureSizeSuffix
    end
    return ''
end

function Renderer.getTextureSizeFactor()
    if Renderer.tCurScreenInfo then
        return Renderer.tCurScreenInfo.nTextureSizeFactor
    end
    return 1
end

function Renderer.getCurScreenInfo()
    return Renderer.tCurScreenInfo
end

return Renderer
