local SpriteSheet = {}
local g_tTileDecks = { }
local g_tSpriteDecks = { } -- the non tile version, we will collpase into one thing later

local Renderer = require('Renderer')
local util = require('DFUtil')
local DFFile = require('DFCommon.File')
local DFGraphics = require('DFCommon.Graphics')

function SpriteSheet:new(texturePackerDefFileName, sAnimDefFileName, centerToBaseX, centerToBaseY, sAlignX, sAlignY, nScaleX, nScaleY, tTextureInfoOverride, loadAsync)
	-- CONSTRUCTOR
	local rSprite = nil
    if texturePackerDefFileName then
        local sSuffix = Renderer.getTextureSizeSuffix()
        if tTextureInfoOverride and tTextureInfoOverride.sTextureSizeSuffix then
            sSuffix = tTextureInfoOverride.sTextureSizeSuffix
        end
        texturePackerDefFileName = texturePackerDefFileName..sSuffix        
        local rDeck = DFGraphics.loadSpriteSheet(texturePackerDefFileName, loadAsync)
        if rDeck then
            rSprite = MOAIProp2D.new()
            rSprite.__typename = "SpriteSheet"
            rSprite:setDeck(rDeck)
            rSprite.names = rDeck.names
            sAlignX = sAlignX or 'center'
            sAlignY = sAlignY or 'bottom'
            rSprite.centerToBaseX = centerToBaseX
            rSprite.centerToBaseY = centerToBaseY
            if nScaleX and nScaleY then
                for i, rect in ipairs(rDeck.rects) do
                    rect.x0 = rect.x0 * nScaleX
                    rect.y0 = rect.y0 * nScaleY
                    rect.x1 = rect.x1 * nScaleX
                    rect.y1 = rect.y1 * nScaleY
                    rect.width = rect.width * nScaleX
                    rect.height = rect.height * nScaleY
                    rect.origWidth = rect.origWidth * nScaleX
                    rect.origHeight = rect.origHeight * nScaleY
                    rDeck:setRect( i, rect.x0, rect.y0, rect.x1, rect.y1)
                end
            end
            local nSizeFactor = Renderer.getTextureSizeFactor()            
            if tTextureInfoOverride and tTextureInfoOverride.nTextureSizeFactor then
                nSizeFactor = tTextureInfoOverride.nTextureSizeFactor
            end
            for sName, data in pairs(rDeck.names) do
                DFGraphics.alignSprite(rDeck, sName, sAlignX, sAlignY, nSizeFactor, nSizeFactor)                
            end

            -- construct the anims
            if sAnimDefFileName then
                local rAnimDef = util.dofile(DFFile.getDataPath(sAnimDefFileName))
                if rAnimDef then
                    local tAnimations = { }
                    for sAnimName, animInfo in pairs(rAnimDef.tAnimInfos) do
                        local rSpriteAnim = { }
                        rSpriteAnim.tAnimDef = animInfo                               
                        local tAnimFrameInfos = {}
                        local nNumFrames = 0
                        if animInfo.sAnimNameBase then
                            -- load up the chain of anims
                            -- we assume starts 0000 until the end
                            -- e.g. 'Blink.swf' will look at Blink.swf/0000 and on
                            -- this is dependent on the flash exporting process
                            local nIndex = animInfo.nAnimStartIndex or 0
                            while true do
                                local nIndexString = ""
                                if nIndex < 1000 then
                                    nIndexString = nIndexString .. "0"
                                end
                                if nIndex < 100 then
                                    nIndexString = nIndexString .. "0"
                                end
                                if nIndex < 10 then
                                    nIndexString = nIndexString .. "0"
                                end
                                nIndexString = nIndexString .. tostring(nIndex)
                                local nDeckIndex = rDeck.names[animInfo.sAnimNameBase..nIndexString]
                                if nDeckIndex then
                                    local rAnimFrameInfo = {}
                                    rAnimFrameInfo.nDeckIndex = nDeckIndex
                                    table.insert(tAnimFrameInfos, rAnimFrameInfo)
                                    nIndex = nIndex + 1
                                    if animInfo.nAnimEndIndex then
                                        if nIndex > animInfo.nAnimEndIndex then
                                            break
                                        end
                                    end                                       
                                else
                                    break
                                end
                            end
                            nNumFrames = util.countitems(tAnimFrameInfos)
                        elseif animInfo.tAnimFrames then
                            local nIndex = 0
                            for i, rCustomFrame in ipairs(animInfo.tAnimFrames) do
                                local nDeckIndex = rDeck.names[rCustomFrame.sAnimFrameName]
                                if nDeckIndex then
                                    local rAnimFrameInfo = {}
                                    rAnimFrameInfo.nDeckIndex = nDeckIndex
                                    rAnimFrameInfo.nSecondsPerFrame = rCustomFrame.nSecondsPerFrame
                                    table.insert(tAnimFrameInfos, rAnimFrameInfo)
                                    nIndex = nIndex + 1
                                    nNumFrames = nIndex
                                end
                            end
                        end
                        if table.getn(tAnimFrameInfos) > 0 then                            
                            local rCurve = MOAIAnimCurve.new()
                            rCurve:reserveKeys(nNumFrames)
                            
                            local nTime = 0
                            for nFrameNum, rFrameInfo in ipairs(tAnimFrameInfos) do
                                rCurve:setKey(nFrameNum, nTime, rFrameInfo.nDeckIndex, MOAIEaseType.FLAT )
                                local nSecondsPerFrame = rFrameInfo.nSecondsPerFrame or animInfo.nSecondsPerFrame or  rAnimDef.nSecondsPerFrame
                                nTime = nTime + nSecondsPerFrame / rAnimDef.nAnimationSpeed
                            end
                            rSpriteAnim.rCurve = rCurve
                            
                            -- Create animation controller
                            local rAnimController = MOAIAnim:new ()
                            rAnimController:reserveLinks(1)
                            rAnimController:setLink(1, rCurve, rSprite, MOAIProp2D.ATTR_INDEX)
                            
                            if animInfo.repeatMode == "loop" then
                                rAnimController:setMode ( MOAITimer.LOOP )
                            end
                            if animInfo.repeatMode == "pingpong" then
                                rAnimController:setMode ( MOAITimer.PING_PONG )
                            end
                            
                            rSpriteAnim.rAnimController = rAnimController
                            table.insert(tAnimations, rSpriteAnim)
                        end
                    end
                    rSprite.tAnimations = tAnimations
                    rSprite.sCurrentAnimation = ""
                    rSprite.rAnimDef = rAnimDef
                end
            end
            rSprite.rDeck = rDeck
        end                            
    end
	
	if not rSprite then
		return nil
	end

    rSprite.tTextureInfoOverride = tTextureInfoOverride
    rSprite.texturePackerDefFileName = texturePackerDefFileName
	
	function rSprite:getAnimation(animName)
        if rSprite.tAnimations then
            local numAnims = #rSprite.tAnimations
            for i=1,numAnims do
                local anim = rSprite.tAnimations[i]
                if anim.tAnimDef.name == animName then
                    return anim
                end
            end
		end
		return nil
	end

    function rSprite:getCurrentAnimationName()
        return rSprite.sCurrentAnimation
    end
	
	function rSprite:playAnimation(animName, flipX, flipY, blocking)
		local anim = rSprite:getAnimation(animName)
		if anim then
            -- Same animation?
            -- Stop current animation...
            rSprite:stopAnimation(rSprite.sCurrentAnimation)
            
            -- Set the scale
            local sx, sy = self:getScl()
            if flipX then
                sx = -math.abs(sx)
            else
                sx = math.abs(sx)
            end
            if flipY then
                sy = -math.abs(sy)
            else
                sy = math.abs(sy)
            end
            rSprite:setScl(sx, sy)
            
            -- Before starting the current one
            rSprite.sCurrentAnimation = animName
			
			-- Play effect?
			if anim.tAnimDef.startEffect then
				local x, y = rSprite:getLoc()
				local priority = rSprite:getPriority()
				local effectInstance = EffectInstance:new(anim.tAnimDef.startEffect, "Gameplay", x, y, priority)
				effectInstance:start()
			end
            
			anim.rAnimController:start()
			
            local duration = rSprite:getDuration()
			if blocking then
				util.sleep(duration)
				while not rSprite:isDone(animName) do
					util.sleep(0.01)
				end
			end
			return duration
		else
			print("Can't play unknown animation: " .. animName)
		end
        return 0
	end
	
    function rSprite:setSpriteAlignment(sName, sAlignX, sAlignY)
        local nSizeFactor = Renderer.getTextureSizeFactor()            
        if rSprite.tTextureInfoOverride and rSprite.tTextureInfoOverride.nTextureSizeFactor then
            nSizeFactor = rSprite.tTextureInfoOverride.nTextureSizeFactor
        end
        if rSprite.rDeck and sName and sAlignX and sAlignY then
            DFGraphics.alignSprite(rSprite.rDeck, sName, sAlignX, sAlignY, nSizeFactor, nSizeFactor)
        end
    end
 
    function rSprite:setSpriteIndex(sIndexKey, flipX, flipY)
        if rSprite.names then
            local nIndex = rSprite.names[sIndexKey]
            if nIndex then
                rSprite:setIndex(nIndex)
            end
        end
		local sx, sy = self:getScl()
		if flipX then
			sx = -math.abs(sx)
        else
            sx = math.abs(sx)
		end
		if flipY then
			sy = -math.abs(sy)
        else
            sy = math.abs(sy)
		end
		rSprite:setScl(sx, sy)		 
    end

	function rSprite:stopAnimation(animName)
		rSprite.sCurrentAnimation = ""
		local anim = rSprite:getAnimation(animName)
		if anim then
			anim.rAnimController:stop()
		end
	end

    function rSprite:stopCurrentAnimation()
		rSprite:stopAnimation(rSprite.sCurrentAnimation)
    end
	
	function rSprite:getRootPosition(centerX, centerY)
		return centerX + rSprite.centerToBaseX, centerY + rSprite.centerToBaseY
	end
	
	function rSprite:getCenterPosition(rootX, rootY)
		return rootX - rSprite.centerToBaseX, rootY - rSprite.centerToBaseY
	end
	
	function rSprite:getDuration()
		local anim = rSprite:getAnimation(rSprite.sCurrentAnimation)
		if anim then
			local length = anim.rAnimController:getLength()
			return length
		end
		return 0
	end
	
    function rSprite:getAttackAnims()
        if rSprite.rAnimDef then
            return rSprite.rAnimDef.tAttackAnims
        else
            return nil
        end
    end

	function rSprite:isDone(animName)
        if animName and animName ~= rSprite.sCurrentAnimation then
            return true
        end
		local anim = rSprite:getAnimation(rSprite.sCurrentAnimation)
		if anim then
			if anim.repeatMode ~= "loop" and anim.repeatMode ~= "pingpong" then
				local t = anim.rAnimController:getTime()
				local length = rSprite:getDuration()
				--print(tostring(t) .. ' >= ' .. tostring(length))
				return t >= length
			else
				return false
			end
		end
		return true
	end

    function rSprite:unloadSpriteSheet()
--        print("RJI$J "..self.texturePackerDefFileName)
        DFGraphics.unloadSpriteSheet(self.texturePackerDefFileName)
    end
    
	return rSprite
end

return SpriteSheet