local Renderer = require('Renderer')
local DFTiled = require('DFCommon.Tiled')
local DFFile = require('DFCommon.File')
local DFGraphics = require('DFCommon.Graphics')
local DFAchievement = require('DFCommon.Achievement')
local util = require('DFUtil')
local Button = require('ui/Button')
local PropTouchManager = require('PropTouchManager')

local TileMap = {}
local Tile = {}
local Room = {}
local Prop = {}
local BaseTile = {}

local bDebugShowTileCoords = false
local bDebugShowPropTouchButtons = false

local kPLAYER_TO_TILE_PRI_INC = 1.0
local kPROP_TO_BG_PRI_INC = 0.3
local kWALL_TO_BG_PRI_INC = 0.2
local kTILE_TO_BG_PRI_INC = 0.1

local kPRI_OFFSET = 1000

local emptyRoomType = "emptyroom"
local tInitialMap =
{
    hub = "hublvl1",
    nMaxCol = 52,
    nMaxRow = 70,
    nTileWidth = 64,
    nTileHeight = 32,
}

-- suffixes are added on processing
local tSpriteSheetToScreenInfo =
{
Walls_MM = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Canteen = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Cubes = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Cubicle = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Bedroom = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Gym = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_GymAlt = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Hub = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Walls_Lab = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Tiles_BaseInt = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Tiles_BaseExt = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Tiles_BaseInt_Lvl2 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
Tiles_BaseInt_Lvl3 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.LARGE, LARGE = Renderer.tScreenInfo.LARGE },
BaseProps = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
BaseProps2 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
BaseProps3 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
BaseProps4 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
BaseProps5 = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
HolidayProps = { SMALL = Renderer.tScreenInfo.MED, MED = Renderer.tScreenInfo.MED, LARGE = Renderer.tScreenInfo.LARGE },
}
TileMap.tTilesetToSpriteSheets =
{
Walls_MM = 'Walls_MM',
Walls_MM_Lvl2 = 'Walls_MM',
Walls_MM_Lvl3 = 'Walls_MM',
Walls_Canteen = 'Walls_Canteen',
Walls_Canteen_Lvl2 = 'Walls_Canteen',
Walls_Canteen_Lvl3 = 'Walls_Canteen',
Walls_Cubes = 'Walls_Cubes',
Walls_Cubes_Lvl2 = 'Walls_Cubes',
Walls_Cubes_Lvl3 = 'Walls_Cubes',
Walls_Cubicle = 'Walls_Cubicle',
Walls_Cubicle_Lvl2 = 'Walls_Cubicle',
Walls_Cubicle_Lvl3 = 'Walls_Cubicle',
Walls_Bedroom = 'Walls_Bedroom',
Walls_Bedroom_Lvl2 = 'Walls_Bedroom',
Walls_Bedroom_Lvl3 = 'Walls_Bedroom',
Walls_Gym = 'Walls_Gym',
Walls_Gym_Lvl2 = 'Walls_Gym',
Walls_Gym_Lvl3 = 'Walls_Gym',
Walls_GymAlt = 'Walls_GymAlt',
Walls_GymAlt_Lvl2 = 'Walls_GymAlt',
Walls_GymAlt_Lvl3 = 'Walls_GymAlt',
Walls_Hub = 'Walls_Hub',
Walls_Hub_Lvl2 = 'Walls_Hub',
Walls_Hub_Lvl3 = 'Walls_Hub',
Walls_Lab = 'Walls_Lab',
Walls_Lab_Lvl2 = 'Walls_Lab',
Walls_Lab_Lvl3 = 'Walls_Lab',
Tiles_BaseInt = 'Tiles_BaseInt',
Tiles_BaseInt_Lvl2 = 'Tiles_BaseInt_Lvl2',
Tiles_BaseInt_Lvl3 = 'Tiles_BaseInt_Lvl3',

MM_BulletinBoard = 'BaseProps',
MM_ChairGreen = 'BaseProps2',
MM_Desk = 'BaseProps2',
MM_Door = 'BaseProps5',
MM_EmployeePic02 = 'BaseProps2',
MM_EmployeePic = 'BaseProps2',
MM_FilingCab02 = 'BaseProps2',
MM_FilingCabinet = 'BaseProps2',
MM_GarbageCan = 'BaseProps2',
MM_Map_Lvl2 = 'BaseProps',
MM_Map_Lvl3 = 'BaseProps',
MM_Map = 'BaseProps2',
MM_Monitor_Lvl2 = 'BaseProps',
MM_Monitor_Lvl3 = 'BaseProps',
MM_Monitor = 'BaseProps2',
MM_NameBoard02_Lvl2 = 'BaseProps',
MM_NameBoard02_Lvl3 = 'BaseProps',
MM_NameBoard02 = 'BaseProps2',
MM_NameBoard_Lvl2 = 'BaseProps',
MM_NameBoard_Lvl3 = 'BaseProps',
MM_NameBoard = 'BaseProps2',
MM_PhoneTable = 'BaseProps2',
MM_Rug = 'BaseProps2',
MM_WaterCooler = 'BaseProps2',
MM_Window2 = 'BaseProps2',
MM_Window = 'BaseProps2',
MM_CoinDoubler = 'BaseProps2',
MM_Juicer = 'BaseProps3',
MM_Yak = 'BaseProps3',
MM_CoffeeMaker = 'BaseProps4',
MM_Crystal = 'BaseProps5',
MM_Desk2 = 'BaseProps5',

Cubes_Access02 = 'BaseProps',
Cubes_Access03 = 'BaseProps',
Cubes_Access04 = 'BaseProps',
Cubes_Access = 'BaseProps',
Cubes_BulletinBoard02 = 'BaseProps',
Cubes_BulletinBoard = 'BaseProps',
Cubes_Calendar02 = 'BaseProps',
Cubes_Calendar = 'BaseProps',
Cubes_ChairBrown = 'BaseProps',
Cubes_Desk = 'BaseProps',
Cubes_FilingCab2 = 'BaseProps',
Cubes_FilingCab = 'BaseProps',
Cubes_Flyer02 = 'BaseProps',
Cubes_Flyer03 = 'BaseProps',
Cubes_Flyer04 = 'BaseProps',
Cubes_Flyer05 = 'BaseProps',
Cubes_Flyer = 'BaseProps',
Cubes_GarbageCan = 'BaseProps',
Cubes_Monitor = 'BaseProps',
Cubes_Monitor_Lvl2 = 'BaseProps',
Cubes_Monitor_Lvl3 = 'BaseProps',
Cubes_PaperPile = 'BaseProps',
Cubes_Photo02 = 'BaseProps',
Cubes_Photo03 = 'BaseProps',
Cubes_Photo04 = 'BaseProps',
Cubes_Photo = 'BaseProps',

Canteen_CafeTable = 'BaseProps',
Canteen_Couch = 'BaseProps',
Canteen_Couch_Lvl2 = 'BaseProps',
Canteen_Couch_Lvl3 = 'BaseProps',
Canteen_TV = 'BaseProps5',
Canteen_TV_Lvl2 = 'BaseProps5',
Canteen_TV_Lvl3 = 'BaseProps5',
Canteen_TV_Med = 'BaseProps5',
Canteen_VendingMachine = 'BaseProps',
Canteen_WaterCooler = 'BaseProps',
Canteen_Window = 'BaseProps',
Canteen_VendingMachine2 = 'BaseProps',
Canteen_Bingo = 'BaseProps',

Bedroom_FirstAid = 'BaseProps',
Bedroom_HangIn = 'BaseProps',
Bedroom_Lamp = 'BaseProps',
Bedroom_Nightlight = 'BaseProps',
Bedroom_Rainbow = 'BaseProps',
Bedroom_Bed = 'BaseProps',
Bedroom_Bed_Lvl2 = 'BaseProps',
Bedroom_Bed_Lvl3 = 'BaseProps',

EmptyRoom_Construction = 'BaseProps',
EmptyRoom_LineCorner2 = 'BaseProps',
EmptyRoom_LineCorner3 = 'BaseProps',
EmptyRoom_LineCorner4 = 'BaseProps',
EmptyRoom_LineCorner = 'BaseProps',
EmptyRoom_LineH = 'BaseProps',
EmptyRoom_LineH2 = 'BaseProps',
EmptyRoom_LineV2 = 'BaseProps',
EmptyRoom_LineV = 'BaseProps',

Gym_Logo = 'BaseProps',
Gym_Mat = 'BaseProps',
Gym_PunchingBag = 'BaseProps',
Gym_PunchingBag_Lvl2 = 'BaseProps',
Gym_PunchingBag_Lvl3 = 'BaseProps',
Gym_Text2 = 'BaseProps',
Gym_Text = 'BaseProps',
Gym_Treadmill = 'BaseProps2',
Gym_Treadmill_Lvl2 = 'BaseProps2',
Gym_Treadmill_Lvl3 = 'BaseProps2',
Gym_3balls_Lvl3 = 'BaseProps4',

Door = 'BaseProps',
DoorOpen = 'BaseProps',
Hub_BulletinBoard = 'BaseProps2',
Hub_Cactus = 'BaseProps2',
Hub_Chart = 'BaseProps2',
Hub_Flagpole = 'BaseProps2',
Hub_Portrait = 'BaseProps2',
Hub_Congratulations = 'BaseProps3',
Hub_Decor = 'BaseProps3',
Hub_Decor2 = 'BaseProps3',
Hub_Balloon = 'BaseProps3',

Lab_Anatomy = 'BaseProps2',
Lab_Bookshelf = 'BaseProps2',
Lab_Chalkboard2 = 'BaseProps2',
Lab_Chalkboard = 'BaseProps',
Lab_Chart2 = 'BaseProps2',
Lab_Chart = 'BaseProps',
Lab_Desk = 'BaseProps2',
Lab_Desk_Lvl2 = 'BaseProps2',
Lab_Desk_Lvl3 = 'BaseProps2',
Lab_Poster = 'BaseProps2',

Level_BrickHole01 = 'BaseProps2',
Level_BrickHole02 = 'BaseProps2',
Level_BrickHole03 = 'BaseProps2',
Level_BrickHole04 = 'BaseProps2',
Level_BrickHole05 = 'BaseProps2',
Level_BrickHole06 = 'BaseProps2',

Level_FloorHole01 = 'BaseProps2',
Level_FloorHole02 = 'BaseProps2',
Level_FloorHole03 = 'BaseProps2',

Tiles_BaseExt = 'Tiles_BaseExt',
Ext_Daisies01 = 'BaseProps3',
Ext_Daisies02 = 'BaseProps3',
Ext_Daisies03 = 'BaseProps3',
Ext_Fireplug = 'BaseProps3',
Ext_Grass01 = 'BaseProps3',
Ext_Grass02 = 'BaseProps3',
Ext_Grass03 = 'BaseProps3',
Ext_Grass04 = 'BaseProps3',
Ext_Grass05 = 'BaseProps3',
Ext_GrassBk01 = 'BaseProps3',
Ext_GrassBk02 = 'BaseProps3',
Ext_GrassBk03 = 'BaseProps3',
Ext_GrassBk04 = 'BaseProps3',
Ext_GrassBk05 = 'BaseProps3',
Ext_Mailbox = 'BaseProps3',
Ext_Manhole = 'BaseProps3',
Ext_Newsstand = 'BaseProps3',
Ext_SewerGrate = 'BaseProps3',
Ext_Stopsign = 'BaseProps3',
Ext_Streetlight = 'BaseProps3',
Ext_Tree2 = 'BaseProps3',
Ext_Tree = 'BaseProps3',
Ext_Car = 'BaseProps3',
Ext_CarPolice = 'BaseProps3',
Ext_Sign = 'BaseProps3',
Ext_ParkingSign = 'BaseProps3',
Ext_Cow = 'BaseProps3',
Ext_Parking01 = 'BaseProps',
Ext_Parking02 = 'BaseProps',
Ext_Parking03 = 'BaseProps',
Ext_Parking04 = 'BaseProps',
Ext_Parking05 = 'BaseProps',
Ext_Parking06 = 'BaseProps',
Ext_Parking07 = 'BaseProps',
Ext_Parking08 = 'BaseProps',
Ext_ParkingMM = 'BaseProps3',

GymAlt_Dummy_Lvl1 = 'BaseProps',
GymAlt_Dummy_Lvl2 = 'BaseProps',
GymAlt_Dummy_Lvl3 = 'BaseProps',
GymAlt_Poster = 'BaseProps',
GymAlt_Poster2 = 'BaseProps',

GymAlt_Bats = 'BaseProps3',
GymAlt_Clubs = 'BaseProps3',
GymAlt_Scorchmark = 'BaseProps3',
GymAlt_Sign = 'BaseProps3',
GymAlt_Poster3 = 'BaseProps3',
Gym_Ball = 'BaseProps3',
Gym_Chart = 'BaseProps3',
Gym_Mat2 = 'BaseProps3',
Gym_Poster = 'BaseProps3',
Bedroom_Bookshelf = 'BaseProps3',
Bedroom_Table = 'BaseProps3',
Canteen_Penant = 'BaseProps3',
Canteen_Poster2 = 'BaseProps3',
Canteen_Rug = 'BaseProps3',
Canteen_Window2 = 'BaseProps3',
Lab_BrainPoster = 'BaseProps3',

Bedroom_Bookshelf_Lvl1 = 'BaseProps4',
Bedroom_Box = 'BaseProps4',
Bedroom_Boxes = 'BaseProps4',
Bedroom_Drawers = 'BaseProps4',
Bedroom_Dresser = 'BaseProps4',
Bedroom_Lamp2 = 'BaseProps4',
Bedroom_Mirror = 'BaseProps4',
Bedroom_Seat = 'BaseProps4',
Bedroom_Seat_Lvl2 = 'BaseProps4',
Bedroom_Shelf = 'BaseProps4',
Bedroom_Shelf_Lvl2 = 'BaseProps4',
Bedroom_Shelf2 = 'BaseProps4',
Bedroom_Table_Lvl2 = 'BaseProps4',

GymAlt_Box_Lvl2 = 'BaseProps3',
GymAlt_BoxesA_Lvl1 = 'BaseProps3',
GymAlt_BoxesB_Lvl1 = 'BaseProps3',
GymAlt_Clubs_Lvl2 = 'BaseProps3',
GymAlt_Equipment_Lvl2 = 'BaseProps3',
GymAlt_Equipment_Lvl3 = 'BaseProps3',
GymAlt_Lumber_Lvl1 = 'BaseProps3',
GymAlt_Mats_Lvl2 = 'BaseProps4',
GymAlt_Mats_Lvl3 = 'BaseProps4',
GymAlt_Shelves_Lvl3 = 'BaseProps4',
GymAlt_ShelvesProps_Lvl3 = 'BaseProps4',

Gym_3Balls_Lvl3 = 'BaseProps4',
Gym_Dumbbells_Lvl2 = 'BaseProps4',
Gym_Dumbbells_Lvl3 = 'BaseProps4',
Gym_Frame = 'BaseProps4',
Gym_Weights = 'BaseProps',
Gym_WeightsA_Lvl2 = 'BaseProps4',
Gym_WeightsA_Lvl3 = 'BaseProps4',
Gym_WeightsB_Lvl2 = 'BaseProps4',
Gym_WeightsB_Lvl3 = 'BaseProps4',

Hub_Newspapers = 'BaseProps4',
Hub_Plant = 'BaseProps4',
Hub_Recycling = 'BaseProps4',
Hub_Seat = 'BaseProps4',
Hub_TrophyCase = 'BaseProps4',
Hub_CakeTable = 'BaseProps4',
Lab_Board_Lvl1 = 'BaseProps4',
Lab_Books_Lvl1 = 'BaseProps4',
Lab_BooksB_Lvl1 = 'BaseProps4',
Lab_Bookshelf_Lvl2 = 'BaseProps4',
Lab_Chair_Lvl3 = 'BaseProps4',
Lab_ScientistGown = 'BaseProps4',
Lab_Skeleton = 'BaseProps4',
Lab_Table_Lvl2 = 'BaseProps4',
Lab_Table_Lvl3 = 'BaseProps4',
Lab_Trash_Lvl1 = 'BaseProps4',
Bedroom_Bookshelf_Lvl2 = 'BaseProps4',
Gym_Weights_Lvl1 = 'BaseProps4',
MM_PencilCup = 'BaseProps4',
Canteen_Candy = 'BaseProps4',
Canteen_Fridge = 'BaseProps4',
Canteen_Plant = 'BaseProps4',
Canteen_Props = 'BaseProps4',
Canteen_Rug_Lvl3 = 'BaseProps4',
Canteen_Table = 'BaseProps4',
MM_LeaderBoard = 'BaseProps4',
Hub_2013Balloon = 'HolidayProps',
Hub_Decor3_NewYear = 'HolidayProps',
Hub_HappyNewYear = 'HolidayProps',
Hub_NewYearTable = 'HolidayProps',
Hub_ChristmasGiftboxA = 'HolidayProps',
Hub_ChristmasGiftboxB = 'HolidayProps',
Hub_ChristmasOrnament = 'HolidayProps',
Hub_ChristmasStockingA = 'HolidayProps',
Hub_ChristmasStockingB = 'HolidayProps',
Hub_ChristmasStockingC = 'HolidayProps',
Hub_ChristmasTree = 'HolidayProps',
Hub_Decor4_Christmas = 'HolidayProps',
Hub_Decor4_Valentine = 'HolidayProps',
Hub_Decor5_Valentine = 'HolidayProps',
Hub_Heart_Valentine = 'HolidayProps',
Hub_ValentineBalloon = 'HolidayProps',
Hub_ValentineDay = 'HolidayProps',
Hub_ValentineTable = 'HolidayProps',
Hub_PizzaTable = 'BaseProps4', 
Hub_WantedPoster = 'BaseProps4',
MM_Blender = 'BaseProps4',
MM_SuperiumTree = 'BaseProps4',
}

function BaseTile:new(nRow, nCol, nIndex, rMap) -- this could be a transform but using a table for simplicity
    Ob = {}
    Ob.x = 0
    Ob.y = 0
    Ob.rMap = rMap
    Ob.nCol = nCol
    Ob.nRow = nRow
    Ob.nIndex = nIndex
    Ob.rRoom = nil
    Ob.tOccupyingTiles = {}
    Ob.tOccupants = {}

    function Ob:addOccupant(rCharacter)
        if not rCharacter then
            return
        end
        for i, rChar in ipairs(self.tOccupants) do
            if rChar == rCharacter then
                return
            end
        end
        table.insert(self.tOccupants, rCharacter)
    end

    function Ob:removeOccupant(rCharacter)
        for i, rChar in ipairs(self.tOccupants) do
            if rChar == rCharacter then
                table.remove(self.tOccupants, i)
            end
        end        
    end

    function Ob:getOccupantPriority()
        local nMaxPri = 0
        for i, rChar in ipairs(self.tOccupants) do
            local nPri = rChar:getPriority()
            if nPri and nPri > nMaxPri then
                nMaxPri = nPri
            end           
        end
        return nMaxPri
    end
    
    function Ob:setOccupyingTile(rOccupyingTile)
        table.insert(self.tOccupyingTiles, rOccupyingTile)
    end

    function Ob:getOccupyingTiles()
        return self.tOccupyingTiles
    end

    function Ob:removeOccupyingTile(rOccupyingTile)
        for i, rTile in ipairs(self.tOccupyingTiles) do
            if rTile == rOccupyingTile then
                table.remove(self.tOccupyingTiles, i)
                break
            end
        end
    end

    function Ob:setLoc(x, y)
        self.x = x
        self.y = y
--[[
        self.rDebugText = g_GUI:CreateTextBox(8, MOAITextBox.CENTER_JUSTIFY)
        self.rDebugText:setColor(0, 0, 0)
        self.rDebugText:setString(tostring(nRow..', '..nCol))
        self.rDebugText:setLoc(x + self.rMap.nTileWidth / 2, y + self.rMap.nTileHeight / 2 + 10)
        self.rDebugText:setColor(1, 1, 1)
        Tile.rRenderLayer:insertProp(self.rDebugText)
        self.rDebugText:setPriority(100000)
]]--
    end

    function Ob:getLoc(x, y)
        return self.x, self.y
    end

    function Ob:getWorldCoords()
        local xPos, yPos = self:getLoc()
        local nParentX, nParentY = self.rMap:getLoc()
        xPos = xPos + nParentX
        yPos = yPos + nParentY
        local nWidth = rMap.nTileWidth
        local nHeight = rMap.nTileHeight
        local nScaleX, nScaleY = self.rMap:getScl()
        nWidth = nWidth * nScaleX
        nHeight = nHeight * nScaleY
        x = xPos + nWidth / 2
        y = yPos + nHeight / 2
        return x, y     
    end

    function Ob:getPriority()
        return self.nIndex
    end

    function Ob:getTilePriority()
        local nMaxPri = 0
        for i, rTile in ipairs(self.tOccupyingTiles) do
            local nPri = rTile:getTilePriority()
            if nPri > nMaxPri then
                nMaxPri = nPri
            end
        end
        return nMaxPri
    end

    function Ob:setRoom(rRoom)
        self.rRoom = rRoom
    end
    
    function Ob:getRoom()
        return self.rRoom
    end

    return Ob
end

function Tile:new(nRow, nCol, rMap)
    Ob = {}
    Ob.nRow = nRow
    Ob.nCol = nCol
    Ob.rMap = rMap

    Ob.tTiles = {}
    Ob.tInteractibleProps = {} 
    Ob.nPriorityOffset = nil

    function Ob:clear()
        for i, rTile in ipairs(self.tTiles) do
            Tile.rRenderLayer:removeProp(rTile)            
        end
        self.tTiles = {}
        for i, rInteractibleProp in ipairs(self.tInteractibleProps) do
            rInteractibleProp:onRemoved()
        end
        self.tInteractibleProps = {}
        self.rBGTile:removeOccupyingTile(self)
        self.rBGTile:setRoom(nil)
    end

    function Ob:setBGTile(rBGTile)
        if self.rBGTile then
            self.rBGTile:removeOccupyingTile(self)
        end
        self.rBGTile = rBGTile
        self.rBGTile:setOccupyingTile(self)
    end

    function Ob:getWorldCoords()
        if self.rBGTile then
            return self.rBGTile:getWorldCoords()
        end
    end

    function Ob:getRoom()
        return self.rRoom
    end

    function Ob:setRoom(rRoom)
        return self.rRoom
    end

    function Ob:addTile(rTile)
        if rTile then
            table.insert(self.tTiles, rTile)
        end
    end

    function Ob:getTiles()
        return self.tTiles
    end

    function Ob:addInteractibleProp(rProp)
        if rProp then
            table.insert(self.tInteractibleProps, rProp)
        end
    end

    function Ob:getInteractibleProps()
        return self.tInteractibleProps
    end

    function Ob:getTilePriority(nDefault)
        local nMaxPri = nDefault or 0        
        if self.nPriorityOffset then
            nMaxPri = nMaxPri + self.nPriorityOffset
        end
        local nNumProps = table.getn(self.tTiles)
        if nNumProps > 0 then
            nMaxPri = self.tTiles[nNumProps]:getPriority() or nMaxPri
        end
        nNumProps = table.getn(self.tInteractibleProps)
        if nNumProps > 0 then
            nMaxPri = self.tInteractibleProps[nNumProps]:getPriority() or nMaxPri
        end
        if self.rBGTile then
            local nPri = self.rBGTile:getOccupantPriority()
            if nPri > nMaxPri then
                nMaxPri = nPri
            end
        end
        if nMaxPri == 0 then
            if self.rBGTile then
                nMaxPri = self.rBGTile:getPriority() or nMaxPri
            end
            if self.nPriorityOffset then
                nMaxPri = nMaxPri + self.nPriorityOffset
            end
        end
 
        return nMaxPri
    end

    function Ob:setTilePriorityOffset(nPriorityOffset)
        self.nPriorityOffset = nPriorityOffset
        local nMaxPriority = 0
        for i, rTile in ipairs(self.tTiles) do
            local nPriority = (rTile:getPriority() or 0) + nPriorityOffset
            if nPriority > nMaxPriority then
                nMaxPriority = nPriority
            end
            rTile:setPriority(nPriority)
            if rTile.rDebugText then
                rTile.rDebugText:setString(tostring(nPriority))
            end
        end
        for i, rInteractibleProp in ipairs(self.tInteractibleProps) do
            local nPriority = (rInteractibleProp:getPriority() or 0) + nPriorityOffset
            if nPriority > nMaxPriority then
                nMaxPriority = nPriority
            end
            rInteractibleProp:setPriority(nPriority)
        end
--[[        
        self.rDebugText = g_GUI:CreateTextBox(8, MOAITextBox.CENTER_JUSTIFY)
        self.rDebugText:setColor(0, 0, 0)
        local x, y = self.rBGTile:getLoc()
        self.rDebugText:setString(tostring(nMaxPriority))
        self.rDebugText:setLoc(x + self.rMap.nTileWidth / 2, y + self.rMap.nTileHeight / 2)
        self.rDebugText:setColor(1, 1, 1)
        Tile.rRenderLayer:insertProp(self.rDebugText)
        self.rDebugText:setPriority(nMaxPriority + 100000)
        if nMaxPriority == 0 then
--            self.rDebugText:setReveal(0)
        end
]]--
        return nMaxPriority
    end

    return Ob
end

function Room:new(roomDef, rTransform, tTiles, rMap, tProps, rFloorGridProp, rRoomSlotInfo, tSaved, tOldProps)
    local Ob = rTransform
    Ob.rRoomDef = roomDef
    Ob.tSaved = tSaved

    function Ob:getRoomDef()
        return self.rRoomDef
    end

    function Ob:getRelCoords(nRow, nCol)
        return nRow - self.nRowOffset, nCol - self.nColOffset
    end

    function Ob:setRoomSlotInfo(rRoomSlotInfo)
        if self.rMap then
            if self.rOffsetTile then
                self.rOffsetTile:setRoom(nil)
            end
            self.rOffsetTile = self.rMap:getTile(rRoomSlotInfo.tileOffsetRow, rRoomSlotInfo.tileOffsetCol)
            self.rOffsetTile:setRoom(self)
            if self.rOffsetTile then
                self.nRowOffset = rRoomSlotInfo.tileOffsetRow - 1 -- need to offset by one since we're really meaning this is the starting tile
                self.nColOffset = rRoomSlotInfo.tileOffsetCol - 1
                self.nMaxRow = self.nRowOffset + self.nHeight
                self.nMaxCol = self.nColOffset + self.nWidth
                local x, y = self.rOffsetTile:getLoc()
                self:setLoc(x, y)     
                for nRow, tTiles in ipairs(Ob.tTiles) do
                    for nCol, rTile in ipairs(tTiles) do
                        rTile:setBGTile(self.rMap:getTile(nRow + self.nRowOffset, nCol + self.nColOffset))
                        rTile.nRow = nRow + self.nRowOffset
                        rTile.nCol = nCol + self.nColOffset
                    end
                end
                self.rRoomSlotInfo = rRoomSlotInfo       

                self:normalizePriorities()
                self:setTilePriorityOffset(rRoomSlotInfo.nPriOffset)
                if self.tProps then
                    for i, rProp in ipairs(self.tProps) do
                        rProp:setTileOffset(self.nRowOffset, self.nColOffset)                      
                    end
                end
            end
        end
    end

    function Ob:normalizePriorities()
        local nMinPri = nil
        for nRow, tTiles in ipairs(Ob.tTiles) do
            for nCol, rTile in ipairs(tTiles) do                
                local nPriority = rTile:getTilePriority()
                if not nMinPri or nPriority < nMinPri then
                    nMinPri = nPriority
                end
            end
        end        
        for nRow, tTiles in ipairs(Ob.tTiles) do
            for nCol, rTile in ipairs(tTiles) do
                rTile:setTilePriorityOffset(0 - nMinPri)
            end
        end                    
    end

    function Ob:setTilePriorityOffset(nPriorityOffset)
        self.nPriorityOffset = nPriorityOffset
        local nMaxPri = 0
        for nRow, tTiles in ipairs(Ob.tTiles) do
            for nCol, rTile in ipairs(tTiles) do
                local nPri = rTile:setTilePriorityOffset(nPriorityOffset)
                if nPri > nMaxPri then
                    nMaxPri = nPri
                end
            end
        end  
        return nMaxPri
    end

    function Ob:getRoomSlotInfo()
        return self.rRoomSlotInfo
    end

    function Ob:getProps()
        return self.tProps
    end

    function Ob:getPersistentProps()
        local tProps = {}
        if self.tProps then
            for i, rProp in ipairs(self.tProps) do
                if rProp:isPersistent() then
                    table.insert(tProps, rProp)
                end
            end
        end
        return tProps
    end

    function Ob:getHighestLevelProp()
        local rHighestLevelProp = nil
        local nHighestLevel = 0
        for i, rProp in ipairs(self.tProps) do
            if rProp:isPersistent() then
                local nLevel = rProp:getPropDef().nLevel
                if nLevel and nLevel > nHighestLevel then
                    rHighestLevelProp = rProp
                    nHighestLevel = nLevel
                end
            end
        end
        return rHighestLevelProp
    end

    function Ob:getHighestLevelAvailableProp()
        local rHighestLevelProp = nil
        local nHighestLevel = 0
        for i, rProp in ipairs(self.tProps) do
            if rProp:isPersistent() and rProp:isUnlocked() and rProp:isAvailable() then
                local nLevel = rProp:getPropDef().nLevel
                if nLevel and nLevel > nHighestLevel then
                    rHighestLevelProp = rProp
                    nHighestLevel = nLevel
                end
            end
        end
        return rHighestLevelProp
    end

    function Ob:setRoomSlotName(sRoomSlotName)
        self.sRoomSlotName = sRoomSlotName
    end

    function Ob:getRoomSlotName()
        return self.sRoomSlotName
    end

    function Ob:isInBounds(nRow, nCol)
        if nRow > self.nRowOffset and nRow <= self.nMaxRow
            and nCol > self.nColOffset and nCol <= self.nMaxCol then
            return true
        end
        return false
    end

    function Ob:getRenderPriority(nAbsRow, nAbsCol)
        if self:isInBounds(nAbsRow, nAbsCol) then
            local nRelRow = nAbsRow - self.nRowOffset
            local nRelCol = nAbsCol - self.nColOffset
            if self.rRoomDef and self.rRoomDef.tCharPriTable and self.rRoomDef.tCharPriTable[nRelRow] then
                return (self.rRoomDef.tCharPriTable[nRelRow][nRelCol] + (self.nPriorityOffset or 0))
            end
        end
        return nil
    end

    function Ob:getTile(nRow, nCol)
        if self.tTiles[nRow] then
            return self.tTiles[nRow][nCol]
        end
        return nil
    end

    function Ob:getCenterPos()
        local nNumRows = table.getn(self.tTiles)
        if nNumRows > 0 then
            local nNumCols = table.getn(self.tTiles[nNumRows])
            if nNumCols > 0 then
                local nRow = math.floor(nNumRows / 2)
                local nCol = math.floor(nNumCols / 2)
                if self.tTiles[nRow][nCol] then
                    return self.tTiles[nRow][nCol]:getWorldCoords()
                end
            end
        end
        return self:getLoc()
    end
    
    function Ob:getRandomTile()
        local nNumTilesRow = table.getn(self.tTiles)
        if nNumTilesRow > 0 then
            local nNumTilesCol = table.getn(self.tTiles[nNumTilesRow])
            if nNumTilesCol > 0 then
                local nRandRow = math.random(2, nNumTilesRow - 1)
                local nRandCol = math.random(2, nNumTilesCol - 1)
                return self.tTiles[nRandRow][nRandCol]
            end
        end
        return nil
    end

    function Ob:getTile(nRow, nCol)
        if self.tTiles[nRow] then
            return self.tTiles[nRow][nCol]
        end
        return nil
    end

    function Ob:getPropOfType(sPropType)
        for i, rProp in ipairs(self.tProps) do
            if rProp:getPropType() == sPropType then
                return rProp
            end
        end
        return nil
    end

    function Ob:getPropsOfType(sPropType)
        local t = {}
        for i, rProp in ipairs(self.tProps) do
            if rProp:getPropType() == sPropType then
                table.insert(t,rProp)
            end
        end
        return t
    end

    function Ob:getNextAvailablePropOfType(sPropType)
        for i, rProp in ipairs(self.tProps) do
            if rProp:getPropType() == sPropType and rProp:isUnlocked() and rProp:isAvailable() then
                return rProp
            end
        end
    end
    
    function Ob:getUnlockedPropOfType(sPropType)
        for i, rProp in ipairs(self.tProps) do
            if rProp:getPropType() == sPropType and rProp:isUnlocked() then
                return rProp
            end
        end
    end        
    
    function Ob:shouldShowRoomDetails()
        local bShow = (not self.bHideRoomDetails)
    
        if self:getRoomType() == "hub" and not self.rRoomDef.baseUpgrade then
            bShow = false
        end
    
        return bShow
    end
    
    function Ob:getRoomLevel()
        return self.rRoomDef.nRoomLevel
    end

    function Ob:getName()
        return self.rRoomDef.visualName
    end
    
    function Ob:getDescription()
        return self.rRoomDef.visualDescription
    end
    
    function Ob:getUpgradeCost()
        return self.rRoomDef.upgradeCost
    end

    function Ob:isEmptyRoom()
        return self.bIsEmptyRoom
    end

    function Ob:getRoomType()
        return self.roomtype
    end

    function Ob:getRoomUpgradeDefs()
        return self.rRoomDef.tRoomUpgrades
    end

    function Ob:getBaseUpgradeDef()
        if self.baseUpgrade then
            return g_CurrentMap:getRoomDef(self.baseUpgrade)
        end
    end

    function Ob:onRemoved()
        for i, tTile in ipairs(self.tTiles) do
            for c, rTile in ipairs(tTile) do
                rTile:clear()
            end
        end
        if self.rFloorGridProp then
            local rLayer = Tile.rTileLayer
            rLayer:removeProp(self.rFloorGridProp)
        end
    end

    function Ob:getDoorLoc()
        if self.sRoomSlotName and self.rRoomDef and self.rRoomDef.tDoorLocs then
            local nRow = self.rRoomDef.tDoorLocs[self.sRoomSlotName][1]
            local nCol = self.rRoomDef.tDoorLocs[self.sRoomSlotName][2]
            if nRow and nCol then
                return nRow + self.nRowOffset, nCol + self.nColOffset
            end            
        end
        return nil
    end

    function Ob:handlePropTouch(eventType, sx, sy)
        local handled = false
        
        -- check to see if we clicked a prop in this room
        if self.tProps then
            for i, rProp in ipairs(self.tProps) do
                if rProp:canHandleTouches() then
                    handled = rProp:onFinger(eventType, sx, sy)
                    if handled then
                        break
                    end
                end
            end
        end
        
        return handled
    end
    
    function Ob:onFinger(eventType, sx, sy)
        handled = false
    
        if false == handled and g_Game:isActionEnabled('RoomSelection') then
            if self.rRoomDef.roomtype == 'hub' then
                if not g_Game:isActionEnabled('HubSelection') then
                    return false
                end
            end
            g_GUI:ShowRoomDetailsUI(self)
            handled = true
        end
        
        return handled
    end
    
    function Ob:openDoor()
        local nRow, nCol = self:getDoorLoc()
        if nRow and nCol then
            local rBGTile = self.rMap:getTile(nRow, nCol)
            if rBGTile then
                local tTiles = rBGTile:getOccupyingTiles()
                if tTiles then
                    for i, rTile in ipairs(tTiles) do
                        local tPropTiles = rTile:getTiles()
                        if tPropTiles then
                            for c, rPropTile in ipairs(tPropTiles) do
                                if rPropTile.tTileProperties and rPropTile.tTileProperties.isDoor then
                                    if rPropTile.playAnimation and rPropTile.tTileProperties.sOpenAnim then
                                        rPropTile:playAnimation(rPropTile.tTileProperties.sOpenAnim)
                                    end
                                    return
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    function Ob:getPreDoorLoc()
        if self.sRoomSlotName and self.rRoomDef and self.rRoomDef.tPreDoorLocs then
            if self.rRoomDef.tPreDoorLocs[self.sRoomSlotName] then
                local nRow = self.rRoomDef.tPreDoorLocs[self.sRoomSlotName][1]
                local nCol = self.rRoomDef.tPreDoorLocs[self.sRoomSlotName][2]
                if nRow and nCol then
                    return nRow + self.nRowOffset, nCol + self.nColOffset
                end
            end
        end
        return nil
    end

    function Ob:getDoorEntryOffsets()
        if self.sRoomSlotName and self.rRoomDef and self.rRoomDef.tDoorEntryOffset then
            if self.rRoomDef.tDoorEntryOffset[self.sRoomSlotName] then
                local nRow = self.rRoomDef.tDoorEntryOffset[self.sRoomSlotName][1]
                local nCol = self.rRoomDef.tDoorEntryOffset[self.sRoomSlotName][2]
                if nRow and nCol then
                    return nRow, nCol
                end
            end
        end
        return 0, 0
    end

    function Ob:shouldTeleportToDoor()
        return self.rRoomDef.bShouldTeleportToDoor
    end

    function Ob:onPropUnlocked(rProp, bTemp)
        if bTemp then return end

        for i, rExistingProp in ipairs(self.tProps) do
            if rExistingProp == rProp then
                rExistingProp:playAnimation()
            
                if self.tSaved.tProps[i] then
                    self.tSaved.tProps[i].bUnlocked = true
                end
            end
        end
    end

    function Ob:onPropUpgraded(rProp)
        for i, rExistingProp in ipairs(self.tProps) do
            if rExistingProp == rProp then
                rExistingProp:playAnimation()
                if self.tSaved.tProps[i] then
                    self.tSaved.tProps[i].sPropDefKey = rProp:getPropDefKey()
                end
            end
        end        
    end

    function Ob:getActivityIcon()
        if self.rRoomDef.iconPath then
            return self.rRoomDef.iconPath
        end
    end

    -- INITIALIZATION
    for key, value in pairs(roomDef) do
        Ob[key] = value
    end
    Ob.rRoomDef = roomDef
    Ob.rMap = rMap
    Ob.sRoomName = sRoomName
    Ob.tTiles = tTiles
    Ob.sRoomSlotName = nil
    Ob.nRoomLevel = 1

    Ob.nRowOffset = 0
    Ob.nColOffset = 0    
    Ob.nWidth = 1
    Ob.nHeight = 1
    Ob.nMaxRow = 1
    Ob.nMaxCol = 1

    if tOldProps and table.getn(tOldProps) > 0 then
        Ob.tProps = tOldProps
        for i, rProp in ipairs(Ob.tProps) do
            if tProps[i] then
                rProp:changePropTile(tProps[i].rPropTile)
            end
        end
    else
        Ob.tProps = tProps
    end
    Ob.rFloorGridProp = rFloorGridProp

    for nRow, tTiles in ipairs(Ob.tTiles) do
        Ob.nHeight = nRow
        for nCol, rTile in ipairs(tTiles) do
            Ob.nWidth = nCol
            rTile:setRoom(Ob)
        end
    end

    if Ob.tProps then
        for i, rProp in ipairs(Ob.tProps) do
            rProp:setRoom(Ob)        
        end
    end    

    if not Ob.tSaved.tProps then
        Ob.tSaved.tProps = {}
        -- unlock props
        for i, rProp in ipairs(Ob.tProps) do
            local rPropDef = rProp:getPropDef()
            if rPropDef then
                Ob.tSaved.tProps[i] = {}
                Ob.tSaved.tProps[i].sPropDefKey = rProp:getPropDefKey()
                Ob.tSaved.tProps[i].proptype = rPropDef.proptype
                Ob.tSaved.tProps[i].bUnlocked = false 
                if rPropDef.bUnlocked then
                    rProp:unlockProp()
                end
            end
        end

        if roomDef.tUnlockedProps then
            for i, sPropName in ipairs(roomDef.tUnlockedProps) do
                for c, rProp in ipairs(Ob.tProps) do
                    if rProp:getPropType() and rProp:getPropType() == sPropName and not rProp:isUnlocked() then
                        rProp:unlockProp()
                        break
                    end
                end
            end
        end
    else
        -- fix up old save games
        for i, tPropSaveTable in ipairs(Ob.tSaved.tProps) do
            if not tPropSaveTable.proptype then
                local rDef = TileMap.getPropDef(tPropSaveTable.sPropDefKey)
                tPropSaveTable.proptype = rDef.proptype
            end
        end
        -- load up old props
        for i, tPropSaveTable in ipairs(Ob.tSaved.tProps) do
            local rProp = Ob.tProps[i]
            if rProp then
                if rProp:getPropDef().proptype == tPropSaveTable.proptype then
                    if tPropSaveTable.sPropDefKey ~= Ob.tProps[i]:getPropDefKey() then                    
                        rProp:upgradeProp(Ob.tSaved.tProps[i].sPropDefKey)
                    end
                    if tPropSaveTable.bUnlocked then
                        rProp:unlockProp()
                    end
                end
            end
        end
        -- go through the rest of the props and unlock those that are unlocked in the def
        for i, rProp in ipairs(Ob.tProps) do
            if not rProp:isUnlocked() then
                local rDef = rProp:getPropDef()
                if rDef and rDef.bUnlocked then
                    rProp:unlockProp()
                end
            end
        end        
    end

    local tHolidays = g_Game:getActiveHolidays()
    if tHolidays then
        for _,h in ipairs(tHolidays) do
            if h.enableProps then
                for _,propType in ipairs(h.enableProps) do
                    local tProps = Ob:getPropsOfType(propType)
                    for _,rProp in ipairs(tProps) do
                        rProp:unlockProp(true)
                    end
                end
            end
        end
    end

    if Ob.tProps then
        for i, rProp in ipairs(Ob.tProps) do
            if rProp:getPropDef() and rProp:getPropDef().sUnlockVar then
                if g_Game:getSaveVar(rProp:getPropDef().sUnlockVar) == true then
                    rProp:unlockProp(true)
                end
            end
        end
    end

    if rRoomSlotInfo then
        Ob:setRoomSlotInfo(rRoomSlotInfo)
    end
    
    return Ob
end

function Prop:new(rPropTile)
    local Ob = {}
    if not rPropTile then
        print("ERROR: Attempted to create a prop without a prop tile")
    end
    Ob.rPropTile = rPropTile
    Ob.tOccupyingTiles = {}
    Ob.bIsUnlocked = false
    Ob.rRoom = nil
    Ob.nRow = rPropTile.nRow
    Ob.nCol = rPropTile.nCol
    Ob.nRowOffset = 0
    Ob.nColOffset = 0
    Ob.bIsPersistent = false
    Ob.tUsers = {}
    Ob.rButton = nil
    Ob.rPropTouchManager = nil
    
    

    function Ob:getInteractionSpriteSheets()
        if self.rPropDef then
            return self.rPropDef.interactionSpriteSheets
        end
    end
	
    function Ob:changePropTile(rTile)
        self.rPropTile = rTile
        if self.sPropDefKey then
            self:upgradeProp(self.sPropDefKey)
        end
        if self.nRowOffset and self.nColOffset then
            self:setTileOffset(self.nRowOffset, self.nColOffset)
        end
    end
    
    function Ob:addUser(rUser)
        table.insert(self.tUsers, rUser)
        rUser:setInteractingProp(self, table.getn(self.tUsers))
    end

    function Ob:removeUser(rUser)
        for i, rCurUser in ipairs(self.tUsers) do
            if rUser == rCurUser then
                table.remove(self.tUsers, i)
                rUser:setInteractingProp(nil)
            end
        end
    end
    
    function Ob:clearUsers()
        self.tUsers = {}
    end

    function Ob:setTileOffset(nRowOffset, nColOffset)
        self.nRowOffset = nRowOffset
        self.nColOffset = nColOffset
        -- if we had a user, move them
        for i, rUser in ipairs(self.tUsers) do
            if self.rPropDef.interactOffset[i] then                
                local nFactor = 2
                local nOffsetX = self.rPropTile:getLoc() + self.rPropDef.interactOffset[i].x * nFactor
                local nOffsetY = self.rPropTile:getLoc() + self.rPropDef.interactOffset[i].y * nFactor
--                rUser:setLoc(nOffsetX, nOffsetY)
                rUser:setPriority(self:getInteractPri())
            end
        end
    end

    function Ob:isAvailable()
        local nMaxUsers = self.rPropDef.nMaxUsers or 1
        if table.getn(self.tUsers) >= nMaxUsers then
            return false
        end
        return true
    end

    function Ob:getUsers()
        return self.tUsers
    end

    function Ob:isPersistent()
        return self.bIsPersistent
    end

    function Ob:setIsPersistent(bIsPersistent)
        self.bIsPersistent = bIsPersistent
    end

    function Ob:onRemoved()
        Tile.rRenderLayer:removeProp(self.rPropTile)
    end

    function Ob:setPropDef(rPropDef, sPropDefKey, rPropTouchManager, rButton, rTilemap)
        self.rTilemap = rTilemap
        self.rPropDef = rPropDef
        self.sPropDefKey = sPropDefKey
        self.rPropTouchManager = rPropTouchManager
        self.rButton = rButton
        
        self:playAnimation()
    end
    
    function Ob:canHandleTouches()
        return self.rButton ~= nil and self:isUnlocked()
    end
    
    function Ob:playAnimation(sAnimName)
        if not sAnimName then
            sAnimName = self.rPropDef.sDefaultAnim
        end
        
        if not sAnimName then
            sAnimName = self.rPropTile.tTileProperties.sDefaultAnim
        end
        
        if sAnimName then
            print("play anim: " .. sAnimName)
            if self.rPropTile.playAnimation then
                self.rPropTile:playAnimation(sAnimName) 
            end
        end
    end
    
    function Ob:onFinger(eventType, sx, sy)
        local handled = false
        
        if self:canHandleTouches() then
            local w, h = self.rButton:getScl()
            local x, y = self.rButton:getLoc()
        
            -- we flip the h here because buttons scale in UI space which is in -y
            h = -h
        
            local roomX, roomY = self.rRoom:getLoc()
        
            x = x + roomX
            y = y + roomY
        
            -- transform sx into world x
            local worldTouchX, worldTouchY = Tile.rRenderLayer:wndToWorld(sx, sy)
        
            if worldTouchX >= x and worldTouchX <= x + w and worldTouchY >= y and worldTouchY <= y + h then
                self:onTouched()
                
                handled = true
            else
                --print("hey I didn't get touched :( " .. worldTouchX .. "," .. worldTouchY .. " - " .. self:getPropType() .. " " .. x .. " " .. y .. " " .. w .. " " .. h)
            end
        end
        
        return handled
    end

    function Ob:onTouched()
        if self.rPropTouchManager then
            self.rPropTouchManager:onPropTouch(self)
        end
    end
    
    function Ob:getPropDefKey()
        return self.sPropDefKey
    end

    function Ob:setRoom(rRoom)
        self.rRoom = rRoom
        if self.rPropTile then
            self.rPropTile:setParent(rRoom)
        end
    end

    function Ob:getRoom()
        return self.rRoom
    end

    function Ob:canUpgrade()
        local sUpgradeDefKey = self:getUpgradeDefKey()
        local rDef = TileMap.getPropDef(sUpgradeDefKey)
        if rDef then
            if self.rRoom:getRoomLevel() >= rDef.roomlvl then
                return true
            end
        end
        return false
    end

    function Ob:getPropType()
        if self.rPropDef then
            return self.rPropDef.proptype
        end
        return nil
    end

    function Ob:getPropDef()
        return self.rPropDef
    end

    function Ob:getUpgradeDefKey()
        if self.rPropDef and self.rPropDef.propUpgrade then
            return self.rPropDef.propUpgrade
        end
        return nil
    end

    function Ob:isUnlocked()
        return self.bIsUnlocked
    end

    function Ob:lockProp()
        self.bIsUnlocked = false
        if self.rPropTile then
            self.rPropTile:setVisible(false)
        end        
    end

    function Ob:unlockProp(bTemp)
        self.bIsUnlocked = true
        if self.rPropTile then
            self.rPropTile:setVisible(true)
        end
        if self.rRoom then
            self.rRoom:onPropUnlocked(self, bTemp)
        end
    end

    function Ob:upgradeProp(sPropDefKey)
        print("UPGRADING TO "..sPropDefKey)
        local rPropDef = TileMap.getPropDef(sPropDefKey)
        self.sPropDefKey = sPropDefKey
        if not rPropDef then
            print("ERROR: Trying to upgrade a prop without a propdef")
            return
        end
        self.rPropDef = rPropDef
        if self.rPropDef and self.rPropDef.imgName then
            local x, y = self.rPropTile:getLoc()
            if self.rPropDef.imgName then
                local sSuffix = Renderer.getTextureSizeSuffix()                
                local sSpriteSheetName = TileMap.tTilesetToSpriteSheets[self.rPropDef.imgName] or 'BaseProps'
                if tSpriteSheetToScreenInfo[sSpriteSheetName] then
                    local tInfo = tSpriteSheetToScreenInfo[sSpriteSheetName][Renderer.getCurScreenInfo().key]
                    if tInfo then
                        sSuffix = tInfo.sTextureSizeSuffix                        
                    end
                end                
                local sName = self.rPropDef.imgName..sSuffix.."_1"
                self.rPropTile:setIndex(self.rPropTile.rSavedDeck.names[sName])
            end
            self.bIsUnlocked = true
        end        
        if self.rRoom then
            self.rRoom:onPropUpgraded(self)
        end
    end

    function Ob:setPriority(nPri)
        if self.rPropTile then
            self.rPropTile:setPriority(nPri)
        end
        for i, rUser in ipairs(self.tUsers) do
            rUser:setPriority(self:getInteractPri())
        end
        if self.rPropTile.rDebugText then
            self.rPropTile.rDebugText:setString(tostring(nPri))
            self.rPropTile.rDebugText:setLoc(64, 64)
        end        
    end

    function Ob:getPriority()
        if self.rPropTile then
            return self.rPropTile:getPriority()
        else
            return 0
        end
    end

    function Ob:getBaseOffsetRow()
        if self.rPropTile then
            if self.rPropTile.tTileProperties.baseoffsetrow then
                return tonumber(self.rPropTile.tTileProperties.baseoffsetrow)
            end
        end
        return 0
    end

    function Ob:getBaseOffsetCol()
        if self.rPropTile then
            if self.rPropTile.tTileProperties.baseoffsetcol then
                return tonumber(self.rPropTile.tTileProperties.baseoffsetcol)
            end
        end
        return 0
    end

    function Ob:getInteractTileCoords()
        local nInteractRowOffset = 0
        local nInteractColOffset = 0
        if self.rPropDef.interactionTileOffset then
            nInteractRowOffset = self.rPropDef.interactionTileOffset[1]
            nInteractColOffset = self.rPropDef.interactionTileOffset[2]
        end
        return self.nRow + self.nRowOffset + nInteractRowOffset, self.nCol + self.nColOffset + nInteractColOffset
    end

    function Ob:getInteractOffset()
        -- This assumes assigning to the slot before getting the offset
        if self.rPropDef and self.rPropDef.interactOffset then
            local nNumUsers = table.getn(self.tUsers)
            if self.rPropDef.interactOffset[nNumUsers] then
                local nFactor = 2
                return self.rPropDef.interactOffset[nNumUsers].x * nFactor, self.rPropDef.interactOffset[nNumUsers].y * nFactor
            end
        end
        return 0, 0
    end

    function Ob:shouldTeleportToProp()
        if self.rPropDef then
            return self.rPropDef.bTeleportToProp
        end
        return false
    end

    function Ob:getInteractPri(nIndex)
        local nBasePri = 0
        if self.rPropTile then
            nBasePri = self.rPropTile:getPriority() or nBasePri
        end
        if nIndex then
            if self.rPropDef.interactPriOffset[nIndex] then
                return nBasePri + self.rPropDef.interactPriOffset[nIndex]
            end
        else           
            local nNumUsers = table.getn(self.tUsers)
            if self.rPropDef.interactPriOffset[nNumUsers] then
                return nBasePri + self.rPropDef.interactPriOffset[nNumUsers]
            end
        end
        return nBasePri
    end

    function Ob:getInteractAnim()
        return self.rPropDef.interactAnim, (self.rPropDef.bInteractAnimFlipX or false)
    end

    return Ob
end

TileMap.tPropDefs = dofile(DFFile.getLevelPath("PropDefs.lua"))
function TileMap.getPropDef(sPropType)
    if sPropType then
        return TileMap.tPropDefs[sPropType]
    end
    return nil
end
function TileMap.updatePropDef(sPropType, tValues)
    g_Game:updateDefWithTweak(TileMap.tPropDefs, sPropType, tValues)
end
function TileMap.getAllPropDefs()
    return TileMap.tPropDefs
end

TileMap.tRoomDefs = dofile(DFFile.getLevelPath("RoomDefs.lua"))
function TileMap.getRoomDef(sRoomDefKey)
    if sRoomDefKey then
        return TileMap.tRoomDefs[sRoomDefKey]
    end
end
function TileMap.updateRoomDef(sRoomDefKey, tValues)
    g_Game:updateDefWithTweak(TileMap.tRoomDefs, sRoomDefKey, tValues)
end

TileMap.tBaseDefs = dofile(DFFile.getLevelPath("BaseDefs.lua"))
function TileMap.getBaseDef(sBaseDefKey)
    if sBaseDefKey then
        return TileMap.tBaseDefs[sBaseDefKey]
    end
end


function TileMap:new(sMapFilename)
    if not Tile.rRenderLayer then
        Tile.rRenderLayer = Renderer.getRenderLayer('Gameplay')    
    end
    if not Tile.rTileLayer then
        Tile.rTileLayer = Renderer.getRenderLayer('Tiles')
    end

    local Ob = MOAITransform.new()

    -- FUNCTIONS
    function Ob:createBaseTiles(nMaxRow, nMaxCol)
        self.nMaxRow = nMaxRow
        self.nMaxCol = nMaxCol
        local tTiles = {}
        local nCurX = 0
        local nCurY = 0
        local nXOffSet = self.nTileWidth / 2
        local nYOffSet = self.nTileHeight / 2
        local nNumTiles = 0
        for nRow = 1, nMaxRow do
            tTiles[nRow] = {}
            for nCol = 1, nMaxCol do
                nNumTiles = nNumTiles + 1
                local rNewTile = BaseTile:new(nRow, nCol, nNumTiles, self)
                local x = nCurX + (nXOffSet * (nCol - 1))
                local y = nCurY - (nYOffSet * (nCol - 1))
                rNewTile:setLoc(x, y)
                tTiles[nRow][nCol] = rNewTile
            end                
            nCurX = nCurX - nXOffSet
            nCurY = nCurY - nYOffSet
        end
        return tTiles
    end

    function Ob:addAnims(rTile, sAnimDefFileName)
        if sAnimDefFileName then
            local rAnimDef = util.dofile(DFFile.getDataPath(sAnimDefFileName))
            if rAnimDef then
                local tAnimations = { }
                for sAnimName, animInfo in pairs(rAnimDef.tAnimInfos) do
                    local rSpriteAnim = { }
                    rSpriteAnim.tAnimDef = animInfo                               
                    local tAnimFrameInfos = {}
                    local nNumFrames = 0
                    if animInfo.tAnimFrames then
                        local nIndex = 0
                        for i, rCustomFrame in ipairs(animInfo.tAnimFrames) do
                            local sSuffix = Renderer.getTextureSizeSuffix()   
                            local sSpriteSheetName = TileMap.tTilesetToSpriteSheets[rCustomFrame.sAnimFrameName] or 'BaseProps'                            
                            if tSpriteSheetToScreenInfo[sSpriteSheetName] then
                                local tInfo = tSpriteSheetToScreenInfo[sSpriteSheetName][Renderer.getCurScreenInfo().key]
                                if tInfo then
                                    sSuffix = tInfo.sTextureSizeSuffix                        
                                end
                            end      
                            local sName = rCustomFrame.sAnimFrameName..sSuffix.."_1"
                            local nDeckIndex = rTile.rSavedDeck.names[sName]
                            if nDeckIndex then
                                local rAnimFrameInfo = {}
                                rAnimFrameInfo.nDeckIndex = nDeckIndex
                                rAnimFrameInfo.nSecondsPerFrame = rCustomFrame.nSecondsPerFrame
                                table.insert(tAnimFrameInfos, rAnimFrameInfo)
                                nIndex = nIndex + 1
                                nNumFrames = nIndex
                            end
                        end
                    end
                    if table.getn(tAnimFrameInfos) > 0 then                            
                        local rCurve = MOAIAnimCurve.new()
                        rCurve:reserveKeys(nNumFrames)
                        
                        local nTime = 0
                        for nFrameNum, rFrameInfo in ipairs(tAnimFrameInfos) do
                            rCurve:setKey(nFrameNum, nTime, rFrameInfo.nDeckIndex, MOAIEaseType.FLAT )
                            local nSecondsPerFrame = rFrameInfo.nSecondsPerFrame or animInfo.nSecondsPerFrame or  rAnimDef.nSecondsPerFrame
                            nTime = nTime + nSecondsPerFrame / rAnimDef.nAnimationSpeed
                        end
                        rSpriteAnim.rCurve = rCurve
                        
                        -- Create animation controller
                        local rAnimController = MOAIAnim:new ()
                        rAnimController:reserveLinks(1)
                        rAnimController:setLink(1, rCurve, rTile, MOAIProp2D.ATTR_INDEX)
                        
                        if animInfo.repeatMode == "loop" then
                            rAnimController:setMode ( MOAITimer.LOOP )
                        end
                        if animInfo.repeatMode == "pingpong" then
                            rAnimController:setMode ( MOAITimer.PING_PONG )
                        end
                        
                        rSpriteAnim.rAnimController = rAnimController
                        table.insert(tAnimations, rSpriteAnim)
                    end
                end
                rTile.tAnimations = tAnimations
                rTile.sCurrentAnimation = ""
                rTile.rAnimDef = rAnimDef

                rTile.playAnimation = function(self, animName)
                    local rAnimToPlay = nil
                    if self.tAnimations then
                        for i, rAnim in ipairs(self.tAnimations) do
                            if rAnim.tAnimDef.name == animName then
                                rAnimToPlay = rAnim
                                break
                            end
                        end
                    end
                    if rAnimToPlay then
                        -- Same animation?
                        -- Stop current animation...
                        if self.rCurrentAnimation then
                            self.rCurrentAnimation.rAnimController:stop()
                        end
                        self.rCurrentAnimation = rAnimToPlay
                        self.rCurrentAnimation.rAnimController:start()
                    end
                end
            end
        end
    end

    function Ob:processMap(tMapInfo)
        local nIndex = 0
        local rTransform = MOAITransform.new()
        local tMapTiles = {}
        local tProps = {}
        local nXOffSet = self.nTileWidth / 2
        local nYOffSet = self.nTileHeight / 2
        local rFloorGridProp = nil
        -- assume that the first layer is the ground
        for i, rLayerInfo in ipairs(tMapInfo.tLayerInfos) do
            local nCurX = 0
            local nCurY = 0
            if rLayerInfo.tProps then
                for nRow, tTiles in ipairs(rLayerInfo.tProps) do
                    for nCol, rTile in ipairs(tTiles) do
                        local rLayer = Tile.rRenderLayer
                        if not tMapTiles[nRow] then
                            tMapTiles[nRow] = {}
                        end
                        if not tMapTiles[nRow][nCol] then
                            tMapTiles[nRow][nCol] = Tile:new(nRow, nCol, self)
                        end
                        if rTile ~= DFTiled.kEMPTY_TILE then
                            local x = nCurX + (nXOffSet * (nCol - 1))
                            local y = nCurY - (nYOffSet * (nCol - 1))
                            rTile.nCol = nCol
                            rTile.nRow = nRow
                            rLayer:insertProp(rTile)
                            nIndex = nIndex + 1
                            local nPri = nIndex * kPRI_OFFSET
                            rTile:setPriority(nPri)
                            rTile:setParent(rTransform)
                            rTile:setLoc(x, y)
                            
                            -- wrap the tiles into one data structure
                            if rTile.tTileProperties.proptype then     
                                local rProp = Prop:new(rTile)
                                rProp:setIsPersistent(true)
                                table.insert(tProps, rProp)

                                local propDef = TileMap.getPropDef(rTile.tTileProperties.proptype)
                                
                                if propDef then
                                    rButton = nil
                                
                                    -- only add a button if we need a button
                                    if propDef.buttonRect then
                                        local buttonRect = propDef.buttonRect
                                        
                                        if buttonRect.x and buttonRect.y and buttonRect.w and buttonRect.h then                                    
                                            rButton = MOAIProp2D.new()
                                            rButton:setDeck(g_GUI:GetOnePixelDeck())
                                            
                                            rButton:setLoc(x + buttonRect.x, y + buttonRect.y)
                                            rButton:setScl(buttonRect.w, -buttonRect.h)
                                            
                                            if bDebugShowPropTouchButtons then
                                                rButton:setColor(0, 1, 1, 1) 
                                                rButton:setPriority(5000)
                                                rButton:setParent(rTransform)
                                                rLayer:insertProp(rButton)
                                            end
                                        end
                                    end
                                    
                                    rProp:setPropDef(propDef, rTile.tTileProperties.proptype, self.rPropTouchManager, rButton, rTile, self)
                                
                                    rTile:setVisible(false)
                                    tMapTiles[nRow][nCol]:addInteractibleProp(rProp)
                                end
                            else
                                if bDebugShowPropTouchButtons then
                                    rTile:setColor(0, 0, 0, 0.25)
                                end
                                tMapTiles[nRow][nCol]:addTile(rTile)
                            end
--[[
                            rTile.rDebugText = g_GUI:CreateTextBox(8, MOAITextBox.CENTER_JUSTIFY)
                            rTile.rDebugText:setColor(0, 0, 0)
                            rTile.rDebugText:setString(tostring(nRow..', '..nCol))
                            rTile.rDebugText:setLoc(64, 32)
                            rTile.rDebugText:setColor(1, 1, 1)
                            rTile.rDebugText:setParent(rTile)
                            Renderer.getRenderLayer('GameplayOverlay'):insertProp(rTile.rDebugText)
                    ]]--
                            if rTile.tTileProperties.animDefFile then
                                self:addAnims(rTile, rTile.tTileProperties.animDefFile)
                                if rTile.tTileProperties.sDefaultAnim then
                                    if rTile.playAnimation then
                                        rTile:playAnimation(rTile.tTileProperties.sDefaultAnim)
                                    end
                                end
                            end
                            
                            if bDebugShowTileCoords then -- debug rendering of the coords
                                rTile.rDebugText = g_GUI:CreateTextBox(8, MOAITextBox.CENTER_JUSTIFY)
                                rTile.rDebugText:setString(nRow..", "..nCol)
                                rTile.rDebugText:setColor(0, 0, 0)
                                rTile.rDebugText:setLoc(x + nXOffSet, y + nYOffSet)
                                rTile.rDebugText:setParent(rTransform)
                                Tile.rRenderLayer:insertProp(rTile.rDebugText)
                            end
                        end
                    end
                    nCurX = nCurX - nXOffSet
                    nCurY = nCurY - nYOffSet
                end

            elseif rLayerInfo.rGridProp then
                local rLayer = Tile.rTileLayer
                local x = (-self.nTileWidth * math.floor(rLayerInfo.nGridWidth / 2)) + self.nTileWidth -- oh boy, magic numbers aplenty
                local y = -self.nTileHeight / 2 * (rLayerInfo.nGridHeight - 1)
                rLayerInfo.rGridProp:setLoc(x, y)
                rLayerInfo.rGridProp:setParent(rTransform)
                rLayer:insertProp(rLayerInfo.rGridProp)
                rFloorGridProp = rLayerInfo.rGridProp
            end
        end
        return rTransform, tMapTiles, tProps, rFloorGridProp
    end        

    function Ob:onFinger(eventType, sx, sy)
        local handled = false
        
        if g_Game:isActionEnabled('TouchProp') then
            for i, rRoom in pairs(self.tMap) do
                handled = rRoom:handlePropTouch(eventType, sx, sy)
                
                if handled then
                    break
                end
            end
        end
            
        return handled
    end
    
    function Ob:getRoomDef(sRoomName)
        return TileMap.getRoomDef(sRoomName)
    end

    function Ob:getBaseDef(sBaseName)
        return TileMap.getBaseDef(sBaseName)
    end

    function Ob:getRoomSlotDef(sRoomSlotName)
        if self.rHubDef and sRoomSlotName then
            return self.rHubDef.tRooms[sRoomSlotName]
        end
    end

    function Ob:setRoomAtSlot(rRoom, sRoomSlotName)
        self.tMap[sRoomSlotName] = rRoom
        rRoom:setRoomSlotName(sRoomSlotName)
    end

    function Ob:refreshPriorities()
        -- fixup all priorities
        local nBasePri = 0
        for nCol = 1, self.nMaxCol do
            for nRow = 1, self.nMaxRow do
                rRoom = self.tBaseTiles[nRow][nCol]:getRoom()
                if rRoom then
                    nBasePri = rRoom:setTilePriorityOffset(nBasePri + 1)
                end
            end
        end
    end

    function Ob:createRoom(rRoomDef, rSlotInfo, tSaved, tOldProps)
        -- get the coords
        if not rRoomDef then
            print("ERROR: Could not find the roomdef")
            return nil
        end
        local tMapInfo = DFTiled.getLayerInfosFromTiled(rRoomDef.tileFile, tSpriteSheetToScreenInfo, 2, TileMap.tTilesetToSpriteSheets, -2, -2)
        if tMapInfo then
            local rTransform, tTiles, tProps, rFloorGridProp = Ob:processMap(tMapInfo, Tile.rRenderLayer)
            rTransform:setParent(self) 
            local rRoom = Room:new(rRoomDef, rTransform, tTiles, self, tProps, rFloorGridProp, rSlotInfo, tSaved, tOldProps)
            return rRoom
        end
        return nil
    end

    function Ob:createRooms(rHubInfo, tSaved)
        if not tSaved.tRooms then
            tSaved.tRooms = {}
        end
        if rHubInfo and rHubInfo.tRooms then
            for sRoomSlotName, rInfo in pairs(rHubInfo.tRooms) do                
                local rRoomSlotDef = self:getRoomSlotDef(sRoomSlotName)
                local sRoomType = rInfo.roomType
                if not tSaved.tRooms[sRoomSlotName] or sRoomSlotName == 'hub' or sRoomSlotName == 'bg' then
                    print('roomtype'..tostring(rInfo.roomType))
                    tSaved.tRooms[sRoomSlotName] = {}
                    tSaved.tRooms[sRoomSlotName].sRoomType = rInfo.roomType
                    tSaved.tRooms[sRoomSlotName].tSaved = {}
                else
                    print('roomtypes'..tostring(tSaved.tRooms[sRoomSlotName].sRoomType))
                    sRoomType = tSaved.tRooms[sRoomSlotName].sRoomType
                end
                if sRoomType then
                    if self.tMap[sRoomSlotName] then                        
                        self.tMap[sRoomSlotName]:onRemoved()                       
                        self.tMap[sRoomSlotName] = nil
                    end
                    -- add in the room
                    local rNewRoom = self:createRoom(self:getRoomDef(sRoomType), rRoomSlotDef, tSaved.tRooms[sRoomSlotName].tSaved)
                    self:setRoomAtSlot(rNewRoom, sRoomSlotName)
                else
                    -- see if a room already exists in this slot
                    if self.tMap[sRoomSlotName] then
                        self.tMap[sRoomSlotName]:setRoomSlotInfo(rRoomSlotDef)
                    else
                        -- add in an emptyroom
                        local rNewRoom = self:createRoom(self:getRoomDef(emptyRoomType), rRoomSlotDef, {}) -- passing empty save table for the empty room
                        self:setRoomAtSlot(rNewRoom, sRoomSlotName)
                    end
                end
            end
        end
    end    

    function Ob:setHub(sHubName, tSaved)
        local rHubDef = self:getBaseDef(sHubName)
        if rHubDef then
            tSaved.sHubName = sHubName
            self.rHubDef = rHubDef
            self:createRooms(rHubDef, tSaved)
        end
    end

    function Ob:getTile(nRow, nCol)
        if self.tBaseTiles[nRow] then
            return self.tBaseTiles[nRow][nCol]
        end 
        return nil
    end

	function Ob:getWorldCoords(nRow, nCol)
        local rTile = self:getTile(nRow, nCol)

        if rTile then
            return rTile:getWorldCoords()
        end
        print("ERROR: Could not find coords for "..nRow.." "..nCol)
        return nil
    end

    function Ob:getTileByCoords(x, y)
        local nScaleX, nScaleY = self:getScl()
        local nTileHalfWidth = ( self.nTileWidth / 2 ) * nScaleX
        local nTileHalfHeight = ( self.nTileHeight / 2 ) * nScaleY
        local nBaseX, nBaseY = self:getLoc()
		local a = x * nTileHalfHeight + nBaseY * nTileHalfWidth - y * nTileHalfWidth - nBaseX * nTileHalfHeight
		local b = 2 * nTileHalfWidth * nTileHalfHeight
		local xPos = a / b
		local yPos = (nBaseY - xPos * nTileHalfHeight - y) / nTileHalfHeight
		local nCol = math.floor(xPos + 0.5) + 1 -- magic numbers, will look into figuring this out later
		local nRow = math.floor(yPos + 0.5) + 2
        local rTile = self:getTile(nRow, nCol)
        if rTile then
            return rTile, nRow, nCol
        else
            return nil
        end
    end

    function Ob:getTileCoords(x, y)
        local nScaleX, nScaleY = self:getScl()
        local nTileHalfWidth = ( self.nTileWidth / 2 ) * nScaleX
        local nTileHalfHeight = ( self.nTileHeight / 2 ) * nScaleY

        local rTile, nRow, nCol = self:getTileByCoords(x, y)
        if rTile == nil then
            print("ERROR: Could not find tile at x:"..x.." y:"..y)
            return nil
        end

		local tileXPos, tileYPos = self:getWorldCoords(nRow, nCol)
        return tileXPos, tileYPos
    end

	function Ob:getTilePosAtWorldPos(x, y)
        local rTile, nRow, nCol = self:getTileByCoords(x, y)
		local x, y = self:getWorldCoords(nRow, nCol)
		return x, y
	end

    function Ob:getRoomCoordAsWorldPos(roomName, row, col)
        local rRoom = self:getRoom(roomName)
        local rTile = rRoom:getTile(row, col)
        return self:getWorldCoords(rTile.nRow, rTile.nCol)
    end
    
    function Ob:getRoomInfo(sRoomTypeName)
        return self:getRoom(sRoomTypeName)
    end

    function Ob:getRoomAtCoord(x, y)
        local rTile, nRow, nCol = self:getTileByCoords(x, y)
        if rTile then
            for i, rRoom in pairs(self.tMap) do
                if rRoom then
                    if rRoom:isInBounds(nRow, nCol) and rRoom:shouldShowRoomDetails() then
                        return rRoom
                    end
                end
            end
        end
        return nil
    end

    function Ob:getOccupyingRoom(x, y)
        local rTile, nRow, nCol = self:getTileByCoords(x, y)
        if rTile then
            for i, rRoom in pairs(self.tMap) do
                if rRoom then
                    if rRoom:isInBounds(nRow, nCol) and rRoom:getRoomType() ~= 'bg' then
                        return rRoom
                    end
                end
            end            
        end
        return nil
    end
    
	function Ob:getRenderPriority(rCharacter, rTile, rRoom)
        if rTile then
            local rOccupyingRoom = nil
            for i, rRoom in pairs(self.tMap) do
                if rRoom then
                    if rRoom:isInBounds(rTile.nRow, rTile.nCol) and rRoom:getRoomType() ~= 'bg' then
                        rOccupyingRoom = rRoom
                    end
                end
            end        
            if rOccupyingRoom then
                local nPri = rOccupyingRoom:getRenderPriority(rTile.nRow, rTile.nCol)
                if nPri then
                    return nPri
                end
            end
        end
        return 100000000
    end

	function Ob:getRoomLevel(sName)
		local rRoomInfo = self:getRoomInfo(sName)        
		local nLevel = 0
		if rRoomInfo then
			nLevel = 1
			if rRoomInfo.nLevel then
				nLevel = rRoomInfo.nLevel
			else
				rRoomInfo.nLevel = nLevel
			end
		end
		return nLevel
	end

	function Ob:getProps(sName)
        -- TODO: prop implementation
        return nil
    end

    function Ob:getRoom(sRoomType)
        -- for now return only one room of type
        for i, rRoom in pairs(self.tMap) do
            if rRoom:getRoomType() == sRoomType then
                return rRoom
            end
        end
        return nil
    end

    function Ob:getAllRooms()
        return self.tMap
    end

    function Ob:getRandomTile(sRoomTypeName)
        local rRoom = self:getRoom(sRoomTypeName)
        if not rRoom then
            return nil
        end            
        local rTile = rRoom:getRandomTile()
        if rTile then
            return rTile, rTile.nRow, rTile.nCol
        end
        return nil
    end

	function Ob:centerOnScreen(offsetX, offsetY, zoomOut, duration)
		Renderer.setCameraLoc(offsetX, offsetY, duration)
		if zoomOut then
			local scale = 2.0 * tMap.width / 640
			Renderer.setCameraScl(scale, scale, duration)
		else
			Renderer.setCameraScl(1, 1, duration)
		end
	end


	function Ob:centerOnRoom(sRoomType, duration)
        if sRoomType then
            local rRoom = self:getRoom(sRoomType)
            if rRoom then
                local x, y = rRoom:getCenterPos()
                Renderer.setCameraLoc(x, y, duration)
            end
        end
	end

    function Ob:getMapExtents()
		local topX, topY = 0, 0
		local downX, downY = 0, 0
		local leftX, leftY = 0, 0
		local rightX, rightY = 0, 0

        if self.tMap then
			
			local nNumRows = g_CurrentMap.nMaxRow 
			if nNumRows > 0 then
				local nWalkRows = nNumRows
				local nValidRows = 0
				while nValidRows == 0 and nWalkRows > 0 do
					local rTestTile = g_CurrentMap:getTile(nWalkRows, 1)
					local numOccupyingTiles = table.getn( rTestTile:getOccupyingTiles() )
					if numOccupyingTiles > 0 then
						nValidRows = nWalkRows
					end					

					nWalkRows = nWalkRows - 1
				end

				local nNumCols = g_CurrentMap.nMaxCol 
				if nNumCols > 0 then
					local nWalkCols = nNumCols
					local nValidCols = 0
					while nValidCols == 0 and nWalkCols > 0 do
						local rTestTile = g_CurrentMap:getTile(1, nWalkCols)
						local numOccupyingTiles = table.getn( rTestTile:getOccupyingTiles() )
						if numOccupyingTiles > 0 then
							nValidCols = nWalkCols
						end					

						nWalkCols = nWalkCols - 1
					end

					-- find the top most tile
					local rTopTile = self:getTile(1, 1)
					topX, topY = rTopTile:getLoc()

					-- find the bottom most tile
					local rBottomTile = self:getTile(nValidRows, nValidCols)
					downX, downY = rBottomTile:getLoc()
					
					-- find the left most tile
					local rLeftTile = self:getTile(nValidRows, 1)
					leftX, leftY = rLeftTile:getLoc()      

					-- find the right most tile
					local rRightTile = self:getTile(1, nValidCols)
					rightX, rightY = rRightTile:getLoc()    


					-- the extents calculate a half tile off, so nudge them back
					local tileX = (rightX - leftX) / nValidRows
					local tileY = (topY - downY) / nValidCols

					topX = topX + (tileX*.5)
					downX = downX + (tileX*.5)
					rightX = rightX + tileX

					topY = topY + tileY 
					downY = downY - tileY 
				end
			end
        end
        
        return topX, topY, downX, downY, leftX, leftY, rightX, rightY
    end
    
    function Ob:createRoomAtRoom(rRoomToRemove, rRoomDef, sRoomDefKey)
        if rRoomToRemove and rRoomDef then
            -- get the offset
            local rRoomSlotInfo = rRoomToRemove:getRoomSlotInfo()
            local sRoomSlotName = rRoomToRemove:getRoomSlotName()
            local tOldProps = rRoomToRemove:getPersistentProps()
            rRoomToRemove:onRemoved()
            self.tMap[sRoomSlotName] = nil
            if not self.tSaveTable.tBases[1].tRooms[sRoomSlotName] then
                self.tSaveTable.tBases[1].tRooms[sRoomSlotName] = {}
            end
            self.tSaveTable.tBases[1].tRooms[sRoomSlotName].sRoomType = sRoomDefKey
            if not self.tSaveTable.tBases[1].tRooms[sRoomSlotName].tSaved then
                self.tSaveTable.tBases[1].tRooms[sRoomSlotName].tSaved = {}
            end
            local rNewRoom = self:createRoom(rRoomDef, rRoomSlotInfo, self.tSaveTable.tBases[1].tRooms[sRoomSlotName].tSaved, tOldProps)
            self:setRoomAtSlot(rNewRoom, sRoomSlotName)
            return rNewRoom
        end
        return nil
    end

    function Ob:updateCharPri()
        -- update all hero priorities
        local tHeroes = g_Game:getActiveHeroes()
        table.insert(tHeroes, g_MiddleManager:getAvatar())
        for i, rHero in ipairs(tHeroes) do
            rHero:updateLayer()
        end
    end

    function Ob:upgradeRoom(rRoom, rRoomDef, sRoomDefKey)
        if rRoom and rRoomDef and sRoomDefKey then
            local nNewRoom = nil
            if rRoomDef then                
                nNewRoom = g_CurrentMap:createRoomAtRoom(rRoom, rRoomDef, sRoomDefKey)
                if rRoomDef.nRoomLevel and rRoomDef.nRoomLevel >= 3 then
                    DFAchievement.updateAchievement('OneRoomMaxLevel', 1)
                    DFAchievement.updateAchievement('AllBaseMaxLevel', rRoomDef.roomtype..rRoomDef.nRoomLevel)
                end
                self:updateCharPri()
            end            
        end
    end

    function Ob:upgradeBase(sBaseDefName)
        if sBaseDefName then
            g_TodoList:onBaseUpgraded()
            self:setHub(sBaseDefName, self.tSaveTable.tBases[1])
            self:updateCharPri()               
            if g_Game then
                g_Game:reEnableProps()
            end
        end
    end

    function Ob:setPropsUnlockedUntilMap(sRoomType, tProps)
        local rRoom = self:getRoom('hub')
        if rRoom and tProps then
            for i, sPropType in ipairs(tProps) do
                local rProp = rRoom:getPropOfType(sPropType)
                if rProp then
                    rProp:unlockProp()
                    table.insert(self.tPropsToBeTempUnlocked, rProp)
                end
            end
        end
    end

    function Ob:onTerritoryMapOpened()
        for i, rProp in ipairs(self.tPropsToBeTempUnlocked) do
            rProp:lockProp()
        end
        self.tPropsToBeTempUnlocked = {}
    end

    -- INITIALIZATION
    Ob.tMap = {}
    -- setup the background
    -- assuming that all rooms have the same tile width and height
    local nFactor = 2
    Ob.nTileWidth = tInitialMap.nTileWidth * 2 - 2 -- temp hack to reduce gaps
    Ob.nTileHeight = tInitialMap.nTileHeight * 2 - 2

    Ob.rPropTouchManager = PropTouchManager:new()
    
    Ob.tBaseTiles = Ob:createBaseTiles(tInitialMap.nMaxRow, tInitialMap.nMaxCol)
    
    Ob.tSaveTable = g_Game:getSaveTable()
    local sHubName = tInitialMap.hub
    if not Ob.tSaveTable.tBases then
        Ob.tSaveTable.tBases = {}
        Ob.tSaveTable.tBases[1] = {}
        Ob.tSaveTable.tBases[1].sHubName = sHubName
    else
        -- for now only one base
        if Ob.tSaveTable.tBases[1].sHubName then
            sHubName = Ob.tSaveTable.tBases[1].sHubName
        end
    end    
    Ob:setHub(sHubName, Ob.tSaveTable.tBases[1])

    Ob.tPropsToBeTempUnlocked = {}

    return Ob
end

return TileMap
