local util = require('DFUtil')
local DFUtil = require('DFCommon.Util')
local DFFile = require('DFCommon.File')
local Item = require('Item')
local TodoListManager = {}
local TodoListItem = {}
local RatingTodoItem = {}
local HubUpgradeTodoItem = {}
local AddRoomTodoItem = {}
local UpgradeRoomTodoItem = {}
local UpgradePropTodoItem = {}
local AddPropTodoItem = {}
local CompleteEventTodoItem = {}
local CollectCoinsTodoItem = {}
local HoodMoraleTodoItem = {}
local DelegateEventTodoItem = {}
local BuyItemTodoItem = {}
local TrainHeroTodoItem = {}
local LevelHeroTodoItem = {}
local ResearchTodoItem = {}

TodoListManager.tTodoInfo = dofile(DFFile.getDataPath("Todo/TodoDefs.lua"))
TodoListManager.tTodoList = TodoListManager.tTodoInfo.tList
TodoListManager.tTodoDefs = TodoListManager.tTodoInfo.tDefs

local kMAX_TODO_ITEMS = 3

function TodoListManager.updateTodoListDef(sDefKey, tValues)
    g_Game:updateDefWithTweak(TodoListManager.tTodoDefs, sDefKey, tValues)    
end

function TodoListItem:new(rDef, tSaved)
    local Ob = {}
    Ob.rDef = rDef
    Ob.tSaved = tSaved

    function Ob:hasCompleted()
        return self.tSaved.bCompleted
    end

    function Ob:setCompleted(bCompleted)
        self.tSaved.bCompleted = bCompleted
    end

    function Ob:getType()
        if self.rDef then
            return self.rDef.type
        end
        return nil
    end

    function Ob:getRewardDef()
        if self.rDef and self.rDef.tPossibleRewards then
            local nPossibility = math.random(100)
            for i, rRewardInfo in ipairs(self.rDef.tPossibleRewards) do
                if nPossibility < rRewardInfo.nPossibility then
                    if rRewardInfo.rDef then
                        return rRewardInfo.rDef
                    end
                end
            end
        end
        return nil
    end

    return Ob
end

function ResearchTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onResearched(sItemKey)
        local rConsumable = Item.getConsumableDef(sItemKey)
        if rConsumable and self.rDef.sItemType == 'Snack' then
            self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
            if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
            return
        end
        local rEquipment = Item.getEquipmentDef(sItemKey)
        if rEquipment and self.rDef.sItemType == 'Equipment' then
            self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
            if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
            return
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.nNumToComplete then
            sString = g_DialogManager:getString(self.rDef.uiLineCode1).." a "..g_DialogManager:getString(self.rDef.uiLineCode2)
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function LevelHeroTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onHeroLevelUp(rHero)
        if rHero and rHero:getStatLevel() >= self.rDef.nNumToComplete then
            self:setCompleted(true)
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode and self.rDef.nNumToComplete then
            sString = g_DialogManager:getString(self.rDef.uiLineCode).." "..self.rDef.nNumToComplete
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function TrainHeroTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onHeroTrained()
        self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
        if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
            self:setCompleted(true)
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.nNumToComplete then
            sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode2)
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function BuyItemTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onConsumableItemPurchased()
        if self.rDef.sItemType == 'Snack' then
            self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
            if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
        end
    end

    function Ob:onEquipmentPurchased()
        if self.rDef.sItemType == 'Equipment' then
            self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
            if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
        end
    end

    function Ob:onCostumePurchased()
        if self.rDef.sItemType == 'Costume' then
            self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
            if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.nNumToComplete then
            if self.rDef.nNumToComplete == 1 then
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." a "..g_DialogManager:getString(self.rDef.uiLineCode2)
            else
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode2)
            end
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function DelegateEventTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onEventDelegated(rEvent)
        self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
        if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
            self:setCompleted(true)
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.uiLineCode3 and self.rDef.nNumToComplete then
            if self.rDef.nNumToComplete > 1 then
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode2)
            else
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode3)
            end
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function HoodMoraleTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onEventRemoved(rEvent, rTerritory)
        if rTerritory and self.rDef.nNumToComplete then
            if rTerritory:getNumEvents() == 0 then
                self.tSaved.nNumCompleted = self.tSaved.nNumCompleted + 1
                if self.tSaved.nNumCompleted >= self.rDef.nNumToComplete then
                    self:setCompleted(true)
                end
            end
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.uiLineCode3 and self.rDef.nNumToComplete then
            if self.rDef.nNumToComplete > 1 then
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode2)
            else
                sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode3)
            end
        end
        return sString        
    end

    if not Ob.tSaved.nNumCompleted then
        Ob.tSaved.nNumCompleted = 0
    end

    return Ob
end

function CollectCoinsTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onMapCoinCollected(nAmount)
        if nAmount then
            self.tSaved.nAmountCollected = self.tSaved.nAmountCollected + nAmount
            if self.tSaved.nAmountCollected >= self.rDef.nNumToComplete then
                self:setCompleted(true)
            end
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode1 and self.rDef.uiLineCode2 and self.rDef.nNumToComplete then
            sString = g_DialogManager:getString(self.rDef.uiLineCode1).." "..self.rDef.nNumToComplete.." "..g_DialogManager:getString(self.rDef.uiLineCode2)
        end
        return sString        
    end

    if not Ob.tSaved.nAmountCollected then
        Ob.tSaved.nAmountCollected = 0
    end

    return Ob
end

function RatingTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onRatingScoreChanged(nAmount)
        if not self:hasCompleted() then
            self.tSaved.nAccumulatedScore = self.tSaved.nAccumulatedScore + nAmount
            if self.rDef.nRatingAmount then
                if self.tSaved.nAccumulatedScore >= self.rDef.nRatingAmount then
                    self:setCompleted(true)
                end
            end
        end
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode and self.rDef.nRatingAmount then
            sString = g_DialogManager:getString(self.rDef.uiLineCode).." "..self.rDef.nRatingAmount
        end
        return sString
    end

    if not Ob.tSaved.nAccumulatedScore then
        Ob.tSaved.nAccumulatedScore = 0
    end

    return Ob
end

function HubUpgradeTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onBaseUpgraded()
        self:setCompleted(true)
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode then
            sString = g_DialogManager:getString(self.rDef.uiLineCode)
        end

        return sString
    end

    return Ob
end

function AddRoomTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onNewRoomAdded()
        self:setCompleted(true)        
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode then
            sString = g_DialogManager:getString(self.rDef.uiLineCode)
        end

        return sString
    end

    return Ob
end

function UpgradeRoomTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onRoomUpgraded()
        self:setCompleted(true)
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode then
            sString = g_DialogManager:getString(self.rDef.uiLineCode)
        end
        
        return sString
    end

    return Ob
end

function UpgradePropTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onPropUpgraded()
        self:setCompleted(true)
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode then
            sString = g_DialogManager:getString(self.rDef.uiLineCode)
        end
        
        return sString
    end

    return Ob
end

function AddPropTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:onPropAdded()
        self:setCompleted(true)
    end

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef.uiLineCode then
            sString = g_DialogManager:getString(self.rDef.uiLineCode)
        end
        
        return sString
    end

    return Ob
end

function CompleteEventTodoItem:new(rDef, tSaved)
    local Ob = TodoListItem:new(rDef, tSaved)

    function Ob:getUIDescription()
        local sString = ""
        if self.rDef then
            local sCrimeType = ""
            if self.rDef.sEventType == 'crime' then
                sCrimeType = g_DialogManager:getString("TODO001TEXT")
            elseif self.rDef.sEventType == 'fire' then
                sCrimeType = g_DialogManager:getString("TODO004TEXT")
            elseif self.rDef.sEventType == 'busrescue' then
                sCrimeType = g_DialogManager:getString("TODO005TEXT")
            end
            sString = g_DialogManager:getString("TODO003TEXT").." "..self.rDef.nNumToComplete.." "..sCrimeType.." "..g_DialogManager:getString("TODO002TEXT").." "..self.tSaved.nNumCompletedEvents.."/"..self.rDef.nNumToComplete
        end
        return sString
    end

    function Ob:onEventEnded(rEvent)
        if not self:hasCompleted() then
            if rEvent and rEvent:isSuccessful() and self.rDef and self.rDef.sEventType and self.rDef.nNumToComplete then
                if rEvent:getEventType() == self.rDef.sEventType then
                    self.tSaved.nNumCompletedEvents = self.tSaved.nNumCompletedEvents + 1
                    if self.tSaved.nNumCompletedEvents >= self.rDef.nNumToComplete then
                        self:setCompleted(true)
                    end
                end
            end
        end
    end

    if not Ob.tSaved.nNumCompletedEvents then
        Ob.tSaved.nNumCompletedEvents = 0
    end

    return Ob
end

local tTodoClasses =
{
    RatingTodoItem = RatingTodoItem,
    HubUpgradeTodoItem = HubUpgradeTodoItem,
    AddRoomTodoItem = AddRoomTodoItem,
    UpgradeRoomTodoItem = UpgradeRoomTodoItem,
    UpgradePropTodoItem = UpgradePropTodoItem,
    AddPropTodoItem = AddPropTodoItem,
    CompleteEventTodoItem = CompleteEventTodoItem,
    CollectCoinsTodoItem = CollectCoinsTodoItem,
    HoodMoraleTodoItem = HoodMoraleTodoItem,
    DelegateEventTodoItem = DelegateEventTodoItem,
    BuyItemTodoItem = BuyItemTodoItem,
    TrainHeroTodoItem = TrainHeroTodoItem,
    LevelHeroTodoItem = LevelHeroTodoItem,
    ResearchTodoItem = ResearchTodoItem,
}

function TodoListManager:init()
    local Ob = {} 

    function Ob:getTodoList()
        return self.tTodoList
    end

    function Ob:canAddItem(sTodoDefKey)
        if TodoListManager.tTodoDefs and sTodoDefKey then
            local rDef = TodoListManager.tTodoDefs[sTodoDefKey]
            if rDef then
                if rDef.type == 'HubUpgradeTodoItem' then
                    -- check to see if the hub is upgradable
                    local rHubDef = g_CurrentMap:getRoomInfo('hub')
                    if rHubDef then
                        if rHubDef.baseUpgrade then
                            if g_MiddleManager:getStarRatingLevel() >= 3 then
                                -- upgradeable
                                return true
                            end
                        else
                            return false
                        end
                    else
                        return false
                    end
                elseif rDef.type == 'AddRoomTodoItem' then
                    -- check to see if there's any rooms to add
                    if g_CurrentMap:getRoom('emptyroom') then
                        return true
                    else
                        return false
                    end
                elseif rDef.type == 'UpgradeRoomTodoItem' then
                    if g_MiddleManager:getStarRatingLevel() < 3 then
                        return false
                    end
                    -- see if any rooms are upgradeable
                    local tRooms = g_CurrentMap:getAllRooms()
                    local bUpgradeAvailable = false
                    if tRooms then
                        for i, rRoom in pairs(tRooms) do
                            if rRoom:getRoomUpgradeDefs() then
                                bUpgradeAvailable = true
                                break
                            end
                        end
                    end
                    if bUpgradeAvailable then
                        return true
                    else
                        return false
                    end
                elseif rDef.type == 'UpgradePropTodoItem' then
                    local tRooms = g_CurrentMap:getAllRooms()
                    local bUpgradeAvailable = false
                    if tRooms then
                        for i, rRoom in pairs(tRooms) do
                            local tProps = rRoom:getProps()    
                            if tProps then
                                for i, rProp in ipairs(tProps) do
                                    if rProp:getUpgradeDefKey() then
                                        bUpgradeAvailable = true
                                        break
                                    end
                                end
                            end
                        end
                    end
                    if bUpgradeAvailable then
                        return true
                    else
                        return false
                    end
                elseif rDef.type == 'AddPropTodoItem' then
                    local tRooms = g_CurrentMap:getAllRooms()
                    local bUpgradeAvailable = false
                    if tRooms then
                        for i, rRoom in pairs(tRooms) do
                            if rRoom:getRoomType() ~= 'hub' and rRoom:getRoomType() ~= 'bg' and rRoom:getRoomType() ~= 'mm' then
                                local tProps = rRoom:getProps()    
                                if tProps then
                                    for i, rProp in ipairs(tProps) do
                                        if not rProp:isUnlocked() then
                                            bUpgradeAvailable = true
                                            break
                                        end
                                    end
                                end
                            end
                        end
                    end
                    if bUpgradeAvailable then
                        return true
                    else
                        return false
                    end                    
                elseif rDef.type == 'CompleteEventTodoItem' then
                    -- see if the event is unlocked
                    if rDef.nReqTerritoryUnlockLevel then
                        if g_MiddleManager:getLevel() < rDef.nReqTerritoryUnlockLevel then
                            return false
                        end
                    end
                elseif rDef.type == 'HoodMoraleTodoItem' then
                    local tTerritories = g_City:getUnlockedTerritories()
                    local nNumUnhappy = 0
                    if tTerritories then
                        for i, rTerritory in ipairs(tTerritories) do
                            local rInfo = rTerritory:getTerritoryHappinessInfo()
                            if rInfo then
                                if rTerritory:getTerritoryHappiness() > 1 then
                                    nNumUnhappy = nNumUnhappy + 1
                                end
                            end
                        end
                    end
                    if nNumUnhappy >= rDef.nNumToComplete then
                        return true
                    else
                        return false
                    end
                elseif rDef.type == 'LevelHeroTodoItem' then
                    if rDef.nMinHeroLevel then
                        local nMinLevel = 99
                        local tHeroes = g_Game:getAllHeroes()
                        for i, rHero in pairs(tHeroes) do
                            if rHero:getStatLevel() < nMinLevel then
                                nMinLevel = rHero:getStatLevel()
                            end
                        end    
                        if nMinLevel >= rDef.nMinHeroLevel then
                            return true
                        else
                            return false
                        end
                    end
                elseif rDef.type == 'BuyItemTodoItem' then
                    if rDef.sItemType == 'Costume' and rDef.nMinHeroLevel then
                        local nMinLevel = 99
                        local tHeroes = g_Game:getAllHeroes()
                        for i, rHero in pairs(tHeroes) do
                            if rHero:getStatLevel() < nMinLevel then
                                nMinLevel = rHero:getStatLevel()
                            end
                        end    
                        if nMinLevel >= rDef.nMinHeroLevel then
                            return true
                        else
                            return false
                        end
                    end
                elseif rDef.type == 'ResearchTodoItem' then
                    if g_CurrentMap:getRoom('lab') then
                        if g_MiddleManager then
                            if rDef.sItemType then
                                if rDef.sItemType == 'Equipment' then
                                    return g_MiddleManager:hasEquipmentToResearch()
                                elseif rDef.sItemType == 'Snack' then
                                    return g_MiddleManager:hasConsumableToResearch()
                                end
                            end
                        end
                        return true
                    end
                    if g_CurrentMap:getRoom('emptyroom') then
                        return true
                    else
                        return false
                    end
                end
            end
        end
        return true -- true for now
    end

    function Ob:createTodoItem(sTodoDefKey, tSaved)
        if TodoListManager.tTodoDefs and sTodoDefKey and tSaved then
            local rDef = TodoListManager.tTodoDefs[sTodoDefKey]
            if rDef.type then
                if tTodoClasses[rDef.type] then
                    local constructorFncn = tTodoClasses[rDef.type].new
                    local rItem = constructorFncn(tTodoClasses[rDef.type], rDef, tSaved)
                    return rItem
                end
            end
        end
        return nil
    end

    function Ob:generateTodoList()
        if not TodoListManager.tTodoList then
            return
        end

        local tPossibleItemList = {}
        -- cull out the list
        for i, sTodoDefKey in ipairs(TodoListManager.tTodoList) do
            local bFound = false
            -- make sure we don't have dupes
            local rDef = TodoListManager.tTodoDefs[sTodoDefKey]
            if rDef then
                for c, rTodoItem in ipairs(self.tTodoList) do
                    if rTodoItem:getType() == rDef.type then
                        bFound = true
                        break
                    end
                end
                if not bFound then
                    if self:canAddItem(sTodoDefKey) then
                        table.insert(tPossibleItemList, sTodoDefKey)
                    end
                end
            end
        end

        local nNumItems = table.getn(self.tTodoList)
        if table.getn(tPossibleItemList) >= (kMAX_TODO_ITEMS - nNumItems) then
            while nNumItems < kMAX_TODO_ITEMS do
                -- get a random def
                local nNumPossibleItems = table.getn(tPossibleItemList)
                local nRandomIndex = math.random(nNumPossibleItems)
                local sTodoDefKey = tPossibleItemList[nRandomIndex]
                table.remove(tPossibleItemList, nRandomIndex)
                local rNewTodoInfo = {}
                rNewTodoInfo.sType = sTodoDefKey
                rNewTodoInfo.tSaved = {}
                local rItem = self:createTodoItem(rNewTodoInfo.sType, rNewTodoInfo.tSaved)
                if rItem then
                    table.insert(self.tTodoList, rItem)
                    table.insert(self.tSaved.tTodoList, rNewTodoInfo)
                end
                nNumItems = nNumItems + 1
            end            
        end
    end

    function Ob:removeTodoListItem(rItem)
        if rItem then
            local nNumItems = table.getn(self.tTodoList)
            for i, rExistingItem in ipairs(self.tTodoList) do
                if rItem == rExistingItem then
                    table.remove(self.tTodoList, i)
                    table.remove(self.tSaved.tTodoList, i)
                    self:generateTodoList()
                    if self.tTodoList[nNumItems] then -- since the last item is the newest one
                        return self.tTodoList[nNumItems]
                    end
                end
            end
        end
    end

    Ob.tTodoList = {}

    local tSaveTable = g_Game:getSaveTable() -- we trust that this will always return a table
    if not tSaveTable.tTodoList then
        tSaveTable.tTodoList = {}
    end
    Ob.tSaved = tSaveTable.tTodoList
    
    if not Ob.tSaved.tTodoList then
        Ob.tSaved.tTodoList = {}
    else
        for i, rTodoInfo in ipairs(Ob.tSaved.tTodoList) do
            local rItem = Ob:createTodoItem(rTodoInfo.sType, rTodoInfo.tSaved)
            if rItem then
                table.insert(Ob.tTodoList, rItem)
            end
        end
    end

    Ob:generateTodoList()

    function Ob:getNumItemsCompleted()
        local nNumCompleted = 0
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem:hasCompleted() then
                nNumCompleted = nNumCompleted + 1
            end
        end
        return nNumCompleted
    end

    -- listener functions/callbacks
    function Ob:onRatingScoreChanged(nAmount)
        if nAmount then
            for i, rTodoItem in ipairs(self.tTodoList) do
                if rTodoItem.onRatingScoreChanged then
                    rTodoItem:onRatingScoreChanged(nAmount)
                end
            end
        end
    end

    function Ob:onEventEnded(rEvent)
        if rEvent then
            for i, rTodoItem in ipairs(self.tTodoList) do
                if rTodoItem.onEventEnded then
                    rTodoItem:onEventEnded(rEvent)
                end
            end
        end
    end

    function Ob:onEventRemoved(rEvent, rTerritory)
        if rEvent then
            for i, rTodoItem in ipairs(self.tTodoList) do
                if rTodoItem.onEventRemoved then
                    rTodoItem:onEventRemoved(rEvent, rTerritory)
                end
            end
        end        
    end

    function Ob:onBaseUpgraded()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onBaseUpgraded then
                rTodoItem:onBaseUpgraded()
            end
        end
    end

    function Ob:onNewRoomAdded()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onNewRoomAdded then
                rTodoItem:onNewRoomAdded()
            end
        end        
    end

    function Ob:onRoomUpgraded(rRoom)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onRoomUpgraded then
                rTodoItem:onRoomUpgraded()
            end
        end
    end

    function Ob:onPropUpgraded(rProp)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onPropUpgraded then
                rTodoItem:onPropUpgraded()
            end
        end        
    end

    function Ob:onPropAdded(rProp)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onPropAdded then
                rTodoItem:onPropAdded()
            end
        end        
    end

    function Ob:onMapCoinCollected(nAmount)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onMapCoinCollected then
                rTodoItem:onMapCoinCollected(nAmount)
            end
        end        
    end

    function Ob:onEventDelegated(rEvent)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onEventDelegated then
                rTodoItem:onEventDelegated(rEvent)
            end
        end        
    end

    function Ob:onConsumableItemPurchased()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onConsumableItemPurchased then
                rTodoItem:onConsumableItemPurchased()
            end
        end                
    end

    function Ob:onEquipmentPurchased()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onEquipmentPurchased then
                rTodoItem:onEquipmentPurchased()
            end
        end                
    end    

    function Ob:onCostumePurchased()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onCostumePurchased then
                rTodoItem:onCostumePurchased()
            end
        end                
    end    

    function Ob:onHeroTrained()
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onHeroTrained then
                rTodoItem:onHeroTrained()
            end
        end    
    end

    function Ob:onHeroLevelUp(rHero)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onHeroLevelUp then
                rTodoItem:onHeroLevelUp(rHero)
            end
        end
    end

    function Ob:onResearched(sItemKey)
        for i, rTodoItem in ipairs(self.tTodoList) do
            if rTodoItem.onResearched then
                rTodoItem:onResearched(sItemKey)
            end
        end                
    end

    return Ob
end

return TodoListManager