local util = require("DFUtil")
local WalkToTask = require("WalkToTask")
local WalkToRoomTask = {}

kTIMESTEP = 0.03

function WalkToRoomTask:new(rActivity, sRoomType)
	local Ob = WalkToTask:new(rActivity, 'WalkToRoomTask')
    Ob.rTargetRoom = g_CurrentMap:getRoom(sRoomType)
	Ob.arrived = false
	Ob.state = 'initial'

	function Ob:Execute()
        if not self.rCharacter then
            self.arrived = true
            return
        end

        local _, nSourceRow, nSourceCol = self.rCharacter:getOccupyingTile()

        local rRoom = self.rCharacter:getOccupyingRoom()
        if not rRoom then
            self.arrived = true
            return
        end
        -- exiting
        if rRoom == self.rTargetRoom then
            self.arrived = true
            return            
        end

        if rRoom ~= self.rTargetRoom and rRoom:getRoomType() ~= 'hub' then
            -- walk to the door loc
            local nPreDoorRow, nPreDoorCol = rRoom:getPreDoorLoc()
            -- walk to the door
            local nDoorRow, nDoorCol = rRoom:getDoorLoc()
            local _, nCurRow, nCurCol = self.rCharacter:getOccupyingTile()
            
            if nPreDoorRow and nPreDoorCol then
                self:WalkToTile(nPreDoorRow, nPreDoorCol, rRoom:shouldTeleportToDoor())
            end

            if nPreDoorCol and nDoorCol and nDoorRow then
                if nPreDoorCol <= nDoorCol then
                    self:WalkToTile(nDoorRow, nDoorCol - 1)
                else
                    self:WalkToTile(nDoorRow, nDoorCol + 1)
                end
            end
            
            if rRoom then
                rRoom:openDoor(true)
            end

            if nDoorRow and nDoorCol then
                local nExitDoorOffsetRow, nExitDoorOffsetCol = rRoom:getDoorEntryOffsets()
                self:WalkToTile(nDoorRow + nExitDoorOffsetRow, nDoorCol + nExitDoorOffsetCol, rRoom:shouldTeleportToDoor())
            end

            -- move to the middle more
            _, nCurRow, nCurCol = self.rCharacter:getOccupyingTile()
            if nSourceCol <= nCurCol then
                self:WalkToTile(nCurRow, nCurCol + 3)
            else
                self:WalkToTile(nCurRow, nCurCol - 3)
            end
        end

        -- at hub
        if self.rTargetRoom:getRoomType() == 'hub' then
            self.arrived = true
            return
        end

        -- walk to the door
        local nTargetRow, nTargetCol = self.rTargetRoom:getDoorLoc()
        local _, nCurRow, nCurCol = self.rCharacter:getOccupyingTile()
        local nOffsetCol = 0
        if nCurCol < nTargetCol then
            nOffsetCol = nOffsetCol - 1
        else
            nOffsetCol = nOffsetCol + 1
        end
        self:WalkToTile(nTargetRow, nCurCol, nil, 0, 0)
        self:WalkToTile(nTargetRow, nTargetCol + nOffsetCol, nil, 0, 0)

        if self.rTargetRoom then
            self.rTargetRoom:openDoor()
            local nExitDoorOffsetRow, nExitDoorOffsetCol = self.rTargetRoom:getDoorEntryOffsets()
            self:WalkToTile(nTargetRow + nExitDoorOffsetRow, nTargetCol + nExitDoorOffsetCol, nil, 0, 0)
        end

        -- walk to the pre door loc
        local nTargetRow, nTargetCol = self.rTargetRoom:getPreDoorLoc()
        if nTargetRow and nTargetCol then
            self:WalkToTile(nTargetRow, nTargetCol, self.rTargetRoom:shouldTeleportToDoor(), 0, 0)
        end

        self.arrived = true
        return 
	end
	
	function Ob:IsDone()
		return self.arrived
	end

	return Ob
end

return WalkToRoomTask