local util = require("DFUtil")

local Activity = {
    kACTIVITY_DURATION = 15,
    kPP_PER_MIN = 0.15,
}

function Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = {}
    Ob.__typename = "Activity"
	Ob.rHero = rHero
    Ob.rDef = rDef
	Ob.tTasks = {}
    Ob.bStarted = false
    Ob.tSaved = tSaved
    Ob.tSaved.bRunUntilSuccess = bRunUntilSuccess
	
	--print("creating Activity: "..sActivityType)

	function Ob:Run()
		Ob:EndRun()
	end
	
	function Ob:EndRun()
		while GetGameState() ~= "Run" do
			util.sleep(0.1)
		end
		
		if Ob.sActivityType == "Test" then
			while true do
				util.sleep(0.5)
			end
		end
		
		Ob:End()
	end
	
	function Ob:AddTask(task)
		table.insert(Ob.tTasks, task)
	end
	
	function Ob:Execute()
		local executeTasks = #Ob.tTasks > 0
		while executeTasks do
			local numTasks = #Ob.tTasks
			if numTasks > 0 then
				local taskIndex = 1
				local task = Ob.tTasks[taskIndex]
				task:Execute()
				if task:IsDone() then
					table.remove(Ob.tTasks, taskIndex)
				end
			end
			executeTasks = #Ob.tTasks > 0
			util.sleep(0)
		end
		
		Ob:Run()
	end
	
	function Ob:Start()
        if not self.bStarted then
            self.rThread = MOAIThread.new()
            self.rThread:run(self.Execute, self)
            if self.OnStart then
                self:OnStart()
            end
            self.bStarted = true
        end
	end

    function Ob:hasStarted()
        return self.bStarted
    end

	function Ob:End(bForceKilled)
		if self.OnEnd then
			self:OnEnd(bForceKilled)
		end
		if self.rHero then
			self.rHero:onActivityEnd(self, bForceKilled)
		end
		if bForceKilled then
			if self.rThread then
				self.rThread:stop()
			end
		end
		
		if not bForceKilled then
			g_Game:onActivityEnd(self)
		end
	end

	function Ob:getOwnerHero()
		return self.rHero
	end

	function Ob:getActivityType()
		return self.rDef.sActivityType
	end

	function Ob:getIconPath()
		return self.rDef.iconPath
	end

    function Ob:getCurActiveIconPath()
        return self.rDef.curActiveIconPath
    end

	function Ob:getIndex()
		return self.nActivityIndex
	end

	function Ob:setIndex(nIndex)
		self.nActivityIndex = nIndex
	end

	function Ob:getActivityUIName()
		return self.rDef.sActivityUIName
	end

    function Ob:shouldRunUntilSuccess()
        return self.tSaved.bRunUntilSuccess
    end

    function Ob:getShortDuration()
        return self.rDef.nShortDuration
    end
    
    function Ob:computeRushCost(seconds)
        return math.min( math.ceil( (seconds / 60) * Activity.kPP_PER_MIN ), 9 )
    end

	return Ob
end

return Activity