local MMMoraleActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local UIProgressBar = require('ui/UIProgressBar')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local DFGraphics = require('DFCommon.Graphics')
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

local kHERO_IDLE_TIME = 5

function MMMoraleActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved, sDefKey)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved
    if sDefKey then
        Ob.tSaved.sDefKey = sDefKey
    end

    function Ob:getOtherHero()
        return self.rOtherHero
    end

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        local nRow, nCol = self.rProp:getInteractTileCoords()
        local orgX, orgY = g_CurrentMap:getWorldCoords(nRow, nCol)
        self.rHero:setPriority(self.rProp:getInteractPri())
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()

        self.sExtraSpritePath = 'Levels/Tilesets/BaseProps2'..Renderer.getTextureSizeSuffix()
        local rSpriteSheet = DFGraphics.loadSpriteSheet(self.sExtraSpritePath, g_IsInStartupSequence)
        if rSpriteSheet then
            self.rExtraProp = MOAIProp2D:new()
            self.rExtraProp:setDeck(rSpriteSheet)
            local nIndex = rSpriteSheet.names['MM_ChartMorale'..Renderer.getTextureSizeSuffix()..'_1']
            self.rExtraProp:setIndex(nIndex)
            self.rLayer = Renderer.getRenderLayer('GameplayOverlay')
            self.rLayer:insertProp(self.rExtraProp)
            self.rExtraProp:setLoc(orgX + 50, orgY - 30)
        end

        self.rOtherHero = g_Game:getHero(self.tSaved.sDefKey)
        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end

        self.rOtherHero:runActivities(false)
        self.rOtherHero:setSelectable(false)
        util.sleep(0.2)    

        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end    

        local rTile = self.rRoom:getTile(6, 5)
        if rTile then
            local x, y = g_CurrentMap:getWorldCoords(rTile.nRow, rTile.nCol)
            self.rOtherHero:setPosition(x, y, true)
        end
        self.rOtherHero:playAnimation('enter', true, true)
        self.rOtherHero:playAnimation('idlebk1', false, true)

        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end

        local sKey = 'Morale'..self.rOtherHero:getCharDefKey()

        local nTimeSinceHeroIdle = 0
		self.rHero:playAnimation('desktalk', false, bFlipX)

        self.nDuration = self.rDef.nDuration * (1 / g_Game:getDebugTimeScale())
        local nSpeedMod = self.rOtherHero:getActivitySpeedMod(self.rDef.sActivityType)
        if nSpeedMod > 0 then
            self.nDuration = self.nDuration * (1 - (math.min(100, nSpeedMod) / 100))
        end

        self.nElapsedTime = self.rHero:getActivityProgress(sKey) * self.nDuration
        self.bDone = false
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange then
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress(sKey, self.nElapsedTime / self.nDuration)

            nTimeSinceHeroIdle = nTimeSinceHeroIdle + 1
            if nTimeSinceHeroIdle >= kHERO_IDLE_TIME then
                nTimeSinceHeroIdle = 0
                self.rOtherHero:playAnimation('idlebk1', false, true)
            end 
            self.rOtherHero:setOverlayText("MOTIVATING", 3, {0, 1, 0})
            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end
            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)

            -- check on the hero
            if not self.rOtherHero:isActive() then
                self:EndRun()
                return
            end
        end

        self.rHero:setActivityProgress(sKey, 0)

        self:randomizeMorale()

		self:EndRun()	
	end
    
    function Ob:randomizeMorale()
        self.rOtherHero:improveMorale()
        self.rOtherHero:setOverlayText("MORALE+", 1, {0, 0, 1})
    end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max(self.nDuration - self.nElapsedTime, 0)
    end

    function Ob:getActivityProgress()
        if self:getRemainingTime() == 0 then
            return 100
        end
        local nTotalTime = self.rDef.nDuration or 1
        return (((self.nElapsedTime or 0) / nTotalTime) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end
    
    function Ob:rushActivity()
        self.bDone = true
        DFAnalytics.logEvent("act_rshd", { act = "mrle", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_mrle_rshd", { hero = self.tSaved.sDefKey, cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_mrle_rshd", 1)        
        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end
    
	function Ob:OnStart()
		local propName = "mmcomp"
        self.rRoom = g_CurrentMap:getRoom("mm")
        if not self.rRoom then
            -- exit
        end

        self.tSaved.bInRoom = true                
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_mrle", { hero = self.tSaved.sDefKey } )
            DFAnalytics.incrStat("act_mrle", 1)
            self.tSaved.bStarted = true
        end           
        self.rAvailableProp = self.rRoom:getNextAvailablePropOfType('mmcomp')
        self.rAvailableProp:addUser(self.rHero)
	end

	function Ob:OnEnd()
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end

        if self.rExtraProp and self.rLayer and self.sExtraSpritePath then
            self.rLayer:removeProp(self.rExtraProp)
            DFGraphics.unloadSpriteSheet(self.sExtraSpritePath)
        end

        if self.rOtherHero then
            if self.rOtherHero:isActive() then
                self.rOtherHero:runActivities(true)
            end
            self.rOtherHero:setSelectable(true)
        end
        self.tSaved.bStarted = false
	end

	return Ob
end

return MMMoraleActivity