local MMPTJobActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local UIProgressBar = require('ui/UIProgressBar')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

function MMPTJobActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved

	function Ob:Run()
        if not self.rAvailableProp then
            self:EndRun()
            return
        end

        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        local nRow, nCol = self.rProp:getInteractTileCoords()
		local orgX, orgY = g_CurrentMap:getWorldCoords(nRow, nCol)
        self.rHero:setPriority(self.rProp:getInteractPri())
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)

        self.nProgressPerSec = self.rProp:getPropDef().nCashPerSec
        self.nDuration = self.rDef.nDuration
        self.nMaxCash = self.nProgressPerSec * self.nDuration
		-- show the progress

        if self.nProgressPerSec == 0 then
            self.nProgressPerSec = 1
        end
        if not self.tSaved.nAmountCreated then
            self.tSaved.nAmountCreated = 0
        end
        local sActivityName = self:getActivityType()

        self.nProgress = 0
        self.bDone = false
        self.nElapsedTime = self:getActivityProgress('MMPTJob') * self.nDuration
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange then
                self.tSaved.nLastTimeChange = nCurTime
            end
            local nChange = math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.nElapsedTime = self.nElapsedTime + nChange
            self.rHero:setActivityProgress('MMPTJob', self.nElapsedTime / self.nDuration)

            self.nProgress = self.nProgress + (self.nProgressPerSec * nChange)
            local nAmountToAdd = math.floor(self.nProgress)
            if nAmountToAdd > 0 then
                if (self.tSaved.nAmountCreated + nAmountToAdd) >= self.nMaxCash then
                    nAmountToAdd = self.nMaxCash - self.tSaved.nAmountCreated
                end
                self.tSaved.nAmountCreated = self.tSaved.nAmountCreated + nAmountToAdd
                self:addCash(math.floor(nAmountToAdd))
                self.nProgress = self.nProgress - nAmountToAdd
                if self.tSaved.nAmountCreated >= self.nMaxCash then
                    self.bDone = true
                    self.rHero:setOverlayText(self.nMaxCash.." COINS RAISED", 1, {1, 0, 0})
                else
                    self.rHero:setOverlayText("COINS +"..nAmountToAdd, 1, {0, 1, 0})
                end
            end

            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end

		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nMaxCash or not self.tSaved or not self.tSaved.nAmountCreated or not self.nProgressPerSec then
            return 0
        end
        return math.max((self.nMaxCash - (self.tSaved.nAmountCreated + self.nProgress)) / self.nProgressPerSec, 0)
    end

    function Ob:getActivityProgress()
        if self:getRemainingTime() == 0 then
            return 100
        end
        local nTotalTime = self.rDef.nDuration or 1
        return (((self.nElapsedTime or 0) / nTotalTime) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.bDone = true
        local nAmountToAdd = self.nMaxCash - self.tSaved.nAmountCreated    
        nAmountToAdd = math.floor(nAmountToAdd)
        self:addCash(nAmountToAdd)
        self.rHero:setOverlayText(nAmountToAdd.." COINS RAISED", 1, {1, 0, 0})
        
        DFAnalytics.logEvent("act_rshd", { act = "mmjb", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_mmjb_rushed", { cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_mmjb_rushed", 1)                    

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end
    
    function Ob:addCash(nAmountToAdd)
        g_Game:addCash(nAmountToAdd, true)     
    end

	function Ob:OnStart()
		local propName = "mmcomp"
        self.rRoom = g_CurrentMap:getRoom("mm")
        if not self.rRoom then
            -- exit
        end
 
        self.tSaved.bInRoom = true                
        if not self.tSaved.bStarted then
            -- BEN-NOTE: we are seeing this event roughly 5 times per session from our users; we'll just need to rely on the stats instead.
            --DFAnalytics.logEvent("act_mmjb" )
            DFAnalytics.incrStat("act_mmjb", 1)
            self.tSaved.bStarted = true
        end           
        self.rAvailableProp = self.rRoom:getNextAvailablePropOfType('mmcomp')   
        if self.rAvailableProp then
            self.rAvailableProp:addUser(self.rHero)   
        end
	end

	function Ob:OnEnd()
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end

        -- incr general stat here to avoid spamming analytics
        DFAnalytics.incrStat("csh_spnt_total", self.tSaved.nAmountCreated)
        DFAnalytics.incrStat('csh_ernd_mmjb', self.tSaved.nAmountCreated)
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return MMPTJobActivity