local MMPromoteActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local UIProgressBar = require('ui/UIProgressBar')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local DFGraphics = require('DFCommon.Graphics')
local Renderer = require('Renderer')
local Character = require('Character')

local PromotionResponseDefs =
{
    managerLines =
    {
        "BULLET001TEXT", "BULLET004TEXT", "BULLET006TEXT", "BULLET007TEXT", "BULLET008TEXT", 
        "BULLET009TEXT", "BULLET010TEXT", "BULLET011TEXT", "BULLET013TEXT",
    },
    
	-- capt. premium
    heroResponse = 
    {
        -- capt. premium
        TestMale01c =
        {
            "PROM013CPRM", "PROM017CPRM",
        },
        -- hothead
        TestMale02 =
        {
            "PROM014HOTH","PROM019HOTH", "PROM023HOTH",
        },
        -- galaxy girl
        TestFemale01 =
        { 
            "PROM006GALX", "PROM018GALX",
        },	
        -- masked mummy
        TestMale01b =
        {
            "PROM004MUMM", "PROM021MUMM",
        },
        -- @man
        TestMale02b =
        {
            "PROM015TECH", "PROM016TECH", "PROM022TECH",
        },
        -- crimebot
        Robot01 =
        {
            "PROM011CBOT", "PROM012CBOT", "PROM028CBOT", 
        },	
        -- surge protector
        TestFemale02 =
        {
            "PROM007SURG", "PROM020SURG", "PROM025SURG",
        },	
        -- gloomerang
        TestFemale02b =
        {
            "PROM030GLOM", "PROM031GLOM", "PROM032GLOM",
        },
        -- sweet justice
        TestMale01 =
        {
            "PROM008FBOY", "PROM009FBOY", "PROM010FBOY",
        },
        -- cosmic crowbar
        TestMale01d =
        {
            "PROM026CROW", "PROM027CROW", "PROM029CROW",
        },
    },
}

local kHERO_IDLE_TIME = 5

function MMPromoteActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved, sDefKey)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved
    if sDefKey then
        Ob.tSaved.sDefKey = sDefKey
    end

    function Ob:csMode(hero, bCS, row, col, facing)
        if hero ~= self.rHero then hero:runActivities(not bCS) end
        hero:setPlayEnterAnimForNothingActivity(false)
        hero:setSelectable(not bCS)
        if bCS then
            hero:setFacingDirection(facing)
            local breathe = 'breathe'
            local xflip = false
            if facing == "NE" then
                breathe = 'breathe_reverse'
                xflip = true
            end
            hero:playAnimation(breathe, false, xflip)
            hero:setPosition(g_CurrentMap:getRoomCoordAsWorldPos('hub',row, col))
        end
    end

	function Ob:Run()
        self.rOtherHero = g_Game:getHero(self.tSaved.sDefKey)        
        local props = {"congratulationsbanner","congratulationsdecor","congratulationsdecor2","congratulationscaketable","congratulationsballoon",}
        g_CurrentMap:setPropsUnlockedUntilMap('hub', props)

        g_GUI:GetGameOverlayUI():setVisible(false)
        for k,v in pairs(g_Game.tEnabledActions) do
            g_Game.tEnabledActions[k] = false
        end
        g_City:enableEventGeneration(false)  
    
        g_Audio:stopMusic()
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Victory', false)

        self:csMode(self.rHero, true, 3, 6, "SW")
        self:csMode(self.rOtherHero, true, 3, 4, "SW")

	    self.rOtherHero:centerOnScreen()

        local tAllHeroes = g_Game:getActiveHeroes()
        self.tUsedHeroes = {}
        table.insert(self.tUsedHeroes,self.rHero)
        table.insert(self.tUsedHeroes,self.rOtherHero)
        local row = 3
        for i,v in ipairs(tAllHeroes) do
            if not v:isAssignedToEvent() and v ~= self.rOtherHero then
                self:csMode(v, true, 6, row, "NE")
                table.insert(self.tUsedHeroes, v)
                row = row+1
            end
        end

        g_GUI:ShowLevelUpUI(self.rOtherHero)
        
        while g_GUI:GetActivePane() ~= g_GUI.rGameOverlayUI do
            coroutine.yield()
        end
        
        local str = g_DialogManager:getString("PROM001MIDM") .. g_DialogManager:getString(self.rOtherHero:getName()) .. ", " .. g_DialogManager:getString("PROM002MIDM") .. g_DialogManager:getString(Character.tLevelToTitle[self.rOtherHero:getStatLevel() + 1]) .. "! "
        g_GUI:DisplayLowerDialogText(self.rHero, str, true, true)
        g_GUI:DisplayLowerDialogText(self.rHero, g_DialogManager:getString("PROM003MIDM"), true, true)
        
        
        local sHeroKey = self.rOtherHero:getCharDefKey()
        local sReponse = self:SelectRandomHeroResponse(sHeroKey)
        
        -- if @Man, play his tweet anim
        if sHeroKey == "TestMale02b" then
            self.rOtherHero:playAnimation("idle1", false)
        end
        
        g_GUI:DisplayLowerDialogText(self.rOtherHero, g_DialogManager:getString(sReponse), true, true)
        
        util.sleep(.1)

        g_GUI:HideDialogText()
        g_GUI:HideTutorialArrow()

        

        self.rOtherHero:playLevelUpEffect()

        util.sleep(1.5)

		self:EndRun()	
	end
    
    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max(self.nDuration - self.nElapsedTime, 0)
    end

    function Ob:getRushPrice()
        return 0
    end

    function Ob:rushActivity()
        self.bDone = true
        DFAnalytics.logEvent("act_rshd", { act = "prmo", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_prmo_rshd", { hero = self.tSaved.sDefKey, cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_prmo_rshd", 1)        
    end
    
	function Ob:OnStart()
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_prmo", { hero = self.tSaved.sDefKey } )
            DFAnalytics.incrStat("act_prmo", 1)
        end
	end

	function Ob:OnEnd()
        if self.tUsedHeroes then
            for _,v in ipairs(self.tUsedHeroes) do
                self:csMode(v, false)
            end
        end

        self.rOtherHero:levelUp()
        self.rOtherHero:setOverlayText("PROMOTED", 3, {0, 1, 0})

        g_GUI:GetGameOverlayUI():setVisible(true)
        for k,v in pairs(g_Game.tEnabledActions) do
            g_Game.tEnabledActions[k] = true
        end
        g_City:enableEventGeneration(true)  
        g_Audio:stopMusic()
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IngameLoop', true)
	end
    
    function Ob:SelectRandomHeroResponse(hero)
        local out = "PROM005HERO"
        
        local tPossibleLines = {}
        local numPossibleLines = 0   

        for sCheckId, sLines in pairs(PromotionResponseDefs.heroResponse) do
            if hero == sCheckId then
                for i, sLine in ipairs(sLines) do
                    table.insert(tPossibleLines, sLine)
                    numPossibleLines = numPossibleLines + 1
                end
            end
        end
             
        local randomIdx = math.random(numPossibleLines)
        
        out = tPossibleLines[randomIdx]
        
        return out
    end

	return Ob
end

return MMPromoteActivity
