local MMTrainAbilityActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local Item = require('Item')
local TileMap = require('TileMap')
local Renderer = require('Renderer')
local CountdownUI = require('ui/CountdownUI')
local DFAchievement = require('DFCommon.Achievement')

function MMTrainAbilityActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved, sDefKey)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved
    if sDefKey then
        Ob.tSaved.sDefKey = sDefKey
    end

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        local rAbility = self.rHero:getAbility(self.tSaved.sDefKey)

        if not rAbility then
            self:EndRun()
            return            
        end

        self.rProp = self.rAvailableProp
        local nRow, nCol = self.rProp:getInteractTileCoords()
		local orgX, orgY = g_CurrentMap:getWorldCoords(nRow, nCol)
        self.rHero:setPriority(self.rProp:getInteractPri())
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)

        local nDurationMult = self.rProp:getPropDef().nDurationMultiplier or 1
        local nSpeedMod = self.rHero:getTrainingSpeedMod()
        if nSpeedMod > 0 then
            nDurationMult = nDurationMult + (1 / nSpeedMod)
        end
        self.nDuration = rAbility:getUpgradeTime()
        self.nDuration = self.nDuration * nDurationMult
        self.nDuration = self.nDuration * (1 / g_Game:getDebugTimeScale())
        local sKey = 'TrainAbility'..self.tSaved.sDefKey..tostring(rAbility:getCurLevel() + 1)
        self.nElapsedTime = self.rHero:getActivityProgress(sKey) * self.nDuration

        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange then
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress(sKey, self.nElapsedTime / self.nDuration)
            self.rHero:setOverlayText("LEARNING+", 1, {0, 1, 0})

            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end
            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end

        self.rHero:setActivityProgress(sKey, 0)

        self.rHero:setOverlayText("COMPLETED TRAINING", 1, {0, 1, 0})

        if rAbility:isUnlocked() then
            rAbility:incrementLevel()
        else
            rAbility:setUnlocked(true)
        end

		self.rHero:setLoc(orgX, orgY)
		self.rHero:playAnimation("breathe")	
        
		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max(self.nDuration - self.nElapsedTime, 0)
    end

    function Ob:getActivityProgress()
        if not self.nDuration or self.nDuration == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self.nDuration) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.bDone = true
        DFAnalytics.logEvent("act_rshd", { act = "trma", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_trma_rshd", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice(), } )
        DFAnalytics.incrStat("act_trma_rshd", 1)

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end

	function Ob:OnStart()
		local propName = "mmcomp"
        self.rRoom = g_CurrentMap:getRoom("mm")
        if not self.rRoom then
            -- exit
        end

        self.tSaved.bInRoom = true                
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_trma", { hero = self.tSaved.sDefKey } )
            DFAnalytics.incrStat("act_trma", 1)
            self.tSaved.bStarted = true
        end           
        self.rAvailableProp = self.rRoom:getNextAvailablePropOfType('mmcomp')
        self.rAvailableProp:addUser(self.rHero)
	end

	function Ob:OnEnd()
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return MMTrainAbilityActivity
