local MMTrainIntActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local UIProgressBar = require('ui/UIProgressBar')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local DFGraphics = require('DFCommon.Graphics')
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

local kHERO_IDLE_TIME = 5

function MMTrainIntActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved, sDefKey)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved
    if sDefKey then
        Ob.tSaved.sDefKey = sDefKey
    end

    function Ob:getOtherHero()
        return self.rOtherHero
    end

    function Ob:getDuration()
        if self.rDef.tTrainDuration then
            local nIntToTrain = self.rOtherHero:getBaseStatAwareness()
            for i, rInfo in ipairs(self.rDef.tTrainDuration) do
                if nIntToTrain >= rInfo.tRange[1] and nIntToTrain <= rInfo.tRange[2] then
                    local nDuration = rInfo.nDuration
                    local nSpeedMod = self.rOtherHero:getActivitySpeedMod(self.rDef.sActivityType)
                    if nSpeedMod > 0 then
                        nDuration = nDuration * (1 - (math.min(100, nSpeedMod) / 100))
                    end
                    return nDuration
                end
            end
        end
        return 1
    end

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        local nRow, nCol = self.rProp:getInteractTileCoords()
        local orgX, orgY = g_CurrentMap:getWorldCoords(nRow, nCol)
        self.rHero:setPriority(self.rProp:getInteractPri())
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()

        self.sExtraSpritePath = 'Levels/Tilesets/BaseProps2'..Renderer.getTextureSizeSuffix()
        local rSpriteSheet = DFGraphics.loadSpriteSheet(self.sExtraSpritePath, g_IsInStartupSequence)
        if rSpriteSheet then
            self.rExtraProp = MOAIProp2D:new()
            self.rExtraProp:setDeck(rSpriteSheet)
            local nIndex = rSpriteSheet.names['MM_ChartTraining'..Renderer.getTextureSizeSuffix()..'_1']
            self.rExtraProp:setIndex(nIndex)
            self.rLayer = Renderer.getRenderLayer('GameplayOverlay')
            self.rLayer:insertProp(self.rExtraProp)
            self.rExtraProp:setLoc(orgX + 50, orgY - 30)
        end

        self.rOtherHero = g_Game:getHero(self.tSaved.sDefKey)
        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end

        self.rOtherHero:runActivities(false)
        self.rOtherHero:setSelectable(false)
        util.sleep(0.2)

        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end

        self.rOtherHero:playAnimation('exit', true)

        local rTile = self.rRoom:getTile(6, 5)
        if rTile then
            local x, y = g_CurrentMap:getWorldCoords(rTile.nRow, rTile.nCol)
            self.rOtherHero:setPosition(x, y, true)
        end
        self.rOtherHero:playAnimation('enter', true, true)
        self.rOtherHero:playAnimation('idlebk1', false, true)

        if not self.rOtherHero or not self.rOtherHero:isActive() then
            self:EndRun()
            return
        end

        local sKey = 'TrainInt'..self.rOtherHero:getCharDefKey()

        local nTimeSinceHeroIdle = 0
		self.rHero:playAnimation('desktalk', false, bFlipX)

        self.nDuration = self:getDuration()

        self.nElapsedTime = self.rHero:getActivityProgress(sKey) * self.nDuration
        self.bDone = false
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange or self.tSaved.bWasForceKilled then
                self.tSaved.bWasForceKilled = false
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress(sKey, self.nElapsedTime / self.nDuration)

            nTimeSinceHeroIdle = nTimeSinceHeroIdle + 1
            if nTimeSinceHeroIdle >= kHERO_IDLE_TIME then
                nTimeSinceHeroIdle = 0
                self.rOtherHero:playAnimation('idlebk1', false, true)
            end
            local sTextAppend = "+"
            self.rOtherHero:setOverlayText("LEARNING"..sTextAppend, 3, {0, 1, 0})

            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end
            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)

            -- check on the hero
            if not self.rOtherHero:isActive() then
                self:EndRun()
                return
            end
        end

        self.rOtherHero:incrementStatAwareness(1)
        self.rOtherHero:setOverlayText("INT+", 1, {0, 1, 0})
        self.rHero:setActivityProgress(sKey, 0)

		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max(self.nDuration - self.nElapsedTime, 0)
    end

    function Ob:getActivityProgress()
        if self.nDuration == 0 or self.nDuration == nil then
            return 100
        end
        return (((self.nElapsedTime or 0) / self.nDuration) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.bDone = true
        DFAnalytics.logEvent("act_rshd", { act = "trin", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_trin_rshd", { hero = self.tSaved.sDefKey, cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_trin_rshd", 1)                   

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end
    
	function Ob:OnStart()
		local propName = "mmcomp"
        self.rRoom = g_CurrentMap:getRoom("mm")
        if not self.rRoom then
            -- exit
        end

        self.tSaved.bInRoom = true                
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_trin", { hero = self.tSaved.sDefKey } )
            DFAnalytics.incrStat("act_trin", 1)
            self.tSaved.bStarted = true
        end           
        self.rAvailableProp = self.rRoom:getNextAvailablePropOfType('mmcomp')
        self.rAvailableProp:addUser(self.rHero)
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end

        if self.rExtraProp and self.rLayer and self.sExtraSpritePath then
            self.rLayer:removeProp(self.rExtraProp)
            DFGraphics.unloadSpriteSheet(self.sExtraSpritePath)
        end

        if self.rOtherHero then
            if self.rOtherHero:isActive() then
                self.rOtherHero:runActivities(true)
                if self.bDone and not bForceKilled then
                    g_TodoList:onHeroTrained()
                    self.rOtherHero:decrementNumTrainingSessions(1)
                end
            end
            self.rOtherHero:setSelectable(true) 
        end
        if not self.bDone and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        self.tSaved.bStarted = false
	end

	return Ob
end

return MMTrainIntActivity
