local PTJobActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

function PTJobActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        self.rHero:setPriority(self.rProp:getInteractPri())
		local orgX, orgY = Ob.rHero:getLoc()
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)
        self.rHero:setFacingDirection((bFlipX and "SE") or "SW")	

        self.nProgressPerSec = self.rProp:getPropDef().nCashPerSec
        if self.nProgressPerSec == 0 then
            self.nProgressPerSec = 1
        end
        self.nMaxCash = 1
        if self.rRoom then
            self.nMaxCash = self.rRoom:getRoomDef().nMaxCash or self.nMaxCash
        end
        self.nDuration = self.nMaxCash / self.nProgressPerSec
        local nSpeedMod = self.rHero:getActivitySpeedMod(self.rDef.sActivityType)
        if nSpeedMod > 0 then
            self.nDuration = self.nDuration * (1 - (math.min(100, nSpeedMod) / 100))
        end

        if not self.tSaved.nAmountCreated then
            self.tSaved.nAmountCreated = 0
        end
        local sActivityName = self:getActivityType()

        self.nProgress = 0
        self.bDone = false
        self.nElapsedTime = self.rHero:getActivityProgress('PTJob') * self.nDuration
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange or self.tSaved.bWasForceKilled then
                self.tSaved.bWasForceKilled = false
                self.tSaved.nLastTimeChange = nCurTime
            end
            local nChange = math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.nElapsedTime = self.nElapsedTime + nChange
            self.rHero:setActivityProgress('PTJob', self.nElapsedTime / self.nDuration)

            self.nProgress = self.nProgress + (self.nProgressPerSec * nChange)
            local nAmountToAdd = math.floor(self.nProgress)
            if nAmountToAdd > 0 then
                if (self.tSaved.nAmountCreated + nAmountToAdd) >= self.nMaxCash then
                    nAmountToAdd = self.nMaxCash - self.tSaved.nAmountCreated
                end
                self.tSaved.nAmountCreated = self.tSaved.nAmountCreated + nAmountToAdd
                self:addCash(math.floor(nAmountToAdd))
                self.nProgress = self.nProgress - nAmountToAdd
                if self.tSaved.nAmountCreated >= self.nMaxCash then
                    self.bDone = true
                    self.rHero:setOverlayText(self.nMaxCash.." COINS RAISED", 1, {1, 0, 0})
                else
                    self.rHero:setOverlayText("COINS +"..nAmountToAdd, 1, {0, 1, 0})
                end
            end

            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end
        
		self.rHero:setLoc(orgX, orgY)
		self.rHero:playAnimation("breathe")	

		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nMaxCash or not self.tSaved or not self.tSaved.nAmountCreated or not self.nProgressPerSec then
            return 0
        end
        return math.max((self.nMaxCash - (self.tSaved.nAmountCreated + self.nProgress)) / self.nProgressPerSec, 0)
    end

    function Ob:getActivityProgress()
        if self:getRemainingTime() == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self:getRemainingTime()) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.bDone = true
        local nAmountToAdd = self.nMaxCash - self.tSaved.nAmountCreated
        nAmountToAdd = math.floor(nAmountToAdd)
        self:addCash(nAmountToAdd)
        self.rHero:setOverlayText(nAmountToAdd.." COINS RAISED", 1, {1, 0, 0})
        
        DFAnalytics.logEvent("act_rshd", { act = "hrjb", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_hrjb_rshd", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_hrjb_rshd", 1)        

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end
    
    function Ob:addCash(nAmountToAdd)
        g_Game:addCash(nAmountToAdd, true)      
    end

	function Ob:OnStart()
		local propName = "work"
        self.rRoom = g_CurrentMap:getRoom("cube")
        if not self.rRoom then
            -- exit
        end

        local bTeleportToProp = false

        local task = nil
        if not self.tSaved.bInRoom then          
            local task = WalkToRoomTask:new(self, 'cube')
            self:AddTask(task)
        else
            bTeleportToProp = true
        end
        self.tSaved.bInRoom = true        
        
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_hrjb", { hero = self.rHero:getCharDefKey() } )
            DFAnalytics.incrStat("act_hrjb", 1)
            self.tSaved.bStarted = true
        end   
        
        task = WaitForPropTask:new(self, self.rRoom, propName, self.rHero, bTeleportToProp)
		self:AddTask(task)
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if not self.bDone and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end
        self.rHero:setActivityProgress('PTJob', 0)

        -- incr general stat here to avoid spamming analytics
        if self.tSaved.nAmountCreated then
            DFAnalytics.incrStat("csh_spnt_total", self.tSaved.nAmountCreated)
            DFAnalytics.incrStat('csh_ernd_hrjb', self.tSaved.nAmountCreated)
        end
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return PTJobActivity
