local RecreationActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require("DFUtil")
local Activity = require("activity/Activity")
local WalkToRoomTask = require("WalkToRoomTask")
local WaitForPropTask = require("WaitForPropTask")
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

function RecreationActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        if not self.nInteractPri then
            self.nInteractPri = 0
        end
        self.rHero:setPriority(self.nInteractPri) -- these values are set by the wait for prop task
        local orgX, orgY = Ob.rHero:getLoc()
        if not self.nInteractOffsetX or not self.nInteractOffsetY then
            self.nInteractOffsetX = 0
            self.nInteractOffsetY = 0
        end
        self.rHero:setLoc(orgX + self.nInteractOffsetX, orgY + self.nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
        self.rHero:setFacingDirection((bFlipX and "SE") or "SW")	
        self.rHero:playAnimation(sInteractAnimName, false, bFlipX)
        
        local rPropDef = self.rProp:getPropDef()
        self.nDuration = rPropDef.duration or 1
        self.nDuration = self.nDuration * (1 / g_Game:getDebugTimeScale())
        local nSpeedMod = self.rHero:getActivitySpeedMod(self.rDef.sActivityType)
        if nSpeedMod > 0 then
            self.nDuration = self.nDuration * (1 - (math.min(100, nSpeedMod) / 100))
        end

        local nLevel = self.rProp:getPropDef().nLevel or 1

        self.bDone = false

        if self.rHero:isAtMaxMorale() then
            self.bDone = true
        end

        self.nElapsedTime = self.rHero:getActivityProgress('Recreation') * self.nDuration
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange or self.tSaved.bWasForceKilled then
                self.tSaved.bWasForceKilled = false
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress('Recreation', self.nElapsedTime / self.nDuration)

            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end

            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end
        
        self.rHero:setActivityProgress('Recreation', 0)

        self.rHero:setMorale(5) -- UG, this is to avoid the looping of lua requires

        self.rHero:setLoc(orgX, orgY)
        self.rHero:playAnimation("breathe")	            

		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max((self.nDuration - self.nElapsedTime), 0)
    end

    function Ob:getActivityProgress()
        if not self.nDuration or self.nDuration == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self.nDuration) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end
    
    function Ob:rushActivity()
        self.bDone = true
        
        DFAnalytics.logEvent("act_rshd", { act = "recr", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_recr_rshd", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_recr_rshd", 1)        

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end

	function Ob:OnStart()
		local propName = "recreation"
        self.rRoom = g_CurrentMap:getRoom("recroom")
        if self.rRoom then
            local task = nil

            local bTeleportToProp = false

            if not self.tSaved.bInRoom then            
                task = WalkToRoomTask:new(self, 'recroom')
                self:AddTask(task)
            else
                bTeleportToProp = true
            end
            self.tSaved.bInRoom = true 
            
            if not self.tSaved.bStarted then
                DFAnalytics.logEvent("act_recr", { hero = self.rHero:getCharDefKey() } )
                DFAnalytics.incrStat("act_recr", 1)
                self.tSaved.bStarted = true
            end  
            
            task = WaitForPropTask:new(self, self.rRoom, propName, self.rHero, bTeleportToProp)
            self:AddTask(task)
        end
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if not self.bDone and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return RecreationActivity
