local RestActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local WalkToRoomTask = require('WalkToRoomTask')
local WalkToTileTask = require('WalkToTileTask')
local WaitForPropTask = require('WaitForPropTask')
local CountdownUI = require('ui/CountdownUI')
local Renderer = require('Renderer')

function RestActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 160)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rBed = self.rAvailableProp
        self.rHero:setPriority(self.rBed:getInteractPri())		
		local orgX, orgY = self.rHero:getLoc()
        local nInteractOffsetX, nInteractOffsetY = self.rBed:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rBed:getInteractAnim()
        self.rHero:setFacingDirection((bFlipX and "SE") or "SW")	
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)
        if self.rHero then
            self.rHero:setShadowVisible(false)
        end

        local nAddHP = self.rRoom:getRoomDef().nAdditionalHP or 0
        self.nMaxHP = self.rHero:getMaxHP() + nAddHP

        self.nHpRefreshRatePerSec = (self.rBed:getPropDef().hpRefreshRatePerSec or 1)
        if self.nHpRefreshRatePerSec == 0 then
            self.nHpRefreshRatePerSec = 1
        end
        local nSpeedMod = self.rHero:getActivitySpeedMod(self.rDef.sActivityType)
        if nSpeedMod > 0 then
            self.nHpRefreshRatePerSec = self.nHpRefreshRatePerSec * (1 + (math.min(100, nSpeedMod) / 100))
        end
        
        self.nAmountToRefresh = self.nMaxHP - self.rHero:getCurHP()

        self.nElapsedTime = 0
        if self.tSaved.nLastTimeAtChange then
            local nCurTime = MOAISim:getDeviceTime()
            self.nElapsedTime = math.abs(nCurTime - self.tSaved.nLastTimeAtChange)
            local nAmount = self.nHpRefreshRatePerSec * self.nElapsedTime
            self.rHero:incrementHP(nAmount, self.nMaxHP)
            self.nAmountToRefresh = self.nAmountToRefresh - nAmount
        end

        local nLevel = self.rBed:getPropDef().nLevel or 1

        if self.nAmountToRefresh > 0 then
            while self.nAmountToRefresh > 0 do
                local nCurTime = MOAISim:getDeviceTime()
                if not self.tSaved.nLastTimeAtChange or self.tSaved.bWasForceKilled then
                    self.tSaved.bWasForceKilled = false
                    self.tSaved.nLastTimeAtChange = nCurTime
                end
                local nElapsed = math.abs(nCurTime - self.tSaved.nLastTimeAtChange)
                local nAmount = math.floor(nElapsed * self.nHpRefreshRatePerSec)
                self.rHero:incrementHP(nAmount, self.nMaxHP)
                self.nAmountToRefresh = self.nAmountToRefresh - nAmount
                if self.nAmountToRefresh < 0 then
                    self.nAmountToRefresh = 0
                end

                self.tSaved.nLastTimeAtChange = nCurTime
                util.sleep(1)
                self.nElapsedTime = self.nElapsedTime + 1

                self.rCountdownUI:updateProgress((self.nElapsedTime / self:getRemainingTime()) * 100)
            end
        else
            util.sleep(1) -- just to play the sleep anim
        end

		self.rHero:setLoc(orgX, orgY)
		self.rHero:playAnimation("breathe")
		
		self:EndRun()
		
        self.rHero:setOverlayText("+MAX HEALTH", 3, {0, 1, 0})
	end

    function Ob:getRemainingTime()
        if not self.nAmountToRefresh or not self.nHpRefreshRatePerSec then
            return 0
        end
        return math.max((self.nAmountToRefresh / self.nHpRefreshRatePerSec), 0)
    end

    function Ob:getActivityProgress()
        if self:getRemainingTime() == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self:getRemainingTime()) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.rHero:incrementHP(self.nAmountToRefresh, self.nMaxHP)
        self.nAmountToRefresh = 0
        
        DFAnalytics.logEvent("act_rshd", { act = "rest", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_rest_rshd", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_rest_rshd", 1)        

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end

	function Ob:OnStart()
		local propName = "bed"
        self.rRoom = g_CurrentMap:getRoom("bedroom")
        if not self.rRoom then
            -- exit
        end
        local bTeleportToProp = false

        local task = nil
        if not self.tSaved.bInRoom then
            local task = WalkToRoomTask:new(self, 'bedroom')
            self:AddTask(task)
        else
            bTeleportToProp = true
        end
        self.tSaved.bInRoom = true
        
        if not self.tSaved.bStarted then
            -- this is our #1 piece of telemetry, so for now we're removing it to save battery
            --DFAnalytics.logEvent("act_rest", { hero = self.rHero:getCharDefKey() } )
            DFAnalytics.incrStat("act_rest", 1)            
            self.tSaved.bStarted = true
        end
        
        task = WaitForPropTask:new(self, self.rRoom, "bed", self.rHero, bTeleportToProp)
		self:AddTask(task)
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if (not self.nAmountToRefresh or self.nAmountToRefresh > 0) and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end
        if self.rHero then
            self.rHero:setShadowVisible(true)
        end
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return RestActivity
