local TrainAbilityActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local Item = require('Item')
local TileMap = require('TileMap')
local Renderer = require('Renderer')
local CountdownUI = require('ui/CountdownUI')
local DFAchievement = require('DFCommon.Achievement')

function TrainAbilityActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved, sDefKey)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved
    if sDefKey then
        Ob.tSaved.sDefKey = sDefKey
    end

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        if not self.tSaved.sDefKey or not self.rProp then
            self:EndRun()
            return
        end

        self.rHero:setPriority(self.rProp:getInteractPri())
		local orgX, orgY = Ob.rHero:getLoc()
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
        self.rHero:setFacingDirection((bFlipX and "SE") or "SW")	
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)

        local rAbility = self.rHero:getAbility(self.tSaved.sDefKey)

        if not rAbility then
            self:EndRun()
            return            
        end

        self.nDuration = rAbility:getUpgradeTime()
        local nDurationMult = self.rProp:getPropDef().nDurationMultiplier or 1
        local nSpeedMod = self.rHero:getActivitySpeedMod(self.rDef.sActivityType)
        if nSpeedMod > 0 then
            self.nDuration = self.nDuration * (1 - (math.min(100, nSpeedMod) / 100))
        end     
        self.nDuration = self.nDuration * nDurationMult
        self.nDuration = self.nDuration * (1 / g_Game:getDebugTimeScale())

        local sKey = 'TrainAbility'..self.tSaved.sDefKey..tostring(rAbility:getCurLevel() + 1)
        
        self.nElapsedTime = self.rHero:getActivityProgress(sKey) * self.nDuration

        local nLevel = self.rProp:getPropDef().nLevel or 1
        self.bDone = false 
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange or self.tSaved.bWasForceKilled then
                self.tSaved.bWasForceKilled = false
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress(sKey, self.nElapsedTime / self.nDuration)

            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end            
 
            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end
        self.rHero:setActivityProgress(sKey, 0)

        self.rHero:setOverlayText("COMPLETED TRAINING", 1, {0, 1, 0})

        if rAbility:isUnlocked() then
            rAbility:incrementLevel()
        else
            rAbility:setUnlocked(true)
        end

		self.rHero:setLoc(orgX, orgY)
		self.rHero:playAnimation("breathe")	

		self:EndRun()	
	end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max((self.nDuration - self.nElapsedTime), 0)
    end

    function Ob:getActivityProgress()
        if not self.nDuration or self.nDuration == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self.nDuration) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()
        self.bDone = true
        DFAnalytics.logEvent("act_rshd", { act = "trab", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_trab_rshd", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice(), subject = self.tSaved.sDefKey } )
        DFAnalytics.incrStat("act_trab_rshd", 1)

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end

	function Ob:OnStart()
		local propName = "trainingdummy"
        local rRoom = g_CurrentMap:getRoom("dojo")
        if not rRoom then
            -- exit
        end
        local bTeleportToProp = false

		local task = nil
        if not self.tSaved.bInRoom then            
            task = WalkToRoomTask:new(self, 'dojo')
            self:AddTask(task)
        else
            bTeleportToProp = true
        end
        self.tSaved.bInRoom = true   
   
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_trab", { hero = self.rHero:getCharDefKey(), subject = self.tSaved.sDefKey } )
            DFAnalytics.incrStat("act_trab", 1)
            self.tSaved.bStarted = true
        end   
       
        task = WaitForPropTask:new(self, rRoom, propName, self.rHero, bTeleportToProp)
		self:AddTask(task)
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if not self.bDone and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end
        
        self.tSaved.bStarted = false
	end

	return Ob
end

return TrainAbilityActivity
