local TrainHPActivity = {}

local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Activity = require('activity/Activity')
local WalkToRoomTask = require('WalkToRoomTask')
local WaitForPropTask = require('WaitForPropTask')
local Renderer = require('Renderer')
local CountdownUI = require('ui/CountdownUI')

local kTRAIN_DURATION = 120

function TrainHPActivity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
	local Ob = Activity:new(rHero, rDef, nIndex, bRunUntilSuccess, tSaved)
    Ob.tSaved = tSaved

    function Ob:getDuration()
        if self.rDef.tTrainDuration then
            local nHPToTrain = self.rHero:getMaxHP()
            for i, rInfo in ipairs(self.rDef.tTrainDuration) do
                if nHPToTrain >= rInfo.tRange[1] and nHPToTrain <= rInfo.tRange[2] then
                    local nDuration = rInfo.nDuration
                    local nSpeedMod = self.rHero:getActivitySpeedMod(self.rDef.sActivityType)
                    if nSpeedMod > 0 then
                        nDuration = nDuration * (1 - (math.min(100, nSpeedMod) / 100))
                    end                 
                    return nDuration
                end
            end
        end
        return 1
    end

	function Ob:Run()
        local rOverlayLayer = Renderer.getRenderLayer('GameplayOverlay')
        self.rCountdownUI = CountdownUI:new(rOverlayLayer)
        self.rCountdownUI:setLoc(0, 200)
        self.rCountdownUI:setVisible(true)
        self.rCountdownUI:setParent(self.rHero)

        self.rProp = self.rAvailableProp
        self.rHero:setPriority(self.rProp:getInteractPri())
		local orgX, orgY = Ob.rHero:getLoc()
        local nInteractOffsetX, nInteractOffsetY = self.rProp:getInteractOffset()
		self.rHero:setLoc(orgX + nInteractOffsetX, orgY + nInteractOffsetY)
        local sInteractAnimName, bFlipX = self.rProp:getInteractAnim()
        self.rHero:setFacingDirection((bFlipX and "SE") or "SW")	
		self.rHero:playAnimation(sInteractAnimName, false, bFlipX)

        local rPropDef = self.rProp:getPropDef()
        local nPropMult = self.rProp:getPropDef().trainSpeed or 1
        self.nDuration = rPropDef.duration
        self.nDuration = self.nDuration / nPropMult
        self.nDuration = self.nDuration * (1 / g_Game:getDebugTimeScale())

        self.nElapsedTime = 0
        local sActivityName = self:getActivityType()

        local nLevel = self.rProp:getPropDef().nLevel or 1

        self.bDone = false

        self.nElapsedTime = self.rHero:getActivityProgress('TrainHP') * self.nDuration
        while not self.bDone do
            local nCurTime = MOAISim:getDeviceTime()
            if not self.tSaved.nLastTimeChange or self.rHero:isActivityProgressPaused() or  self.tSaved.bWasForceKilled then
                self.tSaved.bWasForceKilled = false
                self.tSaved.nLastTimeChange = nCurTime
            end
            self.nElapsedTime = self.nElapsedTime + math.abs(nCurTime - self.tSaved.nLastTimeChange)
            self.rHero:setActivityProgress('TrainHP', self.nElapsedTime / self.nDuration)

            if self.nElapsedTime >= self.nDuration then
                self.bDone = true
            end            

            self.tSaved.nLastTimeChange = nCurTime
            self.rCountdownUI:updateProgress(self:getActivityProgress())
            util.sleep(1)
        end

        if self.bDone then
            local nHPIncrement = (self.rProp:getPropDef().trainingIncrement or 1) * 10
            self.rHero:incrementMaxHP(nHPIncrement)
            self.rHero:setCurHP(self.rHero:getCurHP() + nHPIncrement)
            self.rHero:setOverlayText("HP TRAINING +"..nHPIncrement, 3, {0, 1, 0})
            self.rHero:setActivityProgress('TrainHP', 0)
        end

		self.rHero:setLoc(orgX, orgY)
		self.rHero:playAnimation("breathe")
        
		self:EndRun()		
	end

    function Ob:getRemainingTime()
        if not self.nDuration or not self.nElapsedTime then
            return 0
        end
        return math.max((self.nDuration - self.nElapsedTime), 0)
    end

    function Ob:getActivityProgress()
        if not self.nDuration or self.nDuration == 0 then
            return 100
        end
        return (((self.nElapsedTime or 0) / self.nDuration) * 100)
    end

    function Ob:getRushPrice()
        return self:computeRushCost(self:getRemainingTime())
    end

    function Ob:rushActivity()           
        self.bDone = true
        
        DFAnalytics.logEvent("act_rshd", { act = "trhp", remaining = self:getRemainingTime() } )
        DFAnalytics.logEvent("act_trhp_rushed", { hero = self.rHero:getCharDefKey(), cost = self:getRushPrice() } )
        DFAnalytics.incrStat("act_trhp_rushed", 1)                    

        g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
    end

	function Ob:OnStart()		
		local propName = "workout"
        self.rRoom = g_CurrentMap:getRoom("gym")
        if not self.rRoom then
            -- exit
        end

        local bTeleportToProp = false

        local task = nil
        if not self.tSaved.bInRoom then            
            local task = WalkToRoomTask:new(self, 'gym')
            self:AddTask(task)
        else
            bTeleportToProp = true
        end
        self.tSaved.bInRoom = true
        
        if not self.tSaved.bStarted then
            DFAnalytics.logEvent("act_trhp", { hero = self.rHero:getCharDefKey() } )
            DFAnalytics.incrStat("act_trhp", 1)
            self.tSaved.bStarted = true
        end

        task = WaitForPropTask:new(self, self.rRoom, propName, self.rHero, bTeleportToProp)
		self:AddTask(task)
	end

	function Ob:OnEnd(bForceKilled)
        if self.rAvailableProp then
            self.rAvailableProp:removeUser(self.rHero)
        end
        if self.bDone and not bForceKilled then
            g_TodoList:onHeroTrained()
            self.rHero:decrementNumTrainingSessions(1)
        end
        if not self.bDone and bForceKilled then
            self.tSaved.bInRoom = false
            self.tSaved.bWasForceKilled = true
        else
            self.tSaved.bWasForceKilled = false
        end
        if self.rCountdownUI then
            self.rCountdownUI:setVisible(false)
        end

        self.tSaved.bStarted = false
	end

	return Ob
end

return TrainHPActivity
