local util = require("DFUtil")
local Character = require("Character")
local Ability = require('Ability/Ability')
local Renderer = require('Renderer')

local AutoResolveEventActor = {}

function AutoResolveEventActor:new(rEvent, rHero, rDef, packLevel)
	local Ob = MOAITransform.new()

	function Ob:getStatPower()
		return self.nPower + self.nPowerStatMod
	end

    function Ob:isDead()
        return (self:getCurHP() <= 0)
    end    

	function Ob:getCurHP()
		return self.nCurHP
	end

	function Ob:setCurHP(nPts)
		self.nCurHP = nPts
	end

    function Ob:isMelee()
        return self.rDef.bIsMelee
    end

    function Ob:isRanged()
        return false
    end    

    function Ob:attackTarget(tPossibleTargets, nDamageFactor, nAdditiveDamage)
        if self.rEvent and tPossibleTargets then
            local rTarget = nil
            local nTargetIndex = 1
            for i, rPossibleTarget in ipairs(tPossibleTargets) do
                if not rPossibleTarget:isDead() and rPossibleTarget:isMelee() then
                    rTarget = rPossibleTarget
                    nTargetIndex = i
                    break
                end
            end
            if not rTarget then
                for i, rPossibleTarget in ipairs(tPossibleTargets) do
                    if not rPossibleTarget:isDead() then
                        rTarget = rPossibleTarget
                        break                    
                    end
                end
            end
            if rTarget then                
                local nPts = self:getStatPower()
                nDamageFactor = nDamageFactor or 1
                nAdditiveDamage = nAdditiveDamage or 0
                if not rTarget:takeDamage(nPts * nDamageFactor + nAdditiveDamage) then
                    return false, nTargetIndex
                end
            end
        end
        return true
    end

	-- true: still alive
	-- false: dead
	function Ob:takeDamage(nPts, rHitEffectDef, rAttacker, bCritical)   
        if self:isDead() then
            return false
        end
		-- round (up)
        nPts = nPts - self.nShieldLevel
		nPts = math.ceil(nPts)
		self.nCurHP = self.nCurHP - nPts
		if self.nCurHP <= 0 then			
			self.nCurHP = 0
            if self.rHero then
                self.rHero:setCurHP(1)  
            end
			return false
		end

        if self.rHero then
            self.rHero:setCurHP(self.nCurHP)                
        end
        return true
	end

    function Ob:heroWasActive()
        return self.bHeroWasActive
    end

    function Ob:getHero()
        return self.rHero
    end

    Ob.rHero = rHero
    Ob.rEvent = rEvent   
    Ob.nShieldLevel = 0
    if rHero then
        rDef = rHero.tCharDef
    end

	Ob.rDef = rDef

	Ob.nLevel = rDef.tInitialStats[packLevel].nLevel
	Ob.nPower = rDef.tInitialStats[packLevel].nPower
	Ob.nCurHP = rDef.tInitialStats[packLevel].nMaxHP
	Ob.nMaxHP = rDef.tInitialStats[packLevel].nMaxHP
    Ob.nAwareness = rDef.tInitialStats[packLevel].nAwareness

    Ob.bHeroWasActive = false
    if rHero then
        Ob.nLevel = rHero:getStatLevel()
        Ob.nPower = rHero:getStatPower()
        Ob.nCurHP = rHero:getCurHP()
        Ob.nMaxHP = rHero:getMaxHP()
        Ob.nAwareness = rHero:getStatAwareness()
        Ob.nShieldLevel = rHero:getStatMod('nShield')
        if rHero:isActive() then
            Ob.bHeroWasActive = true
        end
    end

    Ob.nPowerStatMod = 0

	return Ob
end

return AutoResolveEventActor