local BoxingMissileEvent1 = {}
local BoxingMissileEvent2 = {}
local BoxingMissileEvent3 = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local EffectInstance = require('EffectInstance')
local Renderer = require('Renderer')
local DFAchievement = require('DFCommon.Achievement')

local function boxingMissileEvent1Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end
    
    local rHench1 = self:getActor('Enemy1')
    local rHench2 = self:getActor('Enemy2')
    local rHench3 = self:getActor('Enemy3')

    rHench2:playAnimation('breathe', false, false)
    rHench3:playAnimation('breathe', false, false)

    util.sleep(0.75)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    rHench2:playAnimation('threathen_breathe', false, false)
    rHench1:playAnimation('hero_entrance_react', true, false, true)
    rHench1:playAnimation('hero_entrance_react_end', false, false)

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("MSMS001GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("MSMS002GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:sayLine("MSMS003GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("MSMS004GHMN", 'upper', true, true)
    rHench1:sayLine("MSMS005GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { rCitizen }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        rSpeakingActor = tAliveHeroes[1]
        rSpeakingActor:playAnimation('breathe', true, false)
        rSpeakingActor:sayLine("MSMS006GHRO", 'upper', true, true)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        rEnemy1 = tAliveEnemies[1]
        if rEnemy1 then
            rEnemy1:sayLine("MSMS007GHMN", 'upper', true, true)
            rEnemy1:sayLine("MSMS008GHMN", 'upper', true, true)
        end
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self, onIntro1EventClosedCallback)

end

local function boxingMissileEvent2Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end

    local rHench1 = self:getActor('Enemy1')
    local rHench2 = self:getActor('Enemy2')
    rHench2:playAnimation('breathe', false, false)

    util.sleep(0.75)

    rHench1:sayLine("MSGT001GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench2:sayLine("MSGT002GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("MSGT003GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench2:sayLine("MSGT004GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    --rHench1:sayLine("MSGT005GHMN", 'upper', true, true)
    
    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    rHench2:playAnimation('threathen_breathe', false, false)
    rHench1:playAnimation('hero_entrance_react', true, false, true)
    rHench1:playAnimation('hero_entrance_react_end', false, false)

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("MSGT006GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench2:sayLine("MSGT007GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:playAnimation("attack_breathe", true)
    rSpeakingActor:sayLine("MSGT008GHRO", 'upper', true, true)

    -- clear out the dialog
    g_GUI:HideDialogText()
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { rCitizen }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        rSpeakingActor = tAliveHeroes[1]
        rSpeakingActor:playAnimation('breathe', true, false)
        rSpeakingActor:sayLine("MSGT009GHRO", 'upper', true, true)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        rEnemy1 = tAliveEnemies[1]
        if rEnemy1 then
            rEnemy1:sayLine("MSGT010GHMN", 'upper', true, true)
        end
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self, onIntro1EventClosedCallback)
end

local function actorKOCB(self, rActor)
    local rRocketLauncher = self:getActor('RocketLauncher')
    if rActor == rRocketLauncher then
        rRocketLauncher:setLoc(240, -200)
        rRocketLauncher:setScl(0.5)  
        
        local x,y = rRocketLauncher:getLoc()
        local rEffectInstance = EffectInstance:new('Effects/Explosion.lua', Renderer.getLayer('UIOverlay').RenderLayer)
        rEffectInstance:setScl(6, 6)
        rEffectInstance:setLoc(x+530, y-350)
        rEffectInstance:start()
    end
end

local function boxingMissileEvent3Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    self.actorKOCallback = actorKOCB
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end

    local rMadScientist = self:getActor('MadScientist')    
    rMadScientist:setLoc(800, -210)
    rMadScientist:insertIntoLayer(self.rLayer)

    local rHench1 = self:getActor('Enemy1')

    local rRocketLauncher = self:getActor('RocketLauncher')
    rRocketLauncher:setLoc(200, -170)
    rRocketLauncher:playAnimation('breathe', false, false)

    rMadScientist:moveToLoc(340, -210, 3, 'run', true)
    rMadScientist:playAnimation('fight_breathe', false, false)
    rHench1:sayLine("MSBF001GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rMadScientist:sayLine("MSBF002MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("MSBF003GHMN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rMadScientist:sayLine("MSBF004MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    
    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("MSBF005GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    for i, rEnemy in ipairs(tEnemies) do
        if rEnemy ~= rRocketLauncher then
            util.sleep(math.random(0.2, 0.4))
            rEnemy:playAnimation("hero_entrance_react", true, false)
        end
    end
    
    util.sleep(1)
    
    for i, rEnemy in ipairs(tEnemies) do
        if rEnemy ~= rRocketLauncher then
            rEnemy:playAnimation("hero_entrance_react_end", false, false)
        end
    end

    rMadScientist:sayLine("MSBF006MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("MSBF007GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    rMadScientist:sayLine("MSBF009MADS", 'upper', true, true)
    rMadScientist:sayLine("MSBF008MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("MSBF010MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    rMadScientist:sayLine("MSBF011MADS", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:sayLine("MSBF012GHRO", 'upper', true, true)
    g_GUI:HideDialogText()
    rMadScientist:sayLine("MSBF013MADS", 'upper', true, true)
    rMadScientist:playAnimation('PutOnGoggles', false, false, true)
    rMadScientist:playAnimation('goggle_breathe', false, false)
    --rMadScientist:sayLine("MSBF014MADS", 'upper', true, true)
    rMadScientist:sayLine("MSBF015MADS", 'upper', true, true)
    rMadScientist:playAnimation('press_button_laugh', false, false)
    g_GUI:HideDialogText()
    --rSpeakingActor:sayLine("MSBF016GHRO", 'upper', true, true)
    -- clear out the dialog
    g_GUI:HideDialogText()
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        rMadScientist:playAnimation('fight_breathe', false, false)
        rSpeakingActor = tAliveHeroes[1]
        rSpeakingActor:playAnimation('breathe', true, false)
        rMadScientist:sayLine("MSBF017MADS", 'upper', true, true)
        rMadScientist:playAnimation('remote_defeat', false, false)
        rSpeakingActor:sayLine("MSBF018GHRO", 'upper', true, true)
        rMadScientist:sayLine("MSBF019MADS", 'upper', true, true)
        rSpeakingActor:playAnimation('breathe', false, false)
        rMadScientist:moveToLoc(800, -210, 1, 'run', true)

        self.sWinnerPortraitPath, self.sWinnerPortraitIndex =  tAliveHeroes[1]:getCurPortraitPath()
        self.sLoserPortraitPath, self.sLoserPortraitIndex = rMadScientist:getCurPortraitPath() 

        DFAchievement.updateAchievement('BeatRocketPunch', 1)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        local tHeroActors = self:getAllHeroActors()

        self.sLoserPortraitPath, self.sLoserPortraitIndex = tHeroActors[1]:getCurPortraitPath()
        self.sWinnerPortraitPath, self.sWinnerPortraitIndex = rMadScientist:getCurPortraitPath() 

        rMadScientist:playAnimation('press_button_laugh', false, false)
        rMadScientist:sayLine("MSBF020MADS", 'upper', true, true)

        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self)
end

function BoxingMissileEvent1:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = boxingMissileEvent1Thread

    return Ob
end

function BoxingMissileEvent2:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = boxingMissileEvent2Thread

    return Ob
end

function BoxingMissileEvent3:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = boxingMissileEvent3Thread

    return Ob
end

local tEvents =
{
    BoxingMissileEvent1, BoxingMissileEvent2, BoxingMissileEvent3
}

return tEvents