local EventManager = {}

local Renderer = require('Renderer')
local util = require("DFUtil")
local Character = require("Character")
local RobberyEvent = require("event/RobberyEvent")
local EffectInstance = require("EffectInstance")
local FireRescueEvent = require("event/FireRescueEvent")
local RunawayBusEvent = require("event/RunawayBusEvent")
local EventActor = require("event/EventActor")
local DFFile = require('DFCommon.File')
local Event = require('event/Event')
local IntroEvent = require('event/IntroEvent')
local BoxingMissileEvent = require('event/BoxingMissileEvent')
local RobotFootEvent = require('event/RobotFootEvent')
local PortalEvent = require('event/PortalEvent')
local MonsterFootEvent = require('event/MonsterFootEvent')
local JailBreakEvent = require('event/JailBreakEvent')
local CloneEvent = require('event/CloneEvent')
local CrowbarEvent = require('event/CrowbarEvent')
local AlienEvent = require('event/AlienEvent')
local GodzillaEvent = require('event/GodzillaEvent')
local MostWantedEvent = require('event/MostWantedEvent')
local ValentinesEvent = require('event/ValentinesEvent')

local initHeroPosition = { x = 0, y = 0 }

local tEventClasses =
{
    RobberyEvent = RobberyEvent,    
    FireRescueEvent = FireRescueEvent,
    RunawayBusEvent = RunawayBusEvent,
    IntroEvent1 = IntroEvent[1],
    IntroEvent2 = IntroEvent[2],
    IntroEvent3 = IntroEvent[3],
    BoxingMissileEvent1 = BoxingMissileEvent[1],
    BoxingMissileEvent2 = BoxingMissileEvent[2],
    BoxingMissileEvent3 = BoxingMissileEvent[3],
    RobotFootEvent1 = RobotFootEvent[1],
    RobotFootEvent2 = RobotFootEvent[2],
    RobotFootEvent3 = RobotFootEvent[3],
    RobotFootEvent4 = RobotFootEvent[4],
    RobotFootEvent5 = RobotFootEvent[5],
    PortalEvent1 = PortalEvent[1],
    PortalEvent2 = PortalEvent[2],
    PortalEvent3 = PortalEvent[3],
    MonsterFootEvent1 = MonsterFootEvent[1],
    MonsterFootEvent2 = MonsterFootEvent[2],
    JailBreakEvent1 = JailBreakEvent[1],
    JailBreakEvent2 = JailBreakEvent[2],
    JailBreakEvent3 = JailBreakEvent[3],
    CloneEvent1 = CloneEvent[1],
    CloneEvent2 = CloneEvent[2],
    CloneEvent3 = CloneEvent[3],
    CloneEvent4 = CloneEvent[4],
    CrowbarEvent1 = CrowbarEvent[1],
    CrowbarEvent2 = CrowbarEvent[2],
    CrowbarEvent3 = CrowbarEvent[3],
    AlienEvent1 = AlienEvent[1],
    AlienEvent2 = AlienEvent[2],
    AlienEvent3 = AlienEvent[3],
    AlienEvent4 = AlienEvent[4],
    GodzillaEvent1 = GodzillaEvent[1],
    GodzillaEvent2 = GodzillaEvent[2],
    MostWantedEvent1 = MostWantedEvent,
    MostWantedEvent2 = MostWantedEvent,
    MostWantedEvent3 = MostWantedEvent,    
    MostWantedEvent4 = MostWantedEvent,
    MostWantedEvent5 = MostWantedEvent,    
    MostWantedEvent6 = MostWantedEvent,
    MostWantedEvent7 = MostWantedEvent,
    MostWantedEvent8 = MostWantedEvent,    
    MostWantedEvent9 = MostWantedEvent,
    MostWantedEvent10 = MostWantedEvent,
    ValentinesEvent1 = ValentinesEvent.tEvents[1],
    ValentinesEvent2 = ValentinesEvent.tEvents[2],
}

local tEventSetInfos =
{
    Valentines = ValentinesEvent.rEventSetInfo
}

EventManager.tEventInfos = dofile(DFFile.getDataPath("Events/EventDefs.lua"))
EventManager.tEventDefs = EventManager.tEventInfos.tEventDefs
EventManager.tRandomEventOverrides = EventManager.tEventInfos.tRandomEventOverrides
EventManager.tEndGameEvents = EventManager.tEventInfos.tEndGameEvents
EventManager.rRareEventSpawnInfo = EventManager.tEventInfos.rRareEventSpawnInfo

function EventManager.updateEventDef(sDefKey, tValues)
    g_Game:updateDefWithTweak(EventManager.tEventDefs, sDefKey, tValues)
end

function EventManager.updateRareEventSpawnInfo(sDefKey, tValues)
    g_Game:updateDefWithTweak(EventManager.rRareEventSpawnInfo, sDefKey, tValues)    
end

function EventManager:staticInit()
	local Ob = {}
    Ob.tQueuedEvents = {}
	Ob.rLayer = Renderer.getLayer("EventOverlay").RenderLayer
    Ob.tActiveEventSets = {}

    local tSaveTable = g_Game:getSaveTable()
    if not tSaveTable.tEventSets then
        tSaveTable.tEventSets = {}
    end
    if not tSaveTable.tRareEvents then
        tSaveTable.tRareEvents = {}
    end
    Ob.tRareEventSaved = tSaveTable.tRareEvents
    if EventManager.rRareEventSpawnInfo then
        for i, rSpawnInfo in ipairs(EventManager.rRareEventSpawnInfo) do
            if rSpawnInfo.sKey then
                if not Ob.tRareEventSaved[rSpawnInfo.sKey] then
                    Ob.tRareEventSaved[rSpawnInfo.sKey] = {}
                    Ob.tRareEventSaved[rSpawnInfo.sKey].nNumPassedWOSpawning = 0
                end
            end
        end
    end

    local tActiveHolidays = g_Game:getActiveHolidays()
    if tActiveHolidays then
        for i, rHolidayInfo in ipairs(tActiveHolidays) do
            if rHolidayInfo.tCustomEventInfo then
                local sEventSetName = rHolidayInfo.tCustomEventInfo.sEventSetName
                if sEventSetName then
                    local rEventInfo = tEventSetInfos[sEventSetName]
                    if rEventInfo then
                        if not tSaveTable.tEventSets[sEventSetName] then
                            tSaveTable.tEventSets[sEventSetName] = {}
                        end
                        Ob.tActiveEventSets[sEventSetName] = rEventInfo:new(rHolidayInfo.tCustomEventInfo, tSaveTable.tEventSets[sEventSetName])                     
                    end
                end
            end
        end
    end

    function Ob:getRandomEventOverride()
        return EventManager.tRandomEventOverrides
    end

    function Ob:getCurEvent()
        return self.rCurEvent
    end
    
	function Ob:startEvent(rEvent)
		if not self.rLayer then
			print("ERROR: Event managers can't get a renderlayer")
		end				

		if rEvent then
			-- set the background
            self.rCurEvent = rEvent
			rEvent:start(self.rLayer)
            g_City:enableEventGeneration(false)
		end
	end

    function Ob:autoResolveEvent(rEvent)
        if rEvent then
            rEvent:autoResolve()
            self:endEvent(rEvent, nil, true)
        end
    end

	function Ob:endEvent(rEvent, endScreenCB, bAutoResolved)
		if rEvent then
            self.tLastUsedHeroes = rEvent:getAllHeroes()
            if rEvent.rTerritory then
                if rEvent:isSuccessful() then
                    rEvent.rTerritory:setEventSuccessful(rEvent)
                    g_Game:onEventSuccessful(self)
                    rEvent.rTerritory:incrementHappinessLevel()
                    for i, rEventSet in pairs(self.tActiveEventSets) do
                        rEventSet:onEventSuccessful(rEvent)                        
                    end
                else
                    rEvent.rTerritory:setEventFailed(rEvent)
                end
            end
            if endScreenCB then
                rEvent:setEndScreenCloseCB(endScreenCB)
            end
            if not bAutoResolved then
                rEvent:onEnd()
            end
            if rEvent:shouldShowNewspaper() then
                g_GUI:ShowNewspaper(rEvent)
            else
                g_GUI:ShowEventEndScreenUI(rEvent)
            end
            g_TodoList:onEventEnded(rEvent)
            local rDef = rEvent:getDef()
            if self.tRareEventSaved then
                if rDef then
                    if rDef.sRareEventKey then
                        if self.tRareEventSaved[rDef.sRareEventKey] then
                            self.tRareEventSaved[rDef.sRareEventKey].nNumPassedWOSpawning = 0
                        end
                    else
                        for key, rInfo in pairs(self.tRareEventSaved) do
                            if rInfo.nNumPassedWOSpawning then
                                rInfo.nNumPassedWOSpawning = rInfo.nNumPassedWOSpawning + 1
--                                print("SETTIGN "..key.." as "..rInfo.nNumPassedWOSpawning)
                            end
                        end
                    end
                end
            end
            self.rCurEvent = nil
		end		
	end

    function Ob:queueEvent(rEvent)
        if rEvent then
            table.insert(self.tQueuedEvents, rEvent)
        end
    end

    function Ob:startNextEventInQueue()
        if table.getn(self.tQueuedEvents) > 0 then
            g_eventManager:startEvent(self.tQueuedEvents[1])
            table.remove(self.tQueuedEvents, 1)
            return true
        else
            return false
        end
    end
    
    function Ob:forceCreateEvent(tHeroes, rTerritory, sDefKey)
        local rNewEvent = rTerritory:addEvent(sDefKey)
        for i, rHero in ipairs(tHeroes) do
            rNewEvent:addHeroToEvent(rHero)
        end
        self:startEvent(rNewEvent)
    end

    function Ob:getConstructorFncn(sEventClassName)
        if tEventClasses[sEventClassName] then
            if tEventClasses[sEventClassName].new then
                return tEventClasses[sEventClassName].new
            end
        end
        return nil
    end

    function Ob:getEventDef(sEventDefKey)
        if EventManager.tEventDefs and sEventDefKey then
            return EventManager.tEventDefs[sEventDefKey]
        end
        return nil
    end

    function Ob:getEndGameRandomEventDef()
        if EventManager.tEndGameEvents then
            return EventManager.tEndGameEvents[math.random(table.getn(EventManager.tEndGameEvents))]
        end
    end

    function Ob:getLastUsedHeroes()
        return self.tLastUsedHeroes
    end

    function Ob:runEventEndScreenClosedThread()
        -- check to see if we have any timed events running
        for i, rEventSet in pairs(self.tActiveEventSets) do
            local rThreadFncn = rEventSet:getPostEventScreenThread()
            if rThreadFncn then
                local rThread = MOAIThread.new()
                rThread:run(rThreadFncn, rEventSet)
                return true
            end
        end
        return nil
    end

    function Ob:getHolidayEventKey()
        for i, rEventSet in pairs(self.tActiveEventSets) do
            local sDefKey = rEventSet:getNextEventKey()
            if sDefKey then
                return sDefKey
            end
        end
        return nil
    end

    function Ob:getActiveHolidaySet(sEventKey)
        if sEventKey then
            return self.tActiveEventSets[sEventKey]
        end
        return nil
    end

    function Ob:getRareEventKey()
        -- process the info in order
        if EventManager.rRareEventSpawnInfo and self.tRareEventSaved then
            for i, rSpawnInfo in ipairs(EventManager.rRareEventSpawnInfo) do
                if rSpawnInfo.sKey and rSpawnInfo.rSpawnChanceInfo and rSpawnInfo.tEvents and self.tRareEventSaved[rSpawnInfo.sKey] then
                    local bCanSpawn = true
                    if rSpawnInfo.nMaxConcurrent then
                        local nNumExistingEvents = g_City:getNumRareEvents(rSpawnInfo.sKey)
                        if nNumExistingEvents >= rSpawnInfo.nMaxConcurrent then
                            bCanSpawn = false
                        end
                    end
                    if bCanSpawn then
                        local nNumEventsPassed = self.tRareEventSaved[rSpawnInfo.sKey].nNumPassedWOSpawning
                        local nChance = 0
                        for i, rInfo in ipairs(rSpawnInfo.rSpawnChanceInfo) do
                            if rInfo.tNumWOSpawn then
                                if rInfo.tNumWOSpawn[1] and rInfo.tNumWOSpawn[2] then
                                    if nNumEventsPassed >= rInfo.tNumWOSpawn[1] and nNumEventsPassed < rInfo.tNumWOSpawn[2] then
                                        nChance = rInfo.nChance
                                        break
                                    end
                                elseif rInfo.tNumWOSpawn[1] and not rInfo.tNumWOSpawn[2] then
                                    -- only upper limit
                                    if nNumEventsPassed >= rInfo.tNumWOSpawn[1] then
                                        nChance = rInfo.nChance
                                        break
                                    end
                                end
                            end
                        end
                        local nRandom = math.random(1, 100)
                        if nRandom <= nChance then
                            -- found one
                            if rSpawnInfo.tEvents then
                                local tPossibleEvents = {}
                                for i, rEventInfo in ipairs(rSpawnInfo.tEvents) do
                                    if rEventInfo.sEventKey and rEventInfo.nMinMMLevel then
                                        if g_MiddleManager and g_MiddleManager:getLevel() >= rEventInfo.nMinMMLevel then
                                            table.insert(tPossibleEvents, rEventInfo.sEventKey)
                                        end
                                    end
                                end
                                if table.getn(tPossibleEvents) > 0 then
                                    return tPossibleEvents[math.random(table.getn(tPossibleEvents))]
                                end
                            end
                        end
                    end
                end
            end
        end
        return nil
    end

    function Ob:onFinger(eventType, sx, sy)
        if self.rCurEvent and self.rCurEvent.onFinger then
            self.rCurEvent:onFinger(eventType, sx, sy)
        end
    end

    local kTIME_STEP = 0.33

	return Ob
end

return EventManager
