local EventSet = {}

function EventSet:new(rDef, tSaved)
    local Ob = {}
    Ob.rDef = rDef
    Ob.tSaved = tSaved

    Ob.tPostEventScreenThread = {}
    if not Ob.tSaved.nNumEventsCompleted then
        Ob.tSaved.nNumEventsCompleted = 0
    end
    if not Ob.tSaved.nNumOtherEventsCompleted then
        Ob.tSaved.nNumOtherEventsCompleted = 0
    end

    function Ob:canBeActive()
        if g_MiddleManager:getLevel() == 2 then
            local rTutManager = g_Game:getTutorialManager()
            if rTutManager then
                if rTutManager:getSaveVar('bGetHeroTutorialCompleted') == true then
                    return true
                end
            end
            return false
        elseif g_MiddleManager:getLevel() > 2 then
            return true
        end
        return false
    end

    function Ob:shouldShowHolidayUI()
        if self:canBeActive() then
            if self.tSaved.introCompleted == true then
                return true
            end
        end
    end

    function Ob:canAddEvent()
        if not self:canBeActive() then
            return false
        end
        if self.rDef.nMaxConcurrEvents then
            if g_City:getActiveEventsInSet(self.rDef.sEventSetName) >= self.rDef.nMaxConcurrEvents then
                return false
            end
        end
        if self.tSaved.nNumEventsCompleted > 0 then
            if self.rDef.nNumEventsToComplete then
                if self.tSaved.nNumEventsCompleted >= self.rDef.nNumEventsToComplete then        
                    return false
                end
            end
        end
        return true
    end
    
    function Ob:getNextEventKey()
        if not self:canAddEvent() then
            return nil
        end
        if self.rDef.tEventInfos then
            if self.tSaved.nNumOtherEventsCompleted >= self.rDef.nOtherEventsToAddEvent or
                self.tSaved.nNumEventsCompleted == 0 then
                for i, rEventInfo in ipairs(self.rDef.tEventInfos) do
                    if self.tSaved.nNumEventsCompleted >= rEventInfo.nMinEventsCompl then
                        if rEventInfo.tEventKeys then
                            local nNumItems = table.getn(rEventInfo.tEventKeys)
                            self.tSaved.nNumOtherEventsCompleted = 0
                            return rEventInfo.tEventKeys[math.random(nNumItems)]
                        end
                    end
                end
            end
        end
        return nil
    end

    function Ob:onEventSuccessful(rEvent)                       
        if rEvent then
            local rEventDef = rEvent:getDef()
            if rEventDef then
                if rEventDef.sEventSetName == self.rDef.sEventSetName then                    
                    self.tSaved.nNumEventsCompleted = self.tSaved.nNumEventsCompleted + 1
                    if self.rDef.nNumEventsToComplete then
                        if self.tSaved.nNumEventsCompleted > self.rDef.nNumEventsToComplete then
                            self.tSaved.nNumEventsCompleted = self.rDef.nNumEventsToComplete
                        end
                    end
                else
                    if self:canAddEvent() then
                        self.tSaved.nNumOtherEventsCompleted = self.tSaved.nNumOtherEventsCompleted + 1
                    end
                end
            end
        end
    end

    function Ob:addPostEventThreadFncn(sName, fncn)
        if sName and fncn then
            self.tPostEventScreenThread[sName] = fncn
        end
    end

    function Ob:getPostEventScreenThread()
        if not self:canBeActive() then
            return nil
        end
        if self.rDef then
            if self.rDef.tPostEventScreenThread then
                for i, rThreadInfo in ipairs(self.rDef.tPostEventScreenThread) do
                    if rThreadInfo.nNumEventsCompl and rThreadInfo.sThreadName then
                        if self.tSaved.nNumEventsCompleted >= rThreadInfo.nNumEventsCompl then
                            if not self.tSaved[rThreadInfo.sThreadName..'Completed'] then
                                if self.tPostEventScreenThread[rThreadInfo.sThreadName] then
                                    return self.tPostEventScreenThread[rThreadInfo.sThreadName] -- return the function pointer
                                end
                            end                           
                        end
                    end
                end
            end            
        end
        return nil
    end    

    function Ob:setPostEventThreadCompleted(sName)
        if self.tSaved and sName then
            self.tSaved[sName..'Completed'] = true
        end
    end

    function Ob:getProgress()
        if self.rDef.nNumEventsToComplete and self.rDef.nNumEventsToComplete > 0 then
            return (self.tSaved.nNumEventsCompleted / self.rDef.nNumEventsToComplete)
        end
        return 1        
    end

    return Ob
end

return EventSet