local FireRescueEvent = {}
local Renderer = require('Renderer')
local Event = require('event/Event')
local util = require("DFUtil")
local Character = require("Character")
local EffectInstance = require('EffectInstance')
local SpriteSheet = require('SpriteSheet')
local DFAchievement = require('DFCommon.Achievement')

local tFireInfo =
{
{ loc = { -40, -30 }, rot = -3, scale = { 1.4, 1.4 } },
{ loc = { -80, 258 }, rot = 0, scale = { 1.4, 1.4 } },
{ loc = { 340, 262 }, rot = 0, scale = { 1, 1 } },
{ loc = { 390, -64 }, rot = -3, scale = { 1.6, 1.6 } },
}
local tFailureProps =
{
    'RescueFailure_TV',
    'RescueFailure_Plant',
    'RescueFailure_Teddy',
}
local function defaultFireRescueEventThread(self)
    self:enableInteractiveUIElements(false)
    Renderer.getRenderLayer("EventOverlay"):setSortMode(MOAILayer2D.SORT_PRIORITY_ASCENDING)

    self.tFires = {}
    -- add in the fires
    for i, tFireInfo in ipairs(tFireInfo) do
        local rFire = EffectInstance:new('Effects/BuildingFire.lua', self.rLayer)
        rFire:setLoc(tFireInfo.loc[1], tFireInfo.loc[2])
        rFire:setScl(tFireInfo.scale[1], tFireInfo.scale[2])
        if tFireInfo.rot then
            rFire:setAttr(MOAITransform2D.ATTR_Z_ROT, tFireInfo.rot)
        end
        table.insert(self.tFires, rFire)
        rFire:start()
    end

    -- add in the panicking citizen
    local sCitiManDefKey = 'CitiMan'
    local sCitiGirlDefKey = 'CitiGirl'
    local sPanickingCitizenDefKey = sCitiManDefKey
    
    if math.random(1, 2) == 1 then
        sPanickingCitizenDefKey = sCitiGirlDefKey
    end
    local rPanickingCitizen = self:createEventActorFromDefKey(sPanickingCitizenDefKey)
    local rSlot = self.rEventBG:getNextSlot('PanickedCitizenLoc')
    local x, y = rSlot:getLoc()
    rPanickingCitizen:insertIntoLayer(self.rLayer)
    rPanickingCitizen:setLoc(x, y)
    rPanickingCitizen:playAnimation('callforhelp', false, false, false)

    rSlot = self.rEventBG:getNextSlot('HeroLocs')
    x, y = rSlot:getLoc()    

    local tHeroActors = self:getAllHeroActors()
    -- allow for only one hero
    local rHeroActor = tHeroActors[1]

    local bSuccessful = false
    local nDesiredAwareness = self.rDef.nDesiredAwareness or 5
    local nRandomNum = math.random(nDesiredAwareness)
    if rHeroActor:getStatAwareness() >= nRandomNum then
        bSuccessful = true
    end

    util.sleep(0.5)

    rHeroActor:setLoc(x, y)
    rHeroActor:insertIntoLayer(self.rLayer)	        
    rHeroActor:enter(true)

    -- jump into building
    rHeroActor:playAnimation('jumpprep', true, false, true)
    util.sleep(0.25)
    rSlot = self.rEventBG:getNextSlot('WindowEnterLoc1')
    x, y = rSlot:getLoc()
    rHeroActor:moveToLoc(x, y, 0.5, 'jumpin', true)
    rHeroActor:setVisible(false)
    -- play the crash effect    
    local rWindowBreak = EffectInstance:new('Effects/WindowBreak.lua', self.rLayer)
    rWindowBreak:start()
    rWindowBreak:setLoc(20, 280)

    util.sleep(3)
    rHeroActor:setVisible(true)

    if bSuccessful then
        -- success
        rSlot = self.rEventBG:getNextSlot('CitizenStartLoc')
        x, y = rSlot:getLoc()
        local sRescuedCitizenDefKey = sCitiManDefKey    
        if sPanickingCitizenDefKey == sCitiManDefKey then
            sRescuedCitizenDefKey = sCitiGirlDefKey
        end
        local rRescuedCitizen = self:createEventActorFromDefKey(sRescuedCitizenDefKey)
        rRescuedCitizen:setLoc(x, y)
        rRescuedCitizen:insertIntoLayer(self.rLayer)
        
        rSlot = self.rEventBG:getNextSlot('HeroLandLoc')
        x, y = rSlot:getLoc()
        rHeroActor:moveToLoc(x, y, 0.75, 'jumpout', false)
        
        rSlot = self.rEventBG:getNextSlot('CitizenLandLoc')
        x, y = rSlot:getLoc()
        rRescuedCitizen:moveToLoc(x, y, 0.75, 'carried', true)
        
        rRescuedCitizen:playAnimation('land', false, false, false)
        rHeroActor:playAnimation('jumpland', false, false, true)
        rRescuedCitizen:removeFromLayer(self.rLayer)

        DFAchievement.updateAchievement('RescueTenFromFire', 1)
        DFAchievement.updateAchievement('RescueHundredFromFire', 1)
    else
        
        local rProp = SpriteSheet:new('Levels/EventBG/storefront_1')
        rProp:setSpriteIndex(tFailureProps[math.random(table.getn(tFailureProps))])
        rProp:setScl(0.5, 0.5)
        rSlot = self.rEventBG:getNextSlot('CitizenStartLoc')
        local nPropX, nPropY = rSlot:getLoc()
        rProp:setLoc(nPropX, nPropY)

        self.rLayer:insertProp(rProp)

        rSlot = self.rEventBG:getNextSlot('HeroLandLoc')
        x, y = rSlot:getLoc()
        rHeroActor:moveToLoc(x, y, 0.75, 'jumpout', false)

        rSlot = self.rEventBG:getNextSlot('CitizenLandLoc')      
        local nTargX, nTargY = rSlot:getLoc()
        rProp:moveLoc(nTargX - nPropX, nTargY - nPropY, 0.75, MOAIEaseType.LINEAR)
        util.sleep(0.75)        
        rProp:moveLoc(0, -30, 0.15, MOAIEaseType.LINEAR)
        rHeroActor:playAnimation('jumpland', false, false, true)
        util.sleep(0.5)
        rPanickingCitizen:playAnimation('failreact', false, false, true)
        util.sleep(1)

        self.rLayer:removeProp(rProp)
    end
    
    rPanickingCitizen:removeFromLayer(self.rLayer)

    Renderer.getRenderLayer("EventOverlay"):setSortMode(MOAILayer2D.SORT_Y_DESCENDING)

    for i, rFire in ipairs(self.tFires) do
        rFire:stop()
    end

    self:setSuccessful(bSuccessful)
    g_eventManager:endEvent(self)     
end

function FireRescueEvent:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFireRescueEventSpriteSheetInfo'
                         
    if rDef.eventThreadFncn then
        Ob.eventThread = rDef.eventThreadFncn
    else
        Ob.eventThread = defaultFireRescueEventThread
    end

    function Ob:customAutoResolveFncn()
        local bSuccessful = false
        if tHeroActors then
            if self.rDef then
                local tHeroActors = self:getAllHeroActors()
                if tHeroActors and tHeroActors[1] then
                    local nDesiredAwareness = self.rDef.nDesiredAwareness or 5
                    local nRandomNum = math.random(nDesiredAwareness)
                    if tHeroActors[1]:getStatAwareness() >= nRandomNum then
                        bSuccessful = true
                    end
                end
            end
        end
        return bSuccessful
    end

    return Ob
end

FireRescueEvent.rDef = 
{ 
    -- level 1
    {   
        {   
            nLevel = 1,
            bgDefFile = "BurningBuilding.lua",
            tActors =
            {
--                Citizen = { rCharDef = Character.tOtherActorDefinitions.CitiGirl, },
            },
            eventClassName = 'FireRescueEvent',
			Rewards =
	        {
	            WinXP = 1,
                MaxCash = 20,
                MaxPP = 1,
            },
            nMaxHeroes = 1,
        },
    },
}

return FireRescueEvent