local GodzillaEvent1 = {}
local GodzillaEvent2 = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local DFAchievement = require('DFCommon.Achievement')

local function _checkMonsterHealth(rEvent)
    local rGodzilla = rEvent:getActor('Beastfly')
    if rGodzilla then
        if rGodzilla:getCurHP() <= 2 then
            return "Player"
        end
    end
    return nil
end

local function GodzillaEvent1Thread(self)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    

    local rGodzilla = self:getActor('Beastfly')
    rGodzilla:insertIntoLayer(self.rLayer)
    rGodzilla:setLoc(230, -210)
    rGodzilla:setImmortal(true)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:setScl(.5)
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)        
    end

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("CDF1001HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF1002DRAG", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("CDF1003HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF1004DRAG", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("CDF1005HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF1006DRAG", 'upper', true, true)
    rGodzilla:sayLine("CDF1007DRAG", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("CDF1008HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF1009DRAG", 'upper', true, true)
    rGodzilla:sayLine("CDF1010DRAG", 'upper', true, true)

    -- clear out the dialog
    g_GUI:HideDialogText()
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens, _checkMonsterHealth)
    if bSuccessful then
        util.sleep(2)
        for i, rHero in ipairs(tAliveHeroes) do
            rHero:playAnimation('breathe', true, false)
        end
        rGodzilla:sayLine("CDF1011DRAG", 'upper', true, true)
        g_GUI:HideDialogText()
        rGodzilla:playAnimation('breathe', true, false)
        rGodzilla:moveToLoc(270, 380, 0.5, nil, true)
        util.sleep(1)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        rSpeakingActor:sayLine("CDF2010HERO", 'upper', true, true)
        g_GUI:HideDialogText()
        
        rGodzilla:sayLine("CDF2011DRAG", 'upper', true, true)
        rGodzilla:sayLine("CDF2012DRAG", 'upper', true, true)
        g_GUI:HideDialogText()

        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self, onIntro1EventClosedCallback)
end

local function GodzillaEvent2Thread(self)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    

    local rGodzilla = self:getActor('Beastfly')
    rGodzilla:insertIntoLayer(self.rLayer)
    rGodzilla:setLoc(300, -250)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("CDF2001HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF2002DRAG", 'upper', true, true)
    g_GUI:HideDialogText()
    rSpeakingActor:sayLine("CDF2003HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    rGodzilla:sayLine("CDF2004DRAG", 'upper', true, true)
    rGodzilla:sayLine("CDF2013DRAG", 'upper', true, true)

    -- clear out the dialog
    g_GUI:HideDialogText()
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        util.sleep(2)
        for i, rHero in ipairs(tAliveHeroes) do
            rHero:playAnimation('breathe', true, false)
        end

        rGodzilla:sayLine("CDF2005DRAG", 'upper', true, true)
        g_GUI:HideDialogText()
        rSpeakingActor:sayLine("CDF2006HERO", 'upper', true, true)
        g_GUI:HideDialogText()
        rGodzilla:sayLine("CDF2007DRAG", 'upper', true, true)
        g_GUI:HideDialogText()
        rSpeakingActor:sayLine("CDF2008HERO", 'upper', true, true)
        g_GUI:HideDialogText()
        rGodzilla:sayLine("CDF2009DRAG", 'upper', true, true)  
        DFAchievement.updateAchievement('BeatBeastfly', 1)

        self.sWinnerPortraitPath, self.sWinnerPortraitIndex = tAliveHeroes[1]:getCurPortraitPath()
        self.sLoserPortraitPath, self.sLoserPortraitIndex = rMonsterFoot:getCurPortraitPath()

        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        local tHeroActors = self:getAllHeroActors()           
        self.sLoserPortraitPath, self.sLoserPortraitIndex = tHeroActors[1]:getCurPortraitPath()
        self.sWinnerPortraitPath, self.sWinnerPortraitIndex = rMonsterFoot:getCurPortraitPath() 

        rSpeakingActor:sayLine("CDF2010HERO", 'upper', true, true)
        g_GUI:HideDialogText()
        
        rGodzilla:sayLine("CDF2011DRAG", 'upper', true, true)
        rGodzilla:sayLine("CDF2012DRAG", 'upper', true, true)
        g_GUI:HideDialogText()

        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self, onIntro1EventClosedCallback)
end

function GodzillaEvent1:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = GodzillaEvent1Thread

    return Ob
end

function GodzillaEvent2:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = GodzillaEvent2Thread

    return Ob
end

local tEvents =
{
    GodzillaEvent1, GodzillaEvent2
}

return tEvents