local JailBreakEvent1 = {}
local JailBreakEvent2 = {}
local JailBreakEvent3 = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local Renderer = require('Renderer')
local DFAchievement = require('DFCommon.Achievement')

local function jailBreakEvent1Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)

    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end
    
    local rHench1 = self:getActor('Enemy1')
    local rHench2 = self:getActor('Enemy2')
    local rHench3 = self:getActor('Enemy3')
    local rHench4 = self:getActor('Enemy4')
    local rHench5 = self:getActor('Enemy5')
    local rHench6 = self:getActor('Enemy6')

    rHench4:playAnimation('breathe', false, false)
    rHench5:playAnimation('breathe', false, false)
    rHench6:playAnimation('breathe', false, false)

    util.sleep(0.75)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    rHench4:playAnimation('threathen_breathe', false, false)
    rHench5:playAnimation('threathen_breathe', false, false)
    rHench6:playAnimation('threathen_breathe', false, false)

    rHench2:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.2)
    rHench1:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.1)
    rHench3:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(1)

    rHench1:playAnimation('hero_entrance_react_end', false, false)
    rHench2:playAnimation('hero_entrance_react_end', false, false)
    rHench3:playAnimation('hero_entrance_react_end', false, false)

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("JBIN001HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:sayLine("JBIN015HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("JBIN002HMAN", 'upper', true, true)
    g_GUI:HideDialogText()

    rSpeakingActor:sayLine("JBIN003HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("JBIN004HMAN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:sayLine("JBIN005HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rHench1:sayLine("JBIN006HMAN", 'upper', true, true)
    g_GUI:HideDialogText()
    
    rSpeakingActor:sayLine("JBIN007HERO", 'upper', true, true)
    g_GUI:HideDialogText()
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        local rMM = g_MiddleManager:getAvatar()
        rSpeakingActor = tAliveHeroes[1]
        rMM:sayLine("JBIN009MIDM", 'upper', true, true)
        rSpeakingActor:sayLine("JBIN010HERO", 'upper', true, true)

        rMM:sayLine("JBIN011MIDM", 'upper', true, true)
        rSpeakingActor:sayLine("JBIN012HERO", 'upper', true, true)
        rMM:sayLine("JBIN013MIDM", 'upper', true, true)
        rSpeakingActor:sayLine("JBIN014HERO", 'upper', true, true)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        rEnemy1 = tAliveEnemies[1]
        if rEnemy1 then
            rEnemy1:sayLine("JBIN008HMAN", 'upper', true, true)
        end
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()

    g_eventManager:endEvent(self)
end

local function jailBreakEvent2Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)

    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end
    
    local rHench1 = self:getActor('Enemy1')
    local rHench2 = self:getActor('Enemy2')
    local rHench3 = self:getActor('Enemy3')
    local rHench4 = self:getActor('Enemy4')
    local rHench5 = self:getActor('Enemy5')
    local rHench6 = self:getActor('Enemy6')

    --skullface arrives!
    local rSkullface = self:getActor('Skullface')
    rSkullface:insertIntoLayer(self.rLayer)
    rSkullface:setLoc(340, -180)
    rSkullface:playAnimation('fight_breathe', false, false)
    
    rHench1:sayLine("JBST001HMAN", 'upper', true, true)
    rHench2:sayLine("JBST002HMAN", 'upper', true, true)
    rSkullface:sayLine("JBST003SKUL", 'upper', true, true)
    rHench3:sayLine("JBST004HMAN", 'upper', true, true)
    rSkullface:sayLine("JBST005SKUL", 'upper', true, true)
 
    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    rHench4:playAnimation('threathen_breathe', false, false)
    rHench5:playAnimation('threathen_breathe', false, false)
    rHench6:playAnimation('threathen_breathe', false, false)

    rHench2:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.2)
    rHench1:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.1)
    rHench3:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(1)

    rHench1:playAnimation('hero_entrance_react_end', false, false)
    rHench2:playAnimation('hero_entrance_react_end', false, false)
    rHench3:playAnimation('hero_entrance_react_end', false, false)

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("JBST006HERO", 'upper', true, true)
    rSkullface:sayLine("JBST007SKUL", 'upper', true, true)
    rSpeakingActor:sayLine("JBST008HERO", 'upper', true, true)
    rSkullface:sayLine("JBST009SKUL", 'upper', true, true)
    rHench1:sayLine("JBST010HMAN", 'upper', true, true)
    g_GUI:HideDialogText()
    rSkullface:moveToLoc(600, -180, 2, 'run', true)
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        rHench1:sayLine("JBST011HMAN", 'upper', true, true)
        rSpeakingActor:sayLine("JBST014HERO", 'upper', true, true)
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
    else
        rEnemy1 = tAliveEnemies[1]
        if rEnemy1 then
            rEnemy1:sayLine("JBST012HMAN", 'upper', true, true)
            rEnemy1:sayLine("JBST013HMAN", 'upper', true, true)
        end
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end

    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()

    g_eventManager:endEvent(self)
end

local function jailBreakEvent3Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)

    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end
    
    local rHench1 = self:getActor('Enemy1')
    local rHench2 = self:getActor('Enemy2')
    local rHench3 = self:getActor('Enemy3')
    local rHench4 = self:getActor('Enemy4')
    local rHench5 = self:getActor('Enemy5')
    local rHench6 = self:getActor('Enemy6')

    --skullface arrives!
    local rSkullface = self:getActor('Skullface')
    rSkullface:insertIntoLayer(self.rLayer)
    rSkullface:setLoc(340, -100)
    rSkullface:playAnimation('fight_breathe', false, false)
    
    rSkullface:sayLine("JBBB001SKUL", 'upper', true, true)
    rSkullface:sayLine("JBBB002SKUL", 'upper', true, true)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    rHench4:playAnimation('threathen_breathe', false, false)
    rHench5:playAnimation('threathen_breathe', false, false)
    rHench6:playAnimation('threathen_breathe', false, false)

    rHench2:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.2)
    rHench1:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(0.1)
    rHench3:playAnimation('hero_entrance_react', true, false, false)
    util.sleep(1)

    rHench1:playAnimation('hero_entrance_react_end', false, false)
    rHench2:playAnimation('hero_entrance_react_end', false, false)
    rHench3:playAnimation('hero_entrance_react_end', false, false)

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("JBBB003HERO", 'upper', true, true)
    rSkullface:sayLine("JBBB004SKUL", 'upper', true, true)
    rSpeakingActor:sayLine("JBBB005HERO", 'upper', true, true)
    rSkullface:sayLine("JBBB006SKUL", 'upper', true, true)

    rSpeakingActor:sayLine("JBBB007HERO", 'upper', true, true)
    rSkullface:sayLine("JBBB008SKUL", 'upper', true, true)
    rSpeakingActor:sayLine("JBBB009HERO", 'upper', true, true)
    rSkullface:sayLine("JBBB010SKUL", 'upper', true, true)

    g_GUI:HideDialogText()
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { }
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        tEnemyWaves = { { rSkullface } }
        bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tAliveHeroes, tEnemyWaves, tCitizens)
        if bSuccessful then
            local rMM = g_MiddleManager:getAvatar()
            rSpeakingActor = tAliveHeroes[1]
            rMM:sayLine("JBBB011MIDM", 'upper', true, true)
            rSpeakingActor:sayLine("JBBB012HERO", 'upper', true, true)
            rMM:sayLine("JBBB013MIDM", 'upper', true, true)

            self.sWinnerPortraitPath, self.sWinnerPortraitIndex =  tAliveHeroes[1]:getCurPortraitPath()
            self.sLoserPortraitPath, self.sLoserPortraitIndex = rSkullface:getCurPortraitPath()
            DFAchievement.updateAchievement('BeatJailBreak', 1)
            self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
        else
            local tHeroActors = self:getAllHeroActors()           
            self.sLoserPortraitPath, self.sLoserPortraitIndex = tHeroActors[1]:getCurPortraitPath()
            self.sWinnerPortraitPath, self.sWinnerPortraitIndex = rSkullface:getCurPortraitPath() 
            rSkullface:playAnimation("cheer", false, false)            
            g_Audio:playCue('MMOJ_Audio/SFX/EnemySFX/SkullFaceKackle')
            rSkullface:sayLine("JBBB014SKUL", 'upper', true, true)            
            self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)            
        end
    else
        local tHeroActors = self:getAllHeroActors()
        self.sLoserPortraitPath, self.sLoserPortraitIndex = tHeroActors[1]:getCurPortraitPath()
        self.sWinnerPortraitPath, self.sWinnerPortraitIndex = rSkullface:getCurPortraitPath() 
        rSkullface:playAnimation("cheer", false, false)        
        g_Audio:playCue('MMOJ_Audio/SFX/EnemySFX/SkullFaceKackle')
        rSkullface:sayLine("JBBB014SKUL", 'upper', true, true)

        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end

    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()

    g_eventManager:endEvent(self)
end

function JailBreakEvent1:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = jailBreakEvent1Thread

    return Ob
end

function JailBreakEvent2:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = jailBreakEvent2Thread

    return Ob
end

function JailBreakEvent3:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = jailBreakEvent3Thread

    return Ob
end

local tEvents =
{
    JailBreakEvent1, JailBreakEvent2, JailBreakEvent3
}

return tEvents