local MostWantedEvent = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local Item = require('Item')
local DFAchievement = require('DFCommon.Achievement')

local bDBG_ImInAHurry = false

local function defaultMostWantedEventThread(self)
    self.rEventUI:enableInteractiveUIElements(false)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)
   
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:insertIntoLayer(self.rLayer)
        rEnemy:setLoc(500, -100)
        --rEnemy:playAnimation("fight_breathe", false, false)
        rEnemy:moveToLoc(125, -160, 3, "run", true)
        rEnemy:playAnimation("fight_breathe", false, false)
    end

    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    if not bDBG_ImInAHurry then
        util.sleep(0.75)
    end
    
    for i, rHeroActor in ipairs(tHeroActors) do
        if not bDBG_ImInAHurry then
            util.sleep(math.random(0.1, 0.3))
        end
        rHeroActor:insertIntoLayer(self.rLayer)	            
        rHeroActor:enter(true)
        rHeroActor:playAnimation("attack_breathe", true)
    end
    
    local sSequentialSet = nil
    local rSpeakingActor = tHeroActors[math.random(table.getn(tHeroActors))]
    local rSpeakingEnemy = tEnemies[math.random(table.getn(tEnemies))]
    
    -- if enemies require specific dialog, force it
    if rSpeakingEnemy.rDef.bForceSpecialDialog then 
        sSequentialSet = rSpeakingEnemy.rDef.sSequentialResponseSet
    end       

    if not bDBG_ImInAHurry then
        
        if sSequentialSet then
            -- play staged sequential lines
            local tDialogSets = g_DialogManager:getDialogSetInstance(sSequentialSet)
            local tSet = tDialogSets[math.random(table.getn(tDialogSets))]
            for i, rLineInfo in ipairs(tSet.tLines) do
                if rLineInfo.sSpeaker == 'Hero' then
                    rSpeakingActor:sayLine(rLineInfo.lineCode, 'upper', true, true)
                else
                    rSpeakingEnemy:sayLine(rLineInfo.lineCode, 'upper', true, true)
                end
            end        
        end
    end
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }   
    
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    self.rEventUI:enableInteractiveUIElements(false)
    if bSuccessful then
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
        DFAchievement.updateAchievement('BeatOneMostWanted', 1)
        DFAchievement.updateAchievement('BeatAllMostWanted', 1)
        if self.rDef and self.rDef.tActors and self.rDef.tActors.Enemies and self.rDef.tActors.Enemies.Enemy1 and self.rDef.tActors.Enemies.Enemy1.sCharDefKey then
            -- save that this most wanted thug is defeated
            g_Game:saveVar('Beat'..self.rDef.tActors.Enemies.Enemy1.sCharDefKey, true)
        end
        if self.sDefKey then
            g_Game:saveVar('Completed'..self.sDefKey, true)
        end
    else
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens)
    end

    self:setSuccessful(bSuccessful)
    
    g_eventManager:endEvent(self)
end

function MostWantedEvent:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    if rDef.eventThreadFncn then
        Ob.eventThread = rDef.eventThreadFncn
    else
        Ob.eventThread = defaultMostWantedEventThread
    end

    return Ob
end

return MostWantedEvent
