local RobberyEvent = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local DFAchievement = require('DFCommon.Achievement')

local bDBG_ImInAHurry = false

local function defaultRobberyEventThread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    local rCitizen = self:getActor("Citizen")   
    -- setup extra actors
    -- hack hack until we actually set these guys up as actors
    local bCitiFlipX = false
    if rCitizen.rDef == Character.getCharDef('CitiGirl') then
        bCitiFlipX = true
    end
    rCitizen:playAnimation("threathened_breathe", bCitiFlipX, false)
    self:snapActorToSlot("CitizenLocs", rCitizen)
    rCitizen:insertIntoLayer(self.rLayer)
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:insertIntoLayer(self.rLayer)
        rEnemy:playAnimation("fight_breathe", true, false)
    end

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("threathen_breathe", true, false)
        util.sleep(math.random(10, 13) * 0.01)
    end

    if not bDBG_ImInAHurry then
        util.sleep(0.75)
    end
    
    for i, rHeroActor in ipairs(tHeroActors) do
        if not bDBG_ImInAHurry then
            util.sleep(math.random(0.1, 0.3))
        end
        rHeroActor:insertIntoLayer(self.rLayer)	            
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end
    
    for i, rEnemy in ipairs(tEnemies) do
        if not bDBG_ImInAHurry then
            util.sleep(math.random(10, 13) * 0.01)          
        end
        local bBlock = false
        if i == table.getn(tEnemies) and not bDBG_ImInAHurry then
            bBlock = true
        end
        rEnemy:playAnimation("hero_entrance_react", true, false, bBlock)
    end
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("hero_entrance_react_end", false, false)
        util.sleep(math.random(10, 13) * 0.01)
    end

    local sSequentialSet = nil
    local rSpeakingActor = tHeroActors[math.random(table.getn(tHeroActors))]
    local rSpeakingEnemy = tEnemies[math.random(table.getn(tEnemies))]
    
    -- if enemies require specific dialog, force it
    if rSpeakingEnemy.rDef.bForceSpecialDialog then 
        sSequentialSet = rSpeakingEnemy.rDef.sSequentialResponseSet
    else    
        -- if hero has specific lines... randomly try to grab one
        if rSpeakingActor.rDef.sSequentialResponseSet then
            if math.random(1, 4) == 1 then
                sSequentialSet = rSpeakingActor.rDef.sSequentialResponseSet
            end
        end        
        -- if no hero lines grabbed and there ARE enemy sequential lines, randomly try to grab one
        if not sSequentialSet and rSpeakingEnemy.rDef.sSequentialResponseSet then
            if math.random(1, 4) == 1 then
                sSequentialSet = rSpeakingEnemy.rDef.sSequentialResponseSet
            end
        end
    end
 
       

    if not bDBG_ImInAHurry then
        
        if sSequentialSet then
            -- play staged sequential lines
            local tDialogSets = g_DialogManager:getDialogSetInstance(sSequentialSet)
            local tSet = tDialogSets[math.random(table.getn(tDialogSets))]
            for i, rLineInfo in ipairs(tSet.tLines) do
                if rLineInfo.sSpeaker == 'Hero' then
                    rSpeakingActor:sayLine(rLineInfo.lineCode, 'upper', true, true)
                else
                    rSpeakingEnemy:sayLine(rLineInfo.lineCode, 'upper', true, true)
                end
            end        
        else            
            -- or just use random one liners
            rSpeakingActor:sayLineFromDialogset("HeroRobberyEntrance", 'upper', true, true)
            local sResponseSet = 'GenericVillianReact'
            if rSpeakingEnemy.rDef.sRandomResponseSet then
                if math.random(1, 4) == 1 then
                    sResponseSet = rSpeakingEnemy.rDef.sRandomResponseSet
                end
            end
            rSpeakingEnemy:sayLineFromDialogset(sResponseSet, 'upper', true, true)
        end
    end
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    rCitizen:playAnimation("watch_fight", bCitiFlipX, false)
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { rCitizen }
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("threathen_breathe", false, false)
    end
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    self.rEventUI:enableInteractiveUIElements(false)
    if bSuccessful then
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
        if self.rDef and self.rDef.sRareEventKey then
            if self.rDef.sRareEventKey == 'seniorthugs' then
                DFAchievement.updateAchievement('BeatSeniorThug', 1)
            end
        end
    else
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens)
    end

    self:setSuccessful(bSuccessful)
    
    g_eventManager:endEvent(self)
end

function RobberyEvent:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    if rDef.eventThreadFncn then
        Ob.eventThread = rDef.eventThreadFncn
    else
        Ob.eventThread = defaultRobberyEventThread
    end

    return Ob
end

return RobberyEvent
