local RunawayBusEvent = {}
local Renderer = require('Renderer')
local Event = require('event/Event')
local util = require("DFUtil")
local Character = require("Character")
local DFAchievement = require('DFCommon.Achievement')

local function defaultRunawayBusEventThread(self)
    self:enableInteractiveUIElements(false)
    Renderer.getRenderLayer("EventOverlay"):setSortMode(MOAILayer2D.SORT_PRIORITY_ASCENDING)
    -- TEMP

    local rBus = self:getActor('RunawayBus')
    local rSlot = self.rEventBG:getNextSlot('BusStartLoc')
    local x, y = rSlot:getLoc()
    rBus:setLoc(x, y)
    rBus:insertIntoLayer(self.rLayer)
    rBus:playAnimation('run', false, false)

    rSlot = self.rEventBG:getNextSlot('HeroLocs')
    x, y = rSlot:getLoc()
    local tHeroActors = self:getAllHeroActors()
    -- allow for only one hero
    local rHeroActor = tHeroActors[1]

    local bSuccessful = false
    local nDesiredPower = self.rDef.nDesiredPower or 5
    local nRandomNum = math.random(nDesiredPower)
    if rHeroActor:getStatPower() >= nRandomNum then
        bSuccessful = true
    end
    if bSuccessful then
        g_Audio:playCue('MMOJ_Audio/SFX/Battle/BusSuccess')
    else
        g_Audio:playCue('MMOJ_Audio/SFX/Battle/BusFail')
    end

    util.sleep(0.5)

    rHeroActor:setLoc(x, y)
    rHeroActor:insertIntoLayer(self.rLayer)	        
    rHeroActor:enter(true)
    rHeroActor:playAnimation('breathe', true, false)    
    
    rSlot = self.rEventBG:getNextSlot('BusAtHeroLoc')
    x, y = rSlot:getLoc()
    rBus:moveToLoc(x, y, 0.75, 'run', true)

    rHeroActor:playAnimation('stopbus', true, false)

    if bSuccessful then
        -- success
        rSlot = self.rEventBG:getNextSlot('HeroSuccessLoc')
        x, y = rSlot:getLoc()
        rHeroActor:moveToLoc(x, y, 1, nil, false)
        rSlot = self.rEventBG:getNextSlot('BusSuccessLoc')
        x, y = rSlot:getLoc()
        rBus:moveToLoc(x, y, 1, 'run', true)
        
        rBus:playAnimation('stopped', false, false)
        util.sleep(1)
        
        rHeroActor:playAnimation('stopbussuccess', true, false, true)

        DFAchievement.updateAchievement('StopTenVehicles', 1)
        DFAchievement.updateAchievement('StopHundredVehicles', 1)
        
        util.sleep(0.5)
    else
        -- failure
        rSlot = self.rEventBG:getNextSlot('HeroFailureLoc')
        x, y = rSlot:getLoc()
        rHeroActor:moveToLoc(x, y, 4, nil, false)
        rSlot = self.rEventBG:getNextSlot('BusFailureLoc')
        x, y = rSlot:getLoc()
        rBus:moveToLoc(x, y, 4, 'run', true)
    end

    Renderer.getRenderLayer("EventOverlay"):setSortMode(MOAILayer2D.SORT_Y_DESCENDING)
    self:setSuccessful(bSuccessful)
    g_eventManager:endEvent(self)     
end

function RunawayBusEvent:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tRunawayBusEventSpriteSheetInfo'
                         
    if rDef.eventThreadFncn then
        Ob.eventThread = rDef.eventThreadFncn
    else
        Ob.eventThread = defaultRunawayBusEventThread
    end

    function Ob:customAutoResolveFncn()
        local bSuccessful = false
        if tHeroActors then
            if self.rDef then
                local tHeroActors = self:getAllHeroActors()
                if tHeroActors and tHeroActors[1] then
                    local nDesiredPower = self.rDef.nDesiredPower or 5
                    local nRandomNum = math.random(nDesiredPower)
                    if tHeroActors[1]:getStatPower() >= nRandomNum then
                        bSuccessful = true
                    end
                end
            end
        end
        return bSuccessful
    end

    return Ob
end

return RunawayBusEvent