local ValentinesEvent1 = {}
local ValentinesEvent2 = {}
local ValentinesEventSetInfo = {}
local Event = require('event/Event')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local Character = require("Character")
local Renderer = require('Renderer')
local DFAchievement = require('DFCommon.Achievement')
local EventSet = require('event/EventSet')
local EffectInstance = require('EffectInstance')

local function valentinesEvent1Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    -- setup hero
    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tEnemies = self:getAllEventEnemies()    
    self:moveAllEnemiesToSlot(tEnemies, true)

    local rCitizen = self:getActor("Citizen")   
    -- setup extra actors
    -- hack hack until we actually set these guys up as actors
    local bCitiFlipX = false
    if rCitizen.rDef == Character.getCharDef('CitiGirl') then
        bCitiFlipX = true
    end
    rCitizen:playAnimation("threathened_breathe", bCitiFlipX, false)
    self:snapActorToSlot("CitizenLocs", rCitizen)
    rCitizen:insertIntoLayer(self.rLayer)
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:insertIntoLayer(self.rLayer)
        rEnemy:playAnimation("fight_breathe", true, false)
    end

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("threathen_breathe", true, false)
        util.sleep(math.random(10, 13) * 0.01)
    end
    
    util.sleep(0.75)    
    
    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)	            
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end
    
    for i, rEnemy in ipairs(tEnemies) do
        util.sleep(math.random(10, 13) * 0.01)          
        local bBlock = false
        if i == table.getn(tEnemies) and not bDBG_ImInAHurry then
            bBlock = true
        end
        rEnemy:playAnimation("hero_entrance_react", true, false, bBlock)
    end
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("hero_entrance_react_end", false, false)
        util.sleep(math.random(10, 13) * 0.01)
    end

    local sSequentialSet = 'ValentineThugEncounter'
    local rSpeakingActor = tHeroActors[math.random(table.getn(tHeroActors))]
    local rSpeakingEnemy = tEnemies[1]
    
    -- play staged sequential lines
    local tDialogSets = g_DialogManager:getDialogSetInstance(sSequentialSet)
    local tSet = tDialogSets[math.random(table.getn(tDialogSets))]
    for i, rLineInfo in ipairs(tSet.tLines) do
        if rLineInfo.sSpeaker == 'Hero' then
            rSpeakingActor:sayLine(rLineInfo.lineCode, 'upper', true, true)
        else
            rSpeakingEnemy:sayLine(rLineInfo.lineCode, 'upper', true, true)
        end
    end        
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    rCitizen:playAnimation("watch_fight", bCitiFlipX, false)
    
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tEnemies }       
    local tCitizens = { rCitizen }
    
    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation("threathen_breathe", false, false)
    end
    self.rEventUI:enableInteractiveUIElements(true)
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    self.rEventUI:enableInteractiveUIElements(false)
    if bSuccessful then
        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
        if math.random(100) < 25 then
            local rMM = g_MiddleManager:getAvatar()
            local tDialogSets = g_DialogManager:getDialogSetInstance('ValentinesMMPostBattle')
            if tDialogSets and rMM then
                local tSet = tDialogSets[math.random(table.getn(tDialogSets))]
                if tSet then
                    for i, rLineInfo in ipairs(tSet.tLines) do
                        rMM:sayLine(rLineInfo.lineCode, 'upper', true, true)
                    end
                end
                g_GUI:HideDialogText()
            end
        end
    else
        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens)
    end

    self:setSuccessful(bSuccessful)
    
    g_eventManager:endEvent(self)
end

local function _checkHealth(rEvent)
    local rEnemy = rEvent:getActor('AlienBoss')
    if rEnemy then
        if rEnemy:getCurHP() <= 2 then
            return "Player"
        end
    end
    return nil
end

local function actorKOCB(self, rActor)
    local rGun = self:getActor('LoveGunLarge')
    if rActor == rGun then
        rGun:setLoc(240, -200)
        rGun:setScl(0.5)  
        
        local x,y = rGun:getLoc()
        local rEffectInstance = EffectInstance:new('Effects/Explosion.lua', Renderer.getLayer('UIOverlay').RenderLayer)
        rEffectInstance:setScl(6, 6)
        rEffectInstance:setLoc(x+530, y-350)
        rEffectInstance:start()
    end
end

local function valentinesEvent2Thread(self)
    self.rEventUI:enableInteractiveUIElements(false)
    self.actorKOCallback = actorKOCB

    local tHeroActors = self:getAllHeroActors()
    self:moveAllHeroesToSlot(tHeroActors, true)

    -- setup enemy
    local tAllEnemies = self:getAllEventEnemies()    
    local rMadScientist = self:getActor('MadScientist')   
    local rLoveGun = self:getActor('LoveGunLarge')

    local tEnemies = {}

    for i, rEnemy in ipairs(tAllEnemies) do
        if rEnemy ~= rMadScientist and rEnemy ~= rLoveGun then
            table.insert(tEnemies, rEnemy)
        end
    end

    self:moveAllEnemiesToSlot(tEnemies, true)

    for i, rEnemy in ipairs(tEnemies) do
        rEnemy:playAnimation('breathe', true, false)
        rEnemy:insertIntoLayer(self.rLayer)
    end

    rMadScientist:setLoc(340, -210)
    rMadScientist:insertIntoLayer(self.rLayer)
    rMadScientist:playAnimation('fight_breathe', false, false)

    rLoveGun:setLoc(260, -190)
    rLoveGun:insertIntoLayer(self.rLayer)
    rLoveGun:playAnimation('fight_breathe', false, false)

    util.sleep(0.75)

    for i, rHeroActor in ipairs(tHeroActors) do
        util.sleep(math.random(0.1, 0.3))
        rHeroActor:insertIntoLayer(self.rLayer)
        rHeroActor:enter(true)
        rHeroActor:playAnimation("breathe", true)
    end

    local rSpeakingActor = tHeroActors[1]
    rSpeakingActor:sayLine("VAL1001HERO", 'upper', true, true)

    for i, rEnemy in ipairs(tEnemies) do
        if rEnemy ~= rLoveGun then
            util.sleep(math.random(0.2, 0.4))
            rEnemy:playAnimation("hero_entrance_react", true, false)
        end
    end
    
    util.sleep(1)
    
    for i, rEnemy in ipairs(tEnemies) do
        if rEnemy ~= rLoveGun then
            rEnemy:playAnimation("hero_entrance_react_end", false, false)
        end
    end

    rMadScientist:sayLine("VAL1002MADS", 'upper', true, true)
    rMadScientist:sayLine("VAL1003MADS", 'upper', true, true)
    rSpeakingActor:sayLine("VAL1004HERO", 'upper', true, true)
    rMadScientist:sayLine("VAL1005MADS", 'upper', true, true)
    rMadScientist:sayLine("VAL1016MADS", 'upper', true, true)
    rMadScientist:sayLine("VAL1006MADS", 'upper', true, true)
    rMadScientist:sayLine("VAL1007MADS", 'upper', true, true)
    rSpeakingActor:sayLine("VAL1008HERO", 'upper', true, true)
    rMadScientist:sayLine("VAL1009MADS", 'upper', true, true)

    -- clear out the dialog
    g_GUI:HideDialogText()
    -- fight!
    local tHeroes = tHeroActors
    local tEnemyWaves = { tAllEnemies }
    local tCitizens = { }
    
    local bSuccessful, tAliveHeroes, tAliveEnemies, tAliveCitizens = self:fightThread(tHeroes, tEnemyWaves, tCitizens)
    if bSuccessful then
        rSpeakingActor = tAliveHeroes[1]
        if rSpeakingActor then
            rSpeakingActor:playAnimation('breathe', true, false)
        end

        rMadScientist:sayLine("VAL1010MADS", 'upper', true, true)
        rMadScientist:moveToLoc(800, -210, 1, 'run', true)
        
        local rMM = g_MiddleManager:getAvatar()
        if rMM and rSpeakingActor then
            rMM:sayLine("VAL1011MIDM", 'upper', true, true)
            rSpeakingActor:sayLine("VAL1012HERO", 'upper', true, true)
            rMM:sayLine("VAL1013MIDM", 'upper', true, true)           
            rSpeakingActor:sayLine("VAL1014HERO", 'upper', true, true)            
        end

        self:playPlayerWonSequence(tAliveHeroes, tAliveCitizens)
        DFAchievement.updateAchievement('BeatValentineBoss', 1)
    else
        if rMadScientist then
            rMadScientist:playAnimation('cheer', false, false)
            rMadScientist:sayLine("VAL1015MADS", 'upper', true, true)
        end

        self:playPlayerLostSequence(tAliveEnemies, tAliveCitizens, true)
    end
    -- must be set to be successful
    self:setSuccessful(bSuccessful)
    
    -- clear out the dialog
    g_GUI:HideDialogText()
    
    g_eventManager:endEvent(self)
end


function ValentinesEvent1:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = valentinesEvent1Thread

    return Ob
end

function ValentinesEvent2:new(rTerritory, rDef, tSaved)
    local Ob = Event:new(rTerritory, rDef, tSaved)
    Ob.sSpriteSheetKey = 'tFightEventSpriteSheetInfo'

    Ob.eventThread = valentinesEvent2Thread

    return Ob
end

local function eventIntroThread(rEventSet)
    if rEventSet then
        for k,v in pairs(g_Game.tEnabledActions) do
            g_Game.tEnabledActions[k] = false
        end

        local rMM = g_MiddleManager:getAvatar()
        if rMM then
            g_CurrentMap:centerOnRoom('mm')
            rMM:sayLine("VLMM001MIDM", 'lower', true, true)
            rMM:sayLine("VLMM002MIDM", 'lower', true, true)
            rMM:sayLine("VLMM003MIDM", 'lower', true, true)
            rMM:sayLine("VLMM004MIDM", 'lower', true, true)
            rMM:sayLine("VLMM005MIDM", 'lower', true, true)
            g_GUI:HideDialogText()
        end

        for k,v in pairs(g_Game.tEnabledActions) do
            g_Game.tEnabledActions[k] = true
        end
        rEventSet:setPostEventThreadCompleted('intro')
    end
end

local function eventEndThread(rEventSet)
    if rEventSet then        
        g_GUI:ShowValentinesEndCS()
        rEventSet:setPostEventThreadCompleted('end')
    end
end

function ValentinesEventSetInfo:new(rDef, tSaved)
    local Ob = EventSet:new(rDef, tSaved)

    Ob:addPostEventThreadFncn('intro', eventIntroThread)
    Ob:addPostEventThreadFncn('end', eventEndThread)

    return Ob
end

local tEvents =
{
    rEventSetInfo = ValentinesEventSetInfo,
    tEvents = {  ValentinesEvent1, ValentinesEvent2 },
}

return tEvents