--local jit = require("jit")    
--print("jit status: " .. tostring(jit.status()))

g_EnableTweakables = true
g_EnableNews = true
g_EnableCustomerService = true
g_EnableAnalytics = true
g_EnableIAP = true
g_EnableCrittercism = true
g_UseLocalServer = false

g_IsDevBuild = false

g_ServiceURLs = 
{
    analytics = "http://dfp-mmoj-2.appspot.com/telem",
    tweaks = "http://dfp-mmoj.appspot.com/tweaks",
    csr = "http://dfp-mmoj.appspot.com/checkcsr",
    receipts = "http://dfp-mmoj.appspot.com/shopverify",
    news = "http://dfp-mmoj.appspot.com/news",
    apsalar = "http://e.apsalar.com/api/v1/",
    error = "http://dfp-mmoj-2.appspot.com/error",
}

if g_UseLocalServer then
    g_ServiceURLs.analytics = "http://localhost:8081/telem"
    g_ServiceURLs.tweaks = "http://localhost:8080/tweaks"
    g_ServiceURLs.csr = "http://localhost:8080/checkcsr"
    g_ServiceURLs.receipts = "http://localhost:8080/shopverify"
    g_ServiceURLs.news = "http://localhost:8080/news"
    g_ServiceURLs.error = "http://localhost:8081/error"
end

-- For spinning off various test builds that are like real
-- but with different analytic id's and other useful behavior
-- YOU MUST COMMENT THIS OUT IN CERT
--g_TestBuild = "cdn"

-- This is a last minute hack to async load stuff.
g_IsInStartupSequence = false

local Renderer = require('Renderer')
local Character = require('Character')
local HeadQuarters = require('HeadQuarters')
local input = require('input')
local Map = require('Map')
local Territory = require('Territory')
local EventManager = require('event/EventManager')
local MissionManager = require('mission/MissionManager')
local Gui = require('ui/Gui')
local util = require('DFUtil')
local MiddleManager = require('MiddleManager')
local TileMap = require('TileMap')
local DFAnalytics = require('DFCommon.Analytics')
local DFNetNews = require('DFCommon.NetNews')
local DFNetTweaks = require('DFCommon.NetTweaks')
local DFNetCustomerService = require('DFCommon.NetCustomerService')
local DFAchievement = require('DFCommon.Achievement')
local DFIAP = require('DFCommon.IAP')
local DFFile = require('DFCommon.File')
local DFAudio = require('DFCommon.Audio')
local DFSaveLoad = require('DFCommon.SaveLoad')
local DialogManager = require('DialogManager')
local GameRules = require('GameRules')
local MmojAudio = require('MmojAudio')
local TodoListManager = require('TodoListManager')

g_Game = nil
---------------- HELPER FUNCTIONS ----------------
local function _initializeGameRules()
    g_Game = GameRules.staticInit()
end

local function _initializeHQ()
	g_HQ = HeadQuarters:new()
	g_CurrentMap = g_HQ.rMap
end

local function _initializeEventManager()
	g_eventManager = EventManager:staticInit()
end

local function _initializeChars()
	-- initializing from a set list
	-- setup callbacks for the characters
    local tSaveTable = g_Game:getSaveTable()
    if tSaveTable then
        if tSaveTable.tCharacters then
            -- load in the characters
            for charDefKey, tCharSaveTable in pairs(tSaveTable.tCharacters) do
                g_Game:addHeroToGame(charDefKey)
                if g_Game:isHeroActive(charDefKey) then
                    g_Game:setHeroActive(charDefKey, true)
                end
            end
        else
            if Character.tInitialHero then
                for i, charDefKey in pairs(Character.tInitialHero) do
                    g_Game:addHeroToGame(charDefKey)
                    g_Game:setHeroActive(charDefKey, true)
                end
            else
                print('COULD NOT LOAD CHAR DEFS')
            end	            
        end
    end
end

local function _initializeTerritories()
	g_City = Territory:createCity()
end

local function _initializeMissionManager()
    g_MissionManager = MissionManager:init()
end

local function _initializeMiddleManager()
    g_MiddleManager = MiddleManager:new()
end

local function _initializeDialogManager()
    g_DialogManager = DialogManager:new('MainGame_enUS.lua', 'DialogSets.lua')
end

local function _initializeAchievements(tSaved)
    DFAchievement.init(dofile(DFFile.getDataPath('Achievements/Achievements.lua')), tSaved)
end

local function _initializeAudio()
    g_Audio = MmojAudio:init()
end

local function _initializeCrittercism()
    local function onError(message)
    	-- only want to get 1 traceback through this function so we don't spam errors
    	MOAISim.setTraceback(function(message) end)
    
        local errorLogBuffer = MOAIDataBuffer.new()
        errorLogBuffer:base64Encode(message)    

        local stackTrace = tostring(debug.traceback())       
        local abbrevStackLen = math.min(#stackTrace, 384)  
        local abbrevStackTrace = string.sub(stackTrace, 0, abbrevStackLen)
        
        local stackBuffer = MOAIDataBuffer.new()
        stackBuffer:base64Encode(abbrevStackTrace)
        
        -- call into our own error handler
        local errorVars = {}
        errorVars.buildId = MOAIEnvironment.appVersion
        local sExtraLogString = ""
        if MOAIEnvironment.appVersion then
            sExtraLogString = sExtraLogString .. "appVersion: "..MOAIEnvironment.appVersion.." "
        end
        if MOAIEnvironment.appBuild then
            sExtraLogString = sExtraLogString .. "appBuild: "..MOAIEnvironment.appBuild.." "
        end
        if MOAIEnvironment.devBrand then
            sExtraLogString = sExtraLogString .. "devBrand: "..MOAIEnvironment.devBrand.." "
        end
        if MOAIEnvironment.devName then
            sExtraLogString = sExtraLogString .. "devName: "..MOAIEnvironment.devName.." "
        end
        if MOAIEnvironment.devManufacturer then
            sExtraLogString = sExtraLogString .. "devManufacturer: "..MOAIEnvironment.devManufacturer.." "
        end
        if MOAIEnvironment.devModel then
            sExtraLogString = sExtraLogString .. "devModel: "..MOAIEnvironment.devModel.." "
        end
        if MOAIEnvironment.osVersion then
            sExtraLogString = sExtraLogString .. "osVersion: "..MOAIEnvironment.osVersion.." "
        end
        errorVars.log = message.." "..sExtraLogString --errorLogBuffer:getString()
        errorVars.stack = abbrevStackTrace --stackBuffer:getString()
        errorVars.type = "lua"
        
        local errorUrl = g_ServiceURLs.error .. "?"
        errorUrl = errorUrl .. "&buildId=" .. errorVars.buildId
        errorUrl = errorUrl .. "&log=" .. errorVars.log
        errorUrl = errorUrl .. "&stack=" .. errorVars.stack
        errorUrl = errorUrl .. "&type=" .. errorVars.type
        
        
        --print("calling error URL: " .. errorUrl)
        
        local errorVarsString = MOAIJsonParser.encode(errorVars)
        
        local errorBuffer = MOAIDataBuffer.new()
        errorBuffer:base64Encode(errorVarsString)
        
        print("errorVarsString:\n\n" .. errorVarsString .. "\n\n")
        
        local httpTask = MOAIHttpTask.new()
        httpTask:httpPost(g_ServiceURLs.error, errorBuffer:getString(), nil, nil, true)
        --[[
        -- get trace
        local trace = message .. '\n' .. stackTrace          

		local function forceCrash(trace)
	        print( "going to force crash:\n\n" .. trace )
			
			-- let a small amount of time go by so we can get the HTTP send to process
			util.sleep(0.5)
	
			-- force ze crash
			if MOAICrittercism and MOAICrittercism.forceException then
				MOAICrittercism.forceException(trace)
			end
		end

        local rCrashThread = MOAIThread.new()
		rCrashThread:run(forceCrash, trace)
        ]]--
    end
    
    if MOAICrittercism and g_EnableCrittercism then
        -- disabling actual crittercism, but allow logging
        --MOAICrittercism.init( "50371e56be790e19d800000c", "cextqamnbnwxfigprvqtyjpv48w5", "dmcek9alhpyjfqvigkwdtamf6nl70nur" )
    
        MOAISim.setTraceback(onError)
    elseif g_UseLocalServer then
        -- allow testing errors on windows local servers
        MOAISim.setTraceback(onError)
    end
    
end

local function _initializeTodoListManager()
    g_TodoList = TodoListManager:init()
end

function _loadGame()
    _initializeMiddleManager()
	_initializeEventManager()
	_initializeTerritories()
	_initializeHQ()
	_initializeChars()
    g_MiddleManager:createAvatar()
    _initializeAchievements(g_Game:getAchievementSaveTable())
    _initializeTodoListManager()
    
    SetGameState("Run")
end

---------------- SHUTDOWN HELPER FUNCTIONS ----------------
function _shutdownGame()
	g_GUI:shutdown()
	g_Game:getTutorialManager():shutdown()
    g_Game:removeHeroes()
    Renderer.shutdown()
    MOAISim:forceGarbageCollection()
end

---------------- END HELPER FUNCTIONS ----------------
-- Game States
-- Init
-- Run
-- Pause
function SetGameState(sState)
	print("GameState Set To "..sState)
	g_sGameState = sState
    if g_sGameState == "Run" then
        -- check tweaks and load the data
        g_Game:updateTweaks()       
        g_Game:updateCustomerService()
    elseif g_sGameState == "Load" then
        _loadGame()
    elseif g_sGameState == "Reload" then
        _StartGame(true)
    end
    
    g_IsInStartupSequence = g_sGameState == "Init"    
end

function GetGameState(sState)
	return g_sGameState
end

local function teardownGame()
    if g_EnableAnalytics then
        DFAnalytics.shutdown()
    end
    
    if g_EnableTweakables then
        DFNetTweaks.shutdown()
    end    
    
    if g_EnableNews then
        DFNetNews.shutdown()
    end
    
    if g_EnableIAP then
        DFIAP.shutdown()
    end
    
    if g_EnableCustomerService then
        DFNetCustomerService.shutdown()
    end
end

local function suspendSession()
    if g_EnableAnalytics then
        DFAnalytics.suspend()
    end
    
    -- do a local, non-cloud save
    g_Game:saveGame(false)
end

local function resumeSession()
    if g_EnableAnalytics then
        DFAnalytics.resume()
    end
    
    -- don't worry, this will check if cloud saves are enabled before doing anything
    g_Game:loadCloudSave()
end

function _StartGame(bRestart)

    -- TODO: if this is a restart, shutdown existing systems before restarting them
    if bRestart then
        print("we should definitely be shutting things down right now!")
        _shutdownGame()
    end

    if not bRestart then
        Renderer.initializeRenderer()
        _initializeAudio()   
    end
    
        
    --input:init()
    
    _initializeGameRules()
    _initializeDialogManager()
    
    g_GUI = Gui
	g_GUI:init()	
    
    if not bRestart then
        g_GUI:ShowSplashScreen()
    end
    
    -- check save data for whether this user should send telemetry
    if g_EnableAnalytics then
        local bTrackInternalTelemetry = false
        local tSaved = g_Game:getSaveTable()
        if tSaved.bTrackInternalTelemetry ~= nil then
            bTrackInternalTelemetry = tSaved.bTrackInternalTelemetry
        end
        DFAnalytics.setInternalTelemetryEnabled(bTrackInternalTelemetry)
    end
        
	SetGameState("Init")	
    
    g_bDebugSkipTutorial = false
    g_bDisableAutoSave = false

             --MOAIFmodEventMgr.muteSoundCategory('music', true)
             --MOAIFmodEventMgr.muteSoundCategory('UI', true)
             --MOAIFmodEventMgr.muteSoundCategory('SFX', true)
             --MOAIFmodEventMgr.muteSoundCategory('Ambience', true)
    

    
    g_Game:setRestartFunction(_StartGame)
    
	SetGameState("Run")

    if bRestart then
        _loadGame()
        g_Game:onSplashScreenClosed()
        g_GUI:SetActivePane(nil)
    end
end


local function setupGame()
    -- tweakables
    -- comes before other systems so that we can configure them based on tweaks.
    if g_EnableTweakables then
        DFNetTweaks.init(g_ServiceURLs.tweaks, "7Gj3q02g38FAsEB7KtTViAjxKDoMh8uU", 900)
    end
    
    if DFNetTweaks.tweaks and DFNetTweaks.tServiceURLs and not g_UseLocalServer then
        for serviceId, url in pairs(DFNetTweaks.tServiceURLs) do
            g_ServiceURLs[serviceId] = url
        end
    end

    if g_EnableNews then
        DFNetNews.init(g_ServiceURLs.news, 1800)
    end
    
    if g_EnableCustomerService then
        DFNetCustomerService.init(g_ServiceURLs.csr, "7Gj3q02g38FAsEB7KtTViAjxKDoMh8uU", 900)
    end
    
    MOAISim.setGcFrameMaxDuration (2000) -- Maximum time (in micro second) Lua can spend for GC during one frame
    MOAISim.setGcConstantDuration (1000) -- Amount of time (in micro second) Lua will spend at the end of the frame for GC
    MOAISim.setListener(MOAISim.EVENT_FINALIZE, teardownGame)
    if MOAIApp then
        MOAIApp.setListener(MOAIApp.SESSION_START, resumeSession)
        MOAIApp.setListener(MOAIApp.SESSION_END, suspendSession)
    end

    if g_TestBuild ~= nil then
    	MOAIEnvironment.appID = MOAIEnvironment.appID .. "_" .. g_TestBuild
        MOAIEnvironment.appDisplayName = MOAIEnvironment.appDisplayName .. "_" .. g_TestBuild
        g_EnableAnalytics = true
    end
    
    -- hey let's get random bros
    math.randomseed(os.time())

    -- wow. wow. Have to call random() a few times because the first time you call it is basically deterministic.
    math.random()
    math.random()
    math.random()
    math.random()

    if g_EnableAnalytics then
        local testAnalytics = g_TestBuild ~= nil
        
        local apsalarBaseURI = g_ServiceURLs.apsalar
        local moaiAnalyticsBaseURI = g_ServiceURLs.analytics

        if testAnalytics then
            DFAnalytics.init(apsalarBaseURI, "nathandf", "26bb8c442c2148e2067bf861a1f38535", moaiAnalyticsBaseURI, "7Gj3q02g38FAsEB7KtTViAjxKDoMh8uU")
        else
            DFAnalytics.init(apsalarBaseURI, "mobile", "e2702d656b7d2a389b95a1d2891e1e2d", moaiAnalyticsBaseURI, "7Gj3q02g38FAsEB7KtTViAjxKDoMh8uU")
        end 
        
        local function telemErrorCallback(result)
            -- check result and save a thing if telem should be disabled
            if result and result.error then
                local tSaved = g_Game:getSaveTable()

                if result.error == "stop" then
                    tSaved.bTrackInternalTelemetry = false
                    g_Game:saveGame()
                elseif result.error == "resume" then
                    tSaved.bTrackInternalTelemetry = true
                    g_Game:saveGame()
                end    
            end
        end
       
        DFAnalytics.setInternalTelemetryErrorCallback(telemErrorCallback)
       
        DFAnalytics.start()
       
        if DFNetTweaks.tweaks then
            if DFNetTweaks.tweaks.kSTAT_SEND_INTERVAL then
                DFAnalytics.setStatSendInterval(DFNetTweaks.tweaks.kSTAT_SEND_INTERVAL)
            end
            
            if DFNetTweaks.tweaks.kANALYTICS_ACTIVE_WAIT_INTERVAL then
                DFAnalytics.setActiveWaitInterval(DFNetTweaks.tweaks.kANALYTICS_ACTIVE_WAIT_INTERVAL)
            end
            
            if DFNetTweaks.tweaks.kANALYTICS_IDLE_WAIT_INTERVAL then
                DFAnalytics.setIdleWaitInterval(DFNetTweaks.tweaks.kANALYTICS_IDLE_WAIT_INTERVAL)
            end

            if DFNetTweaks.tweaks.kANALYTICS_MAX_TRANSMISSIONS_BEFORE_IDLE then
                DFAnalytics.setMaxTransmissionsBeforeIdle(DFNetTweaks.tweaks.kANALYTICS_MAX_TRANSMISSIONS_BEFORE_IDLE)
            end
        end
 
        DFAnalytics.incrStat("prg_sessions", 1)       
       
        print("Analytics initialized!") 
    end
    

    if g_EnableCrittercism then
        _initializeCrittercism()
    end

    _StartGame()

--    g_GUI:HideSplashScreen()
--    g_eventManager:forceCreateEvent(g_Game:getActiveHeroes(), g_City.tTerritories.StabbersRange, 'introevent_3')
--    g_eventManager:forceCreateEvent(g_Game:getActiveHeroes(), g_City.tTerritories.StabbersRange, 'firerescuelvl1_1')
--    g_eventManager:forceCreateEvent(g_Game:getActiveHeroes(), g_City.tTerritories.StabbersRange, 'busrescuelvl1_1')
--      g_eventManager:forceCreateEvent(g_Game:getActiveHeroes(), g_City.tTerritories.StabbersRange, 'robberylvl4_4')

--    local x, y = g_CurrentMap:getWorldCoords(11, 9)
--    local rTerritory = g_City:getTerritory("StabbersRange")
--    local rEvent = rTerritory.tEvents[1]
--    rEvent:addHeroToEvent(g_Game:getActiveHeroes()[1])
--    rEvent:setSuccessful(true)
--    g_GUI:ShowEventEndScreenUI(rEvent)
--    g_Game:toggleDebug()
end

setupGame()