local UIElement = require('ui/UIElement')
local DFGraphics = require('DFCommon.Graphics')
local DFUtil = require('DFCommon.Util')
local Ability = require('Ability/Ability')
local util = require('DFUtil')
local AbilityDescriptionCard = {}

local tUITextureSpriteSheetInfo =
{
indexcard = 'UI/shopheropack',
locked = 'UI/hud',
icon_time = 'UI/hud',
icon_int = 'UI/hud',
}

function AbilityDescriptionCard:new(rRenderLayer, rHero, sAbilityKey, bForUpgrade, nWidth, nHeight, rParent)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType ~= MOAITouchSensor.TOUCH_UP then return false end
        for _,v in ipairs(tProps) do
            if v == self.rBackground then
                if self.bForUpgrade then
                    return self.bCanUpgrade
                else
                    return true
                end
            end
        end
        return false
    end

    function Ob:update()
        if self.bForUpgrade then
            self.rAwarenessIcon:setVisible(false)

            local nReqRoomLevel = self.rAbility:getReqUpgradeRoomLevel()
            local rRoom = g_CurrentMap:getRoom("dojo")
            local nRoomLevel = 1
            if rRoom then
                nRoomLevel = rRoom:getRoomDef().nRoomLevel or nRoomLevel
            end

            self.bCanUpgrade = true

            local sSelectText = nil
            local sSelectDeck = nil
            local sSelectSprite = nil
            if self.bMaxLevel then
                self.bCanUpgrade = false
                sSelectText="Max level"
            elseif nReqRoomLevel > nRoomLevel then
                self.bCanUpgrade = false
                sSelectText = "Need level "..nReqRoomLevel.." room"
            elseif self.rHero:getStatAwareness() >= self.rAbility:getReqUpgradeAwareness() and self.rHero:getStatLevel() >= self.rAbility:getReqUpgradeLevel() then
                sSelectSprite = 'icon_time'
                sSelectDeck = 'UI/hud'
                sSelectText = util.convertTimeString(self.rAbility:getUpgradeTime(), true, true)
                local nCurLevel = self.rAbility:getCurLevel()
                if not self.rAbility:isUnlocked() then
                    sSelectText = "Research level 1 ("..sSelectText..")"
                elseif nCurLevel >= Ability.getMaxLevel() then
                    sSelectText = "Max level"
                else
                    sSelectText = "Research level "..(nCurLevel+1).." ("..sSelectText..")"
                end
                print(sSelectText)
            elseif self.rHero:getStatAwareness() < self.rAbility:getReqUpgradeAwareness() then
                self.bCanUpgrade = false
                sSelectSprite = 'icon_int'
                sSelectDeck = 'UI/hud'
                sSelectText = "Need "..tostring(self.rAbility:getReqUpgradeAwareness())
                self.rAwarenessIcon:setVisible(true)
            elseif self.rHero:getStatLevel() < self.rAbility:getReqUpgradeLevel() then
                self.bCanUpgrade = false
                -- TODO MTF: this appears to be the wrong icon!
                sSelectSprite = 'icon_int'
                sSelectDeck = 'UI/hud'
                sSelectText = "Need level "..tostring(self.rAbility:getReqUpgradeLevel())
            end
            assert(sSelectText)
            self.rAbilityLevelText:setString(sSelectText)
            --self.rSelectText:setString(sSelectText)
            --[[
            if sSelectDeck then
                local rDeck = DFGraphics.loadSpriteSheet(sSelectDeck)
                self.rUpgradeStatusIcon:setDeck(rDeck)
                self.rUpgradeStatusIcon:setIndex(rDeck.names[sSelectSprite])
                self.rUpgradeStatusIcon:setVisible(true)
            else
                self.rUpgradeStatusIcon:setVisible(false)
            end
            ]]--
            if self.bCanUpgrade then
                self:setStatus(1)
            else
                self:setStatus(3)
            end
        else
            if not self.rAbility:isUnlocked() then
                self:setStatus(3)
            elseif self.rHero:getActiveAbility() == self.rHero:getAbility(self.sAbilityKey) or self.rHero:isMM() then
                self:setStatus(1)
            else
                self:setStatus(2)
            end
            --[[
            if not self.rAbility:isUnlocked() then
                self.rSelectText:setString("NOT RESEARCHED")
                self.rBackground:setColor(.5, .5, .5)
            elseif self.rHero:isMM() then
                self.rSelectText:setString("")
                self.rBackground:setColor(1, 1, 1)
            elseif self.rHero:getActiveAbility() == self.rHero:getAbility(self.sAbilityKey) then
                self.rBackground:setColor(1, 1, 1)
                self.rSelectText:setString("SELECTED")
            else
                self.rBackground:setColor(.8, .8, .8)
                self.rSelectText:setString("SET ACTIVE")
            end
            ]]--
        end
    end

    function Ob:setStatus(nStatus)
        if nStatus == 1 then -- active / white
            self.rBackground:setColor(1,1,1)
            self.rAbilityLevelText:setColor(0,0,0,1)
            self.rAbilityDescriptionText:setColor(0,0,0,1)
            self.rAbilityNameText:setColor(0,0,0,1) 
        elseif nStatus == 2 then -- available / normal
            self.rBackground:setColor(0,0,0,0)
            self.rAbilityLevelText:setColor(0,0,0,1)
            self.rAbilityDescriptionText:setColor(0,0,0,1)
            self.rAbilityNameText:setColor(0,0,0,1) 
        elseif nStatus == 3 then -- disabled
            local nS = .5
            self.rBackground:setColor(nS*217/255,nS*244/255,nS*241/255, .4)
            self.rAbilityLevelText:setColor(0,0,0,.6)
            self.rAbilityDescriptionText:setColor(0,0,0,.6)
            self.rAbilityNameText:setColor(0,0,0,.6) 
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.bForUpgrade = bForUpgrade
    Ob.rHero = rHero
    Ob.sAbilityKey = sAbilityKey
    local rAbility = rHero:getAbility(sAbilityKey)
    if rAbility == rHero:getActiveAbility() then
        Ob.bActive = true
    end
    Ob.rAbility = rAbility

    Ob.nHMargin = 10
    Ob.nVMargin = 5
    Ob.nIconWidth = 84
    Ob.nIconHeight = 84

    --[[
    Ob.rBackground = Ob:getUITextureProp('indexcard')
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(.82, .5)
    Ob:addElement(Ob.rBackground)
    ]]--

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(nWidth, nHeight)
    Ob.rBackground:setColor(217/255,244/255,241/255)
    Ob:addElement(Ob.rBackground)

    local nCurY = -Ob.nVMargin
    local nTextLineHeight = 20

    Ob.rAbilityNameText = g_GUI:CreateTextBox(rParent.nNameFontSize, MOAITextBox.LEFT_JUSTIFY, rParent.sNameFont)
    Ob.rAbilityNameText:setRect(0, 40, 150, 3)
    Ob.rAbilityNameText:setString(rAbility:getName())
    Ob.rAbilityNameText:setLoc(Ob.nHMargin, nCurY)
    Ob:addElement(Ob.rAbilityNameText)

    nCurY = nCurY - nTextLineHeight
    --local nIconY = nCurY
    --local nIconY = -nHeight * .5
    local nIconY = nCurY - 50

    local sIconName = rAbility:getDef().sIconName
    local sIconSheet = rAbility:getDef().sIconSheet or 'UI/eventdetails'
    tUITextureSpriteSheetInfo[sIconName] = sIconSheet
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rIcon = Ob:getUITextureProp(sIconName)
    local w,h = Ob.rIcon:getDims()
    nCurY = nIconY + Ob.nIconHeight*.5
    nIconY = nIconY + Ob.nIconHeight*.5
    Ob.rIcon:setScl(Ob.nIconWidth/w,Ob.nIconHeight/h)
    Ob.rIcon:setLoc(Ob.nHMargin,nIconY)
    Ob:addElement(Ob.rIcon)

    local nCurX = Ob.nHMargin + Ob.nIconWidth + 5

    local nCurLevel = rAbility:getCurLevel()
    Ob.rAbilityLevelText = g_GUI:CreateTextBox(rParent.nBoldFontSize, MOAITextBox.LEFT_JUSTIFY, rParent.sBoldFont)
    Ob.rAbilityLevelText:setRect(0, 40, 315, 0)
    if bForUpgrade then
        if nCurLevel >= Ability.getMaxLevel() then
            Ob.bMaxLevel = true
        end
        --[[
        -- MTF TODO: These should be linecodes.
        if not rAbility:isUnlocked() then
            Ob.rAbilityLevelText:setString("Research level 1")
        elseif nCurLevel >= Ability.getMaxLevel() then
            Ob.bMaxLevel = true
            Ob.rAbilityLevelText:setString("Max level")
        else
            Ob.rAbilityLevelText:setString("Research level "..(nCurLevel+1))
        end
        ]]--
    else
        if not rAbility:isUnlocked() then
            Ob.rAbilityLevelText:setString("Not researched")
        else
            Ob.rAbilityLevelText:setString("Level "..nCurLevel)
        end
    end
    Ob.rAbilityLevelText:setLoc(nCurX, nCurY)
    Ob:addElement(Ob.rAbilityLevelText)
    nCurY = nCurY - nTextLineHeight

    Ob.rAbilityDescriptionText = g_GUI:CreateTextBox(rParent.nFontSize, MOAITextBox.LEFT_JUSTIFY, rParent.sFont)
    Ob.rAbilityDescriptionText:setRect(0, 240, 315, 0)
    Ob.rAbilityDescriptionText:setString(rAbility:getDescription())
    Ob.rAbilityDescriptionText:setLoc(nCurX, nCurY)
    Ob:addElement(Ob.rAbilityDescriptionText)

    Ob.rAwarenessIcon = Ob:getUITextureProp('icon_int')
    Ob.rAwarenessIcon:setLoc(146, -22)
    Ob.rAwarenessIcon:setScl(0.65)
    Ob.rAwarenessIcon:setVisible(false)
    Ob:addElement(Ob.rAwarenessIcon)

    --[[
    Ob.rSelectText = g_GUI:CreateTextBox(24, MOAITextBox.RIGHT_JUSTIFY, 'type')
    Ob.rSelectText:setRect(0, 80, 180, 0)
    Ob.rSelectText:setString("")
    Ob.rSelectText:setLoc(200, -10)
    Ob:addElement(Ob.rSelectText)

    Ob.rUpgradeStatusIcon = MOAIProp2D.new()
    Ob.rUpgradeStatusIcon:setLoc(290, -40)
    Ob.rUpgradeStatusIcon:setScl(1)
    Ob.rUpgradeStatusIcon:setVisible(false)
    Ob:addElement(Ob.rUpgradeStatusIcon)
    ]]--

    Ob:update()

    return Ob
end

return AbilityDescriptionCard

