-------- HERO SCHEDULER ---------
local AbilitySelectionUI = {}
local DFGraphics = require('DFCommon.Graphics')
local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local AbilityDescriptionCard = require('ui/AbilityDescriptionCard')

local tUITextureSpriteSheetInfo =
{
}

local tAvailableTextColor = { .56, 0.24, 0.24 }

function AbilitySelectionUI:new(rRenderLayer, rParentUI, rHero, bForUpgrade)	
	local Ob = UIElement:new(rRenderLayer)

    function Ob:isVisible()
        return self.bVisible
    end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        for _,v in ipairs(self.tCards) do
            v:setVisible(bVisible)
        end
        g_Audio:playCue('MMOJ_Audio/UI/ClipBoard')
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        local handled = false
        if eventType == MOAITouchSensor.TOUCH_UP then
            for k,v in ipairs(self.tCards) do
                if v:onFinger(eventType, sx, sy, tProps) then
                    if self.bForUpgrade then
                        if self.rHero:isMM() then
                            if self.rHero:getStatAwareness() >= v.rAbility:getReqUpgradeAwareness() and not v.rAbility:isAtMaxLevel() 
                                and self.rHero:getStatLevel() >= v.rAbility:getReqUpgradeLevel() then
                                g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                                self.rHero:addAbilityTrainingActivityAtIndex(1, v.sAbilityKey)
                                g_GUI:HideHeroScheduler()
                            end
                        else
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            self.rHero:addAbilityTrainingActivityAtIndex(1, v.sAbilityKey)
                            g_GUI:HideHeroScheduler()
                        end
                    else
                        if v.rAbility:isUnlocked() then
                            self.rHero:setActiveAbility(v.sAbilityKey)
                            for _,rToUpdate in ipairs(self.tCards) do
                                rToUpdate:update()
                            end
                        end
                    end
                    handled = true
                end
            end
            if not handled then
                for _,v in ipairs(tProps) do
                    if v == self.rBackground then
                        handled = true
                    end
                end
            end
            if self.rParentUI.abilitySelectionCallback then
                self.rParentUI:abilitySelectionCallback(self, handled)
            end
        end
	    return handled
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        for _,v in ipairs(self.tCards) do
            v:onPaneClosed()
        end
    end

    function Ob:addHLine(nY)
        if not self.tHLines then
            self.tHLines = {}
        end
        local rLine = MOAIProp2D.new()
        rLine:setDeck(g_GUI:GetOnePixelDeck())
        rLine:setLoc(self.nHMargin,nY)
        rLine:setScl(self.nColWidth, self.nLineH)
        rLine:setColor(0,0,0)
        Ob:addElement(rLine)
        table.insert(self.tHLines, rLine)
        return nY-self.nLineH
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    --Ob.nWidth = 500
    -- temp: shrinking for now, since we're putting it on the right hand side instead of the left, and 
    -- need to leave room for the clipboard.
    Ob.nWidth = 480
    Ob.nColWidth = 424
    Ob.nCardHeight = 154
    Ob.nCardMargin = 1
    Ob.nHMargin = .5*(Ob.nWidth - Ob.nColWidth)
    Ob.nExtraHMarginOrSomething = 4
    Ob.nVMargin = 20
    Ob.nLineH = 8
    Ob.nTextLineHeight = 19

	-- vars
	Ob.rHero = rHero
    Ob.rParentUI = rParentUI
    Ob.bVisible = false
    Ob.bForUpgrade = bForUpgrade

    Ob.sFont = "type"
    Ob.nFontSize = 16
    Ob.sBoldFont = "ptsans"
    Ob.nBoldFontSize = 16
    Ob.sNameFont = "boogaloo"
    Ob.nNameFontSize = 20

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    local nHeight = (Ob.rHero:isMM() and 640) or 600
    Ob.rBackground:setScl(500, nHeight)
    Ob.rBackground:setColor(217/255,244/255,241/255)
    Ob:addElement(Ob.rBackground)

    Ob.tCards = {}
    local tAbilities = rHero:getAbilities()
    --local i=1
    local nCurY = -Ob.nVMargin
    nCurY = Ob:addHLine(nCurY)

    --[[
    Ob.rTitle = g_GUI:CreateTextBox(Ob.nBoldFontSize, MOAITextBox.LEFT_JUSTIFY, Ob.sBoldFont)
    Ob.rTitle:setRect(0, 60, 300, 0)
    Ob.rTitle:setLoc(Ob.nHMargin+Ob.nExtraHMarginOrSomething, nCurY)
    Ob.rTitle:setString("ABILITY")
    Ob:addElement(Ob.rTitle)
    nCurY = nCurY - Ob.nTextLineHeight

    nCurY = Ob:addHLine(nCurY)
    ]]--

    Ob.rHeading = g_GUI:CreateTextBox(Ob.nBoldFontSize, MOAITextBox.LEFT_JUSTIFY, Ob.sBoldFont)
    Ob.rHeading:setRect(0, 30, 300, 3)
    Ob.rHeading:setLoc(Ob.nHMargin+Ob.nExtraHMarginOrSomething, nCurY)
    Ob.sHeading = ""
    if Ob.bForUpgrade then
        --i = 1.5
        Ob.sHeading = "CHOOSE ABILITY TO RESEARCH"
    elseif Ob.rHero:isMM() then
        Ob.sHeading = "ABILITY"
    else
        Ob.sHeading = "SELECT ACTIVE ABILITY"
    end
    Ob.rHeading:setString(Ob.sHeading)
    Ob:addElement(Ob.rHeading)
    if Ob.sHeading ~= "" then
        nCurY = nCurY - Ob.nTextLineHeight
        nCurY = Ob:addHLine(nCurY)
    end
    nCurY = nCurY - Ob.nCardMargin

    local bMM = rHero:isMM()
    if bMM then
        Ob.nCardHeight = 130
    end
    for k,v in pairs(tAbilities) do
        if not v:isDefault() then
            local card = AbilityDescriptionCard:new(rRenderLayer, rHero, k, bForUpgrade, Ob.nColWidth, Ob.nCardHeight, Ob)
            card:setLoc(Ob.nHMargin,nCurY)
            card:setParent(Ob)
            table.insert(Ob.tCards, card)
            nCurY = nCurY - Ob.nCardHeight
            nCurY = nCurY - Ob.nCardMargin
            nCurY = Ob:addHLine(nCurY)
            nCurY = nCurY - Ob.nCardMargin
        end
    end

    --[[
    if i == 4 then
        Ob.rBackground:setScl(412, 440)
    else
        Ob.rBackground:setScl(412, 600)
    end
    ]]--

	return Ob
end

return AbilitySelectionUI
