local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')
local TrainingSelectionUI = require('ui/TrainingSelectionUI')
local DFGraphics = require('DFCommon.Graphics')

local tUITextureSpriteSheetInfo =
{
ability_button = 'UI/shopheropack',
}

local tAvailableTextColor = { .56, 0.24, 0.24 }

local AbilitySmallButton = {}

function AbilitySmallButton:new(rRenderLayer, rParentUI, rHero, rAbilityOverride)
	local Ob = UIElement:new(rRenderLayer)

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    function Ob:onFinger(eventType, sx,sy,tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rIcon or rProp == self.rAbilityButton then
                    return true
                end
            end
        end
        return false
    end

    function Ob:update()
        local sIconName = nil
        local sIconSheet = nil
        local rAbility = self.rAbilityOverride or self.rHero:getActiveAbility()
        local nCooldownPct = nil
        if rAbility then
            if rAbility:getRealTimeCooldown() then
                if not rAbility:isUnlocked() then
                    nCooldownPct = 1
                else
                    nCooldownPct = 1-rAbility:getTurnCooldownPct()
                end
            end

            sIconName = rAbility:getDef().sIconName
            sIconSheet = rAbility:getDef().sIconSheet or 'UI/eventdetails'
            self.rAbilityNameText:setString(rAbility:getName())
        end
        if sIconName then
            tUITextureSpriteSheetInfo[sIconName] = sIconSheet
            self:loadUISpriteSheets(tUITextureSpriteSheetInfo)
            self.rIcon = self:getUITextureProp(sIconName, self.rIcon)
            if self.bVisible then
                self.rIcon:setVisible(true)
            end
        else
            self.rIcon:setVisible(false)
        end
        if nCooldownPct and nCooldownPct > 0 then
            --self.rCooldownOverlay:setScl(self.nW, self.nH * nCooldownPct)
            self.rCooldownOverlay:setScl(self.nW*nCooldownPct, self.nH)
            self.rCooldownOverlay:setVisible(true)
        else
            self.rCooldownOverlay:setVisible(false)
        end
    end

    function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self.rIcon:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    Ob.rHero = rHero
    Ob.rAbilityOverride = rAbilityOverride 

    Ob.rAbilityButton = Ob:getUITextureProp('ability_button')
    Ob:addElement(Ob.rAbilityButton)

    Ob.rIcon = MOAIProp2D.new()
    Ob.rIcon:setVisible(false)
    Ob.rIcon:setLoc(8,-10)
    Ob.rIcon:setScl(.3,.3)
    Ob:addElement(Ob.rIcon)

    Ob.rAbilityNameText = g_GUI:CreateTextBox(14, MOAITextBox.LEFT_JUSTIFY)
    Ob.rAbilityNameText:setRect(0, 40, 150, 0)
    Ob.rAbilityNameText:setString("")
    Ob.rAbilityNameText:setLoc(55, -18)
    Ob:addElement(Ob.rAbilityNameText)

    Ob.rCooldownOverlay = MOAIProp2D.new()
    Ob.rCooldownOverlay:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rCooldownOverlay:setColor(0.15, 0.15, 0.15, 0.5)
    Ob.rCooldownOverlay:setLoc(0, 0)
    Ob.nW, Ob.nH = Ob.rAbilityButton:getDims()
    Ob.rCooldownOverlay:setScl(Ob.nW, Ob.nH)
    Ob:addElement(Ob.rCooldownOverlay)
    
    Ob:update()

	return Ob
end

return AbilitySmallButton

