local UIElement = require('ui/UIElement')
local AbilityIcon = require('ui/AbilityIcon')
local DFUtil = require('DFCommon.Util')
local Ability = require('Ability/Ability')
local util = require('DFUtil')
local AbilityDetailsPane = {}
local AbilityUpgradeUI = {}

local tUITextureSpriteSheetInfo =
{
indexcard = 'UI/shopheropack',
locked = 'UI/hud',
icon_time = 'UI/hud',
icon_int = 'UI/hud',
}

function AbilityDetailsPane:new(rRenderLayer, rHero, sAbilityKey)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rIcon:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rIcon:onPaneClosed()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        return self.rIcon:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:handleIconPress()
        if not self.bCanUpgrade then
            return
        end
        if self.rHero:getStatAwareness() >= self.rAbility:getReqUpgradeAwareness() and not self.rAbility:isAtMaxLevel() 
            and self.rHero:getStatLevel() >= self.rAbility:getReqUpgradeLevel() then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
            self.rHero:addAbilityTrainingActivityAtIndex(1, self.sAbilityKey)
            g_GUI:HideHeroScheduler()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero
    Ob.sAbilityKey = sAbilityKey
    local rAbility = rHero:getAbility(sAbilityKey)
    Ob.rAbility = rAbility

    Ob.rIcon = AbilityIcon:new(rRenderLayer, Ob.rAbility, Ob)
    Ob.rIcon:setScl(1)
    Ob.rIcon:setParent(Ob)
    Ob.rIcon:setLoc(30, 0)
    local nCurLevel = rAbility:getCurLevel()

    Ob.rAbilityLevelText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY)
    Ob.rAbilityLevelText:setRect(-150, 80, 150, 0)
    if nCurLevel >= Ability.getMaxLevel() then
        Ob.rAbilityLevelText:setString("Max Level")
    else
        nCurLevel = nCurLevel + 1
        Ob.rAbilityLevelText:setString("Level "..nCurLevel)
    end
    Ob.rAbilityLevelText:setLoc(74, -96)
    Ob:addElement(Ob.rAbilityLevelText)

    if not rAbility:isUnlocked() then
        Ob.rAbilityLevelText:setString("Level 1")
    end

    Ob.rAbilityNameText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY)
    Ob.rAbilityNameText:setRect(-150, 80, 150, 0)
    Ob.rAbilityNameText:setString(rAbility:getName())
    Ob.rAbilityNameText:setLoc(74, -114)
    Ob:addElement(Ob.rAbilityNameText)

    Ob.rAbilityDescriptionText = g_GUI:CreateTextBox(14, MOAITextBox.LEFT_JUSTIFY)
    Ob.rAbilityDescriptionText:setRect(0, 240, 180, 0)
    Ob.rAbilityDescriptionText:setString(rAbility:getDescription())
    Ob.rAbilityDescriptionText:setLoc(4, -136)
    Ob:addElement(Ob.rAbilityDescriptionText)

    local rRoom = g_CurrentMap:getRoom("dojo")
    local nRoomLevel = 1
    if rRoom then
        nRoomLevel = rRoom:getRoomDef().nRoomLevel or nRoomLevel
    end

    local nReqRoomLevel = Ob.rAbility:getReqUpgradeRoomLevel()

    Ob.bCanUpgrade = true

    if nReqRoomLevel > nRoomLevel then
        Ob.bCanUpgrade = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT102TEXT").." "..nReqRoomLevel.." "..g_DialogManager:getString("STXT140TEXT"))
        Ob.rCantResearchText:setLoc(-14, -226)
        Ob:addElement(Ob.rCantResearchText)   
    elseif rHero:getStatAwareness() >= rAbility:getReqUpgradeAwareness() and rHero:getStatLevel() >= rAbility:getReqUpgradeLevel() then
        -- show the time
        Ob.rTimeIcon = Ob:getUITextureProp('icon_time')
        Ob.rTimeIcon:setLoc(30, -230)
        Ob.rTimeIcon:setScl(0.6)
        Ob:addElement(Ob.rTimeIcon)

        Ob.rTimeText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rTimeText:setRect(0, 80, 150, 0)
        Ob.rTimeText:setString(util.convertTimeString(rAbility:getUpgradeTime()))
        Ob.rTimeText:setLoc(70, -228)
        Ob:addElement(Ob.rTimeText)
    elseif rHero:getStatAwareness() < rAbility:getReqUpgradeAwareness() then
        Ob.bCanUpgrade = false
        Ob.rAwarenessIcon = Ob:getUITextureProp('icon_int')
        Ob.rAwarenessIcon:setLoc(30, -224)
        Ob.rAwarenessIcon:setScl(0.6)
        Ob:addElement(Ob.rAwarenessIcon)

        Ob.rAwarenessText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rAwarenessText:setRect(0, 80, 150, 0)
        Ob.rAwarenessText:setString(tostring(rAbility:getReqUpgradeAwareness()))
        Ob.rAwarenessText:setLoc(70, -228)
        Ob.rAwarenessText:setColor(1, 0, 0)
        Ob:addElement(Ob.rAwarenessText)
    elseif rHero:getStatLevel() < rAbility:getReqUpgradeLevel() then
        Ob.bCanUpgrade = false
        Ob.rLevelIcon = Ob:getUITextureProp('icon_int')
        Ob.rLevelIcon:setLoc(30, -224)
        Ob.rLevelIcon:setScl(0.6)
        Ob:addElement(Ob.rLevelIcon)

        Ob.rLevelText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rLevelText:setRect(0, 80, 150, 0)
        Ob.rLevelText:setString(tostring(rAbility:getReqUpgradeLevel()))
        Ob.rLevelText:setLoc(70, -228)
        Ob.rLevelText:setColor(1, 0, 0)
        Ob:addElement(Ob.rLevelText)
    end

    return Ob
end

function AbilityUpgradeUI:new(rRenderLayer, rHero, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        for i, rPane in ipairs(self.tAbilityAbilityPanes) do
            rPane:setVisible(bVisible)
        end
    end

    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType ~= MOAITouchSensor.TOUCH_UP then return false end

        local handled = false
        
        for i, rPane in ipairs(self.tAbilityAbilityPanes) do
            handled = rPane:onFinger(eventType, sx, sy, tProps)
        end
        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rBackground then
                    handled = true
                    break
                end
            end
        end
        
        -- not touched anything, close
        if not handled then
            self.rParentUI:setTrainAbilityUIVisible(false)
        end
        
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rHero = rHero
    Ob.rParentUI = rParentUI

    Ob.rBackground = Ob:getUITextureProp('indexcard')
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(1.4, 1.4)
    Ob:addElement(Ob.rBackground)

    Ob.rAbilityLabelText = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY)
    Ob.rAbilityLabelText:setRect(-300, 80, 300, 0)
    Ob.rAbilityLabelText:setString(g_DialogManager:getString("STXT046TEXT"))
    Ob.rAbilityLabelText:setLoc(350, -20)
    Ob:addElement(Ob.rAbilityLabelText)

    local tAbilityKeys = {}
    local tHeroAbilities = DFUtil.deepCopy(rHero:getAbilities())
    for sKey, rAbility in pairs(tHeroAbilities) do
        if rAbility:showInUI() then
            table.insert(tAbilityKeys, sKey)
        end
    end

    local kX_OFFSET = 200
    local nNumItems = 0
    local nInitX = 50
    Ob.tAbilityAbilityPanes = {}
    for i, sAbilityKey in ipairs(tAbilityKeys) do
        local rAbilityPane = AbilityDetailsPane:new(rRenderLayer, rHero, sAbilityKey)
        local nNewX = nInitX + kX_OFFSET * nNumItems
        rAbilityPane:setLoc(nInitX + kX_OFFSET * nNumItems, -74)
        rAbilityPane:setParent(Ob)
        table.insert(Ob.tAbilityAbilityPanes, rAbilityPane)
        nNumItems = nNumItems + 1
    end

    return Ob
end

return AbilityUpgradeUI
