-------- HERO SCHEDULER ---------
local ActivitySelectionUI = {}
local ActivitySelectionButton = {}
local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
--local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')
local TrainingSelectionUI = require('ui/TrainingSelectionUI')

local tUITextureSpriteSheetInfo =
{
activity_sheet_rest = 'UI/hud',
activity_sheet_timeoff = 'UI/hud',
activity_sheet_research = 'UI/hud',
activity_sheet_train = 'UI/hud',
activity_sheet_train_maxhp = 'UI/hud',
activity_sheet_train_ability = 'UI/hud',
activity_sheet_parttimejob = 'UI/hud',
}

local tAvailableTextColor = { .56, 0.24, 0.24 }

function ActivitySelectionUI:new(rRenderLayer, rParentUI, rHero)	
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if not bVisible then
            self.rAbilityUpgradeUI:setVisible(bVisible)
            self.rTrainingSelectionUI:setVisible(bVisible)
        end        
    end

    function Ob:isVisible()
        return self.bVisible
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if self.rAbilityUpgradeUI:isVisible() then
                if self.rAbilityUpgradeUI:onFinger(eventType, sx, sy, tProps) then
                else
                    self.rAbilityUpgradeUI:setVisible(false)
                end
                return true
            end
        end
        if self.rTrainingSelectionUI:isVisible() then
            self.rTrainingSelectionUI:onFinger(eventType, sx, sy, tProps)
            return true
        end
        
        
        -- unhighlight all buttons on any touch event (we'll rehighlight them if the user has pushed down)
        if g_CurrentMap:getRoom('bedroom') then
            self.rRestButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        if g_CurrentMap:getRoom('recroom') then
            self.rRecreationButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        if g_Game:isActionEnabled('HeroTrain') then
            self.rTrainButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        if g_CurrentMap:getRoom('dojo') then
            self.rTrainAbilityButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        if g_CurrentMap:getRoom('lab') then
            self.rResearchButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        if g_CurrentMap:getRoom('cube') then
            self.rPTJobButton:setColor(1.0, 1.0, 1.0, 1.0)
        end
        
        local bHandled = false
        if eventType == MOAITouchSensor.TOUCH_DOWN or eventType == MOAITouchSensor.TOUCH_MOVE then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rRestInvisButton and g_CurrentMap:getRoom('bedroom') then
                        self.rRestButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rRecreationInvisButton and g_CurrentMap:getRoom('recroom') then
                        self.rRecreationButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rTrainInvisButton and g_Game:isActionEnabled('HeroTrain') then
                        self.rTrainButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rTrainAbilityInvisButton and g_CurrentMap:getRoom('dojo') then
                        self.rTrainAbilityButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rResearchInvisButton and g_CurrentMap:getRoom('lab') then
                        self.rResearchButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rPTJobInvisButton and g_CurrentMap:getRoom('cube') then
                        self.rPTJobButton:setColor(0.5, 0.5, 0.5, 1.0)
                    end
                end
            end
        elseif eventType == MOAITouchSensor.TOUCH_UP then   
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rRestInvisButton then
                        if g_CurrentMap:getRoom('bedroom') then
                            self.rHero:setCurActivity('Rest')
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            self.rParentUI:hideActivitySelector()
                            g_GUI:HideHeroScheduler()
                        end
                        return true
                    elseif rProp == self.rRecreationInvisButton then
                        if g_CurrentMap:getRoom('recroom') then
                            self.rHero:setCurActivity('Recreation')
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            self.rParentUI:hideActivitySelector()
                            g_GUI:HideHeroScheduler()
                        end
                        return true
                    elseif rProp == self.rTrainInvisButton then
                        if g_Game:isActionEnabled('HeroTrain') then
                            if g_CurrentMap:getRoom('gym') and self.rHero:getNumTrainingSessions() > 0  then
                                g_Audio:playCue('MMOJ_Audio/UI/FolderTab')
                                self:setTrainingSelectionUIVisible(true)
                            end
                        end
                        return true
                    elseif rProp == self.rTrainAbilityInvisButton then
                        if g_CurrentMap:getRoom('dojo') then
                            g_Audio:playCue('MMOJ_Audio/UI/FolderTab')
                            self:setTrainAbilityUIVisible(true)
                        end
                        return true
                    elseif rProp == self.rResearchInvisButton then
                        if g_CurrentMap:getRoom('lab') then
                            g_GUI:ShowResearchUI(self.rHero, self.nActivityIndex)
                            self.rParentUI:hideActivitySelector()
                        end
                        return true
                    elseif rProp == self.rPTJobInvisButton then
                        if g_CurrentMap:getRoom('cube') then
                            self.rHero:setCurActivity('PTJob')
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            self.rParentUI:hideActivitySelector()
                            g_GUI:HideHeroScheduler()
                        end
                        return true
                    elseif rProp == self.rBackground then
                        bHandled = true
                    end
                end
                --[[
                if g_Game:isActionEnabled('CloseWindow') then
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                    self.rParentUI:hideActivitySelector()
                end
                ]]--
            end
        end
		return bHandled
	end

    function Ob:canTrainPowerWithRoom()
        local rRoom = g_CurrentMap:getRoom("gym")
        if rRoom then            
            if rRoom:getRoomDef().nMaxPower > self.rHero:getStatPower() then
                return true
            end
        end
        return false
    end

    function Ob:canTrainHPWithRoom()
        local rRoom = g_CurrentMap:getRoom("gym")
        if rRoom then
            if rRoom:getRoomDef().nMaxHP > self.rHero:getMaxHP() then
                return true
            end
        end
        return false        
    end

    function Ob:setTrainAbilityUIVisible(bVisible)
        self.rAbilityUpgradeUI:setVisible(bVisible)
    end

    function Ob:setTrainingSelectionUIVisible(bVisible)
        self.rTrainingSelectionUI:setVisible(bVisible)
    end

    function Ob:setActivityIndex(nIndex)
        self.nActivityIndex = nIndex
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rAbilityUpgradeUI:onPaneClosed()
        self.rTrainingSelectionUI:onPaneClosed()
    end

    function Ob:hideActivitySelector()
        self.rParentUI:hideActivitySelector()
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	-- vars
	Ob.rHero = rHero
    Ob.rParentUI = rParentUI
	Ob.nActivityIndex = nil    
    Ob.bVisible = false

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(412, 550)
    Ob.rBackground:setColor(1, 0.78, 0.76)
    Ob:addElement(Ob.rBackground)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(24, -45)
    Ob.rLine1:setScl(365, 5)
    Ob.rLine1:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(24, -129)
    Ob.rLine2:setScl(365, 2)
    Ob.rLine2:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(24, -209)
    Ob.rLine3:setScl(365, 2)
    Ob.rLine3:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine3)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(24, -289)
    Ob.rLine4:setScl(365, 2)
    Ob.rLine4:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine4)

    Ob.rLine5 = MOAIProp2D.new()
    Ob.rLine5:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine5:setLoc(24, -369)
    Ob.rLine5:setScl(365, 2)
    Ob.rLine5:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine5)

    Ob.rLine6 = MOAIProp2D.new()
    Ob.rLine6:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine6:setLoc(24, -449)
    Ob.rLine6:setScl(365, 2)
    Ob.rLine6:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine6)

    Ob.LabelText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.LabelText:setString(g_DialogManager:getString("STXT022TEXT"))
    Ob.LabelText:setRect(0, 80, 380, 0)
    Ob.LabelText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.LabelText:setLoc(20, -14)
    Ob:addElement(Ob.LabelText)

    Ob.rRestInvisButton = MOAIProp2D.new()
    Ob.rRestInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRestInvisButton:setLoc(24, -53)
    Ob.rRestInvisButton:setScl(362, 70)
    Ob.rRestInvisButton:setColor(0, 0.5, 0.5)
    Ob.rRestInvisButton:setVisible(false)
    Ob:addElement(Ob.rRestInvisButton)

    Ob.rRestButton = Ob:getUITextureProp('activity_sheet_rest')
    Ob.rRestButton:setLoc(24, -53)
    Ob:addElement(Ob.rRestButton)
    
    Ob.RestText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.RestText:setString(string.upper(g_DialogManager:getString("ACNA005TEXT")))
    Ob.RestText:setRect(0, 40, 300, 0)
    Ob.RestText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.RestText:setLoc(160, -50)
    Ob:addElement(Ob.RestText)

    if g_CurrentMap:getRoom('bedroom') == nil then
        Ob.rRestButton:setColor(1, 1, 1, 0.5)

        Ob.NoBedroomText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoBedroomText:setString(g_DialogManager:getString("ACRR004TEXT"))
        Ob.NoBedroomText:setRect(0, 40, 300, 0)
        Ob.NoBedroomText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoBedroomText:setLoc(160, -84)
        Ob:addElement(Ob.NoBedroomText)
    else
        Ob.RestDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.RestDescriptionText:setString(g_DialogManager:getString("ACDS005TEXT"))
        Ob.RestDescriptionText:setRect(0, 40, 300, 0)
        Ob.RestDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.RestDescriptionText:setLoc(160, -84)
        Ob:addElement(Ob.RestDescriptionText)
    end

    Ob.rTrainInvisButton = MOAIProp2D.new()
    Ob.rTrainInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTrainInvisButton:setLoc(24, -134)
    Ob.rTrainInvisButton:setScl(362, 70)
    Ob.rTrainInvisButton:setColor(0, 0.5, 0.5)
    Ob.rTrainInvisButton:setVisible(false)
    Ob:addElement(Ob.rTrainInvisButton)

    Ob.rTrainButton = Ob:getUITextureProp('activity_sheet_train')
    Ob.rTrainButton:setLoc(24, -134)
    Ob:addElement(Ob.rTrainButton)

    Ob.TrainText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.TrainText:setString(string.upper(string.upper(g_DialogManager:getString("ACNA014TEXT"))))
    Ob.TrainText:setRect(0, 40, 300, 0)
    Ob.TrainText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.TrainText:setLoc(160, -130)
    Ob:addElement(Ob.TrainText)

    if g_CurrentMap:getRoom('gym') == nil then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.NoGymText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoGymText:setString(g_DialogManager:getString("ACRR005TEXT"))
        Ob.NoGymText:setRect(0, 40, 300, 0)
        Ob.NoGymText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoGymText:setLoc(160, -164)
        Ob.TrainText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoGymText)
    elseif rHero:getNumTrainingSessions() == 0 then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.NoTrainSessionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoTrainSessionText:setString(g_DialogManager:getString("ACDS011TEXT"))
        Ob.NoTrainSessionText:setRect(0, -40, 240, 40)
        Ob.NoTrainSessionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoTrainSessionText:setLoc(160, -199)
        Ob.TrainText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoTrainSessionText)
    else
        Ob.TrainDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainDescriptionText:setString(rHero:getNumTrainingSessions().." "..g_DialogManager:getString("ACNA013TEXT"))
        Ob.TrainDescriptionText:setRect(0, 40, 300, 0)
        Ob.TrainDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.TrainDescriptionText:setLoc(160, -164)
        Ob:addElement(Ob.TrainDescriptionText)
    end
    
    Ob.rTrainAbilityInvisButton = MOAIProp2D.new()
    Ob.rTrainAbilityInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTrainAbilityInvisButton:setLoc(24, -214)
    Ob.rTrainAbilityInvisButton:setScl(362, 70)
    Ob.rTrainAbilityInvisButton:setColor(0, 0.5, 0.5)
    Ob.rTrainAbilityInvisButton:setVisible(false)
    Ob:addElement(Ob.rTrainAbilityInvisButton)

    Ob.rTrainAbilityButton = Ob:getUITextureProp('activity_sheet_train_ability')
    Ob.rTrainAbilityButton:setLoc(24, -214)

    Ob.TrainAbilityText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.TrainAbilityText:setString(string.upper(g_DialogManager:getString("ACNA009TEXT")))
    Ob.TrainAbilityText:setRect(0, 40, 300, 0)
    Ob.TrainAbilityText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.TrainAbilityText:setLoc(160, -210)
    Ob:addElement(Ob.TrainAbilityText)

    Ob:addElement(Ob.rTrainAbilityButton)
    if g_CurrentMap:getRoom('dojo') == nil then
        Ob.rTrainAbilityButton:setColor(1, 1, 1, 0.5)

        Ob.NoGymAbilityText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoGymAbilityText:setString(g_DialogManager:getString("ACRR006TEXT"))
        Ob.NoGymAbilityText:setRect(0, 40, 300, 0)
        Ob.NoGymAbilityText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoGymAbilityText:setLoc(160, -244)
        Ob.TrainAbilityText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoGymAbilityText)
    else
        Ob.TrainAbilityDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainAbilityDescriptionText:setString(g_DialogManager:getString("ACDS009TEXT"))
        Ob.TrainAbilityDescriptionText:setRect(0, 40, 300, 0)
        Ob.TrainAbilityDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.TrainAbilityDescriptionText:setLoc(160, -244)
        Ob.TrainAbilityText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob:addElement(Ob.TrainAbilityDescriptionText)
    end    

    Ob.rResearchInvisButton = MOAIProp2D.new()
    Ob.rResearchInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rResearchInvisButton:setLoc(24, -294)
    Ob.rResearchInvisButton:setScl(362, 70)
    Ob.rResearchInvisButton:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.rResearchInvisButton:setVisible(false)
    Ob:addElement(Ob.rResearchInvisButton)

    Ob.rResearchButton = Ob:getUITextureProp('activity_sheet_research')
    Ob.rResearchButton:setLoc(24, -294)    
    Ob:addElement(Ob.rResearchButton)

    Ob.ResearchText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.ResearchText:setString(string.upper(g_DialogManager:getString("ACNA004TEXT")))
    Ob.ResearchText:setRect(0, 40, 300, 0)
    Ob.ResearchText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.ResearchText:setLoc(160, -290)
    Ob:addElement(Ob.ResearchText)

    if g_CurrentMap:getRoom('lab') == nil then
        Ob.rResearchButton:setColor(1, 1, 1, 0.5)
        Ob.NoLabText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoLabText:setString(g_DialogManager:getString("ACRR003TEXT"))
        Ob.NoLabText:setRect(0, 40, 300, 0)
        Ob.NoLabText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoLabText:setLoc(160, -324)
        Ob.ResearchText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoLabText)
    else
        Ob.ResearchDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.ResearchDescriptionText:setString(g_DialogManager:getString("ACDS004TEXT"))
        Ob.ResearchDescriptionText:setRect(0, 40, 300, 0)
        Ob.ResearchDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.ResearchText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.ResearchDescriptionText:setLoc(160, -324)
        Ob:addElement(Ob.ResearchDescriptionText)
    end

    Ob.rPTJobInvisButton = MOAIProp2D.new()
    Ob.rPTJobInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPTJobInvisButton:setLoc(24, -374)
    Ob.rPTJobInvisButton:setScl(362, 70)
    Ob.rPTJobInvisButton:setColor(0, 0.5, 0.5)
    Ob.rPTJobInvisButton:setVisible(false)
    Ob:addElement(Ob.rPTJobInvisButton)

    Ob.rPTJobButton = Ob:getUITextureProp('activity_sheet_parttimejob')
    Ob.rPTJobButton:setLoc(24, -374)

    Ob.PTJobText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.PTJobText:setString(string.upper(g_DialogManager:getString("ACNA002TEXT")))
    Ob.PTJobText:setRect(0, 40, 300, 0)
    Ob.PTJobText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.PTJobText:setLoc(160, -370)
    Ob:addElement(Ob.PTJobText)

    Ob:addElement(Ob.rPTJobButton)
    if g_CurrentMap:getRoom('cube') == nil then
        Ob.rPTJobButton:setColor(1, 1, 1, 0.5)
        Ob.NoCubiclesText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoCubiclesText:setString(g_DialogManager:getString("ACRR001TEXT"))
        Ob.NoCubiclesText:setRect(0, 40, 300, 0)
        Ob.NoCubiclesText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoCubiclesText:setLoc(160, -404)
        Ob.PTJobText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoCubiclesText)
    else
        Ob.PTJobDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.PTJobDescriptionText:setString(g_DialogManager:getString("ACDS002TEXT"))
        Ob.PTJobDescriptionText:setRect(0, 40, 300, 0)
        Ob.PTJobDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.PTJobDescriptionText:setLoc(160, -404)
        Ob.PTJobText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob:addElement(Ob.PTJobDescriptionText)
    end

    Ob.rRecreationInvisButton = MOAIProp2D.new()
    Ob.rRecreationInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRecreationInvisButton:setLoc(24, -455)
    Ob.rRecreationInvisButton:setScl(362, 70)
    Ob.rRecreationInvisButton:setColor(0, 0.5, 0.5)
    Ob.rRecreationInvisButton:setVisible(false)
    Ob:addElement(Ob.rRecreationInvisButton)

    Ob.rRecreationButton = Ob:getUITextureProp('activity_sheet_timeoff')
    Ob.rRecreationButton:setLoc(24, -455)
    Ob:addElement(Ob.rRecreationButton)

    Ob.RecreationText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.RecreationText:setString(string.upper(g_DialogManager:getString("ACNA003TEXT")))
    Ob.RecreationText:setRect(0, 40, 300, 0)
    Ob.RecreationText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
    Ob.RecreationText:setLoc(160, -450)
    Ob:addElement(Ob.RecreationText)

    if g_CurrentMap:getRoom('recroom') == nil then
        Ob.rRecreationButton:setColor(1, 1, 1, 0.5)

        Ob.NoRecroomText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoRecroomText:setString(g_DialogManager:getString("ACRR002TEXT"))
        Ob.NoRecroomText:setRect(0, 40, 300, 0)
        Ob.NoRecroomText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.NoRecroomText:setLoc(160, -484)
        Ob.RecreationText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3], 0.25)
        Ob:addElement(Ob.NoRecroomText)
    else
        Ob.RecreationDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.RecreationDescriptionText:setString(g_DialogManager:getString("ACDS003TEXT"))
        Ob.RecreationDescriptionText:setRect(0, 40, 300, 0)
        Ob.RecreationDescriptionText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob.RecreationDescriptionText:setLoc(160, -484)
        Ob.RecreationText:setColor(tAvailableTextColor[1], tAvailableTextColor[2], tAvailableTextColor[3])
        Ob:addElement(Ob.RecreationDescriptionText)
    end

    --[[
    Ob.rAbilityUpgradeUI = AbilityUpgradeUI:new(rRenderLayer, rHero, Ob)
    Ob.rAbilityUpgradeUI:setLoc(-360, -130)
    Ob.rAbilityUpgradeUI:setParent(Ob)
    ]]--
    local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
    Ob.rAbilityUpgradeUI = AbilitySelectionUI:new(rOverlayLayer, Ob, rHero, true)
    Ob.rAbilityUpgradeUI:setLoc(-40,0)
    Ob.rAbilityUpgradeUI:setParent(Ob)

    Ob.rTrainingSelectionUI = TrainingSelectionUI:new(rRenderLayer, Ob, rHero)
    Ob.rTrainingSelectionUI:setLoc(0, 0)
    Ob.rTrainingSelectionUI:setParent(Ob)

	return Ob
end

return ActivitySelectionUI
