local UIElement = require('ui/UIElement')
local HeroDetailButton = require('ui/HeroDetailButton')
local ListUI = require('ui/ListUI')
local Renderer = require('Renderer')
local DFIAP = require('DFCommon.IAP')
local DFSaveLoad = require('DFCommon.SaveLoad')

local AndroidOptionsUI = {}
local GameCenterButton = {}
local OptionButton = {}

local tUITextureSpriteSheetInfo =
{
    hero_file = 'UI/hud2',
    soundfx = 'UI/hud2',
    music = 'UI/hud2',
    x = 'UI/hud2',
    gamecenter = 'UI/hud2',
    achievements = 'UI/hud2',
    arrow = 'UI/hud2',
    twohb = 'UI/hud2',
    cloud = 'UI/hud2',
}

local kOPTIONS_SAVE_PATH = "mmojsettings"
local kHIDDEN_X_LOC = -580
local kMINIMIZED_X_LOC = -580 
local kMAXIMIZED_X_LOC = 0

function OptionButton:new(rRenderLayer, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(360, 50)
	Ob.rBackground:setColor(1, 1, 1)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rTopLine = MOAIProp2D.new()
    Ob.rTopLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTopLine:setLoc(0, 0)
    Ob.rTopLine:setScl(360, 3)
    Ob.rTopLine:setColor(.36, .35, .35)
    Ob:addElement(Ob.rTopLine)

    Ob.rLeftLine = MOAIProp2D.new()
    Ob.rLeftLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLeftLine:setLoc(0, 0)
    Ob.rLeftLine:setScl(3, 50)
    Ob.rLeftLine:setColor(.36, .35, .35)
    Ob:addElement(Ob.rLeftLine)

    Ob.rRightLine = MOAIProp2D.new()
    Ob.rRightLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRightLine:setLoc(357, 0)
    Ob.rRightLine:setScl(3, 50)
    Ob.rRightLine:setColor(.36, .35, .35)
    Ob:addElement(Ob.rRightLine)

    Ob.rBottomLine = MOAIProp2D.new()
    Ob.rBottomLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBottomLine:setLoc(0, -50)
    Ob.rBottomLine:setScl(360, 3)
    Ob.rBottomLine:setColor(.36, .35, .35)
    Ob:addElement(Ob.rBottomLine)

    return Ob
end

-- note: hero chooser shouldn't be parented
function AndroidOptionsUI:new(rRenderLayer, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

    Ob.tSettings = {}
    
    function Ob:setVisible(bVisible)
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/Clipboard')
            self:maximize(true)
        end
        self:setElementsVisible(bVisible)
        self.rForumsButton:setVisible(bVisible)
        self.rEmailButton:setVisible(bVisible)
        self.rCreditsButton:setVisible(bVisible)
        if self.rRestorePurchaseButton then
            self.rRestorePurchaseButton:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then   
            if tProps then
                local bTouchedBG = false
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rForumsButton.rBackground then
                        local bIsDebug = (DFHostConfig and DFHostConfig.isDevBuild())
                    
                        if MOAISafariIOS and not g_IsDevBuild then
                            MOAISafariIOS.openURL("http://www.doublefine.com/forums/viewforum/42/")
                            g_GUI:HideOptionsPane()
                        elseif MOAIAppAndroid then
                            MOAIAppAndroid.openURL("http://www.doublefine.com/forums/viewforum/42/")
                            g_GUI:HideOptionsPane()
                        else
                            g_GUI:ShowMMMenu()                        
                        end
                        
                        return true
                    elseif rProp == self.rEmailButton.rBackground then
                        local mailUrl = "mailto:support@doublefine.com"
                    
                        if MOAIEnvironment.udid then
                            
                        
                            mailUrl = mailUrl .. "?subject=Customer%20Service%20Request%20from%20Player%20Id:%20" .. MOAIEnvironment.udid
                        end
                        if MOAISafariIOS then
                            MOAISafariIOS.openURL(mailUrl)
                            g_GUI:HideOptionsPane()
                        elseif MOAIAppAndroid then
                            MOAIAppAndroid.openURL(mailUrl)
                        end
                        return true
                    elseif rProp == self.rCreditsButton.rBackground then
                        -- show credits
                        g_GUI:ShowCreditsUI()
                        return true
                    elseif self.rRestorePurchaseButton and rProp == self.rRestorePurchaseButton.rBackground then
                        if DFIAP.arePurchasesAllowed() then
                            DFIAP.restorePurchases()
                            g_GUI:HideOptionsPane()
                        end
                    elseif rProp == self.rPrivacyButton then
                        -- show privacy
                        if MOAISafariIOS then
                            MOAISafariIOS.openURL("http://www.doublefine.com/privacy/mobile")
                            g_GUI:HideOptionsPane()
                        elseif MOAIAppAndroid then
                            MOAIAppAndroid.openURL("http://www.doublefine.com/privacy/mobile")
                        end
                        return true
                    elseif rProp == self.rBackground then
                        bTouchedBG = true
                    elseif rProp == self.rMusicIcon then
                        self:toggleMusic()
                        return true
                    elseif rProp == self.rSoundFXIcon then
                        self:toggleSFX()
                        return true
                    elseif rProp == self.rCloudIcon then
                        self:toggleCloud()
                        return true
                    end
                end
                if not bTouchedBG then
                    g_GUI:HideOptionsPane()
                end
            end           
        end
        return true
    end

    function Ob:toggleMusic()
        if self:isMusicOn() then
            self:enableMusic(false)
        else
            self:enableMusic(true)
        end
        self:saveSettings()
        self:updateToggleText()
    end

    function Ob:toggleSFX()
        if self:isSFXOn() then
            self:enableSFX(false)
        else
            self:enableSFX(true)
        end
        self:saveSettings()
        self:updateToggleText()
    end
    
    function Ob:toggleCloud()
        local bShowSaveButton = g_Game:getTutorialManager():hasTutorialCompleted('PatrolTutorial')
        
        g_GUI:ShowCloudScreen(bShowSaveButton)
    end

    function Ob:updateToggleText()
        local sSoundFXMode = "STXT089TEXT"
        if not self:isSFXOn() then
            sSoundFXMode = "STXT090TEXT"
            self.rNoSoundFXIcon:setVisible(true)
        else
            self.rNoSoundFXIcon:setVisible(false)            
        end
        self.rSoundFXText:setString(g_DialogManager:getString("STXT087TEXT").." "..g_DialogManager:getString(sSoundFXMode))

        local sMusicMode = "STXT089TEXT"
        if not self:isMusicOn() then
            sMusicMode = "STXT090TEXT"
            self.rNoMusicIcon:setVisible(true)
        else
            self.rNoMusicIcon:setVisible(false)      
        end
        self.rMusicText:setString(g_DialogManager:getString("STXT088TEXT").." "..g_DialogManager:getString(sMusicMode))
    end

    function Ob:isMusicOn()
        local bMusicOn = false
        if nil == self.tSettings.musicEnabled then
            if MOAIFmodEventMgr and not MOAIFmodEventMgr.isSoundCategoryMuted('music') then
                bMusicOn = true
            end
        else
            bMusicOn = self.tSettings.musicEnabled
        end
        return bMusicOn
    end

    function Ob:isSFXOn()
        local bSFXOn = false
        if nil == self.tSettings.sfxEnabled then
            if MOAIFmodEventMgr and not MOAIFmodEventMgr.isSoundCategoryMuted('UI') and not MOAIFmodEventMgr.isSoundCategoryMuted('SFX') and not MOAIFmodEventMgr.isSoundCategoryMuted('Ambience') then
                bSFXOn = true
            end            
        else
            bSFXOn = self.tSettings.sfxEnabled
        end
        return bSFXOn
    end

    function Ob:isCloudOn()
        local bCloudOn = false
        
        -- find out if cloud is on on this device, talk to settings manager or some save file thing
        bCloudOn = (true == self.tSettings.cloudEnabled)
        
        return bCloudOn
    end
    
    function Ob:enableMusic(bEnable)
        if MOAIFmodEventMgr then
            MOAIFmodEventMgr.muteSoundCategory('music', not bEnable)
        end
        
        self.tSettings.musicEnabled = bEnable
    end

    function Ob:enableSFX(bEnable)
        if MOAIFmodEventMgr then
            MOAIFmodEventMgr.muteSoundCategory('UI', not bEnable)
            MOAIFmodEventMgr.muteSoundCategory('SFX', not bEnable)
            MOAIFmodEventMgr.muteSoundCategory('Ambience', not bEnable)
        end
        
        self.tSettings.sfxEnabled = bEnable
    end

    function Ob:isMaximized()
        return self.bMaximized
    end

    function Ob:maximize(bMaximize, bSnap)
        self.bHidden = false
        self.bMaximized = bMaximize
        local nDestXPos = kMAXIMIZED_X_LOC
        if not bMaximize then
            nDestXPos = kMINIMIZED_X_LOC
        end
        local x, y = self:getLoc()
        local xDiff = nDestXPos - x        
        if math.abs(xDiff) > 0 then
            if bSnap then
                self:setLoc(nDestXPos, 0)
            else
                self:moveLoc(xDiff, 0, 0, 0.1, MOAIEaseType.LINEAR)
            end
        end        
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rParentUI = rParentUI
    
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(512, Renderer.getUIHeight())
	Ob.rBackground:setColor(.88, .79, .68)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rPaper = MOAIProp2D.new()
	Ob.rPaper:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rPaper:setLoc(0, 0)
	Ob.rPaper:setScl(490, Renderer.getUIHeight() - 20)
	Ob.rPaper:setColor(.89, .89, .87)
	Ob.rPaper:setLoc(10, -10)
    Ob:addElement(Ob.rPaper)

    Ob.rTopLine = MOAIProp2D.new()
    Ob.rTopLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTopLine:setLoc(60, -50)
    Ob.rTopLine:setScl(392, 3)
    Ob.rTopLine:setColor(.64, .64, .63)
    Ob:addElement(Ob.rTopLine)

    Ob.rLeftLine = MOAIProp2D.new()
    Ob.rLeftLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLeftLine:setLoc(60, -50)
    Ob.rLeftLine:setScl(3, Renderer.getUIHeight() - 100)
    Ob.rLeftLine:setColor(.64, .64, .63)
    Ob:addElement(Ob.rLeftLine)

    Ob.rRightLine = MOAIProp2D.new()
    Ob.rRightLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRightLine:setLoc(449, -50)
    Ob.rRightLine:setScl(3, Renderer.getUIHeight() - 100)
    Ob.rRightLine:setColor(.64, .64, .63)
    Ob:addElement(Ob.rRightLine)

    Ob.rBottomLine = MOAIProp2D.new()
    Ob.rBottomLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBottomLine:setLoc(60, -Renderer.getUIHeight() + 50)
    Ob.rBottomLine:setScl(392, 3)
    Ob.rBottomLine:setColor(.64, .64, .63)
    Ob:addElement(Ob.rBottomLine)

    Ob.rIconDivider1 = MOAIProp2D.new()
    Ob.rIconDivider1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rIconDivider1:setLoc(250, -64)
    Ob.rIconDivider1:setScl(3, 146)
    Ob.rIconDivider1:setColor(.64, .64, .63)
    Ob:addElement(Ob.rIconDivider1)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(76, -210)
    Ob.rLine1:setScl(360, 3)
    Ob.rLine1:setColor(.64, .64, .63)
    Ob:addElement(Ob.rLine1)

    Ob.rTab = Ob:getUITextureProp('hero_file')
    Ob.rTab:setLoc(510, -0)
    Ob:addElement(Ob.rTab)

    Ob.rLabelText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
    Ob.rLabelText:setRect(0, 80, 300, 0)
    Ob.rLabelText:setString(g_DialogManager:getString("STXT086TEXT"))
	Ob.rLabelText:setLoc(534, -34)
    Ob.rLabelText:setRot(0, 0, -90)
	Ob:addElement(Ob.rLabelText)

    local iconX = 130
    local iconY = -86
    
    Ob.rSoundFXIcon = Ob:getUITextureProp('soundfx')
    Ob.rSoundFXIcon:setLoc(iconX, iconY)
    Ob.rSoundFXIcon:setScl(0.75)
    Ob:addElement(Ob.rSoundFXIcon)

    Ob.rNoSoundFXIcon = Ob:getUITextureProp('x')
    Ob.rNoSoundFXIcon:setLoc(iconX - 10, iconY - 10)
    Ob.rNoSoundFXIcon:setScl(0.75)
    Ob:addElement(Ob.rNoSoundFXIcon)

    iconX = iconX + 168
    
    Ob.rMusicIcon = Ob:getUITextureProp('music')
    Ob.rMusicIcon:setLoc(iconX, iconY)
    Ob.rMusicIcon:setScl(0.75)
    Ob:addElement(Ob.rMusicIcon)

    Ob.rNoMusicIcon = Ob:getUITextureProp('x')
    Ob.rNoMusicIcon:setLoc(iconX - 10, iconY - 10)
    Ob.rNoMusicIcon:setScl(0.75)
    Ob:addElement(Ob.rNoMusicIcon)
    
    local textX = 168
    
    Ob.rSoundFXText = g_GUI:CreateTextBox(16, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rSoundFXText:setRect(-150, 80, 150, 0)
	Ob.rSoundFXText:setLoc(textX, -172)
    Ob.rSoundFXText:setColor(.36, .35, .35)
	Ob:addElement(Ob.rSoundFXText)

    textX = textX + 168
    
    Ob.rMusicText = g_GUI:CreateTextBox(16, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rMusicText:setRect(-150, 80, 150, 0)
	Ob.rMusicText:setLoc(textX, -172)
    Ob.rMusicText:setColor(.36, .35, .35)
	Ob:addElement(Ob.rMusicText)
    
    Ob.rForumsButton = OptionButton:new(rRenderLayer, Ob)
    Ob.rForumsButton:setLoc(76, -224)
    Ob.rForumsButton:setParent(Ob)

    Ob.rEmailButton = OptionButton:new(rRenderLayer, Ob)
    Ob.rEmailButton:setLoc(76, -275)
    Ob.rEmailButton:setParent(Ob)

    Ob.rCreditsButton = OptionButton:new(rRenderLayer, Ob)
    Ob.rCreditsButton:setLoc(76, -326)
    Ob.rCreditsButton:setParent(Ob)

    Ob.rForumsIcon = Ob:getUITextureProp('arrow')
    Ob.rForumsIcon:setLoc(96, -234)
    Ob:addOverlayElement(Ob.rForumsIcon)
    
    Ob.rEmailIcon = Ob:getUITextureProp('arrow')
    Ob.rEmailIcon:setLoc(96, -285)
    Ob:addOverlayElement(Ob.rEmailIcon)

    Ob.rCreditsArrow = Ob:getUITextureProp('arrow')
    Ob.rCreditsArrow:setLoc(96, -336)
    Ob:addOverlayElement(Ob.rCreditsArrow)

    Ob.rCreditsIcon = Ob:getUITextureProp('twohb')
    Ob.rCreditsIcon:setLoc(196, -336)
    Ob:addOverlayElement(Ob.rCreditsIcon)

    Ob.rForumsText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rForumsText:setRect(0, 80, 300, 0)
    Ob.rForumsText:setString(g_DialogManager:getString("STXT093TEXT"))
	Ob.rForumsText:setLoc(120, -236)
	Ob:addOverlayElement(Ob.rForumsText)

    Ob.rEmailText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rEmailText:setRect(0, 80, 300, 0)
    Ob.rEmailText:setString(g_DialogManager:getString("STXT141TEXT"))
	Ob.rEmailText:setLoc(120, -287)
	Ob:addOverlayElement(Ob.rEmailText)

    Ob.rCreditsText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rCreditsText:setRect(0, 80, 300, 0)
    Ob.rCreditsText:setString(g_DialogManager:getString("STXT094TEXT"))
	Ob.rCreditsText:setLoc(120, -339)
	Ob:addOverlayElement(Ob.rCreditsText)

    local nAppVersion = MOAIEnvironment.appVersion or 0
    local nBuildNum = MOAIEnvironment.appBuild or 0
    Ob.rBuildText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rBuildText:setRect(-150, 80, 150, 0)
    Ob.rBuildText:setString("Version "..nAppVersion.." ".."Build "..nBuildNum)
	Ob.rBuildText:setLoc(256, -Renderer.getUIHeight() + 46)
    Ob.rBuildText:setColor(.36, .35, .35)
	Ob:addOverlayElement(Ob.rBuildText)

    local nGameID = MOAIEnvironment.udid or 0
    Ob.rGameID = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rGameID:setRect(-300, 80, 300, 0)
    Ob.rGameID:setString("Game ID "..nGameID)
	Ob.rGameID:setLoc(256, -Renderer.getUIHeight() + 30)
    Ob.rGameID:setColor(.36, .35, .35)
	Ob:addOverlayElement(Ob.rGameID)    

    Ob.rPrivacyText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rPrivacyText:setRect(-150, 80, 150, 0)
    Ob.rPrivacyText:setString(g_DialogManager:getString("STXT096TEXT"))
	Ob.rPrivacyText:setLoc(256, -Renderer.getUIHeight() + 82)
    Ob.rPrivacyText:setColor(.36, .35, .35)
	Ob:addOverlayElement(Ob.rPrivacyText)

    Ob.rUnderscoreText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rUnderscoreText:setRect(-150, 80, 150, 0)
    Ob.rUnderscoreText:setString("________________")
	Ob.rUnderscoreText:setLoc(256, -Renderer.getUIHeight() + 82)
    Ob.rUnderscoreText:setColor(.36, .35, .35)
	Ob:addOverlayElement(Ob.rUnderscoreText)

    Ob.rPrivacyButton = MOAIProp2D.new()
	Ob.rPrivacyButton:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rPrivacyButton:setColor(.88, .79, .68)
	Ob.rPrivacyButton:setLoc(206, -530, 0)
    Ob.rPrivacyButton:setScl(100, 60)
    Ob.rPrivacyButton:setVisible(false)
    Ob:addElement(Ob.rPrivacyButton)

    if DFIAP.arePurchasesAllowed() then
        Ob.rRestorePurchaseButton = OptionButton:new(rRenderLayer, Ob)
        Ob.rRestorePurchaseButton:setLoc(76, -377)
        Ob.rRestorePurchaseButton:setParent(Ob)

        Ob.rRestorePurchaseArrow = Ob:getUITextureProp('arrow')
        Ob.rRestorePurchaseArrow:setLoc(96, -387)
        Ob:addOverlayElement(Ob.rRestorePurchaseArrow)
        
        Ob.rRestorePurchaseText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rRestorePurchaseText:setRect(0, 80, 300, 0)
        Ob.rRestorePurchaseText:setString(g_DialogManager:getString("STXT137TEXT"))
        Ob.rRestorePurchaseText:setLoc(120, -390)
        Ob:addOverlayElement(Ob.rRestorePurchaseText)
    end

    function Ob:loadSettings()
        local tSettings = DFSaveLoad.loadTable(kOPTIONS_SAVE_PATH)
        
        if not tSettings then
            tSettings = {}
        end
        
        self.tSettings = tSettings
        
        self:updateToggleText()
    end
    
    function Ob:saveSettings()
        DFSaveLoad.saveTable(self.tSettings, kOPTIONS_SAVE_PATH)
    end
    
    Ob:loadSettings()
    

    return Ob
end

return AndroidOptionsUI