local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')

local tUITextureSpriteSheetInfo =
{
    indexcard = 'UI/shopheropack',
    thumbtack = 'UI/shopheropack',
}

local BulletinBoardUI = {}
local BulletinMessageDefs =
{
    alwaysAvailable =
    {
        "BULLET001TEXT", "BULLET004TEXT", "BULLET006TEXT", "BULLET007TEXT", "BULLET008TEXT", 
        "BULLET009TEXT", "BULLET010TEXT", "BULLET011TEXT", "BULLET013TEXT",
    },
    
    availableWithHero = 
    {
        --capt. premium
        TestMale01c =
        {
            "BULLET003TEXT", "BULLET024TEXT",
        },
        
        --hothead
        TestMale02 =
        {
            "BULLET005TEXT","BULLET033TEXT",
        },

        --galaxy girl
        TestFemale01 =
        {
            "BULLET015TEXT",
        },

        -- masked mummy
        TestMale01b =
        {
            "BULLET002TEXT",
        },

        --@man
        TestMale02b =
        {
            "BULLET014TEXT", "BULLET025TEXT", "BULLET026TEXT",
        },
        
        --crimebot
        Robot01 =
        {
            "BULLET016TEXT", "BULLET020TEXT", "BULLET022TEXT",
        },
        
        --surge protector
        TestFemale02 =
        {
            "BULLET017TEXT", "BULLET023TEXT",
        },
        
        --sweet justice
        TestMale01 =
        {
            "BULLET018TEXT", "BULLET019TEXT", "BULLET027TEXT",
        },
        --crowbar
        TestMale01d =
        {
            "BULLET032TEXT", 
        },
        --gloomerang
        TestFemale02b =
        {
            "BULLET034TEXT",
        },
    },
	
    availableWithTerritory = 
    {
        PummelPark =
        {
            "BULLET013TEXT",
        },
    },   
}

local HolidayBulletins =
{
    valentines =
    {
        availableWithHero = 
        {
            --@man
            TestMale02b =
            {
                "BULLET028TEXT", "BULLET030TEXT",
            },
            --surge protector
            TestFemale02 =
            {
                "BULLET029TEXT",
            },
            --crimebot
            Robot01 =
            {
                "BULLET031TEXT", "BULLET030TEXT",
            },
        },
    },
}

function BulletinBoardUI:new(rRenderLayer)
    local Ob = UIElement:new(rRenderLayer)
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        g_Audio:playCue('MMOJ_Audio/UI/EquipFolder')
    end
    
    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType ~= MOAITouchSensor.TOUCH_UP then return false end

        local handled = false

        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rBackground then
                    handled = true
                    break
                end
            end
        end
        
        -- not touched anything, close
        if not handled then
            g_GUI:HideBulletinBoard()
        end
        
        return true
    end
    
    function Ob:getLines(rBulletinDefs, tPossibleLines)
        local numPossibleLines = 0

        if rBulletinDefs.alwaysAvailable then
            for i, sLine in ipairs(rBulletinDefs.alwaysAvailable) do
                table.insert(tPossibleLines, sLine)
                numPossibleLines = numPossibleLines + 1
            end
        end

        if rBulletinDefs.availableWithHero then
            -- check which heroes are unlocked and insert lines for those
            local heroes = g_Game:getAllHeroes()
            
            for heroId, hero in pairs(heroes) do
                for sCheckId, sLines in pairs(rBulletinDefs.availableWithHero) do
                    if heroId == sCheckId then
                        for i, sLine in ipairs(sLines) do
                            table.insert(tPossibleLines, sLine)
                            numPossibleLines = numPossibleLines + 1
                        end
                    end
                end
            end
        end
        
        if rBulletinDefs.availableWithTerritory then
            -- check which territories are unlocked and insert lines for those
            local territories = g_City:getUnlockedTerritories()
            for idx, ter in pairs(territories) do
                local terId = ter.sDefKey
                
                for sCheckId, sLines in pairs(rBulletinDefs.availableWithTerritory) do
                    if terId == sCheckId then
                        for i, sLine in ipairs(sLines) do
                            table.insert(tPossibleLines, sLine)
                            numPossibleLines = numPossibleLines + 1
                        end
                    end
                end
            end        
        end

        return numPossibleLines
    end

    function Ob:SelectRandomText()
        local out = "BULLET001TEXT"
        
        local tPossibleLines = {}
        local numPossibleLines = 0
        
        local nLines = self:getLines(BulletinMessageDefs, tPossibleLines)
        numPossibleLines = numPossibleLines + nLines

        for sHolidayKey, rBulletinInfo in pairs(HolidayBulletins) do
            if g_Game:isHolidayActive(sHolidayKey) then
                local nLines = self:getLines(rBulletinInfo, tPossibleLines)
                numPossibleLines = numPossibleLines + nLines   
            end
        end
        
        local randomIdx = math.random(numPossibleLines)
        
        out = tPossibleLines[randomIdx]
        
        return out
    end
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    
    Ob.rScrim = MOAIProp2D.new()
	Ob.rScrim:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rScrim:setLoc(0, 0)
	Ob.rScrim:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rScrim:setColor(.4, .4, .4, .5)
	Ob.rScrim:setLoc(0, 0)
    Ob:addElement(Ob.rScrim)
    Ob.rScrim:setParent(nil)

    Ob.rBackground = Ob:getUITextureProp('indexcard')
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(1.4, 1.4)
    Ob:addElement(Ob.rBackground)
    
    local randomLine = Ob:SelectRandomText()
    randomLine = g_DialogManager:getString(randomLine)
    randomLine = string.gsub(randomLine, "\\n", "\n")
    
    Ob.rTextBox = g_GUI:CreateTextBox(28, MOAITextBox.LEFT_JUSTIFY)
    Ob.rTextBox:setRect(0, 300, 620, 0)
    Ob.rTextBox:setString(randomLine)
    Ob.rTextBox:setLoc(30, -80)
    Ob:addElement(Ob.rTextBox)
    
    Ob.rTack = Ob:getUITextureProp('thumbtack')
    Ob.rTack:setLoc(320, 16)
    Ob.rTack:setScl(1.4, 1.4)
    Ob:addElement(Ob.rTack)
    
    local screenX = Renderer.getUIHalfWidth() - 320
    local screenY = -Renderer.getUIHalfHeight() + 200
    
    Ob:setLoc(screenX, screenY)
    
    return Ob
end

return BulletinBoardUI
