local UIElement = require('ui/UIElement')
local Button = {}

function Button:new(rRenderLayer, rParentUI, tSpriteSheetInfo, sButtonTexture, sButtonPressedTexture, tButtonColor, tPressedButtonColor, bYFlip, tPressedOffsetLoc, sButtonText, sButtonTextFont, nTextSize, tButtonTextOffset, tButtonTextColor, tPressedButtonTextColor)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:onFinger(eventType, sx, sy, tProps)  
        if not self.bVisible then
            return false
        end
        local bTouchedButton = false
        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rButton then
                    bTouchedButton = true
                    break
                elseif rProp == self.rPressedButton then
                    bTouchedButton = true
                    break
                end
            end
        end
        if eventType == MOAITouchSensor.TOUCH_DOWN then
            if bTouchedButton then
                self:onPressed(true)
            else
                self:onPressed(false)
            end
        elseif eventType == MOAITouchSensor.TOUCH_MOVE then
            if bTouchedButton then
                self:onPressed(true)
            else
                self:onPressed(false)
            end
        elseif eventType == MOAITouchSensor.TOUCH_UP then
            if bTouchedButton then
                self:onPressed(false)
            end
            if not g_TouchMoved then
                if bTouchedButton then
                    if self.rParentUI and self.rParentUI.onButtonJustPressed then
                        self.rParentUI:onButtonJustPressed(self)
                    end
                end
            end
        end
    end

    function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
    end

    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onPressed(bPressed)
        if self.bPressed == bPressed then
            return
        end
        self.bPressed = bPressed
        if not bPressed then
            if self.rButton then 
                self.rButton:setVisible(true)
                if self.tButtonColor then
                    self.rButton:setColor(self.tButtonColor[1], self.tButtonColor[2], self.tButtonColor[3])
                end
            end
            if self.rPressedButton then
                self.rPressedButton:setVisible(false)
            end
            if self.tPressedOffsetLoc then
                if self.nOrigX and self.nOrigY then
                    self:setLoc(self.nOrigX, self.nOrigY)
                end
            end
            if self.rButtonText then
                if self.tButtonTextColor then
                    self.rButtonText:setColor(self.tButtonTextColor[1], self.tButtonTextColor[2], self.tButtonTextColor[3])
                end
            end
            if self.bYFlip then
                if self.nOrigSclX and self.nOrigSclY then
                    self:setScl(self.nOrigSclX, self.nOrigSclY)
                    if self.rButtonText then
                        if self.nButtonOrigX and self.nButtonOrigY then
                            self.rButtonText:setLoc(self.nButtonOrigX, self.nButtonOrigY)
                        end
                    end
                    if self.rMMReqText then
                        self.rMMReqText:setLoc(self.nButtonOrigX, self.nButtonOrigY)
                    end
                end
            end
        else
            if self.rButton then
                if self.tPressedButtonColor then
                    self.rButton:setColor(self.tPressedButtonColor[1], self.tPressedButtonColor[2], self.tPressedButtonColor[3])
                end
            end
            if self.rPressedButton then
                self.rButton:setVisible(false)
                self.rPressedButton:setVisible(true)
            end
            if self.tPressedOffsetLoc then
                self.nOrigX, self.nOrigY = self:getLoc()
                self:setLoc(self.nOrigX + self.tPressedOffsetLoc[1], self.nOrigY + self.tPressedOffsetLoc[2])
            end
            if self.rButtonText then
                if self.tPressedButtonTextColor then
                    self.rButtonText:setColor(self.tPressedButtonTextColor[1], self.tPressedButtonTextColor[2], self.tPressedButtonTextColor[3])
                end
            end
            if self.bYFlip then
                self.nOrigSclX, self.nOrigSclY  = self:getScl() 
                self:setScl(self.nOrigSclX, -self.nOrigSclY)
                if self.rButtonText then
                    self.nButtonOrigX, self.nButtonOrigY = self.rButtonText:getLoc()
                    self.rButtonText:setLoc(self.nButtonOrigX + self.tPressedOffsetLoc[1], self.nButtonOrigY + self.tPressedOffsetLoc[2])
                end
                if self.rMMReqText then
                    self.rMMReqText:setLoc(self.nButtonOrigX + self.tPressedOffsetLoc[1], self.nButtonOrigY + self.tPressedOffsetLoc[2])
                end
            end
        end
        if self.pressedCallback then
            self.pressedCallback(self, bPressed)
        end  
    end

    function Ob:setText(sLineCode, bNotLineCode, nSize)
        if bNotLineCode then
            self.rButtonText:setString(sLineCode)
        else
            self.rButtonText:setString(g_DialogManager:getString(sLineCode))
        end
        if nSize and self.rButtonText then
            self.rButtonText:setTextSize(nSize)
        end
    end

    function Ob:setButtonScl(x, y)
        if self.rButton then
            self.rButton:setScl(x, y)
        end
        if self.rPressedButton then
            self.rPressedButton:setScl(x, y)
        end
    end

    function Ob:setButtonColorOverride(r, g, b)
        self.tButtonColorOverride = { r, g, b }
        if self.tButtonColorOverride and self.tButtonColorOverride[1] and self.tButtonColorOverride[2] and self.tButtonColorOverride[3] then
            if self.rButton then
                self.rButton:setColor(self.tButtonColorOverride[1], self.tButtonColorOverride[2], self.tButtonColorOverride[3])
            end
        end      
    end

    function Ob:clearButtonColorOverride()
        self.tButtonColorOverride = nil
        if self.tButtonColor and self.rButton then
            self.rButton:setColor(self.tButtonColor[1], self.tButtonColor[2], self.tButtonColor[3])
        end
    end       

    if tSpriteSheetInfo then
        Ob:loadUISpriteSheets(tSpriteSheetInfo)
    end

    if sButtonTexture then
        Ob.rButton = Ob:getUITextureProp(sButtonTexture)       
        Ob:addElement(Ob.rButton)
    end
    if sButtonPressedTexture then
        Ob.rPressedButton = Ob:getUITextureProp(sButtonPressedTexture)
        Ob:addElement(Ob.rPressedButton)
    end
    if sButtonText and sButtonTextFont and nTextSize then
        Ob.rButtonText = g_GUI:CreateTextBox(nTextSize, MOAITextBox.CENTER_JUSTIFY, sButtonTextFont)
        if tButtonTextOffset then
            Ob.rButtonText:setString(g_DialogManager:getString(sButtonText))
            Ob.rButtonText:setLoc(tButtonTextOffset[1], tButtonTextOffset[2])
        end
        Ob:addElement(Ob.rButtonText)
        Ob.rButtonText:setParent(nil)
        Ob.rButtonText:setAttrLink(MOAIProp.INHERIT_LOC, Ob, MOAIProp.TRANSFORM_TRAIT)

        Ob.rMMReqText = g_GUI:CreateTextBox(nTextSize - 8, MOAITextBox.CENTER_JUSTIFY, sButtonTextFont)
        Ob.rMMReqText:setString("MM LVL ")
        if tButtonTextOffset then
            Ob.rMMReqText:setLoc(tButtonTextOffset[1], tButtonTextOffset[2] - 4)
        end
        Ob.rMMReqText:setReveal(0)
        Ob:addElement(Ob.rMMReqText)
        Ob.rMMReqText:setParent(nil)
        Ob.rMMReqText:setAttrLink(MOAIProp.INHERIT_LOC, Ob, MOAIProp.TRANSFORM_TRAIT)
    end
    Ob.tButtonColor = tButtonColor
    Ob.tPressedButtonColor = tPressedButtonColor
    Ob.tPressedOffsetLoc = tPressedOffsetLoc
    Ob.tButtonTextColor = tButtonTextColor
    Ob.tPressedButtonTextColor = tPressedButtonTextColor
    Ob.bYFlip = bYFlip
    Ob.rParentUI = rParentUI

    Ob:onPressed(false)

    return Ob
end

return Button