local CloudSyncUI = {}
local CloudSyncPleaseWaitPopup = {}

local UIElement = require('ui/UIElement')
local Button = require('ui/Button')
local Renderer = require('Renderer')
local util = require('DFUtil')
local DFSaveLoad = require('DFCommon.SaveLoad')

local kPOSTIT_X_SIZE = 512
local kPOSTIT_Y_SIZE = 410

local tUITextureSpriteSheetInfo =
{
    popup_button = 'UI/shopheropack',
    popup = 'UI/shopheropack',
}

function CloudSyncPleaseWaitPopup:new(rRenderLayer, sMessageText, rListener, bShowCancelButton, bShowOKButton)
    local Ob = UIElement:new(rRenderLayer)
    Ob.rListener = rListener
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rCancelButton then
            self.rCancelButton:setVisible(bVisible)
        end
        if self.rOKButton then
            self.rOKButton:setVisible(bVisible)
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rCancelButton or rButton == self.rOKButton then
            g_Audio:playCue('MMOJ_Audio/UI/BackButton')
            self:setVisible(false)
            
            if self.rListener and self.rListener.onPopupCancel then
                self.rListener:onPopupCancel(self)
            end
        end
    end
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rCancelButton then
            self.rCancelButton:onFinger(eventType, sx, sy, tProps)
        end
        if self.rOKButton then
            self.rOKButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rBackground = Ob:getUITextureProp('popup')
    Ob.rBackground:setScl(1.1)
	Ob:addOverlayElement(Ob.rBackground)

    if bShowCancelButton then
        local sCancelText = "SYST033TEXT"
    
        Ob.rCancelButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'popup_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 }, false, nil, sCancelText, 'default', 22, { 50, -5 }, { 0, 0, 0 })    
        Ob.rCancelButton:setLoc(100, -140)
        Ob.rCancelButton:setScl(1.6)
        Ob.rCancelButton:setParent(Ob)
    end
    
    if bShowOKButton then
        local sOKText = "SYST032TEXT"
    
        Ob.rOKButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'popup_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 }, false, nil, sOKText, 'default', 22, { 50, -5 }, { 0, 0, 0 })    
        Ob.rOKButton:setLoc(100, -140)
        Ob.rOKButton:setScl(1.6)
        Ob.rOKButton:setParent(Ob)
    end
    
    Ob.rTextBox = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
    Ob.rTextBox:setRect(0, 300, 260, 0)
    Ob.rTextBox:setString(sMessageText)
    Ob.rTextBox:setLoc(50, -10)
    Ob:addOverlayElement(Ob.rTextBox)

    local screenX = (Renderer.getUIWidth() - 354) / 2
    local screenY = (Renderer.getUIHeight() - 230) / 2 + 64
    Ob:setLoc(screenX, -screenY)
    
    return Ob
end

function CloudSyncUI:new(rRenderLayer, bShowSaveButton, rParentUI)
    local Ob = UIElement:new(rRenderLayer)
    Ob.rParentUI = rParentUI
    
    local sMessageText = g_DialogManager:getString("STXT198TEXT")
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rLoadButton:setVisible(bVisible)
        self.rCancelButton:setVisible(bVisible)
        if self.rSaveButton then
            self.rSaveButton:setVisible(bVisible)
        end
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/PostIt')
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rLoadButton:onPaneClosed()
        if self.rSaveButton then
            self.rSaveButton:onPaneClosed()
        end
        DFSaveLoad.CloudSaves.setOnLoadListener(nil)
    end
    
    function Ob:onPopupCancel()
        print("onPopupCancel()")
        self.rPopup = nil
        self.bIsWaitingForLoad = false
    end
    
    function Ob:onCloudSaveLoaded(sDataKey, tSave)
        print("on cloud save loaded!")
        
        self.bIsWaitingForLoad = false
        
        g_GUI:SetActivePane(nil)
        g_GUI:HideCloudScreen()
        
        g_Game:onCloudSaveLoaded(sDataKey, tSave)
        
        if self.rPopup then
            self.rPopup:setVisible(false)
            self.rPopup = nil
        end
        

        self:onPaneClosed()
        if self.rParentUI then
            self.rParentUI:onCloudSaveLoaded(tSave)
        end
    end
    
    
    function Ob:onLoad()
    
    	if not self.bCloudSavesInitialized then
    		self.rPopup = CloudSyncPleaseWaitPopup:new(self.rRenderLayer, "You must enable iCloud in your iOS Settings in order to use cloud sync.", self, false, true)
    		self.rPopup:setVisible(true)
    	else
			local function loadFunc()
				self.bIsWaitingForLoad = true
				
				local sMessage = g_DialogManager:getString("STXT201TEXT")
			
				DFSaveLoad.CloudSaves.setOnLoadListener(self)
				
				local kCLOUD_SAVE_FILENAME = "mmojcloudsave_0"
				local tData = DFSaveLoad.CloudSaves.loadData(kCLOUD_SAVE_FILENAME, true)
				self.rPopup = CloudSyncPleaseWaitPopup:new(self.rRenderLayer, sMessage, self, true)
				self.rPopup:setVisible(true)
	
				local nTimeWaiting = 0
				local nTimeout = 12.0
				
				while self.bIsWaitingForLoad and nTimeWaiting < nTimeout do
					local nTimeToWait = 0.1
					nTimeWaiting = nTimeWaiting + nTimeToWait
	
					util.sleep(nTimeToWait)
				end
				
				if self.rPopup then
					self.rPopup:setVisible(false)
					self.rPopup = nil
				end
						
				if nTimeWaiting >= nTimeout then
					self.rPopup = CloudSyncPleaseWaitPopup:new(self.rRenderLayer, "Failed to load Cloud Save from iCloud. Please try again later.", self, false, true)
				end
				
				if self.rPopup then
					self.rPopup:setVisible(true)
				end
				
				print("done attempting to load.")
			end
			
			self.rThread = MOAIThread.new()
			self.rThread:run(loadFunc, self)           
		end
    end
    
    function Ob:onSave()
    	if not self.bCloudSavesInitialized then
    		self.rPopup = CloudSyncPleaseWaitPopup:new(self.rRenderLayer, "You must enable iCloud in your iOS Settings in order to use cloud sync.", self, false, true)
    		self.rPopup:setVisible(true)
    	else
			local function saveFunc()
				local sMessage = g_DialogManager:getString("STXT199TEXT")
				
				self.rPopup = CloudSyncPleaseWaitPopup:new(self.rRenderLayer, sMessage)
				self.rPopup:setVisible(true)
				
				-- save to cloud
				g_Game:saveGame(true)
				
				util.sleep(3.0)
				
				if self.rPopup then
					self.rPopup:setVisible(false)
					self.rPopup = nil
				end
				
				g_GUI:HideCloudScreen()
				self:onPaneClosed()
			end
			
			self.rThread = MOAIThread.new()
			self.rThread:run(saveFunc, self)   
        end
    end
    
    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rLoadButton then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
            self:onLoad()
        elseif rButton == self.rSaveButton then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
            self:onSave()
        elseif rButton == self.rCancelButton then
            g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                    
            g_GUI:HideCloudScreen()
            self:onPaneClosed()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        local bHandled = false
        
        if self.rPopup then
            bHandled = self.rPopup:onFinger(eventType, sx, sy, tProps)
        end
    
        if not bHandled then
            if self.rLoadButton then
                self.rLoadButton:onFinger(eventType, sx, sy, tProps)
            end
            if self.rSaveButton then
                self.rSaveButton:onFinger(eventType, sx, sy, tProps)
            end
            if self.rCancelButton then
                self.rCancelButton:onFinger(eventType, sx, sy, tProps)
            end
        end
        
		return true
	end    

    Ob.bCloudSavesInitialized = DFSaveLoad.CloudSaves.init()

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rBackground = Ob:getUITextureProp('popup')
    Ob.rBackground:setScl(1.61)
	Ob:addElement(Ob.rBackground)

    local sLoadText = "STXT195TEXT"
    local sSaveText = "STXT196TEXT"
    local sCancelText = "SYST033TEXT"
    
    Ob.rLoadButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'popup_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 }, false, nil, sLoadText, 'default', 22, { 50, -5 }, { 0, 0, 0 })    
    Ob.rLoadButton:setLoc(180, -200)
    Ob.rLoadButton:setScl(1.6)
    Ob.rLoadButton:setParent(Ob)

    Ob.rCancelButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'popup_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 }, false, nil, sCancelText, 'default', 22, { 50, -5 }, { 0, 0, 0 })    
    Ob.rCancelButton:setLoc(180, -260)
    Ob.rCancelButton:setScl(1.6)
    Ob.rCancelButton:setParent(Ob)

    if bShowSaveButton then
        sMessageText = g_DialogManager:getString("STXT197TEXT")
    
        Ob.rSaveButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'popup_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 }, false, nil, sSaveText, 'default', 22, { 50, -5 }, { 0, 0, 0 })    
        Ob.rSaveButton:setLoc(180, -200)
        Ob.rSaveButton:setScl(1.6)
        Ob.rSaveButton:setParent(Ob)
        
        Ob.rCancelButton:setLoc(180, -260)
        Ob.rLoadButton:setLoc(180, -140)
    end

    if sMessageText then
        Ob.rTextBox = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rTextBox:setRect(0, 300, 400, 0)
        Ob.rTextBox:setString(sMessageText)
        Ob.rTextBox:setLoc(60, -40)
        Ob:addElement(Ob.rTextBox)
    end
    
    local screenX = (Renderer.getUIWidth() - 520) / 2
    local screenY = (Renderer.getUIHeight() - 338) / 2
    Ob:setLoc(screenX, -screenY)
    
    return Ob
end

return CloudSyncUI