local Item = require('Item')
local UIElement = require('ui/UIElement')
local ConsumableDetailButton = require('ui/ConsumableDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local Button = require('ui/Button')
local ConsumableInventoryButton = {}

local tUITextureSpriteSheetInfo = 
{
numberbg = 'UI/shop',
button_buy = 'UI/shop',
}

function ConsumableInventoryButton:new(rRenderLayer, sItemKey, rConsumablePane)
	if not sItemKey then
		print("ERROR: ConsumableInventoryButton created without an Item")
		return nil
	end
    local rDef = Item.getConsumableDef(sItemKey)
    if not rDef then
		print("ERROR: ConsumableInventoryButton created without an Item")
		return nil
    end

	local Ob = ConsumableDetailButton:new(rRenderLayer, sItemKey)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rSellButton then
            self.rSellButton:setVisible(bVisible)
        end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rSellButton then
            if self.sItemKey then
                -- creating the item so that the selling goes through the same codepath
                local rDef = Item.getConsumableDef(self.sItemKey)
                if rDef then
                    if not self.rItem then
                        local constructor = Item.getConstructor(rDef.sItemType)
                        if constructor then
                            self.rItem = constructor(Item, rDef, self.sItemKey)
                        end
                    end
                end
                if self.rItem then
                    local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT098TEXT")..' '..g_DialogManager:getString(rDef.sName)..' '..g_DialogManager:getString("STXT099TEXT").." "..self.rItem:getSellPrice().."?", self, "STXT075TEXT", "STXT076TEXT")
                    rConfirmPane:setLoc(240, -160)
                    g_GUI:SetPopupPane(rConfirmPane)
                end
            end
        end
    end

    function Ob:onFingerCallBack(eventType, sx, sy, tProps)
        if self.rSellButton then
            self.rSellButton:onFinger(eventType, sx, sy, tProps)
        end
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:sellItem()
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:sellItem()
        if self.rItem then
            g_Audio:playCue('MMOJ_Audio/UI/BuyButton')
            g_MiddleManager:sellConsumable(self.sItemKey, self.rItem:getSellPrice())
            self:update()
        end       
    end    

    function Ob:update()
        if self.rItem then
            local nNumItems = g_MiddleManager:getNumConsumablesInInventory(self.sItemKey)
            
            if self.rItem.rDef and self.rItem.rDef.bPermanent then
                if g_Game:isPropEnabled(self.rItem.rDef.sPropName) then
                    nNumItems = 1
                end
            end    
                
            if nNumItems > 0 then
                self:setVisible(true)
                self.bHideFromList = false
                
                if self.rNumText then
                    if nNumItems > 1 then                  
                        self.rNumberBG:setVisible(true)
                        self.rNumText:revealAll()                   
                    else
                        self.rNumberBG:setVisible(true)
                        self.rNumText:revealAll()
                    end
                    self.rNumText:setString(tostring(nNumItems))                    
                end
            else
                self:setVisible(false)
                self.bHideFromList = true
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rConsumablePane = rConsumablePane
    Ob.sItemKey = sItemKey

    local bShowSellButton = true
    local bShowQuantity = true
    
    if rDef then
        local constructor = Item.getConstructor(rDef.sItemType)
        if constructor then
            Ob.rItem = constructor(Item, rDef, sItemKey)
        end
        
        if rDef.bPermanent then
            bShowSellButton = false
            bShowQuantity = false
        end
    end

    if Ob.rCostField then
        Ob.rCostField:setReveal(0)
    end
    if Ob.rCostIcon then
        Ob.rCostIcon:setVisible(false)
    end
    if Ob.rButton then
        Ob.rButton:setVisible(false)
    end

    if bShowQuantity then
        Ob.rNumberBG = Ob:getUITextureProp('numberbg')
        Ob.rNumberBG:setScl(1.2)
        Ob.rNumberBG:setLoc(100, -120)
        Ob:addElement(Ob.rNumberBG)
        
        Ob.rNumText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rNumText:setString("1")
        Ob.rNumText:setRect(-100, 80, 100, 0)
        Ob.rNumText:setColor(1, 1, 1)
        Ob.rNumText:setLoc(120, -117)
        Ob:addElement(Ob.rNumText)
    end
    
    if bShowSellButton then
        Ob.rSellButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { 1, 0, 0 }, nil, true, { 0, -50 }, "STXT101TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
        Ob.rSellButton:setLoc(16, -152)
        Ob.rSellButton:setParent(Ob)
    end
    
    return Ob
end

return ConsumableInventoryButton