local Item = require('Item')
local UIElement = require('ui/UIElement')
local ConsumableDetailButton = require('ui/ConsumableDetailButton')
local DFAnalytics = require('DFCommon.Analytics')
local Renderer = require('Renderer')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')

local ConsumablePurchaseButton = {}

function ConsumablePurchaseButton:new(rRenderLayer, sItemKey)
	if not sItemKey then
		print("ERROR: ConsumablePurchaseButton created without an Item")
		return nil
	end
    local rDef = Item.getConsumableDef(sItemKey)
    if not rDef then
		print("ERROR: ConsumablePurchaseButton created without an Item")
		return nil
    end

	local Ob = ConsumableDetailButton:new(rRenderLayer, sItemKey)

    function Ob:onButtonPressed()
        self:buyItem()
    end

    function Ob:canBuyItem()
        if self.rItemDef.nCost then
            if g_Game:getCurCash() < self.rItemDef.nCost then
                return false
            end
        end
        return true
    end

    function Ob:buyItem()
        if self:canBuyItem() then
            -- show are you sure pop
            rListener = {}
            
            -- used by SimpleConfirmScreenUI
            function rListener:onYesPressed(rConfirmScreen)
                g_Game:removeCash(self.rItemDef.nCost)
                
                DFAnalytics.logEvent("buy_cnsm", { type = self.sItemKey })
                DFAnalytics.incrStat("csh_spnt_cnsm", self.rItemDef.nCost)   
                
                -- a bit of a redundant event here but we want to track these with funnels in apsalar
                DFAnalytics.logEvent("buy_cnsm_" .. self.sItemKey, { cost = self.rItemDef.nCost })
                
                if self.rItemDef.bPermanent then
                    if self.rItemDef.sPropName then
                        -- add prop to world
                        g_Game:enableProp(self.rItemDef.sPropName)
                        g_Game:saveGame()
                    end
                    
                    -- TODO: remove button from list
                    g_CurrentMap:centerOnRoom('mm')
                    
                    -- launch hurray sequence (note: doesn't work for these yet)
                    g_GUI:ShowPremiumPurchaseSequence("prop", self.rItemDef)
                else                
                    g_MiddleManager:addConsumableItemToInventory(self.sItemKey)
                    g_TodoList:onConsumableItemPurchased()
                end

                rConfirmScreen:setVisible(false)
                g_GUI:SetPopupPane(nil)
            end
            
            -- used by SimpleConfirmScreenUI
            function rListener:onNoPressed(rConfirmScreen)
                rConfirmScreen:setVisible(false)
                g_GUI:SetPopupPane(nil)
            end
            
            rListener.sItemKey = self.sItemKey
            rListener.rItemDef = self.rItemDef
            
            local sMessage = g_DialogManager:getString("STXT138TEXT")..' ' .. g_DialogManager:getString(self.rItemDef.sName) .. "?"
            self.rConfirmScreen = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), sMessage, rListener, "STXT075TEXT", "STXT076TEXT")
            self.rConfirmScreen:setLoc(240, -160)
            self.rConfirmScreen:setVisible(true)
    
            g_GUI:SetPopupPane(self.rConfirmScreen, true)
        end
    end

    function Ob:update()
        if self.rButton then
            self.rButton:update()
        end
    end
    
    function Ob:setEquipped(bEquipped)
        
    end

    return Ob
end

return ConsumablePurchaseButton