local CreditsUI = {}
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local util = require('DFUtil')

local kSCROLL_DURATION = 50
local kSCROLL_DIST = 1000

local tUITextureSpriteSheetInfo =
{
    MOAILogo = 'UI/hud2',
}

local creditBlocks = {
    {
        department = "Development Team of Justice",
        names = 
        {
            {title = "Project Lead & Programming", name = "Kee Chi"},
            {title = "Writer & Producer", name = "Gabe Miller"},
            {title = "Art Direction & Art", name = "Mark Hamer"},
            {title = "Interface Design", name = "Joe Kowalski"},
            {title = "Programming", name = "Ben Burbank, Matthew Franklin"},
            {title = "Art", name = "Say Oh"},
            {title = "Visual FX", name = "Panya Inversin"},
            {title = "Animation", name = "Ray Crook, Chris Schultz"},
        }
    },
    {
        department = "Development Team of Justice",
        names = 
        {
            {title = "Audio Direction & Original Music", name = "Brian Min"},
            {title = "Sound Design", name = "Brian Correia, Paul O'Rourke"},
            {title = "", name = "Camden Stoddard"},
            {title = "Lead Tester", name = "Daniel Pangelina"},
            {title = "Testing", name = "Tony Lo, Victor Romero"},
            {title = "Tools Programming", name = "Brandon Dillon, Paul Du Bois"},
            {title = "Additional Programming", name = "Patrick Hackett, Nathan Martz"},
        }
    },
    {
        department = "Double Fine Productions",
        names = 
        {
            {title = "Studio Creative Director", name = "Tim Schafer"},
            {title = "VP of Business Development", name = "Justin Bailey"},
            {title = "Director of Product Development", name = "Isa Anne Stamos"},
            {title = "Director of Technology", name = "Nathan Martz"},
            {title = "Operations Director", name = "Denise Gollaher"},
            {title = "Public Relations", name = "Greg Rice"},
            {title = "Community Manager", name = "Chris Remo"},
            {title = "Administration", name = "Laura O'Brien, Allison Thomas"},
            {title = "IT", name = "Brent Shinn, Justin Honneger"},
        }
    },
    {
        department = "Dracogen Inc.",
        names = 
        {
            {title = "Founder and CEO", name = "Steven Dengler"},
            {title = "Creative Consultants", name = "Beric Dengler"},
            {title = "Creative Consultants", name = "Raffaele Dengler"},
        }
    },
    {
        department = "Double Fine Would Like To Thank:",
        names = 
        {
            {name = "Jane Kenny Chi"},
            {name = "Katrina Tilds"},
            {name = "Michele and Ruby"},
            {name = "Priscilla Cook"},
            {name = "Jess Burbank"},
            {name = "Ruth, Violet, and Valencia"},
            {name = "Malena Annable"},
            {name = "Ron Gilbert"},
        }
    },
    {
        department = "Double Fine Would Like To Thank:",
        names = 
        {
            {name = "Robert Megone"},
            {name = "Elevated Play"},
            {name = "Will Wright"},
            {name = "Our Friends at Zipline Games"},
            {name = "Lucy Lathan & Cory Lewis"},
            {name = "Our friends at Firelight Technologies"},
            {name = "Everyone at Double Fine Productions"},
            {name = "and Steven Dengler!"},
        }
    },
    {
        department = "Dracogen Would Like To Thank:",
        names = 
        {
            {name = "Bruna Pace"},
            {name = "Raffaele Dengler"},
            {name = "Beric Dengler"},
            {name = "Double Fine"},
            {name = "Friends and Family"},
        }
    },
    {
        department = "Legal",
        names = 
        {
            {name = "Middle Manager of Justice, Double Fine Productions and the Double Fine logo are the exclusive trademarks of Double Fine Productions, Inc. All rights reserved."},
            {name = "The DRACOGEN word and logos are copyrighted trade marks of Dracogen Inc., all rights reserved worldwide."},
            {name = "Made with Moai\nwww.getmoai.com\nCopyright (c) 2010-2012 Zipline Games, Inc. All Rights Reserved."},
            {name = "Copyright (c) 2010, ParaType Ltd. (http://www.paratype.com/public), with Reserved Font Names 'PT Sans' and 'ParaType'. This Font Software is licensed under the SIL Open Font License, Version 1.1."},
        }
    },
}

local titleText, creditsText
local block = nil
--grab DF Dev Team Page 1
block = creditBlocks[1]
titleText = ""
creditsText = ""
for i, dev in ipairs(block.names) do
    titleText = titleText .. dev.title .. "\n"
    creditsText = creditsText .. dev.name .. "\n"
end

-- DF Dev Team Page 2
block = creditBlocks[2]
titleTextb = ""
creditsTextb = ""
for i, dev in ipairs(block.names) do
    titleTextb = titleTextb .. dev.title .. "\n"
    creditsTextb = creditsTextb .. dev.name .. "\n"
end

-- Double Fine
local titleText2, creditsText2
block = creditBlocks[3]
titleText2 = ""
creditsText2 = ""
for i, dev in ipairs(block.names) do
    titleText2 = titleText2 .. dev.title .. "\n"
    creditsText2 = creditsText2 .. dev.name .. "\n"
end

-- Dracogen
local titleText3, creditsText3
block = creditBlocks[4]
titleText3 = ""
creditsText3 = ""
for i, dev in ipairs(block.names) do
    titleText3 = titleText3 .. dev.title .. "\n"
    creditsText3 = creditsText3 .. dev.name .. "\n"
end

-- Double Fine Thanks PAGE 1
local creditsText4
block = creditBlocks[5]
creditsText4 = ""
for i, dev in ipairs(block.names) do
    creditsText4 = creditsText4 .. dev.name .. "\n"
end

-- Double Fine Thanks PAGE 2
local creditsText4b
block = creditBlocks[6]
creditsText4b = ""
for i, dev in ipairs(block.names) do
    creditsText4b = creditsText4b .. dev.name .. "\n"
end

-- Dracogen Thanks
local dracogenThanks
block = creditBlocks[7]
dracogenThanks = ""
for i, dev in ipairs(block.names) do
    dracogenThanks = dracogenThanks .. dev.name .. "\n"
end

-- Legal Text
local legalText
block = creditBlocks[8]
legalText = ""
for i, dev in ipairs(block.names) do
    legalText = legalText .. dev.name .. "\n\n"
end


function CreditsUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:onPaneClosed()
        self.rCreditsBGLogo:unloadSpriteSheet()
        self:unloadUISpriteSheets()        
    end

    function Ob:showCreditsBG()
        self.rBackground = MOAIProp2D.new()
        self.rBackground:setDeck(g_GUI:GetOnePixelDeck())
        self.rBackground:setLoc(0, 0)
        self.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
        self.rBackground:setColor(0, 0, 0, 1)
        self.rBackground:setLoc(0, 0)
        self:getRenderLayer():insertProp(self.rBackground)
    
        self.rScreen = self.rCreditsBGLogo
        self:getRenderLayer():insertProp(self.rScreen)
        self.rScreen:setLoc(Renderer.getUIWidth() / 2, -Renderer.getUIHeight() / 2)
    end

    function Ob:clearCreditsBG()
        self:getRenderLayer():removeProp(self.rScreen)
        self.bDracogenLogoShowing = false
    end

    function Ob:creditsThread()
        self:showCreditsBG()
        
        -- set all text to invisible
        self.rCreditsText:moveColor(0, 0, 0, -1, 0)
        self.rTitleText:moveColor(0, 0, 0, -1, 0)
        self.rCreditsTextb:moveColor(0, 0, 0, -1, 0)
        self.rTitleTextb:moveColor(0, 0, 0, -1, 0)
        self.rHeaderText:moveColor(0, 0, 0, -1, 0)
        self.rCreditsText2:moveColor(0, 0, 0, -1, 0)
        self.rTitleText2:moveColor(0, 0, 0, -1, 0)
        self.rHeaderText2:moveColor(0, 0, 0, -1, 0)
        self.rCreditsText3:moveColor(0, 0, 0, -1, 0)
        self.rTitleText3:moveColor(0, 0, 0, -1, 0)
        self.rHeaderText3:moveColor(0, 0, 0, -1, 0)
        self.rCreditsText4:moveColor(0, 0, 0, -1, 0)
        self.rCreditsText4b:moveColor(0, 0, 0, -1, 0)
        self.rHeaderText4:moveColor(0, 0, 0, -1, 0)
        self.rLegalText:moveColor(0, 0, 0, -1, 0)
        self.rDracogenThanks:moveColor(0, 0, 0, -1, 0)
        self.rThanksHeader:moveColor(0, 0, 0, -1, 0)
        
        --DF CREDITS PAGE 1
        
        -- show DF credits
        self.rCreditsText:moveColor(0, 0, 0, 1, 3)
        self.rTitleText:moveColor(0, 0, 0, 1, 3)
        self.rHeaderText:moveColor(0, 0, 0, 1, 3)
        
        self.rBackground:moveColor(0, 0, 0, -1, 1)
  
        util.sleep(1)
        self.bCanClose = true
        util.sleep(6)
        -- fade out DF credits
        self.rCreditsText:moveColor(0, 0, 0, -1, 3)
        self.rTitleText:moveColor(0, 0, 0, -1, 3)
        util.sleep(3)
        
        -- show DF PAGE 2 credits
        self.rCreditsTextb:moveColor(0, 0, 0, 1, 3)
        self.rTitleTextb:moveColor(0, 0, 0, 1, 3)
        self.rHeaderText:moveColor(0, 0, 0, 1, 3)
        
        self.rBackground:moveColor(0, 0, 0, -1, 1)
  
        util.sleep(1)
        self.bCanClose = true
        util.sleep(6)
        -- fade out DF PAGE 2 credits
        self.rCreditsTextb:moveColor(0, 0, 0, -1, 3)
        self.rTitleTextb:moveColor(0, 0, 0, -1, 3)
        self.rHeaderText:moveColor(0, 0, 0, -1, 3)
        
        --Double Fine
        util.sleep(3)
        self.rCreditsText2:moveColor(0, 0, 0, 1, 3)
        self.rTitleText2:moveColor(0, 0, 0, 1, 3)
        self.rHeaderText2:moveColor(0, 0, 0, 1, 3)
        util.sleep(6)        
        self.rCreditsText2:moveColor(0, 0, 0, -1, 3)
        self.rTitleText2:moveColor(0, 0, 0, -1, 3)
        self.rHeaderText2:moveColor(0, 0, 0, -1, 3)
        
        --dracogen 
        util.sleep(3)
        self.rCreditsText3:moveColor(0, 0, 0, 1, 3)
        self.rTitleText3:moveColor(0, 0, 0, 1, 3)
        self.rHeaderText3:moveColor(0, 0, 0, 1, 3)
        self.rDracogenThanks:moveColor(0, 0, 0, 1, 3)
        self.rThanksHeader:moveColor(0, 0, 0, 1, 3)
        util.sleep(6)        
        self.rCreditsText3:moveColor(0, 0, 0, -1, 3)
        self.rTitleText3:moveColor(0, 0, 0, -1, 3)
        self.rHeaderText3:moveColor(0, 0, 0, -1, 3)
        self.rDracogenThanks:moveColor(0, 0, 0, -1, 3)
        self.rThanksHeader:moveColor(0, 0, 0, -1, 3)
        
        --Double Fine Thanks PAGE 1
        util.sleep(3)
        self.rCreditsText4:moveColor(0, 0, 0, 1, 3)
        self.rHeaderText4:moveColor(0, 0, 0, 1, 3)
        util.sleep(6)        
        self.rCreditsText4:moveColor(0, 0, 0, -1, 3)
        
        --Double Fine Thanks PAGE 2
        util.sleep(3)
        self.rCreditsText4b:moveColor(0, 0, 0, 1, 3)
        util.sleep(6)        
        self.rCreditsText4b:moveColor(0, 0, 0, -1, 3)
        self.rHeaderText4:moveColor(0, 0, 0, -1, 3)
        
        --legal text
        util.sleep(3)  
        self.rMOAILogo:moveColor(1, 1, 1, 1, 3)
        self.rLegalText:moveColor(0, 0, 0, 1, 3)
        util.sleep(6)
        self.rMOAILogo:moveColor(-1, -1, -1, -1, 3)
        self.rLegalText:moveColor(0, 0, 0, -1, 3)
        util.sleep(6)
        self.rMOAILogo:setVisible(false)

        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IngameLoop', true)
        g_GUI:HideCreditsUI()
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            self.rCreditsThread = MOAIThread.new()
            self.rCreditsThread:run(self.creditsThread, self)
        else
            self:getRenderLayer():removeProp(self.rScreen)
        end
	end
	
	function Ob:onFinger(eventType, sx, sy)
		if eventType == MOAITouchSensor.TOUCH_UP then
            if self.bCanClose then
                g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IngameLoop', true)
                g_GUI:HideCreditsUI()
            end
		end
		return true
	end
	
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_EndCredits', true)

	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(0, 0, 0)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)    
    
    local nWidth = Renderer.getUIWidth()
    --PAGE 1 - DF
    
    local xOffset = (nWidth - 960) / 2
    
    Ob.rCreditsText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rCreditsText:setRect(0, 1000, 500, 0)
	Ob.rCreditsText:setLoc(480 + xOffset, -140)
    Ob.rCreditsText:setString(creditsText)
    Ob.rCreditsText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsText)
    
    Ob.rCreditsTextb = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rCreditsTextb:setRect(0, 1000, 500, 0)
	Ob.rCreditsTextb:setLoc(480, -140)
    Ob.rCreditsTextb:setString(creditsTextb)
    Ob.rCreditsTextb:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsTextb)
    
    Ob.rTitleText = g_GUI:CreateTextBox(30, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rTitleText:setRect(0, 1000, 500, 0)
	Ob.rTitleText:setLoc(-40 + xOffset, -140)
    Ob.rTitleText:setString(titleText)
    Ob.rTitleText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rTitleText)
    
    Ob.rTitleTextb = g_GUI:CreateTextBox(30, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rTitleTextb:setRect(0, 1000, 500, 0)
	Ob.rTitleTextb:setLoc(-40, -140)
    Ob.rTitleTextb:setString(titleTextb)
    Ob.rTitleTextb:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rTitleTextb)
    
    Ob.rHeaderText = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rHeaderText:setRect(0, 100, 500, 0)
	Ob.rHeaderText:setLoc(220 + xOffset, -60)
    Ob.rHeaderText:setString(creditBlocks[1].department)
    Ob.rHeaderText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rHeaderText)
    
    --PAGE 2 - DOUBLE FINE CREDITS
    Ob.rCreditsText2 = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rCreditsText2:setRect(0, 1000, 500, 0)
	Ob.rCreditsText2:setLoc(480 + xOffset, -140)
    Ob.rCreditsText2:setString(creditsText2)
    Ob.rCreditsText2:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsText2)
    
    Ob.rTitleText2 = g_GUI:CreateTextBox(30, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rTitleText2:setRect(0, 1000, 500, 0)
	Ob.rTitleText2:setLoc(-40 + xOffset, -140)
    Ob.rTitleText2:setString(titleText2)
    Ob.rTitleText2:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rTitleText2)
    
    Ob.rHeaderText2 = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rHeaderText2:setRect(0, 100, 500, 0)
	Ob.rHeaderText2:setLoc(220 + xOffset, -60)
    Ob.rHeaderText2:setString(creditBlocks[3].department)
    Ob.rHeaderText2:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rHeaderText2)
    
    --PAGE 3 - DRACOGEN INC.
    Ob.rCreditsText3 = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rCreditsText3:setRect(0, 1000, 500, 0)
	Ob.rCreditsText3:setLoc(500 + xOffset, -140)
    Ob.rCreditsText3:setString(creditsText3)
    Ob.rCreditsText3:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsText3)
    
    Ob.rTitleText3 = g_GUI:CreateTextBox(30, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rTitleText3:setRect(0, 1000, 500, 0)
	Ob.rTitleText3:setLoc(-20 + xOffset, -140)
    Ob.rTitleText3:setString(titleText3)
    Ob.rTitleText3:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rTitleText3)
    
    Ob.rHeaderText3 = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rHeaderText3:setRect(0, 100, 500, 0)
	Ob.rHeaderText3:setLoc(220 + xOffset, -60)
    Ob.rHeaderText3:setString(creditBlocks[4].department)
    Ob.rHeaderText3:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rHeaderText3)
    
    Ob.rThanksHeader = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rThanksHeader:setRect(0, 100, 800, 0)
	Ob.rThanksHeader:setLoc(80 + xOffset, -270)
    Ob.rThanksHeader:setString(creditBlocks[7].department)
    Ob.rThanksHeader:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rThanksHeader)
    
    Ob.rDracogenThanks = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rDracogenThanks:setRect(0, 1000, 500, 0)
	Ob.rDracogenThanks:setLoc(225 + xOffset, -340)
    Ob.rDracogenThanks:setString(dracogenThanks)
    Ob.rDracogenThanks:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rDracogenThanks)
    
    --PAGE 4 - SPECIAL THANKS
    Ob.rCreditsText4 = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rCreditsText4:setRect(0, 1000, 500, 0)
	Ob.rCreditsText4:setLoc(225 + xOffset, -140)
    Ob.rCreditsText4:setString(creditsText4)
    Ob.rCreditsText4:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsText4)
    
    Ob.rCreditsText4b = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rCreditsText4b:setRect(0, 1000, 500, 0)
	Ob.rCreditsText4b:setLoc(225 + xOffset, -140)
    Ob.rCreditsText4b:setString(creditsText4b)
    Ob.rCreditsText4b:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rCreditsText4b)
    
    Ob.rHeaderText4 = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rHeaderText4:setRect(0, 100, 800, 0)
	Ob.rHeaderText4:setLoc(80 + xOffset, -60)
    Ob.rHeaderText4:setString(creditBlocks[5].department)
    Ob.rHeaderText4:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rHeaderText4)
    
    --PAGE 5 - LEGAL
    Ob.rLegalText = g_GUI:CreateTextBox(25, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rLegalText:setRect(0, 1000, 900, 0)
	Ob.rLegalText:setLoc(35 + xOffset, -140)
    Ob.rLegalText:setString(legalText)
    Ob.rLegalText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rLegalText)
    
    Ob.bCanClose = false
    Ob.rCreditsBGLogo = SpriteSheet:new('UI/creditsBG', nil, 0, 0, 'center', 'center')

    Ob.rMOAILogo = Ob:getUITextureProp('MOAILogo')
    Ob.rMOAILogo:setLoc(450 + xOffset, -296)
    Ob.rMOAILogo:setScl(0.5)
    Ob.rMOAILogo:setColor(0, 0, 0, 0)
    Ob:addOverlayElement(Ob.rMOAILogo)

	return Ob
end

return CreditsUI