local UIElement = require('ui/UIElement')
local DFIAP = require('DFCommon.IAP')
local ShopButton = require('ui/ShopButton')
local PostIt = require('ui/PostIt')
local Renderer = require('Renderer')
local DFAnalytics = require('DFCommon.Analytics')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')

local CurrencyPurchaseButton = {}

local tUITextureSpriteSheetInfo =
{
shop_item = 'UI/shop',
button_buy = 'UI/shop',
icon_pp = 'UI/hud',
icon_cash = 'UI/hud',
icon_coins_xs = 'UI/shop',
icon_coins_m = 'UI/shop',
icon_coins_s = 'UI/shop',
icon_coins_l = 'UI/icons3',
icon_coins_xl = 'UI/icons3',
icon_pp_xs = 'UI/shop',
icon_pp_m = 'UI/shop',
icon_pp_s = 'UI/shop',
icon_pp_l = 'UI/shop',
icon_prop_ppplant = 'UI/icons4',
icon_prop_coindoubler = 'UI/icons4',
shop_itembar = 'UI/shop',
}

function CurrencyPurchaseButton:new(rRenderLayer, rDef, sCurrencyKey)
	if not sCurrencyKey then
		print("ERROR: CurrencyPurchaseButton created without an Item")
		return nil
	end
    if not rDef then
		print("ERROR: CurrencyPurchaseButton created without an Item")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonPressed()
        self:buyCurrency()
    end

    function Ob:onButtonJustPressed(rButton)
        self:onButtonPressed()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rButton:onFinger(eventType, sx, sy, tProps)
		return true
	end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rButton:setVisible(bVisible)
	end

	function Ob:getYSize()
        return 220
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:canBuyCurrency()
        if not self.rDef.sAppleProductID and self.rDef.nPPPrice then
            if g_Game:getCurPP() < self.rDef.nPPPrice then
                return false
            end
        end
        return true
    end

    function Ob:buyCurrency()
        if self.rDef.sAppleProductID then
            local bSuccessfullyRequested = DFIAP.purchaseItem(self.rDef.sAppleProductID)
            -- maybe do some busywaiting here?
            if bSuccessfullyRequested then
                DFAnalytics.logEvent("buy_pppp", { type = self.rDef.sAppleProductID } )
                local rPurchaseWaitPane = PostIt:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT142TEXT"))
                rPurchaseWaitPane:setLoc(240, -160)
                g_GUI:SetPopupPane(rPurchaseWaitPane)
            end
        elseif self:canBuyCurrency() then
            -- show are you sure pop
            rListener = {}
            
            -- used by SimpleConfirmScreenUI
            function rListener:onYesPressed(rConfirmScreen)
                g_Game:removePP(self.rDef.nPPPrice)
                g_Game:addCash(self.rDef.nCashToGive)
                DFAnalytics.logEvent("buy_cash", { type = self.sCurrencyKey } )
                DFAnalytics.logEvent("ppp_spnt", { type = "cash", amount = self.rDef.nPPPrice })
                DFAnalytics.incrStat("ppp_spnt_cash", self.rDef.nPPPrice)
                DFAnalytics.incrStat('csh_ernd_pppp', self.rDef.nCashToGive)
                g_Game:saveGame()
                
                rConfirmScreen:setVisible(false)
                g_GUI:SetPopupPane(nil)
            end
            
            -- used by SimpleConfirmScreenUI
            function rListener:onNoPressed(rConfirmScreen)
                rConfirmScreen:setVisible(false)
                g_GUI:SetPopupPane(nil)
            end
            
            rListener.rDef = self.rDef
            rListener.sCurrencyKey = self.sCurrencyKey
            
            local sMessage = g_DialogManager:getString("STXT138TEXT")..' ' .. self.rDef.nCashToGive .. " Coins?"
            self.rConfirmScreen = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), sMessage, rListener, "STXT075TEXT", "STXT076TEXT")
            self.rConfirmScreen:setLoc(240, -160)
            self.rConfirmScreen:setVisible(true)
    
            g_GUI:SetPopupPane(self.rConfirmScreen, true)
        end
    end

    function Ob:update()
        if self.rButton then
            self.rButton:update()
        end
    end    
    
    Ob.sCurrencyKey = sCurrencyKey
	Ob.rDef = rDef
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

    Ob.rItemBar = Ob:getUITextureProp('shop_itembar')
    Ob.rItemBar:setLoc(15, -152)
    Ob:addElement(Ob.rItemBar)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(g_DialogManager:getString(Ob.rDef.sName))
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(150, -10)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

	Ob.rDescriptionField = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rDescriptionField:setRect(0, 200, 295, 0)
	Ob.rDescriptionField:setLoc(220, -55)
    if Ob.rDef.nCashToGive then
        local tColor = g_Game:getCashColor()   
        Ob.rDescriptionField:setString(tostring(Ob.rDef.nCashToGive))
        Ob.rDescriptionField:setColor(tColor[1], tColor[2], tColor[3])
    elseif Ob.rDef.nPPToGive then
        local tColor = g_Game:getPPColor()   
        Ob.rDescriptionField:setString(tostring(Ob.rDef.nPPToGive))
        Ob.rDescriptionField:setColor(tColor[1], tColor[2], tColor[3])
    else
        Ob.rItemDescription = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY)
        Ob.rItemDescription:setRect(0, 200, 295, 0)
        Ob.rItemDescription:setLoc(150, -50)
        Ob.rItemDescription:setString(g_DialogManager:getString(Ob.rDef.sDescription))
        Ob.rDescriptionField:setReveal(0)
        Ob:addElement(Ob.rItemDescription)
    end
    
	Ob:addElement(Ob.rDescriptionField)	

    if Ob.rDef.nPPToGive or Ob.rDef.nCashToGive then
        local sRewardIconName = 'icon_pp'
        if Ob.rDef.nCashToGive then
            sRewardIconName = 'icon_cash'
        end
        Ob.rItemIcon = Ob:getUITextureProp(sRewardIconName)
        Ob.rItemIcon:setLoc(150, -50)
        Ob:addElement(Ob.rItemIcon)
    end

    if Ob.rDef.nPPPrice then
        Ob.rCostIcon = Ob:getUITextureProp('icon_pp')
        Ob.rCostIcon:setLoc(20, -154)
        Ob:addElement(Ob.rCostIcon)
    end
    
    Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
    if Ob.rDef.nPPPrice then
        local tColor = g_Game:getPPColor()   
        Ob.rCostField:setString(tostring(Ob.rDef.nPPPrice))
        Ob.rCostField:setColor(tColor[1], tColor[2], tColor[3])
        Ob.rCostField:setLoc(390, -156)
    else           
        Ob.rCostField:setString(Ob.rDef.localizedPrice)
        Ob.rCostField:setColor(0, 0, 0)
        Ob.rCostField:setLoc(340, -156)
    end
    Ob:addElement(Ob.rCostField)
    
    Ob.rButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, nil, Ob.rDef.nPPPrice, { .57, .57, .57 })
    Ob.rButton:setLoc(306, -152)
    Ob.rButton:setParent(Ob)
    
    if Ob.rDef.iconName then
        Ob.rIcon = Ob:getUITextureProp(Ob.rDef.iconName)
        if Ob.rIcon then
            Ob.rIcon:setLoc(16, -16)
            Ob:addElement(Ob.rIcon)
        end
    end

    return Ob
end

return CurrencyPurchaseButton