local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local Button = require('ui/Button')

local tUITextureSpriteSheetInfo =
{
    popup_button = 'UI/shopheropack',
}

local DebugDisplayUI = {}

function type_name(o)           
    local typeName = "Unknown"

    if type(o) then
        typeName = type(o)
    end
    
    if typeName == "table" or typeName == "userdata" then
        if o.__typename then
            typeName = o.__typename
        elseif o.getClassName then
            typeName = o:getClassName()
        end
    end
            
	return typeName
end

function DebugDisplayUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:update()
        while true do
            local currentAlloc, maxAlloc = MOAIFmodEventMgr.getMemoryStats()
            self.rDebugProfileReportProp:setUserMemory(currentAlloc)
            
            coroutine.yield()
        end
    end
    
    function Ob:showLuaDebugMetrics(sCountMin)
        print("\n\n\n\n\n\n")
    
        if not sCountMin then
            sCountMin = 2
        end
    
        local function count_all(f)
            local seen = {}
            local count_table
            count_table = function(t)
                if seen[t] then return end
                f(t)
                seen[t] = true
                for k,v in pairs(t) do
                    if type(v) == "table" then
                        count_table(v)
                    elseif type(v) == "userdata" then
                        f(v)
                    end
                end
            end
            count_table(_G)
        end
        
        local function type_count()
            local counts = {}
            local enumerate = function (o)
                local t = type_name(o)
                counts[t] = (counts[t] or 0) + 1
            end
            count_all(enumerate)
            return counts
        end
               
        local types = type_count()
        
        if not self.startTypes then
            self.startTypes = {}
        end
        
        local sortedTypes = { }
        
        local function gt(a, b) return a.count > b.count end
        
        for k, v in pairs(types) do 
            table.insert(sortedTypes, { type = k, count = v }) 
        end
        table.sort(sortedTypes, gt)
        
        for k, v in ipairs(sortedTypes) do
            if v.count >= sCountMin then
                sDelta = ""

                if not self.startTypes[v.type] then
                    self.startTypes[v.type] = v.count
                end
                
                if self.prevTypes then
                    local prevValue = self.prevTypes[v.type]

                    if prevValue then
                        local delta = v.count - prevValue
                        if delta > 0 then
                            sDelta = "*** ( +" .. tostring(delta) .. " )"
                        elseif delta < 0 then
                            sDelta = "*** ( " .. tostring(delta) .. " )"
                        end
                    end
                end
                    
                print("   - type " .. v.type .. ": " .. v.count .. " " .. sDelta .. " ( initial=" .. self.startTypes[v.type] .. " )")
            end
        end
        
        self.prevTypes = types

        
        print("\n\n\n\n\n\n")
    end
    
	function Ob:setVisible(bVisible)
        if bVisible then
            Ob.rDebugProfileReportProp:setFont(g_GUI:getFont('default'))
            Ob.rDebugProfileReportProp:setFontSize(14)
        
            Renderer.getRenderLayer('ProfileOverlay'):insertProp(Ob.rDebugProfileReportProp)
        else
            Renderer.getRenderLayer('ProfileOverlay'):removeProp(Ob.rDebugProfileReportProp)
        end
        --self:setElementsVisible(bVisible)
	end
    
    Ob.rDebugProfileReportProp = MOAIProfileReportBox.new()
    Ob.rDebugProfileReportProp:setFont(g_GUI:getFont('default'))
    Ob.rDebugProfileReportProp:setFontSize(14)
    Ob.rDebugProfileReportProp:setRect(50, 50, Renderer.getUIWidth(), Renderer.getUIHeight())    

    Ob.rThread = MOAIThread.new()
    Ob.rThread:run(Ob.update, Ob)   
    
    return Ob
end

return DebugDisplayUI