local Renderer = require('Renderer')
local util = require('DFUtil')
local UIElement = require('ui/UIElement')

local DialogUI = {}

local kDEFAULT_TEXT_SPEED = 40

function DialogUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), 140)
	Ob.rBackground:setColor(0, 0, 0)
	Ob:addElement(Ob.rBackground)
    
    Ob.rPortrait = nil

    Ob.rNameText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
    Ob.rNameText:setLoc(403, -14)
    Ob.rNameText:setColor(0.5, 0.5, 0.5)
	Ob:addElement(Ob.rNameText)

    Ob.rText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
    Ob.rText:setLoc(102, -40)
    Ob.rText:setRect(0, 200, Renderer.getUIWidth() - 120, 0)
    Ob.rText:setColor(1, 1, 1)
	Ob:addElement(Ob.rText)

    Ob.bButtonPressed = false
    
    function Ob:displayLine(rCharacter, sText, bBlocking, bWaitForInput)
        self.bButtonPressed = false
        if rCharacter then
            self.rNameText:setString(g_DialogManager:getString(rCharacter:getName()))
        else
            self.rNameText:setString("")
        end

        local rRenderLayer = self:getRenderLayer()
        if self.rPortrait then
            rRenderLayer:removeProp(rRenderLayer)
        end
        self.rPortrait = rCharacter:getPortraitProp()
        if self.rPortrait then
            self.rPortrait:setScl(0.65)
            self.rPortrait:setLoc(10, -16)
            self:addElement(self.rPortrait)
            rRenderLayer:insertProp(self.rPortrait)
        end    

        self.rText:setString(sText)
        self.rText:setSpeed(kDEFAULT_TEXT_SPEED)
        self.rSpoolAction = self.rText:spool()
        if self.rTalkSound then
            self.rTalkSound:stop()
        end
        self.rTalkSound = g_Audio:playCue('MMOJ_Audio/UI/MissionDialogue_Lp', true)
        if bBlocking then -- must be calling from a coroutine
            while not self.rSpoolAction:isDone() do
                coroutine.yield()
            end
            if self.rTalkSound then
                self.rTalkSound:stop()
            end
            if bWaitForInput then
                while not self.bButtonPressed do
                    coroutine.yield()
                end
                self.bButtonPressed = false
            end
        else
            while not self.rSpoolAction:isDone() do
                coroutine.yield()
            end
            if self.rTalkSound then
                self.rTalkSound:stop()
            end
        end
    end

	-- functions
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rPortrait then
            self.rPortrait:setVisible(bVisible)
        end
        if not bVisible then
            if self.rTalkSound then
                self.rTalkSound:stop()
            end
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy)
        if eventType == MOAITouchSensor.TOUCH_DOWN then
            if self.rSpoolAction then
                if not self.rSpoolAction:isDone() then
                    self.rText:setSpeed(999999)
                else
                    self.bButtonPressed = true
                end
            end
        end
    end

    function Ob:onViewportResized()
        if self.rBackground then
        	self.rBackground:setLoc(0, 0)
            self.rBackground:setScl(Renderer.getUIWidth(), 140)
        end
        if self.rText then
            self.rText:setRect(0, 200, Renderer.getUIWidth() - 120, 0)
        end
    end

    return Ob
end

return DialogUI
