local Character = require('Character')
local UIElement = require('ui/UIElement')
local EnemyDetailButton = {}

local tUITextureSpriteSheetInfo =
{
    icon_power = 'UI/hud',
    icon_maxhp = 'UI/hud',
}

function EnemyDetailButton:new(rRenderLayer, rDef, nNum)
	if not rDef then
		print("ERROR: EnemyDetailButton created without a def")
		return nil
	end

	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)       
	end

	function Ob:getYSize()
		return 130
	end

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rDef = rDef
    Ob.rParentUI = nil
    Ob.nNum = nNum

    if Ob.rDef.tPortraitPaths then 
        tUITextureSpriteSheetInfo[Ob.rDef.tPortraitIndex.default] = Ob.rDef.tPortraitPaths.default
        Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    end

    Ob.rNameLine1 = MOAIProp2D.new()
    Ob.rNameLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rNameLine1:setLoc(166, -56)
    Ob.rNameLine1:setScl(264, 2)
    Ob.rNameLine1:setColor(0, 0, 0)
    Ob:addElement(Ob.rNameLine1)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(166, -36)
    Ob.rLine1:setScl(2, 20)
    Ob.rLine1:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(428, -36)
    Ob.rLine2:setScl(2, 20)
    Ob.rLine2:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(166, -94)
    Ob.rLine3:setScl(2, 20)
    Ob.rLine3:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine3)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(276, -94)
    Ob.rLine4:setScl(2, 20)
    Ob.rLine4:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine4)

    Ob.rPowerLine1 = MOAIProp2D.new()
    Ob.rPowerLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPowerLine1:setLoc(166, -114)
    Ob.rPowerLine1:setScl(112, 2)
    Ob.rPowerLine1:setColor(0, 0, 0)
    Ob:addElement(Ob.rPowerLine1)

    Ob.rLine5 = MOAIProp2D.new()
    Ob.rLine5:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine5:setLoc(284, -94)
    Ob.rLine5:setScl(2, 20)
    Ob.rLine5:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine5)

    Ob.rLine6 = MOAIProp2D.new()
    Ob.rLine6:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine6:setLoc(428, -94)
    Ob.rLine6:setScl(2, 20)
    Ob.rLine6:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine6)

    Ob.rHealthLine = MOAIProp2D.new()
    Ob.rHealthLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rHealthLine:setLoc(284, -114)
    Ob.rHealthLine:setScl(146, 2)
    Ob.rHealthLine:setColor(0, 0, 0)
    Ob:addElement(Ob.rHealthLine)

    --enemy name
    Ob.rNamefield = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
    Ob.rNamefield:setString(g_DialogManager:getString(rDef.sName))
    Ob.rNamefield:setRect(0, 80, 300, 0)
    Ob.rNamefield:setLoc(172, -20)
    Ob:addElement(Ob.rNamefield)

    Ob.rPaneLine = MOAIProp2D.new()
    Ob.rPaneLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPaneLine:setLoc(0, -124)
    Ob.rPaneLine:setScl(436, 4)
    Ob.rPaneLine:setColor(0, 0, 0)
    Ob:addElement(Ob.rPaneLine)    

	-- add the stat field
    Ob.rPowerIcon = Ob:getUITextureProp('icon_power')
    Ob.rPowerIcon:setLoc(163, -60)
    Ob:addElement(Ob.rPowerIcon)

    Ob.rPowerField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
    Ob.rPowerField:setString(tostring(rDef.tInitialStats[1].nPower))
    Ob.rPowerField:setLoc(221, -80)
    Ob.rPowerField:setRect(0, 80, 300, 0)
    Ob:addElement(Ob.rPowerField)

    if nNum > 1 then        
        Ob.rNumField1 = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'bangers')
        Ob.rNumField1:setString("x")
        Ob.rNumField1:setLoc(120, -46)
        Ob.rNumField1:setRect(0, 80, 300, 0)
        Ob:addElement(Ob.rNumField1)

        Ob.rNumField2 = g_GUI:CreateTextBox(40, MOAITextBox.LEFT_JUSTIFY, 'bangers')
        Ob.rNumField2:setString(tostring(nNum))
        Ob.rNumField2:setLoc(138, -34)
        Ob.rNumField2:setRect(0, 80, 300, 0)
        Ob:addElement(Ob.rNumField2)
    end

    Ob.rMaxHPIcon = Ob:getUITextureProp('icon_maxhp')
    Ob.rMaxHPIcon:setLoc(276, -60)
    Ob:addElement(Ob.rMaxHPIcon)

    Ob.rMaxHPField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
    Ob.rMaxHPField:setString(tostring(rDef.tInitialStats[1].nMaxHP))
    Ob.rMaxHPField:setLoc(332, -80)
    Ob.rMaxHPField:setRect(0, 80, 300, 0)
    Ob:addElement(Ob.rMaxHPField)

    Ob.rFaceIcon = Ob:getUITextureProp(Ob.rDef.tPortraitIndex.default)
    if Ob.rFaceIcon then
        Ob.rFaceIcon:setLoc(11, -7)
        Ob.rFaceIcon:setScl(0.84, 0.84)
        Ob:addElement(Ob.rFaceIcon)
    end

	return Ob
end

return EnemyDetailButton