local Item = require('Item')
local UIElement = require('ui/UIElement')
local ShopButton = require('ui/ShopButton')
local EquipmentDetailButton = {}

local tUITextureSpriteSheetInfo =
{
shop_equipment_expanded = 'UI/shop',
button_buy = 'UI/shop',
icon_cash = 'UI/hud',
lock_research = 'UI/shop',
activity_sheet_research = 'UI/hud',
icon_new = 'UI/hud2',
}

function EquipmentDetailButton:new(rRenderLayer, sItemKey, bIsResearchButton, bSkipResearchCheck)
	if not sItemKey then
		print("ERROR: EquipmentDetailButton created without an Item")
		return nil
	end

    local rDef = Item.getEquipmentDef(sItemKey)
    if not rDef then
		print("ERROR: EquipmentDetailButton created without an Item")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if self.rParentUI then
                self.rParentUI:handleButtonPress(self, eventType)
            end
            self:onButtonPressed()
        end
    end


	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.onFingerCallBack then
            self:onFingerCallBack(eventType, sx, sy, tProps)
        end
        if self.rButton then
            self.rButton:onFinger(eventType, sx, sy, tProps)
        end

		return true
	end

	function Ob:setVisible(bVisible)
        if Ob.bIsVisible ~= bVisible then
            Ob.bIsVisible = bVisible

            local bIsNew = g_MiddleManager:isItemNewInShop(self.sItemKey)
            
            if bIsNew then
                if not self.rNewIcon then
                    self.rNewIcon = self:getUITextureProp('icon_new')
                    self.rNewIcon:setLoc(56, -68)
                    self.rNewIcon:setScl(0.7, 0.7)
                    self:addElement(self.rNewIcon)
                end
            end
            
            self:setElementsVisible(bVisible)
            
            if self.rNewIcon then
                self.rNewIcon:setVisible(bIsNew and bVisible)
            end
            
            if self.rButton then
                self.rButton:setVisible(bVisible)
            end
        end
	end

	function Ob:getYSize()
        return 310
	end

    function Ob:getButton()
        return self.rButton
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rButton then
            self.rButton:onPaneClosed()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sItemKey = sItemKey
	Ob.rItemDef = rDef
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.bResearched = g_MiddleManager:isResearched('equipment', sItemKey)

	Ob.rBackGround = Ob:getUITextureProp('shop_equipment_expanded')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(g_DialogManager:getString(Ob.rItemDef.sName))
	Ob.rNameField:setLoc(420, -10)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

	Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rItemDef.sDescription))
    Ob.rDescriptionField:setRect(0, 300, 450, 0)
	Ob.rDescriptionField:setLoc(16, -130)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	
    
    if Ob.rItemDef.sStat then
        Ob.rStatField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
        Ob.rStatField:setString(g_DialogManager:getString(Ob.rItemDef.sStat))
        Ob.rStatField:setRect(0, 300, 450, 0)
        Ob.rStatField:setLoc(16, -220)
        Ob.rStatField:setColor(0, .5, 0)
        Ob:addElement(Ob.rStatField)	
    end
    
    if Ob.rItemDef.iconName and Ob.rItemDef.sSpriteSheet then
        local tInfo = {}
        tInfo[Ob.rItemDef.iconName] = Ob.rItemDef.sSpriteSheet
        Ob:loadUISpriteSheets(tInfo)
        
        Ob.rIcon = Ob:getUITextureProp(Ob.rItemDef.iconName)
        if Ob.rIcon then
            Ob.rIcon:setLoc(16, -16)
            Ob.rIcon:setScl(0.76, 0.76)
            Ob:addElement(Ob.rIcon)
        end
    end

    if Ob.bResearched or bSkipResearchCheck then
        Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
        Ob.rCostIcon:setLoc(20, -252)
        Ob:addElement(Ob.rCostIcon)
        
        local tCashColor = g_Game:getCashColor()
        Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rCostField:setString(tostring(Ob.rItemDef.nCost))
        Ob.rCostField:setLoc(384, -254)
        Ob.rCostField:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
        Ob:addElement(Ob.rCostField)

        Ob.rButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.rItemDef.nCost, nil, { .57, .57, .57 })
        Ob.rButton:setLoc(306, -248)
        Ob.rButton:setParent(Ob)
    else
        Ob.rLockIcon = Ob:getUITextureProp('lock_research')
        if Ob.rLockIcon then
            Ob.rLockIcon:setLoc(10, -10)
            Ob.rLockIcon:setScl(0.82)
            Ob:addElement(Ob.rLockIcon)
        end
    end

    local nPowerMod = 0
    local nHealthMod = 0
    local nShieldMod = 0
    local nAwarenessMod = 0
    if Ob.rItemDef.tStatModifiers then
        for i, statMod in ipairs(Ob.rItemDef.tStatModifiers) do
            if statMod.statName == 'nPower' then
                nPowerMod = statMod.statAmount
            elseif statMod.statName == 'nMaxHP' then
                nHealthMod = statMod.statAmount
            elseif statMod.statName == 'nShield' then
                nShieldMod = statMod.statAmount
            elseif statMod.statName == 'nAwareness' then
                nAwarenessMod = statMod.statAmount
            end
        end
    end
    if nPowerMod ~= 0 then
        Ob.rPowerModText = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rPowerModText:setLoc(158, -80)
        if nPowerMod > 0 then
            Ob.rPowerModText:setString('+'..tostring(nPowerMod))
            Ob.rPowerModText:setColor(0, 0.5, 0)
        else
            Ob.rPowerModText:setString(tostring(nPowerMod))
            Ob.rPowerModText:setColor(0.5, 0, 0)
        end
        Ob:addElement(Ob.rPowerModText)
    end
    if nHealthMod ~= 0 then
        Ob.rHealthModText = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rHealthModText:setLoc(248, -80)
        if nHealthMod > 0 then
            Ob.rHealthModText:setString('+'..tostring(nHealthMod))
            Ob.rHealthModText:setColor(0, 0.5, 0)
        else
            Ob.rHealthModText:setString(tostring(nHealthMod))
            Ob.rHealthModText:setColor(0.5, 0, 0)
        end
        Ob:addElement(Ob.rHealthModText)
    end
    if nShieldMod ~= 0 then
        Ob.rShieldModText = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rShieldModText:setLoc(334, -80)
        if nShieldMod > 0 then
            Ob.rShieldModText:setString('+'..tostring(nShieldMod))
            Ob.rShieldModText:setColor(0, 0.5, 0)
        else
            Ob.rShieldModText:setString(tostring(nShieldMod))
            Ob.rShieldModText:setColor(0.5, 0, 0)
        end
        Ob:addElement(Ob.rShieldModText)
    end
    if nAwarenessMod ~= 0 then
        Ob.rAwarenessModText = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rAwarenessModText:setLoc(420, -80)
        if nAwarenessMod > 0 then
            Ob.rAwarenessModText:setString('+'..tostring(nAwarenessMod))
            Ob.rAwarenessModText:setColor(0, 0.5, 0)
        else
            Ob.rAwarenessModText:setString(tostring(nAwarenessMod))
            Ob.rAwarenessModText:setColor(0.5, 0, 0)
        end
        Ob:addElement(Ob.rAwarenessModText)
    end

    return Ob
end

return EquipmentDetailButton