local Item = require('Item')
local UIElement = require('ui/UIElement')
local EquipmentDetailButton = require('ui/EquipmentDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local Button = require('ui/Button')
local DFAnalytics = require('DFCommon.Analytics')
local EquipmentInventoryButton = {}

local tUITextureSpriteSheetInfo =
{
numberbg = 'UI/shop',
button_buy = 'UI/shop',
}

function EquipmentInventoryButton:new(rRenderLayer, sItemKey, rEquipmentPane)
	if not sItemKey then
		print("ERROR: EquipmentInventoryButton created without an Item")
		return nil
	end
    local rDef = Item.getEquipmentDef(sItemKey)
    if not rDef then
		print("ERROR: EquipmentInventoryButton created without an Item")
		return nil
    end

	local Ob = EquipmentDetailButton:new(rRenderLayer, sItemKey, nil, true)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rButton then
            self.rButton:setVisible(bVisible)
        end
        self.rSellButton:setVisible(bVisible)
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            self:equipItem(true)
        elseif rButton == self.rSellButton then
            if self.rItem then
                local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT098TEXT")..' '..g_DialogManager:getString(self.rDef.sName)..' '..g_DialogManager:getString("STXT099TEXT").." "..self.rItem:getSellPrice().."?", self, "STXT075TEXT", "STXT076TEXT")
                rConfirmPane:setLoc(240, -160)
                g_GUI:SetPopupPane(rConfirmPane)
            end
        end
    end

    function Ob:onButtonPressed()
        self:equipItem(true)
    end

    function Ob:onFingerCallBack(eventType, sx, sy, tProps)
        self.rSellButton:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:sellItem()
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:sellItem()
        if self.rItem then
            g_Audio:playCue('MMOJ_Audio/UI/BuyButton')
            g_MiddleManager:sellEquipment(self.sItemKey, self.rItem:getSellPrice())
            self:update()
        end
    end

    function Ob:equipItem(bEquip)
        if self.rItem then
            if self.rEquipmentPane then
                local rHeroSelectorUI = self.rEquipmentPane:getHeroSelectorUI()
                if rHeroSelectorUI then
                    local rHero = rHeroSelectorUI:getSelectedHero()
                    if rHero and rHero:hasAvailableSlots() then
                        g_Audio:playCue('MMOJ_Audio/UI/Equip_A')
                        g_MiddleManager:giveEquipmentToHero(self.sItemKey, rHero)
                        
                        DFAnalytics.logEvent("eqp_eqpd", { type = self.sItemKey })
                        self:update()
                    end
                end
            end
        end
    end

    function Ob:update()
        if self.rItem then
            local nNumItems = g_MiddleManager:getNumEquipmentInInventory(self.sItemKey)
            if nNumItems > 0 then
                self:setVisible(true)
                self.bHideFromList = false
                if nNumItems > 1 then                  
                    self.rNumberBG:setVisible(true)
                    self.rNumText:revealAll()                   
                else
                    self.rNumberBG:setVisible(true)
                    self.rNumText:revealAll()
                end
                self.rNumText:setString(tostring(nNumItems))                    
            else
                self:setVisible(false)
                self.bHideFromList = true
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rDef = rDef
    Ob.sItemKey = sItemKey

    if rDef then
        local constructor = Item.getConstructor(rDef.sItemType)
        if constructor then
            Ob.rItem = constructor(Item, rDef, sItemKey)
        end
    end

    Ob.rEquipmentPane = rEquipmentPane
    if Ob.rCostField then
        Ob.rCostField:setReveal(0)
    end
    if Ob.rCostIcon then
        Ob.rCostIcon:setVisible(false)
    end
    local rButton = Ob:getButton()
    if rButton then
        rButton:setText("STXT100TEXT")
    end

    Ob.rNumberBG = Ob:getUITextureProp('numberbg')
    Ob.rNumberBG:setScl(1.2)
    Ob.rNumberBG:setLoc(70, -90)
    Ob:addElement(Ob.rNumberBG)
    
    Ob.rNumText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rNumText:setString("1")
    Ob.rNumText:setRect(-100, 80, 100, 0)
    Ob.rNumText:setColor(1, 1, 1)
    Ob.rNumText:setLoc(90, -87)
    Ob:addElement(Ob.rNumText)

    Ob.rSellButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { 1, 0, 0 }, nil, true, { 0, -50 }, "STXT101TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rSellButton:setLoc(16, -248)
    Ob.rSellButton:setParent(Ob)

    Ob:update()
    
    return Ob
end

return EquipmentInventoryButton