local util = require('DFUtil')
local EquipmentInventoryButton = require('ui/EquipmentInventoryButton')
local ConsumableInventoryButton = require('ui/ConsumableInventoryButton')
local ListUI = require('ui/ListUI')
local UIElement = require('ui/UIElement')
local Item = require('Item')

local EquipmentInventoryUI = {}

function EquipmentInventoryUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:getElementsMaxPriority()
        local nMaxPri = 0
        for i, rButton in ipairs(self.tEquipmentButtons) do
            local nPri = rButton:getElementsMaxPriority()
            if nPri > nMaxPri then
                nMaxPri = nPri
            end
        end
        return nMaxPri
    end

    function Ob:setHeroSelectorUI(rHeroSelectorUI)
        self.rHeroSelectorUI = rHeroSelectorUI
    end

    function Ob:getHeroSelectorUI()
        return self.rHeroSelectorUI
    end

    function Ob:onEquipped(bEquipped)
        if self.rHeroSelectorUI then
            self.rHeroSelectorUI:onEquipped(bEquipped)
        end
    end

    function Ob:onPaneClosed()
        self.rListUI:onPaneClosed()
    end

    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    function Ob:update()
        self.rListUI:recalcListPos()
        self.rListUI:update()
    end

    -- create the equipment buttons
    Ob.tEquipmentButtons = {}
    local tDefKeys = Item.getAllEquipmentDefKeys()
    if tDefKeys then
        for i, sDefKey in ipairs(tDefKeys) do
            local rNewButton = EquipmentInventoryButton:new(Ob.rRenderLayer, sDefKey, Ob)
            table.insert(Ob.tEquipmentButtons, rNewButton)
        end
    end
    local tConsumableInfos = g_MiddleManager:getAllConsumableItemInfos()
    tDefKeys = Item.getAllConsumableDefKeys()
    if tDefKeys then
        for i, sDefKey in ipairs(tDefKeys) do
            local rNewButton = ConsumableInventoryButton:new(Ob.rRenderLayer, sDefKey, Ob)
            table.insert(Ob.tEquipmentButtons, rNewButton)
        end
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tEquipmentButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

return EquipmentInventoryUI