local Item = require('Item')
local UIElement = require('ui/UIElement')
local EquipmentDetailButton = require('ui/EquipmentDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local DFAnalytics = require('DFCommon.Analytics')

local EquipmentPurchaseButton = {}

function EquipmentPurchaseButton:new(rRenderLayer, sItemKey)
	if not sItemKey then
		print("ERROR: EquipmentPurchaseButton created without an Item")
		return nil
	end
    local rDef = Item.getEquipmentDef(sItemKey)
    if not rDef then
		print("ERROR: EquipmentPurchaseButton created without an Item")
		return nil
    end

	local Ob = EquipmentDetailButton:new(rRenderLayer, sItemKey)

    function Ob:onButtonPressed()
        if self:canBuyItem() then
            local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT138TEXT")..' '..g_DialogManager:getString(self.rItemDef.sName).."?", self, "STXT075TEXT", "STXT076TEXT")
            rConfirmPane:setLoc(240, -160)
            g_GUI:SetPopupPane(rConfirmPane)
        end        
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:buyItem()
        -- should switch to the inventory pane
        local rPane = g_GUI:GetActivePane()
        if rPane and rPane.selectTab then
            rPane:selectTab('Inventory')
        end
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:canBuyItem()
        if self.rItemDef.nCost then
            if g_Game:getCurCash() < self.rItemDef.nCost then
                return false
            end
        end
        return true
    end

    function Ob:buyItem()
        if self:canBuyItem() then
            g_Game:removeCash(self.rItemDef.nCost)
            g_MiddleManager:addEquipmentToInventory(self.sItemKey)
            DFAnalytics.logEvent("buy_equp", { type = self.sItemKey } )
            DFAnalytics.incrStat("csh_spnt_equp", self.rItemDef.nCost)
            g_TodoList:onEquipmentPurchased()
            g_Game:saveGame()
        end
    end

    function Ob:setEquipped(bEquipped)
        
    end
    
    function Ob:update()
        if self.rButton then
            self.rButton:update()
        end
    end        

    return Ob
end

return EquipmentPurchaseButton