local Renderer = require('Renderer')
local EnemyDetailButton = require('ui/EnemyDetailButton')
local HeroPreEventButton = require('ui/HeroPreEventButton')
local util = require("DFUtil")
local Character = require('Character')
local HeroChooserUI = require('ui/HeroChooserUI')
local UIElement = require('ui/UIElement')
local ListUI = require('ui/ListUI')
local Button = require('ui/Button')
local EventDetailUI = {}
local EventHeroPane = {}
local EventEnemyPane = {}
local EventHeroSelectorUI = {}
local EventFirePane = {}
local EventBusRescuePane = {}
local AddHeroButton = {}

local tUITextureSpriteSheetInfo =
{
    button_back = 'UI/eventdetails',
    button_back_tap = 'UI/eventdetails',
    button_continue = 'UI/eventdetails',
    button_continue_tap = 'UI/eventdetails',
    stamp_approved = 'UI/eventdetails',
}

function AddHeroButton:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
	end

	function Ob:onFinger(eventType, sx, sy, tProps, bTouchMoved)
        if bTouchMoved then
            return true
        end
        local bButtonPressed = false
        if eventType == MOAITouchSensor.TOUCH_UP then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground then
                        if self.rParentUI then
                            self.rParentUI:handleButtonPress(self, eventType)
                        end        
                    end
                end
            end
        end

		return true
	end   

    function Ob:onPaneClosed()
    end

	function Ob:getYSize()
		return 260
	end

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)    
    Ob.rBackground:setScl(460, 220)
	Ob.rBackground:setColor(.25, .25, .24)
    Ob:addElement(Ob.rBackground)

    Ob.rAddHeroText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rAddHeroText:setString(string.upper(g_DialogManager:getString("STXT014TEXT")))
    Ob.rAddHeroText:setRect(-300, 80, 300, 0)
    Ob.rAddHeroText:setColor(.89, .87, .83)
    Ob.rAddHeroText:setLoc(230, -60)
    Ob:addElement(Ob.rAddHeroText)    

    if not g_Game:areHeroesAvailableToFight() then
        Ob.rAddHeroText:setString(string.upper(g_DialogManager:getString("STXT184TEXT")))   
    end

    return Ob
end

function EventHeroPane:new(rRenderLayer, rParentUI, rEvent)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:getAllHeroes()
        local tHeroes = {}
        return tHeroes
    end

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)   
        if self.rListUI then
            self.rListUI:setVisible(bVisible)
        end
    end
    
    function Ob:update()
        if self.rListUI then
            self.rListUI:update()
        end
        self.rEventHeroSelectorUI:update()
    end

    function Ob:onPaneClosed()
        if self.rListUI then
            self.rListUI:onPaneClosed()
        end
        self.rEventHeroSelectorUI:onPaneClosed()
    end

    function Ob:setEventHeroSelectorUIVisible(bVisible)
        self.rEventHeroSelectorUI:setVisible(bVisible)
        if bVisible then
            --[[
            if self.rEvent then
                local tHeroes = g_Game:getLastHeroSelection(self.rEvent:getEventType())
                if tHeroes then
                    for i, rHero in ipairs(tHeroes) do
                        if rHero:getCurHP() > 0 then
                            self:onHeroSelected(rHero)
                        end
                    end
                end
            end
            ]]--
            self:updateButtons()
        end        
    end

    function Ob:isHeroSelectorOpen()
        if self.rEventHeroSelectorUI and self.rEventHeroSelectorUI:isMaximized() then
            return true
        end
        return false
    end

    function Ob:getSelectableHeroes()
        local tHeroes = {}
        local tAssignedHeroes = self:getAllHeroes()
        for i, rHero in pairs(g_Game:getAllHeroes()) do
            local bAdd = true
            if rHero:isAssignedToEvent() then
                bAdd = false
            elseif rHero:getCurHP() == 0 then
                bAdd = false
            else
                for c, rExistHero in ipairs(tAssignedHeroes) do
                    if rHero == rExistHero then
                        bAdd = false
                        break
                    end
                end
            end
            if bAdd then
                table.insert(tHeroes, rHero)
            end
        end

        return tHeroes
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rEventHeroSelectorUI and self.rEventHeroSelectorUI:isMaximized() then
            self.rEventHeroSelectorUI:onFinger(eventType, sx, sy, tProps)
        else
            if self.rListUI then
                self.rListUI:onFinger(eventType, sx, sy, tProps)
            end
        end
        return false
    end

    function Ob:handleItemSelection(rButton)
        if rButton.hasXButtonPressed then
            if rButton:hasXButtonPressed() then
                g_Audio:playCue('MMOJ_Audio/UI/RemoveHero')
                self.rEvent:removeHeroFromEvent(rButton:getHero())
                self:updateButtons()
            end
        else
            if self:areHeroesAvailable() then
                g_Audio:playCue('MMOJ_Audio/UI/BaseFolder')
                self.rEventHeroSelectorUI:maximize(true)
            end
        end
    end

    function Ob:onHeroSelected(rHero)
        -- replace the old button
        if rHero then           
            self.rEvent:addHeroToEvent(rHero)
            self:updateButtons()
        end
        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
        self.rEventHeroSelectorUI:hide(true)
    end

    function Ob:areHeroesAvailable()
        local tHeroes = self:getSelectableHeroes()
        if table.getn(tHeroes) == 0 then
            return false
        end
        return true
    end

    function Ob:updateButtons()
        local tHeroes = self.rEvent:getAllHeroes()
        local nNumAssignedHeroes = table.getn(tHeroes)
        if nNumAssignedHeroes > 0 then
            self.rParentUI:showAttemptButton(true)
        else
            self.rParentUI:showAttemptButton(false)
        end
        local nNumHeroesToAdd = self.rEvent:getNumMaxHeroes()
        nNumHeroesToAdd = nNumHeroesToAdd - nNumAssignedHeroes
        local tButtons = {}
        for i, rHero in ipairs(tHeroes) do
            local rNewButton = HeroPreEventButton:new(self:getRenderLayer(), rHero, true)
            table.insert(tButtons, rNewButton)
        end
        if nNumHeroesToAdd > 0 then
            for i = 1, nNumHeroesToAdd do
                local rNewButton = AddHeroButton:new(self:getRenderLayer())
                table.insert(tButtons, rNewButton)
            end
        end
            
        if self.rListUI then
            self.rListUI:setVisible(false)
        end
        self.rListUI = ListUI:new(rRenderLayer, tButtons, self)
        self.rListUI:setParent(self)
        self.rListUI:setAnchorLoc(2, -12)
        self.rListUI:setVisible(true)
    end
    
    function Ob:preventCancel()
        for i,v in ipairs(self.rListUI:getItems()) do
--            v.rXButton:setVisible(false)
        end
    end

    Ob.rParentUI = rParentUI    
    Ob.rEvent = rEvent

    Ob.rEventHeroSelectorUI = HeroChooserUI:new(Renderer.getRenderLayer('UIOverlay'), Ob)

    return Ob
end

local tNonCombatUITextureSpriteSheetInfo =
{
    shop_item = 'UI/shop',
    icon_power = 'UI/hud',
    icon_int = 'UI/hud',
    event_stats = 'UI/eventdetails',
    map_icon_fire = 'UI/eventdetails',
    map_icon_runawaycar = 'UI/eventdetails',
}

function EventFirePane:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:update()
        local tAssignedHeroes = self.rEvent:getAllHeroes()
        local nHighestAwareness = 0
        if tAssignedHeroes then
            for i, rHero in ipairs(tAssignedHeroes) do
                local nAwareness = rHero:getStatAwareness()
                if nAwareness > nHighestAwareness then
                    nHighestAwareness = nAwareness
                end
            end
        end
        local nDesiredAwareness = 1
        local rDef = self.rEvent:getDef()
        if rDef.nDesiredAwareness and rDef.nDesiredAwareness > 0 then
            nDesiredAwareness = rDef.nDesiredAwareness
        end
        local nChance = nHighestAwareness / nDesiredAwareness
        if nChance > 1 then
            nChance = 1
        end
        nChance = math.floor(nChance * 100)
        self.rChanceField:setString(nChance.."%")
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob.rEvent = rEvent

    Ob:loadUISpriteSheets(tNonCombatUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('event_stats')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

 	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNameField:setString("FIRE RESCUE!!")
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(10, -10)
	Ob.rNameField:setColor(0, 0, 0)

    local rDef = rEvent:getDef()
    local nDesiredAwareness = 1
    if rDef.nDesiredAwareness then
        nDesiredAwareness = rDef.nDesiredAwareness
    end

	Ob.rRequirementField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rRequirementField:setString("RECOMMENDED:")
    Ob.rRequirementField:setRect(0, 80, 300, 0)
	Ob.rRequirementField:setLoc(10, -60)
	Ob.rRequirementField:setColor(0, 0, 0)
	Ob:addElement(Ob.rRequirementField)	

	Ob.rNumberField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNumberField:setString(tostring(nDesiredAwareness))
    Ob.rNumberField:setRect(0, 80, 300, 0)
	Ob.rNumberField:setLoc(60, -100)
	Ob.rNumberField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNumberField)	

	Ob.rStatIcon = Ob:getUITextureProp('icon_int')
	Ob.rStatIcon:setLoc(0, -84)
	Ob:addElement(Ob.rStatIcon)

	Ob.rIcon = Ob:getUITextureProp('map_icon_fire')
	Ob.rIcon:setLoc(310, -4)
	Ob:addElement(Ob.rIcon)

	Ob.rDescriptionField = g_GUI:CreateTextBox(14, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString("ODDS OF\nSUCCESS:")
    Ob.rDescriptionField:setRect(0, 100, 300, 0)
	Ob.rDescriptionField:setLoc(270, -66)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	

	Ob.rChanceField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rChanceField:setString("0%")
    Ob.rChanceField:setRect(0, 100, 300, 0)
	Ob.rChanceField:setLoc(270, -104)
	Ob.rChanceField:setColor(0, 0, 0)
	Ob:addElement(Ob.rChanceField)	

	Ob:addElement(Ob.rNameField)   
    
    return Ob
end

function EventBusRescuePane:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:update()
        local tAssignedHeroes = self.rEvent:getAllHeroes()
        local nHighestPower = 0
        if tAssignedHeroes then
            for i, rHero in ipairs(tAssignedHeroes) do
                local nPower = rHero:getStatPower()
                if nPower > nHighestPower then
                    nHighestPower = nPower
                end
            end
        end
        local nDesiredPower = 1
        local rDef = self.rEvent:getDef()
        if rDef.nDesiredPower and rDef.nDesiredPower > 0 then
            nDesiredPower = rDef.nDesiredPower
        end
        local nChance = nHighestPower / nDesiredPower
        if nChance > 1 then
            nChance = 1
        end
        nChance = math.floor(nChance * 100)
        self.rChanceField:setString(nChance.."%")
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob.rEvent = rEvent

    Ob:loadUISpriteSheets(tNonCombatUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('event_stats')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

 	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNameField:setString("BUS RESCUE!!")
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(10, -10)
	Ob.rNameField:setColor(0, 0, 0)

    local rDef = rEvent:getDef()
    local nDesiredPower = 1
    if rDef.nDesiredPower then
        nDesiredPower = rDef.nDesiredPower
    end
	Ob.rRequirementField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rRequirementField:setString("RECOMMENDED:")
    Ob.rRequirementField:setRect(0, 80, 300, 0)
	Ob.rRequirementField:setLoc(10, -60)
	Ob.rRequirementField:setColor(0, 0, 0)
	Ob:addElement(Ob.rRequirementField)	

	Ob.rNumberField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNumberField:setString(tostring(nDesiredPower))
    Ob.rNumberField:setRect(0, 80, 300, 0)
	Ob.rNumberField:setLoc(60, -100)
	Ob.rNumberField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNumberField)	

	Ob.rStatIcon = Ob:getUITextureProp('icon_power')
	Ob.rStatIcon:setLoc(0, -84)
	Ob:addElement(Ob.rStatIcon)

	Ob.rIcon = Ob:getUITextureProp('map_icon_runawaycar')
	Ob.rIcon:setLoc(310, -4)
	Ob:addElement(Ob.rIcon)

	Ob.rDescriptionField = g_GUI:CreateTextBox(14, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString("ODDS OF\nSUCCESS:")
    Ob.rDescriptionField:setRect(0, 100, 300, 0)
	Ob.rDescriptionField:setLoc(270, -66)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	

	Ob.rChanceField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
	Ob.rChanceField:setString("0%")
    Ob.rChanceField:setRect(0, 100, 300, 0)
	Ob.rChanceField:setLoc(270, -104)
	Ob.rChanceField:setColor(0, 0, 0)
	Ob:addElement(Ob.rChanceField)	

	Ob:addElement(Ob.rNameField)   
    
    return Ob
end

function EventEnemyPane:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)               
        self.rListUI:setVisible(bVisible)        
    end

    function Ob:onPaneClosed()
        self.rListUI:onPaneClosed()
    end

    function Ob:update()
        self.rOddsText:setString(self.rEvent:getOddsOfSuccess().."%")
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        --self.rListUI:onFinger(eventType, sx, sy, tProps)
        return true
    end

    Ob.rEvent = rEvent

    Ob.tButtons = {}
    local tCharInfos = {}    
    local rDef = rEvent:getDef()
    if rDef and rDef.tActors and rDef.tActors.Enemies then
        for actorName, tActorInfo in pairs(rDef.tActors.Enemies) do
            local rCharDef = Character.getCharDef(tActorInfo.sCharDefKey)
            if rCharDef then
                if not tCharInfos[tActorInfo.sCharDefKey] then
                    tCharInfos[tActorInfo.sCharDefKey] = { rDef = rCharDef, nNum = 0 }
                end
                tCharInfos[tActorInfo.sCharDefKey].nNum = tCharInfos[tActorInfo.sCharDefKey].nNum + 1
            end
        end
    end

    for i, rCharInfo in pairs(tCharInfos) do
        local rButton = EnemyDetailButton:new(rRenderLayer, rCharInfo.rDef, rCharInfo.nNum)
        table.insert(Ob.tButtons, rButton)
    end

	Ob.rOddsTitleText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
	Ob.rOddsTitleText:setString(g_DialogManager:getString("STXT170TEXT"))
    Ob.rOddsTitleText:setRect(0, 80, 300, 0)
	Ob.rOddsTitleText:setColor(0, 0, 0)
	Ob.rOddsTitleText:setLoc(10, -2)
    Ob:addElement(Ob.rOddsTitleText)

	Ob.rOddsText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'bangers')
	Ob.rOddsText:setString("100%")
    Ob.rOddsText:setRect(0, 80, 300, 0)
	Ob.rOddsText:setColor(0, 0, 0)
	Ob.rOddsText:setLoc(290, 0)
    Ob:addElement(Ob.rOddsText)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(270, 4)
    Ob.rLine1:setScl(4, 45)
    Ob.rLine1:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(430, 10)
    Ob.rLine2:setScl(4, 52)
    Ob.rLine2:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine2)

    Ob.rPaneLine = MOAIProp2D.new()
    Ob.rPaneLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPaneLine:setLoc(0, -40)
    Ob.rPaneLine:setScl(434, 4)
    Ob.rPaneLine:setColor(0, 0, 0)
    Ob:addElement(Ob.rPaneLine)

    Ob.rListUI = ListUI:new(rRenderLayer, Ob.tButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, -46)
    
    return Ob
end

function EventDetailUI:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)    

    function Ob:update()
        self.rHeroPane:update()
    
        if self.rFirePane then
            self.rFirePane:update()
        end
        if self.rBusPane then
            self.rBusPane:update()
        end
        if self.rEnemyPane then
            self.rEnemyPane:update()
        end
        if self:canAttemptEvent() then
            self:showAttemptButton(true)
        else
            self:showAttemptButton(false)
        end
    end

    function Ob:canAttemptEvent()
        local tHeroes = self.rEvent:getAllHeroes()
        if tHeroes and table.getn(tHeroes) > 0 then
            return true
        else
            return false
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rHeroPane:setVisible(bVisible)
        if self.rEnemyPane then
            self.rEnemyPane:setVisible(bVisible)
        end
        if self.rFirePane then
            self.rFirePane:setVisible(bVisible)
        end
        if self.rBusPane then
            self.rBusPane:setVisible(bVisible)
        end
        self.rBackButton:setVisible(bVisible)
        self:showAttemptButton(false)
        self.rHeroPane:setEventHeroSelectorUIVisible(bVisible)
        if bVisible and self.rEvent then
            self.rEvent:enableTimeIncrement(false)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroPane:onPaneClosed()
        if self.rEnemyPane then
            self.rEnemyPane:onPaneClosed()
        end
        if self.rFirePane then
            self.rFirePane:onPaneClosed()
        end
        if self.rBusPane then
            self.rBusPane:onPaneClosed()
        end        
        self.rBackButton:onPaneClosed()
        self.rAttemptButton:onPaneClosed()
        self.rDelegateButton:onPaneClosed()
    end

    function Ob:attemptThread()
        util.sleep(0.5)
        -- go back to the map
        if self.rEvent:isAutoResolving() then
            g_GUI:ShowAlertTerritoryMap()
        else
            g_eventManager:queueEvent(self.rEvent)
            g_eventManager:startNextEventInQueue()
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rBackButton then
            if self.rHeroPane:isHeroSelectorOpen() then
                return
            end
            if self.bAttempting then
                return
            end
            if g_Game:isActionEnabled('CloseWindow') then
                g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                self.rEvent:clearAllHeroes()
                self.rEvent:enableTimeIncrement(true)
                g_GUI:ShowAlertTerritoryMap()                    
            end
        elseif rButton == self.rAttemptButton then
            if self:canAttemptEvent() and not self.bAttempting and self.rEvent and self.rEvent:hasHeroesAssigned() then
                self.bAttempting = true
                self.rApproveStamp:setVisible(true)
                g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')
                self.rAttemptThread = MOAIThread:new()
                self.rAttemptThread:run(self.attemptThread, self)
                -- save out the selection
--                g_Game:saveLastHeroSelection(self.rEvent)
				g_Audio:stopMusic()
            end
        elseif rButton == self.rDelegateButton then
            if self:canAttemptEvent() and not self.bAttempting and self.rEvent and self.rEvent:hasHeroesAssigned() and self.rEvent:canBeAutoResolved() then
                local tHeroes = self.rEvent:getAllHeroes()
                self.rEvent:queueToAutoResolve()
                self.bAttempting = true
                self.rApproveStamp:setVisible(true)
                g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')
                self.rAttemptThread = MOAIThread:new()
                self.rAttemptThread:run(self.attemptThread, self)
            end
        end
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        self.rBackButton:onFinger(eventType, sx, sy, tProps)
        if self.bAttemptButtonEnabled then
            self.rAttemptButton:onFinger(eventType, sx, sy, tProps)
        end
        self.rDelegateButton:onFinger(eventType, sx, sy, tProps)
        if self.rAttemptThread then
            return true
        end
        if self.rHeroPane then
            self.rHeroPane:onFinger(eventType, sx, sy, tProps)
        end
        if self.rEnemyPane then
            self.rEnemyPane:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:showAttemptButton(bShow)
        if bShow then
            if self.bAttemptButtonEnabled then
                self.rAttemptButton:setVisible(true)  
            end
            if g_Game:isActionEnabled('AutoResolve') and self.rEvent:canBeAutoResolved() then
                self.rDelegateButton:setVisible(true)    
            end
        else
            if self.bAttemptButtonEnabled then
                self.rAttemptButton:setVisible(false)
            end
            if g_Game:isActionEnabled('AutoResolve') and self.rEvent:canBeAutoResolved() then
                self.rDelegateButton:setVisible(false)
            end
        end
    end

    function Ob:enableAttemptButton(bEnable)
        self.bAttemptButtonEnabled = bEnable
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rEvent = rEvent

    local nXOffset = (Renderer.getUIWidth() - 960) / 2
    Ob:setLoc(nXOffset, 0)
    
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(-nXOffset, 0)    
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(.89, .87, .83)
    Ob:addElement(Ob.rBackground)

    -- add the enemy pane
    if rEvent:getEventType() == 'crime' or rEvent:getEventType() == 'mission' or rEvent:getEventType() == 'mostwanted' or rEvent:getEventType() == 'seniorcrime' then
        Ob.rEnemyPane = EventEnemyPane:new(rRenderLayer, rEvent)
        Ob.rEnemyPane:setLoc(506, -114)
        Ob.rEnemyPane:setParent(Ob)
    elseif rEvent:getEventType() == 'fire' then
        Ob.rFirePane = EventFirePane:new(rRenderLayer, rEvent)
        Ob.rFirePane:setLoc(522, -114)
        Ob.rFirePane:setParent(Ob)
    elseif rEvent:getEventType() == 'busrescue' then
        Ob.rBusPane = EventBusRescuePane:new(rRenderLayer, rEvent)
        Ob.rBusPane:setLoc(522, -114)
        Ob.rBusPane:setParent(Ob)
    end

    Ob.rTopBG = MOAIProp2D.new()
	Ob.rTopBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBG:setLoc(0, 0)
	Ob.rTopBG:setScl(Renderer.getUIWidth(), 110)
	Ob.rTopBG:setColor(.89, .87, .83)
    Ob.rTopBG:setParent(Ob)
    Ob:addOverlayElement(Ob.rTopBG)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(20, -85)
    Ob.rLine1:setScl(920, 4)
    Ob.rLine1:setColor(.25, .25, .24)
    Ob:addOverlayElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(20, -108)
    Ob.rLine2:setScl(920, 4)
    Ob.rLine2:setColor(.25, .25, .24)
    Ob:addOverlayElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(510, -85)
    Ob.rLine3:setScl(920, 4)
    Ob.rLine3:setColor(.25, .25, .24)
    Ob.rLine3:setRot(-90)
    Ob:addOverlayElement(Ob.rLine3)

	Ob.rTitleText = g_GUI:CreateTextBox(36, MOAITextBox.LEFT_JUSTIFY, 'type')
	Ob.rTitleText:setString(g_DialogManager:getString("STXT023TEXT"))
    Ob.rTitleText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTitleText:setColor(0, 0, 0)
	Ob.rTitleText:setLoc(140, -10)
    Ob:addOverlayElement(Ob.rTitleText)

	Ob.rFormLabelText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rFormLabelText:setString(string.upper(g_DialogManager:getString("STXT020TEXT")).." H-23A")
    Ob.rFormLabelText:setRect(-150, 60, 150, 0)
	Ob.rFormLabelText:setColor(0, 0, 0)
	Ob.rFormLabelText:setLoc(190, -60)
    Ob.rFormLabelText:setColor(0.5, 0.5, 0.5)
    Ob:addOverlayElement(Ob.rFormLabelText)

	Ob.rHeroLabelText = g_GUI:CreateTextBox(16, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rHeroLabelText:setString(string.upper(g_DialogManager:getString("STXT024TEXT")))
    Ob.rHeroLabelText:setRect(0, 60, 300, 0)
	Ob.rHeroLabelText:setColor(0, 0, 0)
	Ob.rHeroLabelText:setLoc(110, -88)
    Ob:addOverlayElement(Ob.rHeroLabelText)

	Ob.rEnemyLabelText = g_GUI:CreateTextBox(16, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rEnemyLabelText:setString(string.upper(g_DialogManager:getString("STXT025TEXT")))
    Ob.rEnemyLabelText:setRect(0, 60, 300, 0)
	Ob.rEnemyLabelText:setColor(0, 0, 0)
	Ob.rEnemyLabelText:setLoc(580, -88)
    Ob:addOverlayElement(Ob.rEnemyLabelText)

    -- add the back button
    Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_back', 'button_back_tap')
    Ob.rBackButton:setLoc(20, -10)
    Ob.rBackButton:setParent(Ob)

    Ob.rAttemptButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_continue', 'button_continue_tap', nil, nil, nil, nil, "STXT013TEXT", 'ptsans', 34, { 80, -5 }, { 1, 1, 1 } )
    Ob.rAttemptButton:setLoc(Renderer.getUIWidth() - 188 - nXOffset, -10)    
    Ob.rAttemptButton:setParent(Ob)

    Ob.rDelegateButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_continue', 'button_continue_tap', nil, nil, nil, nil, "STXT158TEXT", 'ptsans', 34, { 80, -5 }, { 1, 1, 1 } )
    Ob.rDelegateButton:setLoc(Renderer.getUIWidth() - 378 - nXOffset, -10)    
    Ob.rDelegateButton:setParent(Ob)

    Ob.rApproveStamp = Ob:getUITextureProp('stamp_approved')
    Ob.rApproveStamp:setLoc(Renderer.getUIWidth() - 800, -400)    
    Ob.rApproveStamp:setRot(30)
    Ob.rApproveStamp:setScl(1.65,1.65)
    Ob:addOverlayElement(Ob.rApproveStamp)
    Ob.rApproveStamp:setVisible(false)

    -- add the hero pane
    Ob.rHeroPane = EventHeroPane:new(rRenderLayer, Ob, rEvent)
    Ob.rHeroPane:setLoc(20, -100)
    Ob.rHeroPane:setParent(Ob)

    Ob.bAttemptButtonEnabled = true

	return Ob
end

return EventDetailUI
