local Renderer = require('Renderer')
local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local Character = require('Character')
local UIProgressBar = require("ui/UIProgressBar")
local UIElement = require('ui/UIElement')
local ResourceBarUI = require('ui/ResourceBarUI')
local SpriteSheet = require('SpriteSheet')
local EffectInstance = require('EffectInstance')
local Item = require('Item')
local Button = require('ui/Button')
local Event = require('event/Event')
local MeteoriteIcon = require('ui/MeteoriteIcon')
local EventEndScreenUI = {}
local EventVictoryScreenUI = {}
local EventFailureScreenUI = {}
local EventVictoryScreenCharPane = {}

local tUITextureSpriteSheetInfo =
{
button_continue = 'UI/eventdetails',
button_continue_tap = 'UI/eventdetails',
stamp_failure = 'UI/eventdetails',
stamp_success = 'UI/eventdetails',
hero_stats = 'UI/eventdetails',
icon_cash = 'UI/hud',
icon_pp = 'UI/hud',
map_icon_cranium = 'UI/eventdetails',
map_icon_monsterfoot = 'UI/eventdetails',
map_icon_robotfoot = 'UI/eventdetails',
map_icon_skullface = 'UI/eventdetails',
map_icon_question = 'UI/eventdetails',
promotion = 'UI/hud2',
star_rating1b = 'UI/eventdetails',
star_rating2b = 'UI/eventdetails',
star_rating3b = 'UI/eventdetails',
star_rating4b = 'UI/eventdetails',
popup_mm = 'UI/shopheropack',
Crystal_RockA = 'UI/hud',
}

local tLoseSummaryLines =
{
    "FLSU001TEXT",
}

local tHintLines =
{
    "FLHI001TEXT",
    "FLHI002TEXT",
    "FLHI003TEXT",
    "FLHI004TEXT",
    "FLHI005TEXT",
    "FLHI006TEXT",
    "FLHI007TEXT",
    "FLHI008TEXT",
    "FLHI009TEXT",
    "FLHI010TEXT",
    "FLHI011TEXT",
    "FLHI012TEXT",
    "FLHI013TEXT",
    "FLHI014TEXT",
    "FLHI015TEXT",
    "FLHI016TEXT",
    "FLHI017TEXT",
    "FLHI018TEXT",
    "FLHI019TEXT",
    "FLHI020TEXT",
    "FLHI021TEXT",
    "FLHI022TEXT",
    "FLHI023TEXT",
}

local tRatingToIcon =
{
    'star_rating1b',
    'star_rating2b',
    'star_rating3b',
    'star_rating4b',
}    

function EventVictoryScreenCharPane:new(rRenderLayer, rHero, rParentUI, nBaseXP)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rXPProgressBar:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rXPProgressBar:onPaneClosed()
    end

    function Ob:showLevelUp()
        self.rXPProgressBar:realizeProgress()
        self.rXPProgressBar.rProgressBar:setColor(249/255, 243/255, 95/255, 1)

        self.rPromotion = Ob:getUITextureProp('promotion')
        self:addElement(self.rPromotion)
        self.rPromotion:setLoc(-14,-80)

        self.rPromoText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'bangers')
        -- MTF TODO: use linecode
        self.rPromoText:setString("PROMOTION EARNED!") --g_DialogManager:getString("STXT045TEXT"))
        local x,y = self.rXPProgressBar:getLoc()
        self.rPromoText:setLoc(x+130, y-4)
        self.rPromoText:setColor(196/255, 145/255, 54/255, 1)
        self:addElement(self.rPromoText)
        
        --self.rPromotionFX:setLoc(x+200, y-80)
        --self.rPromotionFX:start()

        self:setElementsVisible(true)
    end


    function Ob:showDetailsThread()
        if not self.nBaseXP then
            self.nBaseXP = 0
        end

        if self.rHero:isAtMaxLevel() then
            local x,y = self.rXPProgressBar:getLoc()
            self.rPromoText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'bangers')
            -- MTF TODO: use linecode
            self.rPromoText:setString("MAX LEVEL") --g_DialogManager:getString("STXT045TEXT"))
            self.rPromoText:setLoc(x+130, y-4)
            self.rPromoText:setColor(196/255, 145/255, 54/255, 1)
            self:addElement(self.rPromoText)
            rRenderLayer:insertProp(self.rPromoText)
            self.rXPProgressBar:setVisible(true)
            self.rXPProgressBar:update(999999)
            self.rXPProgressBar.rProgressBar:setColor(249/255, 243/255, 95/255, 1)
        else
            local nOldLevel = self.rHero:getStatLevel()
            local nXPIncrement = self.nBaseXP + self.nBaseXP * self.rHero:getXPMuliplier()
            local nOldXP = self.rHero:getCurXP()
            self.rHero:incrementXP(nXPIncrement)
            local nNewXP = self.rHero:getCurXP()

            self.bLevelUp = false
            if self.rHero:isPreppedForLevelUp() then
                self.bLevelUp = true
                nNewXP = self.rHero:getXPForNextLevel() 
            end

            self.rXPProgressBar:setVisible(true)
            if nNewXP > nOldXP then
                self.rXPProgressBar:setProgressPending(nNewXP-nOldXP, 0.5)
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rParentUI = rParentUI
	Ob.rHero = rHero
    Ob.nBaseXP = nBaseXP
    Ob.nFinalXP = nBaseXP

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setScl(0.84, 0.84)
        Ob.rPortrait:setLoc(8, -8)
        Ob:addElement(Ob.rPortrait)
    end    

    Ob.rXPText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY, 'type')    
    Ob.rXPText:setString(g_DialogManager:getString("STXT045TEXT"))
    Ob.rXPText:setLoc(220, -30)
    Ob:addElement(Ob.rXPText)

    Ob.rXPProgressBarBG = MOAIProp2D.new()
	Ob.rXPProgressBarBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rXPProgressBarBG:setLoc(125, -65)
	Ob.rXPProgressBarBG:setScl(320, 50)
	Ob.rXPProgressBarBG:setColor(.52, .54, .52)
    Ob:addElement(Ob.rXPProgressBarBG)    
    
    Ob.rPromotionFX = EffectInstance:new('Effects/PromotionBling.lua', Renderer.getLayer('UIOverlay').RenderLayer)
    Ob.rPromotionFX:setScl(5, 5)
    Ob.rPromotionFX:setLoc(890, -150)

    Ob.rXPProgressBar = UIProgressBar:new(rRenderLayer, rHero:getXPForNextLevel(), { r = 1, g = 1, b = 1, a = 1}, { r = 0, g = 0, b = 0, a = 0}, false, { r = 249/255, g = 243/255, b = 95/255, a = 1})
    Ob.rXPProgressBar:setLoc(128, -68)
    Ob.rXPProgressBar:setScl(316, 44)
    Ob.rXPProgressBar:update(rHero:getCurXP())
    Ob.rXPProgressBar:setParent(Ob)

    return Ob
end

function EventVictoryScreenUI:new(rRenderLayer, rEvent)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rPane in ipairs(self.tHeroPanes) do
            rPane:setVisible(bVisible)
        end 
        self.rCloseButton:setVisible(false)
        if bVisible then
            self.rThread = MOAIThread.new()
            self.rThread:run(self.showDetailsThread, self)
        end
        if not bVisible then
            if self.rBossProgressBar then
                self.rBossProgressBar:setVisible(false)
            end
            if self.rStarXPProgressBar then
                self.rStarXPProgressBar:setVisible(false)
            end
            self.rMeteoriteIcon:setVisible(false)
        end
	end

    function Ob:showDetailsThread()
        self.bIsRunningThread = true
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Victory', false)

        g_Audio:playCue('MMOJ_Audio/UI/PerformanceReview_Impact')
        util.sleep(0.2)

        self.rLevelUpSound = g_Audio:playCue('MMOJ_Audio/UI/LevelUpTone', true)
        for i, rPane in ipairs(self.tHeroPanes) do
            rPane:showDetailsThread()
        end
        util.sleep(0.6)
        if self.rLevelUpSound then
            self.rLevelUpSound:stop()
        end
        for i, rPane in ipairs(self.tHeroPanes) do
            if rPane.rLevelUpSound then
                rPane.rLevelUpSound:stop()
            end
            if rPane.bLevelUp then
                rPane:showLevelUp()
            end
        end
        
        Ob.rManagerLabel:revealAll()  
        Ob.rStarXPProgressBarBG:setVisible(true)
        Ob.rStarXPProgressBar:setVisible(true)
        Ob.rMMPortrait:setVisible(true)
        local nBarX, nBarY = self.rStarXPProgressBar:getLoc()
        
        g_MiddleManager:incrementStarRatingScore(self.rEvent:getTotalRatingScore())        
        if g_MiddleManager:isAtMaxStarLevel() then
            self.rPromoText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'type')
            -- MTF TODO: use linecode
            self.rPromoText:setString("MAX LEVEL") --g_DialogManager:getString("STXT045TEXT"))
            self.rPromoText:setLoc(nBarX+130, nBarY-4)
            self.rPromoText:setColor(196/255, 145/255, 54/255, 1)
            self:addElement(self.rPromoText)
            rRenderLayer:insertProp(self.rPromoText)
            self.rStarXPProgressBar:setVisible(true)
            self.rStarXPProgressBar:update(999999)
            self.rStarXPProgressBar.rProgressBar:setColor(249/255, 243/255, 95/255, 1)
        else
            local nOldXP = g_MiddleManager:getStarRatingXP()
            local nOldLevel = g_MiddleManager:getStarRatingLevel()
            local nNextLevel = g_MiddleManager:getStarRatingXPForNextLevel()
            g_MiddleManager:incrementStarRatingXP(self.rEvent:getTotalRatingScore())
            local nNewXP = g_MiddleManager:getStarRatingXP()
            local nNewLevel = g_MiddleManager:getStarRatingLevel()
    
            if nNewLevel > nOldLevel then
                self.rLevelUpSound = g_Audio:playCue('MMOJ_Audio/UI/LevelUpTone', true)
            end
            if nNewXP > nOldXP then
                self.rStarXPProgressBar:setProgressPending(nNewXP-nOldXP, 0.5)
            else
                self.rStarXPProgressBar:setProgressPending(nNextLevel, 0.5)
            end
            util.sleep(0.5)
            if self.rLevelUpSound then
                self.rLevelUpSound:stop()
            end
            if nNewLevel > nOldLevel then
                print("Leveling up Middle Manager",nNewLevel,nOldLevel)
                self.rStarXPProgressBar:realizeProgress()
                self.rStarXPProgressBar.rProgressBar:setColor(249/255, 243/255, 95/255, 1)
    
                local nBarX, nBarY = self.rStarXPProgressBar:getLoc()
                self.rPromotion = Ob:getUITextureProp('promotion')
                self:addElement(self.rPromotion)
                self.rPromotion:setLoc(nBarX-134,nBarY-16)
    
                self.rPromoText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'bangers')
                -- MTF TODO: use linecode
                self.rPromoText:setString("PROMOTION EARNED!") --g_DialogManager:getString("STXT045TEXT"))
                self.rPromoText:setLoc(nBarX+130, nBarY-4)
                self.rPromoText:setColor(196/255, 145/255, 54/255, 1)
                self:addElement(self.rPromoText)
    
                self:setElementsVisible(true)     
            end                
        end
        util.sleep(0.5)
        
        if self.rTimeRatingIcon then
            self.rTimeRatingIcon:setVisible(true)
            self.rTimeRatingFX:start()
            g_Audio:playCue('MMOJ_Audio/UI/PerformanceReview_Impact')        
            util.sleep(0.25)
        end
        
        if self.nCashReward > 0 then
            self.rCashText:revealAll()
            self.rCashAmount:revealAll()
            self.rCashIcon:setVisible(true)
            self.rCashFX:start()
            g_Audio:playCue('MMOJ_Audio/UI/PerformanceReview_Impact')   
            util.sleep(0.5)
        end

        if self.nPPReward > 0 then
            self.rPPText:revealAll()
            self.rPPAmount:revealAll()
            self.rPPIcon:setVisible(true)
            self.rPPFX:start()
            g_Audio:playCue('MMOJ_Audio/UI/PerformanceReview_Impact')  
            util.sleep(0.5)
        end

        if self.rEvent then
            local rMeteoriteInfo = self.rEvent:getMeteoriteInfo()
            if rMeteoriteInfo and rMeteoriteInfo.bTapped then
                self.rMeteoriteText:revealAll()
                self.rMeteoriteRockIcon:setVisible(true)
                self.rMeteoriteAmount:revealAll()
                self.rMeteoriteIcon:setMeteorite(rMeteoriteInfo.sKey, rMeteoriteInfo.nLevel)
                self.rMeteoriteIcon:setVisible(true)
                g_Audio:playCue('MMOJ_Audio/SFX/Battle/CollectMeteorite')
                util.sleep(0.5)
            end
        end

        Ob.rStarScoreMultField:revealAll()
        util.sleep(0.25)

        Ob.rTotalLabel:revealAll()    
        Ob.rTotalScoreLabel:revealAll()

        if self.rEvent then
            local rEventDef = self.rEvent:getDef()
            if rEventDef then
                if rEventDef.bIsBossFight then
                    self.bClearedArea = true
                else
                    local nReduction = self.rEvent:getDef().nCrimeLevelReduction
                    if nReduction and nReduction > 0 and self.rTerritory and self.rTerritory:getCrimeLevel() > 0 and self.rBossPortrait then
                        util.sleep(0.5)
                        
                        self.rBossPortrait:setVisible(true)
                        
                        if self.rBossProgressLabel then
                            self.rBossProgressLabel:revealAll()
                        end
                        self:runIncrementCrimeLevel(rEvent:getCrimeLevelReduction())
                        self:updateBossProgressBar()
                        
                        util.sleep(0.5)
                    end
                end
            end
        end

        self.rStamp:setVisible(true)
        g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')

        util.sleep(0.5)
        self.rCloseButton:setVisible(true)

        self.bIsRunningThread = false
    end

    function Ob:runIncrementCrimeLevel(nCrimeAmount)
        if self.rTerritory:hasActiveMission() and not self.rEvent:isMissionEvent() and nCrimeAmount ~= 0 then
            return
        end
        local nCurCrimeLevel = self.rTerritory:getCrimeLevel()
        if nCurCrimeLevel > 0 then
            local nOldCrimeLevel = nCurCrimeLevel
            self.rTerritory:addCrimeAmount(-nCrimeAmount)
            nCurCrimeLevel = self.rTerritory:getCrimeLevel() 
            print("NEW CRIME LEVEL "..nCurCrimeLevel)
        end
    end

    function Ob:updateBossProgressBar()
        if self.rBossProgressBar then
            self.rBossProgressBar:setVisible(true)        
            self.rLevelUpSound = g_Audio:playCue('MMOJ_Audio/UI/LevelUpTone', true)
            self.rBossProgressBar:moveToAmount(self.rTerritory:getBossFightProgress(), 1)
            util.sleep(1)
            if self.rLevelUpSound then
                self.rLevelUpSound:stop()
            end        
        end
    end

	function Ob:removeFromLayer()
        for i, rPane in ipairs(self.tHeroPanes) do
            rPane:hide()
        end
		if self.rLayer and self.tElements then
			for i, rElement in ipairs(self.tElements) do
				self.rLayer:removeProp(rElement)
			end
		end
		self.tElements = nil
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rCloseButton then
            g_Audio:playCue('MMOJ_Audio/UI/ContinueButton')

            local cashReward = self.rEvent:getCashReward()
            g_Game:addCash(cashReward)
            DFAnalytics.incrStat('csh_ernd_evnt', cashReward)

            local ppReward = self.rEvent:getPPReward()
            if ppReward > 0 then
                g_Game:addPP(ppReward)        
                DFAnalytics.logEvent("ppp_ernd", { type = "evnt", amount = ppReward })                
                DFAnalytics.incrStat('ppp_ernd_evnt', ppReward)
            end

            if self.sConsumableReward then
                g_MiddleManager:addConsumableItemToInventory(self.sConsumableReward)
            end
            self.rEvent:clearAllHeroes()
            self.rTerritory:removeEvent(self.rEvent)

            if self.rEvent:getRewardOverride() then
                self.rTerritory:onRandomPPRewardGiven(ppReward)
            end

            local sEquipmentDropDefKey = self.rEvent:getEquipmentDropReward()
            if sEquipmentDropDefKey then
                g_MiddleManager:addEquipmentToInventory(sEquipmentDropDefKey)
            end
            if self.bClearedArea then
                self.rTerritory:setCleared(true)
            end
            g_Game:saveGame()                     
            if sEquipmentDropDefKey then
                local rDef = Item.getEquipmentDef(sEquipmentDropDefKey)
                if rDef then
                    local constructor = Item.getConstructor(rDef.sItemType)
                    if constructor then
                        local rItem = constructor(Item, rDef, sItemDefKey)
                        if rItem then
                            g_GUI:ShowPremiumPurchaseSequence('equipment', rItem)
                            return true
                        end
                    end
                end            
            end

            g_GUI:HideEventEndScreenUI()

            return true
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.bIsRunningThread then
            return true
        end
        self.rCloseButton:onFinger(eventType, sx, sy, tProps)
		return true
	end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rEvent = rEvent
    Ob.bIsRunningThread = false

	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(.84, .89, .82)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)

	-- text
	Ob.rHeadlineText = g_GUI:CreateTextBox(44, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rHeadlineText:setString(g_DialogManager:getString("STXT015TEXT"))
    Ob.rHeadlineText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rHeadlineText:setColor(0, 0, 0)
	Ob.rHeadlineText:setLoc(0, -14)
    Ob:addElement(Ob.rHeadlineText)

    --top line (above participants/summary)
    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(20, -85)
    Ob.rLine1:setScl(920, 4)
    Ob.rLine1:setColor(.25, .25, .24)
    Ob:addElement(Ob.rLine1)
    
    -- top second line (below participants/summary)
    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(20, -108)
    Ob.rLine2:setScl(920, 4)
    Ob.rLine2:setColor(.25, .25, .24)
    Ob:addElement(Ob.rLine2)

    --center vertical line
    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(484, -85)
    Ob.rLine3:setScl(920, 4)
    Ob.rLine3:setColor(.25, .25, .24)
    Ob.rLine3:setRot(-90)
    Ob:addElement(Ob.rLine3)

    --bottom line
    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(48, -Renderer.getUIHeight() + 18)
    Ob.rLine4:setScl(880, 4)
    Ob.rLine4:setColor(.25, .25, .24)
    Ob:addElement(Ob.rLine4)

    local nXPReward = rEvent:getXPReward()

    Ob.rHeroPaneParent = MOAITransform:new()
    Ob.rHeroPaneParent:setLoc(0, 0)
    Ob.tHeroPanes = {}
    local tHeroes = rEvent:getAllHeroes()
    local nNumHeroes = table.getn(tHeroes)
    local nInitY = -112
    local kYSIZE = 128
    if nNumHeroes > 0 then
        for i, rHero in ipairs(tHeroes) do
            local rHeroPane = EventVictoryScreenCharPane:new(rRenderLayer, rHero, Ob, nXPReward)
            rHeroPane:setLoc(12, nInitY - ((i - 1) * kYSIZE))            
            rHeroPane:setParent(Ob.rHeroPaneParent)
            table.insert(Ob.tHeroPanes, rHeroPane)
        end
    end
    
    Ob.nCashReward = rEvent:getCashReward() or 0
    Ob.nPPReward = rEvent:getPPReward() or 0

    Ob.rCloseButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_continue', 'button_continue_tap', nil, nil, nil, nil, "STXT016TEXT", 'ptsans', 34, { 80, -5 }, { 1, 1, 1 } )
    Ob.rCloseButton:setLoc(Renderer.getUIWidth() - 188, -10)    
    Ob.rCloseButton:setParent(Ob)

    Ob.rCashText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rCashText:setString(g_DialogManager:getString("STXT182TEXT"))
    Ob.rCashText:setRect(0, 80, 300, 0)
    Ob.rCashText:setColor(.25, .25, .24)
    Ob.rCashText:setLoc(500, -363)
    Ob.rCashText:setReveal(0)
	Ob:addElement(Ob.rCashText)
    
    Ob.rCashAmount = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rCashAmount:setString("...................................  "..Ob.nCashReward)
    Ob.rCashAmount:setRect(0, 50, 400, 0)
    Ob.rCashAmount:setColor(.25, .25, .24)
    Ob.rCashAmount:setLoc(540, -402)
    Ob.rCashAmount:setReveal(0)
	Ob:addElement(Ob.rCashAmount)

    Ob.rCashIcon = Ob:getUITextureProp('icon_cash')
    Ob.rCashIcon:setLoc(510, -395)
    Ob.rCashIcon:setVisible(false)
    Ob:addElement(Ob.rCashIcon)
    
    local xCash,yCash = Ob.rCashIcon:getLoc()
    Ob.rCashFX = EffectInstance:new('Effects/CoinBling.lua', Renderer.getLayer('UIOverlay').RenderLayer)
    Ob.rCashFX:setScl(3, 3)
    Ob.rCashFX:setLoc(xCash+20, yCash-20)

    Ob.rPPText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rPPText:setString(g_DialogManager:getString("STXT182TEXT"))
    Ob.rPPText:setRect(0, 80, 300, 0)
    Ob.rPPText:setColor(.25, .25, .24)
    Ob.rPPText:setLoc(500, -363)
    Ob.rPPText:setReveal(0)
	Ob:addElement(Ob.rPPText)
    
    Ob.rPPAmount = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rPPAmount:setString("...................................  "..Ob.nPPReward)
    Ob.rPPAmount:setRect(0, 50, 400, 0)
    Ob.rPPAmount:setColor(.25, .25, .24)
    Ob.rPPAmount:setLoc(540, -402)
    Ob.rPPAmount:setReveal(0)
	Ob:addElement(Ob.rPPAmount)
    
    Ob.rPPIcon = Ob:getUITextureProp('icon_pp')
    Ob.rPPIcon:setLoc(510, -395)
    Ob:addElement(Ob.rPPIcon)
    Ob.rPPIcon:setVisible(false)
    
    local xPP,yPP = Ob.rPPIcon:getLoc()
    Ob.rPPFX = EffectInstance:new('Effects/SuperiumBling.lua', Renderer.getLayer('UIOverlay').RenderLayer)
    Ob.rPPFX:setScl(3, 3)
    Ob.rPPFX:setLoc(xPP+20, yPP-20)
    
    Ob.rMeteoriteText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rMeteoriteText:setString(g_DialogManager:getString("STXT208TEXT"))
    Ob.rMeteoriteText:setRect(0, 80, 300, 0)
    Ob.rMeteoriteText:setColor(.25, .25, .24)
    Ob.rMeteoriteText:setLoc(500, -462)
    Ob.rMeteoriteText:setReveal(0)
	Ob:addElement(Ob.rMeteoriteText)

    Ob.rMeteoriteRockIcon = Ob:getUITextureProp('Crystal_RockA')
    Ob.rMeteoriteRockIcon:setLoc(506, -482)
    Ob.rMeteoriteRockIcon:setScl(0.5)
    Ob:addElement(Ob.rMeteoriteRockIcon)
    Ob.rMeteoriteRockIcon:setVisible(false)    

    Ob.rMeteoriteAmount = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rMeteoriteAmount:setString("................................")
    Ob.rMeteoriteAmount:setRect(0, 50, 400, 0)
    Ob.rMeteoriteAmount:setColor(.25, .25, .24)
    Ob.rMeteoriteAmount:setLoc(480, -496)
    Ob.rMeteoriteAmount:setReveal(0)
	Ob:addElement(Ob.rMeteoriteAmount)

    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer)
    Ob.rMeteoriteIcon:setLoc(860, -456)
    Ob.rMeteoriteIcon:setScl(0.8)
    Ob.rMeteoriteIcon:setParent(Ob)

    --line underneath manager
    Ob.rLine5 = MOAIProp2D.new()
    Ob.rLine5:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine5:setLoc(480, -250)
    Ob.rLine5:setScl(460, 4)
    Ob.rLine5:setColor(.25, .25, .24)
    Ob:addElement(Ob.rLine5)
    
    --line just above villian progress
    Ob.rLine6 = MOAIProp2D.new()
    Ob.rLine6:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine6:setLoc(480, -540)
    Ob.rLine6:setScl(460, 4)
    Ob.rLine6:setColor(.25, .25, .24)
    Ob:addElement(Ob.rLine6)
    
    --branch score
    Ob.rTotalLabel = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rTotalLabel:setString(g_DialogManager:getString("STXT179TEXT"))
    Ob.rTotalLabel:setRect(0, 80, 300, 0)
    Ob.rTotalLabel:setColor(.25, .25, .24)
    Ob.rTotalLabel:setLoc(500, -264)
    Ob.rTotalLabel:setReveal(0)
	Ob:addElement(Ob.rTotalLabel)

    local nStarRating = Ob.rEvent:getTimeStarRating()
    local sRatingIcon = tRatingToIcon[nStarRating]
    if sRatingIcon then
        Ob.rTimeRatingIcon = Ob:getUITextureProp(sRatingIcon)
        Ob.rTimeRatingIcon:setLoc(510, -298)
        Ob.rTimeRatingIcon:setVisible(false)
        Ob:addElement(Ob.rTimeRatingIcon)
        local xTime,yTime = Ob.rTimeRatingIcon:getLoc()
        Ob.rTimeRatingFX = EffectInstance:new('Effects/StampBling.lua', Renderer.getLayer('UIOverlay').RenderLayer)
        Ob.rTimeRatingFX:setScl(3, 3)
        Ob.rTimeRatingFX:setLoc(xTime+40, yTime-20)
    end

    local nStarRatingScore = nStarRating * Event.getTimeStarScoreMult()
    Ob.rStarScoreMultField = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rStarScoreMultField:setString("x "..Event.getTimeStarScoreMult())
    Ob.rStarScoreMultField:setRect(0, 80, 80, 0)
    Ob.rStarScoreMultField:setColor(.25, .25, .24)
    Ob.rStarScoreMultField:setLoc(610, -300)
    Ob.rStarScoreMultField:setReveal(0)
	Ob:addElement(Ob.rStarScoreMultField)

    Ob.rTotalScoreLabel = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rTotalScoreLabel:setString(".....................  "..tostring(rEvent:getTotalRatingScore()))
    Ob.rTotalScoreLabel:setRect(0, 50, 250, 0)
    Ob.rTotalScoreLabel:setColor(.25, .25, .24)
    Ob.rTotalScoreLabel:setLoc(690, -300)
    Ob.rTotalScoreLabel:setReveal(0)
	Ob:addElement(Ob.rTotalScoreLabel)

    --manager experience
    Ob.rManagerLabel = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rManagerLabel:setString(g_DialogManager:getString("STXT181TEXT"))
    Ob.rManagerLabel:setRect(0, 80, 300, 0)
    Ob.rManagerLabel:setColor(.25, .25, .24)
    Ob.rManagerLabel:setLoc(620, -145)
    Ob.rManagerLabel:setReveal(0)
	Ob:addElement(Ob.rManagerLabel)
    
    Ob.rStarXPProgressBarBG = MOAIProp2D.new()
	Ob.rStarXPProgressBarBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rStarXPProgressBarBG:setLoc(620, -179)
	Ob.rStarXPProgressBarBG:setScl(320, 50)
	Ob.rStarXPProgressBarBG:setColor(.52, .54, .52)
    Ob.rStarXPProgressBarBG:setVisible(false)
    Ob:addElement(Ob.rStarXPProgressBarBG)

    Ob.rStarXPProgressBar = UIProgressBar:new(rRenderLayer, g_MiddleManager:getStarRatingXPForNextLevel(), { r = 1, g = 1, b = 1, a = 1}, { r = 0, g = 0, b = 0, a = 0}, false, { r = 249/255, g = 243/255, b = 95/255, a = 1})
    Ob.rStarXPProgressBar:setLoc(623, -182)
    Ob.rStarXPProgressBar:setScl(316, 44)
    Ob.rStarXPProgressBar:update(g_MiddleManager:getStarRatingXP())
    Ob.rStarXPProgressBar:setParent(Ob)

    Ob.rMMPortrait = g_MiddleManager:getAvatar():getPortraitProp()
    Ob.rMMPortrait:setScl(0.84)
    Ob.rMMPortrait:setLoc(500, -122)
    Ob.rMMPortrait:setVisible(false)
    Ob:addElement(Ob.rMMPortrait)

    Ob.rStamp = Ob:getUITextureProp('stamp_success')
    Ob.rStamp:setLoc(Renderer.getUIWidth() - 800, -400)    
    Ob.rStamp:setRot(42)
    Ob.rStamp:setScl(1.65,1.65)
    Ob:addOverlayElement(Ob.rStamp)
    Ob.rStamp:setVisible(false)

    Ob.rSummaryLabel = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rSummaryLabel:setString(string.upper(g_DialogManager:getString("STXT019SKUL")))
    Ob.rSummaryLabel:setRect(-150, 80, 150, 0)
    Ob.rSummaryLabel:setColor(.25, .25, .24)
    Ob.rSummaryLabel:setLoc(740, -86)
	Ob:addElement(Ob.rSummaryLabel)

    Ob.rParticipantsLabel = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rParticipantsLabel:setString(string.upper(g_DialogManager:getString("STXT018TEXT")))
    Ob.rParticipantsLabel:setRect(-150, 80, 150, 0)
    Ob.rParticipantsLabel:setColor(.25, .25, .24)
    Ob.rParticipantsLabel:setLoc(260, -86)
	Ob:addElement(Ob.rParticipantsLabel)

    Ob.rFormLabel = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rFormLabel:setString(string.upper(g_DialogManager:getString("STXT020TEXT")).."\n".."F-411B")
    Ob.rFormLabel:setRect(-150, 80, 150, 0)
    Ob.rFormLabel:setColor(.25, .25, .24)
    Ob.rFormLabel:setLoc(180, -30)
	Ob:addElement(Ob.rFormLabel)

    Ob.rTerritory = rEvent:getTerritory()   

    if Ob.rTerritory:getCrimeLevel() > 0 then
        Ob.rBossProgressLabel = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rBossProgressLabel:setString(g_DialogManager:getString("STXT105TEXT"))
        Ob.rBossProgressLabel:setRect(0, 80, 500, 0)
        Ob.rBossProgressLabel:setColor(.25, .25, .24)
        Ob.rBossProgressLabel:setLoc(565, -556)
        Ob.rBossProgressLabel:setReveal(0)
        Ob:addElement(Ob.rBossProgressLabel)

        Ob.rBossProgressBar = UIProgressBar:new(rRenderLayer, 100, { r = .9, g = .78, b = .07, a = 1}, { r = 0, g = 0, b = 0, a = 1 }, true)
        Ob.rBossProgressBar:setLoc(518, -588)
        Ob.rBossProgressBar:setScl(340, 20)
        Ob.rBossProgressBar:update(Ob.rTerritory:getBossFightProgress())
        Ob.rBossProgressBar:setParent(Ob)
        
        if Ob.rTerritory:getDef().tMapUIInfo.bossPortraitSpriteName then
            Ob.rBossPortrait = Ob:getUITextureProp(Ob.rTerritory:getDef().tMapUIInfo.bossPortraitSpriteName)
            Ob.rBossPortrait:setLoc(876, -556)
            Ob.rBossPortrait:setVisible(false)
            Ob:addElement(Ob.rBossPortrait)            
        end
    end

	return Ob
end

function EventFailureScreenUI:new(rRenderLayer, rEvent)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)   
        for i, rPane in ipairs(self.tHeroPanes) do
            rPane:setVisible(bVisible)
        end 
        self.rCloseButton:setVisible(false)
        if bVisible then
            self.rThread = MOAIThread.new()
            self.rThread:run(self.showDetailsThread, self)
        else
            self.rMeteoriteIcon:setVisible(false)
        end     
	end

    function Ob:showDetailsThread()
        self.bIsRunningThread = true
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Defeat', false)

        if self.rEvent then
            local rMeteoriteInfo = self.rEvent:getMeteoriteInfo()
            if rMeteoriteInfo and rMeteoriteInfo.bTapped then
                self.rMeteoriteText:revealAll()
                self.rMeteoriteRockIcon:setVisible(true)
                self.rMeteoriteAmount:revealAll()
                self.rMeteoriteIcon:setMeteorite(rMeteoriteInfo.sKey, rMeteoriteInfo.nLevel)
                self.rMeteoriteIcon:setVisible(true)
                g_Audio:playCue('MMOJ_Audio/SFX/Battle/CollectMeteorite')
                util.sleep(0.5)
            end
        end

        util.sleep(0.5)

        self.rStamp:setVisible(true)
        g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')

        util.sleep(0.5)
        self.rCloseButton:setVisible(true)
 
        util.sleep(0.5)

        self.bIsRunningThread = false
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rCloseButton then
            g_Audio:playCue('UI_ContinueButton')
            self.rEvent:setCompleted(false)
            self.rEvent:clearAllHeroes()
            if self.rEvent:getDef().bRemoveAfterFailure then
                self.rTerritory:removeEvent(self.rEvent)
            end
            self.rEvent:enableTimeIncrement(true)
            g_GUI:HideEventEndScreenUI()
            g_Game:saveGame()
            return true
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.bIsRunningThread then
            return true
        end
        self.rCloseButton:onFinger(eventType, sx, sy, tProps)
        return true            
	end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.bIsRunningThread = false

	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(.84, .73, .71)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)
    
    Ob.rMMImage = Ob:getUITextureProp('popup_mm')
    Ob.rMMImage:setLoc(Renderer.getUIWidth() - 140, -220)
    Ob:addElement(Ob.rMMImage)

	-- text
	Ob.rHeadlineText = g_GUI:CreateTextBox(44, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rHeadlineText:setString(g_DialogManager:getString("STXT015TEXT"))
    Ob.rHeadlineText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rHeadlineText:setColor(.47, .23, .14)
	Ob.rHeadlineText:setLoc(0, -14)
    Ob:addElement(Ob.rHeadlineText)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(20, -85)
    Ob.rLine1:setScl(920, 4)
    Ob.rLine1:setColor(.47, .23, .14)
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(20, -108)
    Ob.rLine2:setScl(920, 4)
    Ob.rLine2:setColor(.47, .23, .14)
    Ob:addElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(510, -85)
    Ob.rLine3:setScl(920, 4)
    Ob.rLine3:setColor(.47, .23, .14)
    Ob.rLine3:setRot(-90)
    Ob:addElement(Ob.rLine3)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(48, -Renderer.getUIHeight() + 18)
    Ob.rLine4:setScl(880, 4)
    Ob.rLine4:setColor(.47, .23, .14)
    Ob:addElement(Ob.rLine4)

    Ob.rCloseButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_continue', 'button_continue_tap', nil, nil, nil, nil, "STXT016TEXT", 'ptsans', 34, { 83, -5 }, { 1, 1, 1 } )
    Ob.rCloseButton:setLoc(Renderer.getUIWidth() - 188, -10)    
    Ob.rCloseButton:setParent(Ob)

    Ob.rSummaryLabel = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rSummaryLabel:setString(string.upper(g_DialogManager:getString("STXT019SKUL")))
    Ob.rSummaryLabel:setRect(-150, 80, 150, 0)
    Ob.rSummaryLabel:setColor(.47, .23, .14)
    Ob.rSummaryLabel:setLoc(740, -86)
	Ob:addElement(Ob.rSummaryLabel)

    Ob.rParticipantsLabel = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rParticipantsLabel:setString(string.upper(g_DialogManager:getString("STXT018TEXT")))
    Ob.rParticipantsLabel:setRect(-150, 80, 150, 0)
    Ob.rParticipantsLabel:setColor(.47, .23, .14)
    Ob.rParticipantsLabel:setLoc(260, -86)
	Ob:addElement(Ob.rParticipantsLabel)

    Ob.rFormLabel = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rFormLabel:setString(string.upper(g_DialogManager:getString("STXT020TEXT")).."\nF-411B")
    Ob.rFormLabel:setRect(-150, 80, 150, 0)
    Ob.rFormLabel:setColor(.47, .23, .14)
    Ob.rFormLabel:setLoc(180, -30)
	Ob:addElement(Ob.rFormLabel)

    Ob.rStamp = Ob:getUITextureProp('stamp_failure')
    Ob.rStamp:setLoc(280, -340)
    Ob.rStamp:setRot(10)
    Ob.rStamp:setScl(1.65,1.65)
    Ob:addOverlayElement(Ob.rStamp)
    Ob.rStamp:setVisible(false)

    Ob.rHeroPaneParent = MOAITransform:new()
    Ob.rHeroPaneParent:setLoc(0, 0)
    Ob.tHeroPanes = {}
    local tHeroes = rEvent:getAllHeroes()
    local nNumHeroes = table.getn(tHeroes)
    local nInitY = -112
    local kYSIZE = 128
    if nNumHeroes > 0 then
        for i, rHero in ipairs(tHeroes) do
            local rHeroPane = EventVictoryScreenCharPane:new(rRenderLayer, rHero, Ob, 0)
            rHeroPane:setLoc(48, nInitY - ((i - 1) * kYSIZE))
            rHeroPane:setParent(Ob.rHeroPaneParent)
            table.insert(Ob.tHeroPanes, rHeroPane)
        end
    end

    -- cutting summary text for now
    --Ob.rSummaryText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
    --Ob.rSummaryText:setString(g_DialogManager:getString(tLoseSummaryLines[math.random(table.getn(tLoseSummaryLines))]))
    --Ob.rSummaryText:setRect(0, 200, 400, 0)
    --Ob.rSummaryText:setColor(.47, .23, .14)
    --Ob.rSummaryText:setLoc(520, -120)
	--Ob:addElement(Ob.rSummaryText)

    Ob.rHintText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rHintText:setString(g_DialogManager:getString(tHintLines[math.random(table.getn(tHintLines))]))
    Ob.rHintText:setRect(0, 160, 400, 0)
    Ob.rHintText:setColor(.47, .23, .14)
    Ob.rHintText:setLoc(520, -120)
	Ob:addElement(Ob.rHintText)

    Ob.rMeteoriteText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rMeteoriteText:setString(g_DialogManager:getString("STXT208TEXT"))
    Ob.rMeteoriteText:setRect(0, 80, 300, 0)
    Ob.rMeteoriteText:setColor(.25, .25, .24)
    Ob.rMeteoriteText:setLoc(520, -542)
    Ob.rMeteoriteText:setReveal(0)
	Ob:addElement(Ob.rMeteoriteText)

    Ob.rMeteoriteRockIcon = Ob:getUITextureProp('Crystal_RockA')
    Ob.rMeteoriteRockIcon:setLoc(516, -562)
    Ob.rMeteoriteRockIcon:setScl(0.5)
    Ob:addElement(Ob.rMeteoriteRockIcon)
    Ob.rMeteoriteRockIcon:setVisible(false)    

    Ob.rMeteoriteAmount = g_GUI:CreateTextBox(34, MOAITextBox.RIGHT_JUSTIFY, 'boogaloo')
    Ob.rMeteoriteAmount:setString("................................")
    Ob.rMeteoriteAmount:setRect(0, 50, 400, 0)
    Ob.rMeteoriteAmount:setColor(.25, .25, .24)
    Ob.rMeteoriteAmount:setLoc(490, -576)
    Ob.rMeteoriteAmount:setReveal(0)
	Ob:addElement(Ob.rMeteoriteAmount)

    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer)
    Ob.rMeteoriteIcon:setLoc(870, -536)
    Ob.rMeteoriteIcon:setScl(0.8)
    Ob.rMeteoriteIcon:setParent(Ob)

    Ob.rTerritory = rEvent:getTerritory()
    Ob.rEvent = rEvent

	return Ob
end

function EventEndScreenUI:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)

	-- functions	
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)        
        self.rActivePane:setVisible(bVisible)
        if not bVisible then
            -- resume the office music
            g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IngameLoop', true)
            g_Audio:playAmbience('MMOJ_Audio/Ambience/OfficeAmbience1')
            g_City:enableEventGeneration(true)
        end
	end

    function Ob:update()
        if self.rActivePane and self.rActivePane.update then
            self.rActivePane:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
		if self.rActivePane then
			self.rActivePane:onFinger(eventType, sx, sy, tProps)
		end
		return true
	end

	Ob.rEvent = nil
	Ob.rActivePane = nil

    if rEvent:isSuccessful() then
        Ob.rActivePane = EventVictoryScreenUI:new(rRenderLayer, rEvent)        
    else
        Ob.rActivePane = EventFailureScreenUI:new(rRenderLayer, rEvent)
    end
    Ob.rActivePane:setParent(Ob)

	return Ob
end

return EventEndScreenUI
