local UIElement = require('ui/UIElement')
local Character = require('Character')
local Ability = require('Ability/Ability')
local EventActor = require('event/EventActor')
local Renderer = require('Renderer')
local UIProgressBar = require('ui/UIProgressBar')
local TickProgressBar = require('ui/TickProgressBar')
local SpriteSheet = require('SpriteSheet')
local AbilityIcon = require('ui/AbilityIcon')
local EventUI = {}
local AbilityButton = {}
local AbilityPane = {}
local EventHeroDetailDisplay = {}
local EventStanceDisplay = {}
local ActorUIPane = {}
local StanceUIPane = {}
local MMUIPane = {}
local EventCharPortrait = {}
local EventItemPane = {}
local EventItemButton = {}
local DFGraphics = require('DFCommon.Graphics')
local Item = require('Item')

local tUITextureSpriteSheetInfo = 
{
fightscreen = 'UI/fightscreen',
abilityborder = 'UI/fightscreen',
abilityborder_active = 'UI/fightscreen',
numberbg = 'UI/fightscreen',
icon_power = 'UI/hud',
icon_shield = 'UI/hud',
icon_int = 'UI/hud',
colorbars_bot = 'UI/fightscreen',
colorbars_mid = 'UI/fightscreen',
colorbars_top = 'UI/fightscreen',
screen_crosshairs = 'UI/fightscreen',
backpanel = 'UI/fightscreen',
}

function AbilityButton:new(rRenderLayer, rHeroEventActor, rAbility)
    local Ob = UIElement:new(rRenderLayer)

    Ob.__typename = "AbilityButton"
    
    function Ob:getXOffset()
        return 104
    end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self.rIcon:setVisible(bVisible)
        local cost = self.rAbility:getManaCost()
        self.rManaBar:setVisible(bVisible and cost > 0)
        self:setElementsVisible(bVisible)      
        if self.rStanceIcon then
            self.rStanceIcon:setVisible(bVisible)
            self.rStancePortrait:setVisible(bVisible)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rIcon then
            self.rIcon:onPaneClosed()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    Ob.rHeroEventActor = rHeroEventActor
    Ob.rAbility = rAbility
    
    local kABILITY_PULSE_SCALE_MIN = 1.1
    local kABILITY_PULSE_SCALE_MAX = 1.2    
    local kABILITY_PULSE_ENABLED = true
    
    Ob.nAbilityPulseSpeed = (kABILITY_PULSE_SCALE_MAX - kABILITY_PULSE_SCALE_MIN) * 0.05

    Ob.rAbilityBorderActive = Ob:getUITextureProp('abilityborder_active')
    Ob.rAbilityBorderActive:setLoc(44, -44)
    Ob.rAbilityBorderActive:setScl(1.2)
    Ob:addElement(Ob.rAbilityBorderActive)

    Ob.rAbilityBorder = Ob:getUITextureProp('abilityborder')
    Ob.rAbilityBorder:setLoc(44, -44)
    Ob.rAbilityBorder:setScl(1.2)
    Ob:addElement(Ob.rAbilityBorder)
    
    -- re-add this border up front so the borders look the way they used to when pulsing is disabled
    if true ~= kABILITY_PULSE_ENABLED then
        Ob:removeElement(Ob.rAbilityBorderActive)
        Ob:addElement(Ob.rAbilityBorderActive)
    end
    
    Ob.rButton = MOAIProp2D.new()
    Ob.rButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rButton:setColor(0, 0, 0, 1)
    Ob.rButton:setLoc(0, 0)
    Ob.rButton:setVisible(false)
    Ob.rButton:setScl(90, 90)
    Ob:addElement(Ob.rButton)
    
    Ob.rCooldownOverlay = MOAIProp2D.new()
    Ob.rCooldownOverlay:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rCooldownOverlay:setColor(0.15, 0.15, 0.15, 0.5)
    Ob.rCooldownOverlay:setLoc(0, 0)
    Ob.nCooldownOverlayScale = 90
    Ob.rCooldownOverlay:setScl(Ob.nCooldownOverlayScale)
    Ob:addElement(Ob.rCooldownOverlay)

    Ob.rPressedButtonOverlay = MOAIProp2D.new()
    Ob.rPressedButtonOverlay:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPressedButtonOverlay:setColor(.91, .89, .43, 0.8)
    Ob.rPressedButtonOverlay:setLoc(0, 0)
    Ob.rPressedButtonOverlay:setScl(90)
    Ob.rPressedButtonOverlay:setVisible(false)
    Ob:addOverlayElement(Ob.rPressedButtonOverlay)

    Ob.rHeroDefeatedButtonOverlay = MOAIProp2D.new()
    Ob.rHeroDefeatedButtonOverlay:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rHeroDefeatedButtonOverlay:setColor(.4, .4, .4, 0.8)
    Ob.rHeroDefeatedButtonOverlay:setLoc(0, 0)
    Ob.rHeroDefeatedButtonOverlay:setScl(90)
    Ob.rHeroDefeatedButtonOverlay:setVisible(false)
    Ob:addOverlayElement(Ob.rHeroDefeatedButtonOverlay)

    Ob.rIcon = AbilityIcon:new(rRenderLayer, rAbility)
    Ob.rIcon:setParent(Ob)

    Ob.rManaBar = TickProgressBar:new(rRenderLayer, 86, {r=131/255,g=48/255,b=242/255}, {r=42/255,g=13/255,b=77/255}, {r=0/255,g=0/255,b=0/255}, {r=237/255,g=21/255,b=88/255}) --,  rHeroEventActor:getHero():getMaxMana(), 
    Ob.rManaBar:setLoc(0,-102)
    Ob.rManaBar:setParent(Ob)

    if rAbility:getDef().sAbilityType == "AbilityCycleStance" then
        Ob.rStancePortrait = rHeroEventActor:getHero():getPortraitProp()
        Ob.rStancePortrait:setParent(Ob)
        Ob.rStancePortrait:setLoc(0, 0)
        Ob.rStancePortrait:setScl(0.76, 0.76)
        Ob:addElement(Ob.rStancePortrait)

        Ob.rStanceIcon = Ob:getUITextureProp('icon_int')
        Ob.rStanceIcon:setLoc(44, -44)
        --Ob.rStanceIcon:setScl(0.76, 0.76)
        Ob:addElement(Ob.rStanceIcon)
    end

    function Ob:update()
        if self.rHeroEventActor:isDead() then
            self.rAbilityBorder:setVisible(true)
            self.rCooldownOverlay:setVisible(false)
            self.rPressedButtonOverlay:setVisible(false)
            self.rHeroDefeatedButtonOverlay:setVisible(true)
            self.rAbilityBorderActive:setVisible(false)
            self.rManaBar:setVisible(false)
            return
        end

        local nPending = 0
        if self.bAbilityQueued then
            if self.rAbility:isActive() then
                self.bAbilityQueued = false
                self.rPressedButtonOverlay:setVisible(false)
            else
                self.rPressedButtonOverlay:setVisible(true)
                nPending = self.rAbility:getManaCost()
            end
        else
            --self.rManaBar:setProgressPending(nil)
        end
        
        local pulseScale = kABILITY_PULSE_SCALE_MAX
        
        if self.rAbility:canPerformAbility() then
            self.rAbilityBorderActive:setVisible(true)
            
            if true ~= self.bAbilityQueued and kABILITY_PULSE_ENABLED then
                pulseScale = self.rAbilityBorder:getScl() + self.nAbilityPulseSpeed
                if self.nAbilityPulseSpeed > 0 and pulseScale >= kABILITY_PULSE_SCALE_MAX then
                    self.nAbilityPulseSpeed = -self.nAbilityPulseSpeed
                elseif self.nAbilityPulseSpeed < 0 and pulseScale <= kABILITY_PULSE_SCALE_MIN then
                    self.nAbilityPulseSpeed = -self.nAbilityPulseSpeed
                end
            end
        else
            self.rAbilityBorderActive:setVisible(false)
        end     
        
        self.rAbilityBorder:setScl(pulseScale)

        local nMaxMana = rHeroEventActor:getHero():getMaxMana()
        local nCurMana = rHeroEventActor:getHero():getCurMana()
        self.rManaBar:update(nCurMana-nPending, nMaxMana-nCurMana, nPending)
        
        local nPct = self.rAbility:getTurnCooldownPct()
        self.rCooldownOverlay:setScl(self.nCooldownOverlayScale, self.nCooldownOverlayScale * (1 - nPct))
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType ~= MOAITouchSensor.TOUCH_UP then return end

        if not self.rHeroEventActor:canPerformAbilities() then
            return true
        end
        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rButton then
                    if not self.rAbility:isActive() and self.rAbility:canPerformAbility() then
                        -- trigger the ability
                        if not self.bHidden and self.bVisible then
                            g_Audio:playCue('MMOJ_Audio/UI/RushTraining')
                            if self.rStanceIcon then
                                self.rAbility:performAbility(self.rHeroEventActor, nil, self.rHeroEventActor.rEvent)
                                local stanceIcon = "icon_int"
                                if self.rHeroEventActor.nCurStance == EventActor.STANCE_OFFENSE then
                                    stanceIcon = "icon_power"
                                elseif self.rHeroEventActor.nCurStance == EventActor.STANCE_DEFENSE then
                                    stanceIcon = "icon_shield"
                                end

                                local rSpriteSheet = self.tTextureToSpriteSheet[stanceIcon]
                                self.rStanceIcon:setIndex(rSpriteSheet.names[stanceIcon])
                            elseif self.rHeroEventActor:getHero().sCharDefKey == "MiddleManager" then
                                self.rAbility:performAbility(self.rHeroEventActor, nil, self.rHeroEventActor.rEvent)
                            else
                                self.rAbility:setChargeTime(self.rAbility:getMaxChargeTime())
                                self.rHeroEventActor:setQueuedAbility(self.rAbility)
                                if self.rAbility.bButton then
                                    local nMaxMana = rHeroEventActor:getHero():getMaxMana()
                                    local nCurMana = rHeroEventActor:getHero():getCurMana()
                                    local nPending = self.rAbility:getManaCost()
                                    self.rManaBar:update(nCurMana-nPending, nMaxMana-nCurMana, nPending)
                                end
                                self.bAbilityQueued = true
                            end
                        end
                    end
                    return true
                end
            end
        end
        return false
    end    

    return Ob
end

function AbilityPane:new(rRenderLayer, rHeroEventActor, bStance, rAbilityOverride)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)        
        if self.rButton then     
            self.rButton:setVisible(bVisible)
        end       
	end     

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rButton then      
            self.rButton:unloadUISpriteSheets()
        end
        self.rHeroEventActor = nil
    end   

    function Ob:update()
        if self.rButton then     
            self.rButton:update()
        end        
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if self.rButton then            
                self.rButton:onFinger(eventType, sx, sy, tProps)
            end
        end
        return false
    end    

    Ob.rHeroEventActor = rHeroEventActor
    local rAbility
    if rAbilityOverride then
        rAbility = rAbilityOverride
    elseif bStance then
        rAbility = rHeroEventActor:getHero():getStanceAbility()
    else
        rAbility = rHeroEventActor:getActiveAbility()
    end
    
    if rAbility then
        Ob.rButton = AbilityButton:new(rRenderLayer, rHeroEventActor, rAbility)
        rAbility.bButton = true
        Ob.rButton:setParent(Ob)
        Ob.rButton:setLoc(30, 0)
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    return Ob
end

function EventCharPortrait:new(rRenderLayer, rEventActor, rParentPane)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
	end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end    

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            local bHandled = false
            for i, rProp in ipairs(tProps) do
                if rProp == self.rIcon then
                    self.rParentPane:handleButtonPress()
                    return true
                end
            end
        end
        return false
    end

	Ob.rEventActor = rEventActor
    Ob.rParentPane = rParentPane

    local rHero = rEventActor:getHero()
    if rHero then
        Ob.rIcon = rHero:getPortraitProp()
        Ob.rIcon:setLoc(27, -24)
        Ob.rIcon:setScl(0.76, 0.76)
        Ob:addElement(Ob.rIcon)
    end

    return Ob
end

function EventHeroDetailDisplay:new(rRenderLayer, rEventActor, rActorPane, bStance, rAbilityOverride)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rAbilityPane:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rAbilityPane:onPaneClosed()
        self.rEventActor = nil
    end

	function Ob:update()
        self.rAbilityPane:update()
	end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            return self.rAbilityPane:onFinger(eventType, sx, sy, tProps)
        end
        return false
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

	Ob.rEventActor = rEventActor
    Ob.rActorPane = rActorPane

    Ob.rAbilityPane = AbilityPane:new(rRenderLayer, rEventActor, bStance, rAbilityOverride)
    Ob.rAbilityPane:setLoc(0, 0)
    Ob.rAbilityPane:setParent(Ob)

	return Ob
end

function ActorUIPane:new(rRenderLayer, bIncrementRight, rElementAtEnd, rEvent, tActors, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        for i, rButton in ipairs(self.tButtons) do
            rButton:setVisible(bVisible)
        end        
    end

    function Ob:onPaneClosed()
        for i, rButton in ipairs(self.tButtons) do
            rButton:onPaneClosed()
        end
    end

    function Ob:addHeroDetailDisplay(rEventActor, rAbilityOverride, nXOffsetOverride)
        local rNewButton = EventHeroDetailDisplay:new(self:getRenderLayer(), rEventActor, self, false, rAbilityOverride)
        local kX_OFFSET = nXOffsetOverride or 140
        if not self.bIncrementRight then
            kX_OFFSET = 0 - kX_OFFSET
        end
        if rNewButton then
            local nNumItems = table.getn(self.tButtons)
            table.insert(self.tButtons, rNewButton)
            rNewButton:setLoc(kX_OFFSET * nNumItems, 0)
            rNewButton:setParent(self)
            nNumItems = table.getn(self.tButtons)
            -- shift the last button
            if self.rElementAtEnd then
                self.rElementAtEnd:setLoc(kX_OFFSET * nNumItems, 0)
            end
        end
    end

    function Ob:enableAbilityButtons(bEnable)
        for i, rButton in ipairs(self.tButtons) do
            rButton:enableAbilityButton(bEnable)
        end
    end

    function Ob:update()
        for i, rButton in ipairs(self.tButtons) do
            rButton:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rParentUI and not self.rParentUI:areInteractiveElementsEnabled() then
            return true
        end
        if tProps then
            local bHandled = false
            for i, rButton in ipairs(self.tButtons) do
                bHandled = rButton:onFinger(eventType, sx, sy, tProps)
                if bHandled then
                    return true
                end
            end
        end
        return false
    end

    Ob.rEvent = rEvent
    Ob.tButtons = {}
    Ob.bVisible = false
    Ob.bIncrementRight = bIncrementRight
    Ob.rActivePane = nil
    Ob.rParentUI = rParentUI
    if rElementAtEnd then
        Ob.rElementAtEnd = rElementAtEnd
        Ob.rElementAtEnd:setParent(Ob)
        Ob.rElementAtEnd:setLoc(0, 0)
    end

    if tActors then
        for index, rActor in pairs(tActors) do	
            if rActor:getHero().sCharDefKey == "MiddleManager" then
                local tAbilities = rActor:getHero():getAbilities()
                for _,v in pairs(tAbilities) do
                    if v:isUnlocked() then
                        Ob:addHeroDetailDisplay(rActor, v, 160)
                    end
                end
            else
                Ob:addHeroDetailDisplay(rActor)
            end
        end
    end

    return Ob
end

function StanceUIPane:new(rRenderLayer, bIncrementRight, rElementAtEnd, rEvent, tActors, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        for i, rButton in ipairs(self.tButtons) do
            rButton:setVisible(bVisible)
        end        
    end

    function Ob:onPaneClosed()
        for i, rButton in ipairs(self.tButtons) do
            rButton:onPaneClosed()
        end
    end

    function Ob:addStanceDisplay(rEventActor)
        local rNewButton = EventHeroDetailDisplay:new(self:getRenderLayer(), rEventActor, self, true)
        local kX_OFFSET = 110
        if not self.bIncrementRight then
            kX_OFFSET = 0 - kX_OFFSET
        end
        if rNewButton then
            local nNumItems = table.getn(self.tButtons)
            table.insert(self.tButtons, rNewButton)
            rNewButton:setLoc(kX_OFFSET * nNumItems, 0)
            rNewButton:setParent(self)
            nNumItems = table.getn(self.tButtons)
            -- shift the last button
            if self.rElementAtEnd then
                self.rElementAtEnd:setLoc(kX_OFFSET * nNumItems, 0)
            end
        end
    end

    function Ob:enableAbilityButtons(bEnable)
        for i, rButton in ipairs(self.tButtons) do
            rButton:enableAbilityButton(bEnable)
        end
    end

    function Ob:update()
        for i, rButton in ipairs(self.tButtons) do
            rButton:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rParentUI and not self.rParentUI:areInteractiveElementsEnabled() then
            return true
        end
        if tProps then
            local bHandled = false
            for i, rButton in ipairs(self.tButtons) do
                bHandled = rButton:onFinger(eventType, sx, sy, tProps)
                if bHandled then
                    return true
                end
            end
        end
        return false
    end

    Ob.rEvent = rEvent
    Ob.tButtons = {}
    Ob.bVisible = false
    Ob.bIncrementRight = bIncrementRight
    Ob.rActivePane = nil
    Ob.rParentUI = rParentUI
    if rElementAtEnd then
        Ob.rElementAtEnd = rElementAtEnd
        Ob.rElementAtEnd:setParent(Ob)
        Ob.rElementAtEnd:setLoc(0, 0)
    end

    if tActors then
        for index, rActor in pairs(tActors) do	
            Ob:addStanceDisplay(rActor)
        end
    end

    return Ob
end

function EventItemButton:new(rRenderLayer, sItemDefKey, rEvent, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rIcon then
                        self:useItem()
                    end
                end
            end
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:useItem()
        if g_MiddleManager:useConsumable(self.sItemDefKey, self.rEvent) then
            if self.rDef.sUseCue then
                g_Audio:playCue(self.rDef.sUseCue)
            end
            local nNumItems = g_MiddleManager:getNumConsumablesInInventory(self.sItemDefKey)
            if nNumItems > 1 then
                self.rNumText:setString(tostring(nNumItems))
            else
                if self.rNumberBG and self.rNumText then
                    self.rNumberBG:setVisible(false)
                    self.rNumText:setReveal(0)
                end
            end
            if nNumItems <= 0 then
                self.rParentUI:onItemUsed(self)
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sItemDefKey = sItemDefKey
    Ob.rEvent = rEvent
    Ob.rDef = Item.getConsumableDef(sItemDefKey)
    local nNumItems = g_MiddleManager:getNumConsumablesInInventory(sItemDefKey)

    Ob.bUsed = false
    if Ob.rDef.iconName and Ob.rDef.sSpriteSheet then
        local tInfo = {}
        tInfo[Ob.rDef.iconName] = Ob.rDef.sSpriteSheet
        Ob:loadUISpriteSheets(tInfo)

        Ob.rIcon = Ob:getUITextureProp(Ob.rDef.iconName)
        if Ob.rIcon then
            Ob.rIcon:setLoc(0, 0)
            Ob.rIcon:setScl(0.75, 0.75)
            Ob:addElement(Ob.rIcon)
        end
    end
    if nNumItems > 1 then
        Ob.rNumberBG = Ob:getUITextureProp('numberbg')
        Ob.rNumberBG:setScl(1.2)
        Ob.rNumberBG:setLoc(74, -84)
        Ob:addElement(Ob.rNumberBG)

        Ob.rNumText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rNumText:setString(tostring(nNumItems))
        Ob.rNumText:setRect(-100, 80, 100, 0)
        Ob.rNumText:setColor(1, 1, 1)
        Ob.rNumText:setLoc(72, -71)
        Ob:addElement(Ob.rNumText)
    end

    Ob.rParentUI = rParentUI
    
    return Ob
end

function EventItemPane:new(rRenderLayer, rEvent, rParentUI)
	local Ob = UIElement:new(rRenderLayer)
    
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rButton in ipairs(self.tButtons) do
            rButton:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        for i, rButton in ipairs(self.tButtons) do
            rButton:onPaneClosed()
        end
    end

    function Ob:onItemUsed(rButton)
        if rButton then
            rButton:setVisible(false)
            rButton:onPaneClosed()
            for i, rExistingButton in ipairs(self.tButtons) do
                if rExistingButton == rButton then
                    table.remove(self.tButtons, i)
                end
            end            
            for i, rExistingButton in ipairs(self.tButtons) do
                rExistingButton:setLoc((i - 1) * self.nXOffset, 0)        
            end
        end
    end

    function Ob:scrollList(dx)
        local x, y = self:getLoc()
        local nNewX = x + dx
        if nNewX > self.nOrigX then
            nNewX = self.nOrigX
        end

        local nNumButtons = table.getn(self.tButtons)
        local nMinX = self.nOrigX - ((nNumButtons - 1) * self.nItemXSize)
        if nNewX < nMinX then
            nNewX = nMinX
        end

        if nNumButtons > 0 then
            local nNumItemsToHide = 0
            if nNewX < self.nOrigX then
                nNumItemsToHide = math.floor((self.nOrigX - nNewX) / self.nItemXSize)
            end
            for i, rButton in ipairs(self.tButtons) do
                if i <= nNumItemsToHide then
                    rButton:setVisible(false)
                else
                    rButton:setVisible(true)
                end
            end
        end

        self:setLoc(nNewX, y)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rParentUI and not self.rParentUI:areInteractiveElementsEnabled() then
            return true
        end
		if eventType == MOAITouchSensor.TOUCH_MOVE then
            if not self.bTouchMoved and not self.bTouchedInbounds then
                return true
            end
            if not self.lastSx then self.lastSx = sx end            
			local dx = sx - self.lastSx
            local kTOUCHMOVE_SCROLL_THRESHOLD = 5
			if self.bTouchMoved or math.abs(dx) > kTOUCHMOVE_SCROLL_THRESHOLD then
                self.bTouchMoved = true
                self.lastSx = sx
                self:scrollList(dx)
            end
        elseif eventType == MOAITouchSensor.TOUCH_DOWN then
            self.lastSx = sx
            self.bTouchedInbounds = false
            if tProps then
                local rItemPaneBG = self.rParentUI:getItemPaneBG()
                for i, rProp in ipairs(tProps) do
                    if rProp == rItemPaneBG then
                        self.bTouchedInbounds = true
                    end
                end
            end                   
        elseif eventType == MOAITouchSensor.TOUCH_UP then
            if not self.bTouchMoved and self.bTouchedInbounds then
                if tProps then
                    for i, rButton in ipairs(self.tButtons) do
                        rButton:onFinger(eventType, sx, sy, tProps)
                    end
                end        
            end            
            self.bTouchMoved = false
        end
    end

    function Ob:setAnchorPt(x, y)
        self.nOrigX = x
        self.nOrigY = y
        self:setLoc(x, y)
    end

    Ob.tButtons = {}
    local tItemInfos = g_MiddleManager:getAllConsumableItemInfos()
    
    Ob.nItemXSize = 100
    Ob.nXOffset = 100
    local nNumItems = 0
    
    local tButtonsByKey = {}
    
    for i, rItemInfo in pairs(tItemInfos) do
        local rNewButton = EventItemButton:new(rRenderLayer, rItemInfo.sItemDefKey, rEvent, Ob)
        tButtonsByKey[rItemInfo.sItemDefKey] = rNewButton
        
        table.insert(Ob.tButtons, rNewButton)
        rNewButton:setLoc(nNumItems * Ob.nXOffset, 0)        
        rNewButton:setParent(Ob)
        rNewButton:setScl(0.8)
        nNumItems = nNumItems + 1
    end
    
    Ob.nOrigX = 0
    Ob.nOrigY = 0
    Ob.bTouchMoved = false
    Ob.bTouchedInbounds = false
    Ob.rParentUI = rParentUI

    return Ob
end

function EventUI:new(rRenderLayer, rEvent)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rHeroUIPane:setVisible(bVisible)
        if self.rStanceUIPane then self.rStanceUIPane:setVisible(bVisible) end
        if self.rMMUIPane then self.rMMUIPane:setVisible(bVisible) end
        self.rItemPane:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroUIPane:onPaneClosed()
        if self.rStanceUIPane then self.rStanceUIPane:onPaneClosed() end
        if self.rMMUIPane then self.rMMUIPane:onPaneClosed() end
        self.rItemPane:onPaneClosed()
    end

    function Ob:update()
        if self.bInteractiveElementsEnabled then
            self.rHeroUIPane:update()
            if self.rStanceUIPane then self.rStanceUIPane:update() end
            if self.rMMUIPane then self.rMMUIPane:update() end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if not self.bInteractiveElementsEnabled then
            return true
        end
        self.rHeroUIPane:onFinger(eventType, sx, sy, tProps)
        if self.rStanceUIPane then self.rStanceUIPane:onFinger(eventType, sx, sy, tProps) end
        if self.rMMUIPane then self.rMMUIPane:onFinger(eventType, sx, sy, tProps) end
        self.rItemPane:onFinger(eventType, sx, sy, tProps)
        return true
    end

    function Ob:enableInteractiveUIElements(bEnable)
        local bChange = false
        if bEnable ~= self.bInteractiveElementsEnabled then
            bChange = true
        end

        self.bInteractiveElementsEnabled = bEnable  
        self.rHeroUIPane:setVisible(bEnable)  
        if self.rStanceUIPane then self.rStanceUIPane:setVisible(bEnable) end
        if self.rMMUIPane then self.rMMUIPane:setVisible(bEnable) end
        if self.rItemPane then self.rItemPane:setVisible(bEnable) end
        if bEnable and bChange then
            g_Game:getTutorialManager():onWindowOpen('EventInteractiveUI')
        end
    end

    function Ob:enableMMAbilities(bEnable)
        if self.rMMUIPane then self.rMMUIPane:setVisible(bEnable) end        
    end

    function Ob:areInteractiveElementsEnabled()
        return self.bInteractiveElementsEnabled
    end

    function Ob:getItemPaneBG()
        return self.rItemPaneBG
    end

    local kScreenWidth = 960
    local kScreenHeight = 480
    
    local kScreenTopLeftX = (Renderer.getUIWidth() - kScreenWidth) / 2
    local kScreenTopLeftY = (Renderer.getUIHeight() - kScreenHeight) / 2
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    Ob.rEvent = rEvent
    Ob.bInteractiveElementsEnabled = true

    Ob.rItemPaneBG = Ob:getUITextureProp('backpanel')
    Ob.rItemPaneBG:setLoc(756 + kScreenTopLeftX, -Renderer.getUIHalfHeight() + 270)
    Ob.rItemPaneBG:setScl(1.6,1)
    Ob:addElement(Ob.rItemPaneBG)

    Ob.rItemPane = EventItemPane:new(rRenderLayer, rEvent, Ob)
    Ob.rItemPane:setAnchorPt(602 + kScreenTopLeftX, -Renderer.getUIHalfHeight() + 306)
    Ob.rItemPane:setParent(Ob)

    Ob.rAbilityPaneBG = Ob:getUITextureProp('backpanel')
    Ob.rAbilityPaneBG:setLoc(220 + kScreenTopLeftX, -Renderer.getUIHalfHeight() + 270)
    Ob.rAbilityPaneBG:setScl(1.6,1)
    Ob:addElement(Ob.rAbilityPaneBG)

    Ob.rMMAbilityPaneBG = Ob:getUITextureProp('backpanel')
    Ob.rMMAbilityPaneBG:setLoc(476 + kScreenTopLeftX, -Renderer.getUIHalfHeight() - 280)
    Ob.rMMAbilityPaneBG:setScl(2, 1.2)
    Ob:addElement(Ob.rMMAbilityPaneBG)

    -- add some boxes for widescreen displays!
    Ob.rLeftBorderBox = MOAIProp2D.new()
    Ob.rLeftBorderBox:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLeftBorderBox:setLoc(0, 0)
    Ob.rLeftBorderBox:setScl(Renderer.getUIHalfWidth() - 480, Renderer.getUIHeight())
    Ob.rLeftBorderBox:setColor(0.18, 0.192, 0.188)
    Ob:addOverlayElement(Ob.rLeftBorderBox)
    
    Ob.rRightBorderBox = MOAIProp2D.new()
    Ob.rRightBorderBox:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRightBorderBox:setLoc(Renderer.getUIHalfWidth() + 480, 0)
    Ob.rRightBorderBox:setScl(Renderer.getUIHalfWidth() - 480, Renderer.getUIHeight())
    Ob.rRightBorderBox:setColor(0.18, 0.192, 0.188)
    Ob:addOverlayElement(Ob.rRightBorderBox)
    
    Ob.rBackground = Ob:getUITextureProp('fightscreen')
    Ob.rBackground:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob:addOverlayElement(Ob.rBackground)
    
    Ob.tFailureBars = {}
    local suffixes = {"bot","mid","top"}
    local nBottomX, nBottomY = Ob.rMMAbilityPaneBG:getLoc()
    local nBottomW, nBottomH = Ob.rMMAbilityPaneBG:getDims()
    nBottomH = nBottomH * 1.6
    local nYBarBottom = nBottomY+nBottomH*.5
    for i=3,1,-1 do
        local sTextureName = 'colorbars_'..suffixes[i]
        local rSpriteSheet = Ob.tTextureToSpriteSheet[sTextureName] 

        Ob.tFailureBars[i] = Ob:getUITextureProp(sTextureName)

        -- Texture filter "fix": rather than set up another spritesheet so we can set a custom filter mode,
        -- shrink the uv so it filters properly at large scale.
        local idx = rSpriteSheet.names[sTextureName]
        local x0,y0,x1,y1,x2,y2,x3,y3 = rSpriteSheet:getUVQuad(idx)
        if y0 == y1 then
            local diff = y2-y1
            diff = diff*.1
            y0 = y0+diff
            y1 = y1+diff
            y2 = y2-diff
            y3 = y3-diff
        else
            local diff = y1-y0
            diff = diff*.1
            y0 = y0+diff
            y3 = y3+diff
            y1 = y1-diff
            y2 = y2-diff
        end

        rSpriteSheet:setUVQuad(idx, x0,y0, x1,y1, x2,y2, x3,y3 )

        -- HACK: The third bar image is made to scale up crazy high, because setScl and
        -- alignSprite don't get along. You want to pass the scale to alignSprite, but
        -- since alignSprite is called every time loadUISpriteSheets is called, we can't
        -- be sure that if we call alignSprite here someone else won't come along and fuck
        -- up our alignment+scaling.
        -- So instead of properly fixing the sprite's scaled rect, we just make it crazy
        -- big so it doesn't matter if the scale or position is slightly wrong.
        local y = nYBarBottom-10
        if i == 2 then y = y+60
        elseif i == 3 then y = -Renderer.getUIHeight() end

        local h = 60
        if i == 3 then h = 3*Renderer.getUIHeight() end
        local sx,sy = Ob.tFailureBars[i]:getDims()
        local yScl = h/sy

        Ob.tFailureBars[i]:setLoc(Renderer.getUIHalfWidth(), y+h*.5)
        Ob.tFailureBars[i]:setScl(1,yScl)

        Ob.tFailureBars[i]:setVisible(false)
        Ob:addElement(Ob.tFailureBars[i])
    end

    Ob.rVictoryText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rVictoryText:setString("")
    Ob.rVictoryText:setRect(-130, 140, 130, 0)
    Ob.rVictoryText:setColor(1, 1, 1)
    Ob.rVictoryText:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight()+140)
    Ob.rVictoryText:setScl(1.5,1.5)
    Ob:addElement(Ob.rVictoryText)

    Ob.rCursor = MOAIProp2D.new()
    Ob.rCursor:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rCursor:setColor(1,1,1,1)
    Ob.rCursor:setLoc(-9999,-9999)
    Ob.rCursor:setScl(3, 65*1.5)
    Ob:addElement(Ob.rCursor)

    
    Ob.rStrangeRect = MOAIProp2D.new()
    Ob.rStrangeRect:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rStrangeRect:setColor(1,1,1,1)
    Ob.rStrangeRect:setLoc(-9999,-9999)
    Ob.rStrangeRect:setScl(30,7)
    Ob:addElement(Ob.rStrangeRect)

    Ob.tReticles = {}
    for i=1,4 do
        local nHorizLength = 180
        local nColorVal = .4
        local nWidth = 7
        local xMargin = 55 + ((Renderer.getUIWidth() - 960) / 2)
        local yOffset = (Renderer.getUIHeight() - 640) / 2
        local xCorner = ((i == 1 or i == 3) and xMargin) or (Renderer.getUIWidth()-xMargin)
        local yCorner = ((i == 1 or i == 2) and -(140 + yOffset)) or -(500 + yOffset)
        local horiz = MOAIProp2D.new()
        horiz:setDeck(g_GUI:GetOnePixelDeck())
        horiz:setColor(nColorVal,nColorVal,nColorVal,nColorVal)

        if i == 2 or i == 4 then
            horiz:setLoc(xCorner-nHorizLength+nWidth,yCorner)
        else
            horiz:setLoc(xCorner,yCorner)
        end

        horiz:setScl(nHorizLength, nWidth)
        Ob:addElement(horiz)

        local nVertLength = 90
        local vert = MOAIProp2D.new()
        vert:setDeck(g_GUI:GetOnePixelDeck())
        vert:setColor(nColorVal,nColorVal,nColorVal,nColorVal)

        if i == 3 or i == 4 then
            vert:setLoc(xCorner,yCorner+nVertLength)
        else
            vert:setLoc(xCorner,yCorner-nWidth)
        end

        vert:setScl(nWidth, nVertLength)
        Ob:addElement(vert)

        --nColorVal = .6
        if i == 1 then
            Ob.rHeroCamText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'bangers')
            Ob.rHeroCamText:setString(tostring("HERO CAM"))
            Ob.rHeroCamText:setRect(-50, 40, 100, 0)
            --Ob.rHeroCamText:setColor(nColorVal,nColorVal,nColorVal,nColorVal)
            Ob.rHeroCamText:setColor(1,1,1,.5)
            Ob.rHeroCamText:setLoc(xCorner+45, yCorner-10)
            Ob:addElement(Ob.rHeroCamText)
        elseif i == 2 then
            Ob.rLiveText = g_GUI:CreateTextBox(35, MOAITextBox.CENTER_JUSTIFY, 'bangers')
            Ob.rLiveText:setString(tostring("LIVE"))
            Ob.rLiveText:setRect(-50, 40, 50, 0)
            --Ob.rLiveText:setColor(nColorVal,nColorVal,nColorVal,nColorVal)
            Ob.rLiveText:setColor(1,1,1,.5)
            Ob.rLiveText:setLoc(xCorner-30, yCorner-10)
            Ob:addElement(Ob.rLiveText)
        end
    end

    local tHeroes = rEvent:getAllHeroActors()
    Ob.rHeroUIPane = ActorUIPane:new(Renderer.getRenderLayer('UIOverlay'), true, nil, rEvent, tHeroes, Ob)
    Ob.rHeroUIPane:setLoc(2 + kScreenTopLeftX, -Renderer.getUIHalfHeight() + 302)
    Ob.rHeroUIPane:setScl(0.75)
    Ob.rHeroUIPane:setParent(Ob)

    -- HACK: uh... the middle manager pretends to be an EventActor here.
    -- May want to re-architect this.
    local ea = {}
    function ea:getHero() return g_MiddleManager:getAvatar() end
    function ea:isDead() return false end
    function ea:canPerformAbilities() return true end
    function ea:isHero() return true end
    ea.rEvent = rEvent

    Ob.rMMUIPane = ActorUIPane:new(Renderer.getRenderLayer('UIOverlay'), true, nil, rEvent, {ea}, Ob)
    Ob.rMMUIPane:setLoc(250 + kScreenTopLeftX, -Renderer.getUIHalfHeight() - 244)
    Ob.rMMUIPane:setScl(0.75)
    Ob.rMMUIPane:setParent(Ob)

    --[[
    Ob.rStanceUIPane = StanceUIPane:new(Renderer.getRenderLayer('UIOverlay'), true, nil, rEvent, tHeroes, Ob)
    Ob.rStanceUIPane:setLoc(22 + kScreenTopLeftX, -Renderer.getUIHalfHeight() - 200)
    Ob.rStanceUIPane:setScl(0.75)
    Ob.rStanceUIPane:setParent(Ob)
    ]]--

	return Ob
end

return EventUI
