local Renderer = require('Renderer')
local Character = require('Character')
local util = require('DFUtil')
local ResourceBarUI = require('ui/ResourceBarUI')
local DFGraphics = require('DFCommon.Graphics')
local UIElement = require('ui/UIElement')
local MainMenuUI = require('ui/MainMenuUI')
local SpriteSheet = require('SpriteSheet')
local Button = require('ui/Button')
local GameOverlayUI = {}
local ScoreTab = {}

local tUITextureSpriteSheetInfo =
{
map_button = 'UI/hud',
button_roster = 'UI/hud',
button_shop = 'UI/hud',
button_help = 'UI/hud',
map_tab = 'UI/hud2',
map_number = 'UI/hud2',
clipboardtab = 'UI/hud2',
clipboard = 'UI/hud2',
branchtab = 'UI/hud2',
icon_new = 'UI/hud2',
tab1 = 'UI/shop', -- add the shop one to preload the shop textures
}

local kCOUNTUP_TIME = 3

function ScoreTab:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)   
   
        self.nPrevTime = MOAISim:getElapsedTime()
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()        
    end

    function Ob:isMoving()
        return false
    end

    function Ob:show()
        self:seekLoc(Renderer.getUIWidth() / 2 - 100, 0, 0, 1.2)
    end
    
    function Ob:hide()
        self:seekLoc(Renderer.getUIWidth() / 2 - 100, 200, 0, 1.2)
    end
    
    function Ob:update()    
        local kSECONDS_BEFORE_SLIDE_AWAY = 5.5
    
        if g_MiddleManager then
            if not self.nPrevTime then
                self.nPrevTime = 0
            end
            
            nCurTime = MOAISim:getElapsedTime()
            
            local nElapsed = nCurTime - self.nPrevTime
            
            self.nPrevTime = nCurTime
        
            if not self.nTimeSinceUpdated then
                self.nTimeSinceUpdated = 0
            end
            self.nTimeSinceUpdated = self.nTimeSinceUpdated + nElapsed

            local curLocX, curLocY, curLocZ = self:getWorldLoc()
            curLocX = Renderer.getUIWidth() / 2 - 100
            local kOFFSCREEN_LOC_Y = 200
        
            local endPos = 0
        
            if self.nTimeSinceUpdated > kSECONDS_BEFORE_SLIDE_AWAY then
                endPos = kOFFSCREEN_LOC_Y
            end
        
            curLocY = curLocY + ((endPos - curLocY) * .05)
            self:setLoc(curLocX, curLocY, curLocZ)
        
            if not self.nCurScore then 
                self.nCurScore = g_MiddleManager:getStarRatingScore()    
            end
            self.nDesiredScore = g_MiddleManager:getStarRatingScore()
            if self.nDesiredScore ~= self.nCurScore then
                self.nTimeSinceUpdated = 0
            
                if not self.nScoreDiff then
                    self.nScoreDiff = self.nDesiredScore - self.nCurScore
                end
                self.nCurScore = self.nCurScore + math.floor(self.nScoreDiff / (kCOUNTUP_TIME))
                if self.nCurScore >= self.nDesiredScore then
                    self.nCurScore = self.nDesiredScore
                    self.nScoreDiff = nil
                end
            end
            self.rScoreText:setString(tostring(self.nCurScore))
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rMapTab = Ob:getUITextureProp('branchtab')
    Ob.rMapTab:setLoc(0, 0)
    Ob:addElement(Ob.rMapTab)    

	Ob.rLabel = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rLabel:setString("BRANCH RATING")
    Ob.rLabel:setRect(0, 100, 80, 0)
    Ob.rLabel:setColor(.36, .3, .25)
	Ob.rLabel:setLoc(6, -4)
    Ob:addElement(Ob.rLabel)

	Ob.rScoreText = g_GUI:CreateTextBox(26, MOAITextBox.RIGHT_JUSTIFY, 'type')
	Ob.rScoreText:setString("")
    Ob.rScoreText:setRect(300, 100, 0, 0)
    Ob.rScoreText:setColor(.36, .3, .25)
	Ob.rScoreText:setLoc(224, -6)
    Ob:addElement(Ob.rScoreText)

    return Ob
end

function GameOverlayUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rResourceBar = ResourceBarUI:new(rRenderLayer)
    Ob.rResourceBar:setLoc(0, -Renderer.getUIHeight() - 100)
    Ob.rResourceBar:setParent(Ob)

    Ob.rMapTab = Ob:getUITextureProp('map_tab')
    Ob.rMapTab:setLoc(0, 0)
    Ob:addElement(Ob.rMapTab)

    Ob.rClipboardTab = Ob:getUITextureProp('clipboardtab')
    Ob.rClipboardTab:setLoc(Renderer.getUIWidth() - 160, 0)
    Ob:addElement(Ob.rClipboardTab)
    
    Ob.rClipboard = Ob:getUITextureProp('clipboard')
    Ob.rClipboard:setLoc(Renderer.getUIWidth() - 82, 0)
    Ob:addElement(Ob.rClipboard)    

    Ob.rAlertButton = SpriteSheet:new('UI/hud2', 'Characters/AnimDefs/HudAnims.lua', 0, 0, 'left', 'top')
	Ob.rAlertButton:setLoc(0, 20)
    Ob.rAlertButton:setParent(Ob)
    Ob:addElement(Ob.rAlertButton)

    Ob.rNumberBG = Ob:getUITextureProp('map_number')
    Ob.rNumberBG:setLoc(90, -40)
    Ob.rNumberBG:setVisible(false)
    Ob:addOverlayElement(Ob.rNumberBG)

	Ob.rNumEventsText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rNumEventsText:setString("10")
    Ob.rNumEventsText:setRect(-150, 80, 150, 0)
	Ob.rNumEventsText:setColor(1, 1, 1)
	Ob.rNumEventsText:setLoc(114, -50)
    Ob.rNumEventsText:setReveal(0)
    Ob:addOverlayElement(Ob.rNumEventsText)

    Ob.rShopButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_shop', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 })
    Ob.rShopButton:setLoc(Renderer.getUIWidth() - 292, -Renderer.getUIHeight() + 64)
    Ob.rShopButton:setParent(Ob)

    Ob.rShopNewAlert = Ob:getUITextureProp('icon_new')
    Ob.rShopNewAlert:setLoc(Renderer.getUIWidth() - 300, -(Renderer.getUIHeight() - 70))
    Ob.rShopNewAlert:setScl(0.7, 0.7)
    Ob.rShopNewAlert:setVisible(false)
    Ob:addOverlayElement(Ob.rShopNewAlert)
    
    Ob.rRosterButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_roster', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 })
    Ob.rRosterButton:setLoc(Renderer.getUIWidth() - 194, -Renderer.getUIHeight() + 64)
    Ob.rRosterButton:setParent(Ob)
    
    Ob.rHelpButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_help', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 })
    Ob.rHelpButton:setLoc(Renderer.getUIWidth() - 96, -Renderer.getUIHeight() + 64)
    Ob.rHelpButton:setParent(Ob)

    Ob.rScoreTab = ScoreTab:new(rRenderLayer)
    Ob.rScoreTab:setLoc(Renderer.getUIWidth() / 2 - 100, 0)
    Ob.rScoreTab:setParent(Ob)

    Ob.rTodoNumberBG = Ob:getUITextureProp('map_number')
    Ob.rTodoNumberBG:setLoc(90, -40)
    Ob.rTodoNumberBG:setVisible(false)
    Ob:addOverlayElement(Ob.rTodoNumberBG)

	Ob.rNumTodoText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rNumTodoText:setString("10")
    Ob.rNumTodoText:setRect(-150, 80, 150, 0)
	Ob.rNumTodoText:setColor(1, 1, 1)
	Ob.rNumTodoText:setLoc(114, -50)
    Ob.rNumTodoText:setReveal(0)
    Ob:addOverlayElement(Ob.rNumTodoText)

    function Ob:setTodoListVisible(bVisible)
        if self.bVisible then
            self.rClipboardTab:setVisible(bVisible)
            self.rClipboard:setVisible(bVisible)
        end
    end
    
    function Ob:setScoreTabVisible(bVisible)
        if self.bVisible then
            self.rScoreTab:setVisible(bVisible)
        end
    end

	-- functions
	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        self.rResourceBar:setVisible(bVisible)
        self.rShopButton:setVisible(bVisible)
        
        if bVisible and g_MiddleManager and g_MiddleManager:areAnyItemsNew() then
            self.rShopNewAlert:setVisible(true)
        else
            self.rShopNewAlert:setVisible(false)
        end
        
        self.rRosterButton:setVisible(bVisible)
        self.rHelpButton:setVisible(bVisible)
        self.rMapTab:setVisible(bVisible)
        self.rScoreTab:setVisible(bVisible)
        if not bVisible then            
            if self.rAlertButton then
                self.rAlertButton:setVisible(false)
            end
        end
	end

    function Ob:onPaneClosed()
        -- do nothing, leave resident
    end

	function Ob:update()
        self.rResourceBar:setLoc(0, -Renderer.getUIHeight() + 74)
        self.rShopButton:setLoc(Renderer.getUIWidth() - 292, -Renderer.getUIHeight() + 64)
        self.rShopNewAlert:setLoc(Renderer.getUIWidth() - 308, -(Renderer.getUIHeight() - 82))
        self.rRosterButton:setLoc(Renderer.getUIWidth() - 194, -Renderer.getUIHeight() + 64)
        self.rHelpButton:setLoc(Renderer.getUIWidth() - 96, -Renderer.getUIHeight() + 64)
        self.rClipboardTab:setLoc(Renderer.getUIWidth() - 160, 0)
        self.rClipboard:setLoc(Renderer.getUIWidth() - 82, 0)
        self.rTodoNumberBG:setLoc(Renderer.getUIWidth() - 100, -40)
        self.rNumTodoText:setLoc(Renderer.getUIWidth() - 76, -50)

        if not self.rScoreTab:isMoving() then
            self.rScoreTab:setLoc(Renderer.getUIWidth() / 2 - 100, 0)
            self.rScoreTab:update()
        end
        self.rResourceBar:update()
		if self.bVisible then
            if g_Game:isActionEnabled('OpenMap') then    
                self.rMapTab:setVisible(true)
                if g_City and g_City:areEventsQueued() then
                    -- show the alert icon
                    self.rAlertButton:setVisible(true)
                    if not self.bMapVisible then
                        self.rAlertButton:playAnimation('openmap')
                    end
                    self.bMapVisible = true
                    self.rNumberBG:setVisible(true)
                    self.rNumEventsText:revealAll()
                    self.rNumEventsText:setString(tostring(g_City:getNumEventsAvailable()))
                else
                    self.bMapVisible = false
                    self.rNumberBG:setVisible(false)
                    self.rNumEventsText:setReveal(0)     
                    self.rAlertButton:setVisible(true)
                    self.rAlertButton:playAnimation('opened')
                end
            else
                self.bMapVisible = false
                self.rMapTab:setVisible(false)
                self.rNumberBG:setVisible(false)
                self.rNumEventsText:setReveal(0)     
                self.rAlertButton:setVisible(false)
                self.rMapTab:setVisible(false)
            end
            if g_Game:isActionEnabled('Shop') then
                self.rShopButton:setVisible(true)
                self.rShopNewAlert:setVisible(g_MiddleManager:areAnyItemsNew())
            else
                self.rShopButton:setVisible(false)
                self.rShopNewAlert:setVisible(false)
            end
            if g_Game:isActionEnabled('ShowRoster') then
                self.rRosterButton:setVisible(true)
            else
                self.rRosterButton:setVisible(false)
            end
            if g_Game:isActionEnabled('ShowOptions') then
                self.rHelpButton:setVisible(true)
            else
                self.rHelpButton:setVisible(false)
            end
            if g_Game:isActionEnabled('OpenTodoList') then
                self.rClipboardTab:setVisible(true)
                self.rClipboard:setVisible(true)
                local nNumCompleted = g_TodoList:getNumItemsCompleted()
                if nNumCompleted > 0 then
                    self.rTodoNumberBG:setVisible(true)
                    self.rNumTodoText:revealAll()
                    self.rNumTodoText:setString(tostring(nNumCompleted))
                else
                    self.rTodoNumberBG:setVisible(false)
                    self.rNumTodoText:setReveal(0)                    
                end
            else
                self.rClipboardTab:setVisible(false)
                self.rClipboard:setVisible(false)
                self.rTodoNumberBG:setVisible(false)
                self.rNumTodoText:setReveal(0)
            end
		end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rShopButton then
                if g_Game:isActionEnabled('Shop') then
                    g_Audio:playCue('MMOJ_Audio/UI/Clipboard')
                    g_GUI:ShowShopUI()
                    return true
                end
            elseif rButton == self.rRosterButton then
                if g_Game:isActionEnabled('ShowRoster') then
                    g_GUI:ShowRosterMenu()
                    return true
                end
            elseif rButton == self.rHelpButton then
                if g_Game:isActionEnabled('ShowOptions') then
                    g_GUI:ShowOptionsPane()
                    --g_GUI:ShowMMMenu()
                    return true
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rShopButton:onFinger(eventType, sx, sy, tProps)
        self.rRosterButton:onFinger(eventType, sx, sy, tProps)
        self.rHelpButton:onFinger(eventType, sx, sy, tProps)
        self.rResourceBar:onFinger(eventType, sx, sy, tProps)
        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rAlertButton then
                    if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then
                        if g_Game:isActionEnabled('OpenMap') then
                            g_GUI:ShowAlertTerritoryMap()
                            return true
                        end
                    end
                elseif rProp == self.rClipboard then
                    if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then
                        if g_Game:isActionEnabled('OpenTodoList') then
                            g_GUI:ShowTodoList()
                            return true
                        end
                    end
                end
            end
        end
		return false
	end

	return Ob
end

return GameOverlayUI