local Gui = {}

local DFGraphics = require 'DFCommon.Graphics'
local file = require 'DFCommon.File'
local Renderer = require('Renderer')
local NothingActivity = require('activity/NothingActivity')
local Activity = require('activity/Activity')
local HeroSchedulerUI = require('ui/HeroSchedulerUI')
local TerritoryMapUI = require('ui/TerritoryMapUI')
local CloudSyncUI = require('ui/CloudSyncUI')
local EventDetailUI = require('ui/EventDetailUI')
local ScrollableUI = require('ui/ScrollableUI')
local GameOverlayUI = require('ui/GameOverlayUI')
local EventUI = require('ui/EventUI')
local RoomDetailsUI = require('ui/RoomDetailsUI')
local HeroCardSelectorUI = require('ui/HeroCardSelectorUI')
local LetterUI = require('ui/LetterUI')
local PromotionUI = require('ui/PromotionUI')
local SplashScreen = require('ui/SplashScreen')
local ShopUI = require('ui/ShopUI')
local DialogUI = require('ui/DialogUI')
local MiddleManagerUI = require('ui/MiddleManagerUI')
local PropActionHeroSelectorUI = require('ui/PropActionHeroSelectorUI')
local PremiumPurchaseSequenceUI = require('ui/PremiumPurchaseSequenceUI')
local EventEndScreenUI = require('ui/EventEndScreenUI')
local Newspaper = require('ui/Newspaper')
local RewardScreenUI = require('ui/RewardScreenUI')
local ResearchUI = require('ui/ResearchUI')
local TutorialArrow = require('ui/TutorialArrow')
local PostIt = require('ui/PostIt')
local HeroRosterUI = require('ui/HeroRosterUI')
local HeroCardDetailsUI = require('ui/HeroCardDetailsUI')
local MMSchedulerUI = require('ui/MMSchedulerUI')
local OptionsUI = require('ui/OptionsUI')
local AndroidOptionsUI = require('ui/AndroidOptionsUI')
local CreditsUI = require('ui/CreditsUI')
local util = require('DFUtil')
local FontList = require('ui/FontList')
local TodoListUI = require('ui/TodoListUI')
local BulletinBoardUI = require('ui/BulletinBoardUI')
local IntroCutscene = require('ui/IntroCutscene')
local ValentinesTerritoryMapUI = require('ui/ValentinesTerritoryMapUI')
local ValentinesEndCutscene = require('ui/ValentinesEndCutscene')
local MeteoriteCombinerUI = require('ui/MeteoriteCombinerUI')
local MeteoriteTutorialUI = require('ui/MeteoriteTutorialUI')
local MostWantedUI = require('ui/MostWantedUI')

-- vars
Gui.rCurActivePane = nil
Gui.rPopupPane = nil

local tCustomHolidayUI =
{
    ValentinesTerritoryMapUI = ValentinesTerritoryMapUI,
}

function Gui:getTexture(sTextureName) -- wired to look in the ui dir
    if tResSpecificTextures[sTextureName] then
        local viewportSizeX, viewportSizeY = Renderer.getViewportSize()
        sTextureName = 'UI/'..tResSpecificTextures[sTextureName]..tResSpecificTextSuffix[viewportSizeX..'x'..viewportSizeY]
        return DFGraphics.loadTexture(sTextureName)
    else
        return DFGraphics.loadTexture(sTextureName)
    end
end

function Gui:releaseTexture(sTextureName) -- wired to look in the ui dir
    if tResSpecificTextures[sTextureName] then
        local viewportSizeX, viewportSizeY = Renderer.getViewportSize()
        sTextureName = 'UI/'..tResSpecificTextures[sTextureName]..tResSpecificTextSuffix[viewportSizeX..'x'..viewportSizeY]
        DFGraphics.unloadTexture(sTextureName)
    else
        DFGraphics.unloadTexture(sTextureName)
    end
end

function Gui:init()
	self.rRenderLayer = Renderer.getRenderLayer('UI')
	
	-- create the fonts
	self.tFonts = {}
	self:initFont()
	
	self.rEventDetailDeck = nil
	self.rHeroSelectorDeck = nil
	self.rAddButtonDeck = nil
	self.rOnePixelDeck = nil
	self.rGreenButtonDeck = nil
	self.rEventDisplayBox = nil
    self.tSelectableHeroes = nil

	self.rGameOverlayUI = GameOverlayUI:new(self.rRenderLayer)

    self.rUpperDialogUI = DialogUI:new(Renderer.getRenderLayer('UIOverlay'))

    self.rLowerDialogUI = DialogUI:new(Renderer.getRenderLayer('UIOverlay'))

    self.rChooserListener = nil
    
    self.tWindowCloseListener = {}
    self.tWindowOpenListener = {}
    
    self.tPostIts = {}

    self.rOnePixelTexture = DFGraphics.loadTexture('MiscNonTPS/1x1.png')

    self.rUIThread = MOAIThread.new()
    self.rUIThread:run(self.uiThread, self, self.rUIThread)
end

function Gui:shutdown()
    self:HideTutorialArrow()
    self:SetActivePane(nil)
    self:SetPopupPane(nil)
end

function Gui:uiThread(rThread)
    while self.rUIThread == rThread do
        if self.rCurActivePane and self.rCurActivePane.update then
            self.rCurActivePane:update()
        end
        coroutine.yield()
    end
    print("end current UI thread")
end

function Gui:initFont()
    self.tFonts = {}
    local tFonts = { 'Fonts/default.font', 'Fonts/bangers.font', 'Fonts/type.font', 'Fonts/ptsans.font', 'Fonts/boogaloo.font' }
    for i, sFontName in ipairs(tFonts) do
        self.tFonts[sFontName] = DFGraphics.loadFont(sFontName)
    end
    local i = 0
--[[
    self.tFonts = {}
	local charcodes = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789 .,:;!?()@&/-%+$£¥?'#_"
    
	local hiresFonts = true
    if Renderer.getCurScreenInfo() and Renderer.getCurScreenInfo().key == 'SMALL' then
        hiresFonts = false
    end
    hiresFonts = true

    function initMOAIFont(fontPath, defaultFontSize)
        -- this is the recommended way to load a font lua... I don't like it!

        local fontFile = io.open ( file.getDataPath(fontPath .. ".lua"), 'r' )
        local font = loadstring ( fontFile:read ( '*all' ))()
        fontFile:close ()
        
        local image = MOAIImage.new()
        image:load(file.getDataPath(fontPath .. ".png"), MOAIImage.PREMULTIPLY_ALPHA)
        
        font:setDefaultSize(defaultFontSize)
        
        local glyphCache = MOAIStaticGlyphCache.new ()
        glyphCache:setColorFormat ( MOAIImage.COLOR_FMT_RGBA_8888 )
        font:setCache ( glyphCache )
        
        font:setImage(image)
        
        return font
    end
    
    local tFontList = FontList:get()
    
    for i, tFontInfo in ipairs(tFontList) do
        local font = nil
        
        if hiresFonts then
            font = initMOAIFont("Fonts/" .. tFontInfo.fontFile .. "_hires", tFontInfo.defaultSize)
        else
            font = initMOAIFont("Fonts/" .. tFontInfo.fontFile, tFontInfo.defaultSizeSmall)
        end
        
        for idx, fontAlias in ipairs(tFontInfo.aliases) do
            self.tFonts[fontAlias] = font
        end
    end
]]--
end

function Gui:getFont(sName)
	if not sName then sName = "default" end
    sName = 'Fonts/'..sName..'.font'
	return self.tFonts[sName]
end

function Gui:clearAndReloadFonts()
    self.tFonts = nil
    MOAISim:forceGarbageCollection()
    self:initFont()
end

function Gui:CreateTextBox(nSize, hAlignment, sFont, vAlignment)
	local rTextBox = MOAITextBox.new()
    local actualFont = self:getFont(sFont)
	rTextBox:setFont(actualFont)
    local rStyle = rTextBox:affirmStyle()
    if actualFont.bmFont then
        rStyle:setSize(actualFont:getDefaultSize())
        local scale = nSize/actualFont:getDefaultSize()
        scale = 1 - scale
        scale = scale * 0.5
        scale = 1 - scale
        rTextBox:setGlyphScale(scale)
    else
        rStyle:setSize(nSize)
        rTextBox:setGlyphScale(1)
    end
	rTextBox:setRect(-300, 80, 300, 0)
	rTextBox:setAlignment(hAlignment, vAlignment)
	rTextBox:setYFlip(true)
	rTextBox:setColor(0, 0, 0)
	
	return rTextBox
end

function Gui:GetEventDetailDeck()
	if not self.rEventDetailDeck then
		self.rEventDetailDeck = MOAIGfxQuad2D.new()
		self.rEventDetailDeck:setTexture (util.getAssetName("UI/event_halftone.png"))
		self.rEventDetailDeck:setRect(-320, -160, 320, 160)
	end
	return self.rEventDetailDeck
end

function Gui:GetHeroSelectorDeck()
	if not self.rHeroSelectorDeck then
		self.rHeroSelectorDeck = MOAIGfxQuad2D.new()
		self.rHeroSelectorDeck:setTexture (util.getAssetName("UI/event_hero.png"))
		self.rHeroSelectorDeck:setRect(-320, -108, 320, 108)
	end
	return self.rHeroSelectorDeck
end

function Gui:GetAddButtonDeck()
	if not self.rAddButtonDeck then
		self.rAddButtonDeck = MOAIGfxQuad2D.new()
		self.rAddButtonDeck:setTexture (util.getAssetName("UI/button_add_hero.png"))
		self.rAddButtonDeck:setRect(-320, -60, 320, 60)
	end
	return self.rAddButtonDeck
end

function Gui:GetOnePixelTexture()
    if not self.rOnePixelTexture then
        self.rOnePixelTexture = DFGraphics.loadTexture('MiscNonTPS/1x1.png')
    end    
    return self.rOnePixelTexture
end

function Gui:GetOnePixelDeck()
	if not self.rOnePixelDeck then
		self.rOnePixelDeck = MOAIGfxQuad2D.new() 
		self.rOnePixelDeck:setTexture(self:GetOnePixelTexture())
        self.rOnePixelDeck:setRect(0, -1, 1, 0)
	end
	return self.rOnePixelDeck
end

function Gui:GetGreenButtonDeck()
	if not self.rGreenButtonDeck then
		self.rGreenButtonDeck = MOAIGfxQuad2D.new()
		self.rGreenButtonDeck:setTexture(util.getAssetName("UI/button_green.png"))
		self.rGreenButtonDeck:setRect(-320, -60, 320, 60)
	end
	return self.rGreenButtonDeck
end

function Gui:GetEventDisplayBoxDeck()
	if not self.rEventDisplayBox then
		self.rEventDisplayBox = MOAIGfxQuad2D.new()
		self.rEventDisplayBox:setTexture(util.getAssetName("UI/CombatStatbox.png"))
		self.rEventDisplayBox:setRect(-56, -56, 56, 56)
	end
	return self.rEventDisplayBox
end

function Gui:GetActivePane()
    return self.rCurActivePane
end

function Gui:SetActivePane(rPane, bPauseGame)
    local rOldPane = self.rCurActivePane
	if self.rCurActivePane ~= nil then
		self.rCurActivePane:setVisible(false)
        if self.rCurActivePane.onPaneClosed then
            self.rCurActivePane:onPaneClosed()
        end
	end
	if bPauseGame then
		SetGameState("Pause")
	else
		SetGameState("Run")
	end
	self.rCurActivePane = rPane
	-- for no ui, show the game overlay
	if not rPane then
		self:ShowGameOverlayUI()
	end
end

function Gui:SetPopupPane(rPane, bPauseGame)
    if self.rPopupPane then
        self.rPopupPane:setVisible(false)
    end
    if rPane then
        rPane:setVisible(true)
        self.rPopupPane = rPane
    end
	if bPauseGame then
		SetGameState("Pause")
	else
		SetGameState("Run")
	end
    self.rPopupPane = rPane
end

function Gui:addWindowCloseListener(rListener)
    table.insert(self.tWindowCloseListener, rListener)
end

function Gui:removeWindowCloseListener(rListenerToRemove)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        if rListener == rListenerToRemove then
            table.remove(self.tWindowCloseListener, i)
            break
        end
    end
end

function Gui:addWindowOpenListener(rListener)
    table.insert(self.tWindowOpenListener, rListener)
end

function Gui:removeWindowOpenListener(rListenerToRemove)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        if rListener == rListenerToRemove then
            table.remove(self.tWindowOpenListener, i)
            break
        end
    end
end

function Gui:ShowHeroScheduler(rHero)
	if not rHero then
		print("no hero passed to ShowHeroScheduler!")
		return
	end
    local rHeroSchedulerUI = nil
    if rHero == g_MiddleManager:getAvatar() then
        rHeroSchedulerUI = MMSchedulerUI:new(self.rRenderLayer, rHero)
    else        
        rHeroSchedulerUI = HeroSchedulerUI:new(self.rRenderLayer, rHero)
    end
    rHeroSchedulerUI:setLoc(0, 0)
	self:SetActivePane(rHeroSchedulerUI)
	rHeroSchedulerUI:setVisible(true)
	rHero:centerOnScreen(-250, 0, 0.25)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('HeroScheduler')
    end
end

function Gui:HideHeroScheduler()
	self:SetActivePane(nil)	
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('HeroScheduler')
    end
end

function Gui:ShowPremiumPurchaseSequence(sItemType, rItem)
    local rPremiumPurchaseUI = PremiumPurchaseSequenceUI:new(self.rRenderLayer, sItemType, rItem)
    self:SetActivePane(rPremiumPurchaseUI, true)
    rPremiumPurchaseUI:setVisible(true)
end

function Gui:HidePremiumPurchaseSequence()
    self:SetActivePane(nil)
end

function Gui:ShowLetterScreen(sTitle, sMessage, rListener)
    local rLetterScreen = LetterUI:new(Renderer.getRenderLayer('UIOverlay'), sTitle, sMessage, rListener)
    rLetterScreen:setVisible(true)   
    
    g_GUI:SetPopupPane(rLetterScreen, true)
end

function Gui:HideLetterScreen()
    g_GUI:SetPopupPane(nil)
end

function Gui:ShowCloudScreen(bShowSaveButton, rParentUI)
    local rCloudScreen = CloudSyncUI:new(Renderer.getRenderLayer('UIOverlay'), bShowSaveButton, rParentUI)
    rCloudScreen:setVisible(true)   
    
    g_GUI:SetPopupPane(rCloudScreen, true)
end

function Gui:HideCloudScreen()
    g_GUI:SetPopupPane(nil)
end

function Gui:ShowEventDetailUI(rEvent)
    local rEventDetailUI = EventDetailUI:new(self.rRenderLayer, rEvent)
	self:SetActivePane(rEventDetailUI, false)
	rEventDetailUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('EventDetail')
    end
end

function Gui:HideEventDetailUI()
	self:SetActivePane(nil)	
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('EventDetail')
    end
end

function Gui:ShowTerritoryMap(rHero, nActivityIndex)
    local rTerritoryMapUI = TerritoryMapUI:new(self.rRenderLayer, TerritoryMapUI.eMapModes.MM_AreaSelect, rHero, nActivityIndex)
	self:SetActivePane(rTerritoryMapUI, true)
	rTerritoryMapUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('TerritoryMap')
    end
end

function Gui:HideTerritoryMap()
	self:SetActivePane(nil)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('TerritoryMap')
    end
end

function Gui:ShowAlertTerritoryMap()
    local rTerritoryMapUI = TerritoryMapUI:new(self.rRenderLayer, TerritoryMapUI.eMapModes.MM_EventSelect)
	self:SetActivePane(rTerritoryMapUI, true)
	rTerritoryMapUI:setVisible(true)    
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('TerritoryMap')
    end
end

function Gui:ShowShopUI(rHero)
    local rShopUI = ShopUI:new(self.rRenderLayer)
    self:SetActivePane(rShopUI, true)
    rShopUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('ShopUI')
    end
end

function Gui:HideShopUI()
    self:SetActivePane(nil)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('ShopUI')
    end
end

function Gui:ShowCharInventoryUI(rHero)
    local rShopUI = ShopUI:new(self.rRenderLayer, rHero)
	self:SetActivePane(rShopUI, true)
    rShopUI:setVisible(true, 'Inventory')
end

function Gui:ShowHeroShopUI()
    local rShopUI = ShopUI:new(self.rRenderLayer)
    self:SetActivePane(rShopUI, true)
    rShopUI:setVisible(true)
    rShopUI:selectTab('Shop')
    rShopUI.rShopCategoryPane:selectCategory('Heroes')
end

function Gui:ShowCurrencyUI()
    if self.rCurActivePane and self.rCurActivePane.bIsShopUI then
        rShopUI = self.rCurActivePane
    else
        rShopUI = ShopUI:new(self.rRenderLayer)
        self:SetActivePane(rShopUI, true)
        rShopUI:setVisible(true)
    end
    rShopUI:selectTab('Shop')
    rShopUI.rShopCategoryPane:selectCategory('RMT')
end

function Gui:ShowRMTUI()
    if g_Game:isActionEnabled('RMTPurchase') and g_Game:isActionEnabled('Shop') then
        if self.rCurActivePane and self.rCurActivePane.bIsShopUI then
            rShopUI = self.rCurActivePane
        else
            rShopUI = ShopUI:new(self.rRenderLayer)
            self:SetActivePane(rShopUI, true)
            rShopUI:setVisible(true)
        end
        rShopUI:selectTab('Shop')
        rShopUI.rShopCategoryPane:selectCategory('RMT')
    end
end

function Gui:ShowEventEndScreenUI(rEvent)
	local rEventEndScreenUI = EventEndScreenUI:new(self.rRenderLayer, rEvent)
	self:SetActivePane(rEventEndScreenUI, true)
	rEventEndScreenUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('EventEndScreen')
    end
end

function Gui:HideEventEndScreenUI()
	self:SetActivePane(nil)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('EventEndScreen')
    end
end

function Gui:HideHeroSelector()
	self.rCurActivePane:setVisible(false)
	self.rCurActivePane = nil
end

function Gui:ShowPropActionHeroSelector(rProp, tHeroes, rChooserListener, sLabel)
    if not tHeroes then
        return
    end
    
    local numHeroes = 0
    
    for i,rHero in pairs(tHeroes) do
        numHeroes = numHeroes + 1
    end
    
    if numHeroes == 0 then
        return
    end
    
    local rHeroChooserUI = PropActionHeroSelectorUI:new(self.rRenderLayer)
    rHeroChooserUI:Show(rProp, tHeroes, rChooserListener, sLabel)
    self:SetActivePane(rHeroChooserUI)
    
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('HeroChooser')
    end
end

function Gui:HidePropActionHeroSelectorUI()
    self:SetActivePane(nil)
    
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('HeroChooser')
    end
end

function Gui:ShowRoomDetailsUI(rRoom)
	if not rRoom or not rRoom:shouldShowRoomDetails() then
		return
	end
	
    local rRoomDetailsUI = RoomDetailsUI:new(self.rRenderLayer, rRoom)
    rRoomDetailsUI:setLoc(0, 0)
	self:SetActivePane(rRoomDetailsUI)
	rRoomDetailsUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('RoomDetails')
    end
end

function Gui:HideRoomDetailsUI()
	self:SetActivePane(nil)	
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('RoomDetails')
    end
end

function Gui:ShowMMMenu()
    self.rMiddleManagerUI = MiddleManagerUI:new(self.rRenderLayer)
    self.rMiddleManagerUI:setLoc(Renderer.getUIWidth() / 2, -100)
    self:SetActivePane(self.rMiddleManagerUI, true)
    self.rMiddleManagerUI:setVisible(true)
end

function Gui:HideMMMenu()
    self:SetActivePane(nil)
end

function Gui:ShowGameOverlayUI()
	self:SetActivePane(self.rGameOverlayUI, false)
	self.rGameOverlayUI:setVisible(true)
    
    g_Game:onGameOverlayShown()
end

function Gui:HideGameOverlayUI()
	self:SetActivePane(nil)
end

function Gui:GetGameOverlayUI()
	return self.rGameOverlayUI
end

function Gui:ShowHeroCardSelectorUI(rPackDef)	
	local rHeroCardSelectorUI = HeroCardSelectorUI:new(self.rRenderLayer, rPackDef)
	self:SetActivePane(rHeroCardSelectorUI, true)
	rHeroCardSelectorUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('HeroCardSelector')
    end
end

function Gui:HideHeroCardSelectorUI()
	self:SetActivePane(nil)	
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('HeroCardSelector')
    end
end

function Gui:ShowLevelUpUI(rHero)
	local rLevelUpUI = PromotionUI:new(self.rRenderLayer, rHero)
    rLevelUpUI:setLoc((Renderer.getUIWidth() / 2) - (rLevelUpUI:getWidth() / 2), -50)
	self:SetActivePane(rLevelUpUI, true)
	rLevelUpUI:setVisible(true)
end

function Gui:HideLevelUpUI()
	self:SetActivePane(nil)		
end

function Gui:ShowRewardPane(rRewardInfo, closeCB, rTerritory)
    if rRewardInfo then
        local rRewardScreenUI = RewardScreenUI:new(self.rRenderLayer, rRewardInfo, closeCB, rTerritory)
        self:SetActivePane(rRewardScreenUI, true)
        rRewardScreenUI:setVisible(true)
        for i, rListener in ipairs(self.tWindowOpenListener) do
            rListener:onWindowOpen('RewardPane')
        end
    end
end

function Gui:HideRewardPane()
	self:SetActivePane(nil)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('RewardPane')
    end    
end

function Gui:ShowNewspaper(rEvent)
    local rNewsPaper = Newspaper:new(self.rRenderLayer, rEvent)
    self:SetActivePane(rNewsPaper, true)
    rNewsPaper:setVisible(true)
end

function Gui:ShowResearchUI(rHero, nActivityIndex)
    if rHero then
        local rResearchUI = ResearchUI:new(self.rRenderLayer, rHero, nActivityIndex)
        self:SetActivePane(rResearchUI, true)
        rResearchUI:setVisible(true)
    end
end

function Gui:ShowMeteoriteCombiner()
    local rMeteoriteUI = MeteoriteCombinerUI:new(self.rRenderLayer)
    self:SetActivePane(rMeteoriteUI)
    rMeteoriteUI:setVisible(true)
end

function Gui:HideMeteoriteCombiner()
    self:SetActivePane(nil)
end

function Gui:ShowRosterMenu()
    local rHeroRosterUI = HeroRosterUI:new(self.rRenderLayer)
    self:SetActivePane(rHeroRosterUI, false)
    rHeroRosterUI:setVisible(true)
end

function Gui:HideRosterMenu()
    self:SetActivePane(nil)
    for i, rListener in ipairs(self.tWindowCloseListener) do
        rListener:onWindowClosed('HeroRoster')
    end
end

function Gui:ShowHeroCardDetailsUI(rHero)
    local rHeroCardDetailsUI = HeroCardDetailsUI:new(self.rRenderLayer, rHero)
    self:SetActivePane(rHeroCardDetailsUI, true)
    rHeroCardDetailsUI:setVisible(true)
end

function Gui:HideHeroCardDetailsUI()
    self:SetActivePane(nil)
end

function Gui:ShowSplashScreen()
	local rSplashScreen = SplashScreen:new(self.rRenderLayer)
	self:SetActivePane(rSplashScreen, true)
	rSplashScreen:setVisible(true)
end

function Gui:ShowEventUI(rEvent)
	local rEventUI = EventUI:new(self.rRenderLayer, rEvent)
    self:SetActivePane(rEventUI, true)
    rEventUI:setVisible(true)
    for i, rListener in ipairs(self.tWindowOpenListener) do
        rListener:onWindowOpen('EventUI')
    end

    return rEventUI
end

function Gui:HideSplashScreen()
	self:ShowGameOverlayUI()
    if g_Game then
        g_Game:onSplashScreenClosed()
    end
end

function Gui:DisplayUpperDialogText(rCharacter, sText, bBlocking, bWaitForInput)
    if not self.rUpperDialogUI then
        self.rUpperDialogUI = DialogUI:new(Renderer.getRenderLayer('UIOverlay'))
    end
    self.rUpperDialogUI:setLoc(0, 0)
    self.rUpperDialogUI:setVisible(true)
    self.rUpperDialogUI:displayLine(rCharacter, sText, bBlocking, bWaitForInput)
end

function Gui:DisplayLowerDialogText(rCharacter, sText, bBlocking, bWaitForInput)
    if not self.rLowerDialogUI then
        self.rLowerDialogUI = DialogUI:new(Renderer.getRenderLayer('UIOverlay'))
    end
    self.rLowerDialogUI:setLoc(0, -Renderer.getUIHeight() + 140)
    self.rLowerDialogUI:setVisible(true)
    self.rLowerDialogUI:displayLine(rCharacter, sText, bBlocking, bWaitForInput)
end


function Gui:HideDialogText()
    if self.rUpperDialogUI then
        self.rUpperDialogUI:setVisible(false)
    end
    if self.rLowerDialogUI then
        self.rLowerDialogUI:setVisible(false)
    end
    self.rUpperDialogUI = nil
    self.rLowerDialogUI = nil
end

function Gui:ShowTutorialArrow(x, y, nRot, nAnimX, nAnimY, tColor, rParent, rLayerOverride)
    local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
    -- for now only one arrow
    if self.rTutorialArrow then
        self:HideTutorialArrow()
    end
    self.rTutorialArrow = TutorialArrow:new(rLayerOverride or rOverlayLayer)
    self.rTutorialArrow:setLoc(x, y)
    self.rTutorialArrow:setAttr(MOAITransform2D.ATTR_Z_ROT, nRot)
    self.rTutorialArrow:setColor(tColor)
    self.rTutorialArrow:setVisible(true)
    if nAnimX and nAnimY then
        self.rTutorialArrow:animate(nAnimX, nAnimY)
    end
    if rParent then
        self.rTutorialArrow:setParent(rParent)
    end
    return self.rTutorialArrow
end

function Gui:HideTutorialArrow()
    if self.rTutorialArrow then
        self.rTutorialArrow:setVisible(false)
        self.rTutorialArrow:onPaneClosed()
        self.rTutorialArrow = nil
    end    
end

function Gui:ShowPostIt(sMessageText, x, y, rLayerOverride, bHideWhenTapped, bBlockInputUntilTapped)
    local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
    local rPostIt = PostIt:new(rLayerOverride or rOverlayLayer, sMessageText)
    rPostIt:setLoc(x, y)
    if bHideWhenTapped then
        rPostIt:setHideWhenTapped(bHideWhenTapped)
    end
    if bBlockInputUntilTapped then
        rPostIt:setBlockInputUntilTapped(bBlockInputUntilTapped)
    end
    rPostIt:setVisible(true)
    table.insert(self.tPostIts, rPostIt)

    return rPostIt
end

function Gui:ClearPostIts()
    for i, rPostIt in ipairs(self.tPostIts) do
        rPostIt:setVisible(false)
        rPostIt:onPaneClosed()
    end
    self.tPostIts = {}
end

function Gui:HasPostIts()
    if self.tPostIts and table.getn(self.tPostIts) > 0 then
        return true
    end
    return false
end

function Gui:ShowOptionsPane()
    local rOptionsUI = nil
    if MOAIEnvironment.osBrand == "Android" then
        rOptionsUI = AndroidOptionsUI:new(self.rRenderLayer)
    else
        rOptionsUI = OptionsUI:new(self.rRenderLayer)
    end
    self:SetActivePane(rOptionsUI, true)
    rOptionsUI:setVisible(true)
end

function Gui:HideOptionsPane()
    self:SetActivePane(nil)
end

function Gui:ShowCreditsUI()
    local rCreditsUI = CreditsUI:new(self.rRenderLayer)
    self:SetActivePane(rCreditsUI, true)
    rCreditsUI:setVisible(true)
end

function Gui:HideCreditsUI()
    self:SetActivePane(nil)
end

function Gui:ShowTodoList()
    local rTodoList = TodoListUI:new(self.rRenderLayer)
    self:SetActivePane(rTodoList, false)
    rTodoList:setVisible(true)
end

function Gui:HideTodoList()
    self:SetActivePane(nil)
end

function Gui:ShowBulletinBoard()
    local rBulletinBoard = BulletinBoardUI:new(self.rRenderLayer)
    self:SetActivePane(rBulletinBoard, false)
    rBulletinBoard:setVisible(true)
end

function Gui:HideBulletinBoard()
    self:SetActivePane(nil)
end

function Gui:ShowIntroCS()
    local rIntroCS = IntroCutscene:new(self.rRenderLayer)
    self:SetActivePane(rIntroCS)
    rIntroCS:setVisible(true)

    return rIntroCS
end

function Gui:HideIntroCS()
    self:SetActivePane(nil)
end

function Gui:ShowValentinesEndCS()
    if not self.bValentinesCSRunning then
        self.bValentinesCSRunning = true
        local rCS = ValentinesEndCutscene:new(self.rRenderLayer)
        self:SetActivePane(rCS)
        rCS:setVisible(true)
    end

    return rCS
end

function Gui:HideValentinesEndCS()
    self.bValentinesCSRunning = false
    self:SetActivePane(nil)
end

function Gui:ShowMeteoriteTutorialUI()
    local rPane = MeteoriteTutorialUI:new(self.rRenderLayer)
    self:SetActivePane(rPane)
    rPane:setVisible(true)
end

function Gui:HideMeteoriteTutorialUI()
    self:SetActivePane(nil)
end

function Gui:ShowMostWantedUI()
    local rPane = MostWantedUI:new(self.rRenderLayer)
    self:SetActivePane(rPane)
    rPane:setVisible(true)
end

function Gui:HideMostWantedUI()
    self:SetActivePane(nil)
end

function Gui:getCustomHolidayUI(sName)
    if sName and tCustomHolidayUI[sName] then
        return tCustomHolidayUI[sName]
    end
end

function Gui:GetHolidayGUI(sUIKey)
    local tUIClasses = {}
    if sUIKey then
        local tHolidays = g_Game:getActiveHolidays()
        if tHolidays then
            for i, rHolidayInfo in ipairs(tHolidays) do
                if rHolidayInfo.tCustomUI and rHolidayInfo.tCustomUI[sUIKey] then
                    local uiClass = self:getCustomHolidayUI(rHolidayInfo.tCustomUI[sUIKey])
                    table.insert(tUIClasses, uiClass)
                end
            end
        end
    end
    return tUIClasses
end

function Gui:onFinger(eventType, sx, sy)
    local tLayer = Renderer.getLayer('UI')
    local x, y = tLayer.RenderLayer:wndToWorld(sx, sy)
    local tProps = { tLayer.Partition:propListForPoint(x, y) }
    local tOverlayLayer = Renderer.getLayer('UIOverlay')
    x, y = tOverlayLayer.RenderLayer:wndToWorld(sx, sy)
    local tOverlayProps = { tOverlayLayer.Partition:propListForPoint(x, y) }
    for i, rProp in pairs(tOverlayProps) do
        table.insert(tProps, rProp)
    end
    -- always pass the to dialogui
    if self.rUpperDialogUI then
        self.rUpperDialogUI:onFinger(eventType, sx, sy, tProps)
    end
    if self.rLowerDialogUI then
        self.rLowerDialogUI:onFinger(eventType, sx, sy, tProps)
    end
    local nIndex = 0
    local nNumPostIts = table.getn(self.tPostIts)    
    if eventType == MOAITouchSensor.TOUCH_UP then    
        while nIndex <= nNumPostIts do
            nIndex = nIndex + 1
            if self.tPostIts[nIndex] then
                if self.tPostIts[nIndex]:shouldHideWhenTapped() then
                    self.tPostIts[nIndex]:setVisible(false)
                    self.tPostIts[nIndex]:onPaneClosed()
                    table.remove(self.tPostIts, nIndex)
                    nIndex = nIndex - 1
                end
            end
        end
    end
    for i, rPostIt in ipairs(self.tPostIts) do
        if rPostIt:shouldBlockInputUntilTapped() then
            return true
        end
    end
    if self.rPopupPane then
        if self.rPopupPane.onFinger then
            self.rPopupPane:onFinger(eventType, sx, sy, tProps)
        end
        return true
    elseif self.rCurActivePane then
        local bResult = self.rCurActivePane:onFinger(eventType, sx, sy, tProps)
        if self.rCurActivePane == self.rGameOverlayUI then
            return bResult
        else
            return true
        end
	end			
	-- This is how you go in a out of the overview mode
	return false
end

function Gui:onViewportResized()
    if self.rCurActivePane and self.rCurActivePane.onViewportResized then
        self.rCurActivePane:onViewportResized()
    end
    if self.rUpperDialogUI and self.rUpperDialogUI.onViewportResized then
        self.rUpperDialogUI:onViewportResized()
    end
    if self.rLowerDialogUI and self.rLowerDialogUI.onViewportResized then
        self.rLowerDialogUI:setLoc(0, -Renderer.getUIHeight() + 140)
        self.rLowerDialogUI:onViewportResized()
    end
end

return Gui
