local UIElement = require('ui/UIElement')
local Character = require('Character')
local SpriteSheet = require('SpriteSheet')
local HeroCard = {}

local tUITextureSpriteSheetInfo =
{
    card_rainbow = 'UI/heropacks',
    card_silver = 'UI/heropacks',
    card_default = 'UI/heropacks',
    card_back_rainbow = 'UI/heropacks',
    card_back_silver = 'UI/heropacks',
    card_back_default = 'UI/heropacks',
    flip = 'UI/hud',
}

local tPackLevelToPortraitName =
{
   'lvl1',
   'lvl1',
   'lvl1',
}

function HeroCard:new(rRenderLayer, sHeroDefKey, nPackLevel, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps and eventType == MOAITouchSensor.TOUCH_UP then        
            local bHitCard = false
            for i, rProp in ipairs(tProps) do
                if rProp == self.rFlipIcon then
                    self:flipCard()
                    return true
                elseif rProp == self.rBackGround then
                    bHitCard = true
                end
            end                    
            if bHitCard and self.rParentUI and self.rParentUI.onCardSelected then
                self.rParentUI:onCardSelected(self)
            end
        end
    end

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()        
    end

    function Ob:getDef()
        return self.rDef
    end

    function Ob:addHero()
        self.rParentUI:addHero(self.sHeroDefKey)
    end

    function Ob:flipCard()
        g_Audio:playCue('MMOJ_Audio/UI/CardFlip')
        self:setCardBackVisible(not self.bCardBackVisible)
    end

    function Ob:setCardBackVisible(bVisible)
        self.bCardBackVisible = bVisible
        self.rCardBackGround:setVisible(bVisible)
        if bVisible then
            self.rDescriptionText:revealAll()
            self.rNumberText:revealAll()
        else
            self.rDescriptionText:setReveal(0)
            self.rNumberText:setReveal(0)
        end
    end

    Ob.sHeroDefKey = sHeroDefKey
    Ob.rDef = Character.getCharDef(sHeroDefKey)
    Ob.rParentUI = rParentUI

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    if nPackLevel == 3 then        
        Ob.rBackGround = Ob:getUITextureProp('card_rainbow')
        Ob.rCardBackGround = Ob:getUITextureProp('card_back_rainbow')
    elseif nPackLevel == 2 then
        Ob.rBackGround = Ob:getUITextureProp('card_silver')
        Ob.rCardBackGround = Ob:getUITextureProp('card_back_silver')
    else
        Ob.rBackGround = Ob:getUITextureProp('card_default')
        Ob.rCardBackGround = Ob:getUITextureProp('card_back_default')
    end
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

	-- name
	Ob.rNameText = g_GUI:CreateTextBox(23, MOAITextBox.CENTER_JUSTIFY)
	Ob.rNameText:setRect(-200, 60, 200, 0)
	Ob.rNameText:setString(g_DialogManager:getString(Ob.rDef.sName))
	Ob.rNameText:setLoc(154, -20)
	Ob.rNameText:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameText)

    if Ob.rDef.tCardPath and Ob.rDef.tCardIndex then
        local sIndex = Ob.rDef.tCardIndex[tPackLevelToPortraitName[nPackLevel]]
        if sIndex then
            local rInfo = {}
            rInfo[sIndex] = Ob.rDef.tCardPath[tPackLevelToPortraitName[nPackLevel]]
            Ob:loadUISpriteSheets(rInfo)
            Ob.rPortrait = Ob:getUITextureProp(sIndex)
            Ob.rPortrait:setLoc(16, -72)
            Ob:addElement(Ob.rPortrait)
        end
    end

	-- stats
	local tStats = Ob.rDef.tInitialStats[nPackLevel]

	Ob.rPowerText = g_GUI:CreateTextBox(34, MOAITextBox.CENTER_JUSTIFY)
	Ob.rPowerText:setRect(-150, 60, 150, 0)
	Ob.rPowerText:setString(tostring(tStats.nPower))
	Ob.rPowerText:setLoc(60, -400)
	Ob.rPowerText:setColor(0, 0, 0)
	Ob:addElement(Ob.rPowerText)

	Ob.rHPText = g_GUI:CreateTextBox(34, MOAITextBox.CENTER_JUSTIFY)
	Ob.rHPText:setRect(-150, 60, 150, 0)
	Ob.rHPText:setString(tostring(tStats.nMaxHP))
	Ob.rHPText:setLoc(154, -400)
	Ob.rHPText:setColor(0, 0, 0)
	Ob:addElement(Ob.rHPText)

	Ob.rAwarenessText = g_GUI:CreateTextBox(34, MOAITextBox.CENTER_JUSTIFY)
	Ob.rAwarenessText:setRect(-150, 60, 150, 0)
	Ob.rAwarenessText:setString(tostring(tStats.nAwareness))
	Ob.rAwarenessText:setLoc(250, -400)
	Ob.rAwarenessText:setColor(0, 0, 0)
	Ob:addElement(Ob.rAwarenessText)

    Ob.rCardBackGround:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rCardBackGround)

	Ob.rNumberText = g_GUI:CreateTextBox(16, MOAITextBox.CENTER_JUSTIFY)
	Ob.rNumberText:setRect(0, 280, 280, 0)
	Ob.rNumberText:setString("# "..Ob.rDef.nCardNum.." "..g_DialogManager:getString("STXT128TEXT").." "..g_Game:getTotalNumHeroes())
	Ob.rNumberText:setLoc(10, -128)
	Ob.rNumberText:setColor(0, 0, 0) 
    Ob:addOverlayElement(Ob.rNumberText)

	Ob.rDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionText:setRect(0, 300, 260, 0)
	Ob.rDescriptionText:setString(g_DialogManager:getString(Ob.rDef.sDescription))
	Ob.rDescriptionText:setLoc(28, -170)
	Ob.rDescriptionText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rDescriptionText)   

    Ob.rFlipIcon = Ob:getUITextureProp('flip')
    Ob.rFlipIcon:setLoc(250, -440)
    Ob:addOverlayElement(Ob.rFlipIcon)

    Ob:setCardBackVisible(false)

	return Ob
end

return HeroCard