local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local HeroCard = require('ui/HeroCard')
local AbilityIcon = require('ui/AbilityIcon')
local Button = require('ui/Button')
local util = require('DFUtil')
local HeroCardDetailsUI = {}
local ResumeUI = require('ui/ResumeUI')

local tUITextureSpriteSheetInfo =
{
folder_corner = 'UI/hud',
flip = 'UI/hud',
back_button = 'UI/hud',
back_button_tap = 'UI/hud',
}

function HeroCardDetailsUI:new(rRenderLayer, rHero)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if g_Game:isActionEnabled('CloseWindow') then
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                    g_GUI:HideHeroCardDetailsUI()
                    
                    g_GUI:ShowHeroScheduler(self.rHero)
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rHeroCard:onFinger(eventType, sx, sy, tProps)
        self.rBackButton:onFinger(eventType, sx, sy, tProps)        
        return true
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroCard:onPaneClosed()
        self.rBackButton:setVisible(bVisible)
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rHeroCard:setVisible(bVisible)
        local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
        if rOverlayLayer then
            if bVisible then
                rOverlayLayer:insertProp(self.rCornerButton)
            else
                rOverlayLayer:removeProp(self.rCornerButton)
            end
        end
        self.rResumePane:setVisible(bVisible)
        self.rBackButton:setVisible(bVisible)
	end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)    
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(.855, .710, .549)
    Ob:addElement(Ob.rBackground)

    -- add the back button
    Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'back_button', 'back_button_tap')
    Ob.rBackButton:setLoc(16, 10)
    Ob.rBackButton:setParent(Ob)

    Ob.rCornerButton = Ob:getUITextureProp('folder_corner')
    Ob.rCornerButton:setLoc(308, 0)
    Ob.rCornerButton:setScl(-1, 1)
    Ob.rCornerButton:setParent(Ob)

    Ob.rHeroCard = HeroCard:new(rRenderLayer, rHero:getCharDefKey(), rHero:getPackLevel(), Ob)
    Ob.rHeroCard:setLoc(20, -100)

    Ob.rResumePane = ResumeUI:new(rRenderLayer, rHero)
    Ob.rResumePane:setLoc(Renderer.getUIWidth() / 2 - 80, -10)
    Ob.rResumePane:setParent(Ob)

    return Ob
end

return HeroCardDetailsUI
