-------- HERO CARD SELECTOR ---------
local HeroCardSelectorUI = {}
local HeroCard = require('ui/HeroCard')
local Renderer = require('Renderer')
local Character = require('Character')
local util = require("DFUtil")
local DFFile = require('DFCommon.File')
local DFGraphics = require('DFCommon.Graphics')
local UIElement = require('ui/UIElement')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local DFAchievement = require('DFCommon.Achievement')
local DFAnalytics = require('DFCommon.Analytics')
local Button = require('ui/Button')

local tUITextureSpriteSheetInfo =
{
icon_power = 'UI/hud',
icon_maxhp = 'UI/hud',
icon_int = 'UI/hud',
button_back = 'UI/eventdetails',
button_back_tap = 'UI/eventdetails',
}

local g_DebugConditionalHeroes = false

function HeroCardSelectorUI:new(rRenderLayer, rPackDef)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rBackButton then
            self.rBackButton:setVisible(bVisible)
        end
        for i, rCard in ipairs(self.tCards) do
            rCard:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rBackButton then
            self.rBackButton:onPaneClosed()
        end
        for i, rCard in ipairs(self.tCards) do
            rCard:onPaneClosed()
        end
    end

    function Ob:onCardSelected(rCard)
        if not self.rConfirmPane then
            self.rSelectedCard = rCard
            self.rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT129TEXT").." "..g_DialogManager:getString(rCard:getDef().sName).."?", self, "STXT075TEXT", "STXT076TEXT")
            self.rConfirmPane:setLoc(240, -160)
            self.rConfirmPane:setVisible(true)    
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if g_Game:isActionEnabled('CloseWindow') then
                    if not self.rSelectedCard then
                        g_GUI:ShowShopUI()
                        return
                    end
                end
            end
        end
    end

    function Ob:addHero(sHeroDefKey)
        local nPackLevel = 0
        
        if self.rPackDef then
            nPackLevel = self.rPackDef.nPackLevel
        end
    
        local rNewHero = g_Game:addHeroToGame(sHeroDefKey, nPackLevel)    
        
        if rNewHero then
            rNewHero:setPlayEnterAnimForNothingActivity(true)
            DFAchievement.updateAchievement('RecruitNewHero', 1)

            if nPackLevel >= 3 then
                DFAchievement.updateAchievement('MaxLevelOneHero', 1)
                DFAchievement.updateAchievement('RecruitDeluxeHero', 1)
            end
            
            local heroCount = g_Game:getNumAllHeroes()
            
            if heroCount >= 8 then
                DFAchievement.updateAchievement('RecruitEightHeroes', 1)
            end     
            
            local rDef = Character.getCharDef(sHeroDefKey)
            if rDef and rDef.sCustomAchievement then
                DFAchievement.updateAchievement(rDef.sCustomAchievement , 1)
            end
            -- record an event for the user buying each hero (so they can be easily funnelled with apsalar)
            DFAnalytics.logEvent("buy_hero_" .. heroCount, { lvl = nPackLevel, amount = self.rPackDef.nPPPrice, heroKey = sHeroDefKey })
            
            g_GUI:HideHeroCardSelectorUI()
            g_GUI:ShowPremiumPurchaseSequence("hero", rNewHero)
        end
    end

    function Ob:onYesPressed()
        if self.rWarningPane then
            self.rWarningPane:setVisible(false)
            g_GUI:ShowRosterMenu()
            return
        end
        -- we are charging here in case the app is closed before hero selection for any reason
        g_Game:removePP(self.rPackDef.nPPPrice)
        DFAnalytics.logEvent("ppp_spnt", { type = "hrpk", amount = self.rPackDef.nPPPrice })
        DFAnalytics.incrStat("ppp_spnt_hrpk", self.rPackDef.nPPPrice)
        if self.rConfirmPane then
            self.rConfirmPane:setVisible(false)
            self.rConfirmPane = nil
        end
        self:selectHero()
        self.rSelectedCard = nil
        g_Game:saveGame()
        g_CurrentMap:centerOnRoom('hub', 0) -- snap to the room    
    end

    function Ob:onNoPressed()
        if self.rConfirmPane then
            self.rConfirmPane:setVisible(false)
            self.rConfirmPane = nil
        end
        self.rSelectedCard = nil
    end

    function Ob:selectHero()
        if self.rSelectedCard then
            self.rSelectedCard:addHero()
        end
    end

	function Ob:generaterateHeroes()
		local tHeroDefKeys = {}
		local tExistingHeroName = {}
		for i, rHero in pairs(g_Game:getAllHeroes()) do
			tExistingHeroName[rHero:getName()] = 1
		end
		local nSlotsLeft = 3
		if self.rPackDef then
			local tHeroPool = {}
            -- add conditional heroes, these are guaranteed to be added
            -- if we have many conditional heroes, we'll not add it by default
            if self.rPackDef.tConditionalHeroes then
                for i, heroInfo in ipairs(self.rPackDef.tConditionalHeroes) do
                    if nSlotsLeft > 0 then
                        if g_Game:getSaveVar(heroInfo.sFlagToCheck) or g_DebugConditionalHeroes then
                            if heroInfo.sCharDef then
                                local rHeroDef = Character.tCharDefinitions[heroInfo.sCharDef]
                                if rHeroDef and not tExistingHeroName[rHeroDef.sName] then
                                    table.insert(tHeroDefKeys, heroInfo.sCharDef)
                                    nSlotsLeft = nSlotsLeft - 1
                                end
                            end
                        end
                    end
                end
            end
			-- add a guaranteed hero if we don't have them
			if self.rPackDef.tGuaranteedHeroes then
				for i, sHeroDefKey in ipairs(self.rPackDef.tGuaranteedHeroes) do
					local rHeroDef = Character.tCharDefinitions[sHeroDefKey]
					if rHeroDef and not tExistingHeroName[rHeroDef.sName] then
                        table.insert(tHeroPool, sHeroDefKey)
                    end
				end
				-- pick one at random
				local nNumItems = util.countitems(tHeroPool)
				if nNumItems > 1 then
					table.insert(tHeroDefKeys, tHeroPool[math.random(nNumItems)])
					nSlotsLeft = nSlotsLeft - 1
				end
				tHeroPool = {}
			end
			for i, sHeroDefKey in ipairs(self.rPackDef.tPossibleHeroes) do
				local rHeroDef = Character.tCharDefinitions[sHeroDefKey]
				if not tExistingHeroName[rHeroDef.sName] then
                    table.insert(tHeroPool, sHeroDefKey)
                end
			end
			while nSlotsLeft > 0 and table.getn(tHeroPool) > 0 do
				local nNumItems = table.getn(tHeroPool)
				local nIndex = math.random(nNumItems)
				table.insert(tHeroDefKeys, tHeroPool[nIndex])
				table.remove(tHeroPool, nIndex)
				nSlotsLeft = nSlotsLeft - 1
			end			
		end
		return tHeroDefKeys
	end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rConfirmPane then
            self.rConfirmPane:onFinger(eventType, sx, sy, tProps)
            return true
        elseif self.rWarningPane then
            self.rWarningPane:onFinger(eventType, sx, sy, tProps)
            return true
        end
        if self.rBackButton then
            self.rBackButton:onFinger(eventType, sx, sy, tProps)
        end
        for i, rCard in ipairs(self.tCards) do
            rCard:onFinger(eventType, sx, sy, tProps)
        end
        return true
	end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rPackDef = rPackDef

    local nXOffset = (Renderer.getUIWidth() - 960) / 2
    Ob:setLoc(nXOffset, 0)
    
	-- Background
    local nWidth = Renderer.getUIWidth()
    local nHeight = Renderer.getUIHeight()
	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(-nXOffset, 0)
	Ob.rBackground:setScl(nWidth, nHeight)
	Ob.rBackground:setColor(.17, .26, .43)
	Ob:addElement(Ob.rBackground)

    Ob.rTextBox = g_GUI:CreateTextBox(55, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rTextBox:setString(g_DialogManager:getString("STXT121TEXT"))
    Ob.rTextBox:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTextBox:setLoc(-nXOffset, -10)
	Ob.rTextBox:setColor(1, 1, 1)	
	Ob:addOverlayElement(Ob.rTextBox)

    Ob.tCards = {}

    if rPackDef then
        -- draw the packs
        local kX_OFFSET = 310
        tHeroDefKeys = Ob:generaterateHeroes()
        for i, sHeroDefKey in ipairs(tHeroDefKeys) do
            local rDef = Character.getCharDef(sHeroDefKey)
            if rDef then
                local rCard = HeroCard:new(rRenderLayer, sHeroDefKey, Ob.rPackDef.nPackLevel, Ob)
                rCard:setParent(Ob)
                table.insert(Ob.tCards, rCard)
                rCard:setScl(0.9, 0.9)
                rCard:setLoc(28 + (i - 1) * kX_OFFSET, -90)
            end
        end
	end

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(140, -550)
    Ob.rLine1:setScl(4, 70)
    Ob.rLine1:setColor(1, 1, 1)
    Ob:addElement(Ob.rLine1)

    Ob.rPowerIcon = Ob:getUITextureProp('icon_power')
    Ob.rPowerIcon:setLoc(140, -530)
    Ob.rPowerIcon:setColor(0, 0, 0)
    Ob:addElement(Ob.rPowerIcon)

    Ob.rPowerLabel = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rPowerLabel:setString(g_DialogManager:getString("STXT122TEXT"))
    Ob.rPowerLabel:setRect(0, 80, 300, 0)
	Ob.rPowerLabel:setLoc(220, -540)
	Ob.rPowerLabel:setColor(1, 1, 1)
	Ob:addElement(Ob.rPowerLabel)

    Ob.rPowerDescription = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rPowerDescription:setString(g_DialogManager:getString("STXT123TEXT"))
    Ob.rPowerDescription:setRect(0, 100, 200, 0)
	Ob.rPowerDescription:setLoc(160, -584)
	Ob.rPowerDescription:setColor(1, 1, 1)
	Ob:addElement(Ob.rPowerDescription)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(350, -550)
    Ob.rLine2:setScl(4, 70)
    Ob.rLine2:setColor(1, 1, 1)
    Ob:addElement(Ob.rLine2)

    Ob.rMaxHPIcon = Ob:getUITextureProp('icon_maxhp')
    Ob.rMaxHPIcon:setLoc(360, -530)
    Ob.rMaxHPIcon:setColor(1, 1, 1)
    Ob:addElement(Ob.rMaxHPIcon)

    Ob.rMaxHPLabel = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rMaxHPLabel:setString(g_DialogManager:getString("STXT124TEXT"))
    Ob.rMaxHPLabel:setRect(0, 80, 300, 0)
	Ob.rMaxHPLabel:setLoc(430, -540)
	Ob.rMaxHPLabel:setColor(1, 1, 1)
	Ob:addElement(Ob.rMaxHPLabel)

    Ob.rMaxHPDescription = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rMaxHPDescription:setString(g_DialogManager:getString("STXT125TEXT"))
    Ob.rMaxHPDescription:setRect(0, 100, 200, 0)
	Ob.rMaxHPDescription:setLoc(370, -584)
	Ob.rMaxHPDescription:setColor(1, 1, 1)
	Ob:addElement(Ob.rMaxHPDescription)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(560, -550)
    Ob.rLine3:setScl(4, 70)
    Ob.rLine3:setColor(1, 1, 1)
    Ob:addElement(Ob.rLine3)

    Ob.rIntIcon = Ob:getUITextureProp('icon_int')
    Ob.rIntIcon:setLoc(570, -530)
    Ob.rIntIcon:setColor(1, 1, 1)
    Ob:addElement(Ob.rIntIcon)

    Ob.rIntLabel = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rIntLabel:setString(g_DialogManager:getString("STXT126TEXT"))
    Ob.rIntLabel:setRect(0, 80, 300, 0)
	Ob.rIntLabel:setLoc(640, -540)
	Ob.rIntLabel:setColor(1, 1, 1)
	Ob:addElement(Ob.rIntLabel)

    Ob.rIntDescription = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rIntDescription:setString(g_DialogManager:getString("STXT127TEXT"))
    Ob.rIntDescription:setRect(0, 100, 300, 0)
	Ob.rIntDescription:setLoc(580, -584)
	Ob.rIntDescription:setColor(1, 1, 1)
	Ob:addElement(Ob.rIntDescription)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(854, -550)
    Ob.rLine4:setScl(4, 70)
    Ob.rLine4:setColor(1, 1, 1)
    Ob:addElement(Ob.rLine4)

    -- add the back button
    if g_Game:isActionEnabled('CloseWindow') then
        Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_back', 'button_back_tap')
        Ob.rBackButton:setLoc(20, -10)
        Ob.rBackButton:setParent(Ob)
    end

	return Ob
end

return HeroCardSelectorUI