local UIElement = require('ui/UIElement')
local HeroDetailButton = require('ui/HeroDetailButton')
local ListUI = require('ui/ListUI')
local Renderer = require('Renderer')
local util = require('DFUtil')
local HeroChooserUI = {}

local tUITextureSpriteSheetInfo =
{
    hero_file = 'UI/hud2',
    hero_chooser = 'UI/hud2',
    tape_label1 = 'UI/shop',
}

local kHIDDEN_X_LOC = -580
local kMINIMIZED_X_LOC = -580 
local kMAXIMIZED_X_LOC = 0

-- note: hero chooser shouldn't be parented
function HeroChooserUI:new(rRenderLayer, rParentUI, sLabel)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/Clipboard')
        end
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

    function Ob:update()
        if self.rListUI then
            self.rListUI:update()
        end
    end
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rListUI:onPaneClosed()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if not self.bHidden then
            self.rListUI:onFinger(eventType, sx, sy, tProps)
        end
        if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then   
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground then
                        bTouchedSomething = true
                        return true
                    end
                end
            end
            self:hide(false)
            if self.rParentUI and self.rParentUI.onHeroChooserCancelled then
                self.rParentUI:onHeroChooserCancelled()
            end
        end
        return true
    end

    function Ob:handleItemSelection(rButton)
        if self.rParentUI and rButton and self.bMaximized then
            g_Audio:playCue('MMOJ_Audio/UI/AddHero')
            self.rParentUI:onHeroSelected(rButton:getHero()) 
        end
    end

    function Ob:populateList(tHeroes)
        if not tHeroes then
            return
        end
        local tButtons = {}
        for i, rHero in ipairs(tHeroes) do
            local rNewButton = HeroDetailButton:new(rRenderLayer, rHero, nil, true)
            table.insert(tButtons, rNewButton)
        end
        self.rListUI:setList(tButtons)
    end

    function Ob:isMaximized()
        return self.bMaximized
    end

    function Ob:maximize(bMaximize, bSnap)
        if self.bMaximized == bMaximize then
            return
        end
        self.bHidden = false
        self.bMaximized = bMaximize
        if bMaximize then
            self:populateList(self.rParentUI:getSelectableHeroes())
        end
        self.nDestXPos = kMAXIMIZED_X_LOC
        if not bMaximize then
            self.nDestXPos = kMINIMIZED_X_LOC
        end
        local x, y = self:getLoc()
        local xDiff = self.nDestXPos - x        
        if math.abs(xDiff) > 0 then
            if bSnap then
                self:setLoc(self.nDestXPos, 0)
            else
                self:moveLoc(xDiff, 0, 0, 0.1, MOAIEaseType.LINEAR)
            end
        end        
    end

    function Ob:hide(bHide, bSnap)
        self.bMaximized = false
        self.bHidden = bHide
        self.nDestXPos = kHIDDEN_X_LOC
        if not bHide then
            self.nDestXPos = kMINIMIZED_X_LOC
        end

        if bSnap then
            self:setLoc(self.nDestXPos, 0)
        else
            self:seekLoc(self.nDestXPos, 0, 0, 0.1, MOAIEaseType.LINEAR)            
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rParentUI = rParentUI
    
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(512, Renderer.getUIHeight())
	Ob.rBackground:setColor(.88, .79, .68)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rTab = Ob:getUITextureProp('hero_file')
    Ob.rTab:setLoc(510, -50)
    Ob:addElement(Ob.rTab)

    Ob.rLabelText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
    Ob.rLabelText:setRect(0, 80, 300, 0)
    Ob.rLabelText:setString(g_DialogManager:getString("STXT154TEXT"))
	Ob.rLabelText:setLoc(534, -84)
    Ob.rLabelText:setRot(0, 0, -90)
	Ob:addElement(Ob.rLabelText)

    Ob.rListUI = ListUI:new(rRenderLayer, {}, Ob)
    Ob.rListUI:setParent(Ob)
    
    local nAnchorY = 0
    
    if sLabel then
        nAnchorY = -64
    
        Ob.titleBox = MOAIProp2D.new()
        Ob.titleBox:setDeck(g_GUI:GetOnePixelDeck())
        Ob.titleBox:setLoc(0, 0)
        Ob.titleBox:setScl(512, 70)
        Ob.titleBox:setColor(.88, .79, .68)
        Ob.titleBox:setLoc(0, 0)
        Ob:addOverlayElement(Ob.titleBox)
        
        Ob.rHeaderTape = Ob:getUITextureProp('tape_label1')
        Ob.rHeaderTape:setLoc(22, -14) 
        Ob.rHeaderTape:setScl(2.1, 1)
        Ob:addOverlayElement(Ob.rHeaderTape)
        
        Ob.rHeaderText = g_GUI:CreateTextBox(28, MOAITextBox.LEFT_JUSTIFY)
        Ob.rHeaderText:setString(g_DialogManager:getString(sLabel))
        Ob.rHeaderText:setLoc(335, -16)   
        Ob:addOverlayElement(Ob.rHeaderText)
    end
    
    Ob.rListUI:setAnchorLoc(14, nAnchorY)
    
    Ob:hide(false, true)
    
    return Ob
end

return HeroChooserUI
