local UIElement = require('ui/UIElement')
local Character = require('Character')
local ShopButton = require('ui/ShopButton')
local HeroCostumeDetailButton = {}

local tUITextureSpriteSheetInfo =
{
shop_item = 'UI/shop',
button_buy = 'UI/shop',
icon_cash = 'UI/hud',
locked = 'UI/hud',
female01_lvl2 = 'UI/icons2',
female01_lvl3 = 'UI/icons2',
female02_lvl2 = 'UI/icons2',
female02_lvl3 = 'UI/icons2',
female02b_lvl2 = 'UI/icons2',
male01_lvl2 = 'UI/icons2',
male01_lvl3 = 'UI/icons2',
male01b_lvl2 = 'UI/icons2',
male01b_lvl3 = 'UI/icons2',
male01c_lvl2 = 'UI/icons2',
male01c_lvl3 = 'UI/icons2',
male01d_lvl2 = 'UI/icons2',
male01d_lvl3 = 'UI/icons2',
male02_lvl2 = 'UI/icons2',
male02_lvl3 = 'UI/icons2',
male02b_lvl2 = 'UI/icons2',
male02b_lvl3 = 'UI/icons4',
robot01_lvl2 = 'UI/icons2',
robot01_lvl3 = 'UI/icons4',
shop_itembar = 'UI/shop',
}

function HeroCostumeDetailButton:new(rRenderLayer, rHero, sCostumeKey)

    if not rHero then
		print("ERROR: HeroCostumeDetailButton created without a Hero")
		return nil        
    end
	if not sCostumeKey then
		print("ERROR: HeroCostumeDetailButton created without a Costume")
		return nil
	end

    local rDef = Character.getCostumeDef(sCostumeKey)
    if not rDef then
		print("ERROR: HeroCostumeDetailButton created without a Costume")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)
    Ob.rDef = rDef
    Ob.rHero = rHero

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if self.rParentUI then
                self.rParentUI:handleButtonPress(self, eventType)
            end
            self:onButtonPressed()
        end
    end    

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rButton then
            self.rButton:onFinger(eventType, sx, sy, tProps)
        end
		return true
	end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rButton then
            self.rButton:setVisible(bVisible)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rButton then
            self.rButton:onPaneClosed()
        end
    end

	function Ob:getYSize()
        return 220
	end

    function Ob:getButton()
        return self.rButton
    end 

    function Ob:update()
        if self.rButton then
            self.rButton:update()
        end
    end
            
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sCostumeKey = sCostumeKey
	Ob.rDef = rDef
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

    Ob.rItemBar = Ob:getUITextureProp('shop_itembar')
    Ob.rItemBar:setLoc(15, -152)
    Ob:addElement(Ob.rItemBar)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(g_DialogManager:getString(Ob.rDef.sName))
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(150, -10)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

	Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rDef.sDescription))
    Ob.rDescriptionField:setRect(0, 80, 300, 0)
	Ob.rDescriptionField:setLoc(150, -50)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	
        
    Ob.rIcon = Ob:getUITextureProp(Ob.rDef.iconName)
    Ob.rIcon:setLoc(16, -16)
    Ob:addElement(Ob.rIcon)

    if Ob.rDef.nReqLevel > rHero:getStatLevel() then
        Ob.rLockedField = g_GUI:CreateTextBox(28, MOAITextBox.LEFT_JUSTIFY)
        Ob.rLockedField:setString(g_DialogManager:getString("STXT011TEXT").." " ..Ob.rDef.nReqLevel)
        Ob.rLockedField:setRect(0, 80, 600, 0)
        Ob.rLockedField:setLoc(52, -158)
        Ob.rLockedField:setColor(1, 0, 0)
        Ob:addElement(Ob.rLockedField)  
  
        Ob.rLockIcon = Ob:getUITextureProp('locked')
        if Ob.rLockIcon then
            Ob.rLockIcon:setLoc(35, -25)
            Ob.rLockIcon:setScl(1.5, 1.5)
            Ob:addElement(Ob.rLockIcon)
        end      
    else    
        Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
        Ob.rCostIcon:setLoc(20, -154)
        Ob:addElement(Ob.rCostIcon)
    
        local tCashColor = g_Game:getCashColor()
        Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rCostField:setString(tostring(Ob.rDef.nCost))
        Ob.rCostField:setLoc(390, -158)
        Ob.rCostField:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
        Ob:addElement(Ob.rCostField)

        Ob.rButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.rDef.nCost, nil, { .57, .57, .57 })
        Ob.rButton:setLoc(306, -152)
        Ob.rButton:setParent(Ob)        
    end
    
    if Ob.rDef.sEffectDescription then
        Ob.rStatField = g_GUI:CreateTextBox(14, MOAITextBox.LEFT_JUSTIFY)
        Ob.rStatField:setString(g_DialogManager:getString(Ob.rDef.sEffectDescription))
        Ob.rStatField:setRect(0, 300, 300, 0)
        Ob.rStatField:setLoc(150, -118)
        Ob.rStatField:setColor(0, .5, 0)
        Ob:addElement(Ob.rStatField)	
    end    
    return Ob
end

return HeroCostumeDetailButton