local Character = require('Character')
local UIElement = require('ui/UIElement')
local HeroCostumeDetailButton = require('ui/HeroCostumeDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local DFAchievement = require('DFCommon.Achievement')
local DFAnalytics = require('DFCommon.Analytics')
local Renderer = require('Renderer')

local HeroCostumePurchaseButton = {}

function HeroCostumePurchaseButton:new(rRenderLayer, rHero, sCostumeKey)
	if not sCostumeKey then
		print("ERROR: HeroCostumePurchaseButton created without an Item")
		return nil
	end
    local rDef = Character.getCostumeDef(sCostumeKey)
    if not rDef then
		print("ERROR: HeroCostumePurchaseButton created without an Item")
		return nil
    end

	local Ob = HeroCostumeDetailButton:new(rRenderLayer, rHero, sCostumeKey)
    Ob.rDef = rDef
    Ob.rHero = rHero
    Ob.sCostumeKey = sCostumeKey

    function Ob:onButtonPressed()
        if self.rHero and self:canBuyItem() then
            local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT138TEXT")..' '..g_DialogManager:getString(self.rDef.sName).."?", self, "STXT075TEXT", "STXT076TEXT")
            rConfirmPane:setLoc(240, -160)
            g_GUI:SetPopupPane(rConfirmPane)
        end        
    end

    function Ob:canBuyItem()
        if self.rDef.nCost then
            if g_Game:getCurCash() < self.rDef.nCost then
                return false
            end
            if self.rDef.nReqLevel > self.rHero:getStatLevel() then
                return false
            end
        end
        return true
    end

    function Ob:buyItem()
        if self.rHero and self:canBuyItem() then
            DFAchievement.updateAchievement('BoughtOneHeroCostume', 1)
            if self.rDef.sAchievementID then
                DFAchievement.updateAchievement(self.rDef.sAchievementID, self.sCostumeKey)
            end
            g_Game:removeCash(self.rDef.nCost)
            self.rHero:unlockCostume(self.rDef.sDefKey)
            self.rHero:setCostume(self.rDef.sDefKey)
            DFAnalytics.logEvent("buy_cstm", {type = self.sCostumeKey} )
            DFAnalytics.incrStat("csh_spnt_cstm", self.rDef.nCost)
            g_Game:saveGame()
            g_GUI:SetActivePane(nil)
            g_TodoList:onCostumePurchased()
        end
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:buyItem()
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end
    
    return Ob
end

return HeroCostumePurchaseButton