local UIElement = require('ui/UIElement')
local UIProgressBar = require("ui/UIProgressBar")
local TickProgressBar = require("ui/TickProgressBar")
local util = require("DFUtil")
local Character = require('Character')
local HeroDetailButton = {}

local tUITextureSpriteSheetInfo =
{
    hero_chooser = 'UI/hud2',
    button_x = 'UI/hud',
    button_buy = 'UI/hud',
    activity_band = 'UI/eventdetails',
activity_sheet_rest = 'UI/hud',
activity_sheet_timeoff = 'UI/hud',
activity_sheet_research = 'UI/hud',
activity_sheet_train = 'UI/hud',
activity_sheet_train_maxhp = 'UI/hud',
activity_sheet_train_ability = 'UI/hud',
activity_sheet_parttimejob = 'UI/hud',
    morale_superlow = 'UI/hud',
    morale_low = 'UI/hud',
    morale_med = 'UI/hud',
    morale_high = 'UI/hud',
    morale_superhigh = 'UI/hud',
activity_sheet_motivate = 'UI/hud2',
activity_sheet_train_int = 'UI/hud',
}

function HeroDetailButton:new(rRenderLayer, rHero, bShowActiveButton, bShowCurrentActivity)
	if not rHero then
		print("ERROR: HeroSelectorButton created without a Hero")
		return nil
	end

	local Ob = UIElement:new(rRenderLayer)
    Ob.bShowCurrentActivity = bShowCurrentActivity

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:update()
        if self.rHero then
            self.rStatHealthField:setString(tostring(self.rHero:getMaxHP()))
            self.rHPBar:update(rHero:getCurHP())
            --self.rManaBar:update(rHero:getCurMana())
            self.rManaBar:update(rHero:getCurMana(), rHero:getMaxMana()-rHero:getCurMana())
            if self.rActiveButton and self.rActiveButtonText then
                local bActive = self.rHero:isActive()
                if bActive then
                    self.rPortrait:setColor(1, 1, 1)
                    self.rBackGround:setColor(1, 1, 1)
                    self.rActiveButton:setColor(0, 1, 0)
                    self.rActiveButtonText:setString(g_DialogManager:getString("STXT033TEXT"))
                else
                    self.rPortrait:setColor(0.5, 0.5, 0.5)
                    self.rBackGround:setColor(0.5, 0.5, 0.5)
                    self.rActiveButton:setColor(1, 1, 1)
                    self.rActiveButtonText:setString(g_DialogManager:getString("STXT034TEXT"))
                end
            end
            
            -- hack to get around a small crash
            if self.rHero.getCurActivity then            
                local rActivity = self.rHero:getCurActivity()
                if not rActivity then
                    local rMMActivity = g_MiddleManager:getAvatar():getCurActivity()
                    if rMMActivity then
                        if rMMActivity:getActivityType() == 'MMMoraleActivity' or rMMActivity:getActivityType() == 'MMTrainIntActivity' then
                            if rMMActivity:getOtherHero() == self.rHero then
                                rActivity = rMMActivity
                            end
                        end
                    end
                end
                if rActivity then
                    if rActivity:getActivityType() == 'Nothing' then
                        self.rActivityBand:setVisible(false)
                        self.rCurActivityText:setReveal(0)
                        self.rActivityTimeText:setReveal(0)     
                        self.rActivityIcon:setVisible(false)                        
                    else
                        local nPri = self.rHPBar:getElementsMaxPriority()
                        self.rActivityBand:setVisible(true)
                        self.rActivityBand:setPriority(nPri+1)
                        self.rCurActivityText:revealAll()
                        self.rActivityTimeText:revealAll()
                        self.rCurActivityText:setString(g_DialogManager:getString(rActivity:getActivityUIName()))
                        -- tried this, but the bInRoom flag is set when the walk task starts, not when it finishes.
                        --local bHasStarted = rActivity.tSaved and rActivity.tSaved.bInRoom 
                        --local sTime = (bHasStarted and util.convertTimeString(nTimeRemaining)) or "--:--"
                        local nTimeRemaining = rActivity:getRemainingTime() or 0
                        local sTime = (nTimeRemaining > 0 and util.convertTimeString(nTimeRemaining)) or "--:--"
                        self.rActivityTimeText:setString(sTime.." "..g_DialogManager:getString("STXT169TEXT"))
                        self.rActivityIcon = self:getUITextureProp(rActivity:getCurActiveIconPath(), self.rActivityIcon)
                        self.rActivityIcon:setVisible(true)

                        self.rActivityBand:setPriority(nPri+1)
                        self.rCurActivityText:setPriority(nPri+2)
                        self.rActivityTimeText:setPriority(nPri+3)
                        self.rActivityIcon:setPriority(nPri+4)
                    end              
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps, bTouchMoved)
        if bTouchMoved then
            return true
        end
        local bButtonPressed = false
        if eventType == MOAITouchSensor.TOUCH_UP then
            if tProps then                
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackGround then
                        if self.rParentUI then
                            self.rParentUI:handleButtonPress(self, eventType)
                        end
                        break
                    end
                end
            end
        end

		return true
	end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rManaBar:setVisible(bVisible)
        self.rHPBar:setVisible(bVisible)
        if bVisible then
            local nPri = self.rHPBar:getElementsMaxPriority()
            self.rNameField:setPriority(nPri + 1)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:toggleHeroActive()
        if self.rHero then
            if self.rHero:isActive() then
                g_Game:setHeroActive(self.rHero:getCharDefKey(), false)
            else
                if g_Game:canAddActiveHeroes() then                        
                    g_Game:setHeroActive(self.rHero:getCharDefKey(), true)
                    self.rHero:runActivities(true)
                end
            end
        end
    end

	function Ob:getYSize()
        local rActivity = nil
        if self.bShowCurrentActivity then 
            rActivity = self.rHero:getCurActivity()
            if rActivity and rActivity:getActivityType() == 'Nothing' then
                rActivity = nil
            end
        end
        return (rActivity and 290) or 270
	end

    function Ob:getHero()
        return self.rHero
    end

	Ob.rHero = rHero
    Ob.rParentUI = nil

    local rActivity = Ob.bShowCurrentActivity and rHero and rHero:getCurActivity()
    if rActivity and rActivity:getActivityType() == 'Nothing' then
        rActivity = nil
    end
        
    Ob.nOffset = (rActivity and -30) or 0

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('hero_chooser') 
	Ob.rBackGround:setLoc(0, -20+Ob.nOffset)
	Ob:addElement(Ob.rBackGround)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNameField:setString(g_DialogManager:getString(rHero:getName()))
	Ob.rNameField:setLoc(330, -60+Ob.nOffset)
	Ob:addElement(Ob.rNameField)

    Ob.rStatPowerField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatPowerField:setString(tostring(rHero:getStatPower()))
    Ob.rStatPowerField:setLoc(184, -170+Ob.nOffset)
    Ob.rStatPowerField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatPowerField)

    Ob.rStatHealthField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatHealthField:setString(tostring(rHero:getMaxHP()))
    Ob.rStatHealthField:setLoc(262, -170+Ob.nOffset)
    Ob.rStatHealthField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatHealthField)

    Ob.rStatShieldField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatShieldField:setString(tostring(rHero:getStatMod('nShield')))
    Ob.rStatShieldField:setLoc(342, -170+Ob.nOffset)
    Ob.rStatShieldField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatShieldField)

    Ob.rStatAwarenessField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatAwarenessField:setString(tostring(rHero:getStatAwareness()))
    Ob.rStatAwarenessField:setLoc(422, -170+Ob.nOffset)
    Ob.rStatAwarenessField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatAwarenessField)


    --Ob.rManaBar = TickProgressBar:new(rRenderLayer, rHero:getMaxMana(), {r=61/255,g=201/255,b=217/255}, {r=1/255,g=22/255,b=64/255},nil, false)
    Ob.rManaBar = TickProgressBar:new(rRenderLayer, 254, {r=131/255,g=48/255,b=242/255}, {r=42/255,g=13/255,b=77/255}, {r=0/255,g=0/255,b=0/255}, {r=237/255,g=21/255,b=88/255}) --,  rHeroEventActor:getHero():getMaxMana(), 
    Ob.rManaBar:setLoc(147, -96+Ob.nOffset)
    --Ob.rManaBar:setScl(314, 13)
    Ob.rManaBar:update(rHero:getCurMana(), rHero:getMaxMana()-rHero:getCurMana())
    Ob.rManaBar:setParent(Ob)

    Ob.rHPBar = UIProgressBar:new(rRenderLayer, rHero:getMaxHP(), { r = 0.51, g = 0.93, b = 0.33 }, nil, false)
    Ob.rHPBar:setLoc(24, -59+Ob.nOffset)
    Ob.rHPBar:setScl(372, 30)
    Ob.rHPBar:update(rHero:getCurHP())
    Ob.rHPBar:setParent(Ob)

    Ob.rMoraleIcon = Ob:getUITextureProp(Character.tMoraleSprites[rHero:getMorale()])
    Ob.rMoraleIcon:setLoc(385, -60+Ob.nOffset)
    Ob.rMoraleIcon:setScl(.7,.7)
    Ob:addElement(Ob.rMoraleIcon)

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setScl(0.9, 0.9)
        Ob.rPortrait:setLoc(24, -92+Ob.nOffset)
        Ob:addElement(Ob.rPortrait)
    end    

    if bShowActiveButton then
        Ob.rActiveButton = Ob:getUITextureProp('button_buy')
        Ob.rActiveButton:setLoc(20, -208+Ob.nOffset)
        Ob.rActiveButton:setScl(0.8, 0.7)
        Ob.rActiveButton:setColor(0, 1, 0)
        Ob:addElement(Ob.rActiveButton)
        
        Ob.rActiveButtonText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY)
        Ob.rActiveButtonText:setString(g_DialogManager:getString("STXT033TEXT"))
        Ob.rActiveButtonText:setLoc(84, -210+Ob.nOffset)
        Ob.rActiveButtonText:setColor(1, 1, 1)
        Ob:addElement(Ob.rActiveButtonText)
    end

    Ob.nActivityBandOffset = -5
    Ob.rActivityBand = Ob:getUITextureProp('activity_band')
    Ob.rActivityBand:setVisible(false)
    Ob.rActivityBand:setLoc(6, Ob.nActivityBandOffset)
    Ob:addElement(Ob.rActivityBand)

    Ob.rCurActivityText = g_GUI:CreateTextBox(28, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rCurActivityText:setString("")
    Ob.rCurActivityText:setLoc(440, -2+Ob.nActivityBandOffset)
    Ob.rCurActivityText:setColor(1, 1, 1)
    Ob.rCurActivityText:setReveal(0)
    Ob:addElement(Ob.rCurActivityText)

    Ob.rActivityTimeText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rActivityTimeText:setString("Finished In")
    Ob.rActivityTimeText:setLoc(440, -40+Ob.nActivityBandOffset)
    Ob.rActivityTimeText:setColor(1, 1, 1)
    Ob.rActivityTimeText:setReveal(0)
    Ob:addElement(Ob.rActivityTimeText)

    Ob.rActivityIcon = Ob:getUITextureProp('activity_sheet_rest')
    Ob.rActivityIcon:setVisible(false)
    Ob.rActivityIcon:setLoc(10, -4+Ob.nActivityBandOffset)
    Ob.rActivityIcon:setVisible(false)
    Ob:addElement(Ob.rActivityIcon)

	return Ob
end

return HeroDetailButton
