local Character = require('Character')
local Item = require('Item')
local UIElement = require('ui/UIElement')
local UIProgressBar = require('ui/UIProgressBar')
local MeteoriteIcon = require('ui/MeteoriteIcon')
local MiddleManager = require('MiddleManager')
local HeroDetailInventoryButton = {}
local HeroInventoryButton = {}
local HeroMeteoriteButton = {}

local tUITextureSpriteSheetInfo =
{
hero_chooser_inventory = 'UI/shop',
middlemanager_inventory = 'UI/shop',
locked = 'UI/hud',
button_x = 'UI/hud',
icon_equipment_shield1 = 'UI/icons3',
circle_fill = 'UI/hud',
}

local tStatModPlusColor = { 0, .5, 0 }
local tStatModMinusColor = { .5, 0, 0 }
local tStatModColor = { 0, 0, 0 }

function HeroMeteoriteButton:new(rRenderLayer, rHero)
    local Ob = UIElement:new(rRenderLayer)
    
    function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        self.rMeteoriteIcon:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rXButton then
                        if self.rMeteorite then
                            g_Audio:playCue('MMOJ_Audio/UI/Equip_A')
                            self.rHero:removeEquippedMeteorite()
                            self:update()    
                            local rShopUI = g_GUI:GetActivePane()
                            if rShopUI and rShopUI.showMeteoritePane then
                                if rShopUI:getActiveTabName() then
                                    if rShopUI:getActiveTabName() == 'Meteorite' then
                                        rShopUI:showMeteoritePane()
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
        return true
    end

    function Ob:onSelected(bSelected)
        if bSelected then
            self.rMeteoriteIcon:clearColorOverride()
            self.rXButton:setColor(1, 1, 1)
        else
            self.rMeteoriteIcon:setColorOverride(.5, .5, .5)
            self.rXButton:setColor(.5, .5, .5)
        end
    end

    function Ob:update()
        if self.rHero then
            local rMeteorite = self.rHero:getEquippedMeteorite()
            if rMeteorite then                
                if self.rMeteorite ~= rMeteorite then
                    self.rMeteorite = rMeteorite
                    self.rMeteoriteIcon:setVisible(true)
                    self.rXButton:setVisible(true)
                    self.rMeteoriteIcon:setMeteorite(rMeteorite:getItemDefKey(), rMeteorite:getLevel())
                    self.rBGCircle:setColor(0, 0, 0)
                    self.rFGCircle:setColor(0, 0.8, 0)
                end
            else
                self.rXButton:setVisible(false)
                self.rMeteoriteIcon:setVisible(false)
                self.rBGCircle:setColor(0.5, 0.5, 0.5)
                self.rFGCircle:setColor(0.75, 0.75, 0.75)
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero

    Ob.rBGCircle = Ob:getUITextureProp('circle_fill')
    Ob.rBGCircle:setLoc(0, 0)
    Ob.rBGCircle:setScl(1.2)
    Ob:addElement(Ob.rBGCircle)

    Ob.rFGCircle = Ob:getUITextureProp('circle_fill')
    Ob.rFGCircle:setLoc(4, -4)
    Ob.rFGCircle:setScl(1.08)
    Ob:addElement(Ob.rFGCircle)
    
    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer)
    Ob.rMeteoriteIcon:setParent(Ob)
    Ob.rMeteoriteIcon:setScl(0.6)
    Ob.rMeteoriteIcon:setLoc(6, -6)

    Ob.rXButton = Ob:getUITextureProp('button_x')
    Ob.rXButton:setLoc(68, 0)
    Ob.rXButton:setScl(0.7)
    Ob:addElement(Ob.rXButton)

    Ob.rMeteorite = nil

    return Ob
end

function HeroInventoryButton:new(rRenderLayer, rHero, nSlot)
    local Ob = UIElement:new(rRenderLayer)
    
    function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rXButton then
                        if self.rItem and self.rItem ~= Item.kEMPTY_SLOT then
                            g_Audio:playCue('MMOJ_Audio/UI/Equip_A')
                            self.rHero:unequipItem(self.rItem)
                            self:update()
                        end
                    end
                end
            end
        end
        return true
    end

    function Ob:onSelected(bSelected)
        if bSelected then
            self.rLockedIcon:setColor(1, 1, 1)
            self.rItemIcon:setColor(1, 1, 1)
            self.rXButton:setColor(1, 1, 1)
            self.rUnlockText:setColor(1, 1, 1)
        else
            self.rLockedIcon:setColor(.5, .5, .5)
            self.rItemIcon:setColor(.5, .5, .5)
            self.rXButton:setColor(.5, .5, .5)
            self.rUnlockText:setColor(.5, .5, .5)
        end
    end

    function Ob:update()
        if self.rHero and self.nSlot then
            local rItem, nLevelAtUnlock = self.rHero:getItemAtSlot(self.nSlot)
            if nLevelAtUnlock and nLevelAtUnlock > self.rHero:getStatLevel() then
                self.rItemIcon:setVisible(false)
                self.rLockedIcon:setVisible(true)
                self.rXButton:setVisible(false)
                self.rUnlockText:setString("Level\n"..nLevelAtUnlock)
                self.rUnlockText:revealAll()
                self.rUnlockText:setColor(0, 0, 0)
            else
                if rItem then                
                    if self.rItem ~= rItem then
                        self.rItem = rItem
                        self.rLockedIcon:setVisible(false)
                        self.rUnlockText:setReveal(0)
                        if rItem == Item.kEMPTY_SLOT then
                            self.rItemIcon:setVisible(false)
                            self.rXButton:setVisible(false)
                        else
                            self.rItemIcon:setVisible(true)
                            self.rXButton:setVisible(true)
                            local sIconName = rItem:getItemDef().iconName
                            if sIconName then
                                local sSpriteSheet = rItem:getItemDef().sSpriteSheet
                                if sSpriteSheet then
                                    local tInfo = {}
                                    tInfo[sIconName] = sSpriteSheet
                                    self:loadUISpriteSheets(tInfo)
                                    self:getUITextureProp(sIconName, self.rItemIcon)
                                end
                            end
                        end
                    end
                end
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero
    Ob.nSlot = nSlot

    Ob.rLockedIcon = Ob:getUITextureProp('locked')
    Ob.rLockedIcon:setColor(1, 1, 1, 0.5)
    Ob.rLockedIcon:setLoc(24, -10)
    Ob:addElement(Ob.rLockedIcon)

    Ob.rItemIcon = Ob:getUITextureProp('icon_equipment_shield1') -- to have something by default
    Ob.rItemIcon:setScl(0.78)
    Ob.rItemIcon:setLoc(2, 0)
    Ob:addElement(Ob.rItemIcon)

    Ob.rXButton = Ob:getUITextureProp('button_x')
    Ob.rXButton:setLoc(68, 0)
    Ob.rXButton:setScl(0.7)
    Ob:addElement(Ob.rXButton)

    Ob.rUnlockText = g_GUI:CreateTextBox(14, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rUnlockText:setString('Level 5')
    Ob.rUnlockText:setLoc(54, -48)
    Ob.rUnlockText:setColor(1, 0, 0)
    Ob.rUnlockText:setReveal(0)
    Ob:addElement(Ob.rUnlockText)

    Ob.rItem = nil

    Ob:update()

    return Ob
end

function HeroDetailInventoryButton:new(rRenderLayer, rHero)
	if not rHero then
		print("ERROR: HeroSelectorButton created without a Hero")
		return nil
	end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        local bButtonPressed = false
        for i, rButton in ipairs(self.tButtons) do
            rButton:onFinger(eventType, sx, sy, tProps)
        end
        self.rMeteoriteButton:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if g_TouchMoved then
                return true
            end
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackGround then
                        if self.rParentUI then
                            self.rParentUI:handleButtonPress(self, eventType)
                        end
                        break
                    end
                end
            end
        end

		return true
	end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if self.rHPBar then
            self.rHPBar:setVisible(bVisible)
        end
        for i, rButton in ipairs(self.tButtons) do
            rButton:setVisible(bVisible)
        end
        self.rMeteoriteButton:setVisible(bVisible)
        if bVisible then
            self.rRenderLayer:insertProp(self.rNameField)
        else
            self.rRenderLayer:removeProp(self.rNameField)
        end
        
        if bVisible then
            self:update()
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        for i, rButton in ipairs(self.tButtons) do
            rButton:onPaneClosed()
        end
        self.rMeteoriteButton:onPaneClosed()
    end

	function Ob:getYSize()
		return 290
	end

    function Ob:getHero()
        return self.rHero
    end

    function Ob:onSelected(bSelected)
        self.bSelected = bSelected
        if self.tButtons then
            for i, rButton in ipairs(self.tButtons) do
                rButton:onSelected(bSelected)
            end
        end
        self.rMeteoriteButton:onSelected(bSelected)
        if bSelected then
            self.rBackGround:setColor(1, 1, 1)
            
            if self.rPortrait then
                self.rPortrait:setColor(1, 1, 1)
            end
            if self.rHPBar then
                self.rHPBar:tint(1, 1, 1)
            end
        else
            self.rBackGround:setColor(0.5, 0.5, 0.5)
            
            if self.rPortrait then
                self.rPortrait:setColor(0.5, 0.5, 0.5)
            end
            
            if self.rHPBar then
                self.rHPBar:tint(0.5, 0.5, 0.5)
            end
        end
    end

    function Ob:update()
        if self.bVisible then
            self.rHPBar:update(self.rHero:getCurHP())
        
            self.rStatPowerField:setString(tostring(self.rHero:getStatPower()))
            self.rStatHealthField:setString(tostring(self.rHero:getCurHP()))
            self.rStatShieldField:setString(tostring(rHero:getStatMod('nShield')))
            self.rStatAwarenessField:setString(tostring(self.rHero:getStatAwareness()))
            
            local nPowStatMod = self.rHero:getStatMod('nPower')
            if nPowStatMod > 0 then
                self.rStatPowerField:setColor(tStatModPlusColor[1], tStatModPlusColor[2], tStatModPlusColor[3])
            elseif nPowStatMod < 0 then
                self.rStatPowerField:setColor(tStatModMinusColor[1], tStatModMinusColor[2], tStatModMinusColor[3])
            else
                self.rStatPowerField:setColor(tStatModColor[1], tStatModColor[2], tStatModColor[3])
            end
            local nAwarenessStatMod = self.rHero:getStatMod('nAwareness')
            if nAwarenessStatMod > 0 then
                self.rStatAwarenessField:setColor(tStatModPlusColor[1], tStatModPlusColor[2], tStatModPlusColor[3])
            elseif nAwarenessStatMod < 0 then
                self.rStatAwarenessField:setColor(tStatModMinusColor[1], tStatModMinusColor[2], tStatModMinusColor[3])
            else
                self.rStatAwarenessField:setColor(tStatModColor[1], tStatModColor[2], tStatModColor[3])
            end

            if self.bSelected ~= nil then
                self:onSelected(self.bSelected)
            end

            for i, rButton in ipairs(self.tButtons) do
                rButton:update()
            end
            self.rMeteoriteButton:update()
        end
    end

	Ob.rHero = rHero
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.bIsMiddleManager = false
    local sIconName = 'hero_chooser_inventory'
    if rHero == g_MiddleManager:getAvatar() then
        Ob.bIsMiddleManager = true
        sIconName = 'middlemanager_inventory'
    end
	Ob.rBackGround = Ob:getUITextureProp(sIconName)
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

    Ob.rBarProp = MOAIProp:new()
    Ob:addElement(Ob.rBarProp)

    Ob.rHPBar = UIProgressBar:new(rRenderLayer, rHero:getMaxHP(), { r = 0.51, g = 0.93, b = 0.33 }, nil, false)
    Ob.rHPBar:setLoc(137, -39)
    Ob.rHPBar:setScl(314, 30)
    Ob.rHPBar:update(rHero:getCurHP())
    Ob.rHPBar:setParent(Ob.rBarProp)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNameField:setString(g_DialogManager:getString(rHero:getName()))
	Ob.rNameField:setLoc(446, -40)
	Ob.rNameField:setColor(0, 0, 0)
    Ob.rNameField:setParent(Ob)
	--rRenderLayer:insertProp(Ob.rNameField)
    
    if not Ob.bIsMiddleManager then
        Ob.rStatPowerField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rStatPowerField:setString(tostring(rHero:getStatPower()))
        Ob.rStatPowerField:setLoc(174, -110)
        Ob.rStatPowerField:setColor(0, 0, 0)
        Ob:addElement(Ob.rStatPowerField)
        
        Ob.rStatHealthField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rStatHealthField:setString(tostring(rHero:getCurHP()))
        Ob.rStatHealthField:setLoc(252, -110)
        Ob.rStatHealthField:setColor(0, 0, 0)
        Ob:addElement(Ob.rStatHealthField)

        Ob.rStatShieldField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rStatShieldField:setString(tostring(rHero:getStatMod('nShield')))
        Ob.rStatShieldField:setLoc(332, -110)
        Ob.rStatShieldField:setColor(0, 0, 0)
        Ob:addElement(Ob.rStatShieldField)
        
        Ob.rStatAwarenessField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
        Ob.rStatAwarenessField:setString(tostring(rHero:getStatAwareness()))
        Ob.rStatAwarenessField:setLoc(412, -110)
        Ob.rStatAwarenessField:setColor(0, 0, 0)
        Ob:addElement(Ob.rStatAwarenessField)
    end

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setScl(0.9, 0.9)
        Ob.rPortrait:setLoc(16, -40)
        Ob:addElement(Ob.rPortrait)
    end    
    
    local X_OFFSET = 110
    Ob.tButtons = {}
    for i = 1, Character.getMaxSlots() do
        local rButton = HeroInventoryButton:new(rRenderLayer, rHero, i)
        table.insert(Ob.tButtons, rButton)
        rButton:setLoc(14 + X_OFFSET * (i - 1), -162)
        rButton:setParent(Ob)
    end

    Ob.rMeteoriteButton = HeroMeteoriteButton:new(rRenderLayer, rHero)
    Ob.rMeteoriteButton:setParent(Ob)
    Ob.rMeteoriteButton:setLoc(346, -170)

	return Ob
end

return HeroDetailInventoryButton