local Character = require('Character')
local UIElement = require('ui/UIElement')
local AbilityIcon = require('ui/AbilityIcon')
local UIProgressBar = require("ui/UIProgressBar")
local TickProgressBar = require("ui/TickProgressBar")
local AbilitySmallButton = require('ui/AbilitySmallButton')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')
local HeroPreEventButton = {}

local tUITextureSpriteSheetInfo =
{
    hero_chooser_event = 'UI/hud2',
    button_x = 'UI/hud',
    button_buy = 'UI/hud',
    ability_arrow = 'UI/eventdetails',
}

function HeroPreEventButton:new(rRenderLayer, rHero)
	if not rHero then
		print("ERROR: HeroSelectorButton created without a Hero")
		return nil
	end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end   

    function Ob:abilitySelectionCallback(rSelector, bHandled)
        g_GUI:SetPopupPane(nil)
        self.rAbilityButton:update()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then
            if tProps then
                if self.rAbilitySelectionUI:isVisible() then
                    if not self.rAbilitySelectionUI:onFinger(eventType, sx, sy, tProps) then
                        self.rAbilitySelectionUI:setVisible(false)
                        self.rAbilityButton:update()
                    end
                    return true
                end

                for i, rProp in ipairs(tProps) do
                    if rProp == self.rXButton then
                        self.bXButtonPressed = true
                        if self.rParentUI then
                            self.rParentUI:handleButtonPress(self, eventType)
                        end        
                    elseif self.rAbilityButton:onFinger(eventType, sx, sy, tProps) then
                        --self.rAbilitySelectionUI:setVisible(true)
                        g_GUI:SetPopupPane(self.rAbilitySelectionUI)
                    end
                        --[[
                    elseif rProp == self.rRightButton then
                        g_Audio:playCue('MMOJ_Audio/UI/ChangeAbility')
                        self.rHero:setActiveAbility(self.rHero:getNextAvailableAbilityKey())
                        self:onAbilitySelected()
                    elseif rProp == self.rLeftButton then
                        g_Audio:playCue('MMOJ_Audio/UI/ChangeAbility')
                        self.rHero:setActiveAbility(self.rHero:getPrevAvailableAbilityKey())
                        self:onAbilitySelected()
                    end 
                        ]]--
                end
            end
        end

		return true
	end

    --[[
    function Ob:onAbilitySelected()
        if self.rHero then
            local rAbility = rHero:getActiveAbility()
            if rAbility then
                self.rSpecialNameText:setString(string.upper(rAbility:getName()))
                if self.rAbilityIcon then
                    self.rAbilityIcon:setVisible(false)
                    self.rAbilityIcon:onPaneClosed()
                end
                self.rAbilityIcon = AbilityIcon:new(self:getRenderLayer(), rAbility)
                self.rAbilityIcon:setLoc(300, -164)
                self.rAbilityIcon:setScl(0.76)
                self.rAbilityIcon:setParent(self)
                if self.bVisible then
                    self.rAbilityIcon:setVisible(true)
                end
            end
        end
    end
    ]]--

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        self.rManaBar:setVisible(bVisible)
        self.rHPBar:setVisible(bVisible)
        self.rAbilityButton:setVisible(bVisible)
        if bVisible then
            local nPri = self.rHPBar:getElementsMaxPriority()
            self.rNameField:setPriority(nPri + 1)
        else
            self.rAbilitySelectionUI:setVisible(false)
        end
        if self.rAbilityButton then
            self.rAbilityButton:setVisible(bVisible)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rAbilitySelectionUI:onPaneClosed()
        --[[
        if self.rAbilityIcon then
            self.rAbilityIcon:onPaneClosed()
        end
        ]]--
    end

    function Ob:hasXButtonPressed()
        return self.bXButtonPressed
    end

	function Ob:getYSize()
		return 260
	end

    function Ob:getHero()
        return self.rHero
    end

	Ob.rHero = rHero
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('hero_chooser_event') 
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY)
	Ob.rNameField:setString(g_DialogManager:getString(rHero:getName()))
	Ob.rNameField:setLoc(445, -40)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

    Ob.rStatPowerField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatPowerField:setString(tostring(rHero:getStatPower()))
    Ob.rStatPowerField:setLoc(173, -110)
    Ob.rStatPowerField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatPowerField)

    Ob.rStatHealthField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatHealthField:setString(tostring(rHero:getMaxHP()))
    Ob.rStatHealthField:setLoc(252, -110)
    Ob.rStatHealthField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatHealthField)

    Ob.rStatShieldField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatShieldField:setString(tostring(rHero:getStatMod('nShield')))
    Ob.rStatShieldField:setLoc(332, -110)
    Ob.rStatShieldField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatShieldField)

    Ob.rStatAwarenessField = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rStatAwarenessField:setString(tostring(rHero:getStatAwareness()))
    Ob.rStatAwarenessField:setLoc(411, -110)
    Ob.rStatAwarenessField:setColor(0, 0, 0)
    Ob:addElement(Ob.rStatAwarenessField)

    --Ob.rManaBar = UIProgressBar:new(rRenderLayer, rHero:getMaxMana(), {r=61/255,g=201/255,b=217/255}, {r=1/255,g=22/255,b=64/255},true)
    Ob.rManaBar = TickProgressBar:new(rRenderLayer, 314, {r=131/255,g=48/255,b=242/255}, {r=42/255,g=13/255,b=77/255}, {r=0/255,g=0/255,b=0/255}, {r=237/255,g=21/255,b=88/255}) --,  rHeroEventActor:getHero():getMaxMana(), 
    Ob.rManaBar:setLoc(14, -154)
    --Ob.rManaBar:setScl(314,14)
    Ob.rManaBar:update(rHero:getCurMana(), rHero:getMaxMana()-rHero:getCurMana())
    Ob.rManaBar:setParent(Ob)

    Ob.rHPBar = UIProgressBar:new(rRenderLayer, rHero:getMaxHP(), { r = 0.51, g = 0.93, b = 0.33 }, nil, false)
    Ob.rHPBar:setLoc(138, -40)
    Ob.rHPBar:setScl(314, 28)
    Ob.rHPBar:update(rHero:getCurHP())
    Ob.rHPBar:setParent(Ob)

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setScl(0.9, 0.9)
        Ob.rPortrait:setLoc(14, -40)
        Ob:addElement(Ob.rPortrait)
    end    

    Ob.bXButtonPressed = false
    Ob.rXButton = Ob:getUITextureProp('button_x')
    Ob.rXButton:setLoc(414, -2)
    Ob.rXButton:setScl(0.6)
    Ob:addElement(Ob.rXButton)

    Ob.rAbilityButton = AbilitySmallButton:new(rRenderLayer, Ob, rHero)
    Ob.rAbilityButton:setLoc(216, -178)
    Ob.rAbilityButton:setParent(Ob)

    --[[
    Ob.rSpecialLabelText = g_GUI:CreateTextBox(28, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rSpecialLabelText:setString(g_DialogManager:getString("STXT035TEXT"))
    Ob.rSpecialLabelText:setLoc(324, -160)
    Ob.rSpecialLabelText:setColor(0, 0, 0)
    Ob:addElement(Ob.rSpecialLabelText)

    Ob.rSpecialNameText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rSpecialNameText:setString("")
    Ob.rSpecialNameText:setLoc(324, -190)
    Ob.rSpecialNameText:setColor(0, 0, 0)
    Ob:addElement(Ob.rSpecialNameText)
    ]]--

    --[[
    Ob.rLeftButton = Ob:getUITextureProp('ability_arrow')
    Ob.rLeftButton:setLoc(218, -164)
    Ob:addElement(Ob.rLeftButton)

    Ob.rRightButton = Ob:getUITextureProp('ability_arrow')
    Ob.rRightButton:setLoc(450, -164)
    Ob.rRightButton:setScl(-1, 1)
    Ob:addElement(Ob.rRightButton)   

    local nNumAvailableAbilities = rHero:getNumAvailableAbilities()
    if nNumAvailableAbilities <= 1 then
        Ob.rLeftButton:setVisible(false)
        Ob.rRightButton:setVisible(false)
    end

    Ob:onAbilitySelected()
    ]]--

    Ob.rAbilitySelectionUI = AbilitySelectionUI:new(Ob.rOverlayRenderLayer, Ob, rHero)
    Ob.rAbilitySelectionUI:setLoc(200, -10)
    --Ob.rAbilitySelectionUI:setParent(Ob)

	return Ob
end

return HeroPreEventButton
