local UIElement = require('ui/UIElement')
local HeroDetailButton = require('ui/HeroDetailButton')
local Renderer = require('Renderer')
local ListUI = require('ui/ListUI')
local Button = require('ui/Button')
local HeroRosterUI = {}
local HeroPanel = {}
local HeroButton = {}

local kMAX_HEROES = 8

local tUITextureSpriteSheetInfo =
{
    check_off = 'UI/hud2',
    check_on = 'UI/hud2',
    button_back = 'UI/eventdetails',
    button_back_tap = 'UI/eventdetails',
    button_continue = 'UI/eventdetails',
    button_continue_tap = 'UI/eventdetails',
    icon_pp = 'UI/hud',
}

local tActiveColor = { 0.75, 0.82, 0.71 }
local tInactiveColor = { 0.89, 0.89, 0.85 }

function HeroButton:new(rRenderLayer, rHero, bShowHintLine)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rHeroDetailButton then
            self.rHeroDetailButton:setVisible(bVisible)
        end
    end

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
        if not self.rHero then
            return true
        end
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground then                       
                        self:toggleHeroActive()
                        return true
                    end
                end
            end
        end
        return true
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rHeroDetailButton then
            self.rHeroDetailButton:onPaneClosed()
        end
    end

    function Ob:toggleHeroActive()
        local bHeroActive = self.rHero:isActive()
        if self.rHero:isAssignedToEvent() then
            return
        end
        if bHeroActive then
            g_Game:setHeroActive(self.rHero:getCharDefKey(), false)
        else
            if not g_Game:canAddActiveHeroes() then
                return
            end
            g_Game:setHeroActive(self.rHero:getCharDefKey(), true)
            self.rHero:runActivities(true)
        end
        self:onHeroActive(not bHeroActive)
    end

    function Ob:onHeroActive(bActive)
        if bActive then
            self.rBackground:setColor(tActiveColor[1], tActiveColor[2], tActiveColor[3])
            self.rCheckOnIcon:setVisible(true)
            self.rCheckOffIcon:setVisible(false)
        else
            self.rBackground:setColor(tInactiveColor[1], tInactiveColor[2], tInactiveColor[3])
            self.rCheckOnIcon:setVisible(false)
            self.rCheckOffIcon:setVisible(true)
        end
    end

    function Ob:getYSize()
        return 220
    end

    function Ob:getWidth()
        return Renderer.getUIWidth() / 2
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setColor(tInactiveColor[1], tInactiveColor[2], tInactiveColor[3])
    Ob.rBackground:setScl(Ob:getWidth(), Ob:getYSize())
    Ob:addElement(Ob.rBackground)

    Ob.rTopLine = MOAIProp2D.new()
    Ob.rTopLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTopLine:setLoc(0, 0)
    Ob.rTopLine:setScl(Ob:getWidth(), 2)
    Ob.rTopLine:setColor(.61, .61, .58)
    Ob:addElement(Ob.rTopLine)

    Ob.rBottomLine = MOAIProp2D.new()
    Ob.rBottomLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBottomLine:setLoc(0, -Ob:getYSize())
    Ob.rBottomLine:setScl(Ob:getWidth(), 2)
    Ob.rBottomLine:setColor(.61, .61, .58)
    Ob:addElement(Ob.rBottomLine)    

    Ob.rLeftLine = MOAIProp2D.new()
    Ob.rLeftLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLeftLine:setLoc(0, 0)
    Ob.rLeftLine:setScl(2, Ob:getYSize())
    Ob.rLeftLine:setColor(.61, .61, .58)
    Ob:addElement(Ob.rLeftLine)
    
    Ob.rRightLine = MOAIProp2D.new()
    Ob.rRightLine:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rRightLine:setLoc(0, 0)
    Ob.rRightLine:setScl(2, Ob:getYSize())
    Ob.rRightLine:setColor(.61, .61, .58)
    Ob:addElement(Ob.rRightLine)

    if rHero then
        Ob.rHeroDetailButton = HeroDetailButton:new(rRenderLayer, rHero)
        Ob.rHeroDetailButton:setParent(Ob)
        Ob.rHeroDetailButton:setScl(0.84)
        Ob.rHeroDetailButton:setLoc(Ob:getWidth() - 405, -6)

        Ob.rCheckOnIcon = Ob:getUITextureProp('check_on')
        Ob.rCheckOnIcon:setLoc(8, -32)
        Ob:addElement(Ob.rCheckOnIcon)

        Ob.rCheckOffIcon = Ob:getUITextureProp('check_off')
        Ob.rCheckOffIcon:setLoc(8, -32)
        Ob:addElement(Ob.rCheckOffIcon)

        if rHero:isActive() then
            Ob:onHeroActive(true)
        else
            Ob:onHeroActive(false)
        end
    end

    if bShowHintLine then
        Ob.rHintText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rHintText:setString(g_DialogManager:getString("UIHT008TEXT"))
        Ob.rHintText:setRect(0, 120, 400, 0)
        Ob.rHintText:setColor(.61, .61, .58)
        Ob.rHintText:setLoc(30, -40)
        Ob:addElement(Ob.rHintText)

        Ob.rPPIconIcon = Ob:getUITextureProp('icon_pp')
        Ob.rPPIconIcon:setLoc(112, -36)
        Ob.rPPIconIcon:setScl(0.8)
        Ob:addElement(Ob.rPPIconIcon)
    end

    return Ob
end

function HeroPanel:new(rRenderLayer)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/Clipboard')
        end    
        self:setElementsVisible(bVisible)
        self.rLeftListUI:setVisible(bVisible)
        self.rRightListUI:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rLeftListUI:onFinger(eventType, sx, sy, tProps)
        self.rRightListUI:onFinger(eventType, sx, sy, tProps)

        return true
    end

    local tLeftColHeroes = {}
    local tRightColHeroes = {}
    local tUnorderedHeroes = g_Game:getAllHeroes()
    local tHeroes = {}
    for i, rHero in pairs(tUnorderedHeroes) do
        table.insert(tHeroes, rHero)
    end
    local nNumItems = table.getn(tHeroes)
    if nNumItems > 0 then
        local nHalfNum = math.floor(nNumItems / 2)
        if nHalfNum > 0 then
            for i = 1, nHalfNum do
                table.insert(tLeftColHeroes, tHeroes[i])
            end
            for i = nHalfNum + 1, nNumItems do
                table.insert(tRightColHeroes, tHeroes[i])
            end
        else
            tLeftColHeroes = tHeroes
        end
    end

    local nHalfNumTotalHeroes = g_Game:getTotalNumHeroes() / 2
    local tLeftButtons = {}
    local tRightButtons = {}
    local bAddHintLine = true
    if nNumItems >= g_Game:getTotalNumHeroes() then
        bAddHintLine = false
    end
    for i, rHero in ipairs(tLeftColHeroes) do
        local rButton = HeroButton:new(rRenderLayer, rHero)
        table.insert(tLeftButtons, rButton)
    end
    for i = 1, nHalfNumTotalHeroes do
        if not tLeftButtons[i] then
            local rButton = nil
            if bAddHintLine then
                rButton = HeroButton:new(rRenderLayer, nil, true)
                bAddHintLine = false
            else
                rButton = HeroButton:new(rRenderLayer)
            end
            table.insert(tLeftButtons, rButton)            
        end
    end
    for i, rHero in ipairs(tRightColHeroes) do
        local rButton = HeroButton:new(rRenderLayer, rHero)
        table.insert(tRightButtons, rButton)
    end
    for i = 1, nHalfNumTotalHeroes do
        if not tRightButtons[i] then
            local rButton = nil
            if bAddHintLine then
                rButton = HeroButton:new(rRenderLayer, nil, true)
                bAddHintLine = false
            else
                rButton = HeroButton:new(rRenderLayer)
            end
            table.insert(tRightButtons, rButton)            
        end
    end

    function Ob:update()
        if Ob.rLeftListUI then
            Ob.rLeftListUI:update()
        end
        
        if Ob.rRightListUI then
            Ob.rRightListUI:update()
        end
    end
    
    Ob.rLeftListUI = ListUI:new(rRenderLayer, tLeftButtons, Ob)
    Ob.rLeftListUI:setParent(Ob)
    Ob.rLeftListUI:setAnchorLoc(0, -92)

    Ob.rRightListUI = ListUI:new(rRenderLayer, tRightButtons, Ob)
    Ob.rRightListUI:setParent(Ob)
    Ob.rRightListUI:setAnchorLoc(Renderer.getUIHalfWidth(), -92)

    return Ob
end

function HeroRosterUI:new(rRenderLayer, rParentUI)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rHeroPanel:setVisible(bVisible)
        self.rBackButton:setVisible(bVisible)
        if self.rRecruitButton then
            self.rRecruitButton:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroPanel:onPaneClosed()
        self.rBackButton:onPaneClosed()
        if self.rRecruitButton then        
            self.rRecruitButton:onPaneClosed()
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if g_Game:isActionEnabled('CloseWindow') then
                    g_GUI:HideRosterMenu()
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                end
            elseif rButton == self.rRecruitButton then
                g_GUI:ShowHeroShopUI()
            end
        end
    end

    function Ob:update()
        if self.rHeroPanel then
            self.rHeroPanel:update()
        end
    end
    
    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rRecruitButton then
            self.rRecruitButton:onFinger(eventType, sx, sy, tProps)
        end
        self.rBackButton:onFinger(eventType, sx, sy, tProps)
        self.rHeroPanel:onFinger(eventType, sx, sy, tProps)
        
        return true
    end

    function Ob:handleItemSelection(rItem)
        if rItem then
            local rHero = rItem:getHero()
            if rHero and rHero:isActive() then
                g_GUI:HideRosterMenu()
                rHero:onFinger(MOAITouchSensor.TOUCH_UP)
            end
        end
    end

    function Ob:populateList(tHeroes)
        if not tHeroes then
            return
        end
        self.tButtons = {}
        local bShowActiveButton = true

        for i, rHero in pairs(tHeroes) do
            local rNewButton = HeroDetailButton:new(rRenderLayer, rHero, bShowActiveButton)
            table.insert(self.tButtons, rNewButton)
        end
        self.rListUI:setList(self.tButtons)
    end

    function Ob:maximize(bMaximize)
        local x, y = self:getLoc()
        local xDiff = kMAXIMIZED_X_LOC - x        
        self:moveLoc(xDiff, 0, 0, 0.1, MOAIEaseType.LINEAR)
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rTopBar = MOAIProp2D.new()
	Ob.rTopBar:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBar:setLoc(0, 0)
    Ob.rTopBar:setScl(Renderer.getUIWidth(), 92)                 
	Ob.rTopBar:setColor(0.8, 0.8, 0.8)
    Ob:addOverlayElement(Ob.rTopBar)

	Ob.rTitleText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
	Ob.rTitleText:setString(g_DialogManager:getString("STXT070TEXT").."("..g_DialogManager:getString("STXT071TEXT").." "..g_Game:getNumAllowedActiveHeroes()..")")
    Ob.rTitleText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTitleText:setColor(0, 0, 0)
	Ob.rTitleText:setLoc(0, -20)
    Ob:addOverlayElement(Ob.rTitleText)

    -- add the back button
    Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_back', 'button_back_tap')
    Ob.rBackButton:setLoc(20, -10)
    Ob.rBackButton:setParent(Ob)

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(tInactiveColor[1], tInactiveColor[2], tInactiveColor[3])
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rHeroPanel = HeroPanel:new(rRenderLayer)
    Ob.rHeroPanel:setLoc(0, 0)
    Ob.rHeroPanel:setParent(Ob)

    if (g_Game:getTotalNumHeroes() > g_Game:getNumAllHeroes()) then
        Ob.rRecruitButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_continue', 'button_continue_tap', nil, nil, nil, nil, "STXT072TEXT", 'ptsans', 34, { 80, -5 }, { 1, 1, 1 } )
        Ob.rRecruitButton:setLoc(Renderer.getUIWidth() - 188, -10)
        Ob.rRecruitButton:setParent(Ob)
    end

    return Ob
end

return HeroRosterUI