-------- HERO SCHEDULER ---------
local util = require('DFUtil')
local Character = require('Character')
local Renderer = require('Renderer')
local ActivitySelectionUI = require('ui/ActivitySelectionUI')
local UIProgressBar = require('ui/UIProgressBar')
local StatHelpScreen = require('ui/StatHelpScreen')
local UIElement = require('ui/UIElement')
local ShopButton = require('ui/ShopButton')
local CountdownUI = require('ui/CountdownUI')
local AbilitySmallButton = require('ui/AbilitySmallButton')
local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')
local DFAnalytics = require('DFCommon/Analytics')
local Button = require('ui/Button')
local TickProgressBar = require("ui/TickProgressBar")
local Item = require('Item')

local HeroSchedulerUI = {}
local HeroSchedulerButton = {}

local tUITextureSpriteSheetInfo =
{
    scheduler_enlarge_button = 'UI/hud',
    scheduler_inventory_button = 'UI/hud',
    scheduler_clipboard = 'UI/hud',
    icon_research = 'UI/hud',
    icon_rest = 'UI/hud',
    icon_timeoff = 'UI/hud',
    icon_train_power = 'UI/hud',
    icon_train_maxhp = 'UI/hud',
    icon_train_ability = 'UI/hud',
    icon_nothing = 'UI/hud',
    icon_parttimejob = 'UI/hud',
    icon_patrol = 'UI/hud',
    morale_superlow = 'UI/hud',
    morale_low = 'UI/hud',
    morale_med = 'UI/hud',
    morale_high = 'UI/hud',
    morale_superhigh = 'UI/hud',
    button_buy = 'UI/hud',
    icon_pp = 'UI/hud',
    icon_time = 'UI/hud',
    scheduler_help_button = 'UI/hud2',
    arrow = 'UI/hud3',
    ability_button = 'UI/shopheropack',
    icon_equipment_shield1 = 'UI/icons',
    promote_button = 'UI/shopheropack',
}

local tStatModPlusColor = { 0, .5, 0 }
local tStatModMinusColor = { .5, 0, 0 }
local tStatModColor = { .62, .62, .56 }

function HeroSchedulerButton:new(rRenderLayer, rParentUI, rHero, nIndex)
	local Ob = UIElement:new(rRenderLayer)
	Ob.rActivity = rActivity
	Ob.rHero = rHero
    Ob.nIndex = nIndex
    Ob.sIconPath = nil
    Ob.rParentUI = rParentUI
    Ob.rButton = nil
    Ob:addElement(Ob.rButton)
    Ob.bVisible = false

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        
        self.bVisible = bVisible
        
        if self.rButton then
            self.rButton:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        -- do nothing, leave resident
    end

	function Ob:update()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            local sIconPath = rActivity:getIconPath()
            if self.sIconPath ~= sIconPath then  
                if self.rButton then
                    self.rButton:setVisible(false)
                end
                self.rButton = Button:new(rRenderLayer, self, tUITextureSpriteSheetInfo, sIconPath, nil, {1, 1, 1}, {0.75, 0.75, 0.75})
                self.rButton:setLoc(0, 0)
                self.rButton:setParent(self)
                
                self.rButton:setVisible(self.bVisible)
                self.sIconPath = sIconPath
            end
        else
            if self.rButton then
                self.rButton:setVisible(false)
            end
            
            self.rButton = nil
        end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if g_Game:isActionEnabled('ActivitySelectEnabled') then
                g_Audio:playCue('MMOJ_Audio/UI/AddActivity')
                self.rParentUI:showActivitySelector(self.nIndex)                       
            end
        end
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
		if self.rButton then
            return self.rButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
	end

    function Ob:setPriority(nPri)
        if self.rButton and self.rButton.rButton then
            self.rButton.rButton:setPriority(nPri)
        end
    end

	return Ob
end

function HeroSchedulerUI:new(rRenderLayer, rHero)
	local Ob = UIElement:new(rRenderLayer)

	-- functions
	function Ob:update()
        self:updateText()
        if self.rRushButton then
            self.rRushButton:update()
        end
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:update()
        end
        self.rXPBar:update(self.rHero:getCurXP())    
        if self.rHero:isAtMaxLevel() then
            self.rPromoText:setString("MAX LEVEL") --g_DialogManager:getString("STXT045TEXT"))
            self.rPromoText:revealAll()
            self.rXPBar:update(self.rXPBar.nMaxAmount)
        elseif self.rHero:isPreppedForLevelUp() then
            self.rPromoteButton:setVisible(true)
        else
            self.rPromoText:setReveal(0)
        end

        if self.rAbilityButton then
            local rAbility = Ob.rHero:getActiveAbility()
            if rAbility then
                local sIconName = rAbility:getDef().sIconName
                local sIconSheet = rAbility:getDef().sIconSheet or 'UI/icons'
                if sIconName then
                    tUITextureSpriteSheetInfo[sIconName] = sIconSheet
                    self:loadUISpriteSheets(tUITextureSpriteSheetInfo)
                    self.rAbilityButton = self:getUITextureProp(sIconName, self.rAbilityButton)
                end
            end
            self.rAbilityDescText:setString(g_DialogManager:getString(rAbility:getDef().sAbilityName))
        end
        
        self.rHPBar:update(self.rHero:getCurHP())
        self.rManaBar:update(self.rHero:getCurMana(), self.rHero:getMaxMana()-self.rHero:getCurMana())
        self:updateActivityTimeText()
        local rEquippedItem = self.rHero:getItemAtSlot(1)
        if rEquippedItem == nil or rEquippedItem == Item.kEMPTY_SLOT then
            self.rInventoryText:setString(g_DialogManager:getString("STXT172TEXT"))
            self.rNoInventoryIconText:revealAll()
        else            
            self.rNoInventoryIconText:setReveal(0)
            self.rInventoryText:setString(g_DialogManager:getString(rEquippedItem:getItemDef().sName))
            self.rItemIcon:setVisible(true)
            local sIconName = rEquippedItem:getItemDef().iconName
            if sIconName then
                local sSpriteSheet = rEquippedItem:getItemDef().sSpriteSheet
                if sSpriteSheet then
                    local tInfo = {}
                    tInfo[sIconName] = sSpriteSheet
                    self:loadUISpriteSheets(tInfo)
                    self.rItemIcon = self:getUITextureProp(sIconName, self.rItemIcon)
                end
            end
        end
        self.rBottomBG:setPriority(self.rBackground:getPriority() - 4)
        self.rBottomLeftBG:setPriority(self.rBackground:getPriority() - 3)
        self.rItemIcon:setPriority(self.rBackground:getPriority() - 1)
        if self.rAbilityButton then
            self.rAbilityButton:setPriority(self.rBackground:getPriority() - 1)
        end
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:setPriority(self.rBackground:getPriority() - 1)
        end
        if not self.bCamAttached then
            if not rCameraLocInterpolator == nil or rCameraLocInterpolator:isDone() then
                Renderer.setCameraParent(self.rHero, -250, 20)
                self.bCamAttached = true
            end
        end
	end   

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/ClipBoard')
            self.rHero:onSchedulerOpened(true)
        else
            self.rStatHelpScreen:setVisible(false)
            self.rHero:onSchedulerOpened(false)
        end
        self:setElementsVisible(bVisible)
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:setVisible(bVisible)
        end
        Ob.rScrim:setVisible(false)
        Ob.rHPBar:setVisible(bVisible)
        Ob.rManaBar:setVisible(bVisible)
        Ob.rXPBar:setVisible(bVisible)
        if not bVisible then
            self:showRushButton(false)
            self:setTrainAbilityUIVisible(false)
            self:setAbilitySelectorVisible(false)
            self:hideActivitySelector()
            self.rPromoteButton:setVisible(false)
        else
            self.rNameText:setPriority(self.rHPBar:getElementsMaxPriority())
            self.rBottomBG:setPriority(self.rBackground:getPriority() - 4)
            self.rBottomLeftBG:setPriority(self.rBackground:getPriority() - 3)
            self.rItemIcon:setPriority(self.rBackground:getPriority() - 1)
            if self.rAbilityButton then
                self.rAbilityButton:setPriority(self.rBackground:getPriority() - 1)
            end
            for i, rButton in ipairs(self.tScheduleButtons) do
                rButton:setPriority(self.rBackground:getPriority() - 1)
            end
        end
        if self.rLeftArrow then
            self.rLeftArrow:setVisible(bVisible)
        end
        if self.rRightArrow then
            self.rRightArrow:setVisible(bVisible)
        end
        if self.rDetailsButton then
            self.rDetailsButton:setVisible(bVisible)
        end
        self.rStatHelpButton:setVisible(bVisible)

        if not bVisible and self.bCamAttached then
            Renderer.setCameraParent(nil)
        end
	end

	function Ob:setHero(rHero)
		self.rHero = rHero
    end

    function Ob:abilitySelectionCallback(rSelector, bHandled)
        self:setAbilitySelectorVisible(false)
    end

    function Ob:setAbilitySelectorVisible(bVis)
        self.rAbilitySelectionUI:setVisible(bVis)
        self.rScrim:setVisible(bVis)
        if bVis then
            local nPri = self.rAbilitySelectionUI.rBackground:getPriority() or 0
            self.rScrim:setPriority(nPri-1)
        end
    end
    
    function Ob:isActivitySelectorVisible()
        return self.bActivitySelectorVisible
    end

    function Ob:showActivitySelector(nIndex)
        self.bActivitySelectorVisible = true
        self.rActivitySelectionUI:setActivityIndex(nIndex)
        self.rActivitySelectionUI:setVisible(true)
        self.rScrim:setVisible(true)
        local nPri = self.rActivitySelectionUI.rBackground:getPriority() or 0
        self.rScrim:setPriority(nPri-1)
    end

    function Ob:hideActivitySelector()
        self.bActivitySelectorVisible = false
        self.rActivitySelectionUI:setVisible(false)
        self.rScrim:setVisible(false)
    end

    function Ob:closeScheduler()
        g_GUI:SetActivePane(nil)
    end

    function Ob:updateActivityTimeText()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            if rActivity.getActivityProgress then
                self.rCountdownUI:updateProgress(rActivity:getActivityProgress())
            end
            if rActivity.getRemainingTime then
                self.rRemainingTimeText:revealAll()
                local nRemainingTime = rActivity:getRemainingTime()    
                local nRemainingMinutes = math.floor(nRemainingTime / 60)
                local nRemainingSeconds = math.floor(nRemainingTime % 60)
                local sRemainingMinText = tostring(nRemainingMinutes)
                local sRemainingSecText = tostring(nRemainingSeconds)
                local sTimeText = ""
                if nRemainingMinutes < 10 then
                    sRemainingMinText = "0"..sRemainingMinText
                end
                if nRemainingSeconds < 10 then
                    sRemainingSecText = "0"..sRemainingSecText
                end
                if nRemainingSeconds % 2 == 0 then
                    sTimeText = sRemainingMinText..":"..sRemainingSecText
                else
                    sTimeText = sRemainingMinText.." "..sRemainingSecText
                end
                
                self.rRemainingTimeText:setString(sTimeText.." Left")
                
                if nRemainingTime > 0 then                    
                    if rActivity.getRushPrice then    
                        local nRushPrice = rActivity:getRushPrice() 
                        if nRushPrice > 0 then
                            self:showRushButton(true)
                            self.rRushPriceText:setString(tostring(nRushPrice))
                        end
                    end
                end
            else
                self:showRushButton(false)
                self.rRemainingTimeText:setReveal(0)
            end
        end
    end

    function Ob:attemptRushActivity()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            local nRemainingTime = rActivity:getRemainingTime()    
            local nRushPrice = rActivity:getRushPrice()
            if nRemainingTime > 0 and nRushPrice > 0 then
                if g_Game:getCurPP() >= nRushPrice then
                    g_Game:removePP(nRushPrice)
                    DFAnalytics.logEvent("ppp_spnt", { type = "rush", amount = nRushPrice })
                    DFAnalytics.incrStat('ppp_spnt_rush', nRushPrice)
                    rActivity:rushActivity()
                    return true
                end
            end
        end
        return false
    end

    function Ob:showRushButton(bVisible)
        if bVisible and not self.bVisible then
            return
        end
        self.rRushButton:setVisible(bVisible)
        self.rPPIcon:setVisible(bVisible)
        self.rCountdownUI:setVisible(bVisible)
        self.rRemainingTimeText:setVisible(bVisible)
        if bVisible then
            self.rRushPriceText:revealAll()
        else
            self.rRushPriceText:setReveal(0)
        end
    end

    function Ob:setTrainAbilityUIVisible(bVisible)
        if self.rAbilityUpgradeUI then
            self.rAbilityUpgradeUI:setVisible(bVisible)
        end
    end

    function Ob:updateText()      
		if self.rHero then
            local nStatPower, nBasePower = rHero:getStatPower()
			self.rNameText:setString(g_DialogManager:getString(rHero:getName()))
            if rHero:isAtMaxLevel() then
                self.rStatLevelText:setString("MAX")
            else
                self.rStatLevelText:setString(tostring(rHero:getStatLevel()))
            end
			self.rStatPowerText:setString(tostring(nStatPower))
            local nPowStatMod = self.rHero:getStatMod('nPower')
            if nPowStatMod > 0 then
                self.rStatPowerText:setColor(tStatModPlusColor[1], tStatModPlusColor[2], tStatModPlusColor[3])
            elseif nPowStatMod < 0 then
                self.rStatPowerText:setColor(tStatModMinusColor[1], tStatModMinusColor[2], tStatModMinusColor[3])
            else
                self.rStatPowerText:setColor(tStatModColor[1], tStatModColor[2], tStatModColor[3])
            end
			self.rStatHealthText:setString(tostring(rHero:getMaxHP()))
			self.rStatAwarenessText:setString(tostring(rHero:getStatAwareness()))
            local nAwarenessStatMod = self.rHero:getStatMod('nAwareness')
            if nAwarenessStatMod > 0 then
                self.rStatAwarenessText:setColor(tStatModPlusColor[1], tStatModPlusColor[2], tStatModPlusColor[3])
            elseif nAwarenessStatMod < 0 then
                self.rStatAwarenessText:setColor(tStatModMinusColor[1], tStatModMinusColor[2], tStatModMinusColor[3])
            else
                self.rStatAwarenessText:setColor(tStatModColor[1], tStatModColor[2], tStatModColor[3])
            end
			self.rStatShieldText:setString(tostring(rHero:getStatMod('nShield')))      
            local eMorale = rHero:getMorale()
            if eMorale == Character.eMorale.LOW then
                self.rLowMoraleIcon:setVisible(true)
                self.rMedLowMoraleIcon:setVisible(false)
                self.rMedMoraleIcon:setVisible(false)
                self.rMedHighMoraleIcon:setVisible(false)
                self.rHighMoraleIcon:setVisible(false)
            elseif eMorale == Character.eMorale.MEDLOW then
                self.rLowMoraleIcon:setVisible(false)
                self.rMedLowMoraleIcon:setVisible(true)
                self.rMedMoraleIcon:setVisible(false)
                self.rMedHighMoraleIcon:setVisible(false)
                self.rHighMoraleIcon:setVisible(false)
            elseif eMorale == Character.eMorale.MED then
                self.rLowMoraleIcon:setVisible(false)
                self.rMedLowMoraleIcon:setVisible(false)
                self.rMedMoraleIcon:setVisible(true)
                self.rMedHighMoraleIcon:setVisible(false)
                self.rHighMoraleIcon:setVisible(false)
            elseif eMorale == Character.eMorale.MEDHIGH then
                self.rLowMoraleIcon:setVisible(false)
                self.rMedLowMoraleIcon:setVisible(false)
                self.rMedMoraleIcon:setVisible(false)
                self.rMedHighMoraleIcon:setVisible(true)
                self.rHighMoraleIcon:setVisible(false)
            else
                self.rLowMoraleIcon:setVisible(false)
                self.rMedLowMoraleIcon:setVisible(false)
                self.rMedMoraleIcon:setVisible(false)
                self.rMedHighMoraleIcon:setVisible(false)
                self.rHighMoraleIcon:setVisible(true)
            end
		end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rRushButton then
            if self:attemptRushActivity() then
                g_GUI:HideHeroScheduler()
            end
        elseif rButton == self.rLeftArrow then
            local rNextHero = g_Game:getPrevHero(self.rHero)
            if rNextHero then
                g_GUI:ShowHeroScheduler(rNextHero)
            end
        elseif rButton == self.rRightArrow then
            local rNextHero = g_Game:getNextHero(self.rHero)
            if rNextHero then
                g_GUI:ShowHeroScheduler(rNextHero)
            end
        elseif rButton == self.rDetailsButton then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
            g_GUI:ShowHeroCardDetailsUI(self.rHero)
        elseif rButton == self.rStatHelpButton then
            g_Audio:playCue('MMOJ_Audio/UI/HelpButton')
            self.rStatHelpScreen:setVisible(true)
        elseif rButton == self.rPromoteButton then
            if self.rHero:isPreppedForLevelUp() then
				g_MiddleManager:getAvatar():addMMPromoteActivityAtIndex(1, self.rHero:getCharDefKey())
				g_GUI:HideHeroScheduler()
            end            
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)       
        if tProps then
            if self.rActivitySelectionUI and self.rActivitySelectionUI:isVisible() then
                if self.rActivitySelectionUI:onFinger(eventType, sx, sy, tProps) then
                else
                    if eventType == MOAITouchSensor.TOUCH_UP then
                        if g_Game:isActionEnabled('CloseWindow') then                       
                            self:hideActivitySelector()
                        end
                    end
                end
                return true
            end
            if self.rAbilitySelectionUI and self.rAbilitySelectionUI:isVisible() then
                if eventType == MOAITouchSensor.TOUCH_UP then
                    if self.rAbilitySelectionUI:onFinger(eventType, sx, sy, tProps) then
                    else
                        if g_Game:isActionEnabled('CloseWindow') then
                            self:setAbilitySelectorVisible(false)
                        end
                    end
                end
                return true
            end
            
            if eventType == MOAITouchSensor.TOUCH_UP then
                local bTouchedBG = false
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rItemIcon then
                        if g_Game:isActionEnabled('Shop') then                    
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            g_GUI:ShowCharInventoryUI(self.rHero)    
                            return true
                        end
                    end

                    if self.rAbilityUpgradeUI and self.rAbilityUpgradeUI:isVisible() then
                        self.rAbilityUpgradeUI:onFinger(eventType, sx, sy, tProps)
                        return true
                    end
            
                    if self.rStatHelpScreen and self.rStatHelpScreen:isVisible() then
                        if rProp == self.rStatHelpScreen then
                            g_Audio:playCue('MMOJ_Audio/UI/HelpButton')
                            self.rStatHelpScreen:setVisible(false)
                            return true
                        end
                    end

                    if rProp == self.rAbilityButton then
                        if g_Game:isActionEnabled('OpenAbilityPane') then
                            self:setAbilitySelectorVisible(true)
                        end
                    end

                    if rProp == self.rBackground then
                        bTouchedBG = true
                    end
                end
                if not bTouchedBG then
                    if g_Game:isActionEnabled('CloseWindow') then
                        g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                        g_GUI:HideHeroScheduler()
                    end                    
                end
            end
        end
         
        self.rRushButton:onFinger(eventType, sx, sy, tProps)
        if self.rLeftArrow then
            self.rLeftArrow:onFinger(eventType, sx, sy, tProps)
        end
        if self.rRightArrow then
            self.rRightArrow:onFinger(eventType, sx, sy, tProps)
        end
        if self.rDetailsButton then
            self.rDetailsButton:onFinger(eventType, sx, sy, tProps)
        end
        self.rStatHelpButton:onFinger(eventType, sx, sy, tProps)
        if self.rPromoteButton then
            self.rPromoteButton:onFinger(eventType, sx, sy, tProps)
        end
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:onFinger(eventType, sx, sy, tProps)
        end
        
		return true
	end

    if rHero.tCharDef.tDoodles then
        for i, rInfo in ipairs(rHero.tCharDef.tDoodles) do
            tUITextureSpriteSheetInfo[rInfo.sIndex] = rInfo.sSpriteSheetPath
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
	-- the background
    Ob.rBottomBG = MOAIProp2D.new()
	Ob.rBottomBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBottomBG:setLoc(30, -370)
	Ob.rBottomBG:setScl(490, 150)
	Ob.rBottomBG:setColor(1, 1, 1)
    Ob:addElement(Ob.rBottomBG)

    Ob.rBottomLeftBG = MOAIProp2D.new()
	Ob.rBottomLeftBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBottomLeftBG:setLoc(30, -370)
	Ob.rBottomLeftBG:setScl(150, 150)
	Ob.rBottomLeftBG:setColor(.91, .24, .23)
    Ob:addElement(Ob.rBottomLeftBG)

	Ob.rNoInventoryIconText = g_GUI:CreateTextBox(70, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rNoInventoryIconText:setString("!")
    Ob.rNoInventoryIconText:setRect(-150, 100, 150, 0)
	Ob.rNoInventoryIconText:setLoc(104, -400)
    Ob.rNoInventoryIconText:setColor(1, 1, 1)
    Ob.rNoInventoryIconText:setReveal(0)
    Ob:addOverlayElement(Ob.rNoInventoryIconText)

	Ob.rBackground = Ob:getUITextureProp('scheduler_clipboard')
	Ob.rBackground:setLoc(0, 0)
	Ob:addElement(Ob.rBackground)
	Ob.rHero = rHero

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setLoc(72, -64)
        Ob.rPortrait:setScl(.64)
        Ob:addElement(Ob.rPortrait)
    end

    Ob.tScheduleButtons = {}
    local nButtonXOffset = 214
    local nButtonYOffset = -380
    local nButtonXSize = 132
    for i = 1, 1 do
        local rNewButton = HeroSchedulerButton:new(Ob:getRenderLayer(), Ob, Ob.rHero, i)
        if rNewButton then
            rNewButton:setParent(Ob)
            rNewButton:setLoc(nButtonXOffset + (i - 1) * nButtonXSize, nButtonYOffset)
            Ob.tScheduleButtons[i] = rNewButton
        end
    end

	-- INITIALIZATION
	-- the stat text
	Ob.rStatPowerText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatPowerText:setString("99")
	Ob.rStatPowerText:setLoc(122, -152)
    Ob.rStatPowerText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatPowerText)

	Ob.rStatHealthText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatHealthText:setString("99")
	Ob.rStatHealthText:setLoc(226, -152)
    Ob.rStatHealthText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatHealthText)

	Ob.rStatShieldText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatShieldText:setString("99")
	Ob.rStatShieldText:setLoc(328, -152)
    Ob.rStatShieldText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatShieldText)

	Ob.rStatAwarenessText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatAwarenessText:setString("99")
	Ob.rStatAwarenessText:setLoc(428, -152)
    Ob.rStatAwarenessText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatAwarenessText)

    Ob.rLowMoraleIcon = Ob:getUITextureProp('morale_superlow')
    Ob.rLowMoraleIcon:setLoc(126, -278)
    Ob.rLowMoraleIcon:setScl(.75)
    Ob:addElement(Ob.rLowMoraleIcon)
    Ob.rMedLowMoraleIcon = Ob:getUITextureProp('morale_low')
    Ob.rMedLowMoraleIcon:setLoc(126, -278)
    Ob.rMedLowMoraleIcon:setScl(.75)
    Ob:addElement(Ob.rMedLowMoraleIcon)
    Ob.rMedMoraleIcon = Ob:getUITextureProp('morale_med')
    Ob.rMedMoraleIcon:setLoc(126, -278)
    Ob.rMedMoraleIcon:setScl(.75)
    Ob:addElement(Ob.rMedMoraleIcon)
    Ob.rMedHighMoraleIcon = Ob:getUITextureProp('morale_high')
    Ob.rMedHighMoraleIcon:setLoc(126, -278)
    Ob.rMedHighMoraleIcon:setScl(.75)
    Ob:addElement(Ob.rMedHighMoraleIcon)
    Ob.rHighMoraleIcon = Ob:getUITextureProp('morale_superhigh')
    Ob.rHighMoraleIcon:setLoc(126, -278)
    Ob.rHighMoraleIcon:setScl(.75)
    Ob:addElement(Ob.rHighMoraleIcon)   

	-- the level text
    Ob.rMoraleText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rMoraleText:setString(string.upper(g_DialogManager:getString("STXT044TEXT")))
    Ob.rMoraleText:setLoc(172, -258)
    Ob.rMoraleText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rMoraleText) 

	-- the level text
    Ob.rLevelLabelText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rLevelLabelText:setString(g_DialogManager:getString("STXT003TEXT"))
    Ob.rLevelLabelText:setLoc(272, -258)
    Ob.rLevelLabelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rLevelLabelText) 

	Ob.rStatLevelText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatLevelText:setString("99")
	Ob.rStatLevelText:setLoc(270, -288)
    Ob.rStatLevelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatLevelText)

    Ob.rInventoryLabelText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rInventoryLabelText:setString(string.upper(g_DialogManager:getString("STXT001TEXT")))
    Ob.rInventoryLabelText:setLoc(108, -350)
    Ob.rInventoryLabelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rInventoryLabelText) 

    Ob.rInventoryText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rInventoryText:setString(string.upper(g_DialogManager:getString("STXT001TEXT")))
    Ob.rInventoryText:setLoc(108, -510)
    Ob.rInventoryText:setColor(.62, .62, .56)
    Ob.rInventoryText:setRect(-75, 60, 75, 0)
    Ob:addElement(Ob.rInventoryText)

    Ob.rActivityLabelText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rActivityLabelText:setString(string.upper(g_DialogManager:getString("STXT187TEXT")))
    Ob.rActivityLabelText:setLoc(272, -350)
    Ob.rActivityLabelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rActivityLabelText)

    Ob.rAbilityLabelText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rAbilityLabelText:setString(string.upper(g_DialogManager:getString("STXT188TEXT")))
    Ob.rAbilityLabelText:setLoc(446, -350)
    Ob.rAbilityLabelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rAbilityLabelText)

    Ob.rAbilityDescText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rAbilityDescText:setString("PLACEHOLDER")
    Ob.rAbilityDescText:setLoc(446, -510)
    Ob.rAbilityDescText:setColor(.62, .62, .56)
    Ob.rAbilityDescText:setRect(-75, 60, 75, 0)
    Ob:addElement(Ob.rAbilityDescText)

    if g_Game:isActionEnabled('ShowHeroDetails') then
        -- the details button
        Ob.rDetailsButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'scheduler_enlarge_button', nil, {1, 1, 1}, {0.5, 0.5, 0.5}, false, {0, 0}, "STXT002TEXT", "boogaloo", 20, { 70, -4 }, { 0, 0, 0 })
        Ob.rDetailsButton:setLoc(398, -4)
        Ob.rDetailsButton:setParent(Ob)
    end

    Ob.rPromoteButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'promote_button', nil, {1, 1, 1}, {0.5, 0.5, 0.5}, false, {0, 0}, "STXT133TEXT", "boogaloo", 26, { 92, -2 }, { 0, 0, 0 })
    Ob.rPromoteButton:setLoc(336, -290)
    Ob.rPromoteButton:setParent(Ob)

    Ob.rHPBar = UIProgressBar:new(rRenderLayer, rHero:getMaxHP(), { r = 0.51, g = 0.93, b = 0.33 }, nil, false)
    Ob.rHPBar:setLoc(158, -64)
    Ob.rHPBar:setScl(326, 48)
    Ob.rHPBar:setParent(Ob)

	-- the name text
	Ob.rNameText = g_GUI:CreateTextBox(26, MOAITextBox.CENTER_JUSTIFY)
	Ob.rNameText:setString("Captain Kee")
	Ob.rNameText:setLoc(320, -70)
    --Ob:addOverlayElement(Ob.rNameText)
    Ob:addElement(Ob.rNameText)

    Ob.rManaBar = TickProgressBar:new(rRenderLayer, 326, {r=131/255,g=48/255,b=242/255}, {r=42/255,g=13/255,b=77/255}, {r=0/255,g=0/255,b=0/255}, {r=237/255,g=21/255,b=88/255}) --,  rHeroEventActor:getHero():getMaxMana(), 
    --Ob.rManaBar = UIProgressBar:new(rRenderLayer, rHero:getMaxMana(), {r=61/255,g=201/255,b=217/255}, {r=1/255,g=22/255,b=64/255})
    Ob.rManaBar:setLoc(158, -116)
    Ob.rManaBar:setScl(1, 1.4)
    Ob.rManaBar:setParent(Ob)

    Ob.rXPText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rXPText:setString(g_DialogManager:getString("STXT012TEXT"))
    Ob.rXPText:setLoc(428, -258)
    Ob.rXPText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rXPText) 

    Ob.rXPBar = UIProgressBar:new(rRenderLayer, rHero:getXPForNextLevel(), { r = 1, g = 1, b = 1 }, { r = 0.62, g = 0.62, b = 0.56 }, false)
    Ob.rXPBar:setLoc(342, -296)
    Ob.rXPBar:setScl(172, 28)
    Ob.rXPBar:setParent(Ob)

    Ob.rPromoText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    -- MTF TODO: use linecode
    Ob.rPromoText:setString("PROMOTION EARNED!") --g_DialogManager:getString("STXT045TEXT"))
    local x,y = Ob.rXPBar:getLoc()
    Ob.rPromoText:setLoc(x+86, y-2)
    Ob.rPromoText:setColor(196/255, 145/255, 54/255, 1)
    Ob.rPromoText:setReveal(0)
    --Ob:addOverlayElement(Ob.rPromoText)
    Ob:addElement(Ob.rPromoText)

    Ob.rRemainingTimeText = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'ptsans')
    Ob.rRemainingTimeText:setString("0:90")
    Ob.rRemainingTimeText:setRect(-150, 40, 150, 0)
    Ob.rRemainingTimeText:setColor(.62, .62, .56)
    Ob.rRemainingTimeText:setLoc(276, -510)
    Ob:addElement(Ob.rRemainingTimeText)

    --RUSH BUTTON (need to alter centering based on number of digits
    Ob.rRushButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT116TEXT", 'ptsans', 24, { 45, -9 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, nil, 1, { 0, 0, 0, 0.75 })
    Ob.rRushButton:setLoc(199, -540)
    Ob.rRushButton:setScl(1, 1)
    Ob.rRushButton:setParent(Ob)

    Ob.rPPIcon = Ob:getUITextureProp('icon_pp')
    Ob.rPPIcon:setLoc(276, -548)
    Ob.rPPIcon:setScl(0.76)
    Ob:addOverlayElement(Ob.rPPIcon)

    Ob.rRushPriceText = g_GUI:CreateTextBox(26, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRushPriceText:setString("1")
    Ob.rRushPriceText:setRect(0, 60, 150, 0)
    Ob.rRushPriceText:setColor(1, 1, 1)
    Ob.rRushPriceText:setLoc(316, -548)
    Ob:addOverlayElement(Ob.rRushPriceText)   

    local rAbility = Ob.rHero:getActiveAbility()
    if rAbility then
        local sIconName = rAbility:getDef().sIconName
        local sIconSheet = rAbility:getDef().sIconSheet or 'UI/eventdetails'
        if sIconName then
            tUITextureSpriteSheetInfo[sIconName] = sIconSheet
            Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
            Ob.rAbilityButton = Ob:getUITextureProp(sIconName)
            if Ob.rAbilityButton then
                Ob.rAbilityButton:setLoc(384, -378)
                Ob.rAbilityButton:setScl(1.05)
                Ob:addElement(Ob.rAbilityButton)
            end
        end
    end

    -- help button!
    Ob.rStatHelpButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'scheduler_help_button', nil, {1, 1, 1}, {0.5, 0.5, 0.5})
    Ob.rStatHelpButton:setLoc(34, -272)
    Ob.rStatHelpButton:setScl(1.2)
    Ob.rStatHelpButton:setParent(Ob)    

    Ob.rActivitySelectionUI = ActivitySelectionUI:new(rRenderLayer, Ob, rHero)
    Ob.rActivitySelectionUI:setLoc(550, 0)

    Ob.rStatHelpScreen = StatHelpScreen:new(rRenderLayer)
    Ob.rStatHelpScreen:setLoc(518, -84)
--[[
    if rHero.tCharDef.tDoodles then
        for i, rInfo in ipairs(rHero.tCharDef.tDoodles) do
            local rDoodle = Ob:getUITextureProp(rInfo.sIndex)
            if rDoodle then
                rDoodle:setLoc(rInfo.loc[1], rInfo.loc[2])
                Ob:addElement(rDoodle)
            end
        end
    end
]]--
    Ob.rCountdownUI = CountdownUI:new(rRenderLayer)
    Ob.rCountdownUI:setLoc(306, -370)
    Ob.rCountdownUI:setParent(Ob)

    if g_Game:isActionEnabled('CycleHeroScheduler') then
        Ob.rLeftArrow = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'arrow', nil, {1, 1, 1}, {0.5, 0.5, 0.5})
        Ob.rLeftArrow:setLoc(64, -64)
        Ob.rLeftArrow:setScl(-1, 1)
        Ob.rLeftArrow:setParent(Ob)

        Ob.rRightArrow = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'arrow', nil, {1, 1, 1}, {0.5, 0.5, 0.5})
        Ob.rRightArrow:setLoc(492, -64)
        Ob.rRightArrow:setParent(Ob)
    end

    Ob.rItemIcon = Ob:getUITextureProp('icon_equipment_shield1') -- to have something by default
    Ob.rItemIcon:setScl(1.1)
    Ob.rItemIcon:setLoc(42, -378)
    Ob.rItemIcon:setVisible(false)
    --Ob:addOverlayElement(Ob.rItemIcon)
    Ob:addElement(Ob.rItemIcon)

    Ob.rScrim = MOAIProp2D.new()
	Ob.rScrim:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rScrim:setLoc(0, 0)
	Ob.rScrim:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rScrim:setColor(.4, .4, .4, .5)
	Ob.rScrim:setLoc(0, 0)
    Ob:addElement(Ob.rScrim)
    Ob.rScrim:setVisible(false)
    
    --Ob.rAbilityUpgradeUI = AbilityUpgradeUI:new(rRenderLayer, rHero, Ob)
    --[[
    Ob.rAbilityUpgradeUI = AbilityUpgradeUI:new(Ob.rOverlayRenderLayer, rHero, Ob)
    Ob.rAbilityUpgradeUI:setLoc(0, -130)
    Ob.rAbilityUpgradeUI:setParent(Ob)
    ]]--
    --Ob.rAbilitySelectionUI = AbilitySelectionUI:new(Ob.rOverlayRenderLayer, Ob, rHero)
    Ob.rAbilitySelectionUI = AbilitySelectionUI:new(rRenderLayer, Ob, rHero)
    Ob.rAbilitySelectionUI:setLoc(500, 0)
    Ob.rAbilitySelectionUI:setParent(Ob)

    Ob:showRushButton(false)

	return Ob
end

return HeroSchedulerUI
