local util = require("DFUtil")
local HeroDetailInventoryButton = require("ui/HeroDetailInventoryButton")
local ListUI = require("ui/ListUI")
local UIElement = require('ui/UIElement')

local HeroSelectorUI = {}

function HeroSelectorUI:new(rRenderLayer, tHeroes, rShopUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rListUI:onPaneClosed()
    end

    function Ob:update()
        self.rListUI:update()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:setTopHero(rHero)
        for i, rButton in ipairs(self.tHeroButtons) do
            if rButton:getHero() == rHero then
                self.rListUI:setTopItem(i)
                break
            end
        end
    end
    
    function Ob:handleItemSelection(rItem)
        self.rHero = rItem:getHero()
        if self.rShopUI and rHero then
            self.rShopUI:onHeroSelected(rHero)
        end
        for i, rButton in ipairs(self.tHeroButtons) do
            if rButton == rItem then
                rButton:onSelected(true)
            else
                rButton:onSelected(false)
            end
        end
    end

    function Ob:getSelectedHero()
        return self.rHero
    end

    function Ob:getTopHero()
        return self.tHeroButtons[1]:getHero()
    end

    function Ob:onEquipped(bEquipped)
        self.rListUI:update()
    end

    -- create the hero buttons
    Ob.tHeroButtons = {}
    Ob.rShopUI = rShopUI
    if not tHeroes then
        tHeroes = g_Game:getActiveHeroes()
    end
    for i, rHero in ipairs(tHeroes) do
        if not Ob.rHero then
            Ob.rHero = rHero
        end
        local rNewButton = HeroDetailInventoryButton:new(Ob.rRenderLayer, rHero)
        table.insert(Ob.tHeroButtons, rNewButton)
        if i == 1 then
            rNewButton:onSelected(true)
        else
            rNewButton:onSelected(false)
        end
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tHeroButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

return HeroSelectorUI