local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local DFGraphics = require('DFCommon.Graphics')
local ImageProgressBar = {}

-- quick UI Button
function ImageProgressBar:new(rRenderLayer, nMaxAmount, fgName, bgName)
	local Ob = UIElement:new(rRenderLayer)

    local xBarScale = 1
    local yBarScale = 1

    local tUITextureSpriteSheetInfo = 
    {
    HealthBar_Green = 'UI/fightscreen',
    HealthBar_Red = 'UI/fightscreen',
    }
    
	-- functions
    function Ob:update(nProgressAmount)
        nProgressAmount = math.max(math.min(nProgressAmount,self.nMaxAmount),0)

        self.nCurAmount = nProgressAmount
        local scl = xBarScale * nProgressAmount / self.nMaxAmount
        local t = MOAITransform2D.new()

        local inv = (scl > .01 and 1/scl) or 1
        local rSpriteSheet = self.tTextureToSpriteSheet[fgName]
        local idx = rSpriteSheet.names[fgName]
        -- by scaling uv, we'll start at r.x0*scl instead of r.x0. Still want to start
        -- at r.x0, so transform by r.x0 - r.x0*scl.
        local x0,y0,x1,y1,x2,y2,x3,y3 = rSpriteSheet:getUVQuad(idx)
        if x3 < x0 then
            -- oh, and we get to worried about rotated sprites.
            t:setScl(yBarScale,scl)
            t:setLoc(0, y0 - y0*scl)
        else
            t:setScl(scl,yBarScale)
            t:setLoc(x0 - x0*scl,0)
        end
        self.rFGBar:setUVTransform(t)
        self.rFGBar:setScl(scl, yBarScale)
        -- accursed center-aligned sprites.
        self.rFGBar:setLoc(-(1-scl)*self.nFGWidth*.5, 0)
    end

    function Ob:getCurrentAmount()
        return self.nCurAmount
    end

    function Ob:getDims()
        return self.rBGBar:getDims()
    end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if bVisible then
            self:update(self.nCurAmount)
        end
	end

    function Ob:setPriority(nPriority)
        if self.rBGBar then
            self.rBGBar:setPriority(nPriority)
        end
        if self.rFGBar then
            self.rFGBar:setPriority(nPriority + 3)
        end
    end
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:isVisible()
        return self.bVisible
    end

    Ob.nMaxAmount = nMaxAmount
    Ob.nCurAmount = 0

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo,  'center', 'center')

    Ob.rBGBar = Ob:getUITextureProp(bgName)
    Ob.rBGBar:setLoc(0, 0)
    Ob:addElement(Ob.rBGBar)

    Ob.rFGBar = Ob:getUITextureProp(fgName)
    Ob.rFGBar:setLoc(0, 0)
    Ob.nFGWidth = Ob.rFGBar:getDims()
    Ob:addElement(Ob.rFGBar)

    Ob.bVisible = false

    return Ob
end

return ImageProgressBar
