local IntroCutscene = {}
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local util = require('DFUtil')
local ResumeUI = require('ui/ResumeUI')

local tUITextureSpriteSheetInfo =
{
    Newspaper = 'UI/reward2',
    NewspaperPic = 'UI/intro',
    MM_Newspaper = 'UI/intro',
    skullface = 'UI/intro',
    MM = 'UI/intro',
    hired_stamp = 'UI/hud2',
    newhero_bkgd = 'UI/reward',
    Paperclip = 'UI/intro',
}

function IntroCutscene:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:cutsceneThread()
        g_Audio:stopMusic()
        g_Audio:stopCurAmbience()
        g_Audio:playCue('MMOJ_Audio/SFX/Office/Newspaper')
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Defeat', false)

        self.rNewspaperBGLeft:setVisible(true)
        self.rNewspaperBGRight:setVisible(true)

        self.rNewspaper:moveRot(720, 1.5)
        self.rNewspaper:moveScl(0.85, 0.85, 1.5)
        util.sleep(1.5)
        self.bPressed = false
        while not self.bPressed do
            coroutine.yield()
        end
        self.rFadeupForeground:moveColor(0, 0, 0, 1, 0.75)
        util.sleep(0.75)

        self.rNewspaperBGLeft:setVisible(false)
        self.rNewspaperBGRight:setVisible(false)
        self.rNewspaper:setVisible(false)
        self.rMainCaptionField:setReveal(0)
        self.rSideCaptionField:setReveal(0)
        self.rWinnerPortrait:setVisible(false)
        self.rLoserPortrait:setVisible(false)

        self.rBackground:setVisible(true)
        self.rMM:setVisible(true)
        self.rMMNewspaper:setVisible(true)

        util.sleep(0.25)

        self.rFadeupForeground:moveColor(0, 0, 0, -1, 0.75)       
        util.sleep(1)

        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Tutorial', true)

        local rMM = g_MiddleManager:getAvatar()
        rMM:sayLine("STXT155MIDM", 'lower', true, true)
        self.rMMNewspaper:moveLoc(0, -210, 1, MOAIEaseType.LINEAR)

        rMM:sayLine("STXT156MIDM", 'lower', true, true)
		rMM:sayLine("STXT171MIDM", 'lower', true, true)
        rMM:sayLine("STXT157MIDM", 'lower', true, true)
        g_GUI:HideDialogText()

        self.rFadeupForeground:moveColor(0, 0, 0, 1, 0.75)
        util.sleep(0.75)

        self.rMM:setVisible(false)
        self.rMMNewspaper:setVisible(false)

        util.sleep(0.25)

        self.rBackground:setColor(.64, .19, .13)
        self.rResume:setVisible(true)
        self.rMMPortrait:setVisible(true)
        self.rMMPaperClip:setVisible(true)
        self.rNewspaperBGLeft:setVisible(true)
        self.rNewspaperBGRight:setVisible(true)

        self.rFadeupForeground:moveColor(0, 0, 0, -1, 0.75)
        util.sleep(0.75)

        self.rResume:moveScl(0.9, 0.9, 0.9, 1)
        self.rResume:moveLoc(-240, -70 + Renderer.getUIHalfHeight(), 0, 1)
        util.sleep(1)

        self.bPressed = false
        while not self.bPressed do
            coroutine.yield()
        end

        self.rStamp:setVisible(true)
        g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')

        util.sleep(1.25)

		g_Audio:playAmbience('MMOJ_Audio/Ambience/OfficeAmbience1')
        self.bRunningThread = false
    end

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            if not self.bRunningThread then
                self.bRunningThread = true
                self.rThread = MOAIThread.new()
                self.rThread:run(self.cutsceneThread, self)
            end
        else
            self.rResume:setVisible(false)
            self:alignUISpriteSheets('left', 'top')
        end
    end

    function Ob:isRunning()
        return self.bRunningThread
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()        
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            self.bPressed = true
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    local imageSizeX = 512
    local imageSizeY = 512
        
    -- we double the background up
    local backgroundScaleX = Renderer.getUIWidth() / imageSizeX
    local backgroundScaleY = Renderer.getUIHeight() / imageSizeY

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(.8, .63, .36)
    Ob.rBackground:setVisible(false)
	Ob:addElement(Ob.rBackground)

    Ob.rNewspaperBGLeft = Ob:getUITextureProp('newhero_bkgd')
    Ob.rNewspaperBGLeft:setLoc(0, -Renderer.getUIHeight() / 2)
    Ob.rNewspaperBGLeft:setScl(backgroundScaleX * 1.02, backgroundScaleY)
    Ob.rNewspaperBGLeft:setVisible(false)
    Ob:addElement(Ob.rNewspaperBGLeft)

    Ob.rNewspaperBGRight = Ob:getUITextureProp('newhero_bkgd')
    Ob.rNewspaperBGRight:setLoc(Renderer.getUIWidth(), -Renderer.getUIHeight() / 2)
    Ob.rNewspaperBGRight:setScl(-backgroundScaleX, backgroundScaleY)
    Ob.rNewspaperBGRight:setVisible(false)
    Ob:addElement(Ob.rNewspaperBGRight)
      
    Ob.rNewspaper = Ob:getUITextureProp('Newspaper')
    Ob.rNewspaper:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rNewspaper:setScl(0.1, 0.1)
    Ob:addElement(Ob.rNewspaper)

 	Ob.rMainCaptionField = g_GUI:CreateTextBox(120, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	Ob.rMainCaptionField:setString(g_DialogManager:getString("STXT159TEXT"))
    Ob.rMainCaptionField:setRect(-395, 160, 395, 0)
	Ob.rMainCaptionField:setLoc(-10, 210)
	Ob.rMainCaptionField:setColor(0, 0, 0)
    Ob:addElement(Ob.rMainCaptionField)
    Ob.rMainCaptionField:setParent(Ob.rNewspaper)

 	Ob.rSideCaptionField = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	Ob.rSideCaptionField:setString(g_DialogManager:getString("STXT160TEXT"))
    Ob.rSideCaptionField:setRect(-75, 260, 75, 0)
	Ob.rSideCaptionField:setLoc(-318, 75)
	Ob.rSideCaptionField:setColor(0, 0, 0)
    Ob:addElement(Ob.rSideCaptionField)
    Ob.rSideCaptionField:setParent(Ob.rNewspaper)

    Ob.rWinnerPortrait = Ob:getUITextureProp('skullface')
    Ob.rWinnerPortrait:setLoc(-126, -38)
    Ob.rWinnerPortrait:setScl(1.7)
    Ob:addElement(Ob.rWinnerPortrait)
    Ob.rWinnerPortrait:setParent(Ob.rNewspaper)

    Ob.rLoserPortrait = Ob:getUITextureProp('NewspaperPic')
    Ob.rLoserPortrait:setLoc(100, -39)
    Ob.rLoserPortrait:setScl(0.85)
    Ob:addElement(Ob.rLoserPortrait)
    Ob.rLoserPortrait:setParent(Ob.rNewspaper)

    Ob.rMM = Ob:getUITextureProp('MM')
    Ob.rMM:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMM:setVisible(false)
    Ob:addElement(Ob.rMM)

    Ob.rMMNewspaper = Ob:getUITextureProp('MM_Newspaper')
    Ob.rMMNewspaper:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMMNewspaper:setVisible(false)
    Ob:addElement(Ob.rMMNewspaper)

    Ob.rResume = ResumeUI:new(rRenderLayer, g_MiddleManager:getAvatar())
    Ob.rResume:setLoc(Renderer.getUIHalfWidth() - 10, -Renderer.getUIHalfHeight() + 60)
    Ob.rResume:setScl(0.1)
    Ob.rResume:setParent(Ob)

    Ob.rMMPortrait = g_MiddleManager:getAvatar():getPortraitProp()
    Ob.rMMPortrait:setLoc(430, 0)
    Ob.rMMPortrait:setRot(-10)
    Ob.rMMPortrait:setVisible(false)
    Ob:addOverlayElement(Ob.rMMPortrait)
    Ob.rMMPortrait:setParent(Ob.rResume)

    Ob.rMMPaperClip = Ob:getUITextureProp('Paperclip')
    Ob.rMMPaperClip:setLoc(440, -18)
    Ob.rMMPaperClip:setVisible(false)
    Ob:addOverlayElement(Ob.rMMPaperClip)
    Ob.rMMPaperClip:setParent(Ob.rResume)

    Ob.rStamp = Ob:getUITextureProp('hired_stamp')
    Ob.rStamp:setLoc(Renderer.getUIHalfWidth(), -300)
    Ob.rStamp:setScl(1.5)
    Ob.rStamp:setRot(30)
    Ob.rStamp:setVisible(false)
    Ob:addOverlayElement(Ob.rStamp)

    Ob.rFadeupForeground = MOAIProp2D.new()
	Ob.rFadeupForeground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rFadeupForeground:setLoc(0, 0)
	Ob.rFadeupForeground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rFadeupForeground:setColor(0, 0, 0, 0)
	Ob:addOverlayElement(Ob.rFadeupForeground)

    Ob.bRunningThread = false

    return Ob
end

return IntroCutscene