local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local util = require('DFUtil')

local LetterUI = {}

local tUITextureSpriteSheetInfo =
{
    twohb = 'UI/hud2',
}

function LetterUI:new(rRenderLayer, sTitle, sMessage, rListener)
    local Ob = UIElement:new(rRenderLayer)
    
    Ob.rListener = rListener
    
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end
    
    function Ob:onFinger(eventType, sx, sy)
		if eventType == MOAITouchSensor.TOUCH_DOWN then
            if rListener and rListener.onLetterClosed then
                rListener:onLetterClosed()
            end
            
            self:onPaneClosed()
            g_GUI:HideLetterScreen()
        end
    end
    
    local bgSizeX = 500
    local bgSizeY = 600
    
    local offsetX = (Renderer.getUIWidth() - bgSizeX) / 2
    local offsetY = -(Renderer.getUIHeight() - bgSizeY) / 2
    
    sMessage = string.gsub(sMessage, "\\n", "\n")
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    
    Ob.rScrim = MOAIProp2D.new()
    Ob.rScrim:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rScrim:setLoc(-offsetX, -offsetY)
    Ob.rScrim:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
    Ob.rScrim:setColor(0, 0, 0, 0.5)
    Ob:addElement(Ob.rScrim)
    
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)    
    Ob.rBackground:setScl(bgSizeX, bgSizeY)
	Ob.rBackground:setColor(1, 1, 1)
    Ob:addElement(Ob.rBackground)   

    Ob.rTitleText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rTitleText:setString(sTitle) 
    Ob.rTitleText:setRect(0, 80, 424, 0)
    Ob.rTitleText:setLoc(26, -20)
    Ob:addElement(Ob.rTitleText)
    
    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(24, -64)
    Ob.rLine1:setScl(450, 3)
    Ob.rLine1:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine1)
    
    Ob.rMessageText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rMessageText:setString(sMessage)
    Ob.rMessageText:setRect(0, 500, 450, 0)
    Ob.rMessageText:setLoc(34, -80)
    Ob:addElement(Ob.rMessageText)
    
    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(24, -(bgSizeY - 64))
    Ob.rLine2:setScl(450, 3)
    Ob.rLine2:setColor(0, 0, 0)
    Ob:addElement(Ob.rLine2)
    
    Ob.r2HBLogo = Ob:getUITextureProp('twohb')
    Ob.r2HBLogo:setLoc((bgSizeX / 2) - 16, -(bgSizeY - 40))
    Ob:addElement(Ob.r2HBLogo)
    
    Ob:setLoc(offsetX, offsetY)
    
    return Ob
end


return LetterUI