-------- LEVELUP UI ---------
local LevelUpUI = {}
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local util = require("DFUtil")

local tUITextureSpriteSheetInfo =
{
    levelup_minus_dim = 'UI/levelup',
    levelup_plus = 'UI/levelup',
    levelup_plus_dim = 'UI/levelup',
    levelup_clipboard = 'UI/levelup',
    levelup_minus = 'UI/levelup',
    scheduler_inventory_button = 'UI/hud',
}

function LevelUpUI:new(rRenderLayer, rHero)
	local Ob = UIElement:new(rRenderLayer)

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rHero = rHero

	Ob.nPtsToAllocate = rHero:getPtsToAllocate()
	Ob.nPowerToAllocate = 0
	Ob.nHPToAllocate = 0
	Ob.nAwarenessToAllocate = 0

	-- Background
	Ob.rBackground = Ob:getUITextureProp('levelup_clipboard')
	Ob:addElement(Ob.rBackground)

	-- text
	Ob.rNameText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY)
    Ob.rNameText:setRect(-150, 80, 150, 0)
    Ob.rNameText:setString(g_DialogManager:getString(rHero:getName()))
	Ob.rNameText:setLoc(220, -130)
	Ob:addElement(Ob.rNameText)

	Ob.rNameLabel = g_GUI:CreateTextBox(18, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rNameLabel:setRect(-150, 80, 150, 0)
    Ob.rNameLabel:setString("EMPLOYEE NAME")
	Ob.rNameLabel:setLoc(220, -170)
	Ob:addElement(Ob.rNameLabel)

	-- lvl text
	Ob.rLvlText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY)
    Ob.rLvlText:setRect(-150, 80, 150, 0)
	Ob.rLvlText:setLoc(414, -130)
    Ob.rLvlText:setString(tostring(rHero:getStatLevel()))
	Ob:addElement(Ob.rLvlText)

	Ob.rLvlLabel = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rLvlLabel:setRect(-150, 80, 150, 0)
	Ob.rLvlLabel:setLoc(414, -170)
    Ob.rLvlLabel:setString("LEVEL")
	Ob:addElement(Ob.rLvlLabel)

	Ob.rLvlDesc = g_GUI:CreateTextBox(65, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rLvlDesc:setRect(-150, 80, 150, 0)
	Ob.rLvlDesc:setString("PROMOTION!")
	Ob.rLvlDesc:setLoc(310, -70)
	Ob:addElement(Ob.rLvlDesc)


	-- pts text
    local rUnlockedAbility = rHero:getAbilityAtLevel(rHero:getStatLevel())
    if rUnlockedAbility then
        Ob.rSkillText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'bangers')
        Ob.rSkillText:setRect(-150, 80, 150, 0)
        Ob.rSkillText:setString("NEW SKILL:")
        Ob.rSkillText:setLoc(210, -210)
        Ob:addElement(Ob.rSkillText)

        Ob.rAbilityName = g_GUI:CreateTextBox(36, MOAITextBox.CENTER_JUSTIFY)
        Ob.rAbilityName:setRect(-150, 80, 150, 0)
        Ob.rAbilityName:setString(rUnlockedAbility:getName())
        Ob.rAbilityName:setLoc(370, -200)
        Ob:addElement(Ob.rAbilityName)
    end

	-- stats
	Ob.rPowerText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY)
    Ob.rPowerText:setRect(-150, 80, 150, 0)
	Ob.rPowerText:setString('99')
	Ob.rPowerText:setLoc(130, -370)
	Ob:addElement(Ob.rPowerText)

	Ob.rHealthText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY)
    Ob.rHealthText:setRect(-150, 80, 150, 0)
    Ob.rHealthText:setString('99')
	Ob.rHealthText:setLoc(280, -370)
	Ob:addElement(Ob.rHealthText)

	Ob.rAwarenessText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY)
    Ob.rAwarenessText:setRect(-150, 80, 150, 0)
    Ob.rAwarenessText:setString('99')
	Ob.rAwarenessText:setLoc(430, -370)
	Ob:addElement(Ob.rAwarenessText)

	Ob.rPowerLabel = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rPowerLabel:setRect(-150, 80, 150, 0)
	Ob.rPowerLabel:setString("POWER")
	Ob.rPowerLabel:setLoc(130, -440)
	Ob:addElement(Ob.rPowerLabel)

	Ob.rHealthLabel = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rHealthLabel:setRect(-150, 80, 150, 0)
    Ob.rHealthLabel:setString("HEALTH")
	Ob.rHealthLabel:setLoc(280, -440)
	Ob:addElement(Ob.rHealthLabel)

	Ob.rAwarenessLabel = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rAwarenessLabel:setRect(-150, 80, 150, 0)
    Ob.rAwarenessLabel:setString("INTELLECT")
	Ob.rAwarenessLabel:setLoc(430, -440)
	Ob:addElement(Ob.rAwarenessLabel)

	-- the grey buttons
	Ob.rPowerPlusDimButton = Ob:getUITextureProp('levelup_plus_dim')
	Ob.rPowerPlusDimButton:setLoc(70, -264)
	Ob:addElement(Ob.rPowerPlusDimButton)

	Ob.rPowerMinusDimButton = Ob:getUITextureProp('levelup_minus_dim')
	Ob.rPowerMinusDimButton:setLoc(70, -464)
	Ob:addElement(Ob.rPowerMinusDimButton)	

	Ob.rHPPlusDimButton = Ob:getUITextureProp('levelup_plus_dim')
	Ob.rHPPlusDimButton:setLoc(220, -264)
	Ob:addElement(Ob.rHPPlusDimButton)	

	Ob.rHPMinusDimButton = Ob:getUITextureProp('levelup_minus_dim')
	Ob.rHPMinusDimButton:setLoc(220, -464)
	Ob:addElement(Ob.rHPMinusDimButton)	

	Ob.rAwarenessPlusDimButton = Ob:getUITextureProp('levelup_plus_dim')
	Ob.rAwarenessPlusDimButton:setLoc(370, -264)
	Ob:addElement(Ob.rAwarenessPlusDimButton)	

	Ob.rAwarenessMinusDimButton = Ob:getUITextureProp('levelup_minus_dim')
    Ob.rAwarenessMinusDimButton:setLoc(370, -464)
	Ob:addElement(Ob.rAwarenessMinusDimButton)	

	Ob.rPowerPlusButton = Ob:getUITextureProp('levelup_plus')
    Ob.rPowerPlusButton:setVisible(false)
	Ob.rPowerPlusButton:setLoc(70, -264)
	Ob:addElement(Ob.rPowerPlusButton)

	Ob.rPowerMinusButton = Ob:getUITextureProp('levelup_minus')
    Ob.rPowerMinusButton:setVisible(false)
	Ob.rPowerMinusButton:setLoc(70, -464)
	Ob:addElement(Ob.rPowerMinusButton)	

	Ob.rHPPlusButton = Ob:getUITextureProp('levelup_plus')
    Ob.rHPPlusButton:setVisible(false)
	Ob.rHPPlusButton:setLoc(220, -264)
	Ob:addElement(Ob.rHPPlusButton)	

	Ob.rHPMinusButton = Ob:getUITextureProp('levelup_minus')
    Ob.rHPMinusButton:setVisible(false)
	Ob.rHPMinusButton:setLoc(220, -464)
	Ob:addElement(Ob.rHPMinusButton)	

	Ob.rAwarenessPlusButton = Ob:getUITextureProp('levelup_plus')
    Ob.rAwarenessPlusButton:setVisible(false)
	Ob.rAwarenessPlusButton:setLoc(370, -264)
	Ob:addElement(Ob.rAwarenessPlusButton)	

	Ob.rAwarenessMinusButton = Ob:getUITextureProp('levelup_minus')
    Ob.rAwarenessMinusButton:setVisible(false)
    Ob.rAwarenessMinusButton:setLoc(370, -464)
	Ob:addElement(Ob.rAwarenessMinusButton)	

	Ob.rOKButton = Ob:getUITextureProp('scheduler_inventory_button')
	Ob.rOKButton:setLoc(110, -530)
	Ob:addElement(Ob.rOKButton)	

	Ob.rOKText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'gothic')
	Ob.rOKText:setRect(-100, 60, 100, 0)
	Ob.rOKText:setString("FINISHED")
	Ob.rOKText:setLoc(290, -540)
	Ob.rOKText:setColor(1, 1, 1)	
	Ob:addElement(Ob.rOKText)	

	-- functions
	function Ob:update()
		if not self.rHero then
			return
		end        
		self.rPowerText:setString(tostring(self.rHero:getBaseStatPower() + self.nPowerToAllocate))
		self.rHealthText:setString(tostring(self.rHero:getMaxHP() + self.nHPToAllocate))
		self.rAwarenessText:setString(tostring(self.rHero:getStatAwareness() + self.nAwarenessToAllocate))
		
		if self.nPtsToAllocate == 0 then
            self.rOKText:setString("FINISHED")
            self.rPowerPlusButton:setVisible(false)
            self.rHPPlusButton:setVisible(false)
            self.rAwarenessPlusButton:setVisible(false)
		else
            self.rOKText:setString("ADD "..self.nPtsToAllocate.." MORE POINTS")

            self.rPowerPlusButton:setVisible(true)
            self.rHPPlusButton:setVisible(true)
            self.rAwarenessPlusButton:setVisible(true)
		end

		if self.nPowerToAllocate > 0 then
			self.rPowerMinusButton:setVisible(true)
		else
			self.rPowerMinusButton:setVisible(false)
		end

		if self.nHPToAllocate > 0 then
			self.rHPMinusButton:setVisible(true)
		else
			self.rHPMinusButton:setVisible(false)
		end

		if self.nAwarenessToAllocate > 0 then
			self.rAwarenessMinusButton:setVisible(true)
		else
			self.rAwarenessMinusButton:setVisible(false)
		end
	end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            self:update()
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
		if eventType == MOAITouchSensor.TOUCH_UP and tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rPowerPlusButton then
                    if self.nPtsToAllocate > 0 then
                        self.nPowerToAllocate = self.nPowerToAllocate + 1
                        self.nPtsToAllocate = self.nPtsToAllocate - 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rPowerMinusButton then
                    if self.nPowerToAllocate > 0 then
                        self.nPowerToAllocate = self.nPowerToAllocate - 1
                        self.nPtsToAllocate = self.nPtsToAllocate + 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rHPPlusButton then
                    if self.nPtsToAllocate > 0 then
                        self.nHPToAllocate = self.nHPToAllocate + 10
                        self.nPtsToAllocate = self.nPtsToAllocate - 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rHPMinusButton then
                    if self.nHPToAllocate > 0 then
                        self.nHPToAllocate = self.nHPToAllocate - 10
                        self.nPtsToAllocate = self.nPtsToAllocate + 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rAwarenessPlusButton then
                    if self.nPtsToAllocate > 0 then
                        self.nAwarenessToAllocate = self.nAwarenessToAllocate + 1
                        self.nPtsToAllocate = self.nPtsToAllocate - 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rAwarenessMinusButton then
                    if self.nAwarenessToAllocate > 0 then
                        self.nAwarenessToAllocate = self.nAwarenessToAllocate - 1
                        self.nPtsToAllocate = self.nPtsToAllocate + 1
                        self:update()
                    end
                    return true
                elseif rProp == self.rOKButton then                    
                    self.rHero:incrementStatPower(self.nPowerToAllocate)
                    self.rHero:incrementMaxHP(self.nHPToAllocate)
                    self.rHero:incrementStatAwareness(self.nAwarenessToAllocate)
                    self.rHero:setPtsToAllocate(0)
                    self.rHero:setCurHP(self.rHero:getMaxHP())
                    g_GUI:HideLevelUpUI()
                    return true
                end
            end
		end
		return true
	end

	return Ob
end

return LevelUpUI