-------- HERO SCHEDULER ---------
local MMActivitySelectionUI = {}
local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local HeroChooserUI = require('ui/HeroChooserUI')
local Character = require('Character')
--local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')

local tUITextureSpriteSheetInfo =
{
icon_research = 'UI/hud',
icon_rest = 'UI/hud',
icon_timeoff = 'UI/hud',
icon_train = 'UI/hud',
icon_train_int = 'UI/hud',
icon_nothing = 'UI/hud',
icon_parttimejob = 'UI/hud',
icon_patrol = 'UI/hud',
icon_motivate = 'UI/hud',
icon_cut_meteorites = 'UI/hud',
activity_sheet_patrol = 'UI/hud',
activity_sheet_rest = 'UI/hud',
activity_sheet_timeoff = 'UI/hud',
activity_sheet_train = 'UI/hud',
activity_sheet_train_int = 'UI/hud',
activity_sheet_research = 'UI/hud',
activity_sheet_parttimejob = 'UI/hud',
activity_sheet_motivate = 'UI/hud2',
activity_sheet_promote = 'UI/hud2',
activity_sheet_train_ability = 'UI/hud',
activity_sheet_cut_meteorite = 'UI/hud',
}

local tFGColor = { .56, 0.24, 0.24 }

function MMActivitySelectionUI:new(rRenderLayer, rParentUI, rHero)	
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if not bVisible then
            self.rHeroSelectorUI:setVisible(false)
            self.rAbilityUpgradeUI:setVisible(false)
        end
    end

    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onHeroSelected(rHero)
        if self.rSelectedButton == self.rTrainIntButton then
            self.rHero:addMMTrainIntActivityAtIndex(1, rHero:getCharDefKey())
            g_GUI:HideHeroScheduler()
        elseif self.rSelectedButton == self.rIncreaseMoraleButton then
            self.rHero:addMMMoraleActivityAtIndex(1, rHero:getCharDefKey())
            g_GUI:HideHeroScheduler()
        elseif self.rSelectedButton == self.rPromoteButton then
            self.rHero:addMMPromoteActivityAtIndex(1, rHero:getCharDefKey())
            g_GUI:HideHeroScheduler()
        end
    end

    function Ob:update()
        if self.rHeroSelectorUI then
            self.rHeroSelectorUI:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rAbilityUpgradeUI:isVisible() then
            if eventType == MOAITouchSensor.TOUCH_UP then
                if not self.rAbilityUpgradeUI:onFinger(eventType, sx, sy, tProps) then
                    self.rAbilityUpgradeUI:setVisible(false)
                end
            end
            return true
        end
        if self.rHeroSelectorUI and self.rHeroSelectorUI:isMaximized() then
            self.rHeroSelectorUI:onFinger(eventType, sx, sy, tProps)
            return true
        end
        if eventType == MOAITouchSensor.TOUCH_UP then   
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rPTJobInvisButton then
                        self.rHero:setCurActivity('MMPTJobActivity')
                        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                        self.rParentUI:hideActivitySelector()
                        g_GUI:HideHeroScheduler()
                        self.rSelectedButton = self.rPTJobButton
                        return true
                    elseif rProp == self.rTrainIntInvisButton then
                        if self:areHeroesAvailableToTrain() then
                            g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                            self.rSelectedButton = self.rTrainIntButton
                            self:setHeroSelectorUIVisible(true)
                            return true
                        end
                    elseif rProp == self.rMoraleInvisButton then
                        if self:areHeroesAvailableToMotivate() then
                            g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                            self.rSelectedButton = self.rIncreaseMoraleButton
                            self:setHeroSelectorUIVisible(true)
                            return true     
                        end
                    elseif rProp == self.rTrainAbilityInvisButton then
                        --self.rHero:setCurActivity('MMTrainAbilityActivity')
                        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                        --self.rParentUI:hideActivitySelector()
                        self:setTrainAbilityUIVisible(true)
                        --g_GUI:HideHeroScheduler()
                        self.rSelectedButton = self.rPTJobButton
                        return true
                    elseif rProp == self.rPromoteInvisButton then
                        if self:areHeroesAvailableToPromote() then
                            g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                            self.rSelectedButton = self.rPromoteButton
                            self:setHeroSelectorUIVisible(true)
                            return true     
                        end
                    elseif rProp == self.rMeteoriteInvisButton then
                        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')      
                        g_GUI:ShowMeteoriteCombiner()
                        return true                       
                    end
                end
                if g_Game:isActionEnabled('CloseWindow') then
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                    self.rParentUI:hideActivitySelector()
                end
            end
        end
		return true
	end

    function Ob:areHeroesAvailableToPromote()
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero:isPreppedForLevelUp() and not rHero:isAssignedToEvent() then
                return true
            end
        end
        return false
    end

    function Ob:areHeroesAvailableToTrain()
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero:getNumTrainingSessions() > 0 then
                return true
            end
        end
        return false        
    end

    function Ob:areHeroesAvailableToMotivate()
        for i, rHero in ipairs(g_Game:getActiveHeroes()) do
            if rHero:getMorale() < Character.eMorale.HIGH then
                return true
            end
        end
        return false        
    end

    function Ob:setActivityIndex(nIndex)
        self.nActivityIndex = nIndex
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroSelectorUI:onPaneClosed()
        self.rAbilityUpgradeUI:onPaneClosed()
    end

    function Ob:setHeroSelectorUIVisible(bVisible)
        self.rHeroSelectorUI:setVisible(bVisible)
        if bVisible then
            self.rHeroSelectorUI:maximize(true)
        end        
    end

    function Ob:setTrainAbilityUIVisible(bVisible)
        self.rAbilityUpgradeUI:setVisible(bVisible)
    end

    function Ob:getSelectableHeroes()        
        local tHeroes = {}
        if self.rSelectedButton == self.rTrainIntButton then
            for i, rHero in ipairs(g_Game:getActiveHeroes()) do
                if rHero:getNumTrainingSessions() > 0 and not rHero:isAssignedToEvent() then
                    table.insert(tHeroes, rHero)
                end
            end
        elseif self.rSelectedButton == self.rIncreaseMoraleButton then
            for i, rHero in ipairs(g_Game:getActiveHeroes()) do
                if rHero:getMorale() < Character.eMorale.HIGH and not rHero:isAssignedToEvent() then
                    table.insert(tHeroes, rHero)
                end
            end
        elseif self.rSelectedButton == self.rPromoteButton then
            for i, rHero in ipairs(g_Game:getActiveHeroes()) do
                if rHero:isPreppedForLevelUp() and not rHero:isAssignedToEvent() then
                    table.insert(tHeroes, rHero)
                end
            end
        end

        return tHeroes
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	-- vars
	Ob.rHero = rHero
    Ob.rParentUI = rParentUI
	Ob.nActivityIndex = nil    
    Ob.bVisible = false

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(412, 560)
    Ob.rBackground:setColor(1, 0.78, 0.76)
    Ob:addElement(Ob.rBackground)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(24, -45)
    Ob.rLine1:setScl(365, 5)
    Ob.rLine1:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(24, -129)
    Ob.rLine2:setScl(365, 2)
    Ob.rLine2:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(24, -209)
    Ob.rLine3:setScl(365, 2)
    Ob.rLine3:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine3)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(24, -289)
    Ob.rLine4:setScl(365, 2)
    Ob.rLine4:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine4)

    Ob.rLine5 = MOAIProp2D.new()
    Ob.rLine5:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine5:setLoc(24, -369)
    Ob.rLine5:setScl(365, 2)
    Ob.rLine5:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine5)
    
    Ob.rLine6 = MOAIProp2D.new()
    Ob.rLine6:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine6:setLoc(24, -449)
    Ob.rLine6:setScl(365, 2)
    Ob.rLine6:setColor(.57, .25, .25)
    Ob:addElement(Ob.rLine6)

    Ob.LabelText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.LabelText:setString(g_DialogManager:getString("STXT032TEXT"))
    Ob.LabelText:setRect(0, 80, 380, 0)
    Ob.LabelText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.LabelText:setLoc(20, -14)
    Ob:addElement(Ob.LabelText)

    Ob.rPTJobButton = Ob:getUITextureProp('activity_sheet_parttimejob')
    Ob.rPTJobButton:setLoc(24, -53)
    Ob:addElement(Ob.rPTJobButton)

    Ob.rPTJobInvisButton = MOAIProp2D.new()
    Ob.rPTJobInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rPTJobInvisButton:setLoc(24, -53)
    Ob.rPTJobInvisButton:setScl(362, 70)
    Ob.rPTJobInvisButton:setColor(0, 0.5, 0.5)
    Ob.rPTJobInvisButton:setVisible(false)
    Ob:addElement(Ob.rPTJobInvisButton)
       
    Ob.PTJobText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.PTJobText:setString(string.upper(g_DialogManager:getString("ACNA002TEXT")))
    Ob.PTJobText:setRect(0, 40, 300, 0)
    Ob.PTJobText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.PTJobText:setLoc(160, -50)
    Ob:addElement(Ob.PTJobText)

    Ob.PTJobDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.PTJobDescriptionText:setString(g_DialogManager:getString("ACDS007TEXT"))
    Ob.PTJobDescriptionText:setRect(0, 40, 300, 0)
    Ob.PTJobDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.PTJobDescriptionText:setLoc(160, -84)
    Ob:addElement(Ob.PTJobDescriptionText)

    if g_Game:isActionEnabled('MMTrainInt') then
        Ob.rTrainIntInvisButton = MOAIProp2D.new()
        Ob.rTrainIntInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rTrainIntInvisButton:setLoc(24, -134)
        Ob.rTrainIntInvisButton:setScl(362, 70)
        Ob.rTrainIntInvisButton:setColor(0, 0.5, 0.5)
        Ob.rTrainIntInvisButton:setVisible(false)
        Ob:addElement(Ob.rTrainIntInvisButton)

        Ob.rTrainIntButton = Ob:getUITextureProp('activity_sheet_train_int')
        Ob.rTrainIntButton:setLoc(24, -134)
        Ob:addElement(Ob.rTrainIntButton)
        
        Ob.TrainIntText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.TrainIntText:setString(string.upper(g_DialogManager:getString("ACNA010TEXT")))
        Ob.TrainIntText:setRect(0, 40, 300, 0)
        Ob.TrainIntText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainIntText:setLoc(160, -130)
        Ob:addElement(Ob.TrainIntText)

        if not Ob:areHeroesAvailableToTrain() then
            Ob.rTrainIntButton:setColor(1, 1, 1, 0.5)            
            Ob.NoTrainText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.NoTrainText:setString(g_DialogManager:getString("ACRR010TEXT"))
            Ob.NoTrainText:setRect(0, 40, 300, 0)
            Ob.NoTrainText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.NoTrainText:setLoc(160, -164)
            Ob.TrainIntText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
            Ob:addElement(Ob.NoTrainText)
        else
            Ob.TrainIntDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.TrainIntDescriptionText:setString(g_DialogManager:getString("ACDS010TEXT")) --increases hero INT
            Ob.TrainIntDescriptionText:setRect(0, 40, 300, 0)
            Ob.TrainIntDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.TrainIntDescriptionText:setLoc(160, -158)
            Ob:addElement(Ob.TrainIntDescriptionText)
            
            Ob.TrainIntReqText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.TrainIntReqText:setString(g_DialogManager:getString("ACDS015TEXT")) -- (requires training session)
            Ob.TrainIntReqText:setRect(0, 40, 300, 0)
            Ob.TrainIntReqText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], .5)
            Ob.TrainIntReqText:setLoc(160, -180)
            Ob:addElement(Ob.TrainIntReqText)
        end
    end

    if g_Game:isActionEnabled('MMRaiseMorale') then
        Ob.rMoraleInvisButton = MOAIProp2D.new()
        Ob.rMoraleInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rMoraleInvisButton:setLoc(24, -214)
        Ob.rMoraleInvisButton:setScl(362, 70)
        Ob.rMoraleInvisButton:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.rMoraleInvisButton:setVisible(false)
        Ob:addElement(Ob.rMoraleInvisButton)

        Ob.rIncreaseMoraleButton = Ob:getUITextureProp('activity_sheet_motivate')
        Ob.rIncreaseMoraleButton:setLoc(24, -214)
        Ob:addElement(Ob.rIncreaseMoraleButton)
        
        Ob.IncreaseMoraleText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.IncreaseMoraleText:setString(string.upper(g_DialogManager:getString("ACNA011TEXT")))
        Ob.IncreaseMoraleText:setRect(0, 40, 300, 0)
        Ob.IncreaseMoraleText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.IncreaseMoraleText:setLoc(160, -210)
        Ob:addElement(Ob.IncreaseMoraleText)

        if not Ob:areHeroesAvailableToMotivate() then
            Ob.rIncreaseMoraleButton:setColor(1, 1, 1, 0.5)
            
            Ob.NoMotivateText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.NoMotivateText:setString(g_DialogManager:getString("ACRR009TEXT"))
            Ob.NoMotivateText:setRect(0, 40, 300, 0)
            Ob.NoMotivateText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.NoMotivateText:setLoc(160, -244)
            Ob.IncreaseMoraleText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
            Ob:addElement(Ob.NoMotivateText)
        else
            Ob.IncreaseMoraleDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.IncreaseMoraleDescriptionText:setString(g_DialogManager:getString("ACDS012TEXT"))
            Ob.IncreaseMoraleDescriptionText:setRect(0, 40, 300, 0)
            Ob.IncreaseMoraleDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.IncreaseMoraleDescriptionText:setLoc(160, -244)
            Ob:addElement(Ob.IncreaseMoraleDescriptionText)
        end
    end

    if g_Game:isActionEnabled('MMTrainAbility') then
        Ob.rTrainAbilityInvisButton = MOAIProp2D.new()
        Ob.rTrainAbilityInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rTrainAbilityInvisButton:setLoc(24, -374)
        Ob.rTrainAbilityInvisButton:setScl(362, 70)
        Ob.rTrainAbilityInvisButton:setColor(0, 0.5, 0.5)
        Ob.rTrainAbilityInvisButton:setVisible(false)
        Ob:addElement(Ob.rTrainAbilityInvisButton)
        
        Ob.rTrainAbilityButton = Ob:getUITextureProp('activity_sheet_train_ability')
        Ob.rTrainAbilityButton:setLoc(24, -374)
        
        Ob.TrainAbilityText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.TrainAbilityText:setString(string.upper(g_DialogManager:getString("ACNA009TEXT")))
        Ob.TrainAbilityText:setRect(0, 40, 300, 0)
        Ob.TrainAbilityText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainAbilityText:setLoc(160, -370)
        Ob:addElement(Ob.TrainAbilityText)
        
        Ob:addElement(Ob.rTrainAbilityButton)

        Ob.TrainAbilityDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainAbilityDescriptionText:setString(g_DialogManager:getString("ACDS009TEXT"))
        Ob.TrainAbilityDescriptionText:setRect(0, 40, 300, 0)
        Ob.TrainAbilityDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainAbilityDescriptionText:setLoc(160, -404)
        Ob.TrainAbilityText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob:addElement(Ob.TrainAbilityDescriptionText)
    end

    if g_Game:isActionEnabled('MMPromote') then
        Ob.rPromoteInvisButton = MOAIProp2D.new()
        Ob.rPromoteInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rPromoteInvisButton:setLoc(24, -294)
        Ob.rPromoteInvisButton:setScl(362, 70)
        Ob.rPromoteInvisButton:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.rPromoteInvisButton:setVisible(false)
        Ob:addElement(Ob.rPromoteInvisButton)
        
        Ob.rPromoteButton = Ob:getUITextureProp('activity_sheet_promote')
        Ob.rPromoteButton:setLoc(24, -294)
        
        Ob.PromoteText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.PromoteText:setString(string.upper(g_DialogManager:getString("ACNA012TEXT")))
        Ob.PromoteText:setRect(0, 40, 300, 0)
        Ob.PromoteText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.PromoteText:setLoc(160, -290)
        Ob:addElement(Ob.PromoteText)
        Ob:addElement(Ob.rPromoteButton)

        if not Ob:areHeroesAvailableToPromote() then
            Ob.rPromoteButton:setColor(1, 1, 1, 0.5)
            
            Ob.NoPromoteText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.NoPromoteText:setString(g_DialogManager:getString("ACRR008TEXT"))
            Ob.NoPromoteText:setRect(0, 40, 300, 0)
            Ob.NoPromoteText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.NoPromoteText:setLoc(160, -324)
            Ob.PromoteText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
            Ob:addElement(Ob.NoPromoteText)
        else
            Ob.PromoteDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.PromoteDescriptionText:setString(g_DialogManager:getString("ACDS013TEXT"))
            Ob.PromoteDescriptionText:setRect(0, 40, 300, 0)
            Ob.PromoteDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.PromoteDescriptionText:setLoc(160, -324)
            Ob:addElement(Ob.PromoteDescriptionText)
        end
    
    end

    if g_Game:isActionEnabled('CombineGems') then
        --CUT METEORITES
        Ob.rMeteoriteInvisButton = MOAIProp2D.new()
        Ob.rMeteoriteInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rMeteoriteInvisButton:setLoc(24, -454)
        Ob.rMeteoriteInvisButton:setScl(362, 70)
        Ob.rMeteoriteInvisButton:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.rMeteoriteInvisButton:setVisible(false)
        Ob:addElement(Ob.rMeteoriteInvisButton)
        
        Ob.rMeteoriteButton = Ob:getUITextureProp('activity_sheet_cut_meteorite')
        Ob.rMeteoriteButton:setLoc(24, -454)
        Ob:addElement(Ob.rMeteoriteButton)
        
        Ob.MeteoriteText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.MeteoriteText:setString(string.upper(g_DialogManager:getString("ACNA015TEXT")))
        Ob.MeteoriteText:setRect(0, 40, 300, 0)
        Ob.MeteoriteText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.MeteoriteText:setLoc(160, -450)
        Ob:addElement(Ob.MeteoriteText)    
        
        Ob.MeteoriteDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.MeteoriteDescriptionText:setString(g_DialogManager:getString("ACDS017TEXT"))
        Ob.MeteoriteDescriptionText:setRect(0, 40, 300, 0)
        Ob.MeteoriteDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.MeteoriteDescriptionText:setLoc(160, -484)
        Ob:addElement(Ob.MeteoriteDescriptionText)
    end

    Ob.rHeroSelectorUI = HeroChooserUI:new(Renderer.getRenderLayer('UIOverlay'), Ob)    

    --Ob.rAbilityUpgradeUI = AbilityUpgradeUI:new(rRenderLayer, rHero, Ob)
    --Ob.rAbilityUpgradeUI:setLoc(-360, -130)
    --Ob.rAbilityUpgradeUI:setParent(Ob)
    local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
    Ob.rAbilityUpgradeUI = AbilitySelectionUI:new(rOverlayLayer, Ob, rHero, true)
    --Ob.rAbilityUpgradeUI:setLoc(-360,40)
    Ob.rAbilityUpgradeUI:setLoc(-40,40)
    Ob.rAbilityUpgradeUI:setParent(Ob)

	return Ob
end

return MMActivitySelectionUI
