-------- MM SCHEDULER ---------
local util = require('DFUtil')
local Character = require('Character')
local Renderer = require('Renderer')
local MMActivitySelectionUI = require('ui/MMActivitySelectionUI')
local AbilitySmallButton = require('ui/AbilitySmallButton')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')
local UIProgressBar = require("ui/UIProgressBar")
local UIElement = require('ui/UIElement')
local CountdownUI = require('ui/CountdownUI')
local DFAnalytics = require('DFCommon.Analytics')
local Button = require('ui/Button')
local ShopButton = require('ui/ShopButton')

local MMSchedulerUI = {}
local MMSchedulerButton = {}

local tUITextureSpriteSheetInfo =
{
    mm_clipboard = 'UI/hud2',
    icon_research = 'UI/hud',
    icon_nothing = 'UI/hud',
    icon_parttimejob = 'UI/hud',
    icon_train_int = 'UI/hud',
    icon_motivate = 'UI/hud2',
    icon_train_ability = 'UI/hud',
    button_buy = 'UI/hud',
    icon_pp = 'UI/hud',
    icon_time = 'UI/hud',
    arrow = 'UI/hud3',
}

function MMSchedulerButton:new(rRenderLayer, rParentUI, rHero, nIndex)
	local Ob = UIElement:new(rRenderLayer)
	Ob.rActivity = rActivity
	Ob.rHero = rHero
    Ob.nIndex = nIndex
    Ob.sIconPath = nil
    Ob.rParentUI = rParentUI
    Ob.rButton = nil
    Ob:addElement(Ob.rButton)
    Ob.bVisible = false

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)

        self.bVisible = bVisible
        
        if self.rButton then
            self.rButton:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        -- do nothing, leave resident
    end

	function Ob:update()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            local sIconPath = rActivity:getIconPath()
            if self.sIconPath ~= sIconPath then  
                if self.rButton then
                    self.rButton:setVisible(false)
                end
                self.rButton = Button:new(rRenderLayer, self, tUITextureSpriteSheetInfo, sIconPath, nil, {1, 1, 1}, {0.75, 0.75, 0.75})
                self.rButton:setLoc(0, 0)
                self.rButton:setParent(self)
                
                self.rButton:setVisible(self.bVisible)
                self.sIconPath = sIconPath
            end
        else
            if self.rButton then
                self.rButton:setVisible(false)
            end
            
            self.rButton = nil
        end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if g_Game:isActionEnabled('ActivitySelectEnabled') then
                g_Audio:playCue('MMOJ_Audio/UI/AddActivity')
                self.rParentUI:showActivitySelector(self.nIndex)                       
            end
        end
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
		if self.rButton then
            return self.rButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
	end

    function Ob:setPriority(nPri)
        if self.rButton and self.rButton.rButton then
            self.rButton.rButton:setPriority(nPri)
        end
    end

	return Ob
end

function MMSchedulerUI:new(rRenderLayer, rHero)
	local Ob = UIElement:new(rRenderLayer)

	-- functions
	function Ob:update()
        if self.rRushButton then
            self.rRushButton:update()
        end
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:update()
            rButton:setPriority(self.rBackground:getPriority() - 1)
        end
        self:updateActivityTimeText()
        self.rStatLevelText:setString(tostring(g_MiddleManager:getStarRatingLevel()))
        self.rXPBar:update(g_MiddleManager:getStarRatingXP())
        if self.rActivitySelectionUI then
            self.rActivitySelectionUI:update()
        end
	end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/ClipBoard')
            self.rHero:onSchedulerOpened(true)
        else
            self.rHero:onSchedulerOpened(false)
        end
        self:setElementsVisible(bVisible)
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:setVisible(bVisible)
        end
        self.rActivitySelectionUI:setVisible(false)
        Ob.rScrim:setVisible(false) 
        if not bVisible then
            self:showRushButton(false)
            self:setAbilitySelectionUIVisible(false)
        end
        if self.rLeftArrow then
            self.rLeftArrow:setVisible(bVisible)
        end
        if self.rRightArrow then
            self.rRightArrow:setVisible(bVisible)
        end
        self.rXPBar:setVisible(bVisible)
        for i,v in ipairs(self.tAbilities) do
            v:setVisible(bVisible)
        end
        if bVisible then
            for i, rButton in ipairs(self.tScheduleButtons) do
                rButton:setPriority(self.rBackground:getPriority() - 1)
            end   
        end
	end

	function Ob:setHero(rHero)
		self.rHero = rHero
    end
    
    function Ob:isActivitySelectorVisible()
        return self.bActivitySelectorVisible
    end

    function Ob:showActivitySelector(nIndex)
        self.bActivitySelectorVisible = true
        self.rActivitySelectionUI:setActivityIndex(nIndex)
        self.rActivitySelectionUI:setVisible(true)
        self.rScrim:setVisible(true)
        local nPri = self.rActivitySelectionUI.rBackground:getPriority() or 0
        self.rScrim:setPriority(nPri-1)
    end

    function Ob:hideActivitySelector()
        self.bActivitySelectorVisible = false
        self.rActivitySelectionUI:setVisible(false)
        Ob.rScrim:setVisible(false)
    end

    function Ob:closeScheduler()
        g_GUI:SetActivePane(nil)
    end

    function Ob:updateActivityTimeText()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            if rActivity.getActivityProgress then
                self.rCountdownUI:updateProgress(rActivity:getActivityProgress())
            end
            if rActivity.getRemainingTime then
                self.rRemainingTimeText:revealAll()
                local nRemainingTime = rActivity:getRemainingTime()    
                local nRemainingMinutes = math.floor(nRemainingTime / 60)
                local nRemainingSeconds = math.floor(nRemainingTime % 60)
                local sRemainingMinText = tostring(nRemainingMinutes)
                local sRemainingSecText = tostring(nRemainingSeconds)
                local sTimeText = ""
                if nRemainingMinutes < 10 then
                    sRemainingMinText = "0"..sRemainingMinText
                end
                if nRemainingSeconds < 10 then
                    sRemainingSecText = "0"..sRemainingSecText
                end
                if nRemainingSeconds % 2 == 0 then
                    sTimeText = sRemainingMinText..":"..sRemainingSecText
                else
                    sTimeText = sRemainingMinText.." "..sRemainingSecText
                end
                
                self.rRemainingTimeText:setString(sTimeText.." Left")

                if nRemainingTime > 0 then
                    if rActivity.getRushPrice then
                        local nRushPrice = rActivity:getRushPrice()                       
                        if nRushPrice > 0 then
                            self:showRushButton(true)
                            self.rRushPriceText:setString(tostring(nRushPrice))
                        end
                    end
                end
            else
                self:showRushButton(false)
                self.rRemainingTimeText:setReveal(0)
            end
        end
    end

    function Ob:attemptRushActivity()
        local rActivity = self.rHero:getCurActivity()
        if rActivity then
            local nRemainingTime = rActivity:getRemainingTime()    
            local nRushPrice = rActivity:getRushPrice()
            if nRemainingTime > 0 and nRushPrice > 0 then
                if g_Game:getCurPP() >= nRushPrice then
                    g_Game:removePP(nRushPrice)
                    DFAnalytics.logEvent("ppp_spnt", { type = "rush", amount = nRushPrice })
                    DFAnalytics.incrStat('ppp_spnt_rush', nRushPrice)
                    rActivity:rushActivity()
                    return true
                end
            end
        end
        return false
    end

    function Ob:showRushButton(bVisible)
        if bVisible and not self.bVisible then
            return
        end
        self.rRushButton:setVisible(bVisible)
        self.rPPIcon:setVisible(bVisible)
        self.rCountdownUI:setVisible(bVisible)
        if bVisible then
            self.rRushPriceText:revealAll()
        else
            self.rRushPriceText:setReveal(0)
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rRushButton then
            if self:attemptRushActivity() then
                g_GUI:HideHeroScheduler()
            end
        elseif rButton == self.rLeftArrow then
            local rNextHero = g_Game:getPrevHero(self.rHero)
            if rNextHero then
                g_GUI:ShowHeroScheduler(rNextHero)
            end
        elseif rButton == self.rRightArrow then
            local rNextHero = g_Game:getNextHero(self.rHero)
            if rNextHero then
                g_GUI:ShowHeroScheduler(rNextHero)
            end
        end
    end

    function Ob:setAbilitySelectionUIVisible(bShow)
        self.rAbilitySelectionUI:setVisible(bShow)
        Ob.rScrim:setVisible(bShow)
        if bShow then
            local nPri = 1
            if self.rActivitySelectionUI.rBackground then
                nPri = self.rActivitySelectionUI.rBackground:getPriority() or nPri
            end
            Ob.rScrim:setPriority(nPri-1)
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rActivitySelectionUI and self.rActivitySelectionUI:isVisible() then
            if self.rActivitySelectionUI:onFinger(eventType, sx, sy, tProps) then
            else
                if g_Game:isActionEnabled('CloseWindow') then
                    self.rActivitySelectionUI:setVisible(false)
                end
            end
            return true
        end
        if eventType == MOAITouchSensor.TOUCH_UP and tProps then
            if self.rAbilitySelectionUI and self.rAbilitySelectionUI:isVisible() then
                if self.rAbilitySelectionUI:onFinger(eventType, sx, sy, tProps) then
                else
                    if g_Game:isActionEnabled('CloseWindow') then
                        self:setAbilitySelectionUIVisible(false)
                    end
                end
                return true
            end

            for i,v in ipairs(self.tAbilities) do
                if v:onFinger(eventType, sx, sy, tProps) then
                    if g_Game:isActionEnabled('OpenAbilityPane') then
                        self:setAbilitySelectionUIVisible(not self.rAbilitySelectionUI:isVisible())
                    end
                end
            end

            self.rRushButton:onFinger(eventType, sx, sy, tProps)

            local bTouchedBG = false
            for i, rProp in ipairs(tProps) do
                if rProp == self.rBackground then
                    bTouchedBG = true
                end
            end
            if not bTouchedBG then
                if g_Game:isActionEnabled('CloseWindow') then
                    g_GUI:HideHeroScheduler()
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                end                    
            end
        end
        if self.rLeftArrow then
            self.rLeftArrow:onFinger(eventType, sx, sy, tProps)
        end
        if self.rRightArrow then
            self.rRightArrow:onFinger(eventType, sx, sy, tProps)
        end
        for i, rButton in ipairs(self.tScheduleButtons) do
            rButton:onFinger(eventType, sx, sy, tProps)
        end
		return true
	end
--[[
    if rHero.tCharDef.tDoodles then
        for i, rInfo in ipairs(rHero.tCharDef.tDoodles) do
            tUITextureSpriteSheetInfo[rInfo.sIndex] = rInfo.sSpriteSheetPath
        end
    end
]]--
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
	-- the background
	Ob.rBackground = Ob:getUITextureProp('mm_clipboard')
	Ob.rBackground:setLoc(0, 0)
	Ob:addElement(Ob.rBackground)
	Ob.rHero = rHero

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setLoc(72, -64)
        Ob.rPortrait:setScl(0.64)
        Ob:addElement(Ob.rPortrait)
    end

    Ob.tScheduleButtons = {}
    local nButtonXOffset = 43
    local nButtonYOffset = -354
    local nButtonXSize = 132
    for i = 1, 1 do
        local rNewButton = MMSchedulerButton:new(Ob:getRenderLayer(), Ob, Ob.rHero, i)
        self.rMMSchedulerButton = rNewButton
        if rNewButton then
            rNewButton:setParent(Ob)
            rNewButton:setLoc(nButtonXOffset + (i - 1) * nButtonXSize, nButtonYOffset)
            Ob.tScheduleButtons[i] = rNewButton
        end
    end

	-- INITIALIZATION
	-- the name text
	Ob.rNameText = g_GUI:CreateTextBox(28, MOAITextBox.CENTER_JUSTIFY)
	Ob.rNameText:setString(g_DialogManager:getString("NAME015TEXT"))
	Ob.rNameText:setLoc(320, -84)
    --Ob:addOverlayElement(Ob.rNameText)
    Ob:addElement(Ob.rNameText)

    Ob.rRemainingTimeText = g_GUI:CreateTextBox(20, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rRemainingTimeText:setString("0:90 left")
    Ob.rRemainingTimeText:setRect(-150, 40, 150, 0)
    Ob.rRemainingTimeText:setColor(.62, .62, .56)
    Ob.rRemainingTimeText:setLoc(106, -510)
    Ob:addElement(Ob.rRemainingTimeText)

    Ob.rRushButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT116TEXT", 'ptsans', 24, { 34, -4 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, nil, 1, { 0, 0, 0, 0.75 })
    Ob.rRushButton:setScl(0.9, 1)
    Ob.rRushButton:setLoc(38, -540)
    Ob.rRushButton:setParent(Ob)

    Ob.rPPIcon = Ob:getUITextureProp('icon_pp')
    Ob.rPPIcon:setLoc(100, -544)
    Ob.rPPIcon:setScl(0.76)
    Ob:addOverlayElement(Ob.rPPIcon)

    Ob.rRushPriceText = g_GUI:CreateTextBox(26, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRushPriceText:setString("1")
    Ob.rRushPriceText:setRect(0, 60, 150, 0)
    Ob.rRushPriceText:setColor(1, 1, 1)
    Ob.rRushPriceText:setLoc(150, -544)
    Ob:addOverlayElement(Ob.rRushPriceText)   

    Ob.rActivitySelectionUI = MMActivitySelectionUI:new(rRenderLayer, Ob, rHero)
    Ob.rActivitySelectionUI:setLoc(550, -40)

    Ob.rCurActivityLabelText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'ptsans')
    Ob.rCurActivityLabelText:setString(g_DialogManager:getString("STXT047TEXT"))
    Ob.rCurActivityLabelText:setRect(-100, 80, 100, 0)
    Ob.rCurActivityLabelText:setColor(.62, .62, .56)
    Ob.rCurActivityLabelText:setLoc(110, -280)
    Ob:addElement(Ob.rCurActivityLabelText)   

--[[
    if rHero.tCharDef.tDoodles then
        for i, rInfo in ipairs(rHero.tCharDef.tDoodles) do
            local rDoodle = Ob:getUITextureProp(rInfo.sIndex)
            if rDoodle then
                rDoodle:setLoc(rInfo.loc[1], rInfo.loc[2])
                Ob:addElement(rDoodle)
            end
        end
    end
]]--
    Ob.rCountdownUI = CountdownUI:new(rRenderLayer)
    Ob.rCountdownUI:setLoc(126, -346)    
    Ob.rCountdownUI:setParent(Ob)

    if g_Game:isActionEnabled('CycleHeroScheduler') then
        Ob.rLeftArrow = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'arrow', nil, {1, 1, 1}, {0.5, 0.5, 0.5})
        Ob.rLeftArrow:setLoc(64, -64)
        Ob.rLeftArrow:setScl(-1, 1)
        Ob.rLeftArrow:setParent(Ob)

        Ob.rRightArrow = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'arrow', nil, {1, 1, 1}, {0.5, 0.5, 0.5})
        Ob.rRightArrow:setLoc(492, -64)
        Ob.rRightArrow:setParent(Ob)   
    end

    Ob:showRushButton(false)

	-- the level text
    Ob.rLevelLabelText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rLevelLabelText:setString(g_DialogManager:getString("STXT003TEXT"))
    Ob.rLevelLabelText:setLoc(132, -158)
    Ob.rLevelLabelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rLevelLabelText) 

	Ob.rStatLevelText = g_GUI:CreateTextBox(46, MOAITextBox.CENTER_JUSTIFY, 'type')
	Ob.rStatLevelText:setString("99")
	Ob.rStatLevelText:setLoc(130, -184)
    Ob.rStatLevelText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rStatLevelText)

    Ob.rXPText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'ptsans')    
    Ob.rXPText:setString(g_DialogManager:getString("STXT012TEXT"))
    Ob.rXPText:setLoc(332, -158)
    Ob.rXPText:setColor(.62, .62, .56)
    Ob:addElement(Ob.rXPText) 

    Ob.rXPBar = UIProgressBar:new(rRenderLayer, g_MiddleManager:getStarRatingXPForNextLevel(), { r = 1, g = 1, b = 1 }, { r = 0.62, g = 0.62, b = 0.56 }, false)
    Ob.rXPBar:setLoc(204, -192)
    Ob.rXPBar:setScl(232, 34)
    Ob.rXPBar:setParent(Ob)

    Ob.tAbilities = {}
    local tAbilities = rHero:getAbilities()
    local i = 0
    for k,v in pairs(tAbilities) do
        local btn = AbilitySmallButton:new(rRenderLayer, Ob, rHero, v)
        btn:setScl(1.3)
        local kBUTTON_SIZE = 80
        local y = -262 - i * kBUTTON_SIZE
        btn:setLoc(202,y)
        table.insert(Ob.tAbilities, btn)
        i=i+1
    end

    Ob.rScrim = MOAIProp2D.new()
	Ob.rScrim:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rScrim:setLoc(0, 0)
	Ob.rScrim:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rScrim:setColor(.4, .4, .4, .5)
	Ob.rScrim:setLoc(0, 0)
    Ob:addElement(Ob.rScrim)
    Ob.rScrim:setVisible(false)
    
    Ob.rAbilitySelectionUI = AbilitySelectionUI:new(Ob.rOverlayRenderLayer, Ob, rHero)
    Ob.rAbilitySelectionUI:setLoc(520, 0)
    Ob.rAbilitySelectionUI:setParent(Ob)

	return Ob
end

return MMSchedulerUI
