local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local MainMenuUI = {}

local tUITextureSpriteSheetInfo =
{
    popup_button = 'UI/hud',
    popup_activated = 'UI/hud',
}

function MainMenuUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	-- functions
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
	end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rPopupClosed then
                        self:toggleOpen()
                        return true
                    elseif rProp == self.rShopButton then
                        if self.bOpen then
                            g_GUI:ShowShopUI()
                            self:open(false)
                            return true
                        end
                    elseif rProp == self.rHelpButton then
                        if self.bOpen then
                            g_GUI:ShowMMMenu()
                            self:open(false)                        
                            return true
                        end
                    elseif rProp == self.rRosterButton then
                        if self.bOpen then
                            g_GUI:ShowRosterMenu()
                            self:open(false)
                            return true
                        end
                    end
                end
            end
        end
        return false
    end

    function Ob:open(bOpen)
        if not bOpen then
            if not g_Game:isActionEnabled('CloseWindow') then
                return
            end
        end

        self.bOpen = bOpen
        if bOpen then
            self.rPopupOpened:setVisible(true)
            self.rShopButtonText:revealAll()
            self.rHelpButtonText:revealAll()
            self.rCreditsButtonText:revealAll()
            self.rRosterButtonText:revealAll()
            g_Game:getTutorialManager():onWindowOpen('MainMenu')
        else
            self.rPopupOpened:setVisible(false)
            self.rShopButtonText:setReveal(0)
            self.rHelpButtonText:setReveal(0)
            self.rCreditsButtonText:setReveal(0)
            self.rRosterButtonText:setReveal(0)
        end
    end

    function Ob:toggleOpen()
        self:open(not self.bOpen)
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rPopupClosed = Ob:getUITextureProp('popup_button')
    Ob.rPopupClosed:setLoc(0, -180)
    Ob:addOverlayElement(Ob.rPopupClosed)

    Ob.rPopupOpened = Ob:getUITextureProp('popup_activated')
    Ob.rPopupOpened:setLoc(0, 50)
    Ob:addOverlayElement(Ob.rPopupOpened)

    Ob.rShopButtonText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'bangers')
    Ob.rShopButtonText:setString("SHOP")
    Ob.rShopButtonText:setRect(0, 80, 300, 0)
    Ob.rShopButtonText:setColor(1, 1, 1)
	Ob.rShopButtonText:setLoc(52, 30)
    Ob.rShopButtonText:setReveal(0)
	Ob:addOverlayElement(Ob.rShopButtonText)

	Ob.rShopButton = MOAIProp2D.new()
	Ob.rShopButton:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rShopButton:setScl(230, 40)
	Ob.rShopButton:setLoc(10, 30)
    Ob.rShopButton:setColor(1, 1, 1)
    Ob.rShopButton:setVisible(false)
	Ob:addOverlayElement(Ob.rShopButton)

    Ob.rRosterButtonText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'bangers')
    Ob.rRosterButtonText:setString("ROSTER")
    Ob.rRosterButtonText:setRect(0, 80, 300, 0)
    Ob.rRosterButtonText:setColor(1, 1, 1)
	Ob.rRosterButtonText:setLoc(52, -24)
    Ob.rRosterButtonText:setReveal(0)
	Ob:addOverlayElement(Ob.rRosterButtonText)

	Ob.rRosterButton = MOAIProp2D.new()
	Ob.rRosterButton:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rRosterButton:setScl(230, 40)
	Ob.rRosterButton:setLoc(10, -24)
    Ob.rRosterButton:setColor(1, 1, 1)
    Ob.rRosterButton:setVisible(false)
	Ob:addOverlayElement(Ob.rRosterButton)

    Ob.rHelpButtonText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'bangers')
    Ob.rHelpButtonText:setString("HELP")
    Ob.rHelpButtonText:setRect(0, 80, 300, 0)
    Ob.rHelpButtonText:setColor(1, 1, 1)
	Ob.rHelpButtonText:setLoc(52, -80)
    Ob.rHelpButtonText:setReveal(0)
	Ob:addOverlayElement(Ob.rHelpButtonText)

	Ob.rHelpButton = MOAIProp2D.new()
	Ob.rHelpButton:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rHelpButton:setScl(230, 40)
	Ob.rHelpButton:setLoc(10, -78)
    Ob.rHelpButton:setColor(1, 1, 1)
    Ob.rHelpButton:setVisible(false)
	Ob:addOverlayElement(Ob.rHelpButton)

    Ob.rCreditsButtonText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'bangers')
    Ob.rCreditsButtonText:setString("CREDITS")
    Ob.rCreditsButtonText:setRect(0, 80, 300, 0)
    Ob.rCreditsButtonText:setColor(1, 1, 1)
	Ob.rCreditsButtonText:setLoc(52, -132)
    Ob.rCreditsButtonText:setReveal(0)
	Ob:addOverlayElement(Ob.rCreditsButtonText)

	Ob.rCreditsButton = MOAIProp2D.new()
	Ob.rCreditsButton:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rCreditsButton:setScl(230, 40)
	Ob.rCreditsButton:setLoc(10, -132)
    Ob.rCreditsButton:setColor(1, 1, 1)
    Ob.rCreditsButton:setVisible(false)
	Ob:addOverlayElement(Ob.rCreditsButton)

    return Ob
end

return MainMenuUI