local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local Item = require('Item')
local ListUI = require('ui/ListUI')
local MeteoriteIcon = require('ui/MeteoriteIcon')
local Button = require('ui/Button')
local DFAnalytics = require('DFCommon.Analytics')

local MeteoriteInvIcon = {}
local MeteoriteCombinerUI = {}
local MeteoriteListUI = {}
local MeteoriteDetailButton = {}
local MeteoriteRecipeUI = {}
local MeteoriteRecipeButton = {}
local MeteoriteRecipeIcon = {}

local tUITextureSpriteSheetInfo =
{
    back_button = 'UI/hud',
    back_button_tap = 'UI/hud',
    shop_item = 'UI/shop',
    numberbg = 'UI/shop',
    circle_fill = 'UI/hud',
    icon_crystal_bg = 'UI/icons3',
    button_buy = 'UI/shop',
    lock_research = 'UI/shop',
}

local tNoCombineColor = { 0.75, 0.75, 0.75 }

function MeteoriteInvIcon:new(rRenderLayer, sItemKey, nLevel)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self.rMeteoriteIcon:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rMeteoriteIcon:onPaneClosed()
    end

    function Ob:update()
        local nNumItems = g_MiddleManager:getNumMeteoritesInInventory(self.sItemKey, self.nLevel)
        if nNumItems > 0 then            
            self.rNumberBG:setVisible(true)
            self.rNumText:revealAll()
            self.rNumText:setString(tostring(nNumItems))
            self.rMeteoriteIcon:clearColorOverride()
            self.rMeteoriteIcon:setAlphaOverride(1)
        else
            self.rNumberBG:setVisible(false)
            self.rNumText:setReveal(0)
            self.rMeteoriteIcon:setColorOverride(.1,.1,.1)
            self.rMeteoriteIcon:setAlphaOverride(0.4)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.sItemKey = sItemKey
    Ob.nLevel = nLevel

    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer, sItemKey, nLevel)
    Ob.rMeteoriteIcon:setParent(Ob)
    Ob.rMeteoriteIcon:setLoc(0, 0)

    Ob.rNumberBG = Ob:getUITextureProp('numberbg')
    Ob.rNumberBG:setScl(1.2)
    Ob.rNumberBG:setLoc(70, -74)
    Ob:addElement(Ob.rNumberBG)

    Ob.rNumText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rNumText:setString("1")
    Ob.rNumText:setRect(-100, 80, 100, 0)
    Ob.rNumText:setColor(1, 1, 1)
    Ob.rNumText:setLoc(89, -72)
    Ob:addElement(Ob.rNumText)

    return Ob
end

function MeteoriteDetailButton:new(rRenderLayer, sDefKey, rMeteoriteListUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rIcon in ipairs(self.tMeteoriteIcons) do
            rIcon:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        for i, rIcon in ipairs(self.tMeteoriteIcons) do
            rIcon:onPaneClosed()
        end
    end

	function Ob:getYSize()
        return 165
	end

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:update()
        for i, rIcon in ipairs(self.tMeteoriteIcons) do
            rIcon:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps, bTouchMoved)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP and not bTouchMoved then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackGround then
                        if self.rMeteoriteListUI then
                            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                            self.rMeteoriteListUI:onMeteoriteButtonSelected(self)
                        end
                    end
                end
                return true
            end
        end
    end

    function Ob:onSelected(bSelected)
        if bSelected then
            self.rBackGround:setColor(1, 1, 1)
        else
            self.rBackGround:setColor(0.75, 0.75, 0.75)
        end
    end

    function Ob:getDefKey()
        return self.sDefKey
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rParentUI = nil
    Ob.rMeteoriteListUI = rMeteoriteListUI

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
    Ob.rBackGround:setScl(1, 0.75)
	Ob:addElement(Ob.rBackGround)

    Ob.rDef = Item.getMeteoriteDef(sDefKey)
    Ob.sDefKey = sDefKey

    if Ob.rDef then
        Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.rNameField:setString(g_DialogManager:getString(Ob.rDef.sName))
        Ob.rNameField:setRect(0, 80, 300, 0)
        Ob.rNameField:setLoc(15, -10)
        Ob.rNameField:setColor(0, 0, 0)
        Ob:addElement(Ob.rNameField)      

        Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
        Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rDef.sDescription))
        Ob.rDescriptionField:setRect(0, 200, 615, 0)
        Ob.rDescriptionField:setLoc(15, -50)
        Ob.rDescriptionField:setColor(0, 0, 0)
        Ob:addElement(Ob.rDescriptionField)

        Ob.tMeteoriteIcons = {}
        local kYPos = -56
        local kInitX = -35
        for i = 1, Item.kMETEORITE_MAX_LEVEL do
            local rIcon = MeteoriteInvIcon:new(rRenderLayer, sDefKey, i)
            rIcon:setParent(Ob)
            if i == Item.kMETEORITE_MAX_LEVEL then -- hax!
                rIcon:setLoc(kInitX + (i - 1) * 78, kYPos)
            else
                rIcon:setLoc(kInitX + (i - 1) * 74, kYPos)
            end
            table.insert(Ob.tMeteoriteIcons, rIcon)
        end
    end

    return Ob
end

function MeteoriteListUI:new(rRenderLayer, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rListUI:onPaneClosed()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:update()
        self.rListUI:update()
    end

    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    function Ob:onMeteoriteButtonSelected(rButton)
        if self.rParentUI and rButton then            
            self.rParentUI:selectMeteorite(rButton:getDefKey())
        end 
    end

    function Ob:selectMeteorite(sDefKey)
        for i, rExistingButton in ipairs(self.tMeteoriteButtons) do
            if rExistingButton:getDefKey() == sDefKey then
                rExistingButton:onSelected(true)
            else
                rExistingButton:onSelected(false)
            end
        end
    end

    Ob.tMeteoriteButtons = {}
    Ob.rParentUI = rParentUI
    local tDefKeys = Item.getAllMeteoriteDefKeys()
    for i, sDefKey in ipairs(tDefKeys) do
        local rDef = Item.getMeteoriteDef(sDefKey)
        if rDef then
            if rDef.bResearched or g_MiddleManager:isResearched('meteorite', sDefKey) then
                local rNewButton = MeteoriteDetailButton:new(Ob.rRenderLayer, sDefKey, Ob)
                table.insert(Ob.tMeteoriteButtons, rNewButton)
            end
        end
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tMeteoriteButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

function MeteoriteRecipeIcon:new(rRenderLayer, sDefKey, nLevel, nNumRequired)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rMeteoriteIcon:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rMeteoriteIcon:onPaneClosed()
    end

    function Ob:isValid()
        return self.bIsValid
    end

    function Ob:setValid(bSet)
        if bSet == self.bIsValid then
            return
        end
        if bSet then
--            self.rMeteoriteIcon:clearColorOverride()
            self.rBGCircle:setColor(0, 0, 0)
            self.rFGCircle:setColor(.65, 0.94, .59)
        else
--            self.rMeteoriteIcon:setColorOverride(0.5, 0.5, 0.5)
            self.rBGCircle:setColor(0.5, 0.5, 0.5)
            self.rFGCircle:setColor(0.9, 0.9, 0.9)
        end
        self.bIsValid = bSet
    end

    function Ob:update()
        local nNumInInventory = g_MiddleManager:getNumMeteoritesInInventory(self.sDefKey, self.nLevel)
        if nNumInInventory >= self.nNumRequired then
            self:setValid(true)
        else
            self:setValid(false)            
        end
    end

    function Ob:getMeteoriteInfo()
        return self.sDefKey, self.nLevel
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.sDefKey = sDefKey
    Ob.nLevel = nLevel
    Ob.nNumRequired = nNumRequired or 1

    Ob.rBGCircle = Ob:getUITextureProp('circle_fill')
    Ob.rBGCircle:setLoc(0, 0)
    Ob.rBGCircle:setScl(1.2)
    Ob:addElement(Ob.rBGCircle)

    Ob.rFGCircle = Ob:getUITextureProp('circle_fill')
    Ob.rFGCircle:setLoc(4, -4)
    Ob.rFGCircle:setScl(1.08)
    Ob:addElement(Ob.rFGCircle)
    
    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer, sDefKey, nLevel)
    Ob.rMeteoriteIcon:setParent(Ob)
    Ob.rMeteoriteIcon:setLoc(-22, 18)

    return Ob
end

function MeteoriteRecipeButton:new(rRenderLayer, sDefKey, nLevel, bUseRecipe)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rIcon in ipairs(self.tIcons) do
            rIcon:setVisible(bVisible)
        end
        self.rIcon:setVisible(bVisible)
        self.rCombineButton:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:getYSize()
        return 220
	end

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rCombineButton then
            if self.bCombine then -- updated on the update call
                if self.tIcons[1] and self.tIcons[2] and self.tIcons[3] then
                    if self.tIcons[1]:isValid() and self.tIcons[2]:isValid() and self.tIcons[3]:isValid() then
                        local sKey1, nLevel1 = self.tIcons[1]:getMeteoriteInfo()
                        local sKey2, nLevel2 = self.tIcons[2]:getMeteoriteInfo()
                        local sKey3, nLevel3 = self.tIcons[3]:getMeteoriteInfo()
                        local sNewDefKey, sNewLevel = g_MiddleManager:combineMeteorite(sKey1, nLevel1, sKey2, nLevel2, sKey3, nLevel3) -- maybe should be flashier?
                        g_Audio:playCue('MMOJ_Audio/UI/CombineMeteorite')
                        -- analytics
                        if sNewDefKey and sNewLevel then
                            local sAnalyticsKey = sNewDefKey..sNewLevel                
                            DFAnalytics.logEvent("cmb_mete", { type = sAnalyticsKey } )
                        end
                    end
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rCombineButton:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:update()
        for i, rIcon in ipairs(self.tIcons) do
            rIcon:update()            
        end
        self.bCombine = true
        if self.tIcons[1] then
            if self.tIcons[1]:isValid() then
                self.rLine1:setColor(0, 0, 0)
            else
                self.bCombine = false
                self.rLine1:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            end
        end
        if self.tIcons[2] then
            if self.tIcons[2]:isValid() then
                self.rLine2:setColor(0, 0, 0)
            else
                self.bCombine = false
                self.rLine2:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            end
        end
        if self.tIcons[3] then
            if self.tIcons[3]:isValid() then
            else
                self.bCombine = false
            end
        end
        if self.bCombine then
            self.rLine2:setColor(0, 0, 0)
            self.rLine3:setColor(0, 0, 0)
            self.rBoxLLine:setColor(0, 0, 0)
            self.rBoxTopLine:setColor(0, 0, 0)
            self.rBoxRLine:setColor(0, 0, 0)
            self.rBoxBottomLine:setColor(0, 0, 0)
            self.rCombineButton:clearButtonColorOverride()
        else
            self.rLine2:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rLine3:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rBoxLLine:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rBoxTopLine:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rBoxRLine:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rBoxBottomLine:setColor(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
            self.rCombineButton:setButtonColorOverride(tNoCombineColor[1], tNoCombineColor[2], tNoCombineColor[3])
        end
    end

    function Ob:getStatString()
        local sString = ""
        if self.rDef.sPlusMinus then
            sString = sString .. g_DialogManager:getString(self.rDef.sPlusMinus)
        end
        if self.rDef.tInfo and self.nLevel then
            local rInfo = self.rDef.tInfo[self.nLevel]
            if rInfo.tAttackMods or rInfo.tFightBuffs then -- only report the first one for now
                local rBuffInfo = nil
                if rInfo.tAttackMods and rInfo.tAttackMods.tBuffs then
                    rBuffInfo = rInfo.tAttackMods.tBuffs[1]
                elseif rInfo.tFightBuffs and rInfo.tFightBuffs.tBuffs then
                    rBuffInfo = rInfo.tFightBuffs.tBuffs[1]
                end
                if rBuffInfo and rBuffInfo.nAmount then
                    local nAmount = math.abs(rBuffInfo.nAmount)
                    if rBuffInfo.sBuffType == 'CoinBuff' or rBuffInfo.sBuffType == 'DodgeBuff' then
                        nAmount = nAmount * 100
                        sString = sString .. nAmount
                    else                    
                        sString = sString .. nAmount .. " "
                    end
                end
            elseif rInfo.nXPMultiplier then
                sString = sString .. (rInfo.nXPMultiplier * 100)
            elseif rInfo.nManaCostMultiplier then
                sString = sString .. (rInfo.nManaCostMultiplier * 100)
            elseif rInfo.nSplashDamageMultiplier then
                sString = sString .. " " .. (rInfo.nSplashDamageMultiplier * 100)
            end
        end
        if self.rDef.sEffectText then
            sString = sString .. g_DialogManager:getString(Ob.rDef.sEffectText)
        end
        return sString
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rParentUI = nil

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
    Ob.rBackGround:setScl(1, 1)
	Ob:addElement(Ob.rBackGround)

    Ob.rDef = Item.getMeteoriteDef(sDefKey)
    Ob.nLevel = nLevel

    if Ob.rDef then
        local sName = ""
        if nLevel and Item.tMeteoriteNames[nLevel] then
            sName = g_DialogManager:getString(Item.tMeteoriteNames[nLevel])
        end
        
        Ob.rIconBG = Ob:getUITextureProp('icon_crystal_bg')
        Ob.rIconBG:setScl(0.75)
        Ob.rIconBG:setLoc(20, -20)
        Ob:addElement(Ob.rIconBG)

        Ob.rIcon = MeteoriteIcon:new(rRenderLayer, sDefKey, nLevel)
        Ob.rIcon:setLoc(2, -6)
        Ob.rIcon:setParent(Ob)

        Ob.rNameText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.rNameText:setString(g_DialogManager:getString(Ob.rDef.sName).." "..sName)
        Ob.rNameText:setRect(0, 200, 315, 0)
        Ob.rNameText:setLoc(130, -10)
        Ob.rNameText:setColor(0, 0, 0)
        Ob:addElement(Ob.rNameText)
        
        Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
        Ob.rDescriptionField:setString(Ob:getStatString())
        Ob.rDescriptionField:setRect(0, 200, 315, 0)
        Ob.rDescriptionField:setLoc(130, -50)
        Ob.rDescriptionField:setColor(0, 0, 0)
        Ob:addElement(Ob.rDescriptionField)

        Ob.tIcons = {}
        local kYPos = -120
        local kInitX = 20
        if bUseRecipe and Ob.rDef.tRecipe and Ob.rDef.tRecipe[1] and Ob.rDef.tRecipe[2] and Ob.rDef.tRecipe[3] then
            local tNumAdded = {}
            for i, sMeteoriteKey in ipairs(Ob.rDef.tRecipe) do 
                if not tNumAdded[sMeteoriteKey] then
                    tNumAdded[sMeteoriteKey] = 0
                end
            end            
            for i = 1, 3 do
                tNumAdded[Ob.rDef.tRecipe[i]] = tNumAdded[Ob.rDef.tRecipe[i]] + 1
                local rIcon = MeteoriteRecipeIcon:new(rRenderLayer, Ob.rDef.tRecipe[i], nLevel, tNumAdded[Ob.rDef.tRecipe[i]])
                rIcon:setParent(Ob)
                rIcon:setLoc(kInitX + (i - 1) * 100, kYPos)
                table.insert(Ob.tIcons, rIcon)                
            end
        else
            for i = 1, 3 do
                local rIcon = MeteoriteRecipeIcon:new(rRenderLayer, sDefKey, nLevel - 1, i)
                rIcon:setParent(Ob)
                rIcon:setLoc(kInitX + (i - 1) * 100, kYPos)
                table.insert(Ob.tIcons, rIcon)
            end
        end

        Ob.rLine1 = MOAIProp2D.new()
        Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rLine1:setLoc(100, -162)
        Ob.rLine1:setScl(30, 4)
        Ob.rLine1:setColor(0, 0, 0)
        Ob:addElement(Ob.rLine1)

        Ob.rLine2 = MOAIProp2D.new()
        Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rLine2:setLoc(200, -162)
        Ob.rLine2:setScl(30, 4)
        Ob.rLine2:setColor(0, 0, 0)
        Ob:addElement(Ob.rLine2)

        Ob.rLine3 = MOAIProp2D.new()
        Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rLine3:setLoc(300, -162)
        Ob.rLine3:setScl(30, 4)
        Ob.rLine3:setColor(0, 0, 0)
        Ob:addElement(Ob.rLine3)

        Ob.rBoxLLine = MOAIProp2D.new()
        Ob.rBoxLLine:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rBoxLLine:setLoc(330, -134)
        Ob.rBoxLLine:setScl(4, 60)
        Ob.rBoxLLine:setColor(0, 0, 0)
        Ob:addElement(Ob.rBoxLLine)

        Ob.rBoxTopLine = MOAIProp2D.new()
        Ob.rBoxTopLine:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rBoxTopLine:setLoc(330, -132)
        Ob.rBoxTopLine:setScl(124, 4)
        Ob.rBoxTopLine:setColor(0, 0, 0)
        Ob:addElement(Ob.rBoxTopLine)

        Ob.rBoxRLine = MOAIProp2D.new()
        Ob.rBoxRLine:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rBoxRLine:setLoc(450, -134)
        Ob.rBoxRLine:setScl(4, 60)
        Ob.rBoxRLine:setColor(0, 0, 0)
        Ob:addElement(Ob.rBoxRLine)

        Ob.rBoxBottomLine = MOAIProp2D.new()
        Ob.rBoxBottomLine:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rBoxBottomLine:setLoc(330, -192)
        Ob.rBoxBottomLine:setScl(124, 4)
        Ob.rBoxBottomLine:setColor(0, 0, 0)
        Ob:addElement(Ob.rBoxBottomLine)

        Ob.rCombineButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { 1, 0, 0 }, nil, true, { 0, -50 }, "STXT207TEXT", 'ptsans', 24, { 54, -6 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
        Ob.rCombineButton:setLoc(336, -139)
        Ob.rCombineButton:setButtonScl(0.7, 1)
        Ob.rCombineButton:setParent(Ob)
    end

    return Ob
end

function MeteoriteRecipeUI:new(rRenderLayer, sDefKey, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rListUI:onPaneClosed()
    end

    function Ob:update()
        self.rListUI:update()
    end

    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    Ob.sDefkey = sDefKey
    Ob.rParentUI = rParentUI
    
    Ob.tRecipeButtons = {}
    Ob.rDef = Item.getMeteoriteDef(sDefKey)
    if Ob.rDef then
        for i = 1, Item.kMETEORITE_MAX_LEVEL do
            if i > 1 then
                local rButton = MeteoriteRecipeButton:new(Ob.rRenderLayer, sDefKey, i)
                table.insert(Ob.tRecipeButtons, rButton)
            end
            if Ob.rDef.tRecipe then
                local rButton = MeteoriteRecipeButton:new(Ob.rRenderLayer, sDefKey, i, true)
                table.insert(Ob.tRecipeButtons, rButton)
            end
        end
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tRecipeButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)
    
    return Ob
end

function MeteoriteCombinerUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rMeteoriteList:setVisible(bVisible)
        if not bVisible then
            for i, rList in pairs(self.tRecipeLists) do
                rList:setVisible(false)
            end
            if self.rCurRecipeList then
                self.rCurRecipeList:setVisible(false)
            end
        else
            local tDefKeys = Item.getAllMeteoriteDefKeys()
            
            if table.getn(tDefKeys) > 0 then
                self:selectMeteorite(tDefKeys[1])
            end
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rMeteoriteList:unloadUISpriteSheets()
    end

    function Ob:onBackButtonPressed(bPressed)
        self.rBackButton:setVisible(not bPressed)
        self.rBackPressedButton:setVisible(bPressed)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_DOWN then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackButton then
                        self:onBackButtonPressed(true)
                        return true
                    end
                end
            elseif eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackPressedButton then
                        self:onBackButtonPressed(false)
                        if g_Game:isActionEnabled('CloseWindow') then
                            g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                            g_GUI:SetActivePane(nil)
                            return true
                        end
                    end
                end
            end
        end
        local tLayer = Renderer.getLayer('UI')
        local x, y = tLayer.RenderLayer:wndToWorld(sx, sy)
        if x > Renderer.getUIHalfWidth() then        
            if self.rCurRecipeList then
                self.rCurRecipeList:onFinger(eventType, sx, sy, tProps)
            end
        else
            self.rMeteoriteList:onFinger(eventType, sx, sy, tProps)
        end

        return true
    end

    function Ob:update()
        self.rMeteoriteList:update()
        if self.rCurRecipeList then
            self.rCurRecipeList:update()
        end
    end

    function Ob:selectMeteorite(sMeteoriteDefKey)
        if sMeteoriteDefKey then
            if self.rCurRecipeList then
                self.rCurRecipeList:setVisible(false)
                self.rCurRecipeList:onPaneClosed()
                self.rCurRecipeList = nil
            end
            self.rCurRecipeList = MeteoriteRecipeUI:new(self.rRenderLayer, sMeteoriteDefKey, Ob)
            self.rCurRecipeList:setParent(Ob)
            self.rCurRecipeList:setLoc(Renderer.getUIHalfWidth(), -90)
            self.rCurRecipeList:setVisible(true)

            self.rMeteoriteList:selectMeteorite(sMeteoriteDefKey)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    -- add the bottom BG
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())                 
	Ob.rBackground:setColor(.47, .58, .36)
    Ob:addElement(Ob.rBackground)

    Ob.rLeftBG = MOAIProp2D.new()
	Ob.rLeftBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rLeftBG:setLoc(0, 0)
    Ob.rLeftBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())                 
	Ob.rLeftBG:setColor(.9, .85, .95)
    Ob:addElement(Ob.rLeftBG)    

    Ob.rRightBG = MOAIProp2D.new()
	Ob.rRightBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rRightBG:setLoc(Renderer.getUIHalfWidth(), -92)
    Ob.rRightBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())
	Ob.rRightBG:setColor(.46, .35, .57)
    Ob:addElement(Ob.rRightBG)

    Ob.rTopBar = MOAIProp2D.new()
	Ob.rTopBar:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBar:setLoc(0, 0)
    Ob.rTopBar:setScl(Renderer.getUIWidth(), 92)                 
	Ob.rTopBar:setColor(.64, .5, .79)
    Ob:addOverlayElement(Ob.rTopBar)

	Ob.rTitleText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
	Ob.rTitleText:setString(g_DialogManager:getString("STXT206TEXT"))
    Ob.rTitleText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTitleText:setColor(.89, .88, .91)
	Ob.rTitleText:setLoc(0, -14)
    Ob:addOverlayElement(Ob.rTitleText)

    -- add the back button
    Ob.rBackButton = Ob:getUITextureProp('back_button')
    Ob.rBackButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackButton)

    Ob.rBackPressedButton = Ob:getUITextureProp('back_button_tap')
    Ob.rBackPressedButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackPressedButton)  

    Ob.rMeteoriteList = MeteoriteListUI:new(Ob.rRenderLayer, Ob)
    Ob.rMeteoriteList:setParent(Ob)
    Ob.rMeteoriteList:setLoc(0, -90)

    Ob.tRecipeLists = {}
    Ob.rCurRecipeList = nil

    Ob:onBackButtonPressed(false)  

    return Ob
end

return MeteoriteCombinerUI