local Item = require('Item')
local UIElement = require('ui/UIElement')
local Button = require('ui/Button')
local MeteoriteIcon = require('ui/MeteoriteIcon')
local MeteoriteDetailButton = {}

local tUITextureSpriteSheetInfo =
{
shop_item = 'UI/shop',
button_buy = 'UI/shop',
icon_crystal_bg = 'UI/icons3',
shop_itembar = 'UI/shop',
}

function MeteoriteDetailButton:new(rRenderLayer, sItemKey, nLevel)
	if not sItemKey then
		print("ERROR: MeteoriteDetailButton created without an Item")
		return nil
	end

    local rDef = Item.getMeteoriteDef(sItemKey)
    if not rDef then
		print("ERROR: MeteoriteDetailButton created without an Item")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if self.rParentUI then
                self.rParentUI:handleButtonPress(self, eventType)
            end
            self:onButtonPressed()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.onFingerCallBack then
            self:onFingerCallBack(eventType, sx, sy, tProps)
        end
        if self.rButton then
            self.rButton:onFinger(eventType, sx, sy, tProps)
        end

		return true
	end

	function Ob:setVisible(bVisible)
        if Ob.bIsVisible ~= bVisible then
            Ob.bIsVisible = bVisible

            self:setElementsVisible(bVisible)            
            self.rMeteoriteIcon:setVisible(bVisible)

            if self.rButton then
                self.rButton:setVisible(bVisible)
            end
        end
	end

	function Ob:getYSize()
        return 220
	end

    function Ob:getButton()
        return self.rButton
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rButton then
            self.rButton:onPaneClosed()
        end
    end

    function Ob:getStatString()
        local sString = ""
        if self.rItemDef.sPlusMinus then
            sString = sString .. g_DialogManager:getString(self.rItemDef.sPlusMinus)
        end
        if self.rItemDef.tInfo and self.nLevel then
            local rInfo = self.rItemDef.tInfo[self.nLevel]
            if rInfo.tAttackMods or rInfo.tFightBuffs then -- only report the first one for now
                local rBuffInfo = nil
                if rInfo.tAttackMods and rInfo.tAttackMods.tBuffs then
                    rBuffInfo = rInfo.tAttackMods.tBuffs[1]
                elseif rInfo.tFightBuffs and rInfo.tFightBuffs.tBuffs then
                    rBuffInfo = rInfo.tFightBuffs.tBuffs[1]
                end
                if rBuffInfo and rBuffInfo.nAmount then
                    local nAmount = math.abs(rBuffInfo.nAmount)
                    if rBuffInfo.sBuffType == 'CoinBuff' or rBuffInfo.sBuffType == 'DodgeBuff' then
                        nAmount = nAmount * 100
                    end
                    sString = sString .. nAmount .. " "
                end
            elseif rInfo.nXPMultiplier then
                sString = sString .. (rInfo.nXPMultiplier * 100)
            elseif rInfo.nManaCostMultiplier then
                sString = sString .. (rInfo.nManaCostMultiplier * 100)
            elseif rInfo.nSplashDamageMultiplier then
                sString = sString .. (rInfo.nSplashDamageMultiplier * 100)
            end
        end
        if self.rItemDef.sEffectText then
            sString = sString .. g_DialogManager:getString(Ob.rItemDef.sEffectText)
        end
        return sString
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sItemKey = sItemKey
	Ob.rItemDef = rDef
    Ob.nLevel = nLevel
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

    Ob.rItemBar = Ob:getUITextureProp('shop_itembar')
    Ob.rItemBar:setLoc(15, -152)
    Ob:addElement(Ob.rItemBar)

	-- add the textfields
    local sNameText = g_DialogManager:getString(Ob.rItemDef.sName)
    sNameText = sNameText.." "..g_DialogManager:getString(Item.tMeteoriteNames[nLevel])

	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(sNameText)
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(155, -10)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

	Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rItemDef.sDescription))
    Ob.rDescriptionField:setRect(0, 200, 315, 0)
	Ob.rDescriptionField:setLoc(155, -50)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	
    
    Ob.rStatField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rStatField:setString(Ob:getStatString())
    Ob.rStatField:setRect(0, 200, 320, 0)
	Ob.rStatField:setLoc(155, -125)
	Ob.rStatField:setColor(0, .5, 0)
	Ob:addElement(Ob.rStatField)	

    Ob.rMeteoriteIconBG = Ob:getUITextureProp('icon_crystal_bg')
    Ob.rMeteoriteIconBG:setLoc(18, -18)
    Ob:addElement(Ob.rMeteoriteIconBG)

    Ob.rMeteoriteIcon = MeteoriteIcon:new(rRenderLayer, sItemKey, nLevel)
    Ob.rMeteoriteIcon:setParent(Ob)
    Ob.rMeteoriteIcon:setLoc(16, -12)

    Ob.rButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT100TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rButton:setLoc(306, -152)
    Ob.rButton:setParent(Ob)

    return Ob
end

return MeteoriteDetailButton