local Item = require('Item')
local UIElement = require('ui/UIElement')
local MeteoriteDetailButton = require('ui/MeteoriteDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local Button = require('ui/Button')
local MeteoriteInventoryButton = {}

local tUITextureSpriteSheetInfo = 
{
numberbg = 'UI/shop',
button_buy = 'UI/shop',
}

function MeteoriteInventoryButton:new(rRenderLayer, sItemKey, nLevel, rInventoryPane)
	if not sItemKey then
		print("ERROR: MeteoriteInventoryButton created without an Item")
		return nil
	end
    local rDef = Item.getMeteoriteDef(sItemKey)
    if not rDef then
		print("ERROR: MeteoriteInventoryButton created without an Item")
		return nil
    end

	local Ob = MeteoriteDetailButton:new(rRenderLayer, sItemKey, nLevel)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rButton then
            self.rButton:setVisible(bVisible)
        end
        if self.rSellButton then
            self.rSellButton:setVisible(bVisible)
        end
        self.rMeteoriteIcon:setVisible(bVisible)
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            self:equipMeteorite()
        elseif rButton == self.rSellButton then
            if self.rItemDef and self.nLevel then
                local nSellPrice = self.rItemDef.tSellPrice[self.nLevel] or 10
                local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT098TEXT")..' '..Item.getMeteoriteName(self.sItemKey, self.nLevel)..' '..g_DialogManager:getString("STXT099TEXT").." "..nSellPrice.."?", self, "STXT075TEXT", "STXT076TEXT")
                rConfirmPane:setLoc(240, -160)
                g_GUI:SetPopupPane(rConfirmPane)
            end
        end
    end

    function Ob:onFingerCallBack(eventType, sx, sy, tProps)
        if self.rSellButton then
            self.rSellButton:onFinger(eventType, sx, sy, tProps)
        end
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:sellItem()
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:sellItem()
        if self.rItemDef and self.nLevel then
            g_Audio:playCue('MMOJ_Audio/UI/BuyButton')            
            g_MiddleManager:sellMeteorite(self.sItemKey, self.nLevel)
            self:update()
        end       
    end    

    function Ob:equipMeteorite()
        if self.sItemKey and self.nLevel then
            if self.rInventoryPane then
                local rHeroSelectorUI = self.rInventoryPane:getHeroSelectorUI()
                if rHeroSelectorUI then
                    local rHero = rHeroSelectorUI:getSelectedHero()
                    if rHero then
                        g_Audio:playCue('MMOJ_Audio/UI/Equip_A')
                        g_MiddleManager:giveHeroMeteorite(rHero, self.sItemKey, self.nLevel)
                        self:update()
                    end
                end
            end
        end
    end

    function Ob:update()
        local nNumItems = g_MiddleManager:getNumMeteoritesInInventory(self.sItemKey, self.nLevel)
        
        if nNumItems > 0 then
            self:setVisible(true)
            self.bHideFromList = false
            
            if self.rNumText then
                if nNumItems > 1 then                  
                    self.rNumberBG:setVisible(true)
                    self.rNumText:revealAll()                   
                else
                    self.rNumberBG:setVisible(true)
                    self.rNumText:revealAll()
                end
                self.rNumText:setString(tostring(nNumItems))                    
            end
        else
            self:setVisible(false)
            self.bHideFromList = true
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sItemKey = sItemKey
    Ob.nLevel = nLevel
    Ob.rInventoryPane = rInventoryPane
    Ob.rItemDef = rDef

    Ob.rNumberBG = Ob:getUITextureProp('numberbg')
    Ob.rNumberBG:setScl(1.2)
    Ob.rNumberBG:setLoc(100, -120)
    Ob:addElement(Ob.rNumberBG)
    
    Ob.rNumText = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rNumText:setString("1")
    Ob.rNumText:setRect(-100, 80, 100, 0)
    Ob.rNumText:setColor(1, 1, 1)
    Ob.rNumText:setLoc(120, -117)
    Ob:addElement(Ob.rNumText)

    Ob.rSellButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { 1, 0, 0 }, nil, true, { 0, -50 }, "STXT101TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rSellButton:setLoc(16, -152)
    Ob.rSellButton:setParent(Ob)
    
    return Ob
end

return MeteoriteInventoryButton