local Item = require('Item')
local UIElement = require('ui/UIElement')
local MeteoriteDetailButton = require('ui/MeteoriteDetailButton')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local DFAnalytics = require('DFCommon.Analytics')

local MeteoritePurchaseButton = {}

local tUITextureSpriteSheetInfo =
{
icon_pp = 'UI/hud',    
}

function MeteoritePurchaseButton:new(rRenderLayer, sItemKey, nLevel)
	if not sItemKey or not nLevel then
		print("ERROR: MeteoritePurchaseButton created without an Item")
		return nil
	end
    local rDef = Item.getMeteoriteDef(sItemKey)
    if not rDef then
		print("ERROR: MeteoritePurchaseButton created without an Item")
		return nil
    end

	local Ob = MeteoriteDetailButton:new(rRenderLayer, sItemKey, nLevel)

    function Ob:onButtonPressed()
        if self:canBuyItem() then
            local sName = g_DialogManager:getString(self.rItemDef.sName)
            if self.nLevel then
                if Item.tMeteoriteNames[self.nLevel] then
                    sName = sName..' '..g_DialogManager:getString(Item.tMeteoriteNames[self.nLevel])
                end
            end
            local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT138TEXT")..' '..sName.."?", self, "STXT075TEXT", "STXT076TEXT")
            rConfirmPane:setLoc(240, -160)
            g_GUI:SetPopupPane(rConfirmPane)
        end        
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        self:buyItem()
        -- should switch to the inventory pane
        local rPane = g_GUI:GetActivePane()
        if rPane and rPane.selectTab then
            rPane:selectTab('Meteorite')
        end
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:canBuyItem()
        if g_Game:getCurPP() < self:getPurchaseCost() then
            return false
        end
        return true
    end

    function Ob:getPurchaseCost()
        if self.rItemDef and self.rItemDef.tPurchasePrice and self.nLevel then
            if self.rItemDef.tPurchasePrice[self.nLevel] then
                return self.rItemDef.tPurchasePrice[self.nLevel]
            end
        end
        return 1
    end

    function Ob:buyItem()
        if self:canBuyItem() then
            g_Game:removePP(self:getPurchaseCost())
            g_MiddleManager:addMeteoriteToInventory(self.sItemKey, self.nLevel)            
            -- analytics
            if self.sItemKey and self.nLevel then
                local sAnalyticsKey = self.sItemKey..self.nLevel
                DFAnalytics.logEvent("buy_mete", { type = sAnalyticsKey } )
                DFAnalytics.incrStat("ppp_spnt_mete", self:getPurchaseCost())
            end
            g_Game:saveGame()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    local rButton = Ob:getButton()
    if rButton then
        rButton:setText(string.upper(g_DialogManager:getString("STXT004TEXT")), true)
    end    

    Ob.rCostIcon = Ob:getUITextureProp('icon_pp')
    Ob.rCostIcon:setLoc(20, -154)
    Ob:addElement(Ob.rCostIcon)    

    local tColor = g_Game:getPPColor()   
    Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rCostField:setString(tostring(Ob:getPurchaseCost()))
    Ob.rCostField:setColor(tColor[1], tColor[2], tColor[3])
    Ob.rCostField:setLoc(390, -156)    
    Ob:addElement(Ob.rCostField)

    return Ob
end

return MeteoritePurchaseButton