local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local Character = require('Character')

local MostWantedUI = {}
local MostWantedPortrait = {}

local tUITextureSpriteSheetInfo =
{
    back_button = 'UI/hud',
    back_button_tap = 'UI/hud',
}

function MostWantedPortrait:new(rRenderLayer, sCharDefKey, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onSelected(bSelected)
        self.rBackground:setVisible(bSelected)
        self.rBackgroundShadow:setVisible(bSelected)
        if bSelected and self.rParentUI and self.rDef and self.rDef.sDescription then
            self.rParentUI:setDescriptionText(self.rDef.sDescription, self.rDef.sName)
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rPortrait then
                        if self.rParentUI then
                            self.rParentUI:selectButton(self)
                            return true
                        end
                    end
                end
            end
        end
        return true
    end

    Ob.rParentUI = rParentUI
    
    -- selection box drop shadow
    Ob.rBackgroundShadow = MOAIProp2D.new()
	Ob.rBackgroundShadow:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackgroundShadow:setLoc(-23, 9)
    Ob.rBackgroundShadow:setScl(184, 180)
	Ob.rBackgroundShadow:setColor(.5, .49, .42)
    Ob.rBackgroundShadow:setVisible(false)
    Ob:addElement(Ob.rBackgroundShadow)
    
    -- selection box for the portrait
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(-28, 14)
    Ob.rBackground:setScl(184, 180)
	Ob.rBackground:setColor(1, 1, 1)
    Ob.rBackground:setVisible(false)
    Ob:addElement(Ob.rBackground)    

    Ob.rDef = Character.getCharDef(sCharDefKey)
    if Ob.rDef then
        if Ob.rDef.tPortraitPaths and Ob.rDef.tPortraitIndex then
            local sIndex = Ob.rDef.tPortraitIndex['default']
            if Ob.rDef.tPortraitPaths['default'] and sIndex then
                local tUITextureSpriteSheetInfo = { stamp_captured = 'UI/hud' }
                tUITextureSpriteSheetInfo[sIndex] = Ob.rDef.tPortraitPaths['default']
                Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
                Ob.rPortrait = Ob:getUITextureProp(sIndex)
                Ob:addElement(Ob.rPortrait)
            end
        end
        if Ob.rDef.sName then
            Ob.rNameText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')    
            Ob.rNameText:setString(string.upper(g_DialogManager:getString(Ob.rDef.sName)))
            Ob.rNameText:setLoc(64,-132)
            Ob:addElement(Ob.rNameText)       
        end
    end

    local nAngle = math.random(-20, 20)
    Ob.rStampButton = Ob:getUITextureProp('stamp_captured')
    if nAngle > 0 then
        Ob.rStampButton:setLoc(0, -30)
    else
        Ob.rStampButton:setLoc(0, 0)
    end
    Ob.rStampButton:setScl(0.8)
    Ob.rStampButton:setRot(nAngle)
    Ob:addElement(Ob.rStampButton)    

    if g_Game:getSaveVar('Beat'..sCharDefKey) then
        Ob.rStampButton:setVisible(true)
    else
        Ob.rStampButton:setVisible(false)
    end

    return Ob
end

function MostWantedUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rPortrait in ipairs(self.tPortraits) do
            rPortrait:setVisible(bVisible)
        end
        if bVisible then
            if self.tPortraits[1] then
                self:selectButton(self.tPortraits[1])
            end
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onBackButtonPressed(bPressed)
        self.rBackButton:setVisible(not bPressed)
        self.rBackPressedButton:setVisible(bPressed)
    end

    function Ob:selectButton(rButton)
        if rButton then
            for i, rExistingButton in ipairs(self.tPortraits) do
                if rButton == rExistingButton then
                    rExistingButton:onSelected(true)
                else
                    rExistingButton:onSelected(false)
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_DOWN then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackButton then
                        self:onBackButtonPressed(true)
                        return true
                    end
                end
            elseif eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackPressedButton then
                        self:onBackButtonPressed(false)
                        if g_Game:isActionEnabled('CloseWindow') then
                            g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                            g_GUI:SetActivePane(nil)
                            return true
                        end
                    end
                end
            end
        end
        for i, rButton in ipairs(self.tPortraits) do
            rButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:setDescriptionText(sText, sTitleText)
        if sText then
            self.rDescriptionText:setString(g_DialogManager:getString(sText))
            self.rRapSheetTitleText:setString(g_DialogManager:getString(sTitleText) .. " " .. g_DialogManager:getString("STXT212TEXT"))            
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    -- tan BG
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())                 
	Ob.rBackground:setColor(.83, .81, .70)
    Ob:addElement(Ob.rBackground)
    
    --gray header bar
    Ob.rTopBar = MOAIProp2D.new()
	Ob.rTopBar:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBar:setLoc(0, 0)
    Ob.rTopBar:setScl(Renderer.getUIWidth(), 92)                 
	Ob.rTopBar:setColor(.37, .37, .37)
    Ob:addOverlayElement(Ob.rTopBar)
    
    -- Most Wanted Thugs header
	Ob.rTitleText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
	Ob.rTitleText:setString(g_DialogManager:getString("STXT211TEXT"))
    Ob.rTitleText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTitleText:setColor(.73, .73, .73)
	Ob.rTitleText:setLoc(0, -14)
    Ob:addOverlayElement(Ob.rTitleText)

    -- add the back button
    Ob.rBackButton = Ob:getUITextureProp('back_button')
    Ob.rBackButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackButton)

    Ob.rBackPressedButton = Ob:getUITextureProp('back_button_tap')
    Ob.rBackPressedButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackPressedButton)      

    Ob.rPortraitTransform = MOAITransform.new()
    Ob.rPortraitTransform:setParent(Ob)
    Ob.rPortraitTransform:setLoc(50, -120)

    Ob.tPortraits = {}
    local kMAX_NUM_ROW = 5
    local kX_SIZE = 180
    local kY_SIZE = 180
    local nNumAdded = 0
    local tMostWantedThugs = Character.getMostWantedThugs()
    if tMostWantedThugs then
        for i, sCharDefKey in ipairs(tMostWantedThugs) do
            local rPortrait = MostWantedPortrait:new(rRenderLayer, sCharDefKey, Ob)
            if rPortrait then
                rPortrait:setParent(Ob.rPortraitTransform)
                local nWhole = math.modf(nNumAdded / kMAX_NUM_ROW)
                local nRemainder = nNumAdded % kMAX_NUM_ROW
                rPortrait:setLoc(kX_SIZE * nRemainder, -kY_SIZE * nWhole)
                nNumAdded = nNumAdded + 1
                table.insert(Ob.tPortraits, rPortrait)
            end
        end
    end
    
    -- black rap sheet BG
    Ob.rBottomBG = MOAIProp2D.new()
	Ob.rBottomBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBottomBG:setLoc(0, -Renderer.getUIHeight() + 160)
	Ob.rBottomBG:setScl(Renderer.getUIWidth(), 160)
	Ob.rBottomBG:setColor(0, 0, 0)
    Ob:addOverlayElement(Ob.rBottomBG)
    
    -- rap sheet title
    Ob.rRapSheetTitleText = g_GUI:CreateTextBox(32, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
	Ob.rRapSheetTitleText:setString(g_DialogManager:getString("STXT212TEXT"))
    Ob.rRapSheetTitleText:setRect(-Renderer.getUIHalfWidth() + 30, 200, Renderer.getUIHalfWidth() - 30, 0)
	Ob.rRapSheetTitleText:setColor(1, 1, 1)
	Ob.rRapSheetTitleText:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHeight() + 155)
    Ob:addOverlayElement(Ob.rRapSheetTitleText)
    
    -- rap sheet blurbs
    Ob.rDescriptionText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY)
    Ob.rDescriptionText:setString("")    
    Ob.rDescriptionText:setRect(-Renderer.getUIHalfWidth() + 30, 200, Renderer.getUIHalfWidth() - 30, 0)
    Ob.rDescriptionText:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHeight() + 110)
    Ob.rDescriptionText:setColor(1, 1, 1)
    Ob:addOverlayElement(Ob.rDescriptionText)    

    Ob:onBackButtonPressed(false)

    return Ob
end

return MostWantedUI