local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local util = require('DFUtil')
local Newspaper = {}

local tUITextureSpriteSheetInfo =
{
    Newspaper = 'UI/reward2',
    newhero_bkgd = 'UI/reward',
}

function Newspaper:new(rRenderLayer, rEvent)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:showPaperThead()
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Stinger', false)    
        g_Audio:playCue('MMOJ_Audio/SFX/Office/Newspaper')
        self.rNewspaper:moveRot(720, 1.5)
        self.rNewspaper:moveScl(0.85, 0.85, 1.5)
        util.sleep(1.5)
        self.bRunningThread = false
        -- show the tweet button
    end

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            self.bRunningThread = true
            self.rThread = MOAIThread.new()
            self.rThread:run(self.showPaperThead, self)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rWinnerPortrait:unloadSpriteSheet()
        self.rLoserPortrait:unloadSpriteSheet()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.bRunningThread then
            return
        end
        if eventType == MOAITouchSensor.TOUCH_UP then
            g_GUI:ShowEventEndScreenUI(self.rEvent)
        end
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    Ob.rEvent = rEvent
    local sWinnerPortraitPath, sWinnerPortraitIndex, sLoserPortraitPath, sLoserPortraitIndex, sHeadline, sMainCaption, sSideCaption, nHeadlineFontSize, nHeadlineVerticalOffset = rEvent:getNewspaperInfo()

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(1, 1, 1)
	Ob:addElement(Ob.rBackground)

    local imageSizeX = 512
    local imageSizeY = 512
    -- we double the background up
    local backgroundScaleX = Renderer.getUIWidth() / imageSizeX
    local backgroundScaleY = Renderer.getUIHeight() / imageSizeY

    Ob.rNewspaperBGLeft = Ob:getUITextureProp('newhero_bkgd')
    Ob.rNewspaperBGLeft:setLoc(0, -Renderer.getUIHeight() / 2)
    Ob.rNewspaperBGLeft:setScl(backgroundScaleX * 1.02, backgroundScaleY)
    Ob:addElement(Ob.rNewspaperBGLeft)

    Ob.rNewspaperBGRight = Ob:getUITextureProp('newhero_bkgd')
    Ob.rNewspaperBGRight:setLoc(Renderer.getUIWidth(), -Renderer.getUIHeight() / 2)
    Ob.rNewspaperBGRight:setScl(-backgroundScaleX, backgroundScaleY)
    Ob:addElement(Ob.rNewspaperBGRight)
      
    Ob.rNewspaper = Ob:getUITextureProp('Newspaper')
    Ob.rNewspaper:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rNewspaper:setScl(0.1, 0.1)
    Ob:addElement(Ob.rNewspaper)

    self.sHeadline = sHeadline

 	--Ob.rHeadlineField = g_GUI:CreateTextBox(144, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	--Ob.rHeadlineField:setString(g_DialogManager:getString(sHeadline))
    --Ob.rHeadlineField:setRect(-340, 160, 340, 0)
	--Ob.rHeadlineField:setLoc(0, 180)
	--Ob.rHeadlineField:setColor(0, 0, 0)
    --Ob:addElement(Ob.rHeadlineField)
    --Ob.rHeadlineField:setParent(Ob.rNewspaper)

 	Ob.rMainCaptionField = g_GUI:CreateTextBox( nHeadlineFontSize, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	Ob.rMainCaptionField:setString(g_DialogManager:getString(sMainCaption))
    Ob.rMainCaptionField:setRect(-395, 160, 395, 0)
	Ob.rMainCaptionField:setLoc(-10, 210 + nHeadlineVerticalOffset)
	Ob.rMainCaptionField:setColor(0, 0, 0)
    Ob:addElement(Ob.rMainCaptionField)
    Ob.rMainCaptionField:setParent(Ob.rNewspaper)

 	Ob.rSideCaptionField = g_GUI:CreateTextBox(22, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	Ob.rSideCaptionField:setString(g_DialogManager:getString(sSideCaption))
    Ob.rSideCaptionField:setRect(-75, 260, 75, 0)
	Ob.rSideCaptionField:setLoc(-318, 75)
	Ob.rSideCaptionField:setColor(0, 0, 0)
    Ob:addElement(Ob.rSideCaptionField)
    Ob.rSideCaptionField:setParent(Ob.rNewspaper)
    Ob.rWinnerPortrait = SpriteSheet:new(sWinnerPortraitPath, nil, 0, 0, 'left', 'top')
    Ob.rWinnerPortrait:setSpriteIndex(sWinnerPortraitIndex)
    Ob.rWinnerPortrait:setLoc(-228, 70)
    Ob.rWinnerPortrait:setScl(1.7)

    Ob:addElement(Ob.rWinnerPortrait)
    Ob.rWinnerPortrait:setParent(Ob.rNewspaper)
    Ob.rLoserPortrait = SpriteSheet:new(sLoserPortraitPath, nil, 0, 0, 'left', 'top')
    Ob.rLoserPortrait:setSpriteIndex(sLoserPortraitIndex)
    Ob.rLoserPortrait:setLoc(-4, 70)
    Ob.rLoserPortrait:setScl(1.7)
    Ob:addElement(Ob.rLoserPortrait)
    Ob.rLoserPortrait:setParent(Ob.rNewspaper)
    return Ob
end

return Newspaper