local Character = require('Character')
local UIElement = require('ui/UIElement')
local DFAchievement = require('DFCommon.Achievement')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local Renderer = require('Renderer')
local ShopButton = require('ui/ShopButton')
local Button = require('ui/Button')
local DFAnalytics = require('DFCommon.Analytics')

local PackPurchaseButton = {}

local tUITextureSpriteSheetInfo =
{
button_buy = 'UI/shop',
shop_pack_cheap = 'UI/shopheropack',
shop_pack_expensive = 'UI/shopheropack',
shop_pack_average = 'UI/shopheropack',
shop_pack_bar = 'UI/shopheropack',
icon_pp = 'UI/hud',
shop_locked = 'UI/hud',
}

function PackPurchaseButton:new(rRenderLayer, sPackDefKey)
	if not sPackDefKey then
		print("ERROR: PackPurchaseButton created without a Pack")
		return nil
	end

    local rDef = Character.getPackDef(sPackDefKey)
    if not rDef then
		print("ERROR: PackPurchaseButton created without a Pack")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onYesPressed()
        g_GUI:SetPopupPane(nil)
        DFAnalytics.logEvent("buy_hrpk", { type = self.sPackDefKey } )
        g_GUI:ShowHeroCardSelectorUI(self.rPackDef)        
    end

    function Ob:onNoPressed()
        g_GUI:SetPopupPane(nil)
    end

    function Ob:update()
        if self.rButton.update then
            self.rButton:update()
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if self.rParentUI then
                self.rParentUI:handleButtonPress(self, eventType)
            end
            self:onButtonPressed()
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rButton:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)  
        self.rButton:onFinger(eventType, sx, sy, tProps)  
    end

	function Ob:getYSize()
        return 276
	end

    function Ob:onButtonPressed()
        if g_Game:getNumAllHeroes() >= g_Game:getTotalNumHeroes() then
            return -- at max
        end
        if self:canAffordPack() then
            local rConfirmPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT074TEXT")..' '..g_DialogManager:getString(Ob.rPackDef.sName).."?", self, "STXT075TEXT", "STXT076TEXT")
            rConfirmPane:setLoc(240, -160)
            g_GUI:SetPopupPane(rConfirmPane)
        end
    end

    function Ob:canAffordPack()
        if g_Game:getCurPP() >= self.rPackDef.nPPPrice then
            return true
        end
        return false
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sPackDefKey = sPackDefKey
	Ob.rPackDef = rDef
    Ob.rParentUI = nil

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setColor(1, 1, 1)
    Ob.rBackground:setLoc(-6, -6)
    Ob.rBackground:setScl(466, 266)
    Ob:addElement(Ob.rBackground)

    Ob.rPackBar = Ob:getUITextureProp('shop_pack_bar')
    Ob.rPackBar:setLoc(165, -218)
    Ob:addElement(Ob.rPackBar)

	Ob.rPackIcon = Ob:getUITextureProp(rDef.packImg)
    if Ob.rPackIcon then
        Ob.rPackIcon:setLoc(0, -10)
        Ob.rPackIcon:setScl(0.5)
        Ob:addElement(Ob.rPackIcon)
    end

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(g_DialogManager:getString(Ob.rPackDef.sName))
	Ob.rNameField:setLoc(464, -6)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

    Ob.rCostIcon = Ob:getUITextureProp('icon_pp')
    Ob.rCostIcon:setLoc(170, -218)
    Ob:addElement(Ob.rCostIcon)

	Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rPackDef.sDescription))
    Ob.rDescriptionField:setRect(0, 200, 295, 0)
	Ob.rDescriptionField:setLoc(165, -44)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	
    
    Ob.rStatField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rStatField:setString(g_DialogManager:getString(Ob.rPackDef.sStatDescription))
    Ob.rStatField:setRect(0, 200, 295, 0)
	Ob.rStatField:setLoc(168, -190)
	Ob.rStatField:setColor(0, .5, 0)
	Ob:addElement(Ob.rStatField)	

    Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rCostField:setString(tostring(rDef.nPPPrice))
    Ob.rCostField:setLoc(534, -222)
    Ob.rCostField:setColor(.24, .3, .57)
    Ob:addElement(Ob.rCostField)

    if g_Game:getNumAllHeroes() >= g_Game:getTotalNumHeroes() then
        Ob.rButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { .57, .57, .57 }, nil, true, { 0, -50 }, "STXT136TEXT", 'ptsans', 32, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
        Ob.rButton:setLoc(298, -218)
        Ob.rButton:setParent(Ob)
    else
        Ob.rButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, nil, Ob.rPackDef.nPPPrice, { .57, .57, .57 })
        Ob.rButton:setLoc(298, -218)
        Ob.rButton:setParent(Ob)
    end

    return Ob
end

return PackPurchaseButton