local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local util = require('DFUtil')
local Sprite = require('SpriteSheet')
local SimpleConfirmScreenUI = require('ui/SimpleConfirmScreenUI')
local DFGraphics = require('DFCommon.Graphics')

local tUITextureSpriteSheetInfo =
{
    newhero_bkgd = 'UI/reward',
    hired_stamp = 'UI/hud2',
    superium = 'UI/hud2',
}

local PremiumPurchaseSequenceUI = {}

function PremiumPurchaseSequenceUI:new(rRenderLayer, sPurchaseType, rItem)
    local Ob = UIElement:new(rRenderLayer)
    
    Ob.rItem = rItem
    Ob.sPurchaseType = sPurchaseType
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Stinger', false)    
        else
			g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IngameLoop', true)
        end
    end
    
    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        
        if self.sPurchaseType == "hero" then
            local rHero = self.rItem
            local sHeroDefKey = rHero:getCharDefKey()
            
            -- only show this if you have added a hero that is not active
            if not g_Game:isHeroActive(sHeroDefKey) then
                local rListener = {}
                function rListener:onYesPressed()
                    g_GUI:SetPopupPane(nil)
                end
            
                local rWarningPane = SimpleConfirmScreenUI:new(Renderer.getRenderLayer('UIOverlay'), g_DialogManager:getString("STXT021TEXT"), rListener, "STXT130TEXT")
                rWarningPane:setLoc(240, -160)
                rWarningPane:setVisible(true)
                g_GUI:SetPopupPane(rWarningPane)
            end      
        end
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
        return true
    end
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    
    local imageSizeX = 512
    local imageSizeY = 512
        
    -- we double the background up
    local backgroundScaleX = Renderer.getUIWidth() / (imageSizeX * 2)
    local backgroundScaleY = Renderer.getUIHeight() / imageSizeY
    
    Ob.rLeftBackground = Ob:getUITextureProp('newhero_bkgd')
    Ob.rLeftBackground:setLoc(0, 0)
    -- add an extra bit of scaling to prevent gaps on hardware
    Ob.rLeftBackground:setScl(backgroundScaleX * 1.02, backgroundScaleY)
    Ob.rLeftBackground:setPriority(1)
    Ob:addElement(Ob.rLeftBackground)

    Ob.rRightBackground = Ob:getUITextureProp('newhero_bkgd')
    Ob.rRightBackground:setLoc(Renderer.getUIWidth(), 0)
    Ob.rRightBackground:setScl(-backgroundScaleX, backgroundScaleY)
    Ob.rRightBackground:setPriority(2)
    Ob:addElement(Ob.rRightBackground)
    
    Ob.rTopLetterbox = MOAIProp2D.new()
	Ob.rTopLetterbox:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopLetterbox:setLoc(0, 0)
	Ob.rTopLetterbox:setScl(Renderer.getUIWidth(), 140)
	Ob.rTopLetterbox:setColor(0, 0, 0)
    Ob.rTopLetterbox:setParent(Ob)
    Ob:addOverlayElement(Ob.rTopLetterbox)

    Ob.rBottomLetterbox = MOAIProp2D.new()
	Ob.rBottomLetterbox:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBottomLetterbox:setLoc(0, -(Renderer.getUIHeight() - 140))
	Ob.rBottomLetterbox:setScl(Renderer.getUIWidth(), 140)
	Ob.rBottomLetterbox:setColor(0, 0, 0)
    Ob.rBottomLetterbox:setParent(Ob)
    Ob:addOverlayElement(Ob.rBottomLetterbox)
    
    Ob.rText = g_GUI:CreateTextBox(72, MOAITextBox.CENTER_JUSTIFY, 'bangers')
	Ob.rText:setString("PREMIUM PURCHASE!")
    Ob.rText:setRect(-Renderer.getUIHalfWidth(), 0, Renderer.getUIWidth(), 200)
	Ob.rText:setLoc(-600, -374)
	Ob.rText:setColor(47 / 255, 120 / 255, 163 / 255)
    Ob.rText:setPriority(5)
    Ob:addOverlayElement(Ob.rText)
    
    
    Ob.rHiredStamp = Ob:getUITextureProp('hired_stamp')
    Ob.rHiredStamp:setLoc(Renderer.getUIHalfWidth() + 32, -(Renderer.getUIHalfHeight() + 140))    
    Ob.rHiredStamp:setRot(30)
    Ob.rHiredStamp:setScl(1.4,1.4)
    Ob:addOverlayElement(Ob.rHiredStamp)
    Ob.rHiredStamp:setVisible(false)
    
    function Ob:showSuperium(rItem)
        self:showItem(rItem, "CURR012TEXT", 0, 44, 'superium')
    end
    
    function Ob:showProp(rItem)
        local sSpriteSheetName = rItem.sPremiumPurchaseSpriteSheet
        if sSpriteSheetName and TileMap and rItem.sPremiumPurchaseSprite and TileMap.tTilesetToSpriteSheets and TileMap.tTilesetToSpriteSheets[rItem.sPremiumPurchaseSprite] then
            sSpriteSheetName = TileMap.tTilesetToSpriteSheets[rItem.sPremiumPurchaseSprite]
        end
        self.sExtraSpritePath = 'Levels/Tilesets/'..sSpriteSheetName
        local sSuffix = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].sTextureSizeSuffix
        local nSizeFactor = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].nTextureSizeFactor
        local rSpriteSheet = DFGraphics.loadSpriteSheet(self.sExtraSpritePath..sSuffix, g_IsInStartupSequence)
        local sSpriteKey = rItem.sPremiumPurchaseSprite .. sSuffix .. '_1'

        local nWidth = 256
        local nHeight = 256
        
        if rSpriteSheet then
            self.rItemProp = MOAIProp2D:new()
            self.rItemProp:setDeck(rSpriteSheet)
            
            local nIndex = rSpriteSheet.names[sSpriteKey]
            
            self.rItemProp:setIndex(nIndex)

            local rRect = rSpriteSheet.rects[nIndex]
            
            nWidth = rRect.width
            nHeight = rRect.height
        end
        
        if rItem.nPremiumPurchaseOverrideWidth then
            nWidth = rItem.nPremiumPurchaseOverrideWidth
        end
        
        if rItem.nPremiumPurchaseOverrideHeight then
            nHeight = rItem.nPremiumPurchaseOverrideHeight
        end
    
        self:showItem(rItem, "CURR013TEXT", nWidth, nHeight, nil, self.rItemProp, false, 1, 1)
    end
    
    function Ob:showItem(rItem, sDescription, nWidth, nHeight, sImageId, rSprite, bRotate, nScaleXOverride, nScaleYOverride, sTopTextOverride, sItemTextOverride)
        self.rText:setVisible(false)
        
        if bRotate == nil then
            bRotate = true
        end
        
        if not sDescription then
            sDescription = "CURR012TEXT"
        end
        
        if not sImageId then
            sImageId = 'superium'
        end
        
        local sTopText = sTopTextOverride or "CURR011TEXT"
        local rYouPurchasedText = g_GUI:CreateTextBox(36, MOAITextBox.CENTER_JUSTIFY, 'bangers')
        rYouPurchasedText:setString(g_DialogManager:getString(sTopText))
        rYouPurchasedText:setRect(0, 0, Renderer.getUIWidth(), 200)
        rYouPurchasedText:setLoc(0, -344)
        rYouPurchasedText:setColor(47 / 255, 120 / 255, 163 / 255)
        rYouPurchasedText:setPriority(5)
        rYouPurchasedText:setVisible(false)
        Ob:addOverlayElement(rYouPurchasedText)
        
        if rItem.nPPToGive then
            local nAmount = rItem.nPPToGive
            
            local rValueText = g_GUI:CreateTextBox(60, MOAITextBox.LEFT_JUSTIFY, 'bangers')
            
            local sValueString = "+%1"
            sValueString = string.gsub(sValueString, "%%1", nAmount)
            
            rValueText:setString(sValueString)
            rValueText:setRect(0, 0, 148, 100)
            rValueText:setLoc(Renderer.getUIHalfWidth() + 80, -(Renderer.getUIHalfHeight()+88))
            rValueText:setColor(47 / 255, 120 / 255, 163 / 255)
            rValueText:setPriority(5)
            rValueText:setVisible(false)
            Ob:addOverlayElement(rValueText)
        end
        
        local rDescriptionText = g_GUI:CreateTextBox(36, MOAITextBox.CENTER_JUSTIFY, 'bangers')
        rDescriptionText:setString(g_DialogManager:getString(sDescription))
        rDescriptionText:setRect(0, 0, Renderer.getUIWidth(), 200)
        rDescriptionText:setLoc(0, -(Renderer.getUIHeight() + 12))
        rDescriptionText:setColor(47 / 255, 120 / 255, 163 / 255)
        rDescriptionText:setPriority(5)
        rDescriptionText:setVisible(false)
        Ob:addOverlayElement(rDescriptionText)
        
        if not rSprite then
            rSprite = self:getUITextureProp(sImageId)
            rSprite:setPiv(44, -44)
            rSprite:setLoc(Renderer.getUIHalfWidth() - (nWidth), -(Renderer.getUIHalfHeight() + nHeight - 16))
        else
            rSprite:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight() - 16)
        end

        rSprite:setScl(0.1, 0.1)
        self.rRenderLayer:insertProp(rSprite)
            
        self.rText:setRect(0, 0, Renderer.getUIWidth(), 200)
        self.rText:setString(g_DialogManager:getString(sItemTextOverride or rItem.sName))
        self.rText:setLoc(0, -374, 0, 1.0, MOAIEaseType.SOFT_SMOOTH)
            
        self.rSprite = rSprite
        self.rDescriptionText = rDescriptionText
        
        if rValueText then
            self.rValueText = rValueText
        end
        self.rYouPurchasedText = rYouPurchasedText
        
        function coolAnimation()
            local nScaleX = nScaleXOverride or 2.0
            local nScaleY = nScaleYOverride or 2.0

            rSprite:seekLoc(Renderer.getUIHalfWidth() - (nWidth), -(Renderer.getUIHalfHeight() + nHeight - 16), 2.5, MOAIEaseType.SOFT_SMOOTH)
            rSprite:seekScl(nScaleX, nScaleY, 2.5, MOAIEaseType.SOFT_SMOOTH)
            
            if bRotate then
                rSprite:seekRot(720, 2.5)
            end

            --util.sleep(0.25)
            
            util.sleep(3.0)
            
            self.rText:setVisible(true)
            self.rDescriptionText:setVisible(true)
            
            if self.rValueText then
                self.rValueText:setVisible(true)
            end
            self.rYouPurchasedText:setVisible(true)
            
            util.sleep(4.0)
            
            self.rRenderLayer:removeProp(self.rSprite)
            
            if self.sExtraSpritePath then
                DFGraphics.unloadSpriteSheet(self.sExtraSpritePath)
            end
            
            g_GUI:HidePremiumPurchaseSequence()   
        end
        
        self.rThread = MOAIThread.new()
        self.rThread:run(coolAnimation, self)   
    end
    
    function Ob:showHero(rHero)
        kHERO_OFFSET = 150

        local sHeroKey = rHero:getCharDefKey()
        local rDef = rHero.tCharDef
                
        self.rText:setString(g_DialogManager:getString(rDef.sName))
                
        -- hack! need to just load something valid and apparently default crashes
        sCostumeKey = "lvl1"
    
        local tSpriteSheetDefPaths = rDef.tFightEventSpriteSheetInfo.tSpriteSheetDefPaths[sCostumeKey]
        sAnimDefFileName = rDef.tFightEventSpriteSheetInfo.sAnimDefFileName

        -- TODO: play cool sound
        --g_Audio:playCue('MMOJ_Audio/SFX/HeroSFX/MummyHandSmash')

        -- get spritesheet for hero in combat, and get a sprite for the run animation
        local rSprite = nil
        local rRunSprite = nil
        
        local sAnimName = rDef.sAnimForPurchaseMoment
        self.sSoundCue = rDef.sPurchaseAnimSoundCue
        if not sAnimName then
            sAnimName = "run"
        end
        
        -- find the spritesheet that has this animation!
        local spritesheetDef = tSpriteSheetDefPaths[1]
        for i, def in ipairs(tSpriteSheetDefPaths) do
            local testSprite = Sprite:new(def, sAnimDefFileName, 0, 0)
            local animCheck = testSprite:getAnimation(sAnimName)
            if animCheck then
                rSprite = testSprite
                spritesheetDef = def
            end
            
            local runCheck = testSprite:getAnimation("run")
            if runCheck then
                rRunSprite = testSprite
            end
            
            if not runCheck and not animCheck then
                testSprite:unloadSpriteSheet()
            end
        end
        
        if not rSprite then
            rSprite = Sprite:new(spritesheetDef, sAnimDefFileName, 0, 0)
        end
        
        rSprite.rCharEnt = self

        self.rRenderLayer:insertProp(rRunSprite)
        self.rRenderLayer:insertProp(rSprite)

        g_HackSprite = rSprite
        
        local heroY = Renderer:getUIHalfHeight() + kHERO_OFFSET
        
        rSprite:setLoc(Renderer:getUIWidth(), -heroY)
        rSprite:setScl(1.5, 1.5)
        rSprite:setVisible(false)
        rSprite:setPriority(50)
        
        rRunSprite:setLoc(Renderer:getUIWidth(), -heroY)
        rRunSprite:setScl(1.5, 1.5)
        rRunSprite:setVisible(true)
        rRunSprite:setPriority(50)
        
        -- play cool animation
        function coolAnimation()
            self.rRunSprite:playAnimation("run")
            
            local heroPosX = Renderer:getUIHalfWidth()
            local heroPosY = -(Renderer:getUIHalfHeight() + kHERO_OFFSET)
            
            self.rRunSprite:seekLoc(heroPosX, heroPosY, 1.5, MOAIEaseType.SOFT_SMOOTH)
            
            util.sleep(0.5)
        
            self.rText:seekLoc(0, -374, 0, 1.0, MOAIEaseType.SOFT_SMOOTH)
        
            util.sleep(1.0)
        
            self.rRunSprite:setVisible(false)
            
            if self.sSoundCue then
                g_Audio:playCue(self.sSoundCue)
            end
            self.rSprite:setLoc(heroPosX, heroPosY)
            self.rSprite:playAnimation(self.sAnimName)
            self.rSprite:setVisible(true)
            
            util.sleep(2.0)
            
            -- show "HIRED" stamp
            self.rHiredStamp:setVisible(true)
            g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')
            
            util.sleep(1.5)
            
            self.rSprite:setVisible(false)
            self.rRunSprite:setVisible(true)
            self.rRunSprite:playAnimation("run")
            
            self.rRunSprite:seekLoc(-148, heroPosY, 1.5, MOAIEaseType.SOFT_SMOOTH)
            
            util.sleep(2.0)
            
            if g_Game:canAddActiveHeroes() then
                local rHero = self.rItem
                local sHeroDefKey = rHero:getCharDefKey()
            
                g_Game:setHeroActive(sHeroDefKey, true)
                if g_Game:getTutorialManager():hasTutorialCompleted('PatrolTutorial') then -- tutorial will handle activating this hero
                    rHero:runActivities(true)
                end
            end
            
            if self.rRunSprite ~= self.rSprite then
                self.rRenderLayer:removeProp(self.rRunSprite)
                self.rRunSprite:unloadSpriteSheet()
            end
            
            self.rRenderLayer:removeProp(self.rSprite)
            self.rSprite:unloadSpriteSheet()  
            
            self.rSprite = nil
            self.rRunSprite = nil
            
            
            
            g_GUI:HidePremiumPurchaseSequence()     
        end
        
        self.sAnimName = sAnimName
        
        self.rSprite = rSprite
        self.rRunSprite = rRunSprite
        
        self.rThread = MOAIThread.new()
        self.rThread:run(coolAnimation, self)   
    end

    function Ob:showEquipment(rItem)
        local rDef = rItem:getItemDef()
        if rDef and rDef.sSpriteSheet and rDef.iconName then
            tUITextureSpriteSheetInfo[rDef.iconName] = rDef.sSpriteSheet
            self:loadUISpriteSheets(tUITextureSpriteSheetInfo)            
                        
            self:showItem(rItem, "STXT210TEXT", 0, 0, rDef.iconName, nil, false, 1, 1, "STXT209TEXT", rItem:getItemDef().sName)
        end
    end    
    
    if sPurchaseType == "hero" then
        Ob:showHero(rItem)
    elseif sPurchaseType == "superium" then
        Ob:showSuperium(rItem)
    elseif sPurchaseType == "prop" then
        Ob:showProp(rItem)
    elseif sPurchaseType == "equipment" then
        Ob:showEquipment(rItem)
    end

    return Ob
end

--newhero_bkgd

return PremiumPurchaseSequenceUI
