local PromotionUI = {}
local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local HeroChooserUI = require('ui/HeroChooserUI')
local Character = require('Character')

local tUITextureSpriteSheetInfo =
{
justice_icon = 'UI/hud2',
stamp_approved = 'UI/eventdetails',
}

function PromotionUI:new(rRenderLayer, rHero)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

    function Ob:closeThread()
        util.sleep(1)
        g_GUI:HideLevelUpUI()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rCloseThread then
            return true
        end
        if eventType == MOAITouchSensor.TOUCH_UP then   
            self.rApproveStamp:setVisible(true)
            g_Audio:playCue('MMOJ_Audio/UI/SuccessStamp')
            self.rCloseThread = MOAIThread:new()
            self.rCloseThread:run(self.closeThread, self)
        end
		return true
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:getWidth()
        return 500
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	-- vars
	Ob.rHero = rHero

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(Ob:getWidth(), 544)
    Ob.rBackground:setColor(0.93, 0.99, 1)
    Ob:addElement(Ob.rBackground)

    Ob.rLogo = Ob:getUITextureProp('justice_icon')
    Ob.rLogo:setColor(0.47, 0.58, 0.58)
    Ob.rLogo:setLoc(210, -20)
    Ob.rLogo:setScl(0.5)
    Ob:addElement(Ob.rLogo)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(25, -80)
    Ob.rLine1:setScl(450, 2)
    Ob.rLine1:setColor(0.47, 0.58, 0.58)
    Ob:addElement(Ob.rLine1)

    Ob.rFormLabelText = g_GUI:CreateTextBox(12, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rFormLabelText:setString(g_DialogManager:getString("STXT067TEXT")) --official promotion form 19E
    Ob.rFormLabelText:setColor(0.47, 0.58, 0.58)
    Ob.rFormLabelText:setRect(0, 80, 380, 0)
    Ob.rFormLabelText:setLoc(307, -85)
    Ob:addElement(Ob.rFormLabelText)

    Ob.rLine1Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine1Text:setString(g_DialogManager:getString("STXT048TEXT")) --"dear"
    Ob.rLine1Text:setRect(0, 80, 380, 0)
    Ob.rLine1Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine1Text:setLoc(30, -110)
    Ob:addElement(Ob.rLine1Text)

    Ob.rNameText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rNameText:setString(g_DialogManager:getString(rHero:getName())..",") --name,
    Ob.rNameText:setRect(0, 80, 380, 0)
    Ob.rNameText:setColor(0.2, 0.2, 0.2)
    Ob.rNameText:setLoc(83, -110)
    Ob:addElement(Ob.rNameText)

    Ob.rLine2Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine2Text:setString(g_DialogManager:getString("STXT054TEXT")) --you have received a promotion!
    Ob.rLine2Text:setRect(0, 80, 380, 0)
    Ob.rLine2Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine2Text:setLoc(30, -155)
    Ob:addElement(Ob.rLine2Text)

    Ob.rLine3Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine3Text:setString(g_DialogManager:getString("STXT049TEXT")) --we are pleased to offer you the following
    Ob.rLine3Text:setRect(0, 80, 420, 0)
    Ob.rLine3Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine3Text:setLoc(30, -195)
    Ob:addElement(Ob.rLine3Text)

    Ob.rLine3_2Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine3_2Text:setString(g_DialogManager:getString("STXT153TEXT")) -- title and compensation
    Ob.rLine3_2Text:setRect(0, 80, 420, 0)
    Ob.rLine3_2Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine3_2Text:setLoc(30, -220)
    Ob:addElement(Ob.rLine3_2Text)

    Ob.rPortrait = Ob.rHero:getPortraitProp()
    if Ob.rPortrait then
        Ob.rPortrait:setLoc(30, -275)
        Ob:addElement(Ob.rPortrait)
    end

    Ob.rLine5Text = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rLine5Text:setString(g_DialogManager:getString("STXT050TEXT").." "..tostring(rHero:getStatLevel() + 1)) --level 1
    Ob.rLine5Text:setRect(0, 80, 420, 0)
    Ob.rLine5Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine5Text:setLoc(175, -265)
    Ob:addElement(Ob.rLine5Text)
    
    Ob.rTitleText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rTitleText:setString(g_DialogManager:getString(Character.tLevelToTitle[rHero:getStatLevel() + 1])) --intern of justice
    Ob.rTitleText:setRect(0, 80, 380, 0)
    Ob.rTitleText:setColor(0.2, 0.2, 0.2)
    Ob.rTitleText:setLoc(175, -300)
    Ob:addElement(Ob.rTitleText)

    local nNumTrainingSessions = 0
    if Character.tLevelImprovementInfo then
        local rInfo = Character.tLevelImprovementInfo[rHero:getStatLevel() + 1]
        if rInfo then
            nNumTrainingSessions = rInfo.nNumTrainingSessions
        end    
    end
    Ob.rTrainingText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.rTrainingText:setString("+"..nNumTrainingSessions.." "..g_DialogManager:getString("STXT066TEXT")) --+3 training sessions
    Ob.rTrainingText:setRect(0, 80, 380, 0)
    Ob.rTrainingText:setLoc(175, -335)
    Ob.rTrainingText:setColor(0, .7, 0)
    Ob:addElement(Ob.rTrainingText)

    Ob.rLine7Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine7Text:setString(g_DialogManager:getString("STXT052TEXT")) --congratulations!
    Ob.rLine7Text:setRect(0, 80, 420, 0)
    Ob.rLine7Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine7Text:setLoc(30, -435)
    Ob:addElement(Ob.rLine7Text)

    Ob.rLine8Text = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
    Ob.rLine8Text:setString(g_DialogManager:getString("STXT053TEXT")) --management
    Ob.rLine8Text:setRect(0, 80, 420, 0)
    Ob.rLine8Text:setColor(0.2, 0.2, 0.2)
    Ob.rLine8Text:setLoc(30, -475)
    Ob:addElement(Ob.rLine8Text)

    Ob.rApproveStamp = Ob:getUITextureProp('stamp_approved')
    Ob.rApproveStamp:setLoc(0, -340)
    Ob.rApproveStamp:setRot(42)
    Ob.rApproveStamp:setScl(1.25,1.25)
    Ob:addOverlayElement(Ob.rApproveStamp)
    Ob.rApproveStamp:setVisible(false)    

	return Ob
end

return PromotionUI
