local UIElement = require('ui/UIElement')
local HeroChooserUI = require('ui/HeroChooserUI')
local TrainingSelectionUI = require('ui/TrainingSelectionUI')
--local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')
local AbilitySelectionUI = require('ui/AbilitySelectionUI')
local Renderer = require('Renderer')

local PropActionHeroSelectorUI = {}

function PropActionHeroSelectorUI:new(rRenderLayer)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:Show(rProp, tSelectableHeroes, rChooserListener, sLabel)
        self.tSelectableHeroes = tSelectableHeroes
        self.rChooserListener = rChooserListener
        self.rCurrentProp = rProp
    
        local rHeroChooserUI = HeroChooserUI:new(self.rRenderLayer, self, sLabel)
        rHeroChooserUI:setVisible(true)
        rHeroChooserUI:maximize(true)
        
        self.rHeroChooserUI = rHeroChooserUI
    end
    
    function Ob:Hide()
        self.tSelectableHeroes = {}
                    
        self.rCurrentProp = nil
            
        self.setVisible(false)
    end
    
	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if self.rHeroChooserUI then
            self.rHeroChooserUI:setVisible(bVisible)
        end        
        if self.rTrainingSelectionUI then
            self.rTrainingSelectionUI:setVisible(bVisible)
        end        
        if self.rAbilitySelectionUI then
            self.rAbilitySelectionUI:setVisible(bVisible)
        end
    end

    function Ob:isVisible()
        return self.bVisible
    end
    
    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rHeroChooserUI then
            self.rHeroChooserUI:onFinger(eventType, sx, sy, tProps)
        end
        
        if self.rTrainingSelectionUI then
            self.rTrainingSelectionUI:onFinger(eventType, sx, sy, tProps)
        end    
        
        if self.rAbilitySelectionUI then
            self.rAbilitySelectionUI:onFinger(eventType, sx, sy, tProps)
        end
        
        return true
    end
    
    function Ob:update()
        if self.rHeroChooserUI then
            self.rHeroChooserUI:update()
        end
        
  
    end
    
    -- required by HeroChooserUI
    function Ob:getSelectableHeroes()
        return self.tSelectableHeroes
    end

    -- used by HeroChooserUI
    function Ob:onHeroChooserCancelled()
        if self.rHeroChooserUI then
            self.rHeroChooserUI:setVisible(false)
            self.rHeroChooserUI:onPaneClosed()
            self.rHeroChooserUI = nil
        end
        
        self.rCurrentProp = nil
            
        
        g_GUI:HidePropActionHeroSelectorUI()
        
        if self.rChooserListener and self.rChooserListener.onHeroChooserCancelled then
            self.rChooserListener:onHeroChooserCancelled()
        end
    end

    -- required by HeroChooserUI
    function Ob:onHeroSelected(rHero)
        if self.rCurrentProp then
            
            local rProp = self.rCurrentProp
            local propType = rProp:getPropType()

            -- actually use whatever prop was just activated by the specific hero
            if propType == "bed" then
                self:onHeroUseBed(rHero, rProp)           
            elseif propType == "workout" then
                self:onHeroUseWorkout(rHero, rProp)
            elseif propType == "work" then
                self:onHeroWork(rHero, rProp)
            elseif propType == "trainingdummy" then
                self:onHeroUseTrainingDummy(rHero, rProp)
            elseif propType == "research" then
                self:onHeroResearch(rHero, rProp)
            elseif propType == "recreation" then
                self:onHeroUseRecreation(rHero, rProp)
            end
        end
    
        if self.rChooserListener and self.rChooserListener.onHeroSelected then
            self.rChooserListener:onHeroSelected(rHero)
        end
    end
    
    -- required by TrainingSelectionUI
    function Ob:setTrainingSelectionUIVisible(bVisible)
    
    end
    
    -- required by 
    function Ob:hideActivitySelector()
    
    end
    
    -- required by AbilityUpgradeUI
    function Ob:setTrainAbilityUIVisible(bVisible)
    
    end
    
    function Ob:onHeroUseBed(rHero, rProp)
        g_GUI:HidePropActionHeroSelectorUI()
        rHero:setCurActivity('Rest')
        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
    end
    
    function Ob:onHeroUseWorkout(rHero, rProp)
        --g_GUI:HideHeroActivityChooserUI()
        --g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
        
        if self.rTrainingSelectionUI then
            self.rTrainingSelectionUI:setVisible(false)
            self.rTrainingSelectionUI = nil
        end
        
        self.rTrainingSelectionUI = TrainingSelectionUI:new(self.rRenderLayer, self, rHero)
        self.rTrainingSelectionUI:setLoc(516, 0)
        self.rTrainingSelectionUI:setVisible(true)
    end
    
    function Ob:onHeroWork(rHero, rProp)
        rHero:setCurActivity('PTJob')
        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
        g_GUI:HidePropActionHeroSelectorUI()
    end
    
    function Ob:onHeroUseTrainingDummy(rHero, rProp)
        local rOverlayLayer = Renderer.getRenderLayer('UIOverlay')
        
        if self.rAbilitySelectionUI then
            self.rAbilitySelectionUI:setVisible(false)
            self.rAbilitySelectionUI = nil
        end
        
        self.rAbilitySelectionUI = AbilitySelectionUI:new(rOverlayLayer, self, rHero, true)
        self.rAbilitySelectionUI:setParent(self)
        --self.rAbilitySelectionUI:setLoc((Renderer.getUIWidth() - 672) / 2, -(Renderer.getUIHeight() - 640) / 2)
        self.rAbilitySelectionUI:setLoc(500,0)
        self.rAbilitySelectionUI:setVisible(true)
    end
    
    function Ob:onHeroResearch(rHero, rProp)
        g_GUI:ShowResearchUI(rHero, self.nActivityIndex)
        self:hideActivitySelector()
    end
    
    function Ob:onHeroUseRecreation(rHero, rProp)
        rHero:setCurActivity('Recreation')
        g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
        self:hideActivitySelector()
        g_GUI:HidePropActionHeroSelectorUI()
    end
    
    Ob.rCurrentProp = nil
    Ob.rHeroChooserUI = nil
    Ob.rRenderLayer = rRenderLayer
    Ob.rTrainingSelectionUI = nil
    Ob.rAbilitySelectionUI = nil
    
    return Ob
end

return PropActionHeroSelectorUI
