local TileMap = require('TileMap')
local UIElement = require('ui/UIElement')
local ShopButton = require('ui/ShopButton')
local DFAnalytics = require('DFCommon.Analytics')

local PropUpgradeButton = {}

local tUITextureSpriteSheetInfo =
{
    room_slot = 'UI/hud3',
    room_slot_buy = 'UI/hud3',
    icon_cash = 'UI/hud',
}

function PropUpgradeButton:new(rRenderLayer, rProp, rParentUI)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:getProp()
        return self.rProp
    end

    function Ob:getPropDef()
        return self.rDef
    end

    function Ob:getPropDefKey()
        return self.rDefKey
    end

    function Ob:canUpgrade()
        if self.rDef.mmlvl then
            if g_MiddleManager:getStarRatingLevel() < self.rDef.mmlvl then
                return false
            end
        end
        if g_Game:getCurCash() < self.rDef.upgradeCost then
            return false
        end     
        if not self.rProp:isUnlocked() then
            return true
        end
        if not self.bFullyUpgraded and self.rProp and self.rProp:canUpgrade() then
            return true
        end
        return false
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rUpgradeButton then
            if self:canUpgrade() then
                self:upgradeProp()
            end
        end
    end
        
    function Ob:update()
        if self.rUpgradeButton then
            self.rUpgradeButton:update()
        end
    end
        
	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rUpgradeButton then
            self.rUpgradeButton:onFinger(eventType, sx, sy, tProps)
        end
		return true
	end

    function Ob:upgradeProp()
        g_Game:removeCash(self.rDef.upgradeCost)
        DFAnalytics.logEvent('buy_upgr_prop', { type = self.rDefKey, unlock = self.rProp:isUnlocked(), cost = self.nCost } )
        DFAnalytics.incrStat('csh_spnt_props', self.nCost)
        if not self.rProp:isUnlocked() then
            self.rProp:unlockProp()
            DFAnalytics.incrStat('props_unlocked', 1)
            g_TodoList:onPropAdded(self.rProp)
        else
            self.rProp:upgradeProp(self.rProp:getUpgradeDefKey())
            DFAnalytics.incrStat('props_upgraded', 1)
            g_TodoList:onPropUpgraded(self.rProp)
        end
        if self.rParentUI then
            self.rParentUI:onPropUpgraded(self)
        end        
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rUpgradeButton then
            self.rUpgradeButton:setVisible(bVisible)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rUpgradeButton then
            self.rUpgradeButton:onPaneClosed()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rProp = rProp
    Ob.rParentUI = rParentUI

    Ob.bFullyUpgraded = false
    Ob.rDef = rProp:getPropDef()

    if rProp:isUnlocked() then
        local sUpgradeDefKey = rProp:getUpgradeDefKey()
        if not sUpgradeDefKey then
            Ob.bFullyUpgraded = true            
        else
            Ob.rDefKey = sUpgradeDefKey
            Ob.rDef = TileMap.getPropDef(sUpgradeDefKey)
        end
    end

    if Ob.rDef then
        Ob.nCost = rProp:getPropDef().upgradeCost
    end

    if rProp:isUnlocked() and not Ob.bFullyUpgraded then
        Ob.rBackGround = Ob:getUITextureProp('room_slot')
        Ob.rBackGround:setLoc(-18, 24)
        Ob:addElement(Ob.rBackGround)
        if rProp:getUpgradeDefKey() then
            local rUpgradeDef = TileMap.getPropDef(rProp:getUpgradeDefKey())
            if rUpgradeDef then                
                Ob.nCost = rUpgradeDef.upgradeCost
            end
        end
    end

    Ob.rLevelField = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rLevelField:setString(g_DialogManager:getString("STXT006TEXT").." "..Ob.rDef.nLevel.." "..string.upper(g_DialogManager:getString(Ob.rDef.visualName)))
    Ob.rLevelField:setLoc(14, -8)
    Ob.rLevelField:setColor(0, 0, 0)
    Ob.rLevelField:setRect(0, 80, 300, 0)
    Ob:addElement(Ob.rLevelField)

    if not Ob.bFullyUpgraded then
        if Ob.rDef.mmlvl > g_MiddleManager:getStarRatingLevel() then
            Ob.tReqText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
            Ob.tReqText:setString(tostring(g_DialogManager:getString("STXT183TEXT").." "..Ob.rDef.mmlvl))
            Ob.tReqText:setLoc(15, -54)
            Ob.tReqText:setRect(0, 80, 300, 0)
            Ob.tReqText:setColor(.85, 0, 0)
            Ob:addElement(Ob.tReqText)
        else
            Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
            Ob.rCostIcon:setScl(0.5)
            Ob.rCostIcon:setLoc(110, -58)
            Ob:addElement(Ob.rCostIcon)

            local tCashColor = g_Game:getCashColor()    
            Ob.rCostField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'type')
            Ob.rCostField:setString(tostring(Ob.nCost))
            Ob.rCostField:setLoc(140, -54)
            Ob.rCostField:setRect(0, 80, 300, 0)
            Ob.rCostField:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
            Ob:addElement(Ob.rCostField)
        end

        local tButtonColor = {.36, .55, .94}
        local tTextColor = nil
        if rProp:isUnlocked() then
            if not rProp:canUpgrade() then
                tButtonColor = { .57, .57, .57 }
                tTextColor = { .74, .74, .74 }
            end
        end
        Ob.rUpgradeButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'room_slot_buy', nil, tButtonColor, tTextColor, true, { 0, -50 }, "STXT081TEXT", 'ptsans', 36, { 100, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.nCost, nil, { .57, .57, .57 }, { .74, .74, .74 }, Ob.rDef.mmlvl, { .57, .57, .57 }, { .74, .74, .74 })
        Ob.rUpgradeButton:setLoc(8, -84)
        Ob.rUpgradeButton:setParent(Ob)

        if rProp:isUnlocked() then
            Ob.rUpgradeButton:setText("STXT080TEXT")
--            if not rProp:canUpgrade() then
--                Ob.rUpgradeButton:setText("STXT082TEXT")
--            end

            local sUpgradeDefKey = rProp:getUpgradeDefKey()
            if sUpgradeDefKey then
                local rUpgradeDef = TileMap.getPropDef(sUpgradeDefKey)
                if rUpgradeDef then
                    local sPrefix = rUpgradeDef.sUpgradePrefix or ''
                    local sPct = ''                    
                    if rUpgradeDef.nUpgradeVisualPct then
                        sPct = tostring(rUpgradeDef.nUpgradeVisualPct)
                    end
                    Ob.rPctField = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
                    Ob.rPctField:setString(g_DialogManager:getString(sPrefix).." "..sPct..g_DialogManager:getString("PROP031TEXT"))
                    Ob.rPctField:setLoc(14, -30)
                    Ob.rPctField:setColor(0, .5, 0)
                    Ob.rPctField:setRect(0, 80, 300, 0)
                    Ob:addElement(Ob.rPctField)
                end
            end
        end
    end

	return Ob
end

return PropUpgradeButton