local util = require('DFUtil')
local CurrencyPurchaseButton = require('ui/CurrencyPurchaseButton')
local ListUI = require('ui/ListUI')
local UIElement = require('ui/UIElement')
local DFIAP = require('DFCommon.IAP')
local RMTSelectorUI = {}

function RMTSelectorUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
        if bVisible then
            g_Game:getTutorialManager():onWindowOpen('CurrencyShopCategory')
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rListUI:onPaneClosed()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    function Ob:update()
        self.rListUI:update()
    end
        
    -- create the equipment buttons
    Ob.tCurrencyButtons = {}
    local tProductInfo = DFIAP.getProductCatalog()
    for i, sCurrencyDefKey in ipairs(g_Game:getAllCurrencyDefKeys()) do
        local rDef = g_Game:getCurrencyDef(sCurrencyDefKey)
        if rDef then
            if rDef.sAppleProductID then
                if DFIAP.arePurchasesAllowed() and tProductInfo then
                    -- verify that it's in the product list
                    for k, rProductInfo in pairs(tProductInfo) do
                        if rDef.sAppleProductID == rProductInfo.id then
                            rDef.nRealCashPrice = rProductInfo.price
                            rDef.priceLocale = rProductInfo.priceLocale
                            rDef.localizedPrice = rProductInfo.localizedPrice
                            if rDef.bIsCoinDoubler then
                                if not g_Game:isCoinDoublerEnabled() then
                                    local rNewButton = CurrencyPurchaseButton:new(Ob.rRenderLayer, rDef, sCurrencyDefKey)
                                    table.insert(Ob.tCurrencyButtons, rNewButton)
                                end
                            elseif rDef.bIsPPPlant then
                                if not g_Game:isSuperiumTreeEnabled() then
                                    local rNewButton = CurrencyPurchaseButton:new(Ob.rRenderLayer, rDef, sCurrencyDefKey)
                                    table.insert(Ob.tCurrencyButtons, rNewButton)
                                end
                            else
                                local rNewButton = CurrencyPurchaseButton:new(Ob.rRenderLayer, rDef, sCurrencyDefKey)
                                table.insert(Ob.tCurrencyButtons, rNewButton)
                            end
                            break
                        end
                    end
                end
            else
                local rNewButton = CurrencyPurchaseButton:new(Ob.rRenderLayer, sCurrencyDefKey)
                table.insert(Ob.tCurrencyButtons, rNewButton)                
            end
        end
    end
    for i, sCurrencyDefKey in ipairs(g_Game:getAllExchangeDefKeys()) do
        local rDef = g_Game:getExchangeDef(sCurrencyDefKey)
        if rDef then
            local rNewButton = CurrencyPurchaseButton:new(Ob.rRenderLayer, rDef, sCurrencyDefKey)
            table.insert(Ob.tCurrencyButtons, rNewButton)                
        end
    end

    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tCurrencyButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

return RMTSelectorUI