local Item = require('Item')
local UIElement = require('ui/UIElement')
local ShopButton = require('ui/ShopButton')
local RecipesDetailButton = {}

local tUITextureSpriteSheetInfo =
{
shop_item = 'UI/shop',
button_buy = 'UI/shop',
icon_cash = 'UI/hud',
lock_research = 'UI/shop',
activity_sheet_research = 'UI/hud',
icon_new = 'UI/hud2',
icon_recipe = 'UI/icons4',
}

function RecipesDetailButton:new(rRenderLayer, sItemKey, bIsResearchButton)
	if not sItemKey then
		print("ERROR: RecipesDetailButton created without an Item")
		return nil
	end

    local rDef = Item.getMeteoriteDef(sItemKey)
    if not rDef then
		print("ERROR: RecipesDetailButton created without an Item")
		return nil
    end

	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            if self.rParentUI then
                self.rParentUI:handleButtonPress(self, eventType)
            end
            self:onButtonPressed()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rButton then
            self.rButton:onFinger(eventType, sx, sy, tProps)
        end
        if self.onFingerCallBack then
            self:onFingerCallBack(eventType, sx, sy, tProps)
        end
		return true
	end

	function Ob:setVisible(bVisible)
        if Ob.bIsVisible ~= bVisible then
            Ob.bIsVisible = bVisible

            local bIsNew = g_MiddleManager:isItemNewInShop(self.sItemKey)
            
            if bIsNew then
                if not self.rNewIcon then
                    self.rNewIcon = self:getUITextureProp('icon_new')
                    self.rNewIcon:setLoc(86, -98)
                    self.rNewIcon:setScl(0.7, 0.7)
                    self:addElement(self.rNewIcon)
                end
            end
            
            self:setElementsVisible(bVisible)
            
            if self.rNewIcon then
                self.rNewIcon:setVisible(bIsNew and bVisible)
            end
            
            if self.rButton then
                self.rButton:setVisible(bVisible)
            end
        end
	end

	function Ob:getYSize()
        return 220
	end

    function Ob:getButton()
        return self.rButton
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.sItemKey = sItemKey
	Ob.rItemDef = rDef
    Ob.rParentUI = nil

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rBackGround = Ob:getUITextureProp('shop_item')
	Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

	-- add the textfields
	Ob.rNameField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
	Ob.rNameField:setString(g_DialogManager:getString(Ob.rItemDef.sName))
    Ob.rNameField:setRect(0, 80, 300, 0)
	Ob.rNameField:setLoc(155, -10)
	Ob.rNameField:setColor(0, 0, 0)
	Ob:addElement(Ob.rNameField)

	Ob.rDescriptionField = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescriptionField:setString(g_DialogManager:getString(Ob.rItemDef.sDescription))
    Ob.rDescriptionField:setRect(0, 200, 315, 0)
	Ob.rDescriptionField:setLoc(155, -50)
	Ob.rDescriptionField:setColor(0, 0, 0)
	Ob:addElement(Ob.rDescriptionField)	
    
    
    Ob.rIcon = Ob:getUITextureProp('icon_recipe')
    Ob.rIcon:setLoc(16, -16)
    Ob:addElement(Ob.rIcon)
        

    if g_MiddleManager:isResearched('meteorite', sItemKey) then
        Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
        Ob.rCostIcon:setLoc(20, -154)
        Ob:addElement(Ob.rCostIcon)
        
        local tCashColor = g_Game:getCashColor()
        Ob.rCostField = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rCostField:setString(tostring(Ob.rItemDef.nCost))
        Ob.rCostField:setLoc(390, -156)
        Ob.rCostField:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
        Ob:addElement(Ob.rCostField)
        
        Ob.rButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 36, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.rItemDef.nCost, nil, { .57, .57, .57 })
        Ob.rButton:setLoc(306, -152)
        Ob.rButton:setParent(Ob)
    else
        Ob.rLockIcon = Ob:getUITextureProp('lock_research')
        if Ob.rLockIcon then
            Ob.rLockIcon:setLoc(10, -10)
            Ob.rLockIcon:setScl(1.1)
            Ob:addElement(Ob.rLockIcon)
        end
    end

    return Ob
end

return RecipesDetailButton