local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local EquipmentSelectorUI = require('ui/EquipmentSelectorUI')
local ConsumableSelectorUI = require('ui/ConsumableSelectorUI')
local RecipesSelectorUI = require('ui/RecipesSelectorUI')
local ConsumableDetailButton = require('ui/ConsumableDetailButton')
local EquipmentDetailButton = require('ui/EquipmentDetailButton')
local RecipesDetailButton = require('ui/RecipesDetailButton')
local ListUI = require('ui/ListUI')
local util = require('DFUtil')
local Button = require('ui/Button')
local ResearchUI = {}
local ResearchCategoryPane = {}
local ResearchCategoryButton = {}
local ResearchButton = {}
local ConsumableResearchSelectorUI = {}
local EquipmentResearchSelectorUI = {}
local RecipesResearchSelectorUI = {}
local ResearchEquipmentButton = {}
local ResearchConsumableButton = {}
local ResearchRecipesButton = {}

local tUITextureSpriteSheetInfo =
{
    back_button = 'UI/hud',
    back_button_tap = 'UI/hud',
    research_slot_default = 'UI/shop',
    research_slot_focused = 'UI/shop',
    tape_label1 = 'UI/shop',
    icon_time = 'UI/hud',
    icon_int = 'UI/hud',    
    button_buy = 'UI/shop',
}

function ResearchRecipesButton:new(rRenderLayer, sItemKey, rHero)
	local Ob = RecipesDetailButton:new(rRenderLayer, sItemKey, true)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self:canResearchItem() then
            self.rResearchButton:setVisible(bVisible)
        end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rResearchButton then
            self:researchItem()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self:canResearchItem() then
            self.rResearchButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:canResearchItem()
        return self.bCanResearch
    end

    function Ob:researchItem()
        if self:canResearchItem() then
            if self.rHero then
                self.rHero:addResearchActivityAtIndex(1, self.sItemKey)
            end
            g_GUI:SetActivePane(nil)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero
    Ob.sItemKey = sItemKey

    Ob.rResearchButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT104TEXT", 'ptsans', 26, { 80, -2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rResearchButton:setLoc(306, -152)
    Ob.rResearchButton:setParent(Ob)

    local rRoom = g_CurrentMap:getRoom("lab")
    local nRoomLevel = rRoom:getRoomDef().nRoomLevel
    if not nRoomLevel then
        nRoomLevel = 1
    end

    local nReqRoomLevel = 1
    if Ob.rItemDef.nRoomLevel then
        nReqRoomLevel = Ob.rItemDef.nRoomLevel
    end

    local nDuration = Ob.rItemDef.nResearchDuration or 60
    local rHighestLevelProp = rRoom:getHighestLevelAvailableProp()
    if rHighestLevelProp then
        local nResearchMult = rHighestLevelProp:getPropDef().researchSpeed
        if nResearchMult and nResearchMult > 0 then
            nDuration = nDuration / nResearchMult
        end
    end
    local nElapsed = g_MiddleManager:getResearchProgress(sItemKey) * nDuration
    nDuration = nDuration - nElapsed
    if nDuration < 0 then
        nDuration = 0
    end
    Ob.bCanResearch = true
    if nReqRoomLevel > nRoomLevel then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT102TEXT").." "..Ob.rItemDef.nRoomLevel.." "..g_DialogManager:getString("STXT140TEXT"))
        Ob.rCantResearchText:setLoc(26, -160)
        Ob:addElement(Ob.rCantResearchText)
    elseif rHero:getStatAwareness() < Ob.rItemDef.nReqInt then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT103TEXT"))
        Ob.rCantResearchText:setLoc(26, -160)
        Ob:addElement(Ob.rCantResearchText)

        Ob.rAwarenessIcon = Ob:getUITextureProp('icon_int')
        Ob.rAwarenessIcon:setLoc(100, -145)
        Ob:addElement(Ob.rAwarenessIcon)

        Ob.rAwarenessText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rAwarenessText:setRect(0, 80, 150, 0)
        Ob.rAwarenessText:setString(tostring(Ob.rItemDef.nReqInt))
        Ob.rAwarenessText:setLoc(164, -160)
        Ob.rAwarenessText:setColor(1, 0, 0)
        Ob:addElement(Ob.rAwarenessText)
    elseif g_MiddleManager:isResearching(sItemKey) then
        Ob.bCanResearch = false
        
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 500, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT176TEXT"))
        Ob.rCantResearchText:setLoc(26, -160)
        Ob:addElement(Ob.rCantResearchText)
    else
        Ob.rTimeIcon = Ob:getUITextureProp('icon_time')
        Ob.rTimeIcon:setLoc(20, -156)
        Ob:addElement(Ob.rTimeIcon)

        Ob.rTimeText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rTimeText:setRect(0, 80, 150, 0)
        Ob.rTimeText:setString(util.convertTimeString(nDuration, true))
        Ob.rTimeText:setLoc(80, -160)
        Ob:addElement(Ob.rTimeText)
    end

    return Ob
end

function ResearchEquipmentButton:new(rRenderLayer, sItemKey, rHero)
	local Ob = EquipmentDetailButton:new(rRenderLayer, sItemKey, true)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self:canResearchItem() then
            self.rResearchButton:setVisible(bVisible)
        end
	end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rResearchButton then
            self:researchItem()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self:canResearchItem() then
            self.rResearchButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:canResearchItem()
        return self.bCanResearch
    end

    function Ob:researchItem()
        if self:canResearchItem() then
            if self.rHero then
                self.rHero:addResearchActivityAtIndex(1, self.sItemKey)
            end
            g_GUI:SetActivePane(nil)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero
    Ob.sItemKey = sItemKey

    Ob.rResearchButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT104TEXT", 'ptsans', 26, { 80, -2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rResearchButton:setLoc(306, -248)
    Ob.rResearchButton:setParent(Ob)

    local rRoom = g_CurrentMap:getRoom("lab")
    local nRoomLevel = rRoom:getRoomDef().nRoomLevel
    if not nRoomLevel then
        nRoomLevel = 1
    end

    local nReqRoomLevel = 1
    if Ob.rItemDef.nRoomLevel then
        nReqRoomLevel = Ob.rItemDef.nRoomLevel
    end

    local nDuration = Ob.rItemDef.nResearchDuration or 60
    local rHighestLevelProp = rRoom:getHighestLevelAvailableProp()
    if rHighestLevelProp then
        local nResearchMult = rHighestLevelProp:getPropDef().researchSpeed
        if nResearchMult and nResearchMult > 0 then
            nDuration = nDuration / nResearchMult
        end
    end
    local nElapsed = g_MiddleManager:getResearchProgress(sItemKey) * nDuration
    nDuration = nDuration - nElapsed
    if nDuration < 0 then
        nDuration = 0
    end
    Ob.bCanResearch = true
    if nReqRoomLevel > nRoomLevel then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT102TEXT").." "..Ob.rItemDef.nRoomLevel.." "..g_DialogManager:getString("STXT140TEXT"))
        Ob.rCantResearchText:setLoc(26, -256)
        Ob:addElement(Ob.rCantResearchText)
    elseif rHero:getStatAwareness() < Ob.rItemDef.nReqInt then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT103TEXT"))
        Ob.rCantResearchText:setLoc(26, -250)
        Ob:addElement(Ob.rCantResearchText)

        Ob.rAwarenessIcon = Ob:getUITextureProp('icon_int')
        Ob.rAwarenessIcon:setLoc(100, -238)
        Ob:addElement(Ob.rAwarenessIcon)

        Ob.rAwarenessText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rAwarenessText:setRect(0, 80, 150, 0)
        Ob.rAwarenessText:setString(tostring(Ob.rItemDef.nReqInt))
        Ob.rAwarenessText:setLoc(164, -250)
        Ob.rAwarenessText:setColor(1, 0, 0)
        Ob:addElement(Ob.rAwarenessText)
    elseif g_MiddleManager:isResearching(sItemKey) then
        Ob.bCanResearch = false
        
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 500, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT176TEXT"))
        Ob.rCantResearchText:setLoc(26, -254)
        Ob:addElement(Ob.rCantResearchText)
    else
        Ob.rTimeIcon = Ob:getUITextureProp('icon_time')
        Ob.rTimeIcon:setLoc(20, -250)
        Ob:addElement(Ob.rTimeIcon)

        Ob.rTimeText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rTimeText:setRect(0, 80, 150, 0)
        Ob.rTimeText:setString(util.convertTimeString(nDuration, true))
        Ob.rTimeText:setLoc(80, -256)
        Ob:addElement(Ob.rTimeText)
    end

    return Ob
end

function ResearchConsumableButton:new(rRenderLayer, sItemKey, rHero)
	local Ob = ConsumableDetailButton:new(rRenderLayer, sItemKey, true)

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rResearchButton then
            self:researchItem()
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self:canResearchItem() then
            self.rResearchButton:setVisible(bVisible)
        end
	end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self:canResearchItem() then
            self.rResearchButton:onFinger(eventType, sx, sy, tProps)
        end
		return true
	end

    function Ob:canResearchItem()
        return self.bCanResearch
    end

    function Ob:researchItem()
        if self:canResearchItem() then
            if self.rHero then
                self.rHero:addResearchActivityAtIndex(self.nActivityIndex, self.sItemKey)
            end
            g_GUI:SetActivePane(nil)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rHero = rHero
    Ob.sItemKey = sItemKey

    local rRoom = g_CurrentMap:getRoom("lab")
    local nRoomLevel = rRoom:getRoomDef().nRoomLevel
    if not nRoomLevel then
        nRoomLevel = 1
    end

    local nReqRoomLevel = 1
    if Ob.rItemDef.nRoomLevel then
        nReqRoomLevel = Ob.rItemDef.nRoomLevel
    end

    local nDuration = Ob.rItemDef.nResearchDuration or 60
    local rHighestLevelProp = rRoom:getHighestLevelAvailableProp()
    if rHighestLevelProp then
        local nResearchMult = rHighestLevelProp:getPropDef().researchSpeed
        if nResearchMult and nResearchMult > 0 then
            nDuration = nDuration / nResearchMult
        end
    end
    local nElapsed = g_MiddleManager:getResearchProgress(sItemKey) * nDuration
    nDuration = nDuration - nElapsed
    if nDuration < 0 then
        nDuration = 0
    end

    Ob.rResearchButton = Button:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT104TEXT", 'ptsans', 26, { 80, -2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
    Ob.rResearchButton:setLoc(306, -152)
    Ob.rResearchButton:setParent(Ob)

    Ob.bCanResearch = true
    if nReqRoomLevel > nRoomLevel then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT102TEXT").." "..Ob.rItemDef.nRoomLevel.." "..g_DialogManager:getString("STXT140TEXT"))
        Ob.rCantResearchText:setLoc(26, -160)
        Ob:addElement(Ob.rCantResearchText)
    elseif rHero:getStatAwareness() < Ob.rItemDef.nReqInt then
        Ob.bCanResearch = false
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 300, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT103TEXT"))
        Ob.rCantResearchText:setLoc(26, -160)
        Ob:addElement(Ob.rCantResearchText)

        Ob.rAwarenessIcon = Ob:getUITextureProp('icon_int')
        Ob.rAwarenessIcon:setLoc(100, -142)
        Ob:addElement(Ob.rAwarenessIcon)

        Ob.rAwarenessText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rAwarenessText:setRect(0, 80, 150, 0)
        Ob.rAwarenessText:setString(tostring(Ob.rItemDef.nReqInt))
        Ob.rAwarenessText:setLoc(164, -160)
        Ob.rAwarenessText:setColor(1, 0, 0)
        Ob:addElement(Ob.rAwarenessText)
    elseif g_MiddleManager:isResearching(sItemKey) then
        Ob.bCanResearch = false
        
        Ob.rCantResearchText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
        Ob.rCantResearchText:setRect(0, 80, 500, 0)
        Ob.rCantResearchText:setColor(1, 0, 0)
        Ob.rCantResearchText:setString(g_DialogManager:getString("STXT176TEXT"))
        Ob.rCantResearchText:setLoc(26, -156)
        Ob:addElement(Ob.rCantResearchText)
    else
        Ob.rTimeIcon = Ob:getUITextureProp('icon_time')
        Ob.rTimeIcon:setLoc(20, -154)
        Ob:addElement(Ob.rTimeIcon)

        Ob.rTimeText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rTimeText:setRect(0, 80, 150, 0)
        Ob.rTimeText:setString(util.convertTimeString(nDuration))
        Ob.rTimeText:setLoc(80, -158)
        Ob:addElement(Ob.rTimeText)
    end

    if Ob.rCostIcon then
        Ob.rCostIcon:setVisible(false)
    end

    if Ob.rCostField then
        Ob.rCostField:setReveal(0)
    end

    return Ob
end

function ConsumableResearchSelectorUI:new(rRenderLayer, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:update()
        self.rListUI:update()
    end
    
    function Ob:onPaneClosed()
        self.rListUI:onPaneClosed()
    end

    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    -- create the equipment buttons
    Ob.tConsumableButtons = {}
    local tDefKeys = g_MiddleManager:getConsumableDefKeysForResearch()
    for i, sDefKey in ipairs(tDefKeys) do
        local rNewButton = ResearchConsumableButton:new(Ob.rRenderLayer, sDefKey, rHero, nActivityIndex)
        table.insert(Ob.tConsumableButtons, rNewButton)
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tConsumableButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

function EquipmentResearchSelectorUI:new(rRenderLayer, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:onPaneClosed()
        self.rListUI:onPaneClosed()
    end

    function Ob:update()
        self.rListUI:update()
    end
    
    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    -- create the equipment buttons
    Ob.tEquipmentButtons = {}
    local tDefKeys = g_MiddleManager:getEquipmentDefKeysForResearch()
    for i, sDefKey in ipairs(tDefKeys) do
        local rNewButton = ResearchEquipmentButton:new(Ob.rRenderLayer, sDefKey, rHero, nActivityIndex)
        table.insert(Ob.tEquipmentButtons, rNewButton)
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tEquipmentButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

function RecipesResearchSelectorUI:new(rRenderLayer, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rListUI:setVisible(bVisible)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rListUI:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:onPaneClosed()
        self.rListUI:onPaneClosed()
    end

    function Ob:update()
        self.rListUI:update()
    end
    
    function Ob:setLoc(x, y)
        self.rListUI:setAnchorLoc(x, y)
    end

    -- create the recipe buttons
    Ob.tRecipesButtons = {}
    local tDefKeys = g_MiddleManager:getMeteoriteDefKeysForResearch()
    for i, sDefKey in ipairs(tDefKeys) do
        local rNewButton = ResearchRecipesButton:new(Ob.rRenderLayer, sDefKey, rHero, nActivityIndex)
        table.insert(Ob.tRecipesButtons, rNewButton)
    end
    -- add to the list UI
    Ob.rListUI = ListUI:new(Ob.rRenderLayer, Ob.tRecipesButtons, Ob)
    Ob.rListUI:setParent(Ob)
    Ob.rListUI:setAnchorLoc(0, 0)

    return Ob
end

function ResearchCategoryButton:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)    

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onSelected(bSelected)
        self.rUnselectedButton:setVisible(not bSelected)
        self.rSelectedButton:setVisible(bSelected)
    end

    function Ob:hasElement(rElement)
        local tElements = self:getElements()
        if tElements then
            for i, rExistingElement in ipairs(tElements) do
                if rExistingElement == rElement then
                    return true
                end
            end
        end
        return false
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rUnselectedButton = Ob:getUITextureProp('research_slot_default')
    Ob.rUnselectedButton:setLoc(0, 0)
    Ob:addElement(Ob.rUnselectedButton)
    
    Ob.rSelectedButton = Ob:getUITextureProp('research_slot_focused')
    Ob.rSelectedButton:setLoc(0, 0)
    Ob.rSelectedButton:setVisible(false)
    Ob:addElement(Ob.rSelectedButton)    

    return Ob
end

function ResearchCategoryPane:new(rRenderLayer, rParentUI)
	local Ob = UIElement:new(rRenderLayer)    
    Ob.rParentUI = rParentUI

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rItemsButton:setVisible(bVisible)
        self.rEquipmentButton:setVisible(bVisible)
        self.rRecipesButton:setVisible(bVisible)
        if bVisible then
            local nPri = self.rEquipmentButton:getElementsMaxPriority()
            self.rItemButtonTape:setPriority(nPri + 4)
            self.rItemsButtonText:setPriority(nPri + 5)
            self.rEquipmentButtonTape:setPriority(nPri + 6)
            self.rEquipmentButtonText:setPriority(nPri + 7)
            self.rRecipesButtonTape:setPriority(nPri + 8)
            self.rRecipesButtonText:setPriority(nPri + 9)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rItemsButton:onPaneClosed()
        self.rEquipmentButton:onPaneClosed()
        self.rRecipesButton:onPaneClosed()
    end

    function Ob:selectCategory(sCategoryName)
        if sCategoryName == 'Consumables' then
            self.rItemsButton:onSelected(true)
            self.rEquipmentButton:onSelected(false)
            self.rRecipesButton:onSelected(false)
        elseif sCategoryName == 'Equipment' then
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(true) 
            self.rRecipesButton:onSelected(false)
        elseif sCategoryName == 'Recipes' then
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false) 
            self.rRecipesButton:onSelected(true)
        else 
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)  
            self.rRecipesButton:onSelected(false) 
        end
        self.sCategoryName = sCategoryName
        self.rParentUI:onCategorySelected(sCategoryName)        
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if self.rItemsButton:hasElement(rProp) then
                        g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                        self:selectCategory('Consumables')
                        return true
                    elseif self.rEquipmentButton:hasElement(rProp) then
                        g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                        self:selectCategory('Equipment')
                        return true
                    elseif self.rRecipesButton:hasElement(rProp) then
                        g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
                        self:selectCategory('Recipes')
                        return true
                    end
                end
            end
        end
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rItemsButton = ResearchCategoryButton:new(rRenderLayer)
    Ob.rItemsButton:setLoc(8, -8)
    Ob.rItemsButton:setParent(Ob)

    Ob.rEquipmentButton = ResearchCategoryButton:new(rRenderLayer)
    Ob.rEquipmentButton:setLoc(8, -90)
    Ob.rEquipmentButton:setParent(Ob)
    
    Ob.rRecipesButton = ResearchCategoryButton:new(rRenderLayer)
    Ob.rRecipesButton:setLoc(8, -172)
    Ob.rRecipesButton:setParent(Ob)

    Ob.rItemsButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rItemsButtonText:setString("ITEMS")
    Ob.rItemsButtonText:setLoc(350, -30)   
    Ob:addElement(Ob.rItemsButtonText)

    Ob.rItemButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rItemButtonTape:setLoc(30, -30) 
    Ob.rItemButtonTape:setScl(0.8, 1)
    Ob:addElement(Ob.rItemButtonTape)

    Ob.rEquipmentButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rEquipmentButtonText:setString("EQUIPMENT")
    Ob.rEquipmentButtonText:setLoc(350, -112)
    Ob:addElement(Ob.rEquipmentButtonText)

    Ob.rEquipmentButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rEquipmentButtonTape:setLoc(30, -112) 
    Ob.rEquipmentButtonTape:setScl(1.2, 1)
    Ob:addElement(Ob.rEquipmentButtonTape)
    
    Ob.rRecipesButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRecipesButtonText:setString("Recipes")
    Ob.rRecipesButtonText:setLoc(350, -194)
    Ob:addElement(Ob.rRecipesButtonText)

    Ob.rRecipesButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rRecipesButtonTape:setLoc(30, -194) 
    Ob.rRecipesButtonTape:setScl(.9, 1)
    Ob:addElement(Ob.rRecipesButtonTape)

    Ob.sCategoryName = 'Consumables'

    return Ob
end

function ResearchUI:new(rRenderLayer, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rResearchCategoryPane:setVisible(bVisible)
        self.rEquipmentPane:setVisible(bVisible)
        self.rConsumablesPane:setVisible(bVisible)
        self.rRecipesPane:setVisible(bVisible)
        if bVisible then
            self.rResearchCategoryPane:selectCategory('Consumables')
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rResearchCategoryPane:onPaneClosed()
        self.rEquipmentPane:onPaneClosed()
        self.rConsumablesPane:onPaneClosed()
        self.rRecipesPane:onPaneClosed()
    end

    function Ob:onBackButtonPressed(bPressed)
        self.rBackButton:setVisible(not bPressed)
        self.rBackPressedButton:setVisible(bPressed)
    end

    function Ob:update()
        if self.sCategoryName == 'Equipment' then
            self.rEquipmentPane:update()
        elseif self.sCategoryName == 'Consumables' then
            self.rConsumablesPane:update()
        elseif self.sCategoryName == 'Recipes' then
            self.rRecipesPane:update()
        end
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_DOWN then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackButton then
                        self:onBackButtonPressed(true)
                    end
                end
            elseif eventType == MOAITouchSensor.TOUCH_UP then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackPressedButton then
                        self:onBackButtonPressed(false)
                        if g_Game:isActionEnabled('CloseWindow') then
                            g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                            g_GUI:SetActivePane(nil)
                        end
                    end
                end
            end
        end
        self.rResearchCategoryPane:onFinger(eventType, sx, sy, tProps)
        if self.sCategoryName == 'Equipment' then
            self.rEquipmentPane:onFinger(eventType, sx, sy, tProps)
        elseif self.sCategoryName == 'Consumables' then
            self.rConsumablesPane:onFinger(eventType, sx, sy, tProps)
        elseif self.sCategoryName == 'Recipes' then
            self.rRecipesPane:onFinger(eventType, sx, sy, tProps)
        end

        return true
    end

    function Ob:onCategorySelected(sCategoryName)
        if self.sCategoryName ~= sCategoryName then
            if sCategoryName == 'Equipment' then
                self.rEquipmentPane:setVisible(true)
                self.rConsumablesPane:setVisible(false)
                self.rRecipesPane:setVisible(false)
            elseif sCategoryName == 'Consumables' then
                self.rConsumablesPane:setVisible(true)
                self.rEquipmentPane:setVisible(false)
                self.rRecipesPane:setVisible(false)
            elseif sCategoryName == 'Recipes' then
                self.rConsumablesPane:setVisible(false)
                self.rEquipmentPane:setVisible(false)
                self.rRecipesPane:setVisible(true)
            end
        end
        self.sCategoryName = sCategoryName
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    -- add the bottom BG
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())                 
	Ob.rBackground:setColor(.47, .58, .36)
    Ob:addElement(Ob.rBackground)

    Ob.rLeftBG = MOAIProp2D.new()
	Ob.rLeftBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rLeftBG:setLoc(0, 0)
    Ob.rLeftBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())                 
	Ob.rLeftBG:setColor(.9, .95, .85)
    Ob:addElement(Ob.rLeftBG)

    Ob.rItemPaneBG = MOAIProp2D.new()
	Ob.rItemPaneBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rItemPaneBG:setLoc(Renderer.getUIHalfWidth(), -92)
    Ob.rItemPaneBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())
	Ob.rItemPaneBG:setColor(.47, .58, .36)
    Ob:addElement(Ob.rItemPaneBG)

    Ob.rTopBar = MOAIProp2D.new()
	Ob.rTopBar:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBar:setLoc(0, 0)
    Ob.rTopBar:setScl(Renderer.getUIWidth(), 92)                 
	Ob.rTopBar:setColor(.65, .79, .5)
    Ob:addOverlayElement(Ob.rTopBar)

	Ob.rTitleText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
	Ob.rTitleText:setString(g_DialogManager:getString("STXT069TEXT"))
    Ob.rTitleText:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTitleText:setColor(.9, .95, .85)
	Ob.rTitleText:setLoc(0, -14)
    Ob:addOverlayElement(Ob.rTitleText)

    -- add the back button
    Ob.rBackButton = Ob:getUITextureProp('back_button')
    Ob.rBackButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackButton)

    Ob.rBackPressedButton = Ob:getUITextureProp('back_button_tap')
    Ob.rBackPressedButton:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rBackPressedButton)  

    Ob.rResearchCategoryPane = ResearchCategoryPane:new(rRenderLayer, Ob)
    Ob.rResearchCategoryPane:setLoc(Renderer.getUIHalfWidth() - 480, -90)
    Ob.rResearchCategoryPane:setParent(Ob)

    Ob.rEquipmentPane = EquipmentResearchSelectorUI:new(rRenderLayer, rHero)
    Ob.rEquipmentPane:setLoc(Renderer.getUIHalfWidth(), -90)
    Ob.rEquipmentPane:setParent(Ob)

    Ob.rConsumablesPane = ConsumableResearchSelectorUI:new(rRenderLayer, rHero, nActivityIndex)
    Ob.rConsumablesPane:setLoc(Renderer.getUIHalfWidth(), -90)
    Ob.rConsumablesPane:setParent(Ob)
    
    Ob.rRecipesPane = RecipesResearchSelectorUI:new(rRenderLayer, rHero, nActivityIndex)
    Ob.rRecipesPane:setLoc(Renderer.getUIHalfWidth(), -90)
    Ob.rRecipesPane:setParent(Ob)

    Ob.sActiveTabName = nil
    Ob.sCategoryName = nil    

    Ob:onBackButtonPressed(false)

    return Ob
end

return ResearchUI