local ResourceBarUI = {}
local util = require('DFUtil')
local DFGraphics = require('DFCommon.Graphics')
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local Button = require('ui/Button')

local tUITextureSpriteSheetInfo =
{
    resourcebar = 'UI/hud2',
    plus_button = 'UI/hud2',
}

function ResourceBarUI:new(rRenderLayer)
    local Ob = UIElement:new(rRenderLayer)
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rBGTexture = Ob:getUITextureProp('resourcebar')
    Ob.rBGTexture:setLoc(0, 0)
    if Ob.rBGTexture then
        Ob:addOverlayElement(Ob.rBGTexture)
    end

    local tCashColor = g_Game:getCashColor()
    -- Money Text
    Ob.rCashText = g_GUI:CreateTextBox(38, MOAITextBox.LEFT_JUSTIFY, 'type')
	Ob.rCashText:setString(tostring(g_Game:getCurCash()))
    Ob.rCashText:setRect(0, 0, 180, 50)
	Ob.rCashText:setLoc(70, -68)
	Ob.rCashText:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
	Ob:addOverlayElement(Ob.rCashText)

	-- Prestige Text
	Ob.rPPText = g_GUI:CreateTextBox(38, MOAITextBox.LEFT_JUSTIFY, 'type')
	Ob.rPPText:setString(tostring(g_Game:getCurPP()))
	Ob.rPPText:setRect(0, 0, 180, 50)
	Ob.rPPText:setLoc(390, -68)
	Ob.rPPText:setColor(.24, .3, .57)	
	Ob:addOverlayElement(Ob.rPPText)

    Ob.rRMTButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'plus_button', nil, { 1, 1, 1 }, { 0.5, 0.5, 0.5 })
    Ob.rRMTButton:setLoc(520, -18)
    Ob.rRMTButton:setParent(Ob)

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rRMTButton:setVisible(bVisible)
        self.nCurUIPP = g_Game:getCurPP()
        self.nCurUICash = g_Game:getCurCash()    
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

	function Ob:update()
		if self.rPPText then
            if self.nCurUIPP ~= g_Game:getCurPP() then
                if self.nCurUIPP > g_Game:getCurPP() then
                    self.nCurUIPP = self.nCurUIPP - 1
                else
                    self.nCurUIPP = self.nCurUIPP + 1
                end
            end
            self.rPPText:setString(tostring(self.nCurUIPP))
		end
		if self.rCashText then
            if self.nCurUICash ~= g_Game:getCurCash() then
                local diff = math.abs(self.nCurUICash - g_Game:getCurCash())
                local inc = 1
                inc = math.floor(math.max(diff/10,1))
                if self.nCurUICash > g_Game:getCurCash() then
                    self.nCurUICash = self.nCurUICash - inc
                else
                    self.nCurUICash = self.nCurUICash + inc
                end      
            end
            self.rCashText:setString(tostring(self.nCurUICash))
		end
	end    

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rRMTButton:onFinger(eventType, sx, sy, tProps)
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rRMTButton then
                g_GUI:ShowRMTUI()
            end
        end
    end

    Ob.nCurUIPP = g_Game:getCurPP()
    Ob.nCurUICash = g_Game:getCurCash()
    
    return Ob
end

return ResourceBarUI
