local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local MiddleManager = require('MiddleManager')
local TerritoryManager = require('Territory')
local RewardScreenUI = {}

local tUITextureSpriteSheetInfo =
{
enveope_closed = 'UI/reward2',
enveope_open = 'UI/reward',
reward_paper = 'UI/reward',
icon_cash = 'UI/hud',
icon_pp = 'UI/hud',
}

function RewardScreenUI:new(rRenderLayer, rRewardInfo, closeCB, rTerritory)
    local Ob = UIElement:new(rRenderLayer)
    
    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if not self.bOpened then
                self:openEnvelope()
            else
                g_GUI:HideRewardPane()
                if self.closeCB then
                    self.closeCB()
                end
            end
        end
        return true
    end

    function Ob:openEnvelope()
        g_Audio:playCue('MMOJ_Audio/UI/FolderTab')
        self.bOpened = true

        self.rEnvelopeClosed:setVisible(false)
        self.rEnvelopeClosedText:setReveal(0)

        self.rEnvelopeOpened:setVisible(true)
        self.rRewardSheet:setVisible(true)
        self.rRewardDescriptionText:revealAll()

        self.rLetterBodyText:revealAll()

        if self.rIcon then
            self.rIcon:setVisible(true)
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob.rRewardInfo = rRewardInfo
    Ob.bOpened = false
    Ob.closeCB = closeCB
    
    if not Ob.rRewardInfo then
        return Ob
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)    
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(80/255, 80/255, 80/255)
    Ob:addElement(Ob.rBackground)    

    Ob.rEnvelopeClosed = Ob:getUITextureProp('enveope_closed')
    Ob.rEnvelopeClosed:setLoc(Renderer.getUIHalfWidth() - 280, -180)
    Ob:addElement(Ob.rEnvelopeClosed)

    Ob.rEnvelopeClosedText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rEnvelopeClosedText:setRect(-300, 80, 300, 0)
    Ob.rEnvelopeClosedText:setString(g_DialogManager:getString("COMM002TEXT"))
	Ob.rEnvelopeClosedText:setLoc(Renderer.getUIHalfWidth() + 110, -400)
    Ob.rEnvelopeClosedText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rEnvelopeClosedText)

    Ob.rEnvelopeOpened = Ob:getUITextureProp('enveope_open')
    Ob.rEnvelopeOpened:setLoc(20, -180)
    Ob.rEnvelopeOpened:setRot(20)
    Ob.rEnvelopeOpened:setVisible(false)
    Ob:addElement(Ob.rEnvelopeOpened)

    Ob.rRewardSheet = Ob:getUITextureProp('reward_paper')
    Ob.rRewardSheet:setLoc(280, 0)
    Ob.rRewardSheet:setVisible(false)
    Ob:addElement(Ob.rRewardSheet)
 
    local sPositionText = "RANKS001TEXT" -- default
    local sLetterText = "COMM001TEXT"
    local nMMLevel = g_MiddleManager:getLevel()
    if MiddleManager.tLevelToTitle[nMMLevel] then
        sPositionText = MiddleManager.tLevelToTitle[nMMLevel]
    end
       
    if rTerritory and rTerritory:getDef().sRewardText then
        local rTerDef = rTerritory:getDef()
        
        sLetterText = rTerDef.sRewardText
    end
    Ob.rLetterBodyText = g_GUI:CreateTextBox(22, MOAITextBox.LEFT_JUSTIFY, 'type')
    
    local sMessage = "    "..g_DialogManager:getString(sLetterText).." "..g_DialogManager:getString(sPositionText).."."

    local sShowNewAreaMessage = true ~= g_City:areAllTerritoriesUnlocked()
    
    if sShowNewAreaMessage then
        sMessage = sMessage .. g_DialogManager:getString("COMM009TEXT")
    end
    
    Ob.rLetterBodyText:setString(sMessage)
    Ob.rLetterBodyText:setRect(0, 530, 470, 0)
	Ob.rLetterBodyText:setLoc(325, -110)
    Ob.rLetterBodyText:setColor(0, 0, 0)
    Ob.rLetterBodyText:setReveal(0)
	Ob:addOverlayElement(Ob.rLetterBodyText)

    Ob.rRewardDescriptionText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rRewardDescriptionText:setRect(-150, 80, 150, 0)
	Ob.rRewardDescriptionText:setLoc(555, -405)
    Ob.rRewardDescriptionText:setColor(0, 0, 0)
    Ob.rRewardDescriptionText:setReveal(0)
	Ob:addOverlayElement(Ob.rRewardDescriptionText)

    if Ob.rRewardInfo.type == 'cash' then
        Ob.rIcon = Ob:getUITextureProp('icon_cash')
        Ob.rIcon:setLoc(535, -350)
        Ob:addElement(Ob.rIcon)
        Ob.rRewardDescriptionText:setString('+'..tostring(Ob.rRewardInfo.nAmount))
    elseif Ob.rRewardInfo.type == 'pp' then
        Ob.rIcon = Ob:getUITextureProp('icon_pp')
        Ob.rIcon:setLoc(535, -350)
        Ob:addElement(Ob.rIcon)
        Ob.rRewardDescriptionText:setString('+'..tostring(Ob.rRewardInfo.nAmount))
    elseif Ob.rRewardInfo.type == 'consumable' then
        Ob.rIcon = Ob:getUITextureProp(Ob.rRewardInfo.rItemDef.iconName)
        if Ob.rIcon then
            Ob.rIcon:setLoc(490, -290)
            Ob:addElement(Ob.rIcon)
        end
        Ob.rRewardDescriptionText:setString(g_DialogManager:getString(Ob.rRewardInfo.rItemDef.sName))
    end
    if Ob.rIcon then
        Ob.rIcon:setVisible(false)
    end

    Ob.rRewardDescriptionText:setReveal(0)

    return Ob
end

return RewardScreenUI