-------- ROOM DETAILS ---------
local util = require('DFUtil')
local ResourceBarUI = require('ui/ResourceBarUI')
local RoomUpgradeButton = require('ui/RoomUpgradeButton')
local BaseUpgradeButton = require('ui/BaseUpgradeButton')
local PropUpgradeButton = require('ui/PropUpgradeButton')
local DFAnalytics = require('DFCommon.Analytics')
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local ListUI = require('ui/ListUI')
local DFAchievement = require('DFCommon.Achievement')
local ShopButton = require('ui/ShopButton')
local Button = require('ui/Button')
local NewRoomButton = {}
local NewRoomUI = {}
local RoomDetailsUI = {}
local RoomUpgradeUI = {}
local BaseUpgradeUI = {}

local tUITextureSpriteSheetInfo =
{
    room_panel = 'UI/hud3',
    icon_research = 'UI/hud',
    icon_rest = 'UI/hud',
    icon_timeoff = 'UI/hud',
    icon_train = 'UI/hud',
    icon_train_ability = 'UI/hud',
    icon_parttimejob = 'UI/hud',
    button_buy_disabled = 'UI/shop',
    popup_button = 'UI/hud',
    popup_activated = 'UI/hud',
    justice_icon = 'UI/hud2',
}

local tNewRoomButtonTextureSpriteSheetInfo =
{
    blueprint = 'UI/hud3',
    button_buy = 'UI/hud',
    icon_room_gym = 'UI/hud3',
    icon_room_lab = 'UI/hud3',
    icon_room_parttimejob = 'UI/hud3',
    icon_room_rest = 'UI/hud3',
    icon_room_vacation = 'UI/hud3',
    icon_room_ability = 'UI/hud3',
    shop_locked = 'UI/hud',
    button_buy_disabled = 'UI/shop',
    popup_button = 'UI/hud',
    popup_activated = 'UI/hud',
    icon_cash = 'UI/hud',
}

function NewRoomButton:new(rRenderLayer, rRoomDef, sRoomDefKey)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rButton:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rButton:onPaneClosed()
    end

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rButton then
            self.rParentUI:handleButtonPress(self)
        end
    end

    function Ob:canBuy()
        if g_Game:getCurCash() >= self.rDef.upgradeCost and g_MiddleManager:getStarRatingLevel() >= self.rDef.mmlvl then
            return true
        end
        return false
    end

    function Ob:getRoomDef()
        return self.rDef
    end

    function Ob:getRoomDefKey()
        return self.sRoomDefKey
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rButton:onFinger(eventType, sx, sy, tProps)
        return true
    end

	function Ob:getYSize()
		return 240
	end

    function Ob:update()
        self.rButton:update()
    end

    Ob.rDef = rRoomDef
    Ob.sRoomDefKey = sRoomDefKey
    Ob.rParentUI = nil
    Ob:loadUISpriteSheets(tNewRoomButtonTextureSpriteSheetInfo)

    Ob.rBackground = Ob:getUITextureProp('blueprint')
    Ob.rBackground:setScl(1.2)
    Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)    

    if Ob.rDef.newRoomIconPath then
        Ob.rActivityIcon = Ob:getUITextureProp(Ob.rDef.newRoomIconPath)
        Ob.rActivityIcon:setScl(0.9)
        Ob.rActivityIcon:setLoc(42, -26)
        a = Ob.rActivityIcon
        Ob:addElement(Ob.rActivityIcon)
    end

    Ob.rRoomNameText = g_GUI:CreateTextBox(36, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rRoomNameText:setRect(0, 80, 300, 0)
    Ob.rRoomNameText:setColor(1, 1, 1)
    Ob.rRoomNameText:setString(g_DialogManager:getString(Ob.rDef.visualName))
	Ob.rRoomNameText:setLoc(164, -20)
	Ob:addElement(Ob.rRoomNameText)

    Ob.rRoomDescriptionText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRoomDescriptionText:setString(g_DialogManager:getString(Ob.rDef.visualDescription))
    Ob.rRoomDescriptionText:setRect(0, 80, 300, 0)
    Ob.rRoomDescriptionText:setColor(1, 1, 1)
	Ob.rRoomDescriptionText:setLoc(164, -60)
	Ob:addElement(Ob.rRoomDescriptionText)

    Ob.rButton = ShopButton:new(rRenderLayer, Ob, tNewRoomButtonTextureSpriteSheetInfo, 'button_buy', nil, {.36, .55, .94}, nil, true, { 0, -50 }, "STXT008TEXT", 'ptsans', 28, { 80, 2 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.rDef.upgradeCost, nil, { .57, .57, .57 })
    Ob.rButton:setLoc(328, -150)
    Ob.rButton:setParent(Ob)
    
    Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
    Ob.rCostIcon:setLoc(60, -154)
    Ob:addElement(Ob.rCostIcon)

    Ob.rCostField = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY)
    Ob.rCostField:setString(tostring(Ob.rDef.upgradeCost))
    Ob.rCostField:setLoc(430, -160)
    Ob.rCostField:setColor(1, 1, 1)
    Ob:addElement(Ob.rCostField)

    return Ob
end

function BaseUpgradeUI:new(rRenderLayer, rRoom)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:setVisible(bVisible)
        end
	end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:update()
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:update()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            local bTouchedSomething = false
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground then
                        bTouchedSomething = true
                        break
                    end
                end
            end
            if not bTouchedSomething then
                g_GUI:HideRoomDetailsUI()
                return true
            end
        end
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:onBaseUpgraded()
        g_GUI:HideRoomDetailsUI()
    end

    function Ob:addUpgradePane()
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:setVisible(false)
            self.rRoomUpgradeButton = nil
        end
        local rBaseUpgradeDef = self.rRoom:getBaseUpgradeDef()
        if rBaseUpgradeDef then
            self.rRoomUpgradeButton = BaseUpgradeButton:new(rRenderLayer, self.rRoom, rBaseUpgradeDef)
            self.rRoomUpgradeButton:setLoc(33, -236)
            self.rRoomUpgradeButton:setParent(self)
            self.rRoomUpgradeButton:setParentUI(self)
        end
    end

    Ob.rRoom = rRoom

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)    

	Ob.rBackground = Ob:getUITextureProp('room_panel')
	Ob.rBackground:setLoc(0, 0)
	Ob:addElement(Ob.rBackground)

    Ob.rIcon = Ob:getUITextureProp('justice_icon')
    Ob.rIcon:setLoc(355, -140)
    Ob.rIcon:setScl(0.65)
    Ob:addElement(Ob.rIcon)

    Ob.rRoomNameText = g_GUI:CreateTextBox(50, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rRoomNameText:setRect(0, 80, 300, 0)
    Ob.rRoomNameText:setString(string.upper(g_DialogManager:getString(rRoom:getName())))
	Ob.rRoomNameText:setLoc(40, -104)
	Ob:addElement(Ob.rRoomNameText)

    Ob.rRoomDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rRoomDescriptionText:setString(string.upper(g_DialogManager:getString(rRoom:getDescription())))
    Ob.rRoomDescriptionText:setRect(0, 120, 300, 0)
	Ob.rRoomDescriptionText:setLoc(40, -156)
	Ob:addElement(Ob.rRoomDescriptionText)

    Ob.rRoomLabelText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRoomLabelText:setRect(0, 80, 300, 0)
    Ob.rRoomLabelText:setString(g_DialogManager:getString("STXT010TEXT"))
	Ob.rRoomLabelText:setLoc(536, -108)
    Ob.rRoomLabelText:setRot(0, 0, -90)
	Ob:addElement(Ob.rRoomLabelText)

    Ob:addUpgradePane()

    return Ob
end

function RoomUpgradeUI:new(rRenderLayer, rRoom)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:setVisible(bVisible)
        end
        if self.rProp1UpgradePane then
            self.rProp1UpgradePane:setVisible(bVisible)
        end
        if self.rProp2UpgradePane then
            self.rProp2UpgradePane:setVisible(bVisible)
        end
        if self.rProp3UpgradePane then
            self.rProp3UpgradePane:setVisible(bVisible)
        end
        if self.rProp4UpgradePane then
            self.rProp4UpgradePane:setVisible(bVisible)
        end
	end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:onPaneClosed()
        end
        if self.rProp1UpgradePane then
            self.rProp1UpgradePane:onPaneClosed()
        end
        if self.rProp2UpgradePane then
            self.rProp2UpgradePane:onPaneClosed()
        end
        if self.rProp3UpgradePane then
            self.rProp3UpgradePane:onPaneClosed()
        end
        if self.rProp4UpgradePane then
            self.rProp4UpgradePane:onPaneClosed()
        end
    end

    function Ob:update()
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:update()
        end
        if self.rProp1UpgradePane then
            self.rProp1UpgradePane:update()
        end
        if self.rProp2UpgradePane then
            self.rProp2UpgradePane:update()
        end
        if self.rProp3UpgradePane then
            self.rProp3UpgradePane:update()
        end
        if self.rProp4UpgradePane then
            self.rProp4UpgradePane:update()
        end       
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            local bTouchedSomething = false
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground then
                        bTouchedSomething = true
                        break
                    end
                end
            end
            if not bTouchedSomething then
                g_GUI:HideRoomDetailsUI()
                return true
            end
        end
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:onFinger(eventType, sx, sy, tProps)
        end
        if self.rProp1UpgradePane then
            self.rProp1UpgradePane:onFinger(eventType, sx, sy, tProps)
        end
        if self.rProp2UpgradePane then
            self.rProp2UpgradePane:onFinger(eventType, sx, sy, tProps)
        end
        if self.rProp3UpgradePane then
            self.rProp3UpgradePane:onFinger(eventType, sx, sy, tProps)
        end
        if self.rProp4UpgradePane then
            self.rProp4UpgradePane:onFinger(eventType, sx, sy, tProps)
        end
        return true
    end

    function Ob:addUpgradePane()
        if self.rRoomUpgradeButton then
            self.rRoomUpgradeButton:setVisible(false)
            self.rRoomUpgradeButton = nil
        end
        local tUpgradeRoomDefKeys = self.rRoom:getRoomUpgradeDefs()
        if tUpgradeRoomDefKeys then
            for i, sRoomDefKey in ipairs(tUpgradeRoomDefKeys) do
                local rDef = g_CurrentMap:getRoomDef(sRoomDefKey)
                if rDef.bCanHaveMultiple or g_CurrentMap:getRoom(rDef.roomtype) == nil or rDef.bIsRoomUpgrade then
                    self.rRoomUpgradeButton = RoomUpgradeButton:new(rRenderLayer, self.rRoom, rDef, sRoomDefKey)
                    self.rRoomUpgradeButton:setLoc(33, -236)
                    self.rRoomUpgradeButton:setParent(self)
                    self.rRoomUpgradeButton:setParentUI(self)
                end
            end
        end
    end

    function Ob:addUpgradeButton(rProp)
        local rButton = PropUpgradeButton:new(rRenderLayer, rProp, self)
        rButton:setParent(self)
        return rButton
    end

    function Ob:onRoomUpgraded(rNewRoom)
        -- close the pane
        g_GUI:HideRoomDetailsUI()      
    end

    function Ob:onPropUpgraded(rPane)
        -- close the pane
        g_GUI:HideRoomDetailsUI()             
    end

    Ob.rRoom = rRoom

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)    

	Ob.rBackground = Ob:getUITextureProp('room_panel')
	Ob.rBackground:setLoc(0, 0)
	Ob:addElement(Ob.rBackground)

    Ob.rRoomNameText = g_GUI:CreateTextBox(50, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rRoomNameText:setRect(0, 80, 300, 0)
    Ob.rRoomNameText:setString(string.upper(g_DialogManager:getString(rRoom:getName())))
	Ob.rRoomNameText:setLoc(40, -104)
	Ob:addElement(Ob.rRoomNameText)

    Ob.rRoomDescriptionText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rRoomDescriptionText:setString(string.upper(g_DialogManager:getString(rRoom:getDescription())))
    Ob.rRoomDescriptionText:setRect(0, 80, 300, 0)
	Ob.rRoomDescriptionText:setLoc(40, -156)
	Ob:addElement(Ob.rRoomDescriptionText)

    Ob.rRoomLabelText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRoomLabelText:setRect(0, 80, 300, 0)
    Ob.rRoomLabelText:setString(g_DialogManager:getString("STXT009TEXT"))
	Ob.rRoomLabelText:setLoc(536, -108)
    Ob.rRoomLabelText:setRot(0, 0, -90)
	Ob:addElement(Ob.rRoomLabelText)


    Ob.rIcon = Ob:getUITextureProp('justice_icon')
    Ob.rIcon:setLoc(355, -140)
    Ob.rIcon:setScl(0.65)
    Ob:addElement(Ob.rIcon)
--[[
    Ob.rActivityIcon = Ob:getUITextureProp(rRoom:getActivityIcon())
    Ob.rActivityIcon:setLoc(352, -124)
    Ob:addElement(Ob.rActivityIcon)
]]--
    Ob:addUpgradePane()

    local tProps = rRoom:getProps()    
    if tProps then
        if tProps[1] then
            Ob.rProp1UpgradePane = Ob:addUpgradeButton(tProps[1])
            Ob.rProp1UpgradePane:setLoc(32, -342)
        end
        if tProps[2] then
            Ob.rProp2UpgradePane = Ob:addUpgradeButton(tProps[2])
            Ob.rProp2UpgradePane:setLoc(266, -342)
        end
        if tProps[3] then
            Ob.rProp3UpgradePane = Ob:addUpgradeButton(tProps[3])
            Ob.rProp3UpgradePane:setLoc(32, -492)    
        end
        if tProps[4] then
            Ob.rProp4UpgradePane = Ob:addUpgradeButton(tProps[4])
            Ob.rProp4UpgradePane:setLoc(266, -492)
        end
    end

    return Ob
end

local tNewRoomUITextureSpriteSheetInfo =
{
    back_button = 'UI/hud',
    back_button_tap = 'UI/hud',    
}

function NewRoomUI:new(rRenderLayer, rRoom)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:handleItemSelection(rButton)
        if rButton and rButton:canBuy() then
            local rDef = rButton:getRoomDef()
            if rDef then
                -- create room
                if rDef.upgradeCost then
                    g_Game:removeCash(rDef.upgradeCost)
                    DFAnalytics.incrStat("csh_spnt_nwrm", rDef.upgradeCost)
                end
                DFAnalytics.logEvent("buy_nwrm", { type = rButton:getRoomDefKey() } )
                DFAchievement.updateAchievement('BoughtAllRooms', rButton:getRoomDefKey())
                g_CurrentMap:createRoomAtRoom(self.rRoom, rDef, rButton:getRoomDefKey())
                g_TodoList:onNewRoomAdded()
                g_GUI:HideRoomDetailsUI()             
            end
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rLeftListUI:setVisible(bVisible)
        self.rRightListUI:setVisible(bVisible)
        self.rBackButton:setVisible(bVisible)
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rLeftListUI:onPaneClosed()
        self.rRightListUI:onPaneClosed()
        self.rBackButton:onPaneClosed()
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if g_Game:isActionEnabled('CloseWindow') then
                    g_GUI:HideRoomDetailsUI()
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rBackButton:onFinger(eventType, sx, sy, tProps)
        self.rLeftListUI:onFinger(eventType, sx, sy, tProps)
        self.rRightListUI:onFinger(eventType, sx, sy, tProps)

        return true
    end

    function Ob:update()
        self.rLeftListUI:update()
        self.rRightListUI:update()
    end

    Ob:loadUISpriteSheets(tNewRoomUITextureSpriteSheetInfo)

    Ob.rRoom = rRoom
    Ob.rCurSelectedItem = nil

	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setColor(.4, .4, .4, .5)
	Ob:addElement(Ob.rBackground)

    Ob.rTopBG = MOAIProp2D.new()
	Ob.rTopBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBG:setLoc(0, 0)
	Ob.rTopBG:setScl(Renderer.getUIWidth(), 90)
	Ob.rTopBG:setColor(0, 0, 0)
    Ob.rTopBG:setParent(Ob)
    Ob:addOverlayElement(Ob.rTopBG)

    -- add the back button
    Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tNewRoomUITextureSpriteSheetInfo, 'back_button', 'back_button_tap')
    Ob.rBackButton:setLoc(20, 0)
    Ob.rBackButton:setParent(Ob)

    Ob.rTextBox = g_GUI:CreateTextBox(50, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rTextBox:setString(g_DialogManager:getString("STXT005TEXT"))
    Ob.rTextBox:setRect(0, 80, Renderer.getUIWidth(), 0)
	Ob.rTextBox:setLoc(0, -16)
	Ob.rTextBox:setColor(1, 1, 1)	
	Ob:addOverlayElement(Ob.rTextBox)

    local tLeftColItemDefKeys = {}
    local tRightColItemDefKeys = {}
    local tAllRoomDefKeys = g_Game:getNewRoomKeyListOverride() or rRoom:getRoomUpgradeDefs()
    local tUpgradeRoomDefKeys = {}
    for i, sKey in ipairs(tAllRoomDefKeys) do
        local rDef = g_CurrentMap:getRoomDef(sKey)
        if not g_CurrentMap:getRoom(rDef.roomtype) then
            table.insert(tUpgradeRoomDefKeys, sKey)
        end
    end
    local nNumItems = table.getn(tUpgradeRoomDefKeys)
    if nNumItems > 0 then
        local nHalfNum = math.floor(nNumItems / 2)
        if nHalfNum > 0 then
            for i = 1, nHalfNum do
                table.insert(tLeftColItemDefKeys, tUpgradeRoomDefKeys[i])
            end
            for i = nHalfNum + 1, nNumItems do
                table.insert(tRightColItemDefKeys, tUpgradeRoomDefKeys[i])
            end
        else
            tLeftColItemDefKeys = tUpgradeRoomDefKeys
        end
    end

    local tLeftButtons = {}
    local tRightButtons = {}
    for i, sDefKey in ipairs(tLeftColItemDefKeys) do
        local rDef =  g_CurrentMap:getRoomDef(sDefKey)
        if rDef then
            local rButton = NewRoomButton:new(rRenderLayer, rDef, sDefKey)
            table.insert(tLeftButtons, rButton)
        end
    end
    for i, sDefKey in ipairs(tRightColItemDefKeys) do
        local rDef =  g_CurrentMap:getRoomDef(sDefKey)
        if rDef then
            local rButton = NewRoomButton:new(rRenderLayer, rDef, sDefKey)
            table.insert(tRightButtons, rButton)
        end
    end

    Ob.rLeftListUI = ListUI:new(rRenderLayer, tLeftButtons, Ob)
    Ob.rLeftListUI:setParent(Ob)
    Ob.rLeftListUI:setAnchorLoc(80, -110)
    Ob.rLeftListUI:setScl(0.75, 0.75)

    Ob.rRightListUI = ListUI:new(rRenderLayer, tRightButtons, Ob)
    Ob.rRightListUI:setParent(Ob)
    Ob.rRightListUI:setAnchorLoc(Renderer.getUIHalfWidth() + 30, -110)
    Ob.rRightListUI:setScl(0.75, 0.75)

    Ob.rLeftListUI:setScrollingEnabled(false)
    Ob.rRightListUI:setScrollingEnabled(false)

    return Ob
end

function RoomDetailsUI:new(rRenderLayer, rRoom)
    local Ob = UIElement:new(rRenderLayer)

    Ob.rRoom = rRoom
    Ob.rPane = nil
    if rRoom:isEmptyRoom() then
        Ob.rPane = NewRoomUI:new(rRenderLayer, rRoom)
        Ob.rPane:setLoc(0, 0)
    elseif rRoom:getBaseUpgradeDef() or rRoom:getRoomDef().baseDef then
        Ob.rPane = BaseUpgradeUI:new(rRenderLayer, rRoom)
        Ob.rPane:setLoc(0, -Renderer.getUIHeight() + 720)
    else        
        Ob.rPane = RoomUpgradeUI:new(rRenderLayer, rRoom)
        Ob.rPane:setLoc(0, -Renderer.getUIHeight() + 720)
    end
    Ob.rPane:setParent(Ob)

    -- the resource bar
    Ob.rResourceBar = ResourceBarUI:new(rRenderLayer)
    Ob.rResourceBar:setLoc(0, -Renderer.getUIHeight() + 74)
    Ob.rResourceBar:setParent(Ob)

    function Ob:update()
        if self.rResourceBar then
            self.rResourceBar:update()
        end
        self.rPane:update()
    end

	function Ob:setVisible(bVisible)
		if not self.rRenderLayer then
			return
		end
		if bVisible then
            self.rPane:setVisible(true)
            self.rResourceBar:setVisible(true)                        
        else
            self.rPane:setVisible(false)
            self.rResourceBar:setVisible(false)
        end
        g_Audio:playCue('MMOJ_Audio/UI/BaseFolder')
    end

    function Ob:onPaneClosed()
        self.rPane:onPaneClosed()
        self.rResourceBar:onPaneClosed()
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rPane:onFinger(eventType, sx, sy, tProps)
        return true
    end

	return Ob
end

return RoomDetailsUI