local util = require('DFUtil')
local UIElement = require('ui/UIElement')
local ShopButton = require('ui/ShopButton')
local DFAchievement = require('DFCommon.Achievement')
local DFAnalytics = require('DFCommon.Analytics')

local RoomUpgradeButton = {}

local tUITextureSpriteSheetInfo =
{
    room_upgrade_container = 'UI/hud3',
    room_upgrade_buy = 'UI/hud3',
    icon_cash = 'UI/hud',
}

function RoomUpgradeButton:new(rRenderLayer, rRoom, rRoomDef, sRoomDefKey)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:setParentUI(rUI)
        self.rParentUI = rUI
    end

    function Ob:getRoomDef()
        return self.rRoomDef
    end

    function Ob:getRoomDefKey()
        return self.sRoomDefKey
    end

    function Ob:canUpgrade()
        if g_Game:getCurCash() >= self.rRoomDef.upgradeCost and g_MiddleManager:getStarRatingLevel() >= self.rRoomDef.mmlvl then
            return true
        end
        return false
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rUpgradeButton then
            if self:canUpgrade() then
                self:upgradeRoom()
            end
        end
    end

    function Ob:update()
        self.rUpgradeButton:update()
    end
    
	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rUpgradeButton:onFinger(eventType, sx, sy, tProps)
		return true
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets() 
        self.rUpgradeButton:onPaneClosed()
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rUpgradeButton:setVisible(bVisible)
	end

    function Ob:upgradeRoom()
        g_Game:removeCash(self.rRoomDef.upgradeCost)
        g_CurrentMap:upgradeRoom(self.rRoom, self.rRoomDef, self.sRoomDefKey)
        DFAnalytics.logEvent('buy_upgr_room', { type = self.sRoomDefKey, cost = self.nCost } )
        DFAnalytics.incrStat('csh_spnt_room', self.nCost)
        if self.rParentUI then
            self.rParentUI:onRoomUpgraded(nNewRoom)
        end
        g_TodoList:onRoomUpgraded(self.rRoom)
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)    

    Ob.rRoomDef = rRoomDef
    Ob.sRoomDefKey = sRoomDefKey
    Ob.rRoom = rRoom

    Ob.rParentUI = nil

	Ob.rBackGround = Ob:getUITextureProp('room_upgrade_container')
    Ob.rBackGround:setLoc(0, 0)
	Ob:addElement(Ob.rBackGround)

    Ob.rNameField = g_GUI:CreateTextBox(26, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.rNameField:setString(string.upper(g_DialogManager:getString(rRoomDef.sUpgradeDesc)))
    Ob.rNameField:setLoc(10, -5)
    Ob.rNameField:setColor(1, 1, 1)
    Ob.rNameField:setRect(0, 80, 450, 0)
    Ob:addElement(Ob.rNameField)

    if Ob.rRoomDef.mmlvl > g_MiddleManager:getStarRatingLevel() then
        Ob.tReqText = g_GUI:CreateTextBox(26, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.tReqText:setString(tostring(g_DialogManager:getString("STXT183TEXT").." "..Ob.rRoomDef.mmlvl))
        Ob.tReqText:setLoc(15, -50)
        Ob.tReqText:setRect(0, 80, 300, 0)
        Ob.tReqText:setColor(.85, 0, 0)
        Ob:addElement(Ob.tReqText)
    else
        Ob.rCostIcon = Ob:getUITextureProp('icon_cash')
        Ob.rCostIcon:setScl(0.7)
        Ob.rCostIcon:setLoc(10, -50)
        Ob:addElement(Ob.rCostIcon)

        local tCashColor = g_Game:getCashColor()    
        Ob.nCost = rRoomDef.upgradeCost
        Ob.rCostField = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rCostField:setString(tostring(Ob.nCost))
        Ob.rCostField:setLoc(56, -50)
        Ob.rCostField:setRect(0, 80, 300, 0)
        Ob.rCostField:setColor(tCashColor[1], tCashColor[2], tCashColor[3])
        Ob:addElement(Ob.rCostField)	
    end

    Ob.rUpgradeButton = ShopButton:new(rRenderLayer, Ob, tUITextureSpriteSheetInfo, 'room_upgrade_buy', nil, {.94, .36, .36}, nil, true, { 0, -36 }, "STXT008TEXT", 'ptsans', 30, { 50, 4 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 }, Ob.nCost, nil, { .57, .57, .57 }, { .74, .74, .74 }, Ob.rRoomDef.mmlvl, { .57, .57, .57 }, { .74, .74, .74 })
    Ob.rUpgradeButton:setLoc(338, -50)
    Ob.rUpgradeButton:setParent(Ob)

	return Ob
end

return RoomUpgradeButton