local ScrollableUI = {}
local Renderer = require('Renderer')
local util = require('DFUtil')

function ScrollableUI:new(rLayer, tList, nMaxVisible, sTopLabel, sButtonLabel, cancelCallback, acceptCallback)
	local gfxQuad = MOAIGfxQuad2D.new()
	gfxQuad:setTexture(util.getAssetName("MiscNonTPS/1x1.png"))
	gfxQuad:setRect(-320, -480, 320, 480)

	local Ob = MOAIProp2D.new()
	Ob:setDeck(gfxQuad)
	Ob:setLoc(0, 0)
	Ob:setColor(0, 0, 0)

	Ob.tElements = {}
	Ob.rLayer = rLayer
	Ob.tList = tList
	Ob.nMaxVisible = nMaxVisible	
	Ob.nCurMinIndex = 1
	Ob.rSelectedElement = nil

	Ob.cancelCallback = cancelCallback
	Ob.acceptCallback = acceptCallback

	-- top label
	Ob.rTopLabelText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
	Ob.rTopLabelText:setString(sTopLabel)
	Ob.rTopLabelText:setLoc(220, 400)
	Ob.rTopLabelText:setColor(1, 1, 1)
	Ob.rTopLabelText:setRect(-320, -70, 320, 70)

	-- cancel button
	local cancelButtonGfxQuad = MOAIGfxQuad2D.new()
	cancelButtonGfxQuad:setTexture (util.getAssetName("UI/button_cancel.png"))
	cancelButtonGfxQuad:setRect(-96, -32, 96, 32)

	Ob.rCancelButton = MOAIProp2D.new()
	Ob.rCancelButton:setDeck(cancelButtonGfxQuad)
	Ob.rCancelButton:setLoc(-220, 450)
	
	-- accept button
	local acceptButtonGfxQuad = MOAIGfxQuad2D.new()
	acceptButtonGfxQuad:setTexture(util.getAssetName("UI/button_green.png"))
	acceptButtonGfxQuad:setRect(-320, -60, 320, 60)	

	Ob.rAcceptButtonBG = MOAIProp2D.new()
	Ob.rAcceptButtonBG:setDeck(acceptButtonGfxQuad)
	Ob.rAcceptButtonBG:setLoc(0, -430)

	-- the text
	Ob.rButtonLabel = g_GUI:CreateTextBox(46, MOAITextBox.CENTER_JUSTIFY)
	Ob.rButtonLabel:setRect(-320, -60, 320, 60)
	Ob.rButtonLabel:setString(sButtonLabel)
	Ob.rButtonLabel:setLoc(0, -465)
	Ob.rButtonLabel:setColor(1, 1, 1)

	function Ob:onFinger(eventType, sx, sy)
		local tLayer = Renderer.getLayer("UI")
		local x, y = tLayer.RenderLayer:wndToWorld(sx, sy)			
		local rTouchedProp = tLayer.Partition:propForPoint(x, y)
		if eventType == MOAITouchSensor.TOUCH_DOWN then
			if rTouchedProp == self.rCancelButton then
				if self.cancelCallback then					
					self.cancelCallback(self)
				end
				return true				
			elseif rTouchedProp == self.rAcceptButtonBG or rTouchedProp == self.rButtonLabel then
				if self.acceptCallback then
					self.acceptCallback(self)
				end
			else
				for i, rElement in ipairs(self.tElements) do
					if rElement:containsElement(rTouchedProp) then
						self:selectElement(rElement)
						return true
					end
				end
			end
		end
		return true
	end

	function Ob:selectElement(rElement)
		if self.rSelectedElement and self.rSelectedElement ~= rElement then
			self.rSelectedElement:onSelected(false)
		end
		self.rSelectedElement = rElement
		rElement:onSelected(true)
	end

	function Ob:addElement(rElement)
		table.insert(self.tElements, rElement)
		rElement:setVisible(true)
	end

	function Ob:updateList()
		local kTOP_Y_LOC = 340
		for i, rElement in ipairs(self.tElements) do
			rElement:setVisible(false)
		end
		self.tElements = {}
		local nNumElements = 0
		for i = self.nCurMinIndex, self.nCurMinIndex + self.nMaxVisible - 1 do
			if self.tList[i] then				
				self.tList[i]:setLoc(0, kTOP_Y_LOC - nNumElements * (self.tList[i]:getYSize() * 0.75)) -- assuming all uniform sizes
				nNumElements = nNumElements + 1
				self.tList[i]:setScl(1, 0.75)
				self:addElement(self.tList[i])
				-- TEMP WHILE WE DONT HAVE SCROLLING				
			else
				break
			end
		end
	end

	function Ob:setVisible(bVisible)
		if bVisible then
			self.rLayer:insertProp(self)
			self.rLayer:insertProp(self.rTopLabelText)	
			self.rLayer:insertProp(self.rCancelButton)	
			self.rLayer:insertProp(self.rAcceptButtonBG)
			self.rLayer:insertProp(self.rButtonLabel)
			self:updateList()
		else
			self.rLayer:removeProp(self)
			self.rLayer:removeProp(self.rTopLabelText)
			self.rLayer:removeProp(self.rCancelButton)
			self.rLayer:removeProp(self.rAcceptButtonBG)
			self.rLayer:removeProp(self.rButtonLabel)
			if self.rLayer and self.tElements then
				for i, rElement in ipairs(self.tElements) do
					rElement:setVisible(false)
				end
			end
		end
	end

	return Ob
end

return ScrollableUI