local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local HeroSelectorUI = require('ui/HeroSelectorUI')
local ResourceBarUI = require('ui/ResourceBarUI')
local EquipmentSelectorUI = require('ui/EquipmentSelectorUI')
local EquipmentInventoryUI = require('ui/EquipmentInventoryUI')
local MeteoriteInventoryUI = require('ui/MeteoriteInventoryUI')
local ConsumableInventoryUI = require('ui/ConsumableInventoryUI')
local ConsumableSelectorUI = require('ui/ConsumableSelectorUI')
local HeroPackSelectorUI = require('ui/HeroPackSelectorUI')
local HeroCostumeSelectorUI = require('ui/HeroCostumeSelectorUI')
local MeteoriteSelectorUI = require('ui/MeteoriteSelectorUI')
local RMTSelectorUI = require('ui/RMTSelectorUI')
local Button = require('ui/Button')
local PostIt = require('ui/PostIt')
local DFIAP = require('DFCommon.IAP')
local DFAnalytics = require('DFCommon.Analytics')
local util = require('DFUtil')
local ShopUI = {}
local ShopCategoryPane = {}
local CategoryButton = {}

local tUITextureSpriteSheetInfo =
{
    back_button_blue = 'UI/hud',
    back_button_blue_tap = 'UI/hud',
    tab1 = 'UI/shop',
    tab2 = 'UI/shop',
    shop_slot_default = 'UI/shop',
    shop_slot_focused = 'UI/shop',
    tape_label1 = 'UI/shop',
    icon_new = 'UI/hud2',
}

local tSelectedTextColor = { 126 / 255, 97 / 255, 69 / 255 }
local tUnselectedTextColor = { 198 / 255, 157 / 255, 116 / 255 }

function CategoryButton:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)    

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onSelected(bSelected)
        self.rUnselectedButton:setVisible(not bSelected)
        self.rSelectedButton:setVisible(bSelected)
    end

    function Ob:hasElement(rElement)
        local tElements = self:getElements()
        if tElements then
            for i, rExistingElement in ipairs(tElements) do
                if rExistingElement == rElement then
                    return true
                end
            end
        end
        return false
    end

    function Ob:showNewIcon(bShow)
        self.rNewIcon:setVisible(bShow)
    end
    
    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rUnselectedButton = Ob:getUITextureProp('shop_slot_default')
    Ob.rUnselectedButton:setLoc(0, 0)
    Ob:addElement(Ob.rUnselectedButton)
    
    Ob.rSelectedButton = Ob:getUITextureProp('shop_slot_focused')
    Ob.rSelectedButton:setLoc(0, 0)
    Ob.rSelectedButton:setVisible(false)
    Ob:addElement(Ob.rSelectedButton)    

    Ob.rNewIcon = Ob:getUITextureProp('icon_new')
    Ob.rNewIcon:setLoc(320, -10)
    Ob.rNewIcon:setVisible(false)
    Ob:addElement(Ob.rNewIcon)
    
    return Ob
end

function ShopCategoryPane:new(rRenderLayer, rParentUI)
	local Ob = UIElement:new(rRenderLayer)    
    Ob.rParentUI = rParentUI

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rHeroesButton:setVisible(bVisible)
        self.rItemsButton:setVisible(bVisible)
        self.rEquipmentButton:setVisible(bVisible)
        self.rCostumesButton:setVisible(bVisible)
        self.rMeteoriteButton:setVisible(bVisible)
        self.rRMTButton:setVisible(bVisible)
        if not self.sCategoryName then
            self:selectCategory('Heroes')
        else
            self:selectCategory(self.sCategoryName) 
        end       
        if bVisible then
            local nPri = self.rEquipmentButton:getElementsMaxPriority()
            self.rHeroButtonTape:setPriority(nPri + 2)
            self.rHeroesButtonText:setPriority(nPri + 3)
            self.rItemButtonTape:setPriority(nPri + 4)
            self.rItemsButtonText:setPriority(nPri + 5)
            self.rEquipmentButtonTape:setPriority(nPri + 6)
            self.rEquipmentButtonText:setPriority(nPri + 7)
            self.rCostumeButtonTape:setPriority(nPri + 8)
            self.rCostumesButtonText:setPriority(nPri + 9)
            self.rMeteoriteButtonTape:setPriority(nPri + 10)
            self.rMeteoriteButtonText:setPriority(nPri + 11)
            self.rRMTButtonTape:setPriority(nPri + 12)
            self.rRMTButtonText:setPriority(nPri + 13)

            if not DFIAP.arePurchasesAllowed() or util.countitems(DFIAP.getProductCatalog()) == 0 then
                self.rRMTButton:setVisible(false)
                self.rRMTButtonTape:setVisible(false)
                self.rRMTButtonText:setReveal(0)
            end

            Renderer.ShowBase(false)
        else
            Renderer.ShowBase(true)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroesButton:onPaneClosed()
        self.rItemsButton:onPaneClosed()
        self.rEquipmentButton:onPaneClosed()
        self.rCostumesButton:onPaneClosed()
    end

    function Ob:getCurrentCategory()
        return self.sCategoryName
    end

    function Ob:selectCategory(sCategoryName)
        if self.sCategoryName ~= sCategoryName then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
            
            if self.rCurrentCategoryButton then
                -- hide the flag!
                self.rCurrentCategoryButton:showNewIcon(false)
            end
        end
        
        if sCategoryName == 'Heroes' then
            self.rCurrentCategoryButton = self.rHeroesButton
            self.rHeroesButton:onSelected(true)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(false)
        elseif sCategoryName == 'Consumables' then
            self.rCurrentCategoryButton = self.rItemsButton
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(true)
            self.rEquipmentButton:onSelected(false)
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(false)
        elseif sCategoryName == 'Equipment' then
            self.rCurrentCategoryButton = self.rEquipmentButton
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(true) 
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(false)
        elseif sCategoryName == 'Costumes' then
            self.rCurrentCategoryButton = self.rCostumesButton
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)   
            self.rCostumesButton:onSelected(true)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(false)
        elseif sCategoryName == 'Meteorite' then
            self.rCurrentCategoryButton = self.rMeteoriteButton
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)   
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(true)
            self.rRMTButton:onSelected(false)
        elseif sCategoryName == 'RMT' then
            self.rCurrentCategoryButton = self.rRMTButton
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)   
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(true)
        else 
            self.rCurrentCategoryButton = nil
            self.rHeroesButton:onSelected(false)
            self.rItemsButton:onSelected(false)
            self.rEquipmentButton:onSelected(false)   
            self.rCostumesButton:onSelected(false)
            self.rMeteoriteButton:onSelected(false)
            self.rRMTButton:onSelected(false)
        end
        
        self.sCategoryName = sCategoryName
        self.rParentUI:onCategorySelected(sCategoryName)        
        
        g_MiddleManager:setItemNewFlagForCategory(sCategoryName, false)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if tProps then
            if eventType == MOAITouchSensor.TOUCH_UP and not g_TouchMoved then
                for i, rProp in ipairs(tProps) do
                    if self.rHeroesButton:hasElement(rProp) then    
                        self:selectCategory('Heroes')
                        return true
                    elseif self.rItemsButton:hasElement(rProp) then
                        self:selectCategory('Consumables')
                        return true
                    elseif self.rEquipmentButton:hasElement(rProp) then
                        self:selectCategory('Equipment')
                        return true
                    elseif self.rCostumesButton:hasElement(rProp) then
                        self:selectCategory('Costumes')
                        return true  
                    elseif self.rMeteoriteButton:hasElement(rProp) then
                        self:selectCategory('Meteorite')
                        return true                 
                    elseif self.rRMTButton:hasElement(rProp) then
                        self:selectCategory('RMT')
                        return true                      
                    end
                end
            end
        end
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    local bIsNew = false
    
    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('hero')
    Ob.rHeroesButton = CategoryButton:new(rRenderLayer)
    Ob.rHeroesButton:setLoc(8, -4)
    Ob.rHeroesButton:setScl(1, .9)
    Ob.rHeroesButton:setParent(Ob)

    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('consumable')
    Ob.rItemsButton = CategoryButton:new(rRenderLayer)
    Ob.rItemsButton:showNewIcon(bIsNew)
    Ob.rItemsButton:setLoc(8, -82)
    Ob.rItemsButton:setScl(1, .9)
    Ob.rItemsButton:setParent(Ob)

    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('equipment')
    Ob.rEquipmentButton = CategoryButton:new(rRenderLayer)
    Ob.rEquipmentButton:showNewIcon(bIsNew)
    Ob.rEquipmentButton:setLoc(8, -160)
    Ob.rEquipmentButton:setScl(1, .9)
    Ob.rEquipmentButton:setParent(Ob)

    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('costume')
    Ob.rCostumesButton = CategoryButton:new(rRenderLayer)
    Ob.rCostumesButton:showNewIcon(bIsNew)
    Ob.rCostumesButton:setLoc(8, -238)
    Ob.rCostumesButton:setScl(1, .9)
    Ob.rCostumesButton:setParent(Ob)

    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('currency')
    Ob.rMeteoriteButton = CategoryButton:new(rRenderLayer)
    Ob.rMeteoriteButton:showNewIcon(bIsNew)
    Ob.rMeteoriteButton:setLoc(8, -316)
    Ob.rMeteoriteButton:setScl(1, .9)
    Ob.rMeteoriteButton:setParent(Ob)

    bIsNew = g_MiddleManager:areAnyItemsNewForCategory('rmt')
    Ob.rRMTButton = CategoryButton:new(rRenderLayer)
    Ob.rRMTButton:showNewIcon(bIsNew)
    Ob.rRMTButton:setLoc(8, -394)
    Ob.rRMTButton:setScl(1, .9)
    Ob.rRMTButton:setParent(Ob)

    Ob.rHeroesButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rHeroesButtonText:setString(g_DialogManager:getString("MENU001TEXT"))
    Ob.rHeroesButtonText:setLoc(350, -20)   
    Ob:addElement(Ob.rHeroesButtonText)

    Ob.rHeroButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rHeroButtonTape:setLoc(30, -20) 
    Ob.rHeroButtonTape:setScl(1.3, 1)
    Ob:addElement(Ob.rHeroButtonTape)

    Ob.rItemsButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rItemsButtonText:setString(g_DialogManager:getString("MENU002TEXT"))
    Ob.rItemsButtonText:setLoc(350, -98)   
    Ob:addElement(Ob.rItemsButtonText)

    Ob.rItemButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rItemButtonTape:setLoc(30, -98) 
    Ob.rItemButtonTape:setScl(0.9, 1)
    Ob:addElement(Ob.rItemButtonTape)

    Ob.rEquipmentButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rEquipmentButtonText:setString(g_DialogManager:getString("MENU003TEXT"))
    Ob.rEquipmentButtonText:setLoc(350, -176)
    Ob:addElement(Ob.rEquipmentButtonText)

    Ob.rEquipmentButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rEquipmentButtonTape:setLoc(30, -176) 
    Ob.rEquipmentButtonTape:setScl(1.15, 1)
    Ob:addElement(Ob.rEquipmentButtonTape)

    Ob.rCostumesButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rCostumesButtonText:setString(g_DialogManager:getString("MENU006TEXT"))
    Ob.rCostumesButtonText:setLoc(350, -254)
    Ob:addElement(Ob.rCostumesButtonText)

    Ob.rCostumeButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rCostumeButtonTape:setLoc(30, -254) 
    Ob.rCostumeButtonTape:setScl(1.1, 1)
    Ob:addElement(Ob.rCostumeButtonTape)

    Ob.rMeteoriteButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rMeteoriteButtonText:setString(g_DialogManager:getString("MENU007TEXT"))
    Ob.rMeteoriteButtonText:setLoc(350, -332)
    Ob:addElement(Ob.rMeteoriteButtonText)

    Ob.rMeteoriteButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rMeteoriteButtonTape:setLoc(30, -332) 
    Ob.rMeteoriteButtonTape:setScl(1.2, 1)
    Ob:addElement(Ob.rMeteoriteButtonTape)

    Ob.rRMTButtonText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY)
    Ob.rRMTButtonText:setString(g_DialogManager:getString("MENU005TEXT"))
    Ob.rRMTButtonText:setLoc(350, -410)
    Ob:addElement(Ob.rRMTButtonText)

    Ob.rRMTButtonTape = Ob:getUITextureProp('tape_label1')
    Ob.rRMTButtonTape:setLoc(30, -410) 
    Ob.rRMTButtonTape:setScl(1.4, 1)
    Ob:addElement(Ob.rRMTButtonTape)

    Ob.sCategoryName = 'Consumables'

    return Ob
end

function ShopUI:new(rRenderLayer, rHero)
	local Ob = UIElement:new(rRenderLayer)

    Ob.bIsShopUI = true

	function Ob:setVisible(bVisible, sDefaultTab)
        self:setElementsVisible(bVisible)
        self.rBackButton:setVisible(bVisible)
        if bVisible then
            self.rResourceBar:setVisible(bVisible)    
            if not sDefaultTab then
                self:selectTab('Shop')
            else
                self:selectTab(sDefaultTab)
            end
        else
            self.rHeroSelectorUI:setVisible(bVisible)
            self.rShopCategoryPane:setVisible(bVisible)
            self.rEquipmentPane:setVisible(bVisible)
            self.rConsumablesPane:setVisible(bVisible)
            self.rHeroPackPane:setVisible(bVisible)
            self.rHeroCostumePane:setVisible(bVisible)
            self.rMeteoritePane:setVisible(bVisible)
            self.rRMTPane:setVisible(bVisible)
            self.rResourceBar:setVisible(bVisible)                        
            if self.rActiveEquipmentInventoryPane then
                self.rActiveEquipmentInventoryPane:setVisible(false)
            end
            if self.rActiveMeteoriteInventoryPane then
                self.rActiveMeteoriteInventoryPane:setVisible(bVisible)
            end
        end
        
        -- BEN-NOTE: remove this event entirely to try and get our event count down to save battery
        --if bVisible then
        --    DFAnalytics.logEvent("shp_open")
        --end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rHeroSelectorUI:onPaneClosed()
        self.rShopCategoryPane:onPaneClosed()
        self.rEquipmentPane:onPaneClosed()
        self.rConsumablesPane:onPaneClosed()
        self.rHeroPackPane:onPaneClosed()
        self.rHeroCostumePane:onPaneClosed()
        self.rMeteoritePane:onPaneClosed()
        self.rRMTPane:onPaneClosed()
        self.rResourceBar:onPaneClosed()
        self.rBackButton:onPaneClosed()
        if self.rActiveMeteoriteInventoryPane then
            self.rActiveMeteoriteInventoryPane:onPaneClosed()
        end
        g_Game:saveGame()
    end

    function Ob:getCurrentCategory()
        if self.rShopCategoryPane then
            self.rShopCategoryPane:getCurrentCategory()
        end
        return nil
    end

    function Ob:getActiveTabName()
        return self.sActiveTabName
    end

    function Ob:update()
        self.rResourceBar:update()
        if self.sActiveTabName == 'Inventory' then
            if self.rHeroSelectorUI then
                self.rHeroSelectorUI:update()
            end
            if self.rActiveEquipmentInventoryPane then
                self.rActiveEquipmentInventoryPane:update()
            end
        elseif self.sActiveTabName == 'Shop' then
            if self.sCategoryName == 'Equipment' then
                if self.rEquipmentPane then
                    self.rEquipmentPane:update()
                end
            elseif self.sCategoryName == 'Consumables' then
                if self.rConsumablesPane then
                    self.rConsumablesPane:update()
                end
            elseif self.sCategoryName == 'Heroes' then
                if self.rHeroPackPane then
                    self.rHeroPackPane:update()
                end
            elseif self.sCategoryName == 'Costumes' then
                if self.rHeroCostumePane then
                    self.rHeroCostumePane:update()
                end
            elseif self.sCategoryName == 'Meteorite' then    
                if self.rMeteoritePane then
                    self.rMeteoritePane:update()
                end
            elseif self.sCategoryName == 'RMT' then
                if self.rRMTPane then
                    self.rRMTPane:update()
                end
            end
        elseif self.sActiveTabName == 'Meteorite' then
            if self.rHeroSelectorUI then
                self.rHeroSelectorUI:update()
            end
            if self.rActiveMeteoriteInventoryPane then
                self.rActiveMeteoriteInventoryPane:update()
            end        
        end
    end

    function Ob:onShopTabSelected()
        self.rShopTabText:setColor(tSelectedTextColor[1], tSelectedTextColor[2], tSelectedTextColor[3])
        self.rInventoryText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
        self.rMeteoriteText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])

        self.rMeteoriteTabBG:setVisible(true)
        self.rMeteoriteTabActiveBG:setVisible(false)
        self.rInventoryTabBG:setVisible(true)
        self.rInventoryTabActiveBG:setVisible(false)
        self.rShopTabBG:setVisible(false)
        self.rShopTabActiveBG:setVisible(true)

        self.rShopCategoryPane:setVisible(true)
        self.rHeroSelectorUI:setVisible(false)
        if self.rActiveEquipmentInventoryPane then
            self.rActiveEquipmentInventoryPane:setVisible(false)
        end
        if self.rActiveMeteoriteInventoryPane then
            self.rActiveMeteoriteInventoryPane:setVisible(false)
        end
    end

    function Ob:onInventoryTabSelected()        
        self.rShopTabText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
        self.rInventoryText:setColor(tSelectedTextColor[1], tSelectedTextColor[2], tSelectedTextColor[3])
        self.rMeteoriteText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])

        self.rMeteoriteTabBG:setVisible(true)
        self.rMeteoriteTabActiveBG:setVisible(false)
        self.rInventoryTabBG:setVisible(false)
        self.rInventoryTabActiveBG:setVisible(true)
        self.rShopTabBG:setVisible(true)
        self.rShopTabActiveBG:setVisible(false)

        self.rShopCategoryPane:setVisible(false)
        self.rHeroSelectorUI:setVisible(true)

        self.rEquipmentPane:setVisible(false)
        self.rConsumablesPane:setVisible(false)
        self.rHeroPackPane:setVisible(false)
        self.rHeroCostumePane:setVisible(false)
        self.rMeteoritePane:setVisible(false)
        self.rRMTPane:setVisible(false)
        if self.rActiveMeteoriteInventoryPane then
            self.rActiveMeteoriteInventoryPane:setVisible(false)
        end

        local rHero = self.rHeroSelectorUI:getTopHero()
        self:showInventoryPane()

        self.sCategoryName = nil
    end

    function Ob:onMeteoriteTabSelected()
        self.rShopTabText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
        self.rInventoryText:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
        self.rMeteoriteText:setColor(tSelectedTextColor[1], tSelectedTextColor[2], tSelectedTextColor[3])

        self.rMeteoriteTabBG:setVisible(false)
        self.rMeteoriteTabActiveBG:setVisible(true)
        self.rInventoryTabBG:setVisible(true)
        self.rInventoryTabActiveBG:setVisible(false)
        self.rShopTabBG:setVisible(true)
        self.rShopTabActiveBG:setVisible(false)

        self.rShopCategoryPane:setVisible(false)
        self.rHeroSelectorUI:setVisible(false)

        self.rEquipmentPane:setVisible(false)
        self.rConsumablesPane:setVisible(false)
        self.rHeroPackPane:setVisible(false)
        self.rHeroCostumePane:setVisible(false)
        self.rMeteoritePane:setVisible(false)
        self.rRMTPane:setVisible(false)

        self.rShopCategoryPane:setVisible(false)
        self.rHeroSelectorUI:setVisible(true)

        if self.rActiveEquipmentInventoryPane then
            self.rActiveEquipmentInventoryPane:setVisible(false)
        end

        self:showMeteoritePane()      
    end

    function Ob:showInventoryPane()
        if self.rActiveEquipmentInventoryPane then
            self.rActiveEquipmentInventoryPane:setVisible(false)
        end     
        self.rActiveEquipmentInventoryPane = EquipmentInventoryUI:new(self:getRenderLayer())  
        self.rActiveEquipmentInventoryPane:setHeroSelectorUI(self.rHeroSelectorUI)
        self.rActiveEquipmentInventoryPane:setLoc(Renderer.getUIHalfWidth(), -90)
        self.rActiveEquipmentInventoryPane:setVisible(true)            
    end

    function Ob:showMeteoritePane()
        if self.rActiveMeteoriteInventoryPane then
            self.rActiveMeteoriteInventoryPane:setVisible(false)
        end
        self.rActiveMeteoriteInventoryPane = MeteoriteInventoryUI:new(self.rRenderLayer)
        self.rActiveMeteoriteInventoryPane:setHeroSelectorUI(self.rHeroSelectorUI)
        self.rActiveMeteoriteInventoryPane:setLoc(Renderer.getUIHalfWidth() + 8, -90)
        self.rActiveMeteoriteInventoryPane:setParent(self)
        self.rActiveMeteoriteInventoryPane:setVisible(true)    
    end

    function Ob:selectTab(sTabName)
        if sTabName and self.sActiveTabName ~= sTabName then
            g_Audio:playCue('MMOJ_Audio/UI/FolderTab')
            self.sActiveTabName = sTabName
            local callBack = self['on'..sTabName..'TabSelected']
            if callBack then
                callBack(self)
            end
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if g_Game:isActionEnabled('CloseWindow') then
                    g_GUI:HideShopUI()
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        self.rBackButton:onFinger(eventType, sx, sy, tProps)
        if not g_TouchMoved then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rShopTabButton then
                        if eventType == MOAITouchSensor.TOUCH_UP then
                            self:selectTab('Shop') -- coming from the inventory, this should default to equipment
                            if self.rShopCategoryPane then
                                self.rShopCategoryPane:selectCategory('Equipment')
                            end
                        end
                        return true
                    elseif rProp == self.rInventoryTabButton then
                        if eventType == MOAITouchSensor.TOUCH_UP then
                            self:selectTab('Inventory')
                        end
                        return true
                    elseif rProp == self.rMeteoriteTabButton then
                        if eventType == MOAITouchSensor.TOUCH_UP then
                            self:selectTab('Meteorite')
                        end
                        return true
                    end
                end
            end
        end
        if self.sActiveTabName == 'Shop' then
            self.rShopCategoryPane:onFinger(eventType, sx, sy, tProps)
            if self.sCategoryName == 'Equipment' then
                self.rEquipmentPane:onFinger(eventType, sx, sy, tProps)
            elseif self.sCategoryName == 'Consumables' then
                self.rConsumablesPane:onFinger(eventType, sx, sy, tProps)
            elseif self.sCategoryName == 'Heroes' then
                self.rHeroPackPane:onFinger(eventType, sx, sy, tProps)
            elseif self.sCategoryName == 'Costumes' then
                self.rHeroCostumePane:onFinger(eventType, sx, sy, tProps)
            elseif self.sCategoryName == 'Meteorite' then
                self.rMeteoritePane:onFinger(eventType, sx, sy, tProps)    
            elseif self.sCategoryName == 'RMT' then
                self.rRMTPane:onFinger(eventType, sx, sy, tProps)                            
            end
        elseif self.sActiveTabName == 'Inventory' then
            local tLayer = Renderer.getLayer('UI')
            local x, y = tLayer.RenderLayer:wndToWorld(sx, sy)
            if x > Renderer.getUIHalfWidth() then
                if self.rActiveEquipmentInventoryPane then
                    self.rActiveEquipmentInventoryPane:onFinger(eventType, sx, sy, tProps)
                end
            else
                self.rHeroSelectorUI:onFinger(eventType, sx, sy, tProps)
            end
        elseif self.sActiveTabName == 'Meteorite' then
            local tLayer = Renderer.getLayer('UI')
            local x, y = tLayer.RenderLayer:wndToWorld(sx, sy)
            if x > Renderer.getUIHalfWidth() then
                if self.rActiveMeteoriteInventoryPane then
                    self.rActiveMeteoriteInventoryPane:onFinger(eventType, sx, sy, tProps)
                end
            else
                self.rHeroSelectorUI:onFinger(eventType, sx, sy, tProps)
            end
        end

        self.rResourceBar:onFinger(eventType, sx, sy, tProps)

        return true
    end

    function Ob:onCategorySelected(sCategoryName)
        if self.sCategoryName ~= sCategoryName then
            if self.sActiveTabName == 'Shop' then
                if sCategoryName == 'Equipment' then
                    self.rEquipmentPane:setVisible(true)
                    self.rConsumablesPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(false)
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(false)
                elseif sCategoryName == 'Consumables' then
                    self.rConsumablesPane:setVisible(true)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(false)
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(false)
                elseif sCategoryName == 'Heroes' then
                    self.rConsumablesPane:setVisible(false)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(true)
                    self.rHeroCostumePane:setVisible(false) 
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(false)
                elseif sCategoryName == 'Costumes' then
                    self.rConsumablesPane:setVisible(false)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(true)     
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(false)
                elseif sCategoryName == 'Meteorite' then
                    self.rConsumablesPane:setVisible(false)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(false)     
                    self.rMeteoritePane:setVisible(true)
                    self.rRMTPane:setVisible(false)
                elseif sCategoryName == 'RMT' then
                    self.rConsumablesPane:setVisible(false)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(false)     
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(true)
                else
                    self.rConsumablesPane:setVisible(false)
                    self.rEquipmentPane:setVisible(false)
                    self.rHeroPackPane:setVisible(false)
                    self.rHeroCostumePane:setVisible(false)                    
                    self.rMeteoritePane:setVisible(false)
                    self.rRMTPane:setVisible(false)
                end
            end
        end
        self.sCategoryName = sCategoryName
    end

    function Ob:onHeroSelected(rHero)
        if self.sActiveTabName == 'Inventory' then
            g_Audio:playCue('MMOJ_Audio/UI/CategorySelect')
         end
    end

    function Ob:onIAPCompleted(bSuccessful, sProductID, failCode)
        g_GUI:SetPopupPane(nil) -- hide postit
        
        local bShowGenericPopup = true
        
        local sMessageLineCode = "STXT143TEXT"
        if bSuccessful then
            for i, sDefKey in ipairs(g_Game:getAllCurrencyDefKeys()) do
                local rDef = g_Game:getCurrencyDef(sDefKey)
                if rDef and sProductID == rDef.sAppleProductID then
                    g_GUI:HideShopUI()
                    if rDef.sPremiumPurchaseSprite then
                        g_CurrentMap:centerOnRoom('mm')
                        bShowGenericPopup = false
                        g_GUI:ShowPremiumPurchaseSequence("prop", rDef)
                    elseif rDef.bIsPremiumCurrency then
                        bShowGenericPopup = false
                        g_GUI:ShowPremiumPurchaseSequence("superium", rDef)
                    end
                end
            end
        else
            if failCode == DFIAP.TRANSACTION_STATE_CANCELLED then
                sMessageLineCode = "STXT144TEXT"
            else
                sMessageLineCode = "STXT145TEXT"
            end
        end
        
        if MOAIEnvironment and MOAIEnvironment.androidBillingProvider and MOAIBilling.BILLING_PROVIDER_AMAZON and MOAIEnvironment.androidBillingProvider == MOAIBilling.BILLING_PROVIDER_AMAZON then
            bShowGenericPopup = false
        end

        if bShowGenericPopup then
            g_GUI:ShowPostIt(g_DialogManager:getString(sMessageLineCode), 240, -160, Renderer.getRenderLayer('UIOverlay'), true)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    -- add the bottom BG
    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())                 
	Ob.rBackground:setColor(0, 0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rLeftBG = MOAIProp2D.new()
	Ob.rLeftBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rLeftBG:setLoc(0, 0)
    Ob.rLeftBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())                 
	Ob.rLeftBG:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
    Ob:addElement(Ob.rLeftBG)

    Ob.rItemPaneBG = MOAIProp2D.new()
	Ob.rItemPaneBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rItemPaneBG:setLoc(Renderer.getUIHalfWidth(), -92)
    Ob.rItemPaneBG:setScl(Renderer.getUIHalfWidth(), Renderer.getUIHeight())
	Ob.rItemPaneBG:setColor(tUnselectedTextColor[1], tUnselectedTextColor[2], tUnselectedTextColor[3])
    Ob:addElement(Ob.rItemPaneBG)

    Ob.rTopBar = MOAIProp2D.new()
	Ob.rTopBar:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBar:setLoc(0, 0)
    Ob.rTopBar:setScl(Renderer.getUIWidth(), 92)                 
	Ob.rTopBar:setColor(.97, .93, .73)
    Ob:addOverlayElement(Ob.rTopBar)

    Ob.rMeteoriteTabBG = Ob:getUITextureProp('tab2')
    Ob.rMeteoriteTabBG:setLoc(Renderer.getUIHalfWidth(), 0)
    Ob:addOverlayElement(Ob.rMeteoriteTabBG)
    Ob.rInventoryTabBG = Ob:getUITextureProp('tab2')
    Ob.rInventoryTabBG:setLoc(Renderer.getUIHalfWidth() - 224, 0)
    Ob:addOverlayElement(Ob.rInventoryTabBG)
    Ob.rShopTabBG = Ob:getUITextureProp('tab2')
    Ob.rShopTabBG:setLoc(Renderer.getUIHalfWidth() - 449, 0)
    Ob:addOverlayElement(Ob.rShopTabBG)

    Ob.rMeteoriteTabActiveBG = Ob:getUITextureProp('tab1')
    Ob.rMeteoriteTabActiveBG:setLoc(Renderer.getUIHalfWidth() + 225, 0)
    Ob:addOverlayElement(Ob.rMeteoriteTabActiveBG)
    Ob.rInventoryTabActiveBG = Ob:getUITextureProp('tab1')
    Ob.rInventoryTabActiveBG:setLoc(Renderer.getUIHalfWidth() + 1, 0)
    Ob:addOverlayElement(Ob.rInventoryTabActiveBG)
    Ob.rShopTabActiveBG = Ob:getUITextureProp('tab1')
    Ob.rShopTabActiveBG:setLoc(Renderer.getUIHalfWidth() - 224, 0)
    Ob:addOverlayElement(Ob.rShopTabActiveBG)

    Ob.rShopTabButton = MOAIProp2D.new()
    Ob.rShopTabButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rShopTabButton:setLoc(Renderer.getUIHalfWidth() - 198, -20)
    Ob.rShopTabButton:setScl(200, 70)
    Ob.rShopTabButton:setColor(1, 1, 1)
    Ob.rShopTabButton:setVisible(false)
    Ob:addOverlayElement(Ob.rShopTabButton)

    Ob.rInventoryTabButton = MOAIProp2D.new()
    Ob.rInventoryTabButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rInventoryTabButton:setLoc(Renderer.getUIHalfWidth() + 30, -20)
    Ob.rInventoryTabButton:setScl(200, 70)
    Ob.rInventoryTabButton:setColor(1, 1, 1)
    Ob.rInventoryTabButton:setVisible(false)
    Ob:addOverlayElement(Ob.rInventoryTabButton)

    Ob.rMeteoriteTabButton = MOAIProp2D.new()
    Ob.rMeteoriteTabButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rMeteoriteTabButton:setLoc(Renderer.getUIHalfWidth() + 250, -20)
    Ob.rMeteoriteTabButton:setScl(200, 70)
    Ob.rMeteoriteTabButton:setColor(1, 1, 1)
    Ob.rMeteoriteTabButton:setVisible(false)
    Ob:addOverlayElement(Ob.rMeteoriteTabButton)

    Ob.rShopTabText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rShopTabText:setString("SHOP")
    Ob.rShopTabText:setLoc(Renderer.getUIHalfWidth() - 104, -30)   
    Ob:addOverlayElement(Ob.rShopTabText)

    Ob.rInventoryText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rInventoryText:setString("INVENTORY")
    Ob.rInventoryText:setLoc(Renderer.getUIHalfWidth() + 134, -30)
    Ob:addOverlayElement(Ob.rInventoryText)

    Ob.rMeteoriteText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rMeteoriteText:setString("METEORITE")
    Ob.rMeteoriteText:setLoc(Renderer.getUIHalfWidth() + 358, -30)
    Ob:addOverlayElement(Ob.rMeteoriteText)

    -- add the back button
    Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'back_button_blue', 'back_button_blue_tap')
    Ob.rBackButton:setParent(Ob)

    -- add the hero pane
    Ob.rResourceBar = ResourceBarUI:new(rRenderLayer)
    Ob.rResourceBar:setLoc(0, -Renderer.getUIHeight() + 74)
    Ob.rResourceBar:setParent(Ob)

    local tHeroes = {}
    if not rHero then
        local tAllHeroes = g_Game:getAllHeroes()
    
        for i, rExistingHero in pairs(tAllHeroes) do
            table.insert(tHeroes, rExistingHero)
        end
    else
        tHeroes = { rHero }
    end

    Ob.rHeroSelectorUI = HeroSelectorUI:new(rRenderLayer, tHeroes, Ob)
    Ob.rHeroSelectorUI:setLoc(Renderer.getUIHalfWidth() - 472, -100)
    Ob.rHeroSelectorUI:setParent(Ob)

    Ob.rShopCategoryPane = ShopCategoryPane:new(rRenderLayer, Ob)
    Ob.rShopCategoryPane:setLoc(Renderer.getUIHalfWidth() - 480, -94)
    Ob.rShopCategoryPane:setParent(Ob)

    Ob.rEquipmentPane = EquipmentSelectorUI:new(rRenderLayer)
    Ob.rEquipmentPane:setLoc(Renderer.getUIHalfWidth(), -90)
    Ob.rEquipmentPane:setParent(Ob)

    Ob.rConsumablesPane = ConsumableSelectorUI:new(rRenderLayer)
    Ob.rConsumablesPane:setLoc(Renderer.getUIHalfWidth(), -90)
    Ob.rConsumablesPane:setParent(Ob)

    Ob.rHeroPackPane = HeroPackSelectorUI:new(rRenderLayer)
    Ob.rHeroPackPane:setLoc(Renderer.getUIHalfWidth() + 8, -90)
    Ob.rHeroPackPane:setParent(Ob)

    Ob.rHeroCostumePane = HeroCostumeSelectorUI:new(rRenderLayer)
    Ob.rHeroCostumePane:setLoc(Renderer.getUIHalfWidth() + 8, -90)
    Ob.rHeroCostumePane:setParent(Ob)

    Ob.rMeteoritePane = MeteoriteSelectorUI:new(rRenderLayer)
    Ob.rMeteoritePane:setLoc(Renderer.getUIHalfWidth() + 8, -90)
    Ob.rMeteoritePane:setParent(Ob)

    Ob.rRMTPane = RMTSelectorUI:new(rRenderLayer)
    Ob.rRMTPane:setLoc(Renderer.getUIHalfWidth() + 8, -90)
    Ob.rRMTPane:setParent(Ob)

    -- create the inventory pane for every hero
    Ob.rActiveEquipmentInventoryPane = nil
    Ob.rActiveMeteoriteInventoryPane = nil

    Ob.sActiveTabName = nil
    Ob.sCategoryName = nil    

    return Ob
end

return ShopUI