local SplashScreen = {}
local MainMenuButton = {}
local SplashScreenNewsTicker = {}
local MainMenuUI = {}

local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local util = require('DFUtil')
local input = require('input')
local DFGraphics = require('DFCommon.Graphics')
local Button = require('ui/Button')
local DFNetNews = require('DFCommon.NetNews')

local tUITextureSpriteSheetInfo =
{
shop_item = 'UI/shop',
button_buy = 'UI/shop',
icon_pp = 'UI/hud',
icon_cash = 'UI/hud',
icon_coins_xs = 'UI/shop',
icon_coins_m = 'UI/shop',
icon_coins_s = 'UI/shop',
icon_pp_xs = 'UI/shop',
icon_pp_m = 'UI/shop',
icon_pp_s = 'UI/shop',
icon_pp_l = 'UI/shop',
icon_prop_coindoubler = 'UI/icons4',
}

function MainMenuButton:new(rRenderLayer, rParentUI, sText, tUpColor, tDownColor, nWidth, nHeight, nFontSize)
    local Ob = UIElement:new(rRenderLayer)
    Ob.rParentUI = rParentUI
    
    if not nFontSize then nFontSize = 30 end
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.bVisible = bVisible
    end
    
    function Ob:isVisible()
        return true == self.bVisible
    end
    
    function Ob:onFinger(eventType, sx, sy, tProps)  
        if not self.bVisible then
            return false
        end
        local bTouchedButton = false
        if tProps then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rBackground then
                    bTouchedButton = true
                    break
                elseif rProp == self.rPressedButton then
                    bTouchedButton = true
                    break
                end
            end
        end
        if eventType == MOAITouchSensor.TOUCH_DOWN then
            if bTouchedButton then
                self:onPressed(true)
            else
                self:onPressed(false)
            end
        elseif eventType == MOAITouchSensor.TOUCH_MOVE then
            if bTouchedButton then
                self:onPressed(true)
            else
                self:onPressed(false)
            end
        elseif eventType == MOAITouchSensor.TOUCH_UP then
            if bTouchedButton then
                self:onPressed(false)
            end
            if not g_TouchMoved then
                if bTouchedButton then
                    if self.rParentUI and self.rParentUI.onButtonJustPressed then
                        self.rParentUI:onButtonJustPressed(self)
                    end
                end
            end
        end
    end
    
    function Ob:onPressed(bPressed)
        if bPressed then
            self.rForeground:setColor(self.tDownColor[1] / 255, self.tDownColor[2] / 255, self.tDownColor[3] / 255, 1)    
        else
            self.rForeground:setColor(self.tUpColor[1] / 255, self.tUpColor[2] / 255, self.tUpColor[3] / 255, 1)    
        end
    end
    
    if not nWidth then nWidth = 290 end
    if not nHeight then nHeight = 92 end
    if not tUpColor then tUpColor = { 233, 217, 0 } end
    if not tDownColor then tDownColor = { tUpColor[1] * 0.9, tUpColor[2] * 0.9, tUpColor[3] * 0.9 } end
    
    Ob.tUpColor = tUpColor
    Ob.tDownColor = tDownColor
    
    local kBORDER_SIZE = 5
    
	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(nWidth, nHeight)
	Ob.rBackground:setColor(0, 0, 0, 1)
    Ob:addOverlayElement(Ob.rBackground)      
    
    Ob.rForeground = MOAIProp2D.new()
	Ob.rForeground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rForeground:setLoc(kBORDER_SIZE, -kBORDER_SIZE)
	Ob.rForeground:setScl(nWidth - (kBORDER_SIZE * 2), nHeight - (kBORDER_SIZE * 2))
	Ob.rForeground:setColor(tUpColor[1] / 255, tUpColor[2] / 255, tUpColor[3] / 255, 1)    
    Ob:addOverlayElement(Ob.rForeground)   
    
    Ob.rText = g_GUI:CreateTextBox(nFontSize, MOAITextBox.CENTER_JUSTIFY, 'default', MOAITextBox.CENTER_JUSTIFY)
    Ob.rText:setRect(0, 0, nWidth, nHeight)
	Ob.rText:setLoc(0, -nHeight)
    Ob.rText:setString(sText)
    Ob.rText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rText)    
    
    Ob:setParent(rParentUI)
    
    return Ob
end

function MainMenuUI:new(rRenderLayer, rParentUI)
    local Ob = UIElement:new(rRenderLayer)
    Ob.rParentUI = rParentUI
    
    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rPlayButton:setVisible(bVisible)
        if MOAIEnvironment.osBrand == "iOS" then
            self.rCloudButton:setVisible(bVisible)
        end
    end
    
    function Ob:showLoadingMessage(sMessage)
        if sMessage then
            self.rLoadingText:setString(sMessage)
            
            if false ~= self.bShowNewsMessages then
                self.bShowNewsMessages = false
            
                self.rLoadingText:setVisible(true)
                for i, rNewsText in ipairs(self.tNewsTexts) do
                    rNewsText:setVisible(false)
                    self.tNewsLinkBoxes[i]:setVisible(false)
                end
            end
        else
            if true ~= self.bShowNewsMessages then
                self.bShowNewsMessages = true
                
                self.rLoadingText:setVisible(false)
                for i, rNewsText in ipairs(self.tNewsTexts) do
                    rNewsText:setVisible(true)
                    
                    if rNewsText.link then
                        self.tNewsLinkBoxes[i]:setVisible(true)
                    end
                end
            end
        end
    end
    
    function Ob:unloadSpriteSheet()
        
    end
    
    function Ob:onButtonJustPressed(rButton)
        if rButton == self.rPlayButton then
            if self.rParentUI and self.rParentUI.onLoadGame then
                self.rParentUI:onLoadGame()
            end
        elseif rButton == self.rCloudButton then
            if self.rParentUI and self.rParentUI.onCloudSync then
                self.rParentUI:onCloudSync()
            end
        end
    end
    
    function Ob:onFinger(eventType, sx, sy, tProps)  
        self.rPlayButton:onFinger(eventType, sx, sy, tProps)
        self.rCloudButton:onFinger(eventType, sx, sy, tProps)
        
        if eventType == MOAITouchSensor.TOUCH_DOWN then
            local nNewsY = -self.nNewsY
            if sy > nNewsY - 8 and sy < nNewsY + 64 then
                
                -- find out if sx overlaps any news
                for i, rLinkBox in pairs(self.tNewsLinkBoxes) do
                    local boxX, boxY = rLinkBox:getLoc()
                    local boxW, boxH = rLinkBox:getScl()
                    
                    --print("checking link box ( " .. boxX .. "," .. boxY .. "," .. boxW .. "," .. boxH .. " ) against touch ( " .. sx .. ", " .. sy .. " )")
                    
                    if rLinkBox.link and sx > boxX and sx < boxX + boxW then
                        local sUrl = rLinkBox.link
                        print("launching url: " .. sUrl)
                        if MOAISafariIOS then
                            MOAISafariIOS.openURL(sUrl)
                        end
                    end
                end
            end
        end
    end

    function Ob:onNewsLoaded(tNewsData)
        if not tNewsData then
            return
        end
    
        local latestVersion = tNewsData["latestVersion"] or "1.1.0"
        
        self.tNews = {}
        
        if tNewsData["news"] then
            for i, sNews in ipairs(tNewsData["news"]) do
                table.insert(self.tNews, sNews)
            end
            
            local numNewsItems = #self.tNews
            
            for i, rNewsText in ipairs(self.tNewsTexts) do
            
                -- filthy lua indexing by 1 garbage
                local sNewsIndex = ((i - 1) % numNewsItems) + 1
                local sMessage = self.tNews[sNewsIndex].msg

                local nLinkStart = 1
                local nLinkEnd = #sMessage

                local findChar = string.find(sMessage, "%[")
                if findChar then
                    nLinkStart = findChar
                end                
                sMessage = string.gsub(sMessage, "%[", "")
                
                findChar = string.find(sMessage, "%]")
                if findChar then
                    nLinkEnd = findChar
                end
                sMessage = string.gsub(sMessage, "%]", "")
                
                rNewsText.sMessage = sMessage

                rNewsText:setString(sMessage)
                
                local rLinkBox = self.tNewsLinkBoxes[i]
                
                if self.tNews[sNewsIndex].link then
                   rNewsText.link = self.tNews[sNewsIndex].link
                   rNewsText.linkStart = nLinkStart
                   rNewsText.linkEnd = nLinkEnd
                   rLinkBox.link = rNewsText.link
                else
                    rNewsText.link = nil
                    rNewsText.linkStart = 1
                    rNewsText.linkEnd = 1
                    rLinkBox.link = nil
                end
            end
            
            self:adjustNewsLayout()
        end
        
        if latestVersion ~= MOAIEnvironment.appVersion then
            self:showNewBuildPop(latestVersion)
        end
    end

    function Ob:adjustNewsLayout()
        currentX = 256
    
        self.nTotalNewsWidth = 0
    
        for i, rNewsText in ipairs(self.tNewsTexts) do                    
            local xMin, yMin, xMax, yMax = rNewsText:getStringBounds(1, string.len(rNewsText.sMessage))
            local textWidth = xMax - xMin

            rNewsText.textWidth = textWidth
            
            self.nTotalNewsWidth = self.nTotalNewsWidth + textWidth + kMESSAGE_GAP
            
            local x, y = rNewsText:getLoc()
            rNewsText:setLoc(currentX, y)

            local bHasLink = false
            if rNewsText.link then
                bHasLink = true
            end
            
            local rLinkBox = self.tNewsLinkBoxes[i]
            
            if bHasLink then
                local linkMinX, linkMinY, linkMaxX, linkMaxY = rNewsText:getStringBounds(rNewsText.linkStart, rNewsText.linkEnd - rNewsText.linkStart)
            
                local nLinkX = currentX + linkMinX
                local nLinkWidth = linkMaxX - linkMinX
                
                rLinkBox:setLoc(nLinkX, y - 40)
                rLinkBox:setScl(nLinkWidth - 12, 5)
                rLinkBox.offsetX = nLinkX - currentX
                rLinkBox:setVisible(true)
            else
                rLinkBox:setVisible(false)
            end
            
            currentX = currentX + textWidth + kMESSAGE_GAP
        end
    end
    
    function Ob:showNewBuildPop(sNewVersionId)
        local function onChoice(nResult)
            if MOAIDialogIOS then
                if nResult == MOAIDialogIOS.DIALOG_RESULT_POSITIVE then
                    if MOAISafariIOS then
                        MOAISafariIOS.openURL("itms-apps://ax.itunes.apple.com/WebObjects/MZStore.woa/wa/viewSoftware?id=536013980") --"itms://itunes.com/apps/middle-manager-of-justice")
                    end
                end
            end
        end
        
        if MOAIDialogIOS then
        
            local sTitle = g_DialogManager:getString("STXT202TEXT")
            local sMessage = g_DialogManager:getString("STXT204TEXT")
            
            -- replace the version ID tag
            sMessage = string.gsub(sMessage, "%%1", sNewVersionId)
            local sYes = "Yes"
            local sNo = g_DialogManager:getString("STXT203TEXT")
            
            MOAIDialogIOS.showDialog( sTitle, sMessage, sYes, sNo, nil, false, onChoice )
        end
    end
    
    function Ob:update()
        DFNetNews.updateData()
        if DFNetNews.hasNewData() then
            local tNewsData = DFNetNews.data
            DFNetNews.clearNewData()
            
            self:onNewsLoaded(tNewsData)
        end
    
        for i, rNewsText in ipairs(self.tNewsTexts) do
            if self.bShowNewsMessages then
                local textX, textY = rNewsText:getLoc()
                textX = textX - kTEXT_SCROLL_SPEED           
                rNewsText:setLoc(textX, textY)
                
                local rLinkBox = self.tNewsLinkBoxes[i]
                local linkX, linkY = rLinkBox:getLoc()
                linkX = linkX - kTEXT_SCROLL_SPEED
                rLinkBox:setLoc(linkX, linkY)
                
                local bScrolledOffScreen = self:hasScrolledOffScreen(rNewsText)
                
                if bScrolledOffScreen then
                    -- move to the end
                    if self.nTotalNewsWidth and rNewsText.textWidth then
                        local nEndOfLine = self.nTotalNewsWidth - rNewsText.textWidth
                        rNewsText:setLoc(nEndOfLine, textY)

                        local nLinkOffsetX = rLinkBox.offsetX or 0
                        rLinkBox:setLoc(nEndOfLine + nLinkOffsetX, linkY)
                    end
                end
            end
        end
    end
    
    function Ob:hasScrolledOffScreen(rText)
        local bOffScreen = false
        
        local x, y = rText:getLoc()
        if rText.textWidth then
            if (x + rText.textWidth) < 0 then
                bOffScreen = true
            end
        end
        
        return bOffScreen
    end

    function Ob:onViewportResized()
        local screenX = (Renderer.getUIWidth() - 1024) / 2
        local screenY = (Renderer.getUIHeight() - 768) / 2    
        if self.rNewsBG then
            self.rNewsBG:setLoc(screenX, -(screenY + 490))
--            self.rNewsBG:setScl(screenX, screenY)
        end
        if self.rBackground then
            self.rBackground:setLoc(screenX, -screenY)
        end
        if self.rPlayButton then
            self.rPlayButton:setLoc(screenX + 78, -(screenY + 601))
        end
        if self.rCloudButton then
            self.rCloudButton:setLoc(screenX + 378, -(screenY + 601))
        end
        if self.rLoadingText then
            self.rCloudButton:setLoc(0, -(screenY + 516))
        end
        self.nNewsY = -(screenY + 516)
        for i, rNewsText in ipairs(self.tNewsTexts) do
            rNewsText:setLoc(1024, self.nNewsY)
        end
    end
    
    local screenX = (Renderer.getUIWidth() - 1024) / 2
    local screenY = (Renderer.getUIHeight() - 768) / 2
    
	Ob.rNewsBG = MOAIProp2D.new()
	Ob.rNewsBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rNewsBG:setLoc(screenX, -(screenY + 490))
	Ob.rNewsBG:setScl(1024, 204)
	Ob.rNewsBG:setColor(170/255, 249/255, 249/255, 1)
    Ob:addOverlayElement(Ob.rNewsBG)   
    
    Ob.tNewsTexts = {}
    Ob.tNewsLinkBoxes = {}
    
    local sPlayButtonText = g_DialogManager:getString("STXT189TEXT")
    local sCloudButtonText = g_DialogManager:getString("STXT191TEXT")
    
    if g_Game:isNewGame() then
        sPlayButtonText = g_DialogManager:getString("STXT190TEXT")
    end
    
    -- num pixels per frame
    kTEXT_SCROLL_SPEED = 1
    kMESSAGE_GAP = 32
    kNEWS_BUTTON_WIDTH = 80
    
    Ob.nNewsY = -(screenY + 516)
    
    -- create a bunch of text fields to scroll the news
    for i=1,6 do
        local rNewsText = g_GUI:CreateTextBox(40, MOAITextBox.LEFT_JUSTIFY, 'bangers')
        rNewsText:setRect(0, 1000, 4096, 0)
        rNewsText:setLoc(1024, Ob.nNewsY)
        
        local sMessage = "NEWS NEWS NEWS HEY HERE'S NEWS!"
        
        rNewsText:setString(sMessage)
        rNewsText:setColor(0, 0, 0)
        Ob:addOverlayElement(rNewsText)    
        
        table.insert(Ob.tNewsTexts, rNewsText)
        
        local rNewsUnderline = MOAIProp2D.new()
        rNewsUnderline:setDeck(g_GUI:GetOnePixelDeck())
        rNewsUnderline:setLoc(0, 0)
        rNewsUnderline:setScl(nWidth, nHeight)
        rNewsUnderline:setColor(0, 0, 0, 1)
        Ob:addOverlayElement(rNewsUnderline)   
        table.insert(Ob.tNewsLinkBoxes, rNewsUnderline)
    end
    
    tNewsMessage  = { msg = "Thanks for downloading Middle Manager of Justice! -- ", link = "http://www.doublefine.com/forums/viewforum/42/" }
    tNewsMessage2 = { msg = "Manage Justice TODAY! -- " }
    tDefaultNews = {latestVersion = MOAIEnvironment.appVersion, news = { tNewsMessage, tNewsMessage2 }}
    Ob:onNewsLoaded(tDefaultNews)
    
    Ob.rBackground = SpriteSheet:new('UI/splash', nil, 0, 0, 'left', 'top')
    Ob.rBackground:setLoc(screenX, -screenY)
    Ob:addOverlayElement(Ob.rBackground) 
    
    -- add the buttons
    Ob.rPlayButton = MainMenuButton:new(rRenderLayer, Ob, sPlayButtonText, { 229, 216, 7 })
    Ob.rPlayButton:setLoc(screenX + 78, -(screenY + 601))
    
    Ob.rCloudButton = MainMenuButton:new(rRenderLayer, Ob, sCloudButtonText, { 209, 118, 38 })
    Ob.rCloudButton:setLoc(screenX + 378, -(screenY + 601))
    
    Ob.rLoadingText = g_GUI:CreateTextBox(40, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rLoadingText:setRect(0, 1000, Renderer.getUIWidth(), 0)
	Ob.rLoadingText:setLoc(0, -(screenY + 516))
    Ob.rLoadingText:setString("LOADING")
    Ob.rLoadingText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rLoadingText)   
    
    --Ob:showLoadingMessage("LOADING NEWS")
    Ob:showLoadingMessage(nil)
    
    return Ob
end

function SplashScreen:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:onPaneClosed()
        self.rDracogenLogo:unloadSpriteSheet()
        self.rMainMenuScreen:unloadSpriteSheet()
        self.rDFLogo:unloadSpriteSheet()
    end

    function Ob:showDracogenLogo()   
        self.rScreen = self.rDracogenLogo
        self:getRenderLayer():insertProp(self.rScreen)
        self.rScreen:setLoc(Renderer.getUIWidth() / 2, -Renderer.getUIHeight() / 2)
        self.bDracogenLogoShowing = true
    end

    function Ob:clearDracogenLogo()
        self:getRenderLayer():removeProp(self.rScreen)
        self.bDracogenLogoShowing = false
    end

    function Ob:showDFLogo()
        self.rScreen = self.rDFLogo
        self:getRenderLayer():insertProp(self.rScreen)
        self.rScreen:setLoc(Renderer.getUIWidth() / 2, -Renderer.getUIHeight() / 2)
        self.bDFLogoShowing = true        
    end

    function Ob:clearDFLogo()
        self:getRenderLayer():removeProp(self.rScreen)
        self.bDFLogoShowing = false
    end

    function Ob:showMainMenu()
        self.rScreen = self.rMainMenuScreen
        self.rScreen:setVisible(true)
        self.bLogoShowing = true
    end

    function Ob:hideMainMenu()
        self.rScreen:setVisible(false)
        self.bLogoShowing = false
    end

    function Ob:splashScreenThread()
        if MOAIEnvironment.osBrand ~= "iOS" then
            self:showDFLogo()
            self.rForeground:moveColor(0, 0, 0, -1, 1)
            util.sleep(3)
            self.rForeground:moveColor(0, 0, 0, 1, 0.5)
            util.sleep(0.5)
            self:clearDFLogo()
        end
        self:showDracogenLogo()
        self.rForeground:moveColor(0, 0, 0, -1, 1)
        util.sleep(3)
        self.rForeground:moveColor(0, 0, 0, 1, 0.5)
        util.sleep(0.5)
        self:clearDracogenLogo()
        self.rForeground:moveColor(0, 0, 0, -1, 0.5)
        
        
        self:showMainMenu()

        DFGraphics.blockOnAsyncTextureLoad()

        input:init()

        self.bHandleViewportResize = false
        --self:hideMainMenu()
        --g_GUI:HideSplashScreen()                    
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
	        self.rForeground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
        
            self.rSplashScreenThread = MOAIThread.new()
            self.rSplashScreenThread:run(self.splashScreenThread, self)
        end
	end

    function Ob:update()
        if self.rScreen then
            if self.rScreen.bAutoCenter then
                self.rScreen:setLoc(Renderer.getUIWidth() / 2, -Renderer.getUIHeight() / 2)
            end

            if self.rScreen.update then
                self.rScreen:update()
            end
        end
    end

    function Ob:onLoadGame(bFromCloud)
        if self.bLogoShowing and not self.bIsLoading then
            local function loadFunc()     
                if self.rScreen.showLoadingMessage then
                    self.rScreen:showLoadingMessage("LOADING")
                end
                
                util.sleep(0.1)
     
                DFGraphics.blockOnAsyncTextureLoad()
                
                if not bFromCloud then
                    SetGameState("Load")
                end
                
                if g_Game:wasUserSaveReset() then
                    g_GUI:ShowLetterScreen(g_DialogManager:getString("STXT174TEXT"), g_DialogManager:getString("STXT175TEXT"), self)
                else
                    self:onClose()
                end
            end
            
            self.rThread = MOAIThread.new()
            self.rThread:run(loadFunc, self)   
        end
    end
    
    function Ob:onCloudSync()
        local bShowSaveButton = g_Game:getTutorialManager():hasTutorialCompleted('PatrolTutorial')
        
        g_GUI:ShowCloudScreen(bShowSaveButton, self)
    end
    
    function Ob:onCloudSaveLoaded(tSave)
		self:onLoadGame(true)
        SetGameState("Reload")
    end
    
	
	function Ob:onFinger(eventType, sx, sy, tProps)  
        if self.rScreen and self.rScreen.onFinger then
            self.rScreen:onFinger(eventType, sx, sy, tProps)
        end

		return true
	end
    
    function Ob:onLetterClosed()
        self:onClose()
    end
    
    function Ob:onClose()
        self:hideMainMenu()
        g_GUI:HideSplashScreen()                        
    end
    
    function Ob:onViewportResized()
        if self.bHandleViewportResize then
            if self.rBackground then
                self.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
                self.rBackground:setLoc(0, 0)
            end
            if self.rForeground then
                self.rForeground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
                self.rForeground:setLoc(0, 0)
            end
            if self.rMainMenuScreen then
                self.rMainMenuScreen:onViewportResized()
            end
        end
    end

    g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_IntroLoop', false)

    Ob.bHandleViewportResize = true
    
	Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(0, 0, 0, 1)
	Ob.rBackground:setLoc(0, 0)
    Ob:addElement(Ob.rBackground)      

    Ob.rMainMenuScreen = MainMenuUI:new(rRenderLayer, Ob)
    
	Ob.rForeground = MOAIProp2D.new()
	Ob.rForeground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rForeground:setLoc(0, 0)
	Ob.rForeground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rForeground:setColor(0, 0, 0, 1)
	Ob.rForeground:setLoc(0, 0)
    Ob:addOverlayElement(Ob.rForeground)      
    
    Ob.rDracogenLogo = SpriteSheet:new('UI/dracogen', nil, 0, 0, 'center', 'center')
    Ob.rDracogenLogo.bAutoCenter = true
    
    Ob.rDFLogo = SpriteSheet:new('UI/dflogo', nil, 0, 0, 'center', 'center')
    Ob.rDFLogo.bAutoCenter = true

	return Ob
end

return SplashScreen
