local Renderer = require('Renderer')
local util = require("DFUtil")
local DFGraphics = require("DFCommon.Graphics")
local EnemyDetailButton = require('ui/EnemyDetailButton')
local Character = require('Character')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local Button = require('ui/Button')
local Event = require('event/Event')
local TerritoryManager = require('Territory')
local TerritoryMapUI = {}
local MapPane = {}
local TerritoryButton = {}
local TerritoryPane = {}
local TerritoryEventButton = {}
local CrimeEventDetailPane = {}
local TerritoryRewardButton = {}

TerritoryMapUI.eMapModes = 
{
    MM_AreaSelect = 1,
    MM_EventSelect = 2,
}

local tUITextureSpriteSheetInfo =
{
back_button_blue = 'UI/hud',
back_button_blue_tap = 'UI/hud',
icon_cash = 'UI/hud',
icon_pp = 'UI/hud',
icon_time = 'UI/hud',
map_but_event_on = 'UI/eventdetails',
map_but_event_off = 'UI/eventdetails',
map_but_rare_on = 'UI/eventdetails',
map_but_rare_off = 'UI/eventdetails',
map_but_mostwanted_on = 'UI/eventdetails',
map_but_mostwanted_off = 'UI/eventdetails',
map_but_godzilla_on = 'UI/eventdetails',
map_but_godzilla_off = 'UI/eventdetails',
map_but_boss_on = 'UI/eventdetails',
map_but_boss_off = 'UI/eventdetails',
map_icon_badguy = 'UI/eventdetails',
map_icon_boss = 'UI/eventdetails',
map_icon_fire = 'UI/eventdetails',
map_icon_runawaycar = 'UI/eventdetails',
map_icon_valentine = 'UI/eventdetails',
map_icon_mostwanted = 'UI/eventdetails',
map_icon_godzilla = 'UI/eventdetails',
twohb = 'UI/hud2',
button_buy = 'UI/hud',
morale_superlow = 'UI/hud',
morale_low = 'UI/hud',
morale_med = 'UI/hud',
morale_high = 'UI/hud',
morale_superhigh = 'UI/hud',
map_coinbubble = 'UI/hud2',
star_rating1 = 'UI/eventdetails',
star_rating2 = 'UI/eventdetails',
star_rating3 = 'UI/eventdetails',
star_rating4 = 'UI/eventdetails',
}

local tMapInitLoc =
{
    { -474, 794 },
    { -98, 794 },
    { 0, 536 },
    { 0, 126 },
    { -132, -9 },
    { -474, -100 },
    { -474, 67 },
    { -337, 319 },
    { -474, 633 },
    { -474, 491 },
}

local tRatingToIcon =
{
    'star_rating1',
    'star_rating2',
    'star_rating3',
    'star_rating4',
}    

function CrimeEventDetailPane:new(rRenderLayer, rParentUIPane, rEvent)
    local Ob = MOAITransform.new()
	Ob.tElements = {}
    Ob.rRenderLayer = rRenderLayer
    Ob.rEvent = rEvent
    Ob.rParentUIPane = rParentUIPane
    Ob.bVisible = false

	function Ob:addElement(rElement)
		rElement:setParent(self)
		table.insert(self.tElements, rElement)
	end

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setColor(0, 0, 0)
    Ob:addElement(Ob.rBackground)

    Ob.rEnemiesLabelText = g_GUI:CreateTextBox(3, MOAITextBox.CENTER_JUSTIFY)
    Ob.rEnemiesLabelText:setLoc(0, 10)
    Ob.rEnemiesLabelText:setString("ENEMIES")
    Ob.rEnemiesLabelText:setColor(1, 0, 0)
    Ob:addElement(Ob.rEnemiesLabelText)

    Ob.tButtons = {}
    local nNumItems = 0
    local kYSIZE = 8
    local nInitY = 0
    for actorName, tActor in pairs(rEvent.rDef.tActors) do
        if actorName ~= 'Citizen' then
            local rCharDef = Character.getCharDef(tActor.sCharDefKey)
            if rCharDef then            
                local rButton = EnemyDetailButton:new(Ob.rRenderLayer, rCharDef)
                rButton:setLoc(0, nInitY - nNumItems * kYSIZE)
                rButton:setScl(0.1, 0.1)
                table.insert(Ob.tButtons, rButton)
                rButton:setParent(Ob)
                nNumItems = nNumItems + 1
            end
        end
    end
    local nButtonY = - (nNumItems * kYSIZE + 8)
    Ob.rCancelButton = MOAIProp2D.new()
    Ob.rCancelButton:setDeck(g_GUI:GetGreenButtonDeck())
    Ob.rCancelButton:setColor(1, 0, 0)
    Ob.rCancelButton:setLoc(-12, nButtonY)
    Ob.rCancelButton:setScl(0.03, 0.075)
    Ob:addElement(Ob.rCancelButton)

    Ob.rCancelButtonText = g_GUI:CreateTextBox(2, MOAITextBox.CENTER_JUSTIFY)
    Ob.rCancelButtonText:setString("CLOSE")
    Ob.rCancelButtonText:setColor(1, 1, 1)
    Ob.rCancelButtonText:setLoc(-12, nButtonY + 1)
    Ob:addElement(Ob.rCancelButtonText)

    Ob.rAttemptButton = MOAIProp2D.new()
    Ob.rAttemptButton:setDeck(g_GUI:GetGreenButtonDeck())
    Ob.rAttemptButton:setLoc(12, nButtonY)
    Ob.rAttemptButton:setScl(0.03, 0.075)
    Ob:addElement(Ob.rAttemptButton)


    Ob.rAttemptButtonText = g_GUI:CreateTextBox(2, MOAITextBox.CENTER_JUSTIFY)
    Ob.rAttemptButtonText:setString("ATTEMPT")
    Ob.rAttemptButtonText:setColor(1, 1, 1)
    Ob.rAttemptButtonText:setLoc(12, nButtonY + 1)
    Ob:addElement(Ob.rAttemptButtonText)

    Ob.rBackground:setScl(26, nNumItems * kYSIZE + 14)

	function Ob:setVisible(bVisible)
		if not self.rRenderLayer then
			return
		end
        self.bVisible = bVisible
		if bVisible then
			for i, rElement in ipairs(self.tElements) do
				self.rRenderLayer:insertProp(rElement)
			end
            for i, rButton in ipairs(self.tButtons) do
                rButton:setVisible(true)
            end
		else
			for i, rElement in ipairs(self.tElements) do
				self.rRenderLayer:removeProp(rElement)
			end
            for i, rButton in ipairs(self.tButtons) do
                rButton:setVisible(false)
            end
		end
	end    

    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            for i, rProp in ipairs(tProps) do                
                if rProp == self.rCancelButtonText or rProp == self.rCancelButton then
                    self.rParentUIPane:onSelected(false)
                    return true
                elseif rProp == self.rAttemptButtonText or rProp == self.rAttemptButton then
                    g_GUI:ShowEventDetailUI(self.rEvent)
                end            
            end
        end        
    end

    return Ob
end

TerritoryEventButton.eventTypeInfo =
{
    crime = { sText = "CRIME!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_event_off', sButtonPressedSprite = 'map_but_event_on', sIconSprite = 'map_icon_badguy' },
    mission = { sText = "MISSION!!", color = { r = 0, g = 0, b = 1 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_boss_off', sButtonPressedSprite = 'map_but_boss_on', sIconSprite = 'map_icon_boss' },
    fire = { sText = "FIRE!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_event_off', sButtonPressedSprite = 'map_but_event_on', sIconSprite = 'map_icon_fire' },
    busrescue = { sText = "RESCUE!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_event_off', sButtonPressedSprite = 'map_but_event_on', sIconSprite = 'map_icon_runawaycar' },
    seniorcrime = { sText = "SR. CRIME!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_rare_off', sButtonPressedSprite = 'map_but_rare_on', sIconSprite = 'map_icon_badguy' },
    mostwanted = { sText = "MOST WANTED!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_mostwanted_off', sButtonPressedSprite = 'map_but_mostwanted_on', sIconSprite = 'map_icon_mostwanted' },
    godzilla = { sText = "GODZILLA!!", color = { r = 1, g = 0, b = 0 }, detailPaneConstructor = CrimeEventDetailPane.new, sButtonSprite = 'map_but_godzilla_off', sButtonPressedSprite = 'map_but_godzilla_on', sIconSprite = 'map_icon_godzilla' },
    
}

function TerritoryEventButton:new(rRenderLayer, rParentUIPane, rEvent)
    if not rEvent then
        print("ERROR: No event for TerritoryEventButton")
        return nil
    end
    local sEventType = rEvent:getEventType()
    if not sEventType then
        print("ERROR: No event type for TerritoryEventButton")
        return nil
    end

    local rTypeInfo = TerritoryEventButton.eventTypeInfo[sEventType]
    if not rTypeInfo then
        print("ERROR: Unknown type for TerritoryEventButton")
        return nil
    end

    local Ob = UIElement:new(rRenderLayer)

    function Ob:onPressed(bPressed)
        self.rBackground:setVisible(not bPressed)
        self.rPressedBackground:setVisible(bPressed)
    end

    function Ob:update()
        if self.rEvent then
            local bAutoResolving, nRemainingTime = self.rEvent:isAutoResolving()
            if bAutoResolving then
                self.rIcon:setVisible(false)
                self.rRatingIcon:setVisible(false)
                self.rAutoResolveLabel:revealAll()
                if nRemainingTime <= 0 then
                    self.rAutoResolveLabel:setString("COMPLETE!")
                    self.rTimeText:setReveal(0)
                else
                    self.rAutoResolveLabel:setString("FIGHTING")
                    self.rTimeText:setColor(0, 0, 0)
                    self.rTimeText:setLoc(50, -80)
                    self.rTimeText:setString(util.convertTimeString(nRemainingTime))
                end
            else
                if self.rTimeText then
                    self.rTimeText:setReveal(0)
                    self.rTimeText:setColor(.51, .15, .07)
                    self.rTimeText:setLoc(69, -80)
                    local nTimeLeftAtRating, nRating = self.rEvent:getRemainingTimeAtCurStarLevel()
                    self.rTimeText:setString(util.convertTimeString(nTimeLeftAtRating))
                    local sRatingIcon = tRatingToIcon[nRating]
                    if sRatingIcon then
                        self.rRatingIcon = self:getUITextureProp(sRatingIcon, self.rRatingIcon)
                    end
                end
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rEvent = rEvent
    Ob.rParentUIPane = rParentUIPane
 
    Ob.rBackground = Ob:getUITextureProp(rTypeInfo.sButtonSprite)
    Ob:addElement(Ob.rBackground)

    Ob.rPressedBackground = Ob:getUITextureProp(rTypeInfo.sButtonPressedSprite)
    Ob.rPressedBackground:setVisible(false)
    Ob:addElement(Ob.rPressedBackground)

    local sSpriteName = rTypeInfo.sIconSprite
    if rEvent:getDef().sMapButtonIconOverride then
        sSpriteName = rEvent:getDef().sMapButtonIconOverride
    end
    Ob.rIcon = Ob:getUITextureProp(sSpriteName)
    local iconX
    --hack to fix some centering issues with some icons
    if sSpriteName == 'map_icon_badguy' then
        iconX = 63
    else
        iconX = 55
    end
    Ob.rIcon:setLoc(iconX, -24)
    Ob:addElement(Ob.rIcon)

    if rEvent and not rEvent:isAutoResolving() then
        rEvent:enableTimeIncrement(true)
    end

    Ob.rRatingIcon = Ob:getUITextureProp('star_rating1')
    Ob.rRatingIcon:setScl(0.8)
    Ob.rRatingIcon:setLoc(26, -76)
    Ob:addElement(Ob.rRatingIcon)
    if rEvent then
        Ob.rTimeText = g_GUI:CreateTextBox(20, MOAITextBox.LEFT_JUSTIFY, 'type')
        Ob.rTimeText:setRect(0, 80, 200, 0)
        Ob.rTimeText:setString(util.convertTimeString(rEvent:getElapsedTime()))
        Ob.rTimeText:setLoc(69, -80)
        Ob:addElement(Ob.rTimeText)
    end

    Ob.rAutoResolveLabel = g_GUI:CreateTextBox(34, MOAITextBox.CENTER_JUSTIFY, 'bangers')
    Ob.rAutoResolveLabel:setRect(0, 80, 200, 0)
    Ob.rAutoResolveLabel:setString("FIGHTING")
    Ob.rAutoResolveLabel:setLoc(-20, -24)
    Ob.rAutoResolveLabel:setReveal(0)
    Ob:addElement(Ob.rAutoResolveLabel)

    local tHeroes = rEvent:getAllHeroes()

    function Ob:hasEventAssigned()
        local bHasEventAssigned = false        
        if self.rEvent and not self.rEvent:hasCompleted() or not self.rEvent:isSuccessful() then
            local tHeroes = self.rEvent:getAllHeroes()
            if table.getn(tHeroes) > 0 then
                bHasEventAssigned = true
            end
        end
        return bHasEventAssigned
    end

    function Ob:getEvent()
        return self.rEvent
    end

    function Ob:runAttemptSequence()
    end

    function Ob:startupThread()
        if self.rEvent and self.rEvent:hasCompleted() then
            if self.rEvent:isSuccessful() then
                self.rEvent.rTerritory:removeEvent(self.rEvent)
                util.sleep(0.5)               
                self:setVisible(false)
                util.sleep(0.5)               
                self.rParentUIPane:runIncrementCrimeLevel(0 - self.rEvent:getCrimeLevelReduction())
            else        
                self.rEvent:setCompleted(false) -- clear out
            end
        end
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
	end    

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_DOWN then
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rBackground or rProp == self.rPressedBackground then
                        self:onPressed(true)
                        return true                        
                    end
                end
            end                    
        elseif eventType == MOAITouchSensor.TOUCH_UP then
            self:onPressed(false)
            if g_Game:isActionEnabled('OpenMapEvent') then
                if tProps then
                    for i, rProp in ipairs(tProps) do
                        if rProp == self.rBackground or rProp == self.rPressedBackground then
                            -- show event detail
                            g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                            local bAutoResolving, nRemainingTime = rEvent:isAutoResolving()
                            if bAutoResolving then
                                if nRemainingTime <= 0 then
                                    g_eventManager:autoResolveEvent(self.rEvent)
                                end
                            else
                                g_GUI:ShowEventDetailUI(self.rEvent)
                            end
                            return true                        
                        end
                    end
                end        
            end
        end
        return true
    end

    return Ob
end

function TerritoryRewardButton:new(rRenderLayer, rTerritory)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if not bVisible then
            if self.rThread then
                self.rThread:stop()
            end
            self:getRenderLayer():removeProp(self.rCoin)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if g_Game:isActionEnabled('CollectMapCoins') then
            if self.nAmount > 0 and self.rTerritory then
                if tProps and eventType == MOAITouchSensor.TOUCH_UP then
                    for i, rProp in ipairs(tProps) do
                        if rProp == self.rBackground then
                            if not self.bCollecting then
                                self.rThread = MOAIThread.new()
                                self.rThread:run(self.collectionThread, self)
                            end
                            return true
                        end
                    end
                end
            end
        end
        return true
    end

    function Ob:collectionThread()
        local nAmount = self.rTerritory:collectHappinessReward()
        self.bCollecting = true
        g_Audio:playCue('MMOJ_Audio/UI/CollectCoin')
        self.rCoin:playAnimation('coinbling', false, false, true)
        self:getRenderLayer():removeProp(self.rCoin)
        self.rRewardAmountText:setString("+"..nAmount)
        self.rRewardAmountText:setColor(0, 0, 0, 1)
        self.rRewardAmountText:revealAll()
        self.rRewardAmountText:moveColor(0, 0, 0, -1, 0.75)
        util.sleep(0.75)
        self.rBackground:setScl(1)
        self.rBackground:moveScl(-0.9, -0.9, 0.25)
        self.rBackground:moveLoc(50, -50, 0.25)
        util.sleep(0.2)
        self.rBackground:setVisible(false)
        self.bCollecting = false
        self.bShowing = false
    end

    function Ob:showThread()
        self.bShowThreadRunning = true
        self.bShowing = true
        self.rBackground:setScl(0.1, 0.1)
        self.rBackground:setVisible(true)
        self.rBackground:seekScl(1.0, 1.0, 0.25)
        self.rBackground:moveLoc(-50, 50, 0.25)
        util.sleep(0.25)
        self:getRenderLayer():insertProp(self.rCoin)
        self.rCoin:setScl(0.5, 0.5, 0.25)
        self.rCoin:moveLoc(0.5, -2.5, 0.1)
        self.rCoin:playAnimation('coinpulsate')
        self.bShowThreadRunning = false
    end

    function Ob:update()
        if g_Game:isActionEnabled('CollectMapCoins') then
            if self.rTerritory then
                self.nAmount = self.rTerritory:getHappinessRewardAmount()
                if self.nAmount > 0 then               
                    if not self.bShowing and not self.bShowThreadRunning then

                        self.rThread = MOAIThread.new()
                        self.rThread:run(self.showThread, self)
                    end
                end
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    Ob.rTerritory = rTerritory

    Ob.rBackground = Ob:getUITextureProp('map_coinbubble')
    Ob.rBackground:setVisible(false)
    Ob.rBackground:setLoc(50, -50)
    Ob:addElement(Ob.rBackground)

    Ob.rCoin = SpriteSheet:new('FX/FX_5', 'Characters/AnimDefs/FX_5.lua', nil, nil, nil, 'center', 'center')
    Ob.rCoin:setLoc(26, -18)
    Ob.rCoin:setParent(Ob.rBackground)

    Ob.rRewardAmountText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'bangers')
    Ob.rRewardAmountText:setRect(0, 80, 200, 0)
    Ob.rRewardAmountText:setString("400")
    Ob.rRewardAmountText:setLoc(8, 0)
    Ob.rRewardAmountText:setReveal(0)
    Ob:addElement(Ob.rRewardAmountText)

    Ob.bCollecting = false
    Ob.nAmount = 0

    Ob.bShowing = false
    Ob.bShowThreadRunning = false
    
    return Ob
end

function TerritoryPane:new(rRenderLayer, rParentUIPane, rTerritory, eMapMode, nActivityIndex, rHero)
    local Ob = UIElement:new(rRenderLayer)

    function Ob:getTerritory()
        return self.rTerritory
    end

    function Ob:runIncrementCrimeLevel(nCrimeAmount)
        if self.rTerritory:hasActiveMission() and nCrimeAmount ~= 0 then
            return
        end
        local nCurCrimeLevel = self.rTerritory:getCrimeLevel()
        if nCrimeAmount > 0 then
            if nCurCrimeLevel < 100 then
                self.rTerritory:addCrimeAmount(nCrimeAmount)
                nCurCrimeLevel = self.rTerritory:getCrimeLevel()
            end
        else
            if nCurCrimeLevel > 0 then
                self.rTerritory:addCrimeAmount(nCrimeAmount)
                nCurCrimeLevel = self.rTerritory:getCrimeLevel()
                util.sleep(1)
                if nCurCrimeLevel == 0 then
                    local rRewardInfo = g_Game:generateReward()
                    if rRewardInfo then
                        g_GUI:ShowRewardPane(rRewardInfo)
                    end
                    self.rTerritory:setCleared(true)
                end
            end
        end
    end

    function Ob:runAttemptSequence()
        for i, rPin in ipairs(self.tEventPins) do
            rPin:runAttemptSequence()
        end
    end

    function Ob:startupThread()   
        for i, rPin in ipairs(self.tEventPins) do
            rPin:startupThread()
        end
    end

    function Ob:hasEventAssigned()
        local bHasEventAssigned = false
        for i, rPin in ipairs(self.tEventPins) do
            if rPin:hasEventAssigned() then
                bHasEventAssigned = true
                break
            end
        end
        return bHasEventAssigned
    end

    function Ob:getQueuedEvents()
        local tEvents = {}
        for i, rPin in ipairs(self.tEventPins) do
            if rPin:hasEventAssigned() then
                table.insert(tEvents, rPin:getEvent())
            end
        end
        return tEvents
    end
    
	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
		if bVisible then
            for i, rPin in ipairs(self.tEventPins) do
                rPin:setVisible(true)
            end
		else
            for i, rPin in ipairs(self.tEventPins) do
                rPin:setVisible(false)
            end
		end
        if self.rRewardButton then
            self.rRewardButton:setVisible(bVisible)
        end
	end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        for i, rPin in ipairs(self.tEventPins) do
            rPin:onPaneClosed()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if not self.rTerritory or not self.rTerritory:getUnlocked() then
            return false
        end

        if self.eMapMode == TerritoryMapUI.eMapModes.MM_AreaSelect then
            if eventType == MOAITouchSensor.TOUCH_UP then
                if tProps then
                    for i, rTouchedProp in ipairs(tProps) do
                        if rTouchedProp == self.rFakeButton then
                            self:addPatrolActivity()
                            g_GUI:HideTerritoryMap()
                            return true
                        end
                    end
                end
            end
        end
        for i, rPin in ipairs(self.tEventPins) do
            rPin:onFinger(eventType, sx, sy, tProps)
        end
        if self.rRewardButton then
            self.rRewardButton:onFinger(eventType, sx, sy, tProps)
        end
        return true
	end

    function Ob:addPatrolActivity()
        if self.rHero and self.nActivityIndex then 
            if self.rTerritory then
                self.rHero:addPatrolActivityAtIndex(self.nActivityIndex, self.rTerritory)
            end        
        end
    end

    function Ob:HandleButtonPress(rButton)
        self.rParentUIPane:HandleButtonPress(rButton)
    end

    function Ob:HandlePinPress(rPin)
        if not rPin then
            return
        end
        if self.rCurSelectedPin then
            self.rCurSelectedPin:onSelected(false)
        end
        self.rCurSelectedPin = rPin
        self.rCurSelectedPin:onSelected(true)
    end

    function Ob:update()
        for i, rPin in ipairs(self.tEventPins) do
            rPin:update()
        end
        if self.rHappinessIcon and self.rTerritory then            
            local sIcon = self.rTerritory:getHappinessIconName()
            if sIcon then
                self:getUITextureProp(sIcon, self.rHappinessIcon)
            end
        end
        if self.rRewardButton then
            self.rRewardButton:update()
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rTerritory = rTerritory
    Ob.rParentUIPane = rParentUIPane
    Ob.eMapMode = eMapMode
    Ob.nActivityIndex = nActivityIndex
    Ob.rHero = rHero
    Ob.rCurSelectedPin = nil

    local rButtonDef = rTerritory:getMapUIInfo()

    -- add the name
    Ob.rTerritoryNameText = g_GUI:CreateTextBox(rButtonDef.textSize, MOAITextBox.CENTER_JUSTIFY)
    Ob.rTerritoryNameText:setRect(-150, 160, 0, 0)
    Ob.rTerritoryNameText:setLoc(0, -20)
    if Ob.rTerritory:getUnlocked() then
        Ob.rTerritoryNameText:setString(g_DialogManager:getString(rTerritory:getDef().name))
        Ob.rTerritoryNameText:setColor(1, 1, 1)
    else
        Ob.rTerritoryNameText:setString("LOCKED")
        Ob.rTerritoryNameText:setColor(.190, .190, .190)
    end
    -- disable the text for now
    Ob.rTerritoryNameText:setReveal(0)

    local sOverrideName = rTerritory:getDef().overrideName
    if sOverrideName then
        Ob.rTerritoryNameText:setString(g_DialogManager:getString(sOverrideName))
    end
    Ob:addElement(Ob.rTerritoryNameText)

    if Ob.rTerritory:getUnlocked() then
        -- add the crime level text    
        local nCrimeLevel = rTerritory:getCrimeLevel()
        nCrimeLevel = math.floor(nCrimeLevel)
    end

    Ob.rFakeButton = MOAIProp2D.new()
    Ob.rFakeButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rFakeButton:setLoc(rButtonDef.buttonInfo.xOffset, rButtonDef.buttonInfo.yOffset)
    Ob.rFakeButton:setScl(rButtonDef.buttonInfo.xScale, rButtonDef.buttonInfo.yScale)
    Ob.rFakeButton:setColor(0.5, 0.5, 0.5)
    Ob.rFakeButton:setVisible(false)
    Ob:addElement(Ob.rFakeButton)

    if Ob.rTerritory:getUnlocked() and g_Game:isActionEnabled('CollectMapCoins') then
        Ob.rHappinessIcon = Ob:getUITextureProp('morale_superhigh')
        Ob.rHappinessIcon:setLoc(rButtonDef.happinessLoc.x, rButtonDef.happinessLoc.y)
        Ob.rHappinessIcon:setScl(0.5)
        Ob:addElement(Ob.rHappinessIcon)

        Ob.rRewardButton = TerritoryRewardButton:new(rRenderLayer, rTerritory)
        Ob.rRewardButton:setLoc(rButtonDef.happinessLoc.x - 30, rButtonDef.happinessLoc.y + 35)
        Ob.rRewardButton:setParent(Ob)
    end

    Ob.tEventPins = {}
    local tEvents = rTerritory:getQueuedEvents()
    for i, rEvent in ipairs(tEvents) do
        local rLoc = rButtonDef.eventIndicatorLocs[i]
        if rLoc then
            local rEventPin = TerritoryEventButton:new(Ob.rRenderLayer, Ob, rEvent)    
            rEventPin:setParent(Ob)            
            rEventPin:setLoc(rLoc.x, rLoc.y)
            rEventPin:setScl(0.70, 0.70)

            table.insert(Ob.tEventPins, rEventPin)    
        end
    end 

    return Ob
end

function MapPane:new(rRenderLayer, eMapMode, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)
    
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:setVisible(bVisible)
        end
        if not bVisible then
            if self.rMapBG then
                self.rMapBG:unloadSpriteSheet()
            end
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:onPaneClosed()
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:onFinger(eventType, sx, sy, tProps)
        end
    end

    function Ob:runAttemptSequence()
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:runAttemptSequence()
        end
        -- done crime level adjustments
        for i, rPane in ipairs(self.tTerritoryPanes) do
            local tEvents = rPane:getQueuedEvents()
            for i, rEvent in ipairs(tEvents) do
                g_eventManager:queueEvent(rEvent)
            end
        end
    end

    function Ob:startupThread()       
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:startupThread()
        end        
    end

    function Ob:hasEventsAssigned()
        local bHasEventAssigned = false
        for i, rPane in ipairs(Ob.tTerritoryPanes) do
            if rPane:hasEventAssigned() then
                bHasEventAssigned = true
                break
            end
        end       
        return bHasEventAssigned
    end

    function Ob:update()
        for i, rPane in ipairs(self.tTerritoryPanes) do
            rPane:update()
        end
    end

	-- create the territory map UI
    Ob.rMapBG = SpriteSheet:new('UI/victimopolismap', nil, nil, nil, 'left', 'top')
    Ob.rMapBG:setSpriteIndex('victimopolismap_Med')
	Ob.rMapBG:setLoc(0, 0)
	Ob:addElement(Ob.rMapBG)

    Ob.eMapMode = eMapMode
    Ob.nActivityIndex = nActivityIndex
    
    Ob.tTerritoryPanes = {}
    local tTerritories = g_City:getTerritories()		
    for sTerritoryName, rTerritory in pairs(tTerritories) do			
        local rNewPane = TerritoryPane:new(rRenderLayer, Ob, rTerritory, Ob.eMapMode, nActivityIndex, rHero)
        rNewPane:setParent(Ob)
        local tMapUIInfo = rTerritory:getMapUIInfo()
        if tMapUIInfo then
            rNewPane:setLoc(tMapUIInfo.loc.x, tMapUIInfo.loc.y)
        end
        table.insert(Ob.tTerritoryPanes, rNewPane)
    end

    return Ob
end

function TerritoryMapUI:new(rRenderLayer, eMapMode, rHero, nActivityIndex)
	local Ob = UIElement:new(rRenderLayer)
    
	function Ob:setVisible(bVisible)
        self.rMapPane:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        self.rBackground:setPriority(0)
        self.rBackButton:setVisible(bVisible)
        if bVisible then
            g_Audio:playCue('MMOJ_Audio/UI/MapOpen')
            local tInitLoc = tMapInitLoc[g_MiddleManager:getLevel()]
            if tInitLoc then
                self.rMapPane:setLoc(tInitLoc[1], tInitLoc[2])
            end
            self.rStartupThread = MOAIThread.new()
            self.rStartupThread:run(self.startupThread, self)
            if g_CurrentMap then
                g_CurrentMap:onTerritoryMapOpened()
            end
        end
        for i, rHolidayUI in ipairs(self.tHolidayUI) do
            rHolidayUI:setVisible(bVisible)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
        self.rMapPane:onPaneClosed()
        self.rBackButton:onPaneClosed()
        for i, rHolidayUI in ipairs(self.tHolidayUI) do
            rHolidayUI:onPaneClosed()
        end
    end

    function Ob:moveMap(dx, dy)
        if g_Game:isActionEnabled('ScrollMap') then
            local x, y = self.rMapPane:getLoc()
            local newX = x + dx
            local newY = y - dy
            if newX > self.nMaxX or newX < self.nMinX then
                newX = x
            end
            if newY > self.nMaxY or newY < self.nMinY then
                newY = y
            end
            print("going to "..newX.." "..newY)
            self.rMapPane:setLoc(newX, newY)
        end
    end

    function Ob:setMapScl(nScale)
        self.rMapPane:setScl(nScale)
    end

    function Ob:moveMapScl(nScale, nTime)
        self.rMapPane:moveScl(nScale, nScale, 0, nTime)
    end
                           
    function Ob:setMapLoc(x, y)
        self.rMapPane:setLoc(x, y)
    end

    function Ob:moveMapLoc(x, y, nTime)
        self.rMapPane:moveLoc(x, y, 0, nTime)
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rBackButton then
                if rButton == self.rBackButton then
                    if g_Game:isActionEnabled('CloseWindow') then
                        g_GUI:HideTerritoryMap()
                        g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                    end
                end
            end
        end
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        for i, rHolidayUI in ipairs(self.tHolidayUI) do
            if rHolidayUI.onFinger then
                rHolidayUI:onFinger(eventType, sx, sy, tProps)
            end
        end
        local bPressedTopBG = false
        if eventType == MOAITouchSensor.TOUCH_UP or eventType == MOAITouchSensor.TOUCH_DOWN then
            for i, rProp in ipairs(tProps) do
                if rProp == self.rTopBG then
                    bPressedTopBG = true
                end
            end
        end
        self.rBackButton:onFinger(eventType, sx, sy, tProps)
        if self.bAttemptSequenceStarted or self.bStartupThreadStarted then
            return true
        end
        if not self.bFirstInputIgnored then
            self.bFirstInputIgnored = true
            return true
        end     
        if not self.bTouchMoved and not bPressedTopBG then
            self.rMapPane:onFinger(eventType, sx, sy, tProps)
        end
		if eventType == MOAITouchSensor.TOUCH_MOVE then
            if not self.lastSx then self.lastSx = sx end
            if not self.lastSy then self.lastSy = sy end
            
            local nScreenScaleX, nScreenScaleY = Renderer.getDisplayScl()
            
			local dx = (sx - self.lastSx) / nScreenScaleX
			local dy = (sy - self.lastSy) / nScreenScaleY
            local nDist = math.sqrt(dx * dx + dy *dy)
            local kTOUCHMOVE_SCROLL_THRESHOLD = 5
			if self.bTouchMoved or nDist > kTOUCHMOVE_SCROLL_THRESHOLD then
                self.bTouchMoved = true
                self.lastSx = sx
                self.lastSy = sy

                self:moveMap(dx, dy)
            end
        elseif eventType == MOAITouchSensor.TOUCH_DOWN then
            self.lastSx = sx
            self.lastSy = sy
        elseif eventType == MOAITouchSensor.TOUCH_UP then
            if not self.bTouchMoved then
                if tProps then
                    for i, rProp in ipairs(tProps) do
                        if rProp == self.rAttemptButton or rProp == self.rAttemptButtonText then
                            self.bAttemptSequenceStarted = true
                            self.rAttemptThread = MOAIThread.new()
                            self.rAttemptThread:run(self.attemptThread, self)
                            return true
                        end
                    end
                end
            end            
            self.bTouchMoved = false
        end
		return true
	end

    function Ob:attemptThread()
        if self.bAttemptSequenceStarted then
            self.rMapPane:runAttemptSequence()
            g_GUI:HideTerritoryMap()            
            g_eventManager:startNextEventInQueue()
            self.bAttemptSequenceStarted = false
        end        
    end

    function Ob:startupThread()
        self.bStartupThreadStarted = true
        self.rMapPane:startupThread()
        self.bStartupThreadStarted = false
    end

    function Ob:update()
        self.rMapPane:update()
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rHero = rHero
	Ob.nActivityIndex = nActivityIndex
    Ob.eMapMode = eMapMode
	Ob.tTerritoryPanes = {}
    Ob.bFirstInputIgnored = false

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(0, 0, 0)
    Ob:addElement(Ob.rBackground)

    -- add the map

    Ob.rTopBG = MOAIProp2D.new()
	Ob.rTopBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBG:setLoc(0, 0)
	Ob.rTopBG:setScl(Renderer.getUIWidth(), 90)
	Ob.rTopBG:setColor(0, 0, 0)
    Ob.rTopBG:setParent(Ob)
    Ob:addOverlayElement(Ob.rTopBG)

    -- add the back button
   Ob.rBackButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'back_button_blue', 'back_button_blue_tap')
    Ob.rBackButton:setLoc(0, 0)
    Ob.rBackButton:setParent(Ob)

    Ob.rDescriptionText = g_GUI:CreateTextBox(60, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rDescriptionText:setRect(-Renderer.getUIHalfWidth(), 80, Renderer.getUIHalfWidth(), 0)
    if eMapMode == TerritoryMapUI.eMapModes.MM_AreaSelect then
        Ob.rDescriptionText:setString("Select an area to patrol")
    else
        Ob.rDescriptionText:setString(g_DialogManager:getString("STXT083TEXT"))
    end
	Ob.rDescriptionText:setLoc(Renderer.getUIHalfWidth(), -6)
    Ob.rDescriptionText:setColor(1, 1, 1)
	Ob:addOverlayElement(Ob.rDescriptionText)    

    Ob.rMapPane = MapPane:new(rRenderLayer, eMapMode, rHero, nActivityIndex)
    Ob.rMapPane:setLoc(-474, 794)
    Ob.rMapPane:setScl(1.4, 1.4)
    Ob.rMapPane:setParent(Ob)

    Ob.nMinX = -474
    Ob.nMaxX = 0
    Ob.nMinY = -100
    Ob.nMaxY = 794

	-- text boxes
    Ob.bAttemptSequenceStarted = false
    Ob.bStartupThreadStarted = false

    Ob.tHolidayUI = {}
    local tUIClasses = g_GUI:GetHolidayGUI('TerritoryMapUI')
    if tUIClasses then
        for i, uiClass in ipairs(tUIClasses) do
            local rUI = uiClass:new(rRenderLayer)
            if rUI then
                table.insert(Ob.tHolidayUI, rUI)
            end
        end
    end

	return Ob
end

return TerritoryMapUI