local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local DFGraphics = require('DFCommon.Graphics')
local TickProgressBar = {}

-- quick UI Button
function TickProgressBar:new(rRenderLayer, nMaxWidth, tFullColor, tEmptyColor, tBGColor, tPendingColor)
	local Ob = UIElement:new(rRenderLayer)

    Ob.nMaxSquareWidth = 14
    Ob.nSquareMargin = 3

    local tUITextureSpriteSheetInfo = 
    {
    --HealthBar_Green = 'UI/fightscreen',
    --HealthBar_Red = 'UI/fightscreen',
    }

    --[[
    function Ob:setMaxTicks(nMaxTicks)
        if nMaxTicks ~= self.nMaxTicks then
            self.nMaxTicks = nMaxTicks
        -- Decide on # of squares, and their size.
        -- Assume horizontal layout.
        -- table of allowed tick widths, heights
        --
               

            self:update(self.nFull, self.nEmpty, self.Pending)
        end
    end
    ]]--
    
	-- functions
    function Ob:update(nFull, nEmpty, nPending, bForce)
        nFull = nFull or 0
        nEmpty = nEmpty or 0
        nPending = nPending or 0
        if not bForce and self.nFull == nFull and self.nEmpty == nEmpty and self.nPending == nPending then return end

        self.nFull = nFull
        self.nEmpty = nEmpty
        self.nPending = nPending

        self.nTotal = self.nEmpty+self.nFull+self.nPending

        -- calc square width: 
        -- nMaxWidth >= (nSquareMargin+squareWidth)*nTotal+nSquareMargin
        -- squareWidth = (nMaxWidth-nSquareMargin) / nTotal - nSquareMargin 

        self.nNumRows = 1
        self.nSquareHeight = 15
        self.nSquareWidth = math.floor( (self.nMaxWidth-self.nSquareMargin) / self.nTotal - self.nSquareMargin )
        while self.nSquareWidth < self.nSquareMargin do
            -- Uh-oh, time to go with more rows.
            self.nNumRows = self.nNumRows+1
            self.nSquareHeight = 6
            self.nSquareWidth = math.floor( self.nNumRows*(self.nMaxWidth-self.nSquareMargin) / self.nTotal - self.nSquareMargin )
        end
        self.nSquareWidth = math.min(self.nSquareWidth,self.nMaxSquareWidth)
        self.nMaxSquaresPerRow = math.floor((self.nMaxWidth - self.nSquareMargin) / (self.nSquareWidth+self.nSquareMargin))

        local nWidthInSquares = (self.nNumRows > 1 and self.nMaxSquaresPerRow) or self.nTotal
        self.nBGWidth = nWidthInSquares*(self.nSquareWidth+self.nSquareMargin)+self.nSquareMargin
        self.nBGHeight = self.nSquareMargin+self.nNumRows*(self.nSquareHeight+self.nSquareMargin)
        self.rBG:setScl(self.nBGWidth,self.nBGHeight)

        local nCurRow = 1
        local nCurX = self.nSquareMargin --+ self.nSquareWidth*.5
        local nCurY = -self.nSquareMargin --+ self.nSquareHeight*.5
        for i=1,self.nTotal do
            if not self.squares[i] then
                self.squares[i] = MOAIProp2D.new()
                self.squares.n = i
                self.squares[i]:setDeck(g_GUI:GetOnePixelDeck())
                self:addElement(self.squares[i])
                if self.bVisible then
                    self.rRenderLayer:insertProp(self.squares[i])
                end
            end
            self.squares[i]:setLoc(nCurX, nCurY)
            self.squares[i]:setScl(self.nSquareWidth,self.nSquareHeight)

            if i <= self.nFull then
                self.squares[i]:setColor(self.tFullColor.r,self.tFullColor.g,self.tFullColor.b,self.tFullColor.a or 1)
            elseif i <= self.nFull+self.nPending then
                self.squares[i]:setColor(self.tPendingColor.r,self.tPendingColor.g,self.tPendingColor.b,self.tPendingColor.a or 1)
            else
                self.squares[i]:setColor(self.tEmptyColor.r,self.tEmptyColor.g,self.tEmptyColor.b,self.tEmptyColor.a or 1)
            end

            if i+1 > self.nMaxSquaresPerRow*nCurRow then
                nCurX = self.nSquareMargin
                nCurY = nCurY-self.nSquareHeight-self.nSquareMargin
                nCurRow = nCurRow+1
            else
                nCurX = nCurX+self.nSquareMargin+self.nSquareWidth
            end

        end
        for i=self.nTotal+1,self.squares.n do
            self.squares[i]:setVisible(false)
        end
    end

    function Ob:getDims()
        return self.nBGWidth,self.nBGHeight
    end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if bVisible then
            self:update(self.nFull, self.nEmpty, self.nPending, true)
        end
	end

    --[[
    function Ob:setPriority(nPriority)
    end
    ]]--
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:isVisible()
        return self.bVisible
    end

    Ob.nMaxTicks = nMaxTicks
    Ob.nMaxWidth = nMaxWidth
    Ob.nMaxHeight = nMaxHeight
    Ob.tFullColor = tFullColor
    Ob.tBGColor = tBGColor
    Ob.tPendingColor = tPendingColor
    Ob.tEmptyColor = tEmptyColor

    Ob.nCurTicks = 0

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    Ob.rBG = MOAIProp2D.new()
    Ob.rBG:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBG:setColor(Ob.tBGColor.r,Ob.tBGColor.g,Ob.tBGColor.b,Ob.tBGColor.a or 1)
    Ob:addElement(Ob.rBG)

    Ob.squares = {}
    Ob.squares.n = 0

    Ob.bVisible = false

    return Ob
end

return TickProgressBar
