local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local Item = require('Item')
local TodoListUI = {}
local TodoListUIItem = {}
local TodoRewardPostIt = {}

local tUITextureSpriteSheetInfo =
{
    mm_clipboard = 'UI/hud2',    
    check_off = 'UI/hud2',
    check_on = 'UI/hud2',
    promotion = 'UI/hud2',
}

local tUIRewardTextureSpriteSheetInfo =
{
popup = 'UI/shopheropack',
icon_cash = 'UI/hud',
icon_pp = 'UI/hud',
}

function TodoRewardPostIt:new(rRenderLayer, rRewardInfo, rTodoListItem)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            if self.rTodoListItem then
                self.rTodoListItem:getNewItem()
            end
            g_GUI:SetPopupPane(nil)
        end
        return true
    end

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        g_Audio:playCue('MMOJ_Audio/UI/PostIt')
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob:loadUISpriteSheets(tUIRewardTextureSpriteSheetInfo)

    Ob.rTodoListItem = rTodoListItem

	Ob.rBackground = Ob:getUITextureProp('popup')
	Ob.rBackground:setLoc(0, 0)    
	Ob:addElement(Ob.rBackground)
    
    Ob.rRewardCompleteText = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rRewardCompleteText:setRect(-150, 80, 150, 0)
    Ob.rRewardCompleteText:setString(g_DialogManager:getString("TODO037TEXT")) --task complete!
	Ob.rRewardCompleteText:setLoc(160, 0)
    Ob.rRewardCompleteText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rRewardCompleteText)
    
    Ob.rRewardTakeText = g_GUI:CreateTextBox(30, MOAITextBox.CENTER_JUSTIFY, 'boogaloo')
    Ob.rRewardTakeText:setRect(-150, 80, 150, 0)
    Ob.rRewardTakeText:setString(g_DialogManager:getString("TODO038TEXT")) --take this!
	Ob.rRewardTakeText:setLoc(160, -35)
    Ob.rRewardTakeText:setColor(0, 0, 0)
	Ob:addOverlayElement(Ob.rRewardTakeText)

    Ob.rRewardDescriptionText = g_GUI:CreateTextBox(24, MOAITextBox.CENTER_JUSTIFY, 'type')
    Ob.rRewardDescriptionText:setRect(-150, 80, 150, 0)
	Ob.rRewardDescriptionText:setLoc(160, -173)
    Ob.rRewardDescriptionText:setColor(0, 0, 0)
    Ob.rRewardDescriptionText:setReveal(0)
	Ob:addOverlayElement(Ob.rRewardDescriptionText)
    
    if rRewardInfo.type == 'cash' then
        Ob.rIcon = Ob:getUITextureProp('icon_cash')
        Ob.rIcon:setLoc(130, -105)
        Ob:addElement(Ob.rIcon)
        Ob.rRewardDescriptionText:setString('+'..tostring(rRewardInfo.nAmount))
    elseif rRewardInfo.type == 'pp' then
        Ob.rIcon = Ob:getUITextureProp('icon_pp')
        Ob.rIcon:setLoc(130, -105)
        Ob:addElement(Ob.rIcon)
        Ob.rRewardDescriptionText:setString('+'..tostring(rRewardInfo.nAmount))
    elseif rRewardInfo.type == 'consumable' then
        if rRewardInfo.sItemDefKey then
            local rDef = Item.getConsumableDef(rRewardInfo.sItemDefKey)
            if rDef and rDef.iconName and rDef.sSpriteSheet then
                local tInfo = {}
                tInfo[rDef.iconName] = rDef.sSpriteSheet
                Ob:loadUISpriteSheets(tInfo)
                Ob.rIcon = Ob:getUITextureProp(rDef.iconName)
                if Ob.rIcon then
                    Ob.rIcon:setLoc(110, -75)
                    Ob.rIcon:setScl(0.75, 0.75)
                    Ob:addElement(Ob.rIcon)
a = Ob.rIcon
                end
                Ob.rRewardDescriptionText:setString(g_DialogManager:getString(rDef.sName))                
            end
        end
    end

    return Ob
end

function TodoListUIItem:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rTodoItem and self.rTodoItem:hasCompleted() then
            if eventType == MOAITouchSensor.TOUCH_UP and tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rCheckBox then
                        local rRewardDef = self.rTodoItem:getRewardDef()
                        if rRewardDef then
                            self.rCompletedIcon:setVisible(false)
                            self.rCheckBoxOn:setVisible(true)
                            self.rCheckBox:setVisible(false)
                            local rInfo = g_Game:createReward(rRewardDef)
                            if rInfo then
                                local rPostIt = TodoRewardPostIt:new(self:getRenderLayer(), rInfo, self)
                                if rPostIt then
                                    rPostIt:setLoc(500, -140)
                                    g_GUI:SetPopupPane(rPostIt)
                                end                      
                            end
                        end
                    end
                end
            end
        end
        return true
    end

    function Ob:setTodoItem(rItem)
        if rItem then
            self.rTodoItem = rItem
            -- refresh the info
            self.rDescription:setString(rItem:getUIDescription())
            if rItem:hasCompleted() then
                self.rCompletedIcon:setVisible(true)
                self.rCheckBoxOn:setVisible(false)
                self.rCheckBox:setVisible(true)
            else
                self.rCompletedIcon:setVisible(false)
                self.rCheckBoxOn:setVisible(false)
                self.rCheckBox:setVisible(true)                
            end
        else
            self.rTodoItem = nil
            self.rCompletedIcon:setVisible(false)
            self.rCheckBoxOn:setVisible(false)
            self.rCheckBox:setVisible(true)        
            self.rDescription:setString("")
        end
    end

    function Ob:getNewItem()
        local rNewItem = g_TodoList:removeTodoListItem(self.rTodoItem)
        if rNewItem then
            self:setTodoItem(rNewItem)
        else
            self:setTodoItem(nil)
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)
    
    Ob.rCheckBox = Ob:getUITextureProp('check_off')
	Ob.rCheckBox:setLoc(0, 0)    
	Ob:addElement(Ob.rCheckBox)

    Ob.rCheckBoxOn = Ob:getUITextureProp('check_on')
	Ob.rCheckBoxOn:setLoc(0, 0)    
    Ob.rCheckBoxOn:setVisible(false)
	Ob:addElement(Ob.rCheckBoxOn)

    Ob.rCompletedIcon = Ob:getUITextureProp('promotion')
	Ob.rCompletedIcon:setLoc(5, -14)    
    Ob.rCompletedIcon:setVisible(false)
	Ob:addElement(Ob.rCompletedIcon)

    Ob.rUnderline = MOAIProp2D.new()
	Ob.rUnderline:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rUnderline:setColor(.23, .19, .2)
    Ob.rUnderline:setScl(390, 6)
	Ob.rUnderline:setLoc(80, -58)
    Ob:addElement(Ob.rUnderline)

	Ob.rDescription = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY)
	Ob.rDescription:setString("THIS SPACE FOR RENT")
    Ob.rDescription:setRect(0, 100, 400, 0)
	Ob.rDescription:setLoc(84, -10)
    Ob.rDescription:setColor(.23, .19, .2)
    Ob:addElement(Ob.rDescription)     

    return Ob
end

function TodoListUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        for i, rItem in ipairs(self.tItems) do
            rItem:setVisible(bVisible)
        end
        g_Audio:playCue('MMOJ_Audio/UI/ClipBoard')
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
        for i, rItem in ipairs(self.tItems) do
            rItem:onFinger(eventType, sx, sy, tProps)
        end
        if eventType == MOAITouchSensor.TOUCH_UP and tProps then
            local bTouchedBG = false
            for i, rProp in ipairs(tProps) do
                if rProp == self.rBackground then
                    bTouchedBG = true
                end
            end
            if not bTouchedBG then
                -- exit
                g_GUI:HideTodoList()
            end
        end
        return true
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    Ob.rScrim = MOAIProp2D.new()
	Ob.rScrim:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rScrim:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rScrim:setColor(.4, .4, .4, .5)
	Ob.rScrim:setLoc(0, 0)    
    Ob:addElement(Ob.rScrim)

	Ob.rBackground = Ob:getUITextureProp('mm_clipboard')
	Ob.rBackground:setLoc(0, 0)    
	Ob:addElement(Ob.rBackground)

    Ob.rBGPaper = MOAIProp2D.new()
	Ob.rBGPaper:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBGPaper:setColor(.92, .93, .79)
    Ob.rBGPaper:setScl(530, 540)
	Ob.rBGPaper:setLoc(10, -60)  
    Ob:addElement(Ob.rBGPaper)

	Ob.rHeadLine = g_GUI:CreateTextBox(32, MOAITextBox.CENTER_JUSTIFY)
	Ob.rHeadLine:setString(g_DialogManager:getString("STXT151TEXT"))
    Ob.rHeadLine:setRect(-150, 40, 150, 0)
	Ob.rHeadLine:setLoc(286, -70)
    Ob.rHeadLine:setColor(0.6, 0.06, 0.03)
    Ob:addElement(Ob.rHeadLine) 

    local nInitY = -130
    local kINCREMENT = 100
    Ob.tItems = {}
    local tTodoItems = g_TodoList:getTodoList()
    for i, rItem in ipairs(tTodoItems) do
        local rUIItem = TodoListUIItem:new(rRenderLayer)
        if rUIItem then
            rUIItem:setLoc(40, nInitY - ((i - 1) * kINCREMENT))
            rUIItem:setTodoItem(rItem)
            table.insert(Ob.tItems, rUIItem)            
        end
    end

    return Ob
end

return TodoListUI