local TrainingSelectionUI = {}
local ActivitySelectionButton = {}
local Renderer = require('Renderer')
local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local AbilityUpgradeUI = require('ui/AbilityUpgradeUI')

local tUITextureSpriteSheetInfo =
{
activity_sheet_train_power = 'UI/hud',
activity_sheet_train_maxhp = 'UI/hud',
activity_sheet_train_int = 'UI/hud',
}

local tBGColor = { .78, .85, .72 }
local tFGColor = { .32, .42, .28 }

function TrainingSelectionUI:new(rRenderLayer, rParentUI, rHero)	
	local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
    end

    function Ob:isVisible()
        return self.bVisible
    end

	function Ob:onFinger(eventType, sx, sy, tProps)
    
        -- unhighlight all buttons on any touch event (we'll rehighlight them if the user has pushed down)
        if g_Game:isActionEnabled('HeroTrain') and self.rHero:getNumTrainingSessions() > 0 then
            if g_CurrentMap:getRoom('gym') and self:canTrainPowerWithRoom() then
                self.rTrainButton:setColor(1.0, 1.0, 1.0, 1.0)
            end
            
            if g_CurrentMap:getRoom('gym') and self:canTrainHPWithRoom() then
                self.rTrainHPButton:setColor(1.0, 1.0, 1.0, 1.0)
            end

            if self.rTrainIntButton then
                self.rTrainIntButton:setColor(1.0, 1.0, 1.0, 1.0)
            end
        end
    
        if eventType == MOAITouchSensor.TOUCH_DOWN or eventType == MOAITouchSensor.TOUCH_MOVE then
            if tProps and g_Game:isActionEnabled('HeroTrain') and self.rHero:getNumTrainingSessions() > 0 then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rTrainInvisButton and g_CurrentMap:getRoom('gym') and self:canTrainPowerWithRoom() then
                        self.rTrainButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rTrainHPInvisButton and g_CurrentMap:getRoom('gym') and self:canTrainHPWithRoom() then
                        self.rTrainHPButton:setColor(0.5, 0.5, 0.5, 1.0)
                    elseif rProp == self.rTrainIntInvisButton then
                        self.rTrainIntButton:setColor(0.5, 0.5, 0.5, 1.0)
                    end
                end
            end
        elseif eventType == MOAITouchSensor.TOUCH_UP then   
            if tProps then
                for i, rProp in ipairs(tProps) do
                    if rProp == self.rTrainInvisButton then
                        if g_Game:isActionEnabled('HeroTrain') then
                            if g_CurrentMap:getRoom('gym') and self.rHero:getNumTrainingSessions() > 0 and self:canTrainPowerWithRoom() then
                                self.rHero:setCurActivity('Train')
                                g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                                self.rParentUI:hideActivitySelector()
                                g_GUI:HideHeroScheduler()
                            end
                        end
                        return true
                    elseif rProp == self.rTrainHPInvisButton then
                        if g_Game:isActionEnabled('HeroTrain') then
                            if g_CurrentMap:getRoom('gym') and self.rHero:getNumTrainingSessions() > 0 and self:canTrainHPWithRoom() then
                                self.rHero:setCurActivity('HPTrain')
                                g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                                self.rParentUI:hideActivitySelector()
                                g_GUI:HideHeroScheduler()
                            end
                        end
                        return true

                    elseif rProp == self.rTrainIntInvisButton then
                        if g_Game:isActionEnabled('HeroTrain') then
                            if self.rHero:getNumTrainingSessions() > 0 then
                                g_Audio:playCue('MMOJ_Audio/UI/Select_Type1')
                                self.rParentUI:hideActivitySelector()
                                g_GUI:HideHeroScheduler()
                                g_MiddleManager:getAvatar():addMMTrainIntActivityAtIndex(1, self.rHero:getCharDefKey())
                            end
                        end
                        return true
                    end
                end
                if g_Game:isActionEnabled('CloseWindow') then
                    self.rParentUI:setTrainingSelectionUIVisible(false)
                    g_Audio:playCue('MMOJ_Audio/UI/BackButton')
                end
            end
        end
		return true
	end

    function Ob:setActivityIndex(nIndex)
        self.nActivityIndex = nIndex
    end

    function Ob:canTrainPowerWithRoom()
        local rRoom = g_CurrentMap:getRoom("gym")
        if rRoom then            
            if rRoom:getRoomDef().nMaxPower > self.rHero:getStatPower() then
                return true
            end
        end
        return false
    end

    function Ob:canTrainHPWithRoom()
        local rRoom = g_CurrentMap:getRoom("gym")
        if rRoom then
            if rRoom:getRoomDef().nMaxHP > self.rHero:getMaxHP() then
                return true
            end
        end
        return false        
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	-- vars
	Ob.rHero = rHero
    Ob.rParentUI = rParentUI
	Ob.nActivityIndex = nil    
    Ob.bVisible = false

    Ob.rBackground = MOAIProp2D.new()
    Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBackground:setLoc(0, 0)
    Ob.rBackground:setScl(412, 550)
    Ob.rBackground:setColor(tBGColor[1], tBGColor[2], tBGColor[3])
    Ob:addElement(Ob.rBackground)

    Ob.rLine1 = MOAIProp2D.new()
    Ob.rLine1:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine1:setLoc(24, -45)
    Ob.rLine1:setScl(365, 5)
    Ob.rLine1:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob:addElement(Ob.rLine1)

    Ob.rLine2 = MOAIProp2D.new()
    Ob.rLine2:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine2:setLoc(24, -129)
    Ob.rLine2:setScl(365, 2)
    Ob.rLine2:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob:addElement(Ob.rLine2)

    Ob.rLine3 = MOAIProp2D.new()
    Ob.rLine3:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine3:setLoc(24, -209)
    Ob.rLine3:setScl(365, 2)
    Ob.rLine3:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob:addElement(Ob.rLine3)

    Ob.rLine4 = MOAIProp2D.new()
    Ob.rLine4:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rLine4:setLoc(24, -289)
    Ob.rLine4:setScl(365, 2)
    Ob.rLine4:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob:addElement(Ob.rLine4)

    Ob.LabelText = g_GUI:CreateTextBox(24, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.LabelText:setString(string.upper(g_DialogManager:getString("STXT022TEXT")))
    Ob.LabelText:setRect(0, 80, 380, 0)
    Ob.LabelText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.LabelText:setLoc(20, -14)
    Ob:addElement(Ob.LabelText)

    Ob.SessionText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.SessionText:setString(rHero:getNumTrainingSessions().." "..g_DialogManager:getString("ACNA013TEXT"))
    Ob.SessionText:setRect(0, 80, 380, 0)
    Ob.SessionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.SessionText:setLoc(20, -50)
    Ob:addElement(Ob.SessionText)

    Ob.SessionHintText = g_GUI:CreateTextBox(18, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
    Ob.SessionHintText:setString(g_DialogManager:getString("STXT085TEXT"))
    Ob.SessionHintText:setRect(0, 80, 380, 0)
    Ob.SessionHintText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.SessionHintText:setLoc(20, -86)
    Ob:addElement(Ob.SessionHintText)

    Ob.rTrainInvisButton = MOAIProp2D.new()
    Ob.rTrainInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTrainInvisButton:setLoc(24, -134)
    Ob.rTrainInvisButton:setScl(362, 70)
    Ob.rTrainInvisButton:setColor(0, 0.5, 0.5)
    Ob.rTrainInvisButton:setVisible(false)
    Ob:addElement(Ob.rTrainInvisButton)

    Ob.rTrainButton = Ob:getUITextureProp('activity_sheet_train_power')
    Ob.rTrainButton:setLoc(24, -134)
    Ob:addElement(Ob.rTrainButton)

    Ob.TrainText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.TrainText:setString(string.upper(g_DialogManager:getString("ACNA006TEXT")))
    Ob.TrainText:setRect(0, 40, 300, 0)
    Ob.TrainText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.TrainText:setLoc(160, -130)
    Ob:addElement(Ob.TrainText)

    if g_CurrentMap:getRoom('gym') == nil then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.NoGymText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoGymText:setString(g_DialogManager:getString("ACRR005TEXT"))
        Ob.NoGymText:setRect(0, 40, 300, 0)
        Ob.NoGymText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.NoGymText:setLoc(160, -164)
        Ob.TrainText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob:addElement(Ob.NoGymText)
    elseif rHero:getNumTrainingSessions() == 0 then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.NoTrainSessionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoTrainSessionText:setString(g_DialogManager:getString("ACDS011TEXT"))
        Ob.NoTrainSessionText:setRect(0, 40, 260, 0)
        Ob.NoTrainSessionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.NoTrainSessionText:setLoc(160, -164)
        Ob.TrainText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob:addElement(Ob.NoTrainSessionText)
    elseif not Ob:canTrainPowerWithRoom() then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.TrainPropReqText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainPropReqText:setString(g_DialogManager:getString("ACRR007TEXT"))
        Ob.TrainPropReqText:setRect(0, 40, 300, 0)
        Ob.TrainPropReqText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainPropReqText:setLoc(160, -164)
        Ob.TrainText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob:addElement(Ob.TrainPropReqText)       
    else
        Ob.TrainDescriptionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainDescriptionText:setString(g_DialogManager:getString("ACDS006TEXT"))
        Ob.TrainDescriptionText:setRect(0, 40, 300, 0)
        Ob.TrainDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainDescriptionText:setLoc(160, -164)
        Ob:addElement(Ob.TrainDescriptionText)
    end
    
    Ob.rTrainHPInvisButton = MOAIProp2D.new()
    Ob.rTrainHPInvisButton:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rTrainHPInvisButton:setLoc(24, -214)
    Ob.rTrainHPInvisButton:setScl(362, 70)
    Ob.rTrainHPInvisButton:setColor(0, 0.5, 0.5)
    Ob.rTrainHPInvisButton:setVisible(false)
    Ob:addElement(Ob.rTrainHPInvisButton)

    Ob.rTrainHPButton = Ob:getUITextureProp('activity_sheet_train_maxhp')
    Ob.rTrainHPButton:setLoc(24, -214)

    Ob.TrainHPText = g_GUI:CreateTextBox(34, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
    Ob.TrainHPText:setString(string.upper(g_DialogManager:getString("ACNA008TEXT")))
    Ob.TrainHPText:setRect(0, 40, 300, 0)
    Ob.TrainHPText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
    Ob.TrainHPText:setLoc(160, -210)
    Ob:addElement(Ob.TrainHPText)

    Ob:addElement(Ob.rTrainHPButton)
    if g_CurrentMap:getRoom('gym') == nil then
        Ob.rTrainHPButton:setColor(1, 1, 1, 0.5)

        Ob.NoHPGymText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoHPGymText:setString(g_DialogManager:getString("ACRR005TEXT"))
        Ob.NoHPGymText:setRect(0, 40, 300, 0)
        Ob.NoHPGymText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainHPText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob.NoHPGymText:setLoc(160, -244)
        Ob:addElement(Ob.NoHPGymText)
    elseif rHero:getNumTrainingSessions() == 0 then
        Ob.rTrainHPButton:setColor(1, 1, 1, 0.5)

        Ob.NoTrainHPSessionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.NoTrainHPSessionText:setString(g_DialogManager:getString("ACDS011TEXT"))
        Ob.NoTrainHPSessionText:setRect(0, 40, 260, 0)
        Ob.NoTrainHPSessionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainHPText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob.NoTrainHPSessionText:setLoc(160, -244)
        Ob:addElement(Ob.NoTrainHPSessionText)
    elseif not Ob:canTrainHPWithRoom() then
        Ob.rTrainButton:setColor(1, 1, 1, 0.5)

        Ob.TrainHPPropReqText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainHPPropReqText:setString(g_DialogManager:getString("ACRR007TEXT"))
        Ob.TrainHPPropReqText:setRect(0, 40, 300, 0)
        Ob.TrainHPPropReqText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainHPText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        Ob.TrainHPPropReqText:setLoc(160, -244)
        Ob:addElement(Ob.TrainHPPropReqText)   
    else
        Ob.TrainHPDescriptionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.TrainHPDescriptionText:setString(g_DialogManager:getString("ACDS008TEXT"))
        Ob.TrainHPDescriptionText:setRect(0, 40, 300, 0)
        Ob.TrainHPDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainHPDescriptionText:setLoc(160, -244)
        Ob:addElement(Ob.TrainHPDescriptionText)
    end    

    if g_Game:isActionEnabled('MMTrainInt') then
        Ob.rTrainIntInvisButton = MOAIProp2D.new()
        Ob.rTrainIntInvisButton:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rTrainIntInvisButton:setLoc(24, -294)
        Ob.rTrainIntInvisButton:setScl(362, 70)
        Ob.rTrainIntInvisButton:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.rTrainIntInvisButton:setVisible(false)
        Ob:addElement(Ob.rTrainIntInvisButton)

        Ob.rTrainIntButton = Ob:getUITextureProp('activity_sheet_train_int')
        Ob.rTrainIntButton:setLoc(24, -294)
        Ob:addElement(Ob.rTrainIntButton)

        Ob.TrainIntText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.TrainIntText:setString(string.upper(g_DialogManager:getString("ACNA010TEXT")))
        Ob.TrainIntText:setRect(0, 40, 300, 0)
        Ob.TrainIntText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
        Ob.TrainIntText:setLoc(160, -290)
        Ob:addElement(Ob.TrainIntText)

        if rHero:getNumTrainingSessions() == 0 then
            Ob.rTrainIntButton:setColor(1, 1, 1, 0.5)
        
            Ob.NoIntTrainSessionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.NoIntTrainSessionText:setString(g_DialogManager:getString("ACDS011TEXT"))
            Ob.NoIntTrainSessionText:setRect(0, 40, 300, 0)
            Ob.NoIntTrainSessionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.NoIntTrainSessionText:setLoc(160, -324)
            Ob:addElement(Ob.NoIntTrainSessionText)
            Ob.TrainIntText:setColor(tFGColor[1], tFGColor[2], tFGColor[3], 0.25)
        else
            Ob.TrainIntDescriptionText = g_GUI:CreateTextBox(16, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
            Ob.TrainIntDescriptionText:setString(g_DialogManager:getString("ACDS014TEXT"))
            Ob.TrainIntDescriptionText:setRect(0, 40, 300, 0)
            Ob.TrainIntDescriptionText:setColor(tFGColor[1], tFGColor[2], tFGColor[3])
            Ob.TrainIntDescriptionText:setLoc(160, -324)
            Ob:addElement(Ob.TrainIntDescriptionText)
        end

    end

	return Ob
end

return TrainingSelectionUI