local UIElement = require('ui/UIElement')
local Renderer = require('Renderer')
local DFGraphics = require('DFCommon.Graphics')
local util = require('DFUtil')
local TutorialArrow = {}

local tUITextureSpriteSheetInfo = 
{
    arrow = 'UI/hud',
}

local kARROW_MOVE_TIME = 0.75

function TutorialArrow:new(rRenderLayer)
    local Ob = UIElement:new(rRenderLayer)

	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if not bVisible then
            if self.rThread then
                self.rThread:stop()
            end
        end
 	end    

    function Ob:animate(x, y)
        self.nTargX = x
        self.nTargY = y
        if self.rThread then
            self.rThread:stop()
            self.rThread = nil
        end
        self.rThread = MOAIThread.new()
        self.rThread:run(self.moveThread, self)
    end

    function Ob:moveThread()
        local nOrigX, nOrigY = self:getLoc()
        local bMoveBack = true
        while true do
            if bMoveBack then
                self:seekLoc(self.nTargX, self.nTargY, 0, kARROW_MOVE_TIME)
                bMoveBack = false
            else
                self:seekLoc(nOrigX, nOrigY, 0, kARROW_MOVE_TIME)
                bMoveBack = true
            end
            util.sleep(kARROW_MOVE_TIME)
        end
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()        
    end

    function Ob:setColor(tColor)
        if tColor then
            self.rArrow:setColor(tColor[1], tColor[2], tColor[3], tColor[4])
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

	Ob.rArrow = Ob:getUITextureProp('arrow')
	Ob.rArrow:setLoc(0, 0)
	Ob:addElement(Ob.rArrow)

    return Ob
end

return TutorialArrow
