local DFGraphics = require('DFCommon.Graphics')
local Renderer = require('Renderer')
local UIElement = {}

UIElement.tUIScreenInfo =
{
    SMALL = { sTextureSizeSuffix = '_Med', nTextureSizeFactor = 1 },
    MED = { sTextureSizeSuffix = '_Med', nTextureSizeFactor = 1 },
    LARGE = { key = 'LARGE', sTextureSizeSuffix = '_Lg', nTextureSizeFactor = 0.5 },
}

function UIElement:new(rRenderLayer)
    Ob = MOAITransform.new()
    Ob.__typename = "UIElement"
    Ob.rRenderLayer = rRenderLayer
    Ob.tElements = {}
    Ob.tOverlayElements = {}
    Ob.tNamedElements = {}
    Ob.tUISpriteSheets = {}
    Ob.tTextureToSpriteSheet = {}
    Ob.rOverlayRenderLayer = Renderer.getRenderLayer('UIOverlay')

    function Ob:getRenderLayer()
        return self.rRenderLayer
    end

    function Ob:alignUISpriteSheets(xAlign, yAlign)
        if self.tTextureToSpriteSheet then
            local nSizeFactor = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].nTextureSizeFactor
            local sSuffix = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].sTextureSizeSuffix
            for i, rSpriteSheet in pairs(self.tTextureToSpriteSheet) do
                for sName, data in pairs(rSpriteSheet.names) do
                    DFGraphics.alignSprite(rSpriteSheet, sName, xAlign or "left", yAlign or "top", nSizeFactor, nSizeFactor)
                end
            end
        end
    end

    function Ob:loadUISpriteSheets(tSpriteSheetInfo, xAlign, yAlign)        
        if tSpriteSheetInfo then
            for sTextureName, spriteSheetPath in pairs(tSpriteSheetInfo) do
                local nSizeFactor = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].nTextureSizeFactor
                local sSuffix = UIElement.tUIScreenInfo[Renderer.getCurScreenInfo().key].sTextureSizeSuffix
                local sPath = spriteSheetPath..sSuffix
                if not self.tUISpriteSheets[sPath] then
                    local rSpriteSheet = DFGraphics.loadSpriteSheet(sPath, g_IsInStartupSequence)                    
                    if rSpriteSheet then
                        for sName, data in pairs(rSpriteSheet.names) do
                            DFGraphics.alignSprite(rSpriteSheet, sName, xAlign or "left", yAlign or "top", nSizeFactor, nSizeFactor)
                        end
                        self.tUISpriteSheets[sPath] = rSpriteSheet
                    end
                end
                if self.tUISpriteSheets[sPath] then
                    self.tTextureToSpriteSheet[sTextureName] = self.tUISpriteSheets[sPath]
                end
            end
        end
    end

    function Ob:unloadUISpriteSheets()
        for sPathName, rSpriteSheet in pairs(self.tUISpriteSheets) do            
            DFGraphics.unloadSpriteSheet(sPathName)
        end
    end

    function Ob:getSpriteSheetForTexture(sTextureName)
        if sTextureName then
            return self.tTextureToSpriteSheet[sTextureName]
        end
    end

    function Ob:getUITextureProp(sTextureName, rProp)
        if sTextureName and self.tTextureToSpriteSheet[sTextureName] then
            local rSpriteSheet = self.tTextureToSpriteSheet[sTextureName]
            if not rProp then
                rProp = MOAIProp2D.new()
            end
            rProp:setDeck(rSpriteSheet)
            rProp:setIndex(rSpriteSheet.names[sTextureName])
            return rProp
        else
            print('ERROR: COULD NOT FIND '..tostring(sTextureName))
        end
    end

    function Ob:addOverlayElement(rElement)
        if rElement then
            table.insert(self.tOverlayElements, rElement)
            rElement:setParent(self)
        end        
    end

	function Ob:addElement(rElement)
        if rElement then
            table.insert(self.tElements, rElement)
            rElement:setParent(self)
        end
	end

    function Ob:removeElement(rElement)
        if rElement then
            for i, rExistingElement in ipairs(self.tElements) do
                if rElement == rExistingElement then
                    self.rRenderLayer:removeProp(rExistingElement)
                    table.remove(self.tElements, i)
                    return
                end
            end
        end
    end

    function Ob:getElements()
        return self.tElements
    end

    function Ob:hasElements(tProps)
        for i, rProp in ipairs(tProps) do
            for c, rExistingProp in ipairs(self.tElements) do
                if rExistingProp == rProp then
                    return true
                end
            end
        end
        return false
    end

    function Ob:getElementsMaxPriority()
        local nMaxPri = 0
        for i, rElement in ipairs(self.tElements) do
            local nPri = rElement:getPriority()
            if nPri and nPri > nMaxPri then
                nMaxPri = nPri
            end
        end
        return nMaxPri
    end

    function Ob:setPriOffset(nOffset)
        for i, rElement in ipairs(self.tElements) do
            local nPri = rElement:getPriority()
            if nPri then
                rElement:setPriority(nPri + nOffset)
            end
        end
    end

    -- empty
    function Ob:onPaneClosed()
    end

	-- functions
	function Ob:setElementsVisible(bVisible)
		if not self.rRenderLayer then
			return
		end
		if bVisible then
            for i, rElement in ipairs(self.tElements) do
                self.rRenderLayer:insertProp(rElement)
            end
            for i, rElement in ipairs(self.tOverlayElements) do
                self.rOverlayRenderLayer:insertProp(rElement)
            end
		else
            for i, rElement in ipairs(self.tElements) do
                self.rRenderLayer:removeProp(rElement)
            end
            for i, rElement in ipairs(self.tOverlayElements) do
                self.rOverlayRenderLayer:removeProp(rElement)
            end
		end
	end

    return Ob
end

return UIElement
