local util = require("DFUtil")
local UIElement = require('ui/UIElement')
local UIProgressBar = {}

-- quick UI Button
function UIProgressBar:new(rRenderLayer, nMaxAmount, tColor, tBGColor, bBorder, tChunkColor)
	local Ob = UIElement:new(rRenderLayer)
    -- change scale of inner bar if bBorder = true
    local xBarScale = 1
    local yBarScale = 1
    local xBarLoc = 0
    local yBarLoc = 0
    
    if bBorder then
         xBarScale = .98
         yBarScale = .7
         xBarLoc = .01
         yBarLoc = -.15
    end
	-- functions
    function Ob:update(nProgressAmount)
        if nProgressAmount > self.nMaxAmount then
            nProgressAmount = self.nMaxAmount
        end

        if nProgressAmount ~= self.nCurAmount and self.nPendingProgress then
            local newProgress = math.max(0, nProgressAmount-self.nPendingProgress)
            self.rChunkBar:setLoc((newProgress / self.nMaxAmount) * xBarScale+xBarLoc, yBarLoc)
        end

        self.nCurAmount = nProgressAmount
        self.rProgressBar:setScl((nProgressAmount / self.nMaxAmount) * xBarScale, yBarScale)
    end

    function Ob:getCurrentAmount()
        return self.nCurAmount
    end

    -- this needs to be called from a coroutine
    function Ob:moveToAmount(nProgressAmount, nTime, bBlocking)
        if nProgressAmount > self.nMaxAmount then
            nProgressAmount = self.nMaxAmount
        end
        local nXDiff = (nProgressAmount / self.nMaxAmount) - (self.nCurAmount / self.nMaxAmount)
        self.rProgressBar:moveScl(nXDiff, 0, nTime, MOAIEaseType.LINEAR)
        if bBlocking then
            util.sleep(nTime)
        end
        self.nCurAmount = nProgressAmount
    end

	function Ob:setVisible(bVisible)
        self.bVisible = bVisible
        self:setElementsVisible(bVisible)
        if self.rChunkBar and not self.nPendingProgress then
            self.rChunkBar:setVisible(false)
        end
	end

    function Ob:setPriority(nPriority)
        if self.rBGBar then
            self.rBGBar:setPriority(nPriority)
        end
        if self.rChunkBar then
            self.rChunkBar:setPriority(nPriority + 1)
        end
        if self.rProgressBarBorder then
            self.rProgressBarBorder:setPriority(nPriority + 2)
        end
        if self.rProgressBar then
            self.rProgressBar:setPriority(nPriority + 3)
        end
    end
    
    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()
    end

    function Ob:isVisible()
        return self.bVisible
    end

    function Ob:tint(r, g, b, a)
        if not r then
            r = 1
        end
        if not g then
            g = r
        end
        if not b then
            b = r
        end
        if not a then
            a = 1
        end
    
        if self.tBGColor and self.rBGBar then
            self.rBGBar:setColor(self.tBGColor.r * r, self.tBGColor.g * g, self.tBGColor.b * b, self.tBGColor.a * a)
        end
        if self.tChunkColor and self.rChunkBar then
            self.rChunkBar:setColor(self.tChunkColor.r * r, self.tChunkColor.g * g, self.tChunkColor.b * b, self.tChunkColor.a * a)
        end
        if self.tBorderColor and self.rProgressBarBorder then
            self.rProgressBarBorder:setColor(self.tBorderColor.r * r, self.tBorderColor.g * g, self.tBorderColor.b * b, self.tBorderColor.a * a)
        end
        if self.tBarColor and self.rProgressBar then
            self.rProgressBar:setColor(self.tBarColor.r * r, self.tBarColor.g * g, self.tBarColor.b * b, self.tBarColor.a * a)
        end
    end

    function Ob:realizeProgress()
        local nToAdd = self.nPendingProgress
        self:setProgressPending(nil)
        if nToAdd then
            self:update(nToAdd+self.nCurAmount)
        end
    end
    
    function Ob:setProgressPending(nProgressAmount, nMoveOverTime)
        if not nProgressAmount then
            self.nPendingProgress = nil
            if self.rChunkBar then self.rChunkBar:setVisible(false) end
            return
        end

        if not self.rChunkBar then
            self.rChunkBar = MOAIProp2D.new()
            self.rChunkBar:setDeck(g_GUI:GetOnePixelDeck())
            local r,g,b,a = 0,0,0,.5
            if self.tChunkColor then
                local t = self.tChunkColor
                r,g,b,a = t.r,t.g,t.b,t.a
            end
            self.tChunkColor = { r = r, g = g, b = b, a = a }
            self.rChunkBar:setColor(r, g, b, a)
            self:addElement(Ob.rChunkBar)
        end
        if self.bVisible then
            self.rChunkBar:setVisible(true)
            self.rRenderLayer:insertProp(self.rChunkBar)
        end

        if self.nCurAmount + nProgressAmount > self.nMaxAmount then nProgressAmount = self.nMaxAmount-self.nCurAmount
        elseif self.nCurAmount + nProgressAmount < 0 then nProgressAmount = -self.nCurAmount end

        self.nPendingProgress = nProgressAmount

        self.rChunkBar:setScl((nProgressAmount / self.nMaxAmount) * xBarScale, yBarScale)
        local newProgress = self.nCurAmount --math.max(0, self.nCurAmount-nProgressAmount)
        self.rChunkBar:setLoc((newProgress / self.nMaxAmount) * xBarScale+xBarLoc, yBarLoc)

        if nMoveOverTime and nMoveOverTime > 0 then
            self.rChunkBar:setLoc((self.nCurAmount / self.nMaxAmount) * xBarScale+xBarLoc, yBarLoc)
            self.rChunkBar:setScl(0, yBarScale)
            self.rChunkBar:moveScl((nProgressAmount / self.nMaxAmount) * xBarScale, 0, nMoveOverTime, MOAIEaseType.LINEAR)
        end
    end

    Ob.nMaxAmount = nMaxAmount
    Ob.nCurAmount = 0

    local r = 1
    local g = 0
    local b = 0
    local a = 1
    if tBGColor then
        r = tBGColor.r
        g = tBGColor.g
        b = tBGColor.b
        a = tBGColor.a
    end

    Ob.rBGBar = MOAIProp2D.new()
    Ob.rBGBar:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rBGBar:setLoc(0, 0)
    Ob.rBGBar:setScl(1, 1)
    Ob.rBGBar:setColor(r, g, b, a)
    Ob:addElement(Ob.rBGBar)

    Ob.tBGColor = { r = r, g = g, b = b, a = a}
    
    r = 0
    g = 1
    b = 0
    a = 1
    if tColor then
        if tColor.r then
            r = tColor.r
        else
            r = 1
        end
        
        if tColor.g then
            g = tColor.g
        else
            g = r
        end
        
        if tColor.b then
            b = tColor.b
        else
            b = r
        end
        
        if tColor.a then
            a = tColor.a
        else
            a = 1
        end
    end
    
    Ob.tBarColor = { r = r, g = g, b = b, a = a}
    
    if bBorder then
        Ob.rProgressBarBorder = MOAIProp2D.new()
        Ob.rProgressBarBorder:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rProgressBarBorder:setLoc(xBarLoc, yBarLoc)
        Ob.rProgressBarBorder:setScl(xBarScale, yBarScale)
        Ob.rProgressBarBorder:setColor(.3, .3, .3, 1)
        Ob:addElement(Ob.rProgressBarBorder)
    end
    
    Ob.tBorderColor = { r = .3, g = .3, b = .3, a = 1}
    
    Ob.rProgressBar = MOAIProp2D.new()
    Ob.rProgressBar:setDeck(g_GUI:GetOnePixelDeck())
    Ob.rProgressBar:setLoc(xBarLoc, yBarLoc)
    Ob.rProgressBar:setScl(xBarScale, yBarScale)
    Ob.rProgressBar:setColor(r, g, b, a)
    Ob:addElement(Ob.rProgressBar)

    Ob.bVisible = false
    Ob.tChunkColor = tChunkColor

    return Ob
end

return UIProgressBar
