local ValentinesEndCutscene = {}
local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local SpriteSheet = require('SpriteSheet')
local util = require('DFUtil')
local ResumeUI = require('ui/ResumeUI')

local tUITextureSpriteSheetInfo =
{
    MM_DoorA = 'UI/Cutscene_ValentineA',
    Mindy_Body = 'UI/Cutscene_ValentineA',
    Mindy_FaceA = 'UI/Cutscene_ValentineA',
    Mindy_FaceB = 'UI/Cutscene_ValentineA',
    MM_FaceA = 'UI/Cutscene_ValentineA',
    MM_FaceB = 'UI/Cutscene_ValentineA',
    MM_FaceC = 'UI/Cutscene_ValentineA',
    MM_DoorB = 'UI/Cutscene_ValentineB',
    MM_Body = 'UI/Cutscene_ValentineB',
    MM_Snapshot = 'UI/Cutscene_ValentineC',    
}

function ValentinesEndCutscene:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)

    function Ob:cutsceneThread()
        g_Audio:stopMusic()
        g_Audio:stopCurAmbience()
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Tutorial', false)

        util.sleep(1.5)

        local rMM = g_MiddleManager:getAvatar()
        rMM:sayLine("VLMM015MIDM", 'lower', true, true)
        g_GUI:HideDialogText()
        
        util.sleep(0.5)

        self.rOpenDoor:setVisible(true)
        
        util.sleep(0.5)

        self.rMindy:setVisible(true)
        self.rMindyFaceA:setVisible(true)
        self.rMMFaceA:setVisible(false)
        self.rMMFaceB:setVisible(true)

        util.sleep(2)

        self.rMindyFaceA:setVisible(false)
        self.rMindyFaceB:setVisible(true)
        self.rMMFaceB:setVisible(false)
        self.rMMFaceC:setVisible(true)
        
        self.bPressed = false
        while not self.bPressed do
            coroutine.yield()
        end

        self.rFadeupForeground:moveColor(0, 0, 0, 1, 0.75)
        util.sleep(0.75)

        self.rHouse:setVisible(false)
        self.rOpenDoor:setVisible(false)
        self.rMM:setVisible(false)
        self.rMMFaceC:setVisible(false)
        self.rMindy:setVisible(false)
        self.rMindyFaceB:setVisible(false)
        self.rSnapShot:setVisible(true)
        self.rTopBG:setVisible(false)
        self.rBottomBG:setVisible(false)
        self.rBackground:setColor(1, .75, .74)

        self.rFadeupForeground:moveColor(0, 0, 0, -1, 0.75)
        -- kisses
        g_Audio:playMusic('MMOJ_Audio/Music/MMOJ_Stinger', false)

        util.sleep(0.75)
        
        self.bPressed = false
        while not self.bPressed do
            coroutine.yield()
        end

		g_Audio:playAmbience('MMOJ_Audio/Ambience/OfficeAmbience1')
        self.bRunningThread = false

        g_GUI:HideValentinesEndCS()
    end

    function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if bVisible then
            if not self.bRunningThread then
                self.bRunningThread = true
                self.rThread = MOAIThread.new()
                self.rThread:run(self.cutsceneThread, self)
            end
        else
            self:unloadUISpriteSheets()
        end
    end

    function Ob:isRunning()
        return self.bRunningThread
    end

    function Ob:onPaneClosed()
        self:unloadUISpriteSheets()        
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if eventType == MOAITouchSensor.TOUCH_UP then
            self.bPressed = true
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo, 'center', 'center')

    Ob.rBackground = MOAIProp2D.new()
	Ob.rBackground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBackground:setLoc(0, 0)
	Ob.rBackground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rBackground:setColor(0, 0, 0)
	Ob:addElement(Ob.rBackground)

    Ob.rHouse = Ob:getUITextureProp('MM_DoorA')
    Ob.rHouse:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob:addElement(Ob.rHouse)

    Ob.rOpenDoor = Ob:getUITextureProp('MM_DoorB')
    Ob.rOpenDoor:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rOpenDoor:setVisible(false)
    Ob:addElement(Ob.rOpenDoor)

    Ob.rMM = Ob:getUITextureProp('MM_Body')
    Ob.rMM:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob:addElement(Ob.rMM)

    Ob.rMMFaceA = Ob:getUITextureProp('MM_FaceA')
    Ob.rMMFaceA:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob:addElement(Ob.rMMFaceA)

    Ob.rMMFaceB = Ob:getUITextureProp('MM_FaceB')
    Ob.rMMFaceB:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMMFaceB:setVisible(false)
    Ob:addElement(Ob.rMMFaceB)

    Ob.rMMFaceC = Ob:getUITextureProp('MM_FaceC')
    Ob.rMMFaceC:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMMFaceC:setVisible(false)
    Ob:addElement(Ob.rMMFaceC)

    Ob.rMindy = Ob:getUITextureProp('Mindy_Body')
    Ob.rMindy:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMindy:setVisible(false)
    Ob:addElement(Ob.rMindy)

    Ob.rMindyFaceA = Ob:getUITextureProp('Mindy_FaceA')
    Ob.rMindyFaceA:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMindyFaceA:setVisible(false)
    Ob:addElement(Ob.rMindyFaceA)

    Ob.rMindyFaceB = Ob:getUITextureProp('Mindy_FaceB')
    Ob.rMindyFaceB:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rMindyFaceB:setVisible(false)
    Ob:addElement(Ob.rMindyFaceB)

    Ob.rSnapShot = Ob:getUITextureProp('MM_Snapshot')
    Ob.rSnapShot:setLoc(Renderer.getUIHalfWidth(), -Renderer.getUIHalfHeight())
    Ob.rSnapShot:setVisible(false)
    Ob:addElement(Ob.rSnapShot)

    local kIMG_HEIGHT = 470
    local nBorderSize = Renderer.getUIHalfHeight() - kIMG_HEIGHT / 2

    Ob.rTopBG = MOAIProp2D.new()
	Ob.rTopBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rTopBG:setLoc(0, 0)
	Ob.rTopBG:setScl(Renderer.getUIWidth(), nBorderSize)
	Ob.rTopBG:setColor(0, 0, 0)
	Ob:addElement(Ob.rTopBG)

    Ob.rBottomBG = MOAIProp2D.new()
	Ob.rBottomBG:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rBottomBG:setLoc(0, - Renderer.getUIHeight() + nBorderSize)
	Ob.rBottomBG:setScl(Renderer.getUIWidth(), nBorderSize)
	Ob.rBottomBG:setColor(0, 0, 0)
	Ob:addElement(Ob.rBottomBG)

    Ob.rFadeupForeground = MOAIProp2D.new()
	Ob.rFadeupForeground:setDeck(g_GUI:GetOnePixelDeck())
	Ob.rFadeupForeground:setLoc(0, 0)
	Ob.rFadeupForeground:setScl(Renderer.getUIWidth(), Renderer.getUIHeight())
	Ob.rFadeupForeground:setColor(0, 0, 0, 0)
	Ob:addOverlayElement(Ob.rFadeupForeground)

    Ob.bRunningThread = false

    return Ob
end

return ValentinesEndCutscene