local Renderer = require('Renderer')
local UIElement = require('ui/UIElement')
local UIProgressBar = require('ui/UIProgressBar')
local Button = require('ui/Button')
local ValentinesTerritoryMapUI = {}

local tUITextureSpriteSheetInfo =
{
    specialevent = 'UI/eventdetails',
    specialevent_icon = 'UI/eventdetails',
    button_buy = 'UI/hud',
}

function ValentinesTerritoryMapUI:new(rRenderLayer)
	local Ob = UIElement:new(rRenderLayer)
    
	function Ob:setVisible(bVisible)
        self:setElementsVisible(bVisible)
        if self.rProgressBar then
            self.rProgressBar:setVisible(bVisible)
            
            local nBasePri = self.rProgressBar:getElementsMaxPriority()
            self.rMeterBG:setPriority(nBasePri + 1)
            self.rIcon:setPriority(nBasePri + 2)
            self.rHolidayLabel:setPriority(nBasePri + 3)
            self.rProgressText:setPriority(nBasePri + 4)
        end
        if self.nProgress >= 1 then
            self.rCSButton:setVisible(bVisible)
            self.rNumDaysText:setReveal(0)
            self.rNumDaysLabel:setReveal(0)
        else
            if self.rNumDaysText then
                self.rNumDaysText:revealAll()
            end
            
            if self.rNumDaysLabel then
                self.rNumDaysLabel:revealAll()
            end
        end
    end

    function Ob:onFinger(eventType, sx, sy, tProps)
        if self.rCSButton then
            self.rCSButton:onFinger(eventType, sx, sy, tProps)
        end
    end

    function Ob:onButtonJustPressed(rButton)
        if rButton then
            if rButton == self.rCSButton then
                g_GUI:ShowValentinesEndCS()
            end
        end
    end

    Ob:loadUISpriteSheets(tUITextureSpriteSheetInfo)

    local rEventSet = g_eventManager:getActiveHolidaySet('Valentines')
    local rHolidayDef = g_Game:getHolidayDef('valentines')
    if rEventSet and rHolidayDef and rEventSet:shouldShowHolidayUI() then
        local nDaysLeft = 0
        if rHolidayDef.endDate then
            local nCurrentTime = os.time()
            local osd = os.date("*t", nCurrentTime)
            local endOsd = {}
            for k, v in pairs(osd) do
                endOsd[k] = v
            end
            endOsd.month = rHolidayDef.endDate.month
            endOsd.day = rHolidayDef.endDate.day
            local nDiffTime = os.difftime(os.time(endOsd), os.time(osd))
            nDaysLeft = math.floor(((nDiffTime / 60) / 60) / 24)
        end

        local nMeterBGX = Renderer.getUIHalfWidth() - 370
        local kIMG_SIZE = 640

        Ob.rTopLeftBG = MOAIProp2D.new()
        Ob.rTopLeftBG:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rTopLeftBG:setLoc(0, -90)
        Ob.rTopLeftBG:setScl(nMeterBGX, 72)
        Ob.rTopLeftBG:setColor(0.73, 0.81, 1)
        Ob.rTopLeftBG:setParent(Ob)
        Ob:addOverlayElement(Ob.rTopLeftBG)

        Ob.rTopRightBG = MOAIProp2D.new()
        Ob.rTopRightBG:setDeck(g_GUI:GetOnePixelDeck())
        Ob.rTopRightBG:setLoc(nMeterBGX + kIMG_SIZE, -90)
        Ob.rTopRightBG:setScl(Renderer.getUIWidth() - nMeterBGX - kIMG_SIZE, 72)
        Ob.rTopRightBG:setColor(0.73, 0.81, 1)
        Ob.rTopRightBG:setParent(Ob)
        Ob:addOverlayElement(Ob.rTopRightBG)

        Ob.rMeterBG = Ob:getUITextureProp('specialevent')
        Ob.rMeterBG:setLoc(nMeterBGX, -90)
        Ob.rMeterBG:setParent(Ob)
        Ob:addOverlayElement(Ob.rMeterBG)

        Ob.rIcon = Ob:getUITextureProp('specialevent_icon')
        Ob.rIcon:setLoc(Renderer.getUIHalfWidth() + 207, -101)
        Ob.rIcon:setParent(Ob)
        Ob:addOverlayElement(Ob.rIcon)
        
        Ob.rNumDaysText = g_GUI:CreateTextBox(50, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.rNumDaysText:setLoc(Renderer.getUIHalfWidth() + 580, -92)
        Ob.rNumDaysText:setString(tostring(nDaysLeft))
        Ob.rNumDaysText:setParent(Ob)
        Ob:addOverlayElement(Ob.rNumDaysText)
        
        Ob.rNumDaysLabel = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.rNumDaysLabel:setLoc(Renderer.getUIHalfWidth() + 642, -112)
        if nDaysLeft == 1 then
            Ob.rNumDaysLabel:setString(g_DialogManager:getString("STXT194TEXT"))
        else
            Ob.rNumDaysLabel:setString(g_DialogManager:getString("STXT192TEXT"))
        end
        Ob.rNumDaysLabel:setParent(Ob)
        Ob:addOverlayElement(Ob.rNumDaysLabel)
        
        Ob.rHolidayLabel = g_GUI:CreateTextBox(36, MOAITextBox.LEFT_JUSTIFY, 'boogaloo')
        Ob.rHolidayLabel:setLoc(Renderer.getUIHalfWidth() - 42, -102)
        Ob.rHolidayLabel:setString(g_DialogManager:getString("STXT193TEXT"))
        Ob.rHolidayLabel:setParent(Ob)
        Ob:addOverlayElement(Ob.rHolidayLabel)
        
        Ob.rProgressText = g_GUI:CreateTextBox(30, MOAITextBox.LEFT_JUSTIFY, 'ptsans')
        Ob.rProgressText:setLoc(Renderer.getUIHalfWidth() + 420, -104)
        Ob.rProgressText:setString(math.floor(rEventSet:getProgress() * 100).."%")
        Ob.rProgressText:setColor(1, 1, 1)
        Ob.rProgressText:setParent(Ob)
        Ob:addOverlayElement(Ob.rProgressText)    

        Ob.rProgressBar = UIProgressBar:new(Renderer.getRenderLayer('UIOverlay'), 1, { r = 1, g = .44, b = .55 }, { r = .6, g = .12, b = .21 })
        Ob.rProgressBar:setLoc(nMeterBGX + 10, -100)
        Ob.rProgressBar:setScl(566, 50)
        Ob.rProgressBar:setParent(Ob)
        Ob.rProgressBar:update(rEventSet:getProgress())

        Ob.nProgress = rEventSet:getProgress()

        Ob.rCSButton = Button:new(Renderer.getRenderLayer('UIOverlay'), Ob, tUITextureSpriteSheetInfo, 'button_buy', nil, { 1, .44, .55 }, { .6, .12, .21  }, true, { 0, -50 }, "STXT205TEXT", 'ptsans', 24, { 78, -5 }, { 1, 1, 1 }, { 0.65, 0.65, 0.65 })
        Ob.rCSButton:setParent(Ob)
        Ob.rCSButton:setLoc(Renderer.getUIHalfWidth() + 280, -100)
    end
    
    if not Ob.nProgress then 
        Ob.nProgress = 0 
    end

    return Ob
end

return ValentinesTerritoryMapUI